#!/bin/bash

# this script is part of the make prebuilt procedure
# is expected on the root folder

# create a commit message with the commit id
COMMIT=$(git rev-parse --verify HEAD)
MSG="built from ${COMMIT}"

# after building process has copy everything into 
# the prebuilt folder
cd prebuilt
git commit -m "$MSG" -a
git show --stat
echo "ready to push"
