/*
 This file is part of GNU Taler
 (C) 2021 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

/**
 * Imports.
 */
import {
  HttpStatusCode,
  MerchantAuthMethod,
  TalerMerchantManagementHttpClient,
  URL
} from "@gnu-taler/taler-util";
import {
  ExchangeService,
  GlobalTestState,
  MERCHANT_DEFAULT_LOGIN_SCOPE,
  MerchantService,
  getTestHarnessPaytoForLabel,
  harnessHttpLib,
  setupDb
} from "../harness/harness.js";

/**
 * Test instance deletion and authentication for it
 */
export async function runMerchantInstancesDeleteTest(t: GlobalTestState) {
  // Set up test environment

  const db = await setupDb(t);

  const exchange = ExchangeService.create(t, {
    name: "testexchange-1",
    currency: "TESTKUDOS",
    httpPort: 8081,
    database: db.connStr,
  });

  const merchant = await MerchantService.create(t, {
    name: "testmerchant-1",
    httpPort: 8083,
    database: db.connStr,
  });

  // We add the exchange to the config, but note that the exchange won't be started.
  merchant.addExchange(exchange);

  await merchant.start();
  await merchant.pingUntilAvailable();

  // Base URL for the default instance.
  const baseUrl = merchant.makeInstanceBaseUrl();

  {
    const r = await harnessHttpLib.fetch(new URL("config", baseUrl).href);
    const data = await r.json();
    console.log(data);
    t.assertDeepEqual(data.currency, "TESTKUDOS");
  }

  // Instances should initially be empty
  {
    const r = await harnessHttpLib.fetch(
      new URL("management/instances", baseUrl).href,
    );
    const data = await r.json();
    t.assertDeepEqual(data.instances, []);
  }

  // Add an instance, no auth!
  const { accessToken: adminAccessToken } = await merchant.addInstanceWithWireAccount({
    id: "admin",
    name: "Default Instance",
    paytoUris: [getTestHarnessPaytoForLabel("merchant-default")],
    auth: {
      method: MerchantAuthMethod.TOKEN,
      password: "admin-pwd"
    },
  });

  // Add an instance, no auth!
  await merchant.addInstanceWithWireAccount({
    id: "myinst",
    name: "Second Instance",
    paytoUris: [getTestHarnessPaytoForLabel("merchant-default")],
    auth: {
      method: MerchantAuthMethod.TOKEN,
      password: "myinst-pwd"
    },
  }, {adminAccessToken});

  const merchantApi = new TalerMerchantManagementHttpClient(
    merchant.makeInstanceBaseUrl(),
  );

  const adminPwd = await merchantApi.createAccessToken("admin", "admin-pwd", MERCHANT_DEFAULT_LOGIN_SCOPE)
  t.assertTrue(adminPwd.type === "ok")


  // Check that deleting an instance checks the auth
  // of the default instance.
  {
    const res = await merchantApi.deleteInstance(
      undefined,
      "myinst",
    );
    t.assertTrue(res.type === "fail");
    t.assertTrue(res.case === HttpStatusCode.Unauthorized);
  }
}

runMerchantInstancesDeleteTest.suites = ["merchant"];
