// flow-typed signature: db087c0f8735aace9843e6ebf0fc6db2
// flow-typed version: <<STUB>>/rollup-plugin-replace_v^2.0.0/flow_v0.82.0

/**
 * This is an autogenerated libdef stub for:
 *
 *   'rollup-plugin-replace'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module 'rollup-plugin-replace' {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module 'rollup-plugin-replace/dist/rollup-plugin-replace.cjs' {
  declare module.exports: any;
}

declare module 'rollup-plugin-replace/dist/rollup-plugin-replace.es' {
  declare module.exports: any;
}

declare module 'rollup-plugin-replace/src/index' {
  declare module.exports: any;
}

// Filename aliases
declare module 'rollup-plugin-replace/dist/rollup-plugin-replace.cjs.js' {
  declare module.exports: $Exports<'rollup-plugin-replace/dist/rollup-plugin-replace.cjs'>;
}
declare module 'rollup-plugin-replace/dist/rollup-plugin-replace.es.js' {
  declare module.exports: $Exports<'rollup-plugin-replace/dist/rollup-plugin-replace.es'>;
}
declare module 'rollup-plugin-replace/src/index.js' {
  declare module.exports: $Exports<'rollup-plugin-replace/src/index'>;
}
