/**
 * Customize the displayed name of a useState, useReducer or useRef hook
 * in the devtools panel.
 *
 * @param value Wrapped native hook.
 * @param name Custom name
 */
export function addHookName<T>(value: T, name: string): T;
