#!/bin/bash

[ ! -d prebuilt ] && git worktree add -f prebuilt prebuilt && exit 1

rm -f prebuilt/aml-backoffice/*
find packages/aml-backoffice-ui/dist/prod/ -type f -printf '%P\n' | sort > prebuilt/aml-backoffice/bof

while IFS= read -r file; do
  cp packages/aml-backoffice-ui/dist/prod/$file prebuilt/aml-backoffice/$file
done < prebuilt/aml-backoffice/bof

