declare module "*.crt" {
  const content: string;
  export default content;
}
declare module "*.key" {
  const content: string;
  export default content;
}
declare module "*.html" {
  const content: string;
  export default content;
}
