@keyframes popup {
	from {
		box-shadow: 0 0 0 black;
		opacity: 0;
		transform: scale(0.9);
	}
	to {
		box-shadow: 0 30px 70px rgba(0, 0, 0, 0.5);
		opacity: 1;
		transform: none;
	}
}

@keyframes zoom {
	from {
		opacity: 0;
		transform: scale(0.8);
	}
	to {
		opacity: 1;
		transform: none;
	}
}

@keyframes appear-from-left {
	from {
		opacity: 0;
		transform: translateX(-25px);
	}
	to {
		opacity: 1;
		transform: none;
	}
}
