
export interface StringsType {
  domain: string;
  lang: string;
  completeness: number;
  'plural_forms': string;
  locale_data: {
    messages: Record<string, unknown>;
  };
};
export const strings: Record<string,StringsType> = {};

strings['he'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=4; plural=(n == 1) ? 0 : ((n == 2) ? 1 : ((n > 10 && n % 10 == 0) ? 2 : 3));",
        "lang": "he"
      },
      "Show debug info": [
        ""
      ],
      "Kyc completed": [
        ""
      ],
      "You will be redirected to nowhere": [
        ""
      ],
      "Internal error, please report.": [
        ""
      ],
      "Customer identification": [
        ""
      ],
      "Preferences": [
        ""
      ],
      "Show more information": [
        ""
      ],
      "Hide debug info": [
        ""
      ],
      "The request reached a timeout, check your connection.": [
        ""
      ],
      "The request was cancelled.": [
        ""
      ],
      "Too many requests were made to the server, and this action was throttled.": [
        ""
      ],
      "The server's response was malformed.": [
        ""
      ],
      "Could not complete the request due to a network problem.": [
        ""
      ],
      "Unexpected request error": [
        ""
      ],
      "Unexpected error": [
        ""
      ],
      "Show notifications": [
        ""
      ],
      "Open profile": [
        ""
      ],
      "Open settings": [
        ""
      ],
      "Menu": [
        ""
      ],
      "Close panel": [
        ""
      ],
      "Log out": [
        ""
      ],
      "Sites": [
        ""
      ],
      "Learn more about %1$s": [
        ""
      ],
      "never": [
        ""
      ],
      "in %1$s": [
        ""
      ],
      "%1$s ago": [
        ""
      ],
      "First page": [
        ""
      ],
      "Next": [
        ""
      ],
      "Request timeout": [
        ""
      ],
      "Request cancelled": [
        ""
      ],
      "Request throttled": [
        ""
      ],
      "Malformed response": [
        ""
      ],
      "Network error": [
        ""
      ],
      "Unhandled failure, please report. Code %1$s": [
        ""
      ],
      "Operation failed.": [
        ""
      ],
      "Unexpected error, this is likely a bug. Please report ": [
        ""
      ],
      "The operation failed.": [
        ""
      ],
      "The server version is not supported. Supported version \"%1$s\", server version \"%2$s\"": [
        ""
      ],
      "Terms of service": [
        ""
      ],
      "You must open/download the terms of service to proceed": [
        ""
      ],
      "View in Browser": [
        ""
      ],
      "Download PDF version": [
        ""
      ],
      "You must download to proceed": [
        ""
      ],
      "Do you accept the terms of service?": [
        ""
      ],
      "Challenger EMAIL": [
        ""
      ],
      "Challenge email ownership.": [
        ""
      ],
      "E-Mail": [
        ""
      ],
      "Challenger POSTAL": [
        ""
      ],
      "Challenge address ownership.": [
        ""
      ],
      "Name": [
        ""
      ],
      "Address": [
        ""
      ],
      "Country": [
        ""
      ],
      "Challenger SMS": [
        ""
      ],
      "Challenge phone number ownership.": [
        ""
      ],
      "Phone": [
        ""
      ],
      "Generic note": [
        ""
      ],
      "Free-form, generic note": [
        ""
      ],
      "Note / Memorandum": [
        ""
      ],
      "Free-form notes": [
        ""
      ],
      "Supplemental File Upload": [
        ""
      ],
      "Optional supplemental information for the establishment of the business relationship with the customer.": [
        ""
      ],
      "Supplemental Files": [
        ""
      ],
      "Description": [
        ""
      ],
      "File (PDF)": [
        ""
      ],
      "GmbH": [
        ""
      ],
      "GmbH i.G.": [
        ""
      ],
      "UG": [
        ""
      ],
      "UG i.G.": [
        ""
      ],
      "UHG": [
        ""
      ],
      "KG": [
        ""
      ],
      "e.V.": [
        ""
      ],
      "e.V. i.G.": [
        ""
      ],
      "eG": [
        ""
      ],
      "eG i.G.": [
        ""
      ],
      "PartG": [
        ""
      ],
      "e.K.": [
        ""
      ],
      "AG (nicht börsennotiert)": [
        ""
      ],
      "AG (börsennotiert)": [
        ""
      ],
      "GbR": [
        ""
      ],
      "n.e.V.": [
        ""
      ],
      "Partei": [
        ""
      ],
      "WEG": [
        ""
      ],
      "Anderes": [
        ""
      ],
      "invalid format": [
        ""
      ],
      "it can't be in the future": [
        ""
      ],
      "it can't be greater than 120 years": [
        ""
      ],
      "Invalid e-mail address": [
        ""
      ],
      "Invalid phone number": [
        ""
      ],
      "Personal Details": [
        ""
      ],
      "Personal details of the authorised representative.": [
        ""
      ],
      "First name(s)": [
        ""
      ],
      "As on your ID document": [
        ""
      ],
      "Last name": [
        ""
      ],
      "Date of birth": [
        ""
      ],
      "Nationality": [
        ""
      ],
      "Phone number": [
        ""
      ],
      "Company information": [
        ""
      ],
      "Name of the company": [
        ""
      ],
      "Legal form": [
        ""
      ],
      "Legal form (free-form entry for other)": [
        ""
      ],
      "Commercial register number": [
        ""
      ],
      "Seat of the register court": [
        ""
      ],
      "Founding date": [
        ""
      ],
      "Is the company a non-profit organization?": [
        ""
      ],
      "Industry": [
        ""
      ],
      "Company address": [
        ""
      ],
      "Tax information": [
        ""
      ],
      "Business identification number or tax number": [
        ""
      ],
      "Was the company incorporated in the USA or under US law?": [
        ""
      ],
      "Economically active or inactive?": [
        ""
      ],
      "Eligible for input tax deduction": [
        ""
      ],
      "Persons": [
        ""
      ],
      "Please list all legal representatives, shareholders/partners, and authorized signatories.": [
        ""
      ],
      "Legal representatives / shareholders / partners / authorized signatories": [
        ""
      ],
      "Type of representative": [
        ""
      ],
      "New Customer Identification": [
        ""
      ],
      "Confirmation": [
        ""
      ],
      "Confirmation of private customer / wallet.": [
        ""
      ],
      "Please confirm that you are a private customer that wants to redeem their wallet balance.": [
        ""
      ],
      "Upload multiple documents": [
        ""
      ],
      "Upload multiple documents.": [
        ""
      ],
      "Document upload (%1$s)": [
        ""
      ],
      "Simple form": [
        ""
      ],
      "Full Name": [
        ""
      ],
      "Birthdate": [
        ""
      ],
      "Comment": [
        ""
      ],
      "Resolution": [
        ""
      ],
      "New state": [
        ""
      ],
      "Frozen": [
        ""
      ],
      "Pending": [
        ""
      ],
      "Normal": [
        ""
      ],
      "New threshold": [
        ""
      ],
      "Establishing of the controlling person of operating legal entities and partnerships both not quoted on the stock exchange (K)": [
        ""
      ],
      "for operating legal entities and partnerships that are contracting partner as well as analogously for operating legal entities and partnership that are beneficial owners": [
        ""
      ],
      "Establishment of the controlling person (submitted by customer)": [
        ""
      ],
      "Contracting partner": [
        ""
      ],
      "Controlling person(s)": [
        ""
      ],
      "Reason for control": [
        ""
      ],
      "Holding 25% or more": [
        ""
      ],
      "The person(s) listed below is/are holding 25% or more of the contracting partner's shares (capital shares or voting rights)": [
        ""
      ],
      "Other way": [
        ""
      ],
      "If the capital shares or voting rights cannot be determined or in case there are no capital shares or voting rights 25% or more, the contracting partner hereby declares that the person(s) listed below is/are controlling the contracting partner in other ways": [
        ""
      ],
      "Managing director": [
        ""
      ],
      "In case this/these person(s) cannot be determined or this/these person(s) does/do not exist, the contracting partner hereby declares that the person(s) listed below is/are the managing director(s)": [
        ""
      ],
      "Can't be empty": [
        ""
      ],
      "Full name": [
        ""
      ],
      "Actual address of domicile": [
        ""
      ],
      "Fiduciary holding assets": [
        ""
      ],
      "Is a third person the beneficial owner of the assets held in the account/securities account?": [
        ""
      ],
      "Yes": [
        ""
      ],
      "No": [
        ""
      ],
      "Signature(s)": [
        ""
      ],
      "It is a criminal offence to deliberately provide false information on this form (article 251 of the Swiss Criminal Code, documents forgery)": [
        ""
      ],
      "The contracting partner hereby undertakes to inform automatically of any changes to the information contained herein.": [
        ""
      ],
      "Date": [
        ""
      ],
      "Establishment of the controlling person (submitted by AML officer)": [
        ""
      ],
      "Signed Declaration": [
        ""
      ],
      "Signed declaration by the customer": [
        ""
      ],
      "The uploaded document must contain the customer's signature on the beneficial owner declaration.": [
        ""
      ],
      "Signed Document": [
        ""
      ],
      "Special Clarifications": [
        ""
      ],
      "When a business relationship or transaction is associated with increased risk, appears unusual or evidence exists that the assets are the proceeds of a felony or a qualified tax offence, the member has to perform additional clarifications.": [
        ""
      ],
      "Reason for special clarifications": [
        ""
      ],
      "Reason": [
        ""
      ],
      "Description of the circumstances/transactions, which triggered the special clarifications": [
        ""
      ],
      "can't be empty": [
        ""
      ],
      "Used means of clarification": [
        ""
      ],
      "Means": [
        ""
      ],
      "Gathering of information from the customer, beneficial owner of the assets, controlling person": [
        ""
      ],
      "Consultation of generally accessible sources and databases": [
        ""
      ],
      "Enquiries with trustworthy persons": [
        ""
      ],
      "Other, which?": [
        ""
      ],
      "Other means of clarification:": [
        ""
      ],
      "Optional supplemental information for the special clarifications.": [
        ""
      ],
      "Summary and plausbility check of the gathered information": [
        ""
      ],
      "The results of the clarifications have to be documented and their plausibility has to be checked.": [
        ""
      ],
      "Summary": [
        ""
      ],
      "Gathered/Consulted documents": [
        ""
      ],
      "Result of the special clarification": [
        ""
      ],
      "Result": [
        ""
      ],
      "No suspicion": [
        ""
      ],
      "The plausibility of the circumstances could be checked, no reasonable suspicion pursuant to Art. 9 AMLA (possibly update of customer profile and/or risk profile)": [
        ""
      ],
      "Reasonable suspicion": [
        ""
      ],
      "Reasonable suspicion pursuant to Art. 9 AMLA, duty to file a report with MROS": [
        ""
      ],
      "Simple suspicion": [
        ""
      ],
      "Simple suspicion pursuant to Art. 305 Para. 2 StGB, right to notify MROS": [
        ""
      ],
      "Other, what?": [
        ""
      ],
      "Result clarification": [
        ""
      ],
      "The customer is the person with whom the member concludes the contract with regard to the financial service provided (civil law). Does the\n  member act as director of a domiciliary company, this domiciliary company is the customer.": [
        ""
      ],
      "Identification Form (customer)": [
        ""
      ],
      "The customer has to be identified on entering into a permanent business relationship or on concluding a cash transaction, which meets the according threshold.": [
        ""
      ],
      "Correspondence language:": [
        ""
      ],
      "English": [
        ""
      ],
      "German": [
        ""
      ],
      "French": [
        ""
      ],
      "Italian": [
        ""
      ],
      "Identification form (basic customer information)": [
        ""
      ],
      "Information on customer": [
        ""
      ],
      "Customer type": [
        ""
      ],
      "Natural person (incl. sole proprietors)": [
        ""
      ],
      "Legal entity": [
        ""
      ],
      "Applicable if customer is a natural person": [
        ""
      ],
      "Residential address in Switzerland": [
        ""
      ],
      "Telephone": [
        ""
      ],
      "E-mail": [
        ""
      ],
      "Copy of identification document": [
        ""
      ],
      "Only official government IDs (incl. passports) are accepted. Please scan both sides if applicable.": [
        ""
      ],
      "Sole proprietor": [
        ""
      ],
      "Information on customer (sole proprietor)": [
        ""
      ],
      "Applicable only if customer is a sole proprietor": [
        ""
      ],
      "Company name": [
        ""
      ],
      "Registered office": [
        ""
      ],
      "Company identification document": [
        ""
      ],
      "Information on customer (legal entity)": [
        ""
      ],
      "Applicable if customer is a legal entity": [
        ""
      ],
      "Domicile": [
        ""
      ],
      "Contact person": [
        ""
      ],
      "Copy of identification document (not older than 12 months)": [
        ""
      ],
      "Correspondence Preferences": [
        ""
      ],
      "Information on the natural persons who establish the business relationship for legal entities and partnerships": [
        ""
      ],
      "For legal entities and partnerships the identity of the natural persons who establish the business relationship must be verified.": [
        ""
      ],
      "Establishers of the legal relationship": [
        ""
      ],
      "Residential address": [
        ""
      ],
      "Signing authority of the person": [
        ""
      ],
      "Power of attorney arrangements": [
        ""
      ],
      "Other type of signing authority": [
        ""
      ],
      "Evidence of signing authority:": [
        ""
      ],
      "Company register extract": [
        ""
      ],
      "Mandate": [
        ""
      ],
      "Other": [
        ""
      ],
      "Specify other way of establishing signing authority:": [
        ""
      ],
      "Copy of document that serves as evidence of signing authority:": [
        ""
      ],
      "Customer classification": [
        ""
      ],
      "Classification for the establishment of the beneficial owner of the assets and/or controlling person": [
        ""
      ],
      "The customer is:": [
        ""
      ],
      "A natural person and there are no doubts that this person is the sole beneficial owner of the assets": [
        ""
      ],
      "An operational legal entity or partnership": [
        ""
      ],
      "A foundation (or a similar construct; incl. underlying companies)": [
        ""
      ],
      "A trust (incl. underlying companies)": [
        ""
      ],
      "A life insurance policy with separately managed accounts / securities accounts (so-called insurance wrappers)": [
        ""
      ],
      "Identification Form (acceptance)": [
        ""
      ],
      "Acceptance of business relationship": [
        ""
      ],
      "Date (conclusion of contract):": [
        ""
      ],
      "Accepted via:": [
        ""
      ],
      "Face to face": [
        ""
      ],
      "Authenticated copy of identification document": [
        ""
      ],
      "Residentail address validated": [
        ""
      ],
      "Further information:": [
        ""
      ],
      "Evaluation with regard to embargo procedures/terrorism lists on establishing the business relationship": [
        ""
      ],
      "Verification whether the customer, beneficial owners of the assets, controlling persons, authorized representatives or other involved persons are listed on an embargo/terrorism list.": [
        ""
      ],
      "Embargo/terrorism status:": [
        ""
      ],
      "Not listed on embargo/terrorism list.": [
        ""
      ],
      "Listed on embargo/terrorism list.": [
        ""
      ],
      "Embargo/terrorism information:": [
        ""
      ],
      "Verification date": [
        ""
      ],
      "Supplemental files": [
        ""
      ],
      "Risk Profile AMLA": [
        ""
      ],
      "Evaluation of politically exposed persons (PEP-Check)": [
        ""
      ],
      "This evaluation has to be completed by all members for every business relationship.": [
        ""
      ],
      "Foreign PEP": [
        ""
      ],
      "Is the customer, the beneficial owner or the controlling person or authorised representative a foreign PEP or closely related to such a person?": [
        ""
      ],
      "Domestic PEP": [
        ""
      ],
      "Is the customer, the beneficial owner or the controlling person or authorised representative a domestic PEP or closely related to such a person?": [
        ""
      ],
      "PEP of International Organisatons": [
        ""
      ],
      "Is the customer, the beneficial owner or the controlling person or authorised representative a PEP in International Organizations or closely related to such a person?": [
        ""
      ],
      "Acceptance date": [
        ""
      ],
      "When the decision of the Senior executive body on the acceptance of a business relationship with a PEP was obtain on.": [
        ""
      ],
      "Evaluation \"high risk\" or non-cooperative country": [
        ""
      ],
      "High-risk or non-cooperative country": [
        ""
      ],
      "Is the customer, the beneficial owner or the controlling person or authorised representative in a country considered by the FATF as high-risk or non-cooperative and for which FATF requires increased diligence?": [
        ""
      ],
      "When the decision of the Senior executive body on the acceptance of such a business relationship was obtained on.": [
        ""
      ],
      "Evaluation of business relationship risk": [
        ""
      ],
      "Country risk": [
        ""
      ],
      "Applicable country risk types": [
        ""
      ],
      "Nationality of the customer": [
        ""
      ],
      "Nationality of the beneficial owner of the assets": [
        ""
      ],
      "Domicile/residential address of the customer": [
        ""
      ],
      "Domicile/residential address of the beneficial owner of the assets": [
        ""
      ],
      "Domicile/residential address of the controlling person": [
        ""
      ],
      "Country risk level (nationality)": [
        ""
      ],
      "Risk category according to VQF country list (VQF doc. no. 902.4.1)": [
        ""
      ],
      "Low (Risk 0)": [
        ""
      ],
      "Risk 0 acc. to VQF country list (VQF doc. no. 902.4.1)": [
        ""
      ],
      "Medium (Risk 1)": [
        ""
      ],
      "Risk 1 acc. to VQF country list (VQF doc. no. 902.4.1)": [
        ""
      ],
      "High (Risk 2)": [
        ""
      ],
      "Risk 2 acc. to VQF country list (VQF doc. no. 902.4.1)": [
        ""
      ],
      "Country risk type (place of business activity)": [
        ""
      ],
      "Customer": [
        ""
      ],
      "Beneficial owner": [
        ""
      ],
      "Country risk level (business activity)": [
        ""
      ],
      "Industry risk": [
        ""
      ],
      "Industry risk source": [
        ""
      ],
      "Beneficial owner of the assets": [
        ""
      ],
      "Industry risk level": [
        ""
      ],
      "Clearly defined, transparent, easily comprehensible business activity well known to the member.": [
        ""
      ],
      "Business activity with a high level of cash transactions.": [
        ""
      ],
      "Business activity not well known to the member.": [
        ""
      ],
      "Trade in munitions/arms, raw gem stones/diamonds, jewellery, international trade in exotic animals, casino and lottery business, trade in erotic wares.": [
        ""
      ],
      "Member has no personal knowledge of the customer’s industry.": [
        ""
      ],
      "Contact risk": [
        ""
      ],
      "Type of contact to the customer/benefcial owner of the assets.": [
        ""
      ],
      "Contact risk level": [
        ""
      ],
      "Personal acquaintance between member and customer/beneficial owner of the assets over several years (at least 2) prior to entering into the business relationship": [
        ""
      ],
      "The customer/beneficial owner was not personally known to the member for several years (at least 2) prior to entering into the business relationship; however (a) no business was entered into in the absence of the customer/beneficial owner, or (b) the customer was at least introduced/brokered by a trusted third party.": [
        ""
      ],
      "The customer/beneficial owner was not personally known to the member and business was entered into in the absence of the former (relationship by correspondence) and the customer was not introduced/brokered by a trusted third party.": [
        ""
      ],
      "Summary evaluation": [
        ""
      ],
      "Justification for risk assessment": [
        ""
      ],
      "Risk classification": [
        ""
      ],
      "Conclusion whether the business relationship is with or without increased risk.": [
        ""
      ],
      "No high risk": [
        ""
      ],
      "High risk": [
        ""
      ],
      "When the decision of the Senior executive body on the acceptance of a business relationship with increased risk was obtained on.": [
        ""
      ],
      "Customer Profile": [
        ""
      ],
      "The information below has to refer to the persons from whom the assets originate ultimately (e.g. beneficial owner of the assets, founder/creator of a trust or foundation). Is the customer an operational legal entity or partnership the information may refer to the entity itself (not to the controlling person), unless the entity holds the assets in trust for a third party.": [
        ""
      ],
      "Business activity": [
        ""
      ],
      "Profession, business activities, etc. (former, current, potentially planned)": [
        ""
      ],
      "Financial circumstances": [
        ""
      ],
      "Income and assets, liabilities (estimated)": [
        ""
      ],
      "Origin of the deposited assets involved": [
        ""
      ],
      "Will the the customer deposit assets with Taler Operations AG?": [
        ""
      ],
      "Nature, amount and currency of deposited assets.": [
        ""
      ],
      "Category": [
        ""
      ],
      "Savings": [
        ""
      ],
      "Own business operations": [
        ""
      ],
      "Inheritance": [
        ""
      ],
      "Category clarification": [
        ""
      ],
      "Detail description of the origings/economical background of the assets involved in the business relationship": [
        ""
      ],
      "Nature and purpose of the business relationship": [
        ""
      ],
      "Purpose of the business relationship": [
        ""
      ],
      "Information on the planned development of the business relationship and the assets": [
        ""
      ],
      "Detail on usual business volume": [
        ""
      ],
      "Relationship with third parties": [
        ""
      ],
      "Relation of the customer to the beneficial owner, controlling persons, authorised signatories and other persons involved in the business relationship": [
        ""
      ],
      "Relation to other AMLA-Files": [
        ""
      ],
      "Introducer / agents / references": [
        ""
      ],
      "Further information": [
        ""
      ],
      "Other relevant information": [
        ""
      ],
      "Declaration of identity of the beneficial owner (A)": [
        ""
      ],
      "Declaration of identity of the beneficial owner": [
        ""
      ],
      "Beneficial owner details": [
        ""
      ],
      "Beneficial owner(s)": [
        ""
      ],
      "The person(s) listed below is/are the beneficial owner(s) of the assets involved in the business relationship. If the contracting partner is also the sole beneficial owner of the assets, the contracting partner's detail must be set out below": [
        ""
      ],
      "Domicile address": [
        ""
      ],
      "January": [
        ""
      ],
      "February": [
        ""
      ],
      "March": [
        ""
      ],
      "April": [
        ""
      ],
      "May": [
        ""
      ],
      "June": [
        ""
      ],
      "July": [
        ""
      ],
      "August": [
        ""
      ],
      "September": [
        ""
      ],
      "October": [
        ""
      ],
      "November": [
        ""
      ],
      "December": [
        ""
      ],
      "Previous year": [
        ""
      ],
      "Next year": [
        ""
      ],
      "Previous month": [
        ""
      ],
      "Next month": [
        ""
      ],
      "Cancel": [
        ""
      ],
      "Remove": [
        ""
      ],
      "Confirm": [
        ""
      ],
      "Add new...": [
        ""
      ],
      "No element found": [
        ""
      ],
      "%1$s (Classification level %2$s)": [
        ""
      ],
      "years": [
        ""
      ],
      "months": [
        ""
      ],
      "days": [
        ""
      ],
      "hours": [
        ""
      ],
      "minutes": [
        ""
      ],
      "seconds": [
        ""
      ],
      "Upload a file": [
        ""
      ],
      "Download a copy.": [
        ""
      ],
      "Missing fields": [
        ""
      ],
      "Simple comment": [
        ""
      ],
      "Terms of Service": [
        ""
      ],
      "Name and birthdate": [
        ""
      ],
      "required": [
        ""
      ],
      "Validation function failed. Contact developers %1$s": [
        ""
      ],
      "The form is too big for the server, try uploading smaller files.": [
        ""
      ],
      "The account was not found": [
        ""
      ],
      "Officer disabled or more recent decision was already submitted.": [
        ""
      ],
      "Debug information about the values in the form": [
        ""
      ],
      "Submit": [
        ""
      ],
      "Could not load context information": [
        ""
      ],
      "Could not find form": [
        ""
      ],
      "Form with id '%1$s' is not registered in this application.": [
        ""
      ],
      "Can't upload information": [
        ""
      ],
      "The KYC requirement doesn't have an ID": [
        ""
      ],
      "Mandatory identification requirements are satisfied": [
        ""
      ],
      "Process completed": [
        ""
      ],
      "You can close this window now.": [
        ""
      ],
      "Please, complete the following requirement:": [
        ""
      ],
      "Please, complete any of the following requirements:": [
        ""
      ],
      "not found": [
        ""
      ],
      "conflict": [
        ""
      ],
      "payload is too large": [
        ""
      ],
      "Link generated, you can proceed.": [
        ""
      ],
      "%1$s": [
        ""
      ],
      "Generate link": [
        ""
      ],
      "Open link": [
        ""
      ],
      "Form": [
        ""
      ],
      "Fill form": [
        ""
      ],
      "Trigger KYC balance": [
        ""
      ],
      "You can also use the formId in the UR after \"/test/show-forms/$FORM_ID\"": [
        ""
      ],
      "Trigger KYC Balance": [
        ""
      ],
      "Amount": [
        ""
      ],
      "No kyc triggered.": [
        ""
      ],
      "Forbidden.": [
        ""
      ],
      "Not found.": [
        ""
      ],
      "Conflict.": [
        ""
      ],
      "This actions will trigger wallet balance kyc above 1000000 threshold, the exchange should be properly configured to trigger the desired kyc flow.": [
        ""
      ],
      "Trigger TOPS Terms of service": [
        ""
      ],
      "Trigger GLS onboarding": [
        ""
      ],
      "Trigger VQF 902.1": [
        ""
      ],
      "Trigger VQF 902.4": [
        ""
      ],
      "Trigger VQF 902.5": [
        ""
      ],
      "Trigger VQF 902.9": [
        ""
      ],
      "Trigger VQF 902.11": [
        ""
      ],
      "Trigger VQF 902.12": [
        ""
      ],
      "Trigger VQF 902.13": [
        ""
      ],
      "Trigger VQF 902.14": [
        ""
      ],
      "Trigger VQF 902.15": [
        ""
      ],
      "Challenger test": [
        ""
      ],
      "Trigger VQF 902.9 customer": [
        ""
      ],
      "Trigger VQF 902.9 officer": [
        ""
      ],
      "The URL is wrong": [
        ""
      ],
      "Maybe the validation check expired.": [
        ""
      ],
      "Are you a cross-border truck driver?": [
        ""
      ],
      "How much is in your pockets?": [
        ""
      ],
      "KYC_REQUIREMENT_LINK_DESCRIPTION\u0004%1$s": [
        ""
      ],
      "KYC_REQUIREMENT_INFO_DESCRIPTION\u0004%1$s": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=4; plural=(n == 1) ? 0 : ((n == 2) ? 1 : ((n > 10 && n % 10 == 0) ? 2 : 3));",
  "lang": "he",
  "completeness": 0
};

strings['es'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "es"
      },
      "Show debug info": [
        "Mostrar informacion de depuracion"
      ],
      "Kyc completed": [
        "KYC completado"
      ],
      "You will be redirected to nowhere": [
        "Serias redirigido a ningun lugar"
      ],
      "Internal error, please report.": [
        "Error interno, por favor reporte."
      ],
      "Customer identification": [
        "Identificacion de cliente"
      ],
      "Preferences": [
        "Preferencias"
      ],
      "Show more information": [
        "Mostrar confirmación de extracción"
      ],
      "Hide debug info": [
        "Mostrar informacion de depuracion"
      ],
      "The request reached a timeout, check your connection.": [
        ""
      ],
      "The request was cancelled.": [
        ""
      ],
      "Too many requests were made to the server, and this action was throttled.": [
        ""
      ],
      "The server's response was malformed.": [
        ""
      ],
      "Could not complete the request due to a network problem.": [
        ""
      ],
      "Unexpected request error": [
        ""
      ],
      "Unexpected error": [
        ""
      ],
      "Show notifications": [
        "Identificacion de cliente"
      ],
      "Open profile": [
        ""
      ],
      "Open settings": [
        ""
      ],
      "Menu": [
        ""
      ],
      "Close panel": [
        ""
      ],
      "Log out": [
        ""
      ],
      "Sites": [
        ""
      ],
      "Learn more about %1$s": [
        ""
      ],
      "never": [
        ""
      ],
      "in %1$s": [
        "%1$s"
      ],
      "%1$s ago": [
        "%1$s"
      ],
      "First page": [
        ""
      ],
      "Next": [
        ""
      ],
      "Request timeout": [
        ""
      ],
      "Request cancelled": [
        ""
      ],
      "Request throttled": [
        ""
      ],
      "Malformed response": [
        ""
      ],
      "Network error": [
        ""
      ],
      "Unhandled failure, please report. Code %1$s": [
        ""
      ],
      "Operation failed.": [
        ""
      ],
      "Unexpected error, this is likely a bug. Please report ": [
        ""
      ],
      "The operation failed.": [
        ""
      ],
      "The server version is not supported. Supported version \"%1$s\", server version \"%2$s\"": [
        ""
      ],
      "Terms of service": [
        "Terminos de servicio"
      ],
      "You must open/download the terms of service to proceed": [
        ""
      ],
      "View in Browser": [
        ""
      ],
      "Download PDF version": [
        ""
      ],
      "You must download to proceed": [
        ""
      ],
      "Do you accept the terms of service?": [
        "¿Acepta los términos de servicio?"
      ],
      "Challenger EMAIL": [
        "Probar challenger"
      ],
      "Challenge email ownership.": [
        ""
      ],
      "E-Mail": [
        ""
      ],
      "Challenger POSTAL": [
        "Probar challenger"
      ],
      "Challenge address ownership.": [
        "Probar challenger"
      ],
      "Name": [
        "Nombre completo"
      ],
      "Address": [
        ""
      ],
      "Country": [
        "Monto"
      ],
      "Challenger SMS": [
        "Probar challenger"
      ],
      "Challenge phone number ownership.": [
        ""
      ],
      "Phone": [
        ""
      ],
      "Generic note": [
        ""
      ],
      "Free-form, generic note": [
        ""
      ],
      "Note / Memorandum": [
        ""
      ],
      "Free-form notes": [
        ""
      ],
      "Supplemental File Upload": [
        ""
      ],
      "Optional supplemental information for the establishment of the business relationship with the customer.": [
        ""
      ],
      "Supplemental Files": [
        ""
      ],
      "Description": [
        ""
      ],
      "File (PDF)": [
        ""
      ],
      "GmbH": [
        ""
      ],
      "GmbH i.G.": [
        ""
      ],
      "UG": [
        ""
      ],
      "UG i.G.": [
        ""
      ],
      "UHG": [
        ""
      ],
      "KG": [
        ""
      ],
      "e.V.": [
        ""
      ],
      "e.V. i.G.": [
        ""
      ],
      "eG": [
        ""
      ],
      "eG i.G.": [
        ""
      ],
      "PartG": [
        ""
      ],
      "e.K.": [
        ""
      ],
      "AG (nicht börsennotiert)": [
        ""
      ],
      "AG (börsennotiert)": [
        ""
      ],
      "GbR": [
        ""
      ],
      "n.e.V.": [
        ""
      ],
      "Partei": [
        ""
      ],
      "WEG": [
        ""
      ],
      "Anderes": [
        ""
      ],
      "invalid format": [
        "Llenar el formulario"
      ],
      "it can't be in the future": [
        ""
      ],
      "it can't be greater than 120 years": [
        ""
      ],
      "Invalid e-mail address": [
        ""
      ],
      "Invalid phone number": [
        ""
      ],
      "Personal Details": [
        ""
      ],
      "Personal details of the authorised representative.": [
        ""
      ],
      "First name(s)": [
        ""
      ],
      "As on your ID document": [
        ""
      ],
      "Last name": [
        ""
      ],
      "Date of birth": [
        ""
      ],
      "Nationality": [
        ""
      ],
      "Phone number": [
        ""
      ],
      "Company information": [
        ""
      ],
      "Name of the company": [
        ""
      ],
      "Legal form": [
        "Llenar el formulario"
      ],
      "Legal form (free-form entry for other)": [
        ""
      ],
      "Commercial register number": [
        ""
      ],
      "Seat of the register court": [
        ""
      ],
      "Founding date": [
        ""
      ],
      "Is the company a non-profit organization?": [
        ""
      ],
      "Industry": [
        ""
      ],
      "Company address": [
        ""
      ],
      "Tax information": [
        ""
      ],
      "Business identification number or tax number": [
        ""
      ],
      "Was the company incorporated in the USA or under US law?": [
        ""
      ],
      "Economically active or inactive?": [
        ""
      ],
      "Eligible for input tax deduction": [
        ""
      ],
      "Persons": [
        ""
      ],
      "Please list all legal representatives, shareholders/partners, and authorized signatories.": [
        ""
      ],
      "Legal representatives / shareholders / partners / authorized signatories": [
        ""
      ],
      "Type of representative": [
        ""
      ],
      "New Customer Identification": [
        "Identificacion de cliente"
      ],
      "Confirmation": [
        ""
      ],
      "Confirmation of private customer / wallet.": [
        ""
      ],
      "Please confirm that you are a private customer that wants to redeem their wallet balance.": [
        ""
      ],
      "Upload multiple documents": [
        ""
      ],
      "Upload multiple documents.": [
        ""
      ],
      "Document upload (%1$s)": [
        ""
      ],
      "Simple form": [
        "Formulario simple"
      ],
      "Full Name": [
        "Nombre completo"
      ],
      "Birthdate": [
        "Fecha de nacimiento"
      ],
      "Comment": [
        "Comentario"
      ],
      "Resolution": [
        "Resolucion"
      ],
      "New state": [
        "Nuevo estado"
      ],
      "Frozen": [
        "Congelado"
      ],
      "Pending": [
        "Pendiente"
      ],
      "Normal": [
        "Normal"
      ],
      "New threshold": [
        "Nuevo limite"
      ],
      "Establishing of the controlling person of operating legal entities and partnerships both not quoted on the stock exchange (K)": [
        "Constitución de la persona controladora de las personas jurídicas y sociedades operativas que no cotizan en bolsa (K)"
      ],
      "for operating legal entities and partnerships that are contracting partner as well as analogously for operating legal entities and partnership that are beneficial owners": [
        "para personas jurídicas operativas y sociedades que sean socios contratantes, así como análogamente para personas jurídicas operativas y sociedades que sean beneficiarios finales"
      ],
      "Establishment of the controlling person (submitted by customer)": [
        ""
      ],
      "Contracting partner": [
        ""
      ],
      "Controlling person(s)": [
        ""
      ],
      "Reason for control": [
        ""
      ],
      "Holding 25% or more": [
        ""
      ],
      "The person(s) listed below is/are holding 25% or more of the contracting partner's shares (capital shares or voting rights)": [
        ""
      ],
      "Other way": [
        ""
      ],
      "If the capital shares or voting rights cannot be determined or in case there are no capital shares or voting rights 25% or more, the contracting partner hereby declares that the person(s) listed below is/are controlling the contracting partner in other ways": [
        ""
      ],
      "Managing director": [
        ""
      ],
      "In case this/these person(s) cannot be determined or this/these person(s) does/do not exist, the contracting partner hereby declares that the person(s) listed below is/are the managing director(s)": [
        ""
      ],
      "Can't be empty": [
        ""
      ],
      "Full name": [
        "Nombre completo"
      ],
      "Actual address of domicile": [
        ""
      ],
      "Fiduciary holding assets": [
        ""
      ],
      "Is a third person the beneficial owner of the assets held in the account/securities account?": [
        ""
      ],
      "Yes": [
        "Si"
      ],
      "No": [
        "No"
      ],
      "Signature(s)": [
        ""
      ],
      "It is a criminal offence to deliberately provide false information on this form (article 251 of the Swiss Criminal Code, documents forgery)": [
        ""
      ],
      "The contracting partner hereby undertakes to inform automatically of any changes to the information contained herein.": [
        ""
      ],
      "Date": [
        ""
      ],
      "Establishment of the controlling person (submitted by AML officer)": [
        ""
      ],
      "Signed Declaration": [
        ""
      ],
      "Signed declaration by the customer": [
        ""
      ],
      "The uploaded document must contain the customer's signature on the beneficial owner declaration.": [
        ""
      ],
      "Signed Document": [
        "Comentario simple"
      ],
      "Special Clarifications": [
        "Clarificaciones especiales"
      ],
      "When a business relationship or transaction is associated with increased risk, appears unusual or evidence exists that the assets are the proceeds of a felony or a qualified tax offence, the member has to perform additional clarifications.": [
        "Cuando una relación o transacción comercial esté asociada a un mayor riesgo, parezca inusual o exista evidencia de que los activos son producto de un delito grave o un delito fiscal calificado, el miembro deberá realizar aclaraciones adicionales."
      ],
      "Reason for special clarifications": [
        "Clarificaciones especiales"
      ],
      "Reason": [
        ""
      ],
      "Description of the circumstances/transactions, which triggered the special clarifications": [
        ""
      ],
      "can't be empty": [
        ""
      ],
      "Used means of clarification": [
        "Clarificaciones especiales"
      ],
      "Means": [
        ""
      ],
      "Gathering of information from the customer, beneficial owner of the assets, controlling person": [
        ""
      ],
      "Consultation of generally accessible sources and databases": [
        ""
      ],
      "Enquiries with trustworthy persons": [
        ""
      ],
      "Other, which?": [
        ""
      ],
      "Other means of clarification:": [
        ""
      ],
      "Optional supplemental information for the special clarifications.": [
        ""
      ],
      "Summary and plausbility check of the gathered information": [
        ""
      ],
      "The results of the clarifications have to be documented and their plausibility has to be checked.": [
        ""
      ],
      "Summary": [
        ""
      ],
      "Gathered/Consulted documents": [
        ""
      ],
      "Result of the special clarification": [
        "Clarificaciones especiales"
      ],
      "Result": [
        ""
      ],
      "No suspicion": [
        ""
      ],
      "The plausibility of the circumstances could be checked, no reasonable suspicion pursuant to Art. 9 AMLA (possibly update of customer profile and/or risk profile)": [
        ""
      ],
      "Reasonable suspicion": [
        ""
      ],
      "Reasonable suspicion pursuant to Art. 9 AMLA, duty to file a report with MROS": [
        ""
      ],
      "Simple suspicion": [
        "Comentario simple"
      ],
      "Simple suspicion pursuant to Art. 305 Para. 2 StGB, right to notify MROS": [
        ""
      ],
      "Other, what?": [
        ""
      ],
      "Result clarification": [
        "Clarificaciones especiales"
      ],
      "The customer is the person with whom the member concludes the contract with regard to the financial service provided (civil law). Does the\n  member act as director of a domiciliary company, this domiciliary company is the customer.": [
        ""
      ],
      "Identification Form (customer)": [
        "Formulario de identificacion"
      ],
      "The customer has to be identified on entering into a permanent business relationship or on concluding a cash transaction, which meets the according threshold.": [
        "El cliente deberá ser identificado al iniciar una relación comercial permanente o al concluir una transacción en efectivo que cumpla el umbral correspondiente."
      ],
      "Correspondence language:": [
        ""
      ],
      "English": [
        ""
      ],
      "German": [
        ""
      ],
      "French": [
        ""
      ],
      "Italian": [
        ""
      ],
      "Identification form (basic customer information)": [
        ""
      ],
      "Information on customer": [
        ""
      ],
      "Customer type": [
        "Perfil del cliente"
      ],
      "Natural person (incl. sole proprietors)": [
        ""
      ],
      "Legal entity": [
        ""
      ],
      "Applicable if customer is a natural person": [
        ""
      ],
      "Residential address in Switzerland": [
        ""
      ],
      "Telephone": [
        ""
      ],
      "E-mail": [
        ""
      ],
      "Copy of identification document": [
        "Formulario de identificacion"
      ],
      "Only official government IDs (incl. passports) are accepted. Please scan both sides if applicable.": [
        ""
      ],
      "Sole proprietor": [
        ""
      ],
      "Information on customer (sole proprietor)": [
        ""
      ],
      "Applicable only if customer is a sole proprietor": [
        ""
      ],
      "Company name": [
        ""
      ],
      "Registered office": [
        ""
      ],
      "Company identification document": [
        "Identificacion de cliente"
      ],
      "Information on customer (legal entity)": [
        ""
      ],
      "Applicable if customer is a legal entity": [
        ""
      ],
      "Domicile": [
        ""
      ],
      "Contact person": [
        ""
      ],
      "Copy of identification document (not older than 12 months)": [
        ""
      ],
      "Correspondence Preferences": [
        ""
      ],
      "Information on the natural persons who establish the business relationship for legal entities and partnerships": [
        ""
      ],
      "For legal entities and partnerships the identity of the natural persons who establish the business relationship must be verified.": [
        ""
      ],
      "Establishers of the legal relationship": [
        ""
      ],
      "Residential address": [
        ""
      ],
      "Signing authority of the person": [
        ""
      ],
      "Power of attorney arrangements": [
        ""
      ],
      "Other type of signing authority": [
        ""
      ],
      "Evidence of signing authority:": [
        ""
      ],
      "Company register extract": [
        ""
      ],
      "Mandate": [
        ""
      ],
      "Other": [
        ""
      ],
      "Specify other way of establishing signing authority:": [
        ""
      ],
      "Copy of document that serves as evidence of signing authority:": [
        ""
      ],
      "Customer classification": [
        "Identificacion de cliente"
      ],
      "Classification for the establishment of the beneficial owner of the assets and/or controlling person": [
        ""
      ],
      "The customer is:": [
        ""
      ],
      "A natural person and there are no doubts that this person is the sole beneficial owner of the assets": [
        ""
      ],
      "An operational legal entity or partnership": [
        ""
      ],
      "A foundation (or a similar construct; incl. underlying companies)": [
        ""
      ],
      "A trust (incl. underlying companies)": [
        ""
      ],
      "A life insurance policy with separately managed accounts / securities accounts (so-called insurance wrappers)": [
        "Información sobre pólizas de seguro de vida con cuentas de gestión separada/cuentas de valores (las denominadas \"envoltorios de seguros\")"
      ],
      "Identification Form (acceptance)": [
        "Formulario de identificacion"
      ],
      "Acceptance of business relationship": [
        ""
      ],
      "Date (conclusion of contract):": [
        ""
      ],
      "Accepted via:": [
        ""
      ],
      "Face to face": [
        ""
      ],
      "Authenticated copy of identification document": [
        ""
      ],
      "Residentail address validated": [
        ""
      ],
      "Further information:": [
        "Identificacion de cliente"
      ],
      "Evaluation with regard to embargo procedures/terrorism lists on establishing the business relationship": [
        ""
      ],
      "Verification whether the customer, beneficial owners of the assets, controlling persons, authorized representatives or other involved persons are listed on an embargo/terrorism list.": [
        ""
      ],
      "Embargo/terrorism status:": [
        ""
      ],
      "Not listed on embargo/terrorism list.": [
        ""
      ],
      "Listed on embargo/terrorism list.": [
        ""
      ],
      "Embargo/terrorism information:": [
        ""
      ],
      "Verification date": [
        "Formulario de identificacion"
      ],
      "Supplemental files": [
        ""
      ],
      "Risk Profile AMLA": [
        "Perfil de riesgo AMLA"
      ],
      "Evaluation of politically exposed persons (PEP-Check)": [
        ""
      ],
      "This evaluation has to be completed by all members for every business relationship.": [
        ""
      ],
      "Foreign PEP": [
        ""
      ],
      "Is the customer, the beneficial owner or the controlling person or authorised representative a foreign PEP or closely related to such a person?": [
        ""
      ],
      "Domestic PEP": [
        ""
      ],
      "Is the customer, the beneficial owner or the controlling person or authorised representative a domestic PEP or closely related to such a person?": [
        ""
      ],
      "PEP of International Organisatons": [
        ""
      ],
      "Is the customer, the beneficial owner or the controlling person or authorised representative a PEP in International Organizations or closely related to such a person?": [
        ""
      ],
      "Acceptance date": [
        ""
      ],
      "When the decision of the Senior executive body on the acceptance of a business relationship with a PEP was obtain on.": [
        ""
      ],
      "Evaluation \"high risk\" or non-cooperative country": [
        ""
      ],
      "High-risk or non-cooperative country": [
        ""
      ],
      "Is the customer, the beneficial owner or the controlling person or authorised representative in a country considered by the FATF as high-risk or non-cooperative and for which FATF requires increased diligence?": [
        ""
      ],
      "When the decision of the Senior executive body on the acceptance of such a business relationship was obtained on.": [
        ""
      ],
      "Evaluation of business relationship risk": [
        ""
      ],
      "Country risk": [
        ""
      ],
      "Applicable country risk types": [
        ""
      ],
      "Nationality of the customer": [
        ""
      ],
      "Nationality of the beneficial owner of the assets": [
        "Declaración de identidad del beneficiario efectivo (A)"
      ],
      "Domicile/residential address of the customer": [
        ""
      ],
      "Domicile/residential address of the beneficial owner of the assets": [
        ""
      ],
      "Domicile/residential address of the controlling person": [
        ""
      ],
      "Country risk level (nationality)": [
        ""
      ],
      "Risk category according to VQF country list (VQF doc. no. 902.4.1)": [
        ""
      ],
      "Low (Risk 0)": [
        ""
      ],
      "Risk 0 acc. to VQF country list (VQF doc. no. 902.4.1)": [
        ""
      ],
      "Medium (Risk 1)": [
        ""
      ],
      "Risk 1 acc. to VQF country list (VQF doc. no. 902.4.1)": [
        ""
      ],
      "High (Risk 2)": [
        ""
      ],
      "Risk 2 acc. to VQF country list (VQF doc. no. 902.4.1)": [
        ""
      ],
      "Country risk type (place of business activity)": [
        ""
      ],
      "Customer": [
        "Perfil del cliente"
      ],
      "Beneficial owner": [
        ""
      ],
      "Country risk level (business activity)": [
        ""
      ],
      "Industry risk": [
        ""
      ],
      "Industry risk source": [
        ""
      ],
      "Beneficial owner of the assets": [
        ""
      ],
      "Industry risk level": [
        ""
      ],
      "Clearly defined, transparent, easily comprehensible business activity well known to the member.": [
        ""
      ],
      "Business activity with a high level of cash transactions.": [
        ""
      ],
      "Business activity not well known to the member.": [
        ""
      ],
      "Trade in munitions/arms, raw gem stones/diamonds, jewellery, international trade in exotic animals, casino and lottery business, trade in erotic wares.": [
        ""
      ],
      "Member has no personal knowledge of the customer’s industry.": [
        ""
      ],
      "Contact risk": [
        ""
      ],
      "Type of contact to the customer/benefcial owner of the assets.": [
        ""
      ],
      "Contact risk level": [
        ""
      ],
      "Personal acquaintance between member and customer/beneficial owner of the assets over several years (at least 2) prior to entering into the business relationship": [
        ""
      ],
      "The customer/beneficial owner was not personally known to the member for several years (at least 2) prior to entering into the business relationship; however (a) no business was entered into in the absence of the customer/beneficial owner, or (b) the customer was at least introduced/brokered by a trusted third party.": [
        ""
      ],
      "The customer/beneficial owner was not personally known to the member and business was entered into in the absence of the former (relationship by correspondence) and the customer was not introduced/brokered by a trusted third party.": [
        ""
      ],
      "Summary evaluation": [
        ""
      ],
      "Justification for risk assessment": [
        ""
      ],
      "Risk classification": [
        "Clarificaciones especiales"
      ],
      "Conclusion whether the business relationship is with or without increased risk.": [
        ""
      ],
      "No high risk": [
        ""
      ],
      "High risk": [
        ""
      ],
      "When the decision of the Senior executive body on the acceptance of a business relationship with increased risk was obtained on.": [
        ""
      ],
      "Customer Profile": [
        "Perfil del cliente"
      ],
      "The information below has to refer to the persons from whom the assets originate ultimately (e.g. beneficial owner of the assets, founder/creator of a trust or foundation). Is the customer an operational legal entity or partnership the information may refer to the entity itself (not to the controlling person), unless the entity holds the assets in trust for a third party.": [
        "La siguiente información debe referirse a las personas de las que proceden en última instancia los activos (por ejemplo, el beneficiario efectivo de los activos, el fundador o creador de un fideicomiso o fundación). Si el cliente es una entidad jurídica operativa o una sociedad, la información puede referirse a la propia entidad (no a la persona que ejerce el control), a menos que la entidad mantenga los activos en fideicomiso para un tercero."
      ],
      "Business activity": [
        ""
      ],
      "Profession, business activities, etc. (former, current, potentially planned)": [
        ""
      ],
      "Financial circumstances": [
        ""
      ],
      "Income and assets, liabilities (estimated)": [
        ""
      ],
      "Origin of the deposited assets involved": [
        ""
      ],
      "Will the the customer deposit assets with Taler Operations AG?": [
        ""
      ],
      "Nature, amount and currency of deposited assets.": [
        ""
      ],
      "Category": [
        ""
      ],
      "Savings": [
        ""
      ],
      "Own business operations": [
        ""
      ],
      "Inheritance": [
        ""
      ],
      "Category clarification": [
        "Clarificaciones especiales"
      ],
      "Detail description of the origings/economical background of the assets involved in the business relationship": [
        ""
      ],
      "Nature and purpose of the business relationship": [
        ""
      ],
      "Purpose of the business relationship": [
        ""
      ],
      "Information on the planned development of the business relationship and the assets": [
        ""
      ],
      "Detail on usual business volume": [
        ""
      ],
      "Relationship with third parties": [
        ""
      ],
      "Relation of the customer to the beneficial owner, controlling persons, authorised signatories and other persons involved in the business relationship": [
        ""
      ],
      "Relation to other AMLA-Files": [
        ""
      ],
      "Introducer / agents / references": [
        ""
      ],
      "Further information": [
        "Identificacion de cliente"
      ],
      "Other relevant information": [
        ""
      ],
      "Declaration of identity of the beneficial owner (A)": [
        "Declaración de identidad del beneficiario efectivo (A)"
      ],
      "Declaration of identity of the beneficial owner": [
        "Declaración de identidad del beneficiario efectivo (A)"
      ],
      "Beneficial owner details": [
        ""
      ],
      "Beneficial owner(s)": [
        ""
      ],
      "The person(s) listed below is/are the beneficial owner(s) of the assets involved in the business relationship. If the contracting partner is also the sole beneficial owner of the assets, the contracting partner's detail must be set out below": [
        ""
      ],
      "Domicile address": [
        ""
      ],
      "January": [
        ""
      ],
      "February": [
        ""
      ],
      "March": [
        ""
      ],
      "April": [
        ""
      ],
      "May": [
        ""
      ],
      "June": [
        ""
      ],
      "July": [
        ""
      ],
      "August": [
        ""
      ],
      "September": [
        ""
      ],
      "October": [
        ""
      ],
      "November": [
        ""
      ],
      "December": [
        ""
      ],
      "Previous year": [
        ""
      ],
      "Next year": [
        ""
      ],
      "Previous month": [
        ""
      ],
      "Next month": [
        ""
      ],
      "Cancel": [
        "Cancelar"
      ],
      "Remove": [
        ""
      ],
      "Confirm": [
        ""
      ],
      "Add new...": [
        ""
      ],
      "No element found": [
        "No encontrado"
      ],
      "%1$s (Classification level %2$s)": [
        ""
      ],
      "years": [
        ""
      ],
      "months": [
        ""
      ],
      "days": [
        ""
      ],
      "hours": [
        ""
      ],
      "minutes": [
        ""
      ],
      "seconds": [
        ""
      ],
      "Upload a file": [
        ""
      ],
      "Download a copy.": [
        ""
      ],
      "Missing fields": [
        ""
      ],
      "Simple comment": [
        "Comentario simple"
      ],
      "Terms of Service": [
        "Terminos de servicio"
      ],
      "Name and birthdate": [
        "Nombre y fecha de nacimiento"
      ],
      "required": [
        "No se require KYC"
      ],
      "Validation function failed. Contact developers %1$s": [
        ""
      ],
      "The form is too big for the server, try uploading smaller files.": [
        "El formulario es demasiado grande para el servidor, intente cargar archivos más pequeños."
      ],
      "The account was not found": [
        "No se encontro la cuenta"
      ],
      "Officer disabled or more recent decision was already submitted.": [
        "Oficial incapacitado o decisión más reciente ya fue presentada."
      ],
      "Debug information about the values in the form": [
        "Información de depuración sobre los valores en el formulario"
      ],
      "Submit": [
        "Enviar"
      ],
      "Could not load context information": [
        ""
      ],
      "Could not find form": [
        ""
      ],
      "Form with id '%1$s' is not registered in this application.": [
        ""
      ],
      "Can't upload information": [
        ""
      ],
      "The KYC requirement doesn't have an ID": [
        ""
      ],
      "Mandatory identification requirements are satisfied": [
        "Se cumplen los requisitos de identificación obligatorios"
      ],
      "Process completed": [
        "Proceso terminado"
      ],
      "You can close this window now.": [
        "Puede cerrar esta ventana ahora."
      ],
      "Please, complete the following requirement:": [
        "Por favor, complete el siguiente requerimiento:"
      ],
      "Please, complete any of the following requirements:": [
        "Por favor, complete alguno de los siguientes requisitos:"
      ],
      "not found": [
        "No encontrado"
      ],
      "conflict": [
        "Conflicto"
      ],
      "payload is too large": [
        ""
      ],
      "Link generated, you can proceed.": [
        ""
      ],
      "%1$s": [
        "%1$s"
      ],
      "Generate link": [
        ""
      ],
      "Open link": [
        ""
      ],
      "Form": [
        "Formulario"
      ],
      "Fill form": [
        "Llenar el formulario"
      ],
      "Trigger KYC balance": [
        "Iniciar balance KYC"
      ],
      "You can also use the formId in the UR after \"/test/show-forms/$FORM_ID\"": [
        ""
      ],
      "Trigger KYC Balance": [
        "Iniciar balance KYC"
      ],
      "Amount": [
        "Monto"
      ],
      "No kyc triggered.": [
        "No se require KYC"
      ],
      "Forbidden.": [
        ""
      ],
      "Not found.": [
        "No encontrado"
      ],
      "Conflict.": [
        "Conflicto"
      ],
      "This actions will trigger wallet balance kyc above 1000000 threshold, the exchange should be properly configured to trigger the desired kyc flow.": [
        "Estas acciones activarán el saldo de KYC de la billetera por encima del umbral de 1000000; el intercambio debe estar configurado correctamente para activar el flujo de KYC deseado."
      ],
      "Trigger TOPS Terms of service": [
        "Iniciar TOPS Terms o service"
      ],
      "Trigger GLS onboarding": [
        "Iniciar GLS onboarding"
      ],
      "Trigger VQF 902.1": [
        "Iniciar VQF 902.1"
      ],
      "Trigger VQF 902.4": [
        "Iniciar VQF 902.4"
      ],
      "Trigger VQF 902.5": [
        "Iniciar VQF 902.5"
      ],
      "Trigger VQF 902.9": [
        "Iniciar VQF 902.9"
      ],
      "Trigger VQF 902.11": [
        "Iniciar VQF 902.11"
      ],
      "Trigger VQF 902.12": [
        "Iniciar VQF 902.12"
      ],
      "Trigger VQF 902.13": [
        "Iniciar VQF 902.13"
      ],
      "Trigger VQF 902.14": [
        "Iniciar VQF 902.14"
      ],
      "Trigger VQF 902.15": [
        "Iniciar VQF 902.15"
      ],
      "Challenger test": [
        "Probar challenger"
      ],
      "Trigger VQF 902.9 customer": [
        "Iniciar VQF 902.9"
      ],
      "Trigger VQF 902.9 officer": [
        "Iniciar VQF 902.9"
      ],
      "The URL is wrong": [
        "La URL es incorrecta"
      ],
      "Maybe the validation check expired.": [
        "Quizas la verificacion expiro."
      ],
      "Are you a cross-border truck driver?": [
        "Sos un conductor de camiones limitrofe?"
      ],
      "How much is in your pockets?": [
        "Cuanto dinero hay en tus bolsillos?"
      ],
      "KYC_REQUIREMENT_LINK_DESCRIPTION\u0004%1$s": [
        "%1$s"
      ],
      "KYC_REQUIREMENT_INFO_DESCRIPTION\u0004%1$s": [
        "%1$s"
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "es",
  "completeness": 26
};

strings['en'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "es"
      },
      "Show debug info": [
        ""
      ],
      "Kyc completed": [
        ""
      ],
      "You will be redirected to nowhere": [
        ""
      ],
      "Internal error, please report.": [
        ""
      ],
      "Customer identification": [
        ""
      ],
      "Preferences": [
        ""
      ],
      "Show more information": [
        "Mostrar confirmación de extracción"
      ],
      "Hide debug info": [
        ""
      ],
      "The request reached a timeout, check your connection.": [
        ""
      ],
      "The request was cancelled.": [
        ""
      ],
      "Too many requests were made to the server, and this action was throttled.": [
        ""
      ],
      "The server's response was malformed.": [
        ""
      ],
      "Could not complete the request due to a network problem.": [
        ""
      ],
      "Unexpected request error": [
        ""
      ],
      "Unexpected error": [
        ""
      ],
      "Show notifications": [
        ""
      ],
      "Open profile": [
        ""
      ],
      "Open settings": [
        ""
      ],
      "Menu": [
        ""
      ],
      "Close panel": [
        ""
      ],
      "Log out": [
        ""
      ],
      "Sites": [
        ""
      ],
      "Learn more about %1$s": [
        ""
      ],
      "never": [
        ""
      ],
      "in %1$s": [
        ""
      ],
      "%1$s ago": [
        ""
      ],
      "First page": [
        ""
      ],
      "Next": [
        ""
      ],
      "Request timeout": [
        ""
      ],
      "Request cancelled": [
        ""
      ],
      "Request throttled": [
        ""
      ],
      "Malformed response": [
        ""
      ],
      "Network error": [
        ""
      ],
      "Unhandled failure, please report. Code %1$s": [
        ""
      ],
      "Operation failed.": [
        ""
      ],
      "Unexpected error, this is likely a bug. Please report ": [
        ""
      ],
      "The operation failed.": [
        ""
      ],
      "The server version is not supported. Supported version \"%1$s\", server version \"%2$s\"": [
        ""
      ],
      "Terms of service": [
        ""
      ],
      "You must open/download the terms of service to proceed": [
        ""
      ],
      "View in Browser": [
        ""
      ],
      "Download PDF version": [
        ""
      ],
      "You must download to proceed": [
        ""
      ],
      "Do you accept the terms of service?": [
        ""
      ],
      "Challenger EMAIL": [
        ""
      ],
      "Challenge email ownership.": [
        ""
      ],
      "E-Mail": [
        ""
      ],
      "Challenger POSTAL": [
        ""
      ],
      "Challenge address ownership.": [
        ""
      ],
      "Name": [
        ""
      ],
      "Address": [
        ""
      ],
      "Country": [
        ""
      ],
      "Challenger SMS": [
        ""
      ],
      "Challenge phone number ownership.": [
        ""
      ],
      "Phone": [
        ""
      ],
      "Generic note": [
        ""
      ],
      "Free-form, generic note": [
        ""
      ],
      "Note / Memorandum": [
        ""
      ],
      "Free-form notes": [
        ""
      ],
      "Supplemental File Upload": [
        ""
      ],
      "Optional supplemental information for the establishment of the business relationship with the customer.": [
        ""
      ],
      "Supplemental Files": [
        ""
      ],
      "Description": [
        ""
      ],
      "File (PDF)": [
        ""
      ],
      "GmbH": [
        ""
      ],
      "GmbH i.G.": [
        ""
      ],
      "UG": [
        ""
      ],
      "UG i.G.": [
        ""
      ],
      "UHG": [
        ""
      ],
      "KG": [
        ""
      ],
      "e.V.": [
        ""
      ],
      "e.V. i.G.": [
        ""
      ],
      "eG": [
        ""
      ],
      "eG i.G.": [
        ""
      ],
      "PartG": [
        ""
      ],
      "e.K.": [
        ""
      ],
      "AG (nicht börsennotiert)": [
        ""
      ],
      "AG (börsennotiert)": [
        ""
      ],
      "GbR": [
        ""
      ],
      "n.e.V.": [
        ""
      ],
      "Partei": [
        ""
      ],
      "WEG": [
        ""
      ],
      "Anderes": [
        ""
      ],
      "invalid format": [
        ""
      ],
      "it can't be in the future": [
        ""
      ],
      "it can't be greater than 120 years": [
        ""
      ],
      "Invalid e-mail address": [
        ""
      ],
      "Invalid phone number": [
        ""
      ],
      "Personal Details": [
        ""
      ],
      "Personal details of the authorised representative.": [
        ""
      ],
      "First name(s)": [
        ""
      ],
      "As on your ID document": [
        ""
      ],
      "Last name": [
        ""
      ],
      "Date of birth": [
        ""
      ],
      "Nationality": [
        ""
      ],
      "Phone number": [
        ""
      ],
      "Company information": [
        ""
      ],
      "Name of the company": [
        ""
      ],
      "Legal form": [
        ""
      ],
      "Legal form (free-form entry for other)": [
        ""
      ],
      "Commercial register number": [
        ""
      ],
      "Seat of the register court": [
        ""
      ],
      "Founding date": [
        ""
      ],
      "Is the company a non-profit organization?": [
        ""
      ],
      "Industry": [
        ""
      ],
      "Company address": [
        ""
      ],
      "Tax information": [
        ""
      ],
      "Business identification number or tax number": [
        ""
      ],
      "Was the company incorporated in the USA or under US law?": [
        ""
      ],
      "Economically active or inactive?": [
        ""
      ],
      "Eligible for input tax deduction": [
        ""
      ],
      "Persons": [
        ""
      ],
      "Please list all legal representatives, shareholders/partners, and authorized signatories.": [
        ""
      ],
      "Legal representatives / shareholders / partners / authorized signatories": [
        ""
      ],
      "Type of representative": [
        ""
      ],
      "New Customer Identification": [
        ""
      ],
      "Confirmation": [
        ""
      ],
      "Confirmation of private customer / wallet.": [
        ""
      ],
      "Please confirm that you are a private customer that wants to redeem their wallet balance.": [
        ""
      ],
      "Upload multiple documents": [
        ""
      ],
      "Upload multiple documents.": [
        ""
      ],
      "Document upload (%1$s)": [
        ""
      ],
      "Simple form": [
        ""
      ],
      "Full Name": [
        ""
      ],
      "Birthdate": [
        ""
      ],
      "Comment": [
        ""
      ],
      "Resolution": [
        ""
      ],
      "New state": [
        ""
      ],
      "Frozen": [
        ""
      ],
      "Pending": [
        ""
      ],
      "Normal": [
        ""
      ],
      "New threshold": [
        ""
      ],
      "Establishing of the controlling person of operating legal entities and partnerships both not quoted on the stock exchange (K)": [
        ""
      ],
      "for operating legal entities and partnerships that are contracting partner as well as analogously for operating legal entities and partnership that are beneficial owners": [
        ""
      ],
      "Establishment of the controlling person (submitted by customer)": [
        ""
      ],
      "Contracting partner": [
        ""
      ],
      "Controlling person(s)": [
        ""
      ],
      "Reason for control": [
        ""
      ],
      "Holding 25% or more": [
        ""
      ],
      "The person(s) listed below is/are holding 25% or more of the contracting partner's shares (capital shares or voting rights)": [
        ""
      ],
      "Other way": [
        ""
      ],
      "If the capital shares or voting rights cannot be determined or in case there are no capital shares or voting rights 25% or more, the contracting partner hereby declares that the person(s) listed below is/are controlling the contracting partner in other ways": [
        ""
      ],
      "Managing director": [
        ""
      ],
      "In case this/these person(s) cannot be determined or this/these person(s) does/do not exist, the contracting partner hereby declares that the person(s) listed below is/are the managing director(s)": [
        ""
      ],
      "Can't be empty": [
        ""
      ],
      "Full name": [
        ""
      ],
      "Actual address of domicile": [
        ""
      ],
      "Fiduciary holding assets": [
        ""
      ],
      "Is a third person the beneficial owner of the assets held in the account/securities account?": [
        ""
      ],
      "Yes": [
        ""
      ],
      "No": [
        ""
      ],
      "Signature(s)": [
        ""
      ],
      "It is a criminal offence to deliberately provide false information on this form (article 251 of the Swiss Criminal Code, documents forgery)": [
        ""
      ],
      "The contracting partner hereby undertakes to inform automatically of any changes to the information contained herein.": [
        ""
      ],
      "Date": [
        ""
      ],
      "Establishment of the controlling person (submitted by AML officer)": [
        ""
      ],
      "Signed Declaration": [
        ""
      ],
      "Signed declaration by the customer": [
        ""
      ],
      "The uploaded document must contain the customer's signature on the beneficial owner declaration.": [
        ""
      ],
      "Signed Document": [
        ""
      ],
      "Special Clarifications": [
        ""
      ],
      "When a business relationship or transaction is associated with increased risk, appears unusual or evidence exists that the assets are the proceeds of a felony or a qualified tax offence, the member has to perform additional clarifications.": [
        ""
      ],
      "Reason for special clarifications": [
        ""
      ],
      "Reason": [
        ""
      ],
      "Description of the circumstances/transactions, which triggered the special clarifications": [
        ""
      ],
      "can't be empty": [
        ""
      ],
      "Used means of clarification": [
        ""
      ],
      "Means": [
        ""
      ],
      "Gathering of information from the customer, beneficial owner of the assets, controlling person": [
        ""
      ],
      "Consultation of generally accessible sources and databases": [
        ""
      ],
      "Enquiries with trustworthy persons": [
        ""
      ],
      "Other, which?": [
        ""
      ],
      "Other means of clarification:": [
        ""
      ],
      "Optional supplemental information for the special clarifications.": [
        ""
      ],
      "Summary and plausbility check of the gathered information": [
        ""
      ],
      "The results of the clarifications have to be documented and their plausibility has to be checked.": [
        ""
      ],
      "Summary": [
        ""
      ],
      "Gathered/Consulted documents": [
        ""
      ],
      "Result of the special clarification": [
        ""
      ],
      "Result": [
        ""
      ],
      "No suspicion": [
        ""
      ],
      "The plausibility of the circumstances could be checked, no reasonable suspicion pursuant to Art. 9 AMLA (possibly update of customer profile and/or risk profile)": [
        ""
      ],
      "Reasonable suspicion": [
        ""
      ],
      "Reasonable suspicion pursuant to Art. 9 AMLA, duty to file a report with MROS": [
        ""
      ],
      "Simple suspicion": [
        ""
      ],
      "Simple suspicion pursuant to Art. 305 Para. 2 StGB, right to notify MROS": [
        ""
      ],
      "Other, what?": [
        ""
      ],
      "Result clarification": [
        ""
      ],
      "The customer is the person with whom the member concludes the contract with regard to the financial service provided (civil law). Does the\n  member act as director of a domiciliary company, this domiciliary company is the customer.": [
        ""
      ],
      "Identification Form (customer)": [
        ""
      ],
      "The customer has to be identified on entering into a permanent business relationship or on concluding a cash transaction, which meets the according threshold.": [
        ""
      ],
      "Correspondence language:": [
        ""
      ],
      "English": [
        ""
      ],
      "German": [
        ""
      ],
      "French": [
        ""
      ],
      "Italian": [
        ""
      ],
      "Identification form (basic customer information)": [
        ""
      ],
      "Information on customer": [
        ""
      ],
      "Customer type": [
        ""
      ],
      "Natural person (incl. sole proprietors)": [
        ""
      ],
      "Legal entity": [
        ""
      ],
      "Applicable if customer is a natural person": [
        ""
      ],
      "Residential address in Switzerland": [
        ""
      ],
      "Telephone": [
        ""
      ],
      "E-mail": [
        ""
      ],
      "Copy of identification document": [
        ""
      ],
      "Only official government IDs (incl. passports) are accepted. Please scan both sides if applicable.": [
        ""
      ],
      "Sole proprietor": [
        ""
      ],
      "Information on customer (sole proprietor)": [
        ""
      ],
      "Applicable only if customer is a sole proprietor": [
        ""
      ],
      "Company name": [
        ""
      ],
      "Registered office": [
        ""
      ],
      "Company identification document": [
        ""
      ],
      "Information on customer (legal entity)": [
        ""
      ],
      "Applicable if customer is a legal entity": [
        ""
      ],
      "Domicile": [
        ""
      ],
      "Contact person": [
        ""
      ],
      "Copy of identification document (not older than 12 months)": [
        ""
      ],
      "Correspondence Preferences": [
        ""
      ],
      "Information on the natural persons who establish the business relationship for legal entities and partnerships": [
        ""
      ],
      "For legal entities and partnerships the identity of the natural persons who establish the business relationship must be verified.": [
        ""
      ],
      "Establishers of the legal relationship": [
        ""
      ],
      "Residential address": [
        ""
      ],
      "Signing authority of the person": [
        ""
      ],
      "Power of attorney arrangements": [
        ""
      ],
      "Other type of signing authority": [
        ""
      ],
      "Evidence of signing authority:": [
        ""
      ],
      "Company register extract": [
        ""
      ],
      "Mandate": [
        ""
      ],
      "Other": [
        ""
      ],
      "Specify other way of establishing signing authority:": [
        ""
      ],
      "Copy of document that serves as evidence of signing authority:": [
        ""
      ],
      "Customer classification": [
        ""
      ],
      "Classification for the establishment of the beneficial owner of the assets and/or controlling person": [
        ""
      ],
      "The customer is:": [
        ""
      ],
      "A natural person and there are no doubts that this person is the sole beneficial owner of the assets": [
        ""
      ],
      "An operational legal entity or partnership": [
        ""
      ],
      "A foundation (or a similar construct; incl. underlying companies)": [
        ""
      ],
      "A trust (incl. underlying companies)": [
        ""
      ],
      "A life insurance policy with separately managed accounts / securities accounts (so-called insurance wrappers)": [
        ""
      ],
      "Identification Form (acceptance)": [
        ""
      ],
      "Acceptance of business relationship": [
        ""
      ],
      "Date (conclusion of contract):": [
        ""
      ],
      "Accepted via:": [
        ""
      ],
      "Face to face": [
        ""
      ],
      "Authenticated copy of identification document": [
        ""
      ],
      "Residentail address validated": [
        ""
      ],
      "Further information:": [
        ""
      ],
      "Evaluation with regard to embargo procedures/terrorism lists on establishing the business relationship": [
        ""
      ],
      "Verification whether the customer, beneficial owners of the assets, controlling persons, authorized representatives or other involved persons are listed on an embargo/terrorism list.": [
        ""
      ],
      "Embargo/terrorism status:": [
        ""
      ],
      "Not listed on embargo/terrorism list.": [
        ""
      ],
      "Listed on embargo/terrorism list.": [
        ""
      ],
      "Embargo/terrorism information:": [
        ""
      ],
      "Verification date": [
        ""
      ],
      "Supplemental files": [
        ""
      ],
      "Risk Profile AMLA": [
        ""
      ],
      "Evaluation of politically exposed persons (PEP-Check)": [
        ""
      ],
      "This evaluation has to be completed by all members for every business relationship.": [
        ""
      ],
      "Foreign PEP": [
        ""
      ],
      "Is the customer, the beneficial owner or the controlling person or authorised representative a foreign PEP or closely related to such a person?": [
        ""
      ],
      "Domestic PEP": [
        ""
      ],
      "Is the customer, the beneficial owner or the controlling person or authorised representative a domestic PEP or closely related to such a person?": [
        ""
      ],
      "PEP of International Organisatons": [
        ""
      ],
      "Is the customer, the beneficial owner or the controlling person or authorised representative a PEP in International Organizations or closely related to such a person?": [
        ""
      ],
      "Acceptance date": [
        ""
      ],
      "When the decision of the Senior executive body on the acceptance of a business relationship with a PEP was obtain on.": [
        ""
      ],
      "Evaluation \"high risk\" or non-cooperative country": [
        ""
      ],
      "High-risk or non-cooperative country": [
        ""
      ],
      "Is the customer, the beneficial owner or the controlling person or authorised representative in a country considered by the FATF as high-risk or non-cooperative and for which FATF requires increased diligence?": [
        ""
      ],
      "When the decision of the Senior executive body on the acceptance of such a business relationship was obtained on.": [
        ""
      ],
      "Evaluation of business relationship risk": [
        ""
      ],
      "Country risk": [
        ""
      ],
      "Applicable country risk types": [
        ""
      ],
      "Nationality of the customer": [
        ""
      ],
      "Nationality of the beneficial owner of the assets": [
        ""
      ],
      "Domicile/residential address of the customer": [
        ""
      ],
      "Domicile/residential address of the beneficial owner of the assets": [
        ""
      ],
      "Domicile/residential address of the controlling person": [
        ""
      ],
      "Country risk level (nationality)": [
        ""
      ],
      "Risk category according to VQF country list (VQF doc. no. 902.4.1)": [
        ""
      ],
      "Low (Risk 0)": [
        ""
      ],
      "Risk 0 acc. to VQF country list (VQF doc. no. 902.4.1)": [
        ""
      ],
      "Medium (Risk 1)": [
        ""
      ],
      "Risk 1 acc. to VQF country list (VQF doc. no. 902.4.1)": [
        ""
      ],
      "High (Risk 2)": [
        ""
      ],
      "Risk 2 acc. to VQF country list (VQF doc. no. 902.4.1)": [
        ""
      ],
      "Country risk type (place of business activity)": [
        ""
      ],
      "Customer": [
        ""
      ],
      "Beneficial owner": [
        ""
      ],
      "Country risk level (business activity)": [
        ""
      ],
      "Industry risk": [
        ""
      ],
      "Industry risk source": [
        ""
      ],
      "Beneficial owner of the assets": [
        ""
      ],
      "Industry risk level": [
        ""
      ],
      "Clearly defined, transparent, easily comprehensible business activity well known to the member.": [
        ""
      ],
      "Business activity with a high level of cash transactions.": [
        ""
      ],
      "Business activity not well known to the member.": [
        ""
      ],
      "Trade in munitions/arms, raw gem stones/diamonds, jewellery, international trade in exotic animals, casino and lottery business, trade in erotic wares.": [
        ""
      ],
      "Member has no personal knowledge of the customer’s industry.": [
        ""
      ],
      "Contact risk": [
        ""
      ],
      "Type of contact to the customer/benefcial owner of the assets.": [
        ""
      ],
      "Contact risk level": [
        ""
      ],
      "Personal acquaintance between member and customer/beneficial owner of the assets over several years (at least 2) prior to entering into the business relationship": [
        ""
      ],
      "The customer/beneficial owner was not personally known to the member for several years (at least 2) prior to entering into the business relationship; however (a) no business was entered into in the absence of the customer/beneficial owner, or (b) the customer was at least introduced/brokered by a trusted third party.": [
        ""
      ],
      "The customer/beneficial owner was not personally known to the member and business was entered into in the absence of the former (relationship by correspondence) and the customer was not introduced/brokered by a trusted third party.": [
        ""
      ],
      "Summary evaluation": [
        ""
      ],
      "Justification for risk assessment": [
        ""
      ],
      "Risk classification": [
        ""
      ],
      "Conclusion whether the business relationship is with or without increased risk.": [
        ""
      ],
      "No high risk": [
        ""
      ],
      "High risk": [
        ""
      ],
      "When the decision of the Senior executive body on the acceptance of a business relationship with increased risk was obtained on.": [
        ""
      ],
      "Customer Profile": [
        ""
      ],
      "The information below has to refer to the persons from whom the assets originate ultimately (e.g. beneficial owner of the assets, founder/creator of a trust or foundation). Is the customer an operational legal entity or partnership the information may refer to the entity itself (not to the controlling person), unless the entity holds the assets in trust for a third party.": [
        ""
      ],
      "Business activity": [
        ""
      ],
      "Profession, business activities, etc. (former, current, potentially planned)": [
        ""
      ],
      "Financial circumstances": [
        ""
      ],
      "Income and assets, liabilities (estimated)": [
        ""
      ],
      "Origin of the deposited assets involved": [
        ""
      ],
      "Will the the customer deposit assets with Taler Operations AG?": [
        ""
      ],
      "Nature, amount and currency of deposited assets.": [
        ""
      ],
      "Category": [
        ""
      ],
      "Savings": [
        ""
      ],
      "Own business operations": [
        ""
      ],
      "Inheritance": [
        ""
      ],
      "Category clarification": [
        ""
      ],
      "Detail description of the origings/economical background of the assets involved in the business relationship": [
        ""
      ],
      "Nature and purpose of the business relationship": [
        ""
      ],
      "Purpose of the business relationship": [
        ""
      ],
      "Information on the planned development of the business relationship and the assets": [
        ""
      ],
      "Detail on usual business volume": [
        ""
      ],
      "Relationship with third parties": [
        ""
      ],
      "Relation of the customer to the beneficial owner, controlling persons, authorised signatories and other persons involved in the business relationship": [
        ""
      ],
      "Relation to other AMLA-Files": [
        ""
      ],
      "Introducer / agents / references": [
        ""
      ],
      "Further information": [
        ""
      ],
      "Other relevant information": [
        ""
      ],
      "Declaration of identity of the beneficial owner (A)": [
        ""
      ],
      "Declaration of identity of the beneficial owner": [
        ""
      ],
      "Beneficial owner details": [
        ""
      ],
      "Beneficial owner(s)": [
        ""
      ],
      "The person(s) listed below is/are the beneficial owner(s) of the assets involved in the business relationship. If the contracting partner is also the sole beneficial owner of the assets, the contracting partner's detail must be set out below": [
        ""
      ],
      "Domicile address": [
        ""
      ],
      "January": [
        ""
      ],
      "February": [
        ""
      ],
      "March": [
        ""
      ],
      "April": [
        ""
      ],
      "May": [
        ""
      ],
      "June": [
        ""
      ],
      "July": [
        ""
      ],
      "August": [
        ""
      ],
      "September": [
        ""
      ],
      "October": [
        ""
      ],
      "November": [
        ""
      ],
      "December": [
        ""
      ],
      "Previous year": [
        ""
      ],
      "Next year": [
        ""
      ],
      "Previous month": [
        ""
      ],
      "Next month": [
        ""
      ],
      "Cancel": [
        ""
      ],
      "Remove": [
        ""
      ],
      "Confirm": [
        ""
      ],
      "Add new...": [
        ""
      ],
      "No element found": [
        ""
      ],
      "%1$s (Classification level %2$s)": [
        ""
      ],
      "years": [
        ""
      ],
      "months": [
        ""
      ],
      "days": [
        ""
      ],
      "hours": [
        ""
      ],
      "minutes": [
        ""
      ],
      "seconds": [
        ""
      ],
      "Upload a file": [
        ""
      ],
      "Download a copy.": [
        ""
      ],
      "Missing fields": [
        ""
      ],
      "Simple comment": [
        ""
      ],
      "Terms of Service": [
        ""
      ],
      "Name and birthdate": [
        ""
      ],
      "required": [
        ""
      ],
      "Validation function failed. Contact developers %1$s": [
        ""
      ],
      "The form is too big for the server, try uploading smaller files.": [
        ""
      ],
      "The account was not found": [
        ""
      ],
      "Officer disabled or more recent decision was already submitted.": [
        ""
      ],
      "Debug information about the values in the form": [
        ""
      ],
      "Submit": [
        ""
      ],
      "Could not load context information": [
        ""
      ],
      "Could not find form": [
        ""
      ],
      "Form with id '%1$s' is not registered in this application.": [
        ""
      ],
      "Can't upload information": [
        ""
      ],
      "The KYC requirement doesn't have an ID": [
        ""
      ],
      "Mandatory identification requirements are satisfied": [
        ""
      ],
      "Process completed": [
        ""
      ],
      "You can close this window now.": [
        ""
      ],
      "Please, complete the following requirement:": [
        ""
      ],
      "Please, complete any of the following requirements:": [
        ""
      ],
      "not found": [
        ""
      ],
      "conflict": [
        ""
      ],
      "payload is too large": [
        ""
      ],
      "Link generated, you can proceed.": [
        ""
      ],
      "%1$s": [
        ""
      ],
      "Generate link": [
        ""
      ],
      "Open link": [
        ""
      ],
      "Form": [
        ""
      ],
      "Fill form": [
        ""
      ],
      "Trigger KYC balance": [
        ""
      ],
      "You can also use the formId in the UR after \"/test/show-forms/$FORM_ID\"": [
        ""
      ],
      "Trigger KYC Balance": [
        ""
      ],
      "Amount": [
        ""
      ],
      "No kyc triggered.": [
        ""
      ],
      "Forbidden.": [
        ""
      ],
      "Not found.": [
        ""
      ],
      "Conflict.": [
        ""
      ],
      "This actions will trigger wallet balance kyc above 1000000 threshold, the exchange should be properly configured to trigger the desired kyc flow.": [
        ""
      ],
      "Trigger TOPS Terms of service": [
        ""
      ],
      "Trigger GLS onboarding": [
        ""
      ],
      "Trigger VQF 902.1": [
        ""
      ],
      "Trigger VQF 902.4": [
        ""
      ],
      "Trigger VQF 902.5": [
        ""
      ],
      "Trigger VQF 902.9": [
        ""
      ],
      "Trigger VQF 902.11": [
        ""
      ],
      "Trigger VQF 902.12": [
        ""
      ],
      "Trigger VQF 902.13": [
        ""
      ],
      "Trigger VQF 902.14": [
        ""
      ],
      "Trigger VQF 902.15": [
        ""
      ],
      "Challenger test": [
        ""
      ],
      "Trigger VQF 902.9 customer": [
        ""
      ],
      "Trigger VQF 902.9 officer": [
        ""
      ],
      "The URL is wrong": [
        ""
      ],
      "Maybe the validation check expired.": [
        ""
      ],
      "Are you a cross-border truck driver?": [
        ""
      ],
      "How much is in your pockets?": [
        ""
      ],
      "KYC_REQUIREMENT_LINK_DESCRIPTION\u0004%1$s": [
        ""
      ],
      "KYC_REQUIREMENT_INFO_DESCRIPTION\u0004%1$s": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "es",
  "completeness": 0
};

strings['de'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "de"
      },
      "Show debug info": [
        "Debugging-Informationen anzeigen"
      ],
      "Kyc completed": [
        "KYC abgeschlossen"
      ],
      "You will be redirected to nowhere": [
        "Sie werden nirgendwohin weitergeleitet"
      ],
      "Internal error, please report.": [
        "Interner Fehler, um dessen Mitteilung wir Sie freundlich bitten."
      ],
      "Customer identification": [
        "Identifizierung der Person"
      ],
      "Preferences": [
        "Präferenzen"
      ],
      "Show more information": [
        "Mostrar confirmación de extracción"
      ],
      "Hide debug info": [
        "Debugging-Informationen anzeigen"
      ],
      "The request reached a timeout, check your connection.": [
        ""
      ],
      "The request was cancelled.": [
        ""
      ],
      "Too many requests were made to the server, and this action was throttled.": [
        ""
      ],
      "The server's response was malformed.": [
        ""
      ],
      "Could not complete the request due to a network problem.": [
        ""
      ],
      "Unexpected request error": [
        ""
      ],
      "Unexpected error": [
        ""
      ],
      "Show notifications": [
        "Identifizierung der Person"
      ],
      "Open profile": [
        ""
      ],
      "Open settings": [
        ""
      ],
      "Menu": [
        ""
      ],
      "Close panel": [
        ""
      ],
      "Log out": [
        ""
      ],
      "Sites": [
        ""
      ],
      "Learn more about %1$s": [
        ""
      ],
      "never": [
        ""
      ],
      "in %1$s": [
        "%1$s"
      ],
      "%1$s ago": [
        "%1$s"
      ],
      "First page": [
        ""
      ],
      "Next": [
        ""
      ],
      "Request timeout": [
        ""
      ],
      "Request cancelled": [
        ""
      ],
      "Request throttled": [
        ""
      ],
      "Malformed response": [
        ""
      ],
      "Network error": [
        ""
      ],
      "Unhandled failure, please report. Code %1$s": [
        ""
      ],
      "Operation failed.": [
        ""
      ],
      "Unexpected error, this is likely a bug. Please report ": [
        ""
      ],
      "The operation failed.": [
        ""
      ],
      "The server version is not supported. Supported version \"%1$s\", server version \"%2$s\"": [
        ""
      ],
      "Terms of service": [
        "Allgemeine Geschäftsbedingungen"
      ],
      "You must open/download the terms of service to proceed": [
        ""
      ],
      "View in Browser": [
        ""
      ],
      "Download PDF version": [
        ""
      ],
      "You must download to proceed": [
        ""
      ],
      "Do you accept the terms of service?": [
        "Akzeptieren Sie die Allgemeinen Geschäftsbedingungen"
      ],
      "Challenger EMAIL": [
        "Challenger-Test"
      ],
      "Challenge email ownership.": [
        ""
      ],
      "E-Mail": [
        ""
      ],
      "Challenger POSTAL": [
        "Challenger-Test"
      ],
      "Challenge address ownership.": [
        "Challenger-Test"
      ],
      "Name": [
        "Vollständiger Name"
      ],
      "Address": [
        ""
      ],
      "Country": [
        "Betrag"
      ],
      "Challenger SMS": [
        "Challenger-Test"
      ],
      "Challenge phone number ownership.": [
        ""
      ],
      "Phone": [
        ""
      ],
      "Generic note": [
        ""
      ],
      "Free-form, generic note": [
        ""
      ],
      "Note / Memorandum": [
        ""
      ],
      "Free-form notes": [
        ""
      ],
      "Supplemental File Upload": [
        ""
      ],
      "Optional supplemental information for the establishment of the business relationship with the customer.": [
        ""
      ],
      "Supplemental Files": [
        ""
      ],
      "Description": [
        ""
      ],
      "File (PDF)": [
        ""
      ],
      "GmbH": [
        ""
      ],
      "GmbH i.G.": [
        ""
      ],
      "UG": [
        ""
      ],
      "UG i.G.": [
        ""
      ],
      "UHG": [
        ""
      ],
      "KG": [
        ""
      ],
      "e.V.": [
        ""
      ],
      "e.V. i.G.": [
        ""
      ],
      "eG": [
        ""
      ],
      "eG i.G.": [
        ""
      ],
      "PartG": [
        ""
      ],
      "e.K.": [
        ""
      ],
      "AG (nicht börsennotiert)": [
        ""
      ],
      "AG (börsennotiert)": [
        ""
      ],
      "GbR": [
        ""
      ],
      "n.e.V.": [
        ""
      ],
      "Partei": [
        ""
      ],
      "WEG": [
        ""
      ],
      "Anderes": [
        ""
      ],
      "invalid format": [
        "Formular ausfüllen"
      ],
      "it can't be in the future": [
        ""
      ],
      "it can't be greater than 120 years": [
        ""
      ],
      "Invalid e-mail address": [
        ""
      ],
      "Invalid phone number": [
        ""
      ],
      "Personal Details": [
        ""
      ],
      "Personal details of the authorised representative.": [
        ""
      ],
      "First name(s)": [
        ""
      ],
      "As on your ID document": [
        ""
      ],
      "Last name": [
        ""
      ],
      "Date of birth": [
        ""
      ],
      "Nationality": [
        ""
      ],
      "Phone number": [
        ""
      ],
      "Company information": [
        ""
      ],
      "Name of the company": [
        ""
      ],
      "Legal form": [
        "Formular ausfüllen"
      ],
      "Legal form (free-form entry for other)": [
        ""
      ],
      "Commercial register number": [
        ""
      ],
      "Seat of the register court": [
        ""
      ],
      "Founding date": [
        ""
      ],
      "Is the company a non-profit organization?": [
        ""
      ],
      "Industry": [
        ""
      ],
      "Company address": [
        ""
      ],
      "Tax information": [
        ""
      ],
      "Business identification number or tax number": [
        ""
      ],
      "Was the company incorporated in the USA or under US law?": [
        ""
      ],
      "Economically active or inactive?": [
        ""
      ],
      "Eligible for input tax deduction": [
        ""
      ],
      "Persons": [
        ""
      ],
      "Please list all legal representatives, shareholders/partners, and authorized signatories.": [
        ""
      ],
      "Legal representatives / shareholders / partners / authorized signatories": [
        ""
      ],
      "Type of representative": [
        ""
      ],
      "New Customer Identification": [
        "Identifizierung der Person"
      ],
      "Confirmation": [
        ""
      ],
      "Confirmation of private customer / wallet.": [
        ""
      ],
      "Please confirm that you are a private customer that wants to redeem their wallet balance.": [
        ""
      ],
      "Upload multiple documents": [
        ""
      ],
      "Upload multiple documents.": [
        ""
      ],
      "Document upload (%1$s)": [
        ""
      ],
      "Simple form": [
        "Einfaches Formular"
      ],
      "Full Name": [
        "Vollständiger Name"
      ],
      "Birthdate": [
        "Geburtsdatum"
      ],
      "Comment": [
        "Anmerkung/Kommentar"
      ],
      "Resolution": [
        "Beurteilung"
      ],
      "New state": [
        "Neuer Zustand"
      ],
      "Frozen": [
        "Eingefroren"
      ],
      "Pending": [
        "Ausstehend"
      ],
      "Normal": [
        "Normal"
      ],
      "New threshold": [
        "Neuer Schwellenwert"
      ],
      "Establishing of the controlling person of operating legal entities and partnerships both not quoted on the stock exchange (K)": [
        "Feststellung einer beherrschenden Person von operativ tätigen Rechtssubjekten (Kapital- und Personengesellschaften), die nicht börsennotiert sind (K)"
      ],
      "for operating legal entities and partnerships that are contracting partner as well as analogously for operating legal entities and partnership that are beneficial owners": [
        "für operative Rechtssubjekte (Kapital- und Personengesellschaften), die Vertragspartner sind, sowie analog für Rechtssubjekte, die wirtschaftlich Berechtigte sind"
      ],
      "Establishment of the controlling person (submitted by customer)": [
        ""
      ],
      "Contracting partner": [
        ""
      ],
      "Controlling person(s)": [
        ""
      ],
      "Reason for control": [
        ""
      ],
      "Holding 25% or more": [
        ""
      ],
      "The person(s) listed below is/are holding 25% or more of the contracting partner's shares (capital shares or voting rights)": [
        ""
      ],
      "Other way": [
        ""
      ],
      "If the capital shares or voting rights cannot be determined or in case there are no capital shares or voting rights 25% or more, the contracting partner hereby declares that the person(s) listed below is/are controlling the contracting partner in other ways": [
        ""
      ],
      "Managing director": [
        ""
      ],
      "In case this/these person(s) cannot be determined or this/these person(s) does/do not exist, the contracting partner hereby declares that the person(s) listed below is/are the managing director(s)": [
        ""
      ],
      "Can't be empty": [
        ""
      ],
      "Full name": [
        "Vollständiger Name"
      ],
      "Actual address of domicile": [
        ""
      ],
      "Fiduciary holding assets": [
        ""
      ],
      "Is a third person the beneficial owner of the assets held in the account/securities account?": [
        ""
      ],
      "Yes": [
        "Ja"
      ],
      "No": [
        "Nein"
      ],
      "Signature(s)": [
        ""
      ],
      "It is a criminal offence to deliberately provide false information on this form (article 251 of the Swiss Criminal Code, documents forgery)": [
        ""
      ],
      "The contracting partner hereby undertakes to inform automatically of any changes to the information contained herein.": [
        ""
      ],
      "Date": [
        ""
      ],
      "Establishment of the controlling person (submitted by AML officer)": [
        ""
      ],
      "Signed Declaration": [
        ""
      ],
      "Signed declaration by the customer": [
        ""
      ],
      "The uploaded document must contain the customer's signature on the beneficial owner declaration.": [
        ""
      ],
      "Signed Document": [
        "Einfacher Kommentar"
      ],
      "Special Clarifications": [
        "Spezielle Abklärungen"
      ],
      "When a business relationship or transaction is associated with increased risk, appears unusual or evidence exists that the assets are the proceeds of a felony or a qualified tax offence, the member has to perform additional clarifications.": [
        "Wenn eine Geschäftsbeziehung oder Transaktion mit einem erhöhten Risiko verbunden ist, ungewöhnlich erscheint oder Anhaltspunkte dafür vorliegen, dass es sich bei den Vermögenswerten um Erträge aus einer Straftat oder einem offensichtlichen Steuerdelikt handelt, sind zusätzliche Abklärungen vorzunehmen."
      ],
      "Reason for special clarifications": [
        "Spezielle Abklärungen"
      ],
      "Reason": [
        ""
      ],
      "Description of the circumstances/transactions, which triggered the special clarifications": [
        ""
      ],
      "can't be empty": [
        ""
      ],
      "Used means of clarification": [
        "Spezielle Abklärungen"
      ],
      "Means": [
        ""
      ],
      "Gathering of information from the customer, beneficial owner of the assets, controlling person": [
        ""
      ],
      "Consultation of generally accessible sources and databases": [
        ""
      ],
      "Enquiries with trustworthy persons": [
        ""
      ],
      "Other, which?": [
        ""
      ],
      "Other means of clarification:": [
        ""
      ],
      "Optional supplemental information for the special clarifications.": [
        ""
      ],
      "Summary and plausbility check of the gathered information": [
        ""
      ],
      "The results of the clarifications have to be documented and their plausibility has to be checked.": [
        ""
      ],
      "Summary": [
        ""
      ],
      "Gathered/Consulted documents": [
        ""
      ],
      "Result of the special clarification": [
        "Spezielle Abklärungen"
      ],
      "Result": [
        ""
      ],
      "No suspicion": [
        ""
      ],
      "The plausibility of the circumstances could be checked, no reasonable suspicion pursuant to Art. 9 AMLA (possibly update of customer profile and/or risk profile)": [
        ""
      ],
      "Reasonable suspicion": [
        ""
      ],
      "Reasonable suspicion pursuant to Art. 9 AMLA, duty to file a report with MROS": [
        ""
      ],
      "Simple suspicion": [
        "Einfacher Kommentar"
      ],
      "Simple suspicion pursuant to Art. 305 Para. 2 StGB, right to notify MROS": [
        ""
      ],
      "Other, what?": [
        ""
      ],
      "Result clarification": [
        "Spezielle Abklärungen"
      ],
      "The customer is the person with whom the member concludes the contract with regard to the financial service provided (civil law). Does the\n  member act as director of a domiciliary company, this domiciliary company is the customer.": [
        ""
      ],
      "Identification Form (customer)": [
        "Formular zur Identifizierung"
      ],
      "The customer has to be identified on entering into a permanent business relationship or on concluding a cash transaction, which meets the according threshold.": [
        "Die (natürliche oder juristische) Person muss identifiziert werden bei Aufnahme einer dauerhaften Geschäftsbeziehung oder bei Abschluss eines Bargeschäfts, das einen entsprechenden Schwellenwert erreicht."
      ],
      "Correspondence language:": [
        ""
      ],
      "English": [
        ""
      ],
      "German": [
        ""
      ],
      "French": [
        ""
      ],
      "Italian": [
        ""
      ],
      "Identification form (basic customer information)": [
        ""
      ],
      "Information on customer": [
        ""
      ],
      "Customer type": [
        "Kundenprofil"
      ],
      "Natural person (incl. sole proprietors)": [
        ""
      ],
      "Legal entity": [
        ""
      ],
      "Applicable if customer is a natural person": [
        ""
      ],
      "Residential address in Switzerland": [
        ""
      ],
      "Telephone": [
        ""
      ],
      "E-mail": [
        ""
      ],
      "Copy of identification document": [
        "Formular zur Identifizierung"
      ],
      "Only official government IDs (incl. passports) are accepted. Please scan both sides if applicable.": [
        ""
      ],
      "Sole proprietor": [
        ""
      ],
      "Information on customer (sole proprietor)": [
        ""
      ],
      "Applicable only if customer is a sole proprietor": [
        ""
      ],
      "Company name": [
        ""
      ],
      "Registered office": [
        ""
      ],
      "Company identification document": [
        "Identifizierung der Person"
      ],
      "Information on customer (legal entity)": [
        ""
      ],
      "Applicable if customer is a legal entity": [
        ""
      ],
      "Domicile": [
        ""
      ],
      "Contact person": [
        ""
      ],
      "Copy of identification document (not older than 12 months)": [
        ""
      ],
      "Correspondence Preferences": [
        ""
      ],
      "Information on the natural persons who establish the business relationship for legal entities and partnerships": [
        ""
      ],
      "For legal entities and partnerships the identity of the natural persons who establish the business relationship must be verified.": [
        ""
      ],
      "Establishers of the legal relationship": [
        ""
      ],
      "Residential address": [
        ""
      ],
      "Signing authority of the person": [
        ""
      ],
      "Power of attorney arrangements": [
        ""
      ],
      "Other type of signing authority": [
        ""
      ],
      "Evidence of signing authority:": [
        ""
      ],
      "Company register extract": [
        ""
      ],
      "Mandate": [
        ""
      ],
      "Other": [
        ""
      ],
      "Specify other way of establishing signing authority:": [
        ""
      ],
      "Copy of document that serves as evidence of signing authority:": [
        ""
      ],
      "Customer classification": [
        "Identifizierung der Person"
      ],
      "Classification for the establishment of the beneficial owner of the assets and/or controlling person": [
        ""
      ],
      "The customer is:": [
        ""
      ],
      "A natural person and there are no doubts that this person is the sole beneficial owner of the assets": [
        ""
      ],
      "An operational legal entity or partnership": [
        ""
      ],
      "A foundation (or a similar construct; incl. underlying companies)": [
        ""
      ],
      "A trust (incl. underlying companies)": [
        ""
      ],
      "A life insurance policy with separately managed accounts / securities accounts (so-called insurance wrappers)": [
        "Angaben zu Lebensversicherungen mit getrennt geführten Konten/Depotkonten (sog. Insurance Wrapper)"
      ],
      "Identification Form (acceptance)": [
        "Formular zur Identifizierung"
      ],
      "Acceptance of business relationship": [
        ""
      ],
      "Date (conclusion of contract):": [
        ""
      ],
      "Accepted via:": [
        ""
      ],
      "Face to face": [
        ""
      ],
      "Authenticated copy of identification document": [
        ""
      ],
      "Residentail address validated": [
        ""
      ],
      "Further information:": [
        "Identifizierung der Person"
      ],
      "Evaluation with regard to embargo procedures/terrorism lists on establishing the business relationship": [
        ""
      ],
      "Verification whether the customer, beneficial owners of the assets, controlling persons, authorized representatives or other involved persons are listed on an embargo/terrorism list.": [
        ""
      ],
      "Embargo/terrorism status:": [
        ""
      ],
      "Not listed on embargo/terrorism list.": [
        ""
      ],
      "Listed on embargo/terrorism list.": [
        ""
      ],
      "Embargo/terrorism information:": [
        ""
      ],
      "Verification date": [
        "Formular zur Identifizierung"
      ],
      "Supplemental files": [
        ""
      ],
      "Risk Profile AMLA": [
        "AMLA-Risikoprofil"
      ],
      "Evaluation of politically exposed persons (PEP-Check)": [
        ""
      ],
      "This evaluation has to be completed by all members for every business relationship.": [
        ""
      ],
      "Foreign PEP": [
        ""
      ],
      "Is the customer, the beneficial owner or the controlling person or authorised representative a foreign PEP or closely related to such a person?": [
        ""
      ],
      "Domestic PEP": [
        ""
      ],
      "Is the customer, the beneficial owner or the controlling person or authorised representative a domestic PEP or closely related to such a person?": [
        ""
      ],
      "PEP of International Organisatons": [
        ""
      ],
      "Is the customer, the beneficial owner or the controlling person or authorised representative a PEP in International Organizations or closely related to such a person?": [
        ""
      ],
      "Acceptance date": [
        ""
      ],
      "When the decision of the Senior executive body on the acceptance of a business relationship with a PEP was obtain on.": [
        ""
      ],
      "Evaluation \"high risk\" or non-cooperative country": [
        ""
      ],
      "High-risk or non-cooperative country": [
        ""
      ],
      "Is the customer, the beneficial owner or the controlling person or authorised representative in a country considered by the FATF as high-risk or non-cooperative and for which FATF requires increased diligence?": [
        ""
      ],
      "When the decision of the Senior executive body on the acceptance of such a business relationship was obtained on.": [
        ""
      ],
      "Evaluation of business relationship risk": [
        ""
      ],
      "Country risk": [
        ""
      ],
      "Applicable country risk types": [
        ""
      ],
      "Nationality of the customer": [
        ""
      ],
      "Nationality of the beneficial owner of the assets": [
        "Identitätserklärung des wirtschaftlich Berechtigten (A)"
      ],
      "Domicile/residential address of the customer": [
        ""
      ],
      "Domicile/residential address of the beneficial owner of the assets": [
        ""
      ],
      "Domicile/residential address of the controlling person": [
        ""
      ],
      "Country risk level (nationality)": [
        ""
      ],
      "Risk category according to VQF country list (VQF doc. no. 902.4.1)": [
        ""
      ],
      "Low (Risk 0)": [
        ""
      ],
      "Risk 0 acc. to VQF country list (VQF doc. no. 902.4.1)": [
        ""
      ],
      "Medium (Risk 1)": [
        ""
      ],
      "Risk 1 acc. to VQF country list (VQF doc. no. 902.4.1)": [
        ""
      ],
      "High (Risk 2)": [
        ""
      ],
      "Risk 2 acc. to VQF country list (VQF doc. no. 902.4.1)": [
        ""
      ],
      "Country risk type (place of business activity)": [
        ""
      ],
      "Customer": [
        "Kundenprofil"
      ],
      "Beneficial owner": [
        ""
      ],
      "Country risk level (business activity)": [
        ""
      ],
      "Industry risk": [
        ""
      ],
      "Industry risk source": [
        ""
      ],
      "Beneficial owner of the assets": [
        ""
      ],
      "Industry risk level": [
        ""
      ],
      "Clearly defined, transparent, easily comprehensible business activity well known to the member.": [
        ""
      ],
      "Business activity with a high level of cash transactions.": [
        ""
      ],
      "Business activity not well known to the member.": [
        ""
      ],
      "Trade in munitions/arms, raw gem stones/diamonds, jewellery, international trade in exotic animals, casino and lottery business, trade in erotic wares.": [
        ""
      ],
      "Member has no personal knowledge of the customer’s industry.": [
        ""
      ],
      "Contact risk": [
        ""
      ],
      "Type of contact to the customer/benefcial owner of the assets.": [
        ""
      ],
      "Contact risk level": [
        ""
      ],
      "Personal acquaintance between member and customer/beneficial owner of the assets over several years (at least 2) prior to entering into the business relationship": [
        ""
      ],
      "The customer/beneficial owner was not personally known to the member for several years (at least 2) prior to entering into the business relationship; however (a) no business was entered into in the absence of the customer/beneficial owner, or (b) the customer was at least introduced/brokered by a trusted third party.": [
        ""
      ],
      "The customer/beneficial owner was not personally known to the member and business was entered into in the absence of the former (relationship by correspondence) and the customer was not introduced/brokered by a trusted third party.": [
        ""
      ],
      "Summary evaluation": [
        ""
      ],
      "Justification for risk assessment": [
        ""
      ],
      "Risk classification": [
        "Spezielle Abklärungen"
      ],
      "Conclusion whether the business relationship is with or without increased risk.": [
        ""
      ],
      "No high risk": [
        ""
      ],
      "High risk": [
        ""
      ],
      "When the decision of the Senior executive body on the acceptance of a business relationship with increased risk was obtained on.": [
        ""
      ],
      "Customer Profile": [
        "Kundenprofil"
      ],
      "The information below has to refer to the persons from whom the assets originate ultimately (e.g. beneficial owner of the assets, founder/creator of a trust or foundation). Is the customer an operational legal entity or partnership the information may refer to the entity itself (not to the controlling person), unless the entity holds the assets in trust for a third party.": [
        "Die nachstehenden Angaben müssen sich auf die Personen beziehen, die als wirtschaftlich Berechtigte der Vermögenswerte gelten (z. B. verantwortlicher Eigentümer der Vermögenswerte, Gründer/Stifter eines Trusts oder einer Stiftung). Handelt es sich dabei um eine operativ tätige juristische Person (Kapital- oder Personengesellschaft), können sich die Angaben auf die Firma als juristische Person selbst beziehen (und nicht auf die sie beherrschende Person), es sei denn, die juristische Person hält die Vermögenswerte treuhänderisch für einen Dritten."
      ],
      "Business activity": [
        ""
      ],
      "Profession, business activities, etc. (former, current, potentially planned)": [
        ""
      ],
      "Financial circumstances": [
        ""
      ],
      "Income and assets, liabilities (estimated)": [
        ""
      ],
      "Origin of the deposited assets involved": [
        ""
      ],
      "Will the the customer deposit assets with Taler Operations AG?": [
        ""
      ],
      "Nature, amount and currency of deposited assets.": [
        ""
      ],
      "Category": [
        ""
      ],
      "Savings": [
        ""
      ],
      "Own business operations": [
        ""
      ],
      "Inheritance": [
        ""
      ],
      "Category clarification": [
        "Spezielle Abklärungen"
      ],
      "Detail description of the origings/economical background of the assets involved in the business relationship": [
        ""
      ],
      "Nature and purpose of the business relationship": [
        ""
      ],
      "Purpose of the business relationship": [
        ""
      ],
      "Information on the planned development of the business relationship and the assets": [
        ""
      ],
      "Detail on usual business volume": [
        ""
      ],
      "Relationship with third parties": [
        ""
      ],
      "Relation of the customer to the beneficial owner, controlling persons, authorised signatories and other persons involved in the business relationship": [
        ""
      ],
      "Relation to other AMLA-Files": [
        ""
      ],
      "Introducer / agents / references": [
        ""
      ],
      "Further information": [
        "Identifizierung der Person"
      ],
      "Other relevant information": [
        ""
      ],
      "Declaration of identity of the beneficial owner (A)": [
        "Identitätserklärung des wirtschaftlich Berechtigten (A)"
      ],
      "Declaration of identity of the beneficial owner": [
        "Identitätserklärung des wirtschaftlich Berechtigten (A)"
      ],
      "Beneficial owner details": [
        ""
      ],
      "Beneficial owner(s)": [
        ""
      ],
      "The person(s) listed below is/are the beneficial owner(s) of the assets involved in the business relationship. If the contracting partner is also the sole beneficial owner of the assets, the contracting partner's detail must be set out below": [
        ""
      ],
      "Domicile address": [
        ""
      ],
      "January": [
        ""
      ],
      "February": [
        ""
      ],
      "March": [
        ""
      ],
      "April": [
        ""
      ],
      "May": [
        ""
      ],
      "June": [
        ""
      ],
      "July": [
        ""
      ],
      "August": [
        ""
      ],
      "September": [
        ""
      ],
      "October": [
        ""
      ],
      "November": [
        ""
      ],
      "December": [
        ""
      ],
      "Previous year": [
        ""
      ],
      "Next year": [
        ""
      ],
      "Previous month": [
        ""
      ],
      "Next month": [
        ""
      ],
      "Cancel": [
        "Abbrechen"
      ],
      "Remove": [
        ""
      ],
      "Confirm": [
        ""
      ],
      "Add new...": [
        ""
      ],
      "No element found": [
        "Nichts gefunden"
      ],
      "%1$s (Classification level %2$s)": [
        ""
      ],
      "years": [
        ""
      ],
      "months": [
        ""
      ],
      "days": [
        ""
      ],
      "hours": [
        ""
      ],
      "minutes": [
        ""
      ],
      "seconds": [
        ""
      ],
      "Upload a file": [
        ""
      ],
      "Download a copy.": [
        ""
      ],
      "Missing fields": [
        ""
      ],
      "Simple comment": [
        "Einfacher Kommentar"
      ],
      "Terms of Service": [
        "Allgemeine Geschäftsbedingungen"
      ],
      "Name and birthdate": [
        "Name und Geburtsdatum"
      ],
      "required": [
        "Es ist keine KYC-Prüfung erforderlich"
      ],
      "Validation function failed. Contact developers %1$s": [
        ""
      ],
      "The form is too big for the server, try uploading smaller files.": [
        "Das Formular ist zu groß für den Server. Versuchen Sie, kleinere Dateien hochzuladen."
      ],
      "The account was not found": [
        "Das Konto wurde nicht gefunden"
      ],
      "Officer disabled or more recent decision was already submitted.": [
        "Bereits deaktiviert vom Beauftragten oder eine neuere Entscheidung wurde bereits eingetragen."
      ],
      "Debug information about the values in the form": [
        "Informationen fürs Debugging bezüglich der Werte im Formular"
      ],
      "Submit": [
        "Abenden"
      ],
      "Could not load context information": [
        ""
      ],
      "Could not find form": [
        ""
      ],
      "Form with id '%1$s' is not registered in this application.": [
        ""
      ],
      "Can't upload information": [
        ""
      ],
      "The KYC requirement doesn't have an ID": [
        ""
      ],
      "Mandatory identification requirements are satisfied": [
        "Die obligatorischen Erfordernisse zur Identifizierung sind erfüllt"
      ],
      "Process completed": [
        "Das Verfahren ist abgeschlossen"
      ],
      "You can close this window now.": [
        "Sie können dieses Fenster jetzt schließen."
      ],
      "Please, complete the following requirement:": [
        "Bitte folgendes ausfüllen:"
      ],
      "Please, complete any of the following requirements:": [
        "Folgende Erfordernisse müssen erfüllt sein:"
      ],
      "not found": [
        "Nichts gefunden"
      ],
      "conflict": [
        "Konflikt"
      ],
      "payload is too large": [
        ""
      ],
      "Link generated, you can proceed.": [
        ""
      ],
      "%1$s": [
        "%1$s"
      ],
      "Generate link": [
        ""
      ],
      "Open link": [
        ""
      ],
      "Form": [
        "Formular"
      ],
      "Fill form": [
        "Formular ausfüllen"
      ],
      "Trigger KYC balance": [
        "KYC-Bewertung auslösen"
      ],
      "You can also use the formId in the UR after \"/test/show-forms/$FORM_ID\"": [
        ""
      ],
      "Trigger KYC Balance": [
        "KYC-Saldo auslösen"
      ],
      "Amount": [
        "Betrag"
      ],
      "No kyc triggered.": [
        "Es ist keine KYC-Prüfung erforderlich"
      ],
      "Forbidden.": [
        ""
      ],
      "Not found.": [
        "Nichts gefunden"
      ],
      "Conflict.": [
        "Konflikt"
      ],
      "This actions will trigger wallet balance kyc above 1000000 threshold, the exchange should be properly configured to trigger the desired kyc flow.": [
        "Ein Wallet-Guthaben von über 1.000.000 löst eine KYC-Prüfung aus. Taler Exchange sollte dem entsprechend konfiguriert sein, um den gewünschten Vorgang der KYC-Prüfung auszulösen."
      ],
      "Trigger TOPS Terms of service": [
        "Allgemeine Geschäftsbedingungen von TOPS auslösen"
      ],
      "Trigger GLS onboarding": [
        "GLS-Händleronboarding auslösen"
      ],
      "Trigger VQF 902.1": [
        "Verfahren nach VQF 902.1 auslösen"
      ],
      "Trigger VQF 902.4": [
        "Verfahren nach VQF 902.4 auslösen"
      ],
      "Trigger VQF 902.5": [
        "Verfahren nach VQF 902.5 auslösen"
      ],
      "Trigger VQF 902.9": [
        "Verfahren nach VQF 902.9 auslösen"
      ],
      "Trigger VQF 902.11": [
        "Verfahren nach VQF 902.11 auslösen"
      ],
      "Trigger VQF 902.12": [
        "Verfahren nach VQF 902.12 auslösen"
      ],
      "Trigger VQF 902.13": [
        "Verfahren nach VQF 902.13 auslösen"
      ],
      "Trigger VQF 902.14": [
        "Verfahren nach VQF 902.14 auslösen"
      ],
      "Trigger VQF 902.15": [
        "Verfahren nach VQF 902.15 auslösen"
      ],
      "Challenger test": [
        "Challenger-Test"
      ],
      "Trigger VQF 902.9 customer": [
        "Verfahren nach VQF 902.9 auslösen"
      ],
      "Trigger VQF 902.9 officer": [
        "Verfahren nach VQF 902.9 auslösen"
      ],
      "The URL is wrong": [
        "Die URL ist nicht korrekt"
      ],
      "Maybe the validation check expired.": [
        "Möglicherweise ist der Gültigkeitszeitraum der Überprüfung abgelaufen."
      ],
      "Are you a cross-border truck driver?": [
        "Fahren Sie einen LKW im grenzüberschreitenden Verkehr?"
      ],
      "How much is in your pockets?": [
        "Wie viel Bargeld führen Sie mit sich?"
      ],
      "KYC_REQUIREMENT_LINK_DESCRIPTION\u0004%1$s": [
        "%1$s"
      ],
      "KYC_REQUIREMENT_INFO_DESCRIPTION\u0004%1$s": [
        "%1$s"
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "de",
  "completeness": 26
};

