// flow-typed signature: 74b987f228ae8ecd554f7ca28416e775
// flow-typed version: <<STUB>>/rollup_v^0.66.2/flow_v0.82.0

/**
 * This is an autogenerated libdef stub for:
 *
 *   'rollup'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module 'rollup' {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module 'rollup/dist/rollup.browser' {
  declare module.exports: any;
}

declare module 'rollup/dist/rollup.es' {
  declare module.exports: any;
}

declare module 'rollup/dist/rollup' {
  declare module.exports: any;
}

// Filename aliases
declare module 'rollup/dist/rollup.browser.js' {
  declare module.exports: $Exports<'rollup/dist/rollup.browser'>;
}
declare module 'rollup/dist/rollup.es.js' {
  declare module.exports: $Exports<'rollup/dist/rollup.es'>;
}
declare module 'rollup/dist/rollup.js' {
  declare module.exports: $Exports<'rollup/dist/rollup'>;
}
