/*
 This file is part of GNU Taler
 (C) 2023 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

import test from "ava";
import { generateIban, validateIban } from "./iban.js";

test("iban validation", (t) => {
  t.assert(validateIban("foo").type === "invalid");
  t.assert(validateIban("NL71RABO9996666778").type === "valid");
  t.assert(validateIban("NL71RABO9996666779").type === "invalid");
});

test("iban generation", (t) => {
  let iban1 = generateIban("DE", 10);
  t.assert(validateIban(iban1).type === "valid");
});
