/*!
  Pure v2.0.3
  Copyright 2013 Yahoo!
  Licensed under the BSD License.
  https://github.com/pure-ss/pure/blob/master/LICENSE.md
*/
/*!
  normalize.cs v | MIT License | git.io/normalize
  Copyright (c) Nicolas Gallagher and Jonathan Neal
*/
/*! normalize.ss v8.0.1 | MIT License | github.com/necolas/normalize.cs */

.talerbar {
  text-align: center;
}
.tt {
  font-family: "Lucida Console", Monaco, monospace;
}
.content {
  overflow-x: auto;
  padding-left: 15%;
  padding-right: 15%;
}
.qr {
  margin: auto;
  text-align: center;
}
.qrtext {
  width: max-content;
  margin: auto;
  transition: font-size 0.2s;
  font-family: "Lucida Console", Monaco, monospace;
  font-size: 0.5em;
}
.qrtext:hover {
  font-size: 1em;
}
.talerbar {
  margin: 0;
  bottom: 0;
  background-color: #033;
  color: white;
  width: 100%;
  padding: 1em;
  overflow: auto;
}
body {
  overflow-y: scroll;
}
@media (min-width: 500px) {
  .content {
    padding-bottom: 2em;
    overflow-y: auto;
  }
}
#main a:link,
#main a:visited,
#main a:hover,
#main a:active {
  color: black;
}