'use strict';

import warnAboutDeprecatedESMImport from './warnAboutDeprecatedESMImport.js';
warnAboutDeprecatedESMImport('ExecutionEnvironment');

import { ExecutionEnvironment } from '../esm/history.js';
export default ExecutionEnvironment;
