// flow-typed signature: afb3994794f3c2f840f54e3cb3826ce7
// flow-typed version: <<STUB>>/@babel/core_v^7.1.2/flow_v0.82.0

/**
 * This is an autogenerated libdef stub for:
 *
 *   '@babel/core'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module '@babel/core' {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module '@babel/core/lib/config/caching' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/config-chain' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/config-descriptors' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/files/configuration' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/files/index-browser' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/files/index' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/files/package' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/files/plugins' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/files/types' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/files/utils' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/full' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/helpers/config-api' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/helpers/environment' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/index' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/item' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/partial' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/pattern-to-regex' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/plugin' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/util' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/validation/option-assertions' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/validation/options' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/validation/plugins' {
  declare module.exports: any;
}

declare module '@babel/core/lib/config/validation/removed' {
  declare module.exports: any;
}

declare module '@babel/core/lib/index' {
  declare module.exports: any;
}

declare module '@babel/core/lib/parse' {
  declare module.exports: any;
}

declare module '@babel/core/lib/tools/build-external-helpers' {
  declare module.exports: any;
}

declare module '@babel/core/lib/transform-ast' {
  declare module.exports: any;
}

declare module '@babel/core/lib/transform-file-browser' {
  declare module.exports: any;
}

declare module '@babel/core/lib/transform-file-sync-browser' {
  declare module.exports: any;
}

declare module '@babel/core/lib/transform-file' {
  declare module.exports: any;
}

declare module '@babel/core/lib/transform' {
  declare module.exports: any;
}

declare module '@babel/core/lib/transformation/block-hoist-plugin' {
  declare module.exports: any;
}

declare module '@babel/core/lib/transformation/file/file' {
  declare module.exports: any;
}

declare module '@babel/core/lib/transformation/file/generate' {
  declare module.exports: any;
}

declare module '@babel/core/lib/transformation/file/merge-map' {
  declare module.exports: any;
}

declare module '@babel/core/lib/transformation/index' {
  declare module.exports: any;
}

declare module '@babel/core/lib/transformation/normalize-file' {
  declare module.exports: any;
}

declare module '@babel/core/lib/transformation/normalize-opts' {
  declare module.exports: any;
}

declare module '@babel/core/lib/transformation/plugin-pass' {
  declare module.exports: any;
}

declare module '@babel/core/lib/transformation/util/missing-plugin-helper' {
  declare module.exports: any;
}

// Filename aliases
declare module '@babel/core/lib/config/caching.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/caching'>;
}
declare module '@babel/core/lib/config/config-chain.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/config-chain'>;
}
declare module '@babel/core/lib/config/config-descriptors.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/config-descriptors'>;
}
declare module '@babel/core/lib/config/files/configuration.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/files/configuration'>;
}
declare module '@babel/core/lib/config/files/index-browser.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/files/index-browser'>;
}
declare module '@babel/core/lib/config/files/index.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/files/index'>;
}
declare module '@babel/core/lib/config/files/package.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/files/package'>;
}
declare module '@babel/core/lib/config/files/plugins.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/files/plugins'>;
}
declare module '@babel/core/lib/config/files/types.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/files/types'>;
}
declare module '@babel/core/lib/config/files/utils.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/files/utils'>;
}
declare module '@babel/core/lib/config/full.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/full'>;
}
declare module '@babel/core/lib/config/helpers/config-api.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/helpers/config-api'>;
}
declare module '@babel/core/lib/config/helpers/environment.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/helpers/environment'>;
}
declare module '@babel/core/lib/config/index.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/index'>;
}
declare module '@babel/core/lib/config/item.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/item'>;
}
declare module '@babel/core/lib/config/partial.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/partial'>;
}
declare module '@babel/core/lib/config/pattern-to-regex.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/pattern-to-regex'>;
}
declare module '@babel/core/lib/config/plugin.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/plugin'>;
}
declare module '@babel/core/lib/config/util.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/util'>;
}
declare module '@babel/core/lib/config/validation/option-assertions.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/validation/option-assertions'>;
}
declare module '@babel/core/lib/config/validation/options.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/validation/options'>;
}
declare module '@babel/core/lib/config/validation/plugins.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/validation/plugins'>;
}
declare module '@babel/core/lib/config/validation/removed.js' {
  declare module.exports: $Exports<'@babel/core/lib/config/validation/removed'>;
}
declare module '@babel/core/lib/index.js' {
  declare module.exports: $Exports<'@babel/core/lib/index'>;
}
declare module '@babel/core/lib/parse.js' {
  declare module.exports: $Exports<'@babel/core/lib/parse'>;
}
declare module '@babel/core/lib/tools/build-external-helpers.js' {
  declare module.exports: $Exports<'@babel/core/lib/tools/build-external-helpers'>;
}
declare module '@babel/core/lib/transform-ast.js' {
  declare module.exports: $Exports<'@babel/core/lib/transform-ast'>;
}
declare module '@babel/core/lib/transform-file-browser.js' {
  declare module.exports: $Exports<'@babel/core/lib/transform-file-browser'>;
}
declare module '@babel/core/lib/transform-file-sync-browser.js' {
  declare module.exports: $Exports<'@babel/core/lib/transform-file-sync-browser'>;
}
declare module '@babel/core/lib/transform-file.js' {
  declare module.exports: $Exports<'@babel/core/lib/transform-file'>;
}
declare module '@babel/core/lib/transform.js' {
  declare module.exports: $Exports<'@babel/core/lib/transform'>;
}
declare module '@babel/core/lib/transformation/block-hoist-plugin.js' {
  declare module.exports: $Exports<'@babel/core/lib/transformation/block-hoist-plugin'>;
}
declare module '@babel/core/lib/transformation/file/file.js' {
  declare module.exports: $Exports<'@babel/core/lib/transformation/file/file'>;
}
declare module '@babel/core/lib/transformation/file/generate.js' {
  declare module.exports: $Exports<'@babel/core/lib/transformation/file/generate'>;
}
declare module '@babel/core/lib/transformation/file/merge-map.js' {
  declare module.exports: $Exports<'@babel/core/lib/transformation/file/merge-map'>;
}
declare module '@babel/core/lib/transformation/index.js' {
  declare module.exports: $Exports<'@babel/core/lib/transformation/index'>;
}
declare module '@babel/core/lib/transformation/normalize-file.js' {
  declare module.exports: $Exports<'@babel/core/lib/transformation/normalize-file'>;
}
declare module '@babel/core/lib/transformation/normalize-opts.js' {
  declare module.exports: $Exports<'@babel/core/lib/transformation/normalize-opts'>;
}
declare module '@babel/core/lib/transformation/plugin-pass.js' {
  declare module.exports: $Exports<'@babel/core/lib/transformation/plugin-pass'>;
}
declare module '@babel/core/lib/transformation/util/missing-plugin-helper.js' {
  declare module.exports: $Exports<'@babel/core/lib/transformation/util/missing-plugin-helper'>;
}
