/*
 This file is part of GNU Taler
 (C) 2022 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

import { parseRestoreUri } from "@gnu-taler/taler-util";
import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
import { useAlertContext } from "../../context/alert.js";
import { useBackendContext } from "../../context/backend.js";
import { useTranslationContext } from "@gnu-taler/web-util/browser";
import { Props, State } from "./index.js";

export function useComponentState({
  talerRecoveryUri,
  onCancel,
  onSuccess,
}: Props): State {
  const api = useBackendContext();
  const { pushAlertOnError } = useAlertContext();
  const { i18n } = useTranslationContext();
  if (!talerRecoveryUri) {
    return {
      status: "error",
      error: {
        type: "error",
        message: i18n.str`Missing URI`,
        description: i18n.str`Check the link.`,
        cause: new Error("something"),
        context: {},
      },
    };
  }
  const info = parseRestoreUri(talerRecoveryUri);

  if (!info) {
    return {
      status: "error",
      error: {
        type: "error",
        message: i18n.str`Could not parse the recovery URI`,
        description: i18n.str`Try another URI`,
        cause: new Error("something"),
        context: {},
      },
    };
  }
  const recovery = info;

  async function recoverBackup(): Promise<void> {
    await api.wallet.call(WalletApiOperation.ImportBackupRecovery, {
      recovery: {
        walletRootPriv: recovery.walletRootPriv,
        providers: recovery.providers.map((url) => ({
          name: new URL(url).hostname,
          url,
        })),
      },
    });
    onSuccess();
  }

  return {
    status: "ready",

    accept: {
      onClick: pushAlertOnError(recoverBackup),
    },
    cancel: {
      onClick: pushAlertOnError(onCancel),
    },
    error: undefined,
  };
}
