// This file is generated automatically by `scripts/build/fp.js`. Please, don't change it.

import fn from '../../getISOWeeksInYear/index'
import convertToFP from '../_lib/convertToFP/index'

var getISOWeeksInYear = convertToFP(fn, 1)

export default getISOWeeksInYear
