'use strict';

import warnAboutDeprecatedESMImport from './warnAboutDeprecatedESMImport.js';
warnAboutDeprecatedESMImport('LocationUtils');

import { LocationUtils } from '../esm/history.js';
export default LocationUtils;
