/*
 This file is part of GNU Taler
 (C) 2019 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

import { WalletNotification } from "@gnu-taler/taler-util";
import { HttpRequestLibrary } from "@gnu-taler/taler-util/http";

/**
 * Helpers to initiate a wallet in a host environment.
 */

/**
 */
export interface DefaultNodeWalletArgs {
  /**
   * Location of the wallet database.
   *
   * If not specified, the wallet starts out with an empty database and
   * the wallet database is stored only in memory.
   */
  persistentStoragePath?: string;

  /**
   * Handler for asynchronous notifications from the wallet.
   */
  notifyHandler?: (n: WalletNotification) => void;

  /**
   * If specified, use this as HTTP request library instead
   * of the default one.
   */
  httpLib?: HttpRequestLibrary;

  cryptoWorkerType?: "sync" | "node-worker-thread";
}

/**
 * Generate a random alphanumeric ID.  Does *not* use cryptographically
 * secure randomness.
 */
export function makeTempfileId(length: number): string {
  let result = "";
  const characters = "abcdefghijklmnopqrstuvwxyz0123456789";
  for (let i = 0; i < length; i++) {
    result += characters.charAt(Math.floor(Math.random() * characters.length));
  }
  return result;
}

/**
 * Get the underlying sqlite3 DB filename
 * from the storage path specified by the wallet-core client.
 */
export function getSqlite3FilenameFromStoragePath(
  p: string | undefined,
): string {
  // Allow specifying a directory as the storage path.
  // In that case, we pick the filename and use the sqlite3
  // backend.
  //
  // We still allow specifying a filename for backwards
  // compatibility and control over the exact file.
  //
  // Specifying a directory allows us to automatically
  // migrate to a new DB file in the future.
  if (!p) {
    return ":memory:";
  }
  if (p.endsWith("/")) {
    // Current sqlite3 DB filename.
    // Should rarely if ever change.
    return `${p}talerwalletdb-v30.sqlite3`;
  } else {
    return p;
  }
}
