#!/bin/bash

[ ! -d prebuilt ] && git worktree add -f prebuilt prebuilt && exit 1

rm -f prebuilt/bank/*
find packages/bank-ui/dist/prod/ -type f -printf '%P\n' | sort > prebuilt/bank/bof

while IFS= read -r file; do
  cp packages/bank-ui/dist/prod/$file prebuilt/bank/$file
done < prebuilt/bank/bof

