/*
 This file is part of GNU Anastasis
 (C) 2021-2022 Anastasis SARL

 GNU Anastasis is free software; you can redistribute it and/or modify it under the
 terms of the GNU Affero General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Anastasis is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more details.

 You should have received a copy of the GNU Affero General Public License along with
 GNU Anastasis; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

/**
 *
 * @author Sebastian Javier Marchano (sebasjm)
 */

.is-user-avatar {
  &.has-max-width {
    max-width: $size-base * 7;
  }

  &.is-aligned-center {
    margin: 0 auto;
  }

  img {
    margin: 0 auto;
    border-radius: $radius-rounded;
  }
}

.icon.has-update-mark {
  position: relative;

  &:after {
    content: "";
    width: $icon-update-mark-size;
    height: $icon-update-mark-size;
    position: absolute;
    top: 1px;
    right: 1px;
    background-color: $icon-update-mark-color;
    border-radius: $radius-rounded;
  }
}
