/*
 This file is part of GNU Taler
 (C) 2022-2024 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

/*eslint quote-props: ["error", "consistent"]*/
export const strings: {[s: string]: any} = {};

strings['it'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "it"
      },
      "Show debug info": [
        ""
      ],
      "Use testing dialect": [
        ""
      ],
      "Allow Insecure password": [
        ""
      ],
      "Keep session after reload": [
        ""
      ],
      "Don't compress request": [
        ""
      ],
      "Internal error, please report.": [
        "Registrazione"
      ],
      "Preferences": [
        ""
      ],
      "Password": [
        ""
      ],
      "Required": [
        ""
      ],
      "should have at least 12 characters": [
        ""
      ],
      "should have lowercase and uppercase characters": [
        ""
      ],
      "should have numbers": [
        ""
      ],
      "should have at least one character which is not a number or letter": [
        ""
      ],
      "Repeat password": [
        ""
      ],
      "doesn't match": [
        ""
      ],
      "Create account": [
        ""
      ],
      "Create": [
        ""
      ],
      "Couldn't unlock the account, the password may be wrong.": [
        ""
      ],
      "Account locked": [
        ""
      ],
      "Your account is normally locked anytime you reload. To unlock type your password again.": [
        ""
      ],
      "Unlock": [
        ""
      ],
      "Forget account": [
        ""
      ],
      "There are no rules for operations": [
        ""
      ],
      "This mean that all operation have no limit.": [
        ""
      ],
      "Add custom rule": [
        ""
      ],
      "Operation": [
        ""
      ],
      "Threshold": [
        ""
      ],
      "Escalation": [
        ""
      ],
      "%1$s": [
        ""
      ],
      "(all)": [
        ""
      ],
      "Since": [
        ""
      ],
      "Expired": [
        ""
      ],
      "Expires": [
        ""
      ],
      "Successor measure": [
        ""
      ],
      "AML officer justification": [
        ""
      ],
      "Default rules": [
        ""
      ],
      "Forms": [
        ""
      ],
      "Measures used to get gather information about the customer.": [
        ""
      ],
      "Name": [
        ""
      ],
      "Check": [
        ""
      ],
      "Program": [
        ""
      ],
      "Context": [
        ""
      ],
      "Modify": [
        ""
      ],
      "Info": [
        ""
      ],
      "Will show information to the user.": [
        ""
      ],
      "Instant measures": [
        ""
      ],
      "Triggered immediately without customer interaction.": [
        ""
      ],
      "Input requirement": [
        ""
      ],
      "Already completed.": [
        ""
      ],
      "Waiting to be completed.": [
        ""
      ],
      "Current active rules": [
        ""
      ],
      "There are no rules for this account yet.": [
        ""
      ],
      "New rule form": [
        ""
      ],
      "Add": [
        ""
      ],
      "Cancel": [
        ""
      ],
      "Basic plan": [
        ""
      ],
      "Premium": [
        ""
      ],
      "E-commerce": [
        ""
      ],
      "Point-of-sale": [
        ""
      ],
      "GLS Registered E-Commerce Merchant": [
        ""
      ],
      "Check rules": [
        ""
      ],
      "The account is a wallet and there are bank account operation's limits which may trigger actions that are not required: %1$s": [
        ""
      ],
      "The account isn't a wallet and there are wallet operation's limits which may trigger actions that are not required: %1$s": [
        ""
      ],
      "The account is a wallet and there are wallet operation's limits which are not limited: %1$s": [
        ""
      ],
      "The AML account is bank account, but not all account-related operations are limited (%1$s).": [
        ""
      ],
      "New rules": [
        ""
      ],
      "Reset rules to current state": [
        ""
      ],
      "Set rules to default": [
        ""
      ],
      "Freeze account": [
        ""
      ],
      "Behavior on rule expiration": [
        ""
      ],
      "Reset measure": [
        ""
      ],
      "Reset expiration": [
        ""
      ],
      "Operation type": [
        ""
      ],
      "Timeframe": [
        ""
      ],
      "Exposed": [
        ""
      ],
      "Is this limit communicated to the customer?": [
        ""
      ],
      "Esclation measure": [
        ""
      ],
      "Measures that the customer will need to satisfy to apply for a new threshold.": [
        ""
      ],
      "All measures": [
        ""
      ],
      "Hint the customer that all measure should be completed": [
        ""
      ],
      "Expiration": [
        ""
      ],
      "Predefined shortcuts": [
        ""
      ],
      "In a week": [
        ""
      ],
      "In a month": [
        ""
      ],
      "In a year": [
        ""
      ],
      "Never": [
        ""
      ],
      "Expiration date": [
        ""
      ],
      "Until when are the new rules active.": [
        ""
      ],
      "Measure taken automatically upon expiration of the current decision.": [
        ""
      ],
      "New decision": [
        ""
      ],
      "Show transfers": [
        ""
      ],
      "Case history for selected account": [
        "Storico dei conti pubblici"
      ],
      "ID:": [
        ""
      ],
      "Payto:": [
        ""
      ],
      "Under investigation": [
        ""
      ],
      "This account requires a manual review and is waiting for a decision to be made.": [
        ""
      ],
      "Collected information": [
        ""
      ],
      "Every attribute collection event (via form upload or KYC provider).": [
        ""
      ],
      "The event list is empty": [
        ""
      ],
      "Current account properties": [
        ""
      ],
      "No active decision found": [
        "Credenziali invalide."
      ],
      "There is no decision for this account yet. The account is limited by the default rules.": [
        ""
      ],
      "Previous AML decisions": [
        ""
      ],
      "No aml history found": [
        ""
      ],
      "Current active measures": [
        ""
      ],
      "Now": [
        ""
      ],
      "Public key": [
        ""
      ],
      "Request account activation": [
        ""
      ],
      "Lock account": [
        ""
      ],
      "Operation denied": [
        ""
      ],
      "This account signature is invalid, contact administrator or create a new one.": [
        ""
      ],
      "The designated AML account is not known, contact administrator or create a new one.": [
        ""
      ],
      "The designated AML account is not enabled, contact administrator or create a new one.": [
        ""
      ],
      "High risk accounts. Only with custom rules.": [
        ""
      ],
      "Account with custom rules.": [
        ""
      ],
      "High risk accounts": [
        ""
      ],
      "All account known.": [
        ""
      ],
      "High risk accounts under investigation. Only with custom rules.": [
        ""
      ],
      "Accounts under investigation. Only with custom rules.": [
        ""
      ],
      "High risk accounts under investigation.": [
        ""
      ],
      "Accounts under investigation.": [
        ""
      ],
      "High risk accounts without investigation. Only with custom rules.": [
        ""
      ],
      "Accounts without investigation. Only with custom rules.": [
        ""
      ],
      "High risk accounts without investigation.": [
        ""
      ],
      "Accounts without investigation.": [
        ""
      ],
      "Accounts": [
        "Somma"
      ],
      "Export as file": [
        ""
      ],
      "Export completed": [
        ""
      ],
      "Click here to save the file in your computer.": [
        ""
      ],
      "No results": [
        ""
      ],
      "#": [
        ""
      ],
      "Account Identification": [
        ""
      ],
      "Open time": [
        ""
      ],
      "Status": [
        ""
      ],
      "From %1$s%2$sTo %3$s": [
        ""
      ],
      "Since %1$s": [
        ""
      ],
      "Not opened": [
        ""
      ],
      "High risk": [
        ""
      ],
      "With custom rules": [
        ""
      ],
      "Invalid account": [
        ""
      ],
      "Search by ID": [
        ""
      ],
      "Under investigation: ": [
        ""
      ],
      "High risk:": [
        ""
      ],
      "Open:": [
        ""
      ],
      "Dashboard": [
        ""
      ],
      "Statistics": [
        ""
      ],
      "Accounts open": [
        ""
      ],
      "New files last year": [
        ""
      ],
      "Closes files last year": [
        ""
      ],
      "MROS substantaiated": [
        ""
      ],
      "MROS simple": [
        ""
      ],
      "High risk files": [
        ""
      ],
      "PEP files": [
        ""
      ],
      "Under investiation": [
        ""
      ],
      "Number of accounts that are opened": [
        ""
      ],
      "Number of new GwG files in the last year": [
        ""
      ],
      "Number of GwG files closed in the last year": [
        ""
      ],
      "Number of MROS reports based on Art 9 Abs. 1 GwG (per year)": [
        ""
      ],
      "Number of MROS reports based on Art 305ter Abs. 2 StGB (per year)": [
        ""
      ],
      "Number of GwG files of high-risk customers": [
        ""
      ],
      "Number of GwG files managed with “increased risk” due to PEP status": [
        ""
      ],
      "Number of customers involved in proceedings for which Art 6 GwG did apply": [
        ""
      ],
      "previous": [
        ""
      ],
      "Decreased by": [
        ""
      ],
      "Increased by": [
        ""
      ],
      "Events": [
        ""
      ],
      "The events will be triggered only after the decision has been confirmed.": [
        ""
      ],
      "Calculated": [
        ""
      ],
      "Events based on properties and new attributes that should be triggered": [
        ""
      ],
      "Others": [
        ""
      ],
      "Events that can be triggered manually": [
        ""
      ],
      "Custom list": [
        ""
      ],
      "Events without built-in support.": [
        ""
      ],
      "Complete the form": [
        ""
      ],
      "These measures will ask the customer for information.": [
        ""
      ],
      "change form": [
        ""
      ],
      "Expiration date of the information filled in this form.": [
        ""
      ],
      "Include new information": [
        ""
      ],
      "Select a form to submit new information about the customer or continue to the next step": [
        ""
      ],
      "Form:": [
        ""
      ],
      "Justification": [
        ""
      ],
      "Keep investigation?": [
        ""
      ],
      "Account holder": [
        ""
      ],
      "Full name of the account holder": [
        ""
      ],
      "loading...": [
        ""
      ],
      "Update": [
        ""
      ],
      "Remove": [
        ""
      ],
      "Configure verification type: %1$s": [
        ""
      ],
      "Show complete form": [
        ""
      ],
      "Configure measure": [
        ""
      ],
      "Show as verification": [
        ""
      ],
      "There is already a measure with that name": [
        ""
      ],
      "Only required when no check is specified": [
        ""
      ],
      "Missing check or program": [
        ""
      ],
      "Without a check the program will run automatically": [
        ""
      ],
      "Field name": [
        ""
      ],
      "Type": [
        ""
      ],
      "string": [
        ""
      ],
      "number": [
        ""
      ],
      "boolean": [
        ""
      ],
      "json": [
        ""
      ],
      "Value": [
        ""
      ],
      "There are unsatisfied inputs: %1$s": [
        ""
      ],
      "There are missing inputs: %1$s": [
        ""
      ],
      "There are missing requirements: %1$s": [
        ""
      ],
      "Can't be empty": [
        ""
      ],
      "It should be a number": [
        ""
      ],
      "Not a number": [
        ""
      ],
      "It should be finite": [
        ""
      ],
      "It should be a safe integer": [
        ""
      ],
      "It should be either \"true\" or \"false\"": [
        ""
      ],
      "Couldn't parse as json string: %1$s": [
        ""
      ],
      "Description": [
        ""
      ],
      "Inputs": [
        ""
      ],
      "Output": [
        ""
      ],
      "Requires": [
        ""
      ],
      "Fallback": [
        ""
      ],
      "Name of the verification measure": [
        ""
      ],
      "Read only": [
        ""
      ],
      "Prevent the customer of changing the address": [
        ""
      ],
      "Configure verification measure: \"%1$s\"": [
        ""
      ],
      "Add custom measure": [
        ""
      ],
      "Custom measures": [
        ""
      ],
      "Server measures": [
        ""
      ],
      "New measures": [
        ""
      ],
      "Measures that are available/executed directly after the decision is confirmed.": [
        ""
      ],
      "Reset to default": [
        ""
      ],
      "Properties": [
        ""
      ],
      "Custom properties": [
        ""
      ],
      "Add more properties that not listed above.": [
        ""
      ],
      "New properties": [
        ""
      ],
      "Current note on the GWG File": [
        ""
      ],
      "Customer name or internal alias.": [
        ""
      ],
      "Is the account active for deposit and payments?": [
        ""
      ],
      "Is the account idle?": [
        ""
      ],
      "No operation for a period of time": [
        ""
      ],
      "Does account belong to a domestic PEP?": [
        ""
      ],
      "Does account belong to a foreign PEP?": [
        ""
      ],
      "Does account belong to a international organization PEP?": [
        ""
      ],
      "Does account belong to a high risk customer?": [
        ""
      ],
      "Does account belong to a person from a high risk country?": [
        ""
      ],
      "The MROS reporting state for the account.": [
        ""
      ],
      "None": [
        ""
      ],
      "Pending investigation": [
        ""
      ],
      "Investigated without suspicion": [
        ""
      ],
      "Reported simple suspicion": [
        ""
      ],
      "Reported substantaited suspicion": [
        ""
      ],
      "Informal reason why the AML investigation was triggered": [
        ""
      ],
      "Identifies the sanction list entry that the account matched against": [
        ""
      ],
      "Best match, does not mean it was a good match.": [
        ""
      ],
      "Score for how good the sanction list match was": [
        ""
      ],
      "Score for how much supporting data we had for the sanction list match": [
        ""
      ],
      "Suppress flagging this account when it creates a hit on a sanctions list, this is a false-positive": [
        ""
      ],
      "GLS Customer ID": [
        ""
      ],
      "Wrong credentials for \"%1$s\"": [
        "Credenziali invalide."
      ],
      "The account was not found": [
        "Lista conti pubblici non trovata."
      ],
      "Officer disabled or more recent decision was already submitted.": [
        ""
      ],
      "Confirmation required": [
        ""
      ],
      "This decision declares that the MROS report have concluded. Please make that this is the case": [
        ""
      ],
      "I want to check first!": [
        ""
      ],
      "Confirm decision": [
        "Conferma"
      ],
      "Missing deadline": [
        ""
      ],
      "Deadline should specify when this rules ends and what is the next measures to apply after expiration.": [
        ""
      ],
      "Missing rules": [
        "indirizzo Payto"
      ],
      "Can't make a decision without rules.": [
        ""
      ],
      "Missing active measure": [
        "indirizzo Payto"
      ],
      "You should specify in the measure section.": [
        ""
      ],
      "No customer action required.": [
        ""
      ],
      "No active measure has been selected.": [
        ""
      ],
      "Missing properties": [
        "indirizzo Payto"
      ],
      "You should specify in the properties section.": [
        ""
      ],
      "Missing events": [
        ""
      ],
      "You should specify in the events section.": [
        ""
      ],
      "Missing justification": [
        ""
      ],
      "You must complete the justification section.": [
        ""
      ],
      "Invalid attributes": [
        ""
      ],
      "You should check form errors or submit a decision without attributes.": [
        ""
      ],
      "Clear": [
        "Cancella"
      ],
      "Send decision": [
        ""
      ],
      "Prev": [
        ""
      ],
      "Next": [
        ""
      ],
      "Attributes": [
        ""
      ],
      "Add more information about the customer": [
        ""
      ],
      "Rules": [
        ""
      ],
      "Set the limit of the operations": [
        ""
      ],
      "Trigger notifications.": [
        ""
      ],
      "Measures": [
        ""
      ],
      "Ask the customer to take action.": [
        ""
      ],
      "Describe the decision.": [
        ""
      ],
      "Flag the current account state.": [
        ""
      ],
      "Summary": [
        ""
      ],
      "Review and submit.": [
        ""
      ],
      "Decision for account:": [
        ""
      ],
      "Decision for wallet:": [
        ""
      ],
      "Decision for bank account:": [
        "Trasferisci fondi a un altro conto di questa banca:"
      ],
      "Search account": [
        ""
      ],
      "Check account details": [
        ""
      ],
      "Account most recent decisions": [
        ""
      ],
      "When": [
        ""
      ],
      "LATEST": [
        ""
      ],
      "ACTIVE": [
        ""
      ],
      "Account not found": [
        ""
      ],
      "There is no history known for this account yet.": [
        ""
      ],
      "You can make a decision for this account anyway.": [
        ""
      ],
      "Search": [
        ""
      ],
      "IBAN": [
        ""
      ],
      "Taler Bank": [
        "Banca 'demo'"
      ],
      "Wallet": [
        ""
      ],
      "Generic Payto:// URI": [
        ""
      ],
      "Account type": [
        ""
      ],
      "Should be 52 characters.": [
        ""
      ],
      "Failed to parse: %1$s": [
        ""
      ],
      "Address contains invalid characters.": [
        ""
      ],
      "Can't mix uppercased and lowercased characters.": [
        ""
      ],
      "The addr checksum doesn't match.": [
        ""
      ],
      "Missing separator or address type.": [
        "indirizzo Payto"
      ],
      "Address should have less than 90 characters.": [
        ""
      ],
      "Address should have more than 6 characters.": [
        ""
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "The string should start with payto://": [
        ""
      ],
      "After the target type it should follow a '/' with more information.": [
        ""
      ],
      "The target type is not supported, only x-taler-bank, taler-reserve or iban": [
        ""
      ],
      "IBAN is missing": [
        ""
      ],
      "Bitcoin address is missing": [
        ""
      ],
      "Bank host and account is missing": [
        ""
      ],
      "Exchange host and account is missing": [
        ""
      ],
      "Ethereum address is missing": [
        ""
      ],
      "Invalid IBAN: %1$s": [
        ""
      ],
      "Invalid BTC: %1$s": [
        ""
      ],
      "Invalid reserve: %1$s": [
        ""
      ],
      "Invalid host": [
        ""
      ],
      "Invalid address": [
        "indirizzo Payto"
      ],
      "Payto URI": [
        ""
      ],
      "As defined by RFC 8905": [
        ""
      ],
      "payto://": [
        ""
      ],
      "International Bank Account Number": [
        ""
      ],
      "DE1231231231": [
        ""
      ],
      "Host": [
        ""
      ],
      "Exchange hostname": [
        ""
      ],
      "exchange.taler.net": [
        ""
      ],
      "Remove last '/'": [
        ""
      ],
      "Invalid value": [
        ""
      ],
      "ID": [
        ""
      ],
      "Wallet reserve public key": [
        ""
      ],
      "abcdef1235": [
        ""
      ],
      "Should be 52 characters": [
        ""
      ],
      "Account": [
        "Somma"
      ],
      "Bank account identification": [
        "Bilancio:"
      ],
      "DE123123123": [
        ""
      ],
      "Hostname": [
        ""
      ],
      "Without the scheme, may include subpath: bank.com, bank.com/path/": [
        ""
      ],
      "bank.demo.taler.net": [
        ""
      ],
      "Invalid hostname": [
        ""
      ],
      "Different form version": [
        ""
      ],
      "The collected information don't have the form version used.": [
        ""
      ],
      "The collected information used the form version \"%1$s \" but is not available.": [
        ""
      ],
      "Case details": [
        ""
      ],
      "Next event": [
        ""
      ],
      "Previous event": [
        ""
      ],
      "Direction": [
        ""
      ],
      "Credit": [
        ""
      ],
      "Debit": [
        ""
      ],
      "KYC Auth": [
        ""
      ],
      "Minimum amount": [
        ""
      ],
      "All amounts below the given threshold will be filtered.": [
        ""
      ],
      "Select transaction type": [
        "Ultime transazioni:"
      ],
      "Please select a transaction type to show transactions.": [
        ""
      ],
      "Transfers history for account": [
        "Trasferisci fondi a un altro conto di questa banca:"
      ],
      "Transfers history": [
        ""
      ],
      "No transfers yet.": [
        "Bonifico"
      ],
      "There are no transfer for this account.": [
        "Trasferisci fondi a un altro conto di questa banca:"
      ],
      "There are no transfer reported to the exchange with the current threshold.": [
        ""
      ],
      "Date": [
        ""
      ],
      "Amount": [
        "Somma"
      ],
      "sent": [
        ""
      ],
      "received": [
        ""
      ],
      "to": [
        ""
      ],
      "from": [
        ""
      ],
      "Welcome to exchange config!": [
        "Benvenuti in banca euFin!"
      ],
      "Transfers": [
        ""
      ],
      "Profile": [
        ""
      ],
      "A list of all the pre-define measures in your that can used.": [
        ""
      ],
      "threshold\u0004%1$s every %2$s": [
        ""
      ],
      "operation type\u0004Withdraw": [
        "Conferma il ritiro"
      ],
      "operation type\u0004Deposit": [
        ""
      ],
      "operation type\u0004Merge": [
        ""
      ],
      "operation type\u0004Aggregate": [
        ""
      ],
      "operation type\u0004Balance": [
        ""
      ],
      "operation type\u0004Refund": [
        ""
      ],
      "operation type\u0004Close": [
        ""
      ],
      "operation type\u0004Transaction": [
        "Ultime transazioni:"
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "it",
  "completeness": 6
};

strings['fr'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n > 1;",
        "lang": "fr"
      },
      "Show debug info": [
        ""
      ],
      "Use testing dialect": [
        ""
      ],
      "Allow Insecure password": [
        ""
      ],
      "Keep session after reload": [
        ""
      ],
      "Don't compress request": [
        ""
      ],
      "Internal error, please report.": [
        ""
      ],
      "Preferences": [
        ""
      ],
      "Password": [
        ""
      ],
      "Required": [
        ""
      ],
      "should have at least 12 characters": [
        ""
      ],
      "should have lowercase and uppercase characters": [
        ""
      ],
      "should have numbers": [
        ""
      ],
      "should have at least one character which is not a number or letter": [
        ""
      ],
      "Repeat password": [
        ""
      ],
      "doesn't match": [
        ""
      ],
      "Create account": [
        ""
      ],
      "Create": [
        ""
      ],
      "Couldn't unlock the account, the password may be wrong.": [
        ""
      ],
      "Account locked": [
        ""
      ],
      "Your account is normally locked anytime you reload. To unlock type your password again.": [
        ""
      ],
      "Unlock": [
        ""
      ],
      "Forget account": [
        ""
      ],
      "There are no rules for operations": [
        ""
      ],
      "This mean that all operation have no limit.": [
        ""
      ],
      "Add custom rule": [
        ""
      ],
      "Operation": [
        ""
      ],
      "Threshold": [
        ""
      ],
      "Escalation": [
        ""
      ],
      "%1$s": [
        ""
      ],
      "(all)": [
        ""
      ],
      "Since": [
        ""
      ],
      "Expired": [
        ""
      ],
      "Expires": [
        ""
      ],
      "Successor measure": [
        ""
      ],
      "AML officer justification": [
        ""
      ],
      "Default rules": [
        ""
      ],
      "Forms": [
        ""
      ],
      "Measures used to get gather information about the customer.": [
        ""
      ],
      "Name": [
        ""
      ],
      "Check": [
        ""
      ],
      "Program": [
        ""
      ],
      "Context": [
        ""
      ],
      "Modify": [
        ""
      ],
      "Info": [
        ""
      ],
      "Will show information to the user.": [
        ""
      ],
      "Instant measures": [
        ""
      ],
      "Triggered immediately without customer interaction.": [
        ""
      ],
      "Input requirement": [
        ""
      ],
      "Already completed.": [
        ""
      ],
      "Waiting to be completed.": [
        ""
      ],
      "Current active rules": [
        ""
      ],
      "There are no rules for this account yet.": [
        ""
      ],
      "New rule form": [
        ""
      ],
      "Add": [
        ""
      ],
      "Cancel": [
        ""
      ],
      "Basic plan": [
        ""
      ],
      "Premium": [
        ""
      ],
      "E-commerce": [
        ""
      ],
      "Point-of-sale": [
        ""
      ],
      "GLS Registered E-Commerce Merchant": [
        ""
      ],
      "Check rules": [
        ""
      ],
      "The account is a wallet and there are bank account operation's limits which may trigger actions that are not required: %1$s": [
        ""
      ],
      "The account isn't a wallet and there are wallet operation's limits which may trigger actions that are not required: %1$s": [
        ""
      ],
      "The account is a wallet and there are wallet operation's limits which are not limited: %1$s": [
        ""
      ],
      "The AML account is bank account, but not all account-related operations are limited (%1$s).": [
        ""
      ],
      "New rules": [
        ""
      ],
      "Reset rules to current state": [
        ""
      ],
      "Set rules to default": [
        ""
      ],
      "Freeze account": [
        ""
      ],
      "Behavior on rule expiration": [
        ""
      ],
      "Reset measure": [
        ""
      ],
      "Reset expiration": [
        ""
      ],
      "Operation type": [
        ""
      ],
      "Timeframe": [
        ""
      ],
      "Exposed": [
        ""
      ],
      "Is this limit communicated to the customer?": [
        ""
      ],
      "Esclation measure": [
        ""
      ],
      "Measures that the customer will need to satisfy to apply for a new threshold.": [
        ""
      ],
      "All measures": [
        ""
      ],
      "Hint the customer that all measure should be completed": [
        ""
      ],
      "Expiration": [
        ""
      ],
      "Predefined shortcuts": [
        ""
      ],
      "In a week": [
        ""
      ],
      "In a month": [
        ""
      ],
      "In a year": [
        ""
      ],
      "Never": [
        ""
      ],
      "Expiration date": [
        ""
      ],
      "Until when are the new rules active.": [
        ""
      ],
      "Measure taken automatically upon expiration of the current decision.": [
        ""
      ],
      "New decision": [
        ""
      ],
      "Show transfers": [
        ""
      ],
      "Case history for selected account": [
        ""
      ],
      "ID:": [
        ""
      ],
      "Payto:": [
        ""
      ],
      "Under investigation": [
        ""
      ],
      "This account requires a manual review and is waiting for a decision to be made.": [
        ""
      ],
      "Collected information": [
        ""
      ],
      "Every attribute collection event (via form upload or KYC provider).": [
        ""
      ],
      "The event list is empty": [
        ""
      ],
      "Current account properties": [
        ""
      ],
      "No active decision found": [
        ""
      ],
      "There is no decision for this account yet. The account is limited by the default rules.": [
        ""
      ],
      "Previous AML decisions": [
        ""
      ],
      "No aml history found": [
        ""
      ],
      "Current active measures": [
        ""
      ],
      "Now": [
        ""
      ],
      "Public key": [
        ""
      ],
      "Request account activation": [
        ""
      ],
      "Lock account": [
        ""
      ],
      "Operation denied": [
        ""
      ],
      "This account signature is invalid, contact administrator or create a new one.": [
        ""
      ],
      "The designated AML account is not known, contact administrator or create a new one.": [
        ""
      ],
      "The designated AML account is not enabled, contact administrator or create a new one.": [
        ""
      ],
      "High risk accounts. Only with custom rules.": [
        ""
      ],
      "Account with custom rules.": [
        ""
      ],
      "High risk accounts": [
        ""
      ],
      "All account known.": [
        ""
      ],
      "High risk accounts under investigation. Only with custom rules.": [
        ""
      ],
      "Accounts under investigation. Only with custom rules.": [
        ""
      ],
      "High risk accounts under investigation.": [
        ""
      ],
      "Accounts under investigation.": [
        ""
      ],
      "High risk accounts without investigation. Only with custom rules.": [
        ""
      ],
      "Accounts without investigation. Only with custom rules.": [
        ""
      ],
      "High risk accounts without investigation.": [
        ""
      ],
      "Accounts without investigation.": [
        ""
      ],
      "Accounts": [
        ""
      ],
      "Export as file": [
        ""
      ],
      "Export completed": [
        ""
      ],
      "Click here to save the file in your computer.": [
        ""
      ],
      "No results": [
        ""
      ],
      "#": [
        ""
      ],
      "Account Identification": [
        ""
      ],
      "Open time": [
        ""
      ],
      "Status": [
        ""
      ],
      "From %1$s%2$sTo %3$s": [
        ""
      ],
      "Since %1$s": [
        ""
      ],
      "Not opened": [
        ""
      ],
      "High risk": [
        ""
      ],
      "With custom rules": [
        ""
      ],
      "Invalid account": [
        ""
      ],
      "Search by ID": [
        ""
      ],
      "Under investigation: ": [
        ""
      ],
      "High risk:": [
        ""
      ],
      "Open:": [
        ""
      ],
      "Dashboard": [
        ""
      ],
      "Statistics": [
        ""
      ],
      "Accounts open": [
        ""
      ],
      "New files last year": [
        ""
      ],
      "Closes files last year": [
        ""
      ],
      "MROS substantaiated": [
        ""
      ],
      "MROS simple": [
        ""
      ],
      "High risk files": [
        ""
      ],
      "PEP files": [
        ""
      ],
      "Under investiation": [
        ""
      ],
      "Number of accounts that are opened": [
        ""
      ],
      "Number of new GwG files in the last year": [
        ""
      ],
      "Number of GwG files closed in the last year": [
        ""
      ],
      "Number of MROS reports based on Art 9 Abs. 1 GwG (per year)": [
        ""
      ],
      "Number of MROS reports based on Art 305ter Abs. 2 StGB (per year)": [
        ""
      ],
      "Number of GwG files of high-risk customers": [
        ""
      ],
      "Number of GwG files managed with “increased risk” due to PEP status": [
        ""
      ],
      "Number of customers involved in proceedings for which Art 6 GwG did apply": [
        ""
      ],
      "previous": [
        ""
      ],
      "Decreased by": [
        ""
      ],
      "Increased by": [
        ""
      ],
      "Events": [
        ""
      ],
      "The events will be triggered only after the decision has been confirmed.": [
        ""
      ],
      "Calculated": [
        ""
      ],
      "Events based on properties and new attributes that should be triggered": [
        ""
      ],
      "Others": [
        ""
      ],
      "Events that can be triggered manually": [
        ""
      ],
      "Custom list": [
        ""
      ],
      "Events without built-in support.": [
        ""
      ],
      "Complete the form": [
        ""
      ],
      "These measures will ask the customer for information.": [
        ""
      ],
      "change form": [
        ""
      ],
      "Expiration date of the information filled in this form.": [
        ""
      ],
      "Include new information": [
        ""
      ],
      "Select a form to submit new information about the customer or continue to the next step": [
        ""
      ],
      "Form:": [
        ""
      ],
      "Justification": [
        ""
      ],
      "Keep investigation?": [
        ""
      ],
      "Account holder": [
        ""
      ],
      "Full name of the account holder": [
        ""
      ],
      "loading...": [
        ""
      ],
      "Update": [
        ""
      ],
      "Remove": [
        ""
      ],
      "Configure verification type: %1$s": [
        ""
      ],
      "Show complete form": [
        ""
      ],
      "Configure measure": [
        ""
      ],
      "Show as verification": [
        ""
      ],
      "There is already a measure with that name": [
        ""
      ],
      "Only required when no check is specified": [
        ""
      ],
      "Missing check or program": [
        ""
      ],
      "Without a check the program will run automatically": [
        ""
      ],
      "Field name": [
        ""
      ],
      "Type": [
        ""
      ],
      "string": [
        ""
      ],
      "number": [
        ""
      ],
      "boolean": [
        ""
      ],
      "json": [
        ""
      ],
      "Value": [
        ""
      ],
      "There are unsatisfied inputs: %1$s": [
        ""
      ],
      "There are missing inputs: %1$s": [
        ""
      ],
      "There are missing requirements: %1$s": [
        ""
      ],
      "Can't be empty": [
        ""
      ],
      "It should be a number": [
        ""
      ],
      "Not a number": [
        ""
      ],
      "It should be finite": [
        ""
      ],
      "It should be a safe integer": [
        ""
      ],
      "It should be either \"true\" or \"false\"": [
        ""
      ],
      "Couldn't parse as json string: %1$s": [
        ""
      ],
      "Description": [
        ""
      ],
      "Inputs": [
        ""
      ],
      "Output": [
        ""
      ],
      "Requires": [
        ""
      ],
      "Fallback": [
        ""
      ],
      "Name of the verification measure": [
        ""
      ],
      "Read only": [
        ""
      ],
      "Prevent the customer of changing the address": [
        ""
      ],
      "Configure verification measure: \"%1$s\"": [
        ""
      ],
      "Add custom measure": [
        ""
      ],
      "Custom measures": [
        ""
      ],
      "Server measures": [
        ""
      ],
      "New measures": [
        ""
      ],
      "Measures that are available/executed directly after the decision is confirmed.": [
        ""
      ],
      "Reset to default": [
        ""
      ],
      "Properties": [
        ""
      ],
      "Custom properties": [
        ""
      ],
      "Add more properties that not listed above.": [
        ""
      ],
      "New properties": [
        ""
      ],
      "Current note on the GWG File": [
        ""
      ],
      "Customer name or internal alias.": [
        ""
      ],
      "Is the account active for deposit and payments?": [
        ""
      ],
      "Is the account idle?": [
        ""
      ],
      "No operation for a period of time": [
        ""
      ],
      "Does account belong to a domestic PEP?": [
        ""
      ],
      "Does account belong to a foreign PEP?": [
        ""
      ],
      "Does account belong to a international organization PEP?": [
        ""
      ],
      "Does account belong to a high risk customer?": [
        ""
      ],
      "Does account belong to a person from a high risk country?": [
        ""
      ],
      "The MROS reporting state for the account.": [
        ""
      ],
      "None": [
        ""
      ],
      "Pending investigation": [
        ""
      ],
      "Investigated without suspicion": [
        ""
      ],
      "Reported simple suspicion": [
        ""
      ],
      "Reported substantaited suspicion": [
        ""
      ],
      "Informal reason why the AML investigation was triggered": [
        ""
      ],
      "Identifies the sanction list entry that the account matched against": [
        ""
      ],
      "Best match, does not mean it was a good match.": [
        ""
      ],
      "Score for how good the sanction list match was": [
        ""
      ],
      "Score for how much supporting data we had for the sanction list match": [
        ""
      ],
      "Suppress flagging this account when it creates a hit on a sanctions list, this is a false-positive": [
        ""
      ],
      "GLS Customer ID": [
        ""
      ],
      "Wrong credentials for \"%1$s\"": [
        ""
      ],
      "The account was not found": [
        ""
      ],
      "Officer disabled or more recent decision was already submitted.": [
        ""
      ],
      "Confirmation required": [
        ""
      ],
      "This decision declares that the MROS report have concluded. Please make that this is the case": [
        ""
      ],
      "I want to check first!": [
        ""
      ],
      "Confirm decision": [
        ""
      ],
      "Missing deadline": [
        ""
      ],
      "Deadline should specify when this rules ends and what is the next measures to apply after expiration.": [
        ""
      ],
      "Missing rules": [
        ""
      ],
      "Can't make a decision without rules.": [
        ""
      ],
      "Missing active measure": [
        ""
      ],
      "You should specify in the measure section.": [
        ""
      ],
      "No customer action required.": [
        ""
      ],
      "No active measure has been selected.": [
        ""
      ],
      "Missing properties": [
        ""
      ],
      "You should specify in the properties section.": [
        ""
      ],
      "Missing events": [
        ""
      ],
      "You should specify in the events section.": [
        ""
      ],
      "Missing justification": [
        ""
      ],
      "You must complete the justification section.": [
        ""
      ],
      "Invalid attributes": [
        ""
      ],
      "You should check form errors or submit a decision without attributes.": [
        ""
      ],
      "Clear": [
        ""
      ],
      "Send decision": [
        ""
      ],
      "Prev": [
        ""
      ],
      "Next": [
        ""
      ],
      "Attributes": [
        ""
      ],
      "Add more information about the customer": [
        ""
      ],
      "Rules": [
        ""
      ],
      "Set the limit of the operations": [
        ""
      ],
      "Trigger notifications.": [
        ""
      ],
      "Measures": [
        ""
      ],
      "Ask the customer to take action.": [
        ""
      ],
      "Describe the decision.": [
        ""
      ],
      "Flag the current account state.": [
        ""
      ],
      "Summary": [
        ""
      ],
      "Review and submit.": [
        ""
      ],
      "Decision for account:": [
        ""
      ],
      "Decision for wallet:": [
        ""
      ],
      "Decision for bank account:": [
        ""
      ],
      "Search account": [
        ""
      ],
      "Check account details": [
        ""
      ],
      "Account most recent decisions": [
        ""
      ],
      "When": [
        ""
      ],
      "LATEST": [
        ""
      ],
      "ACTIVE": [
        ""
      ],
      "Account not found": [
        ""
      ],
      "There is no history known for this account yet.": [
        ""
      ],
      "You can make a decision for this account anyway.": [
        ""
      ],
      "Search": [
        ""
      ],
      "IBAN": [
        ""
      ],
      "Taler Bank": [
        ""
      ],
      "Wallet": [
        ""
      ],
      "Generic Payto:// URI": [
        ""
      ],
      "Account type": [
        ""
      ],
      "Should be 52 characters.": [
        ""
      ],
      "Failed to parse: %1$s": [
        ""
      ],
      "Address contains invalid characters.": [
        ""
      ],
      "Can't mix uppercased and lowercased characters.": [
        ""
      ],
      "The addr checksum doesn't match.": [
        ""
      ],
      "Missing separator or address type.": [
        ""
      ],
      "Address should have less than 90 characters.": [
        ""
      ],
      "Address should have more than 6 characters.": [
        ""
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "The string should start with payto://": [
        ""
      ],
      "After the target type it should follow a '/' with more information.": [
        ""
      ],
      "The target type is not supported, only x-taler-bank, taler-reserve or iban": [
        ""
      ],
      "IBAN is missing": [
        ""
      ],
      "Bitcoin address is missing": [
        ""
      ],
      "Bank host and account is missing": [
        ""
      ],
      "Exchange host and account is missing": [
        ""
      ],
      "Ethereum address is missing": [
        ""
      ],
      "Invalid IBAN: %1$s": [
        ""
      ],
      "Invalid BTC: %1$s": [
        ""
      ],
      "Invalid reserve: %1$s": [
        ""
      ],
      "Invalid host": [
        ""
      ],
      "Invalid address": [
        ""
      ],
      "Payto URI": [
        ""
      ],
      "As defined by RFC 8905": [
        ""
      ],
      "payto://": [
        ""
      ],
      "International Bank Account Number": [
        ""
      ],
      "DE1231231231": [
        ""
      ],
      "Host": [
        ""
      ],
      "Exchange hostname": [
        ""
      ],
      "exchange.taler.net": [
        ""
      ],
      "Remove last '/'": [
        ""
      ],
      "Invalid value": [
        ""
      ],
      "ID": [
        ""
      ],
      "Wallet reserve public key": [
        ""
      ],
      "abcdef1235": [
        ""
      ],
      "Should be 52 characters": [
        ""
      ],
      "Account": [
        ""
      ],
      "Bank account identification": [
        ""
      ],
      "DE123123123": [
        ""
      ],
      "Hostname": [
        ""
      ],
      "Without the scheme, may include subpath: bank.com, bank.com/path/": [
        ""
      ],
      "bank.demo.taler.net": [
        ""
      ],
      "Invalid hostname": [
        ""
      ],
      "Different form version": [
        ""
      ],
      "The collected information don't have the form version used.": [
        ""
      ],
      "The collected information used the form version \"%1$s \" but is not available.": [
        ""
      ],
      "Case details": [
        ""
      ],
      "Next event": [
        ""
      ],
      "Previous event": [
        ""
      ],
      "Direction": [
        ""
      ],
      "Credit": [
        ""
      ],
      "Debit": [
        ""
      ],
      "KYC Auth": [
        ""
      ],
      "Minimum amount": [
        ""
      ],
      "All amounts below the given threshold will be filtered.": [
        ""
      ],
      "Select transaction type": [
        ""
      ],
      "Please select a transaction type to show transactions.": [
        ""
      ],
      "Transfers history for account": [
        ""
      ],
      "Transfers history": [
        ""
      ],
      "No transfers yet.": [
        ""
      ],
      "There are no transfer for this account.": [
        ""
      ],
      "There are no transfer reported to the exchange with the current threshold.": [
        ""
      ],
      "Date": [
        ""
      ],
      "Amount": [
        ""
      ],
      "sent": [
        ""
      ],
      "received": [
        ""
      ],
      "to": [
        ""
      ],
      "from": [
        ""
      ],
      "Welcome to exchange config!": [
        ""
      ],
      "Transfers": [
        ""
      ],
      "Profile": [
        ""
      ],
      "A list of all the pre-define measures in your that can used.": [
        ""
      ],
      "threshold\u0004%1$s every %2$s": [
        ""
      ],
      "operation type\u0004Withdraw": [
        ""
      ],
      "operation type\u0004Deposit": [
        ""
      ],
      "operation type\u0004Merge": [
        ""
      ],
      "operation type\u0004Aggregate": [
        ""
      ],
      "operation type\u0004Balance": [
        ""
      ],
      "operation type\u0004Refund": [
        ""
      ],
      "operation type\u0004Close": [
        ""
      ],
      "operation type\u0004Transaction": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n > 1;",
  "lang": "fr",
  "completeness": 0
};

strings['es'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "es"
      },
      "Show debug info": [
        ""
      ],
      "Use testing dialect": [
        ""
      ],
      "Allow Insecure password": [
        ""
      ],
      "Keep session after reload": [
        ""
      ],
      "Don't compress request": [
        ""
      ],
      "Internal error, please report.": [
        "El registro falló, por favor reportelo"
      ],
      "Preferences": [
        ""
      ],
      "Password": [
        "Password:"
      ],
      "Required": [
        ""
      ],
      "should have at least 12 characters": [
        ""
      ],
      "should have lowercase and uppercase characters": [
        ""
      ],
      "should have numbers": [
        ""
      ],
      "should have at least one character which is not a number or letter": [
        ""
      ],
      "Repeat password": [
        "Repita la contraseña:"
      ],
      "doesn't match": [
        "La contraseña no coincide"
      ],
      "Create account": [
        ""
      ],
      "Create": [
        ""
      ],
      "Couldn't unlock the account, the password may be wrong.": [
        ""
      ],
      "Account locked": [
        ""
      ],
      "Your account is normally locked anytime you reload. To unlock type your password again.": [
        ""
      ],
      "Unlock": [
        ""
      ],
      "Forget account": [
        ""
      ],
      "There are no rules for operations": [
        ""
      ],
      "This mean that all operation have no limit.": [
        ""
      ],
      "Add custom rule": [
        ""
      ],
      "Operation": [
        ""
      ],
      "Threshold": [
        ""
      ],
      "Escalation": [
        ""
      ],
      "%1$s": [
        ""
      ],
      "(all)": [
        ""
      ],
      "Since": [
        ""
      ],
      "Expired": [
        ""
      ],
      "Expires": [
        ""
      ],
      "Successor measure": [
        ""
      ],
      "AML officer justification": [
        ""
      ],
      "Default rules": [
        ""
      ],
      "Forms": [
        ""
      ],
      "Measures used to get gather information about the customer.": [
        ""
      ],
      "Name": [
        ""
      ],
      "Check": [
        ""
      ],
      "Program": [
        ""
      ],
      "Context": [
        ""
      ],
      "Modify": [
        ""
      ],
      "Info": [
        ""
      ],
      "Will show information to the user.": [
        ""
      ],
      "Instant measures": [
        ""
      ],
      "Triggered immediately without customer interaction.": [
        ""
      ],
      "Input requirement": [
        ""
      ],
      "Already completed.": [
        ""
      ],
      "Waiting to be completed.": [
        ""
      ],
      "Current active rules": [
        ""
      ],
      "There are no rules for this account yet.": [
        ""
      ],
      "New rule form": [
        ""
      ],
      "Add": [
        ""
      ],
      "Cancel": [
        "Cancelar"
      ],
      "Basic plan": [
        ""
      ],
      "Premium": [
        ""
      ],
      "E-commerce": [
        ""
      ],
      "Point-of-sale": [
        ""
      ],
      "GLS Registered E-Commerce Merchant": [
        ""
      ],
      "Check rules": [
        ""
      ],
      "The account is a wallet and there are bank account operation's limits which may trigger actions that are not required: %1$s": [
        ""
      ],
      "The account isn't a wallet and there are wallet operation's limits which may trigger actions that are not required: %1$s": [
        ""
      ],
      "The account is a wallet and there are wallet operation's limits which are not limited: %1$s": [
        ""
      ],
      "The AML account is bank account, but not all account-related operations are limited (%1$s).": [
        ""
      ],
      "New rules": [
        ""
      ],
      "Reset rules to current state": [
        ""
      ],
      "Set rules to default": [
        ""
      ],
      "Freeze account": [
        ""
      ],
      "Behavior on rule expiration": [
        ""
      ],
      "Reset measure": [
        ""
      ],
      "Reset expiration": [
        "Seleccione opción 1"
      ],
      "Operation type": [
        ""
      ],
      "Timeframe": [
        ""
      ],
      "Exposed": [
        ""
      ],
      "Is this limit communicated to the customer?": [
        ""
      ],
      "Esclation measure": [
        ""
      ],
      "Measures that the customer will need to satisfy to apply for a new threshold.": [
        ""
      ],
      "All measures": [
        ""
      ],
      "Hint the customer that all measure should be completed": [
        ""
      ],
      "Expiration": [
        ""
      ],
      "Predefined shortcuts": [
        ""
      ],
      "In a week": [
        ""
      ],
      "In a month": [
        ""
      ],
      "In a year": [
        ""
      ],
      "Never": [
        ""
      ],
      "Expiration date": [
        ""
      ],
      "Until when are the new rules active.": [
        ""
      ],
      "Measure taken automatically upon expiration of the current decision.": [
        ""
      ],
      "New decision": [
        ""
      ],
      "Show transfers": [
        ""
      ],
      "Case history for selected account": [
        "Historial de cuentas públicas"
      ],
      "ID:": [
        ""
      ],
      "Payto:": [
        "payto URI:"
      ],
      "Under investigation": [
        ""
      ],
      "This account requires a manual review and is waiting for a decision to be made.": [
        ""
      ],
      "Collected information": [
        ""
      ],
      "Every attribute collection event (via form upload or KYC provider).": [
        ""
      ],
      "The event list is empty": [
        ""
      ],
      "Current account properties": [
        ""
      ],
      "No active decision found": [
        "Se dieron las credenciales incorrectas."
      ],
      "There is no decision for this account yet. The account is limited by the default rules.": [
        ""
      ],
      "Previous AML decisions": [
        ""
      ],
      "No aml history found": [
        ""
      ],
      "Current active measures": [
        ""
      ],
      "Now": [
        ""
      ],
      "Public key": [
        ""
      ],
      "Request account activation": [
        ""
      ],
      "Lock account": [
        ""
      ],
      "Operation denied": [
        ""
      ],
      "This account signature is invalid, contact administrator or create a new one.": [
        ""
      ],
      "The designated AML account is not known, contact administrator or create a new one.": [
        ""
      ],
      "The designated AML account is not enabled, contact administrator or create a new one.": [
        ""
      ],
      "High risk accounts. Only with custom rules.": [
        ""
      ],
      "Account with custom rules.": [
        ""
      ],
      "High risk accounts": [
        ""
      ],
      "All account known.": [
        ""
      ],
      "High risk accounts under investigation. Only with custom rules.": [
        ""
      ],
      "Accounts under investigation. Only with custom rules.": [
        ""
      ],
      "High risk accounts under investigation.": [
        ""
      ],
      "Accounts under investigation.": [
        ""
      ],
      "High risk accounts without investigation. Only with custom rules.": [
        ""
      ],
      "Accounts without investigation. Only with custom rules.": [
        ""
      ],
      "High risk accounts without investigation.": [
        ""
      ],
      "Accounts without investigation.": [
        ""
      ],
      "Accounts": [
        "Monto"
      ],
      "Export as file": [
        ""
      ],
      "Export completed": [
        ""
      ],
      "Click here to save the file in your computer.": [
        ""
      ],
      "No results": [
        ""
      ],
      "#": [
        ""
      ],
      "Account Identification": [
        ""
      ],
      "Open time": [
        ""
      ],
      "Status": [
        ""
      ],
      "From %1$s%2$sTo %3$s": [
        ""
      ],
      "Since %1$s": [
        ""
      ],
      "Not opened": [
        ""
      ],
      "High risk": [
        ""
      ],
      "With custom rules": [
        ""
      ],
      "Invalid account": [
        ""
      ],
      "Search by ID": [
        ""
      ],
      "Under investigation: ": [
        ""
      ],
      "High risk:": [
        ""
      ],
      "Open:": [
        ""
      ],
      "Dashboard": [
        ""
      ],
      "Statistics": [
        ""
      ],
      "Accounts open": [
        ""
      ],
      "New files last year": [
        ""
      ],
      "Closes files last year": [
        ""
      ],
      "MROS substantaiated": [
        ""
      ],
      "MROS simple": [
        ""
      ],
      "High risk files": [
        ""
      ],
      "PEP files": [
        ""
      ],
      "Under investiation": [
        ""
      ],
      "Number of accounts that are opened": [
        ""
      ],
      "Number of new GwG files in the last year": [
        ""
      ],
      "Number of GwG files closed in the last year": [
        ""
      ],
      "Number of MROS reports based on Art 9 Abs. 1 GwG (per year)": [
        ""
      ],
      "Number of MROS reports based on Art 305ter Abs. 2 StGB (per year)": [
        ""
      ],
      "Number of GwG files of high-risk customers": [
        ""
      ],
      "Number of GwG files managed with “increased risk” due to PEP status": [
        ""
      ],
      "Number of customers involved in proceedings for which Art 6 GwG did apply": [
        ""
      ],
      "previous": [
        ""
      ],
      "Decreased by": [
        ""
      ],
      "Increased by": [
        ""
      ],
      "Events": [
        ""
      ],
      "The events will be triggered only after the decision has been confirmed.": [
        ""
      ],
      "Calculated": [
        ""
      ],
      "Events based on properties and new attributes that should be triggered": [
        ""
      ],
      "Others": [
        ""
      ],
      "Events that can be triggered manually": [
        ""
      ],
      "Custom list": [
        ""
      ],
      "Events without built-in support.": [
        ""
      ],
      "Complete the form": [
        ""
      ],
      "These measures will ask the customer for information.": [
        ""
      ],
      "change form": [
        ""
      ],
      "Expiration date of the information filled in this form.": [
        ""
      ],
      "Include new information": [
        ""
      ],
      "Select a form to submit new information about the customer or continue to the next step": [
        ""
      ],
      "Form:": [
        ""
      ],
      "Justification": [
        ""
      ],
      "Keep investigation?": [
        ""
      ],
      "Account holder": [
        ""
      ],
      "Full name of the account holder": [
        ""
      ],
      "loading...": [
        ""
      ],
      "Update": [
        ""
      ],
      "Remove": [
        ""
      ],
      "Configure verification type: %1$s": [
        ""
      ],
      "Show complete form": [
        ""
      ],
      "Configure measure": [
        ""
      ],
      "Show as verification": [
        ""
      ],
      "There is already a measure with that name": [
        ""
      ],
      "Only required when no check is specified": [
        ""
      ],
      "Missing check or program": [
        "Falta nombre de usuario"
      ],
      "Without a check the program will run automatically": [
        ""
      ],
      "Field name": [
        ""
      ],
      "Type": [
        ""
      ],
      "string": [
        ""
      ],
      "number": [
        ""
      ],
      "boolean": [
        ""
      ],
      "json": [
        ""
      ],
      "Value": [
        ""
      ],
      "There are unsatisfied inputs: %1$s": [
        ""
      ],
      "There are missing inputs: %1$s": [
        ""
      ],
      "There are missing requirements: %1$s": [
        ""
      ],
      "Can't be empty": [
        ""
      ],
      "It should be a number": [
        ""
      ],
      "Not a number": [
        ""
      ],
      "It should be finite": [
        ""
      ],
      "It should be a safe integer": [
        ""
      ],
      "It should be either \"true\" or \"false\"": [
        ""
      ],
      "Couldn't parse as json string: %1$s": [
        ""
      ],
      "Description": [
        ""
      ],
      "Inputs": [
        ""
      ],
      "Output": [
        ""
      ],
      "Requires": [
        ""
      ],
      "Fallback": [
        ""
      ],
      "Name of the verification measure": [
        ""
      ],
      "Read only": [
        ""
      ],
      "Prevent the customer of changing the address": [
        ""
      ],
      "Configure verification measure: \"%1$s\"": [
        ""
      ],
      "Add custom measure": [
        ""
      ],
      "Custom measures": [
        ""
      ],
      "Server measures": [
        ""
      ],
      "New measures": [
        ""
      ],
      "Measures that are available/executed directly after the decision is confirmed.": [
        ""
      ],
      "Reset to default": [
        ""
      ],
      "Properties": [
        ""
      ],
      "Custom properties": [
        ""
      ],
      "Add more properties that not listed above.": [
        ""
      ],
      "New properties": [
        ""
      ],
      "Current note on the GWG File": [
        ""
      ],
      "Customer name or internal alias.": [
        ""
      ],
      "Is the account active for deposit and payments?": [
        ""
      ],
      "Is the account idle?": [
        ""
      ],
      "No operation for a period of time": [
        ""
      ],
      "Does account belong to a domestic PEP?": [
        ""
      ],
      "Does account belong to a foreign PEP?": [
        ""
      ],
      "Does account belong to a international organization PEP?": [
        ""
      ],
      "Does account belong to a high risk customer?": [
        ""
      ],
      "Does account belong to a person from a high risk country?": [
        ""
      ],
      "The MROS reporting state for the account.": [
        ""
      ],
      "None": [
        ""
      ],
      "Pending investigation": [
        ""
      ],
      "Investigated without suspicion": [
        ""
      ],
      "Reported simple suspicion": [
        ""
      ],
      "Reported substantaited suspicion": [
        ""
      ],
      "Informal reason why the AML investigation was triggered": [
        ""
      ],
      "Identifies the sanction list entry that the account matched against": [
        ""
      ],
      "Best match, does not mean it was a good match.": [
        ""
      ],
      "Score for how good the sanction list match was": [
        ""
      ],
      "Score for how much supporting data we had for the sanction list match": [
        ""
      ],
      "Suppress flagging this account when it creates a hit on a sanctions list, this is a false-positive": [
        ""
      ],
      "GLS Customer ID": [
        ""
      ],
      "Wrong credentials for \"%1$s\"": [
        "Se dieron las credenciales incorrectas."
      ],
      "The account was not found": [
        "La lista de cuentas públicas no fue encontrada."
      ],
      "Officer disabled or more recent decision was already submitted.": [
        ""
      ],
      "Confirmation required": [
        ""
      ],
      "This decision declares that the MROS report have concluded. Please make that this is the case": [
        ""
      ],
      "I want to check first!": [
        ""
      ],
      "Confirm decision": [
        "Confirmar"
      ],
      "Missing deadline": [
        "Falta nombre de usuario"
      ],
      "Deadline should specify when this rules ends and what is the next measures to apply after expiration.": [
        ""
      ],
      "Missing rules": [
        "Falta asunto"
      ],
      "Can't make a decision without rules.": [
        ""
      ],
      "Missing active measure": [
        "Falta direccion payto"
      ],
      "You should specify in the measure section.": [
        ""
      ],
      "No customer action required.": [
        ""
      ],
      "No active measure has been selected.": [
        ""
      ],
      "Missing properties": [
        "Falta direccion payto"
      ],
      "You should specify in the properties section.": [
        ""
      ],
      "Missing events": [
        "Falta monto"
      ],
      "You should specify in the events section.": [
        ""
      ],
      "Missing justification": [
        "Falta monto"
      ],
      "You must complete the justification section.": [
        ""
      ],
      "Invalid attributes": [
        ""
      ],
      "You should check form errors or submit a decision without attributes.": [
        ""
      ],
      "Clear": [
        ""
      ],
      "Send decision": [
        ""
      ],
      "Prev": [
        ""
      ],
      "Next": [
        ""
      ],
      "Attributes": [
        ""
      ],
      "Add more information about the customer": [
        ""
      ],
      "Rules": [
        ""
      ],
      "Set the limit of the operations": [
        ""
      ],
      "Trigger notifications.": [
        ""
      ],
      "Measures": [
        ""
      ],
      "Ask the customer to take action.": [
        ""
      ],
      "Describe the decision.": [
        ""
      ],
      "Flag the current account state.": [
        ""
      ],
      "Summary": [
        ""
      ],
      "Review and submit.": [
        ""
      ],
      "Decision for account:": [
        ""
      ],
      "Decision for wallet:": [
        ""
      ],
      "Decision for bank account:": [
        "Transferir a una cuenta bancaria"
      ],
      "Search account": [
        ""
      ],
      "Check account details": [
        ""
      ],
      "Account most recent decisions": [
        ""
      ],
      "When": [
        ""
      ],
      "LATEST": [
        ""
      ],
      "ACTIVE": [
        ""
      ],
      "Account not found": [
        "Monto no válido"
      ],
      "There is no history known for this account yet.": [
        ""
      ],
      "You can make a decision for this account anyway.": [
        ""
      ],
      "Search": [
        ""
      ],
      "IBAN": [
        ""
      ],
      "Taler Bank": [
        "Logo Taler"
      ],
      "Wallet": [
        ""
      ],
      "Generic Payto:// URI": [
        ""
      ],
      "Account type": [
        ""
      ],
      "Should be 52 characters.": [
        "Debería ser mas grande que 0"
      ],
      "Failed to parse: %1$s": [
        ""
      ],
      "Address contains invalid characters.": [
        ""
      ],
      "Can't mix uppercased and lowercased characters.": [
        ""
      ],
      "The addr checksum doesn't match.": [
        ""
      ],
      "Missing separator or address type.": [
        "Falta direccion payto"
      ],
      "Address should have less than 90 characters.": [
        ""
      ],
      "Address should have more than 6 characters.": [
        ""
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "The string should start with payto://": [
        ""
      ],
      "After the target type it should follow a '/' with more information.": [
        ""
      ],
      "The target type is not supported, only x-taler-bank, taler-reserve or iban": [
        ""
      ],
      "IBAN is missing": [
        "Faltan campo(s)."
      ],
      "Bitcoin address is missing": [
        ""
      ],
      "Bank host and account is missing": [
        ""
      ],
      "Exchange host and account is missing": [
        ""
      ],
      "Ethereum address is missing": [
        ""
      ],
      "Invalid IBAN: %1$s": [
        ""
      ],
      "Invalid BTC: %1$s": [
        ""
      ],
      "Invalid reserve: %1$s": [
        ""
      ],
      "Invalid host": [
        ""
      ],
      "Invalid address": [
        "direccion payto"
      ],
      "Payto URI": [
        "payto URI:"
      ],
      "As defined by RFC 8905": [
        ""
      ],
      "payto://": [
        "payto URI:"
      ],
      "International Bank Account Number": [
        ""
      ],
      "DE1231231231": [
        ""
      ],
      "Host": [
        ""
      ],
      "Exchange hostname": [
        ""
      ],
      "exchange.taler.net": [
        ""
      ],
      "Remove last '/'": [
        ""
      ],
      "Invalid value": [
        ""
      ],
      "ID": [
        ""
      ],
      "Wallet reserve public key": [
        ""
      ],
      "abcdef1235": [
        ""
      ],
      "Should be 52 characters": [
        "Debería ser mas grande que 0"
      ],
      "Account": [
        "Monto"
      ],
      "Bank account identification": [
        "Balance de cuenta bancaria"
      ],
      "DE123123123": [
        ""
      ],
      "Hostname": [
        ""
      ],
      "Without the scheme, may include subpath: bank.com, bank.com/path/": [
        ""
      ],
      "bank.demo.taler.net": [
        ""
      ],
      "Invalid hostname": [
        ""
      ],
      "Different form version": [
        ""
      ],
      "The collected information don't have the form version used.": [
        ""
      ],
      "The collected information used the form version \"%1$s \" but is not available.": [
        ""
      ],
      "Case details": [
        ""
      ],
      "Next event": [
        ""
      ],
      "Previous event": [
        ""
      ],
      "Direction": [
        ""
      ],
      "Credit": [
        ""
      ],
      "Debit": [
        ""
      ],
      "KYC Auth": [
        ""
      ],
      "Minimum amount": [
        "Falta monto"
      ],
      "All amounts below the given threshold will be filtered.": [
        ""
      ],
      "Select transaction type": [
        "Últimas transacciones:"
      ],
      "Please select a transaction type to show transactions.": [
        ""
      ],
      "Transfers history for account": [
        "Transferir a una cuenta bancaria"
      ],
      "Transfers history": [
        "Asunto de transferencia:"
      ],
      "No transfers yet.": [
        "Transferencia bancaria creada!"
      ],
      "There are no transfer for this account.": [
        "Transferir a una cuenta bancaria"
      ],
      "There are no transfer reported to the exchange with the current threshold.": [
        ""
      ],
      "Date": [
        "Fecha"
      ],
      "Amount": [
        "Monto"
      ],
      "sent": [
        ""
      ],
      "received": [
        ""
      ],
      "to": [
        ""
      ],
      "from": [
        ""
      ],
      "Welcome to exchange config!": [
        ""
      ],
      "Transfers": [
        "Asunto de transferencia:"
      ],
      "Profile": [
        ""
      ],
      "A list of all the pre-define measures in your that can used.": [
        ""
      ],
      "threshold\u0004%1$s every %2$s": [
        ""
      ],
      "operation type\u0004Withdraw": [
        "Retirar"
      ],
      "operation type\u0004Deposit": [
        ""
      ],
      "operation type\u0004Merge": [
        ""
      ],
      "operation type\u0004Aggregate": [
        ""
      ],
      "operation type\u0004Balance": [
        "Cancelar"
      ],
      "operation type\u0004Refund": [
        ""
      ],
      "operation type\u0004Close": [
        ""
      ],
      "operation type\u0004Transaction": [
        "Últimas transacciones:"
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "es",
  "completeness": 11
};

strings['en'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=(n != 1);",
        "lang": "en"
      },
      "Show debug info": [
        ""
      ],
      "Use testing dialect": [
        ""
      ],
      "Allow Insecure password": [
        ""
      ],
      "Keep session after reload": [
        ""
      ],
      "Don't compress request": [
        ""
      ],
      "Internal error, please report.": [
        ""
      ],
      "Preferences": [
        ""
      ],
      "Password": [
        ""
      ],
      "Required": [
        ""
      ],
      "should have at least 12 characters": [
        ""
      ],
      "should have lowercase and uppercase characters": [
        ""
      ],
      "should have numbers": [
        ""
      ],
      "should have at least one character which is not a number or letter": [
        ""
      ],
      "Repeat password": [
        ""
      ],
      "doesn't match": [
        ""
      ],
      "Create account": [
        ""
      ],
      "Create": [
        ""
      ],
      "Couldn't unlock the account, the password may be wrong.": [
        ""
      ],
      "Account locked": [
        ""
      ],
      "Your account is normally locked anytime you reload. To unlock type your password again.": [
        ""
      ],
      "Unlock": [
        ""
      ],
      "Forget account": [
        ""
      ],
      "There are no rules for operations": [
        ""
      ],
      "This mean that all operation have no limit.": [
        ""
      ],
      "Add custom rule": [
        ""
      ],
      "Operation": [
        ""
      ],
      "Threshold": [
        ""
      ],
      "Escalation": [
        ""
      ],
      "%1$s": [
        ""
      ],
      "(all)": [
        ""
      ],
      "Since": [
        ""
      ],
      "Expired": [
        ""
      ],
      "Expires": [
        ""
      ],
      "Successor measure": [
        ""
      ],
      "AML officer justification": [
        ""
      ],
      "Default rules": [
        ""
      ],
      "Forms": [
        ""
      ],
      "Measures used to get gather information about the customer.": [
        ""
      ],
      "Name": [
        ""
      ],
      "Check": [
        ""
      ],
      "Program": [
        ""
      ],
      "Context": [
        ""
      ],
      "Modify": [
        ""
      ],
      "Info": [
        ""
      ],
      "Will show information to the user.": [
        ""
      ],
      "Instant measures": [
        ""
      ],
      "Triggered immediately without customer interaction.": [
        ""
      ],
      "Input requirement": [
        ""
      ],
      "Already completed.": [
        ""
      ],
      "Waiting to be completed.": [
        ""
      ],
      "Current active rules": [
        ""
      ],
      "There are no rules for this account yet.": [
        ""
      ],
      "New rule form": [
        ""
      ],
      "Add": [
        ""
      ],
      "Cancel": [
        ""
      ],
      "Basic plan": [
        ""
      ],
      "Premium": [
        ""
      ],
      "E-commerce": [
        ""
      ],
      "Point-of-sale": [
        ""
      ],
      "GLS Registered E-Commerce Merchant": [
        ""
      ],
      "Check rules": [
        ""
      ],
      "The account is a wallet and there are bank account operation's limits which may trigger actions that are not required: %1$s": [
        ""
      ],
      "The account isn't a wallet and there are wallet operation's limits which may trigger actions that are not required: %1$s": [
        ""
      ],
      "The account is a wallet and there are wallet operation's limits which are not limited: %1$s": [
        ""
      ],
      "The AML account is bank account, but not all account-related operations are limited (%1$s).": [
        ""
      ],
      "New rules": [
        ""
      ],
      "Reset rules to current state": [
        ""
      ],
      "Set rules to default": [
        ""
      ],
      "Freeze account": [
        ""
      ],
      "Behavior on rule expiration": [
        ""
      ],
      "Reset measure": [
        ""
      ],
      "Reset expiration": [
        ""
      ],
      "Operation type": [
        ""
      ],
      "Timeframe": [
        ""
      ],
      "Exposed": [
        ""
      ],
      "Is this limit communicated to the customer?": [
        ""
      ],
      "Esclation measure": [
        ""
      ],
      "Measures that the customer will need to satisfy to apply for a new threshold.": [
        ""
      ],
      "All measures": [
        ""
      ],
      "Hint the customer that all measure should be completed": [
        ""
      ],
      "Expiration": [
        ""
      ],
      "Predefined shortcuts": [
        ""
      ],
      "In a week": [
        ""
      ],
      "In a month": [
        ""
      ],
      "In a year": [
        ""
      ],
      "Never": [
        ""
      ],
      "Expiration date": [
        ""
      ],
      "Until when are the new rules active.": [
        ""
      ],
      "Measure taken automatically upon expiration of the current decision.": [
        ""
      ],
      "New decision": [
        ""
      ],
      "Show transfers": [
        ""
      ],
      "Case history for selected account": [
        ""
      ],
      "ID:": [
        ""
      ],
      "Payto:": [
        ""
      ],
      "Under investigation": [
        ""
      ],
      "This account requires a manual review and is waiting for a decision to be made.": [
        ""
      ],
      "Collected information": [
        ""
      ],
      "Every attribute collection event (via form upload or KYC provider).": [
        ""
      ],
      "The event list is empty": [
        ""
      ],
      "Current account properties": [
        ""
      ],
      "No active decision found": [
        ""
      ],
      "There is no decision for this account yet. The account is limited by the default rules.": [
        ""
      ],
      "Previous AML decisions": [
        ""
      ],
      "No aml history found": [
        ""
      ],
      "Current active measures": [
        ""
      ],
      "Now": [
        ""
      ],
      "Public key": [
        ""
      ],
      "Request account activation": [
        ""
      ],
      "Lock account": [
        ""
      ],
      "Operation denied": [
        ""
      ],
      "This account signature is invalid, contact administrator or create a new one.": [
        ""
      ],
      "The designated AML account is not known, contact administrator or create a new one.": [
        ""
      ],
      "The designated AML account is not enabled, contact administrator or create a new one.": [
        ""
      ],
      "High risk accounts. Only with custom rules.": [
        ""
      ],
      "Account with custom rules.": [
        ""
      ],
      "High risk accounts": [
        ""
      ],
      "All account known.": [
        ""
      ],
      "High risk accounts under investigation. Only with custom rules.": [
        ""
      ],
      "Accounts under investigation. Only with custom rules.": [
        ""
      ],
      "High risk accounts under investigation.": [
        ""
      ],
      "Accounts under investigation.": [
        ""
      ],
      "High risk accounts without investigation. Only with custom rules.": [
        ""
      ],
      "Accounts without investigation. Only with custom rules.": [
        ""
      ],
      "High risk accounts without investigation.": [
        ""
      ],
      "Accounts without investigation.": [
        ""
      ],
      "Accounts": [
        ""
      ],
      "Export as file": [
        ""
      ],
      "Export completed": [
        ""
      ],
      "Click here to save the file in your computer.": [
        ""
      ],
      "No results": [
        ""
      ],
      "#": [
        ""
      ],
      "Account Identification": [
        ""
      ],
      "Open time": [
        ""
      ],
      "Status": [
        ""
      ],
      "From %1$s%2$sTo %3$s": [
        ""
      ],
      "Since %1$s": [
        ""
      ],
      "Not opened": [
        ""
      ],
      "High risk": [
        ""
      ],
      "With custom rules": [
        ""
      ],
      "Invalid account": [
        ""
      ],
      "Search by ID": [
        ""
      ],
      "Under investigation: ": [
        ""
      ],
      "High risk:": [
        ""
      ],
      "Open:": [
        ""
      ],
      "Dashboard": [
        ""
      ],
      "Statistics": [
        ""
      ],
      "Accounts open": [
        ""
      ],
      "New files last year": [
        ""
      ],
      "Closes files last year": [
        ""
      ],
      "MROS substantaiated": [
        ""
      ],
      "MROS simple": [
        ""
      ],
      "High risk files": [
        ""
      ],
      "PEP files": [
        ""
      ],
      "Under investiation": [
        ""
      ],
      "Number of accounts that are opened": [
        ""
      ],
      "Number of new GwG files in the last year": [
        ""
      ],
      "Number of GwG files closed in the last year": [
        ""
      ],
      "Number of MROS reports based on Art 9 Abs. 1 GwG (per year)": [
        ""
      ],
      "Number of MROS reports based on Art 305ter Abs. 2 StGB (per year)": [
        ""
      ],
      "Number of GwG files of high-risk customers": [
        ""
      ],
      "Number of GwG files managed with “increased risk” due to PEP status": [
        ""
      ],
      "Number of customers involved in proceedings for which Art 6 GwG did apply": [
        ""
      ],
      "previous": [
        ""
      ],
      "Decreased by": [
        ""
      ],
      "Increased by": [
        ""
      ],
      "Events": [
        ""
      ],
      "The events will be triggered only after the decision has been confirmed.": [
        ""
      ],
      "Calculated": [
        ""
      ],
      "Events based on properties and new attributes that should be triggered": [
        ""
      ],
      "Others": [
        ""
      ],
      "Events that can be triggered manually": [
        ""
      ],
      "Custom list": [
        ""
      ],
      "Events without built-in support.": [
        ""
      ],
      "Complete the form": [
        ""
      ],
      "These measures will ask the customer for information.": [
        ""
      ],
      "change form": [
        ""
      ],
      "Expiration date of the information filled in this form.": [
        ""
      ],
      "Include new information": [
        ""
      ],
      "Select a form to submit new information about the customer or continue to the next step": [
        ""
      ],
      "Form:": [
        ""
      ],
      "Justification": [
        ""
      ],
      "Keep investigation?": [
        ""
      ],
      "Account holder": [
        ""
      ],
      "Full name of the account holder": [
        ""
      ],
      "loading...": [
        ""
      ],
      "Update": [
        ""
      ],
      "Remove": [
        ""
      ],
      "Configure verification type: %1$s": [
        ""
      ],
      "Show complete form": [
        ""
      ],
      "Configure measure": [
        ""
      ],
      "Show as verification": [
        ""
      ],
      "There is already a measure with that name": [
        ""
      ],
      "Only required when no check is specified": [
        ""
      ],
      "Missing check or program": [
        ""
      ],
      "Without a check the program will run automatically": [
        ""
      ],
      "Field name": [
        ""
      ],
      "Type": [
        ""
      ],
      "string": [
        ""
      ],
      "number": [
        ""
      ],
      "boolean": [
        ""
      ],
      "json": [
        ""
      ],
      "Value": [
        ""
      ],
      "There are unsatisfied inputs: %1$s": [
        ""
      ],
      "There are missing inputs: %1$s": [
        ""
      ],
      "There are missing requirements: %1$s": [
        ""
      ],
      "Can't be empty": [
        ""
      ],
      "It should be a number": [
        ""
      ],
      "Not a number": [
        ""
      ],
      "It should be finite": [
        ""
      ],
      "It should be a safe integer": [
        ""
      ],
      "It should be either \"true\" or \"false\"": [
        ""
      ],
      "Couldn't parse as json string: %1$s": [
        ""
      ],
      "Description": [
        ""
      ],
      "Inputs": [
        ""
      ],
      "Output": [
        ""
      ],
      "Requires": [
        ""
      ],
      "Fallback": [
        ""
      ],
      "Name of the verification measure": [
        ""
      ],
      "Read only": [
        ""
      ],
      "Prevent the customer of changing the address": [
        ""
      ],
      "Configure verification measure: \"%1$s\"": [
        ""
      ],
      "Add custom measure": [
        ""
      ],
      "Custom measures": [
        ""
      ],
      "Server measures": [
        ""
      ],
      "New measures": [
        ""
      ],
      "Measures that are available/executed directly after the decision is confirmed.": [
        ""
      ],
      "Reset to default": [
        ""
      ],
      "Properties": [
        ""
      ],
      "Custom properties": [
        ""
      ],
      "Add more properties that not listed above.": [
        ""
      ],
      "New properties": [
        ""
      ],
      "Current note on the GWG File": [
        ""
      ],
      "Customer name or internal alias.": [
        ""
      ],
      "Is the account active for deposit and payments?": [
        ""
      ],
      "Is the account idle?": [
        ""
      ],
      "No operation for a period of time": [
        ""
      ],
      "Does account belong to a domestic PEP?": [
        ""
      ],
      "Does account belong to a foreign PEP?": [
        ""
      ],
      "Does account belong to a international organization PEP?": [
        ""
      ],
      "Does account belong to a high risk customer?": [
        ""
      ],
      "Does account belong to a person from a high risk country?": [
        ""
      ],
      "The MROS reporting state for the account.": [
        ""
      ],
      "None": [
        ""
      ],
      "Pending investigation": [
        ""
      ],
      "Investigated without suspicion": [
        ""
      ],
      "Reported simple suspicion": [
        ""
      ],
      "Reported substantaited suspicion": [
        ""
      ],
      "Informal reason why the AML investigation was triggered": [
        ""
      ],
      "Identifies the sanction list entry that the account matched against": [
        ""
      ],
      "Best match, does not mean it was a good match.": [
        ""
      ],
      "Score for how good the sanction list match was": [
        ""
      ],
      "Score for how much supporting data we had for the sanction list match": [
        ""
      ],
      "Suppress flagging this account when it creates a hit on a sanctions list, this is a false-positive": [
        ""
      ],
      "GLS Customer ID": [
        ""
      ],
      "Wrong credentials for \"%1$s\"": [
        ""
      ],
      "The account was not found": [
        ""
      ],
      "Officer disabled or more recent decision was already submitted.": [
        ""
      ],
      "Confirmation required": [
        ""
      ],
      "This decision declares that the MROS report have concluded. Please make that this is the case": [
        ""
      ],
      "I want to check first!": [
        ""
      ],
      "Confirm decision": [
        ""
      ],
      "Missing deadline": [
        ""
      ],
      "Deadline should specify when this rules ends and what is the next measures to apply after expiration.": [
        ""
      ],
      "Missing rules": [
        ""
      ],
      "Can't make a decision without rules.": [
        ""
      ],
      "Missing active measure": [
        ""
      ],
      "You should specify in the measure section.": [
        ""
      ],
      "No customer action required.": [
        ""
      ],
      "No active measure has been selected.": [
        ""
      ],
      "Missing properties": [
        ""
      ],
      "You should specify in the properties section.": [
        ""
      ],
      "Missing events": [
        ""
      ],
      "You should specify in the events section.": [
        ""
      ],
      "Missing justification": [
        ""
      ],
      "You must complete the justification section.": [
        ""
      ],
      "Invalid attributes": [
        ""
      ],
      "You should check form errors or submit a decision without attributes.": [
        ""
      ],
      "Clear": [
        ""
      ],
      "Send decision": [
        ""
      ],
      "Prev": [
        ""
      ],
      "Next": [
        ""
      ],
      "Attributes": [
        ""
      ],
      "Add more information about the customer": [
        ""
      ],
      "Rules": [
        ""
      ],
      "Set the limit of the operations": [
        ""
      ],
      "Trigger notifications.": [
        ""
      ],
      "Measures": [
        ""
      ],
      "Ask the customer to take action.": [
        ""
      ],
      "Describe the decision.": [
        ""
      ],
      "Flag the current account state.": [
        ""
      ],
      "Summary": [
        ""
      ],
      "Review and submit.": [
        ""
      ],
      "Decision for account:": [
        ""
      ],
      "Decision for wallet:": [
        ""
      ],
      "Decision for bank account:": [
        ""
      ],
      "Search account": [
        ""
      ],
      "Check account details": [
        ""
      ],
      "Account most recent decisions": [
        ""
      ],
      "When": [
        ""
      ],
      "LATEST": [
        ""
      ],
      "ACTIVE": [
        ""
      ],
      "Account not found": [
        ""
      ],
      "There is no history known for this account yet.": [
        ""
      ],
      "You can make a decision for this account anyway.": [
        ""
      ],
      "Search": [
        ""
      ],
      "IBAN": [
        ""
      ],
      "Taler Bank": [
        ""
      ],
      "Wallet": [
        ""
      ],
      "Generic Payto:// URI": [
        ""
      ],
      "Account type": [
        ""
      ],
      "Should be 52 characters.": [
        ""
      ],
      "Failed to parse: %1$s": [
        ""
      ],
      "Address contains invalid characters.": [
        ""
      ],
      "Can't mix uppercased and lowercased characters.": [
        ""
      ],
      "The addr checksum doesn't match.": [
        ""
      ],
      "Missing separator or address type.": [
        ""
      ],
      "Address should have less than 90 characters.": [
        ""
      ],
      "Address should have more than 6 characters.": [
        ""
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "The string should start with payto://": [
        ""
      ],
      "After the target type it should follow a '/' with more information.": [
        ""
      ],
      "The target type is not supported, only x-taler-bank, taler-reserve or iban": [
        ""
      ],
      "IBAN is missing": [
        ""
      ],
      "Bitcoin address is missing": [
        ""
      ],
      "Bank host and account is missing": [
        ""
      ],
      "Exchange host and account is missing": [
        ""
      ],
      "Ethereum address is missing": [
        ""
      ],
      "Invalid IBAN: %1$s": [
        ""
      ],
      "Invalid BTC: %1$s": [
        ""
      ],
      "Invalid reserve: %1$s": [
        ""
      ],
      "Invalid host": [
        ""
      ],
      "Invalid address": [
        ""
      ],
      "Payto URI": [
        ""
      ],
      "As defined by RFC 8905": [
        ""
      ],
      "payto://": [
        ""
      ],
      "International Bank Account Number": [
        ""
      ],
      "DE1231231231": [
        ""
      ],
      "Host": [
        ""
      ],
      "Exchange hostname": [
        ""
      ],
      "exchange.taler.net": [
        ""
      ],
      "Remove last '/'": [
        ""
      ],
      "Invalid value": [
        ""
      ],
      "ID": [
        ""
      ],
      "Wallet reserve public key": [
        ""
      ],
      "abcdef1235": [
        ""
      ],
      "Should be 52 characters": [
        ""
      ],
      "Account": [
        ""
      ],
      "Bank account identification": [
        ""
      ],
      "DE123123123": [
        ""
      ],
      "Hostname": [
        ""
      ],
      "Without the scheme, may include subpath: bank.com, bank.com/path/": [
        ""
      ],
      "bank.demo.taler.net": [
        ""
      ],
      "Invalid hostname": [
        ""
      ],
      "Different form version": [
        ""
      ],
      "The collected information don't have the form version used.": [
        ""
      ],
      "The collected information used the form version \"%1$s \" but is not available.": [
        ""
      ],
      "Case details": [
        ""
      ],
      "Next event": [
        ""
      ],
      "Previous event": [
        ""
      ],
      "Direction": [
        ""
      ],
      "Credit": [
        ""
      ],
      "Debit": [
        ""
      ],
      "KYC Auth": [
        ""
      ],
      "Minimum amount": [
        ""
      ],
      "All amounts below the given threshold will be filtered.": [
        ""
      ],
      "Select transaction type": [
        ""
      ],
      "Please select a transaction type to show transactions.": [
        ""
      ],
      "Transfers history for account": [
        "Charge Taler wallet"
      ],
      "Transfers history": [
        ""
      ],
      "No transfers yet.": [
        ""
      ],
      "There are no transfer for this account.": [
        ""
      ],
      "There are no transfer reported to the exchange with the current threshold.": [
        ""
      ],
      "Date": [
        ""
      ],
      "Amount": [
        ""
      ],
      "sent": [
        ""
      ],
      "received": [
        ""
      ],
      "to": [
        ""
      ],
      "from": [
        ""
      ],
      "Welcome to exchange config!": [
        "Welcome to euFin bank: Taler+IBAN now possible!"
      ],
      "Transfers": [
        ""
      ],
      "Profile": [
        ""
      ],
      "A list of all the pre-define measures in your that can used.": [
        ""
      ],
      "threshold\u0004%1$s every %2$s": [
        ""
      ],
      "operation type\u0004Withdraw": [
        "Confirm withdrawal"
      ],
      "operation type\u0004Deposit": [
        ""
      ],
      "operation type\u0004Merge": [
        ""
      ],
      "operation type\u0004Aggregate": [
        ""
      ],
      "operation type\u0004Balance": [
        ""
      ],
      "operation type\u0004Refund": [
        ""
      ],
      "operation type\u0004Close": [
        "Close"
      ],
      "operation type\u0004Transaction": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=(n != 1);",
  "lang": "en",
  "completeness": 100
};

strings['de'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "de"
      },
      "Show debug info": [
        ""
      ],
      "Use testing dialect": [
        ""
      ],
      "Allow Insecure password": [
        ""
      ],
      "Keep session after reload": [
        ""
      ],
      "Don't compress request": [
        ""
      ],
      "Internal error, please report.": [
        ""
      ],
      "Preferences": [
        ""
      ],
      "Password": [
        ""
      ],
      "Required": [
        ""
      ],
      "should have at least 12 characters": [
        ""
      ],
      "should have lowercase and uppercase characters": [
        ""
      ],
      "should have numbers": [
        ""
      ],
      "should have at least one character which is not a number or letter": [
        ""
      ],
      "Repeat password": [
        ""
      ],
      "doesn't match": [
        ""
      ],
      "Create account": [
        ""
      ],
      "Create": [
        ""
      ],
      "Couldn't unlock the account, the password may be wrong.": [
        ""
      ],
      "Account locked": [
        ""
      ],
      "Your account is normally locked anytime you reload. To unlock type your password again.": [
        ""
      ],
      "Unlock": [
        ""
      ],
      "Forget account": [
        ""
      ],
      "There are no rules for operations": [
        ""
      ],
      "This mean that all operation have no limit.": [
        ""
      ],
      "Add custom rule": [
        ""
      ],
      "Operation": [
        ""
      ],
      "Threshold": [
        ""
      ],
      "Escalation": [
        ""
      ],
      "%1$s": [
        ""
      ],
      "(all)": [
        ""
      ],
      "Since": [
        ""
      ],
      "Expired": [
        ""
      ],
      "Expires": [
        ""
      ],
      "Successor measure": [
        ""
      ],
      "AML officer justification": [
        ""
      ],
      "Default rules": [
        ""
      ],
      "Forms": [
        ""
      ],
      "Measures used to get gather information about the customer.": [
        ""
      ],
      "Name": [
        ""
      ],
      "Check": [
        ""
      ],
      "Program": [
        ""
      ],
      "Context": [
        ""
      ],
      "Modify": [
        ""
      ],
      "Info": [
        ""
      ],
      "Will show information to the user.": [
        ""
      ],
      "Instant measures": [
        ""
      ],
      "Triggered immediately without customer interaction.": [
        ""
      ],
      "Input requirement": [
        ""
      ],
      "Already completed.": [
        ""
      ],
      "Waiting to be completed.": [
        ""
      ],
      "Current active rules": [
        ""
      ],
      "There are no rules for this account yet.": [
        ""
      ],
      "New rule form": [
        ""
      ],
      "Add": [
        ""
      ],
      "Cancel": [
        ""
      ],
      "Basic plan": [
        ""
      ],
      "Premium": [
        ""
      ],
      "E-commerce": [
        ""
      ],
      "Point-of-sale": [
        ""
      ],
      "GLS Registered E-Commerce Merchant": [
        ""
      ],
      "Check rules": [
        ""
      ],
      "The account is a wallet and there are bank account operation's limits which may trigger actions that are not required: %1$s": [
        ""
      ],
      "The account isn't a wallet and there are wallet operation's limits which may trigger actions that are not required: %1$s": [
        ""
      ],
      "The account is a wallet and there are wallet operation's limits which are not limited: %1$s": [
        ""
      ],
      "The AML account is bank account, but not all account-related operations are limited (%1$s).": [
        ""
      ],
      "New rules": [
        ""
      ],
      "Reset rules to current state": [
        ""
      ],
      "Set rules to default": [
        ""
      ],
      "Freeze account": [
        ""
      ],
      "Behavior on rule expiration": [
        ""
      ],
      "Reset measure": [
        ""
      ],
      "Reset expiration": [
        ""
      ],
      "Operation type": [
        ""
      ],
      "Timeframe": [
        ""
      ],
      "Exposed": [
        ""
      ],
      "Is this limit communicated to the customer?": [
        ""
      ],
      "Esclation measure": [
        ""
      ],
      "Measures that the customer will need to satisfy to apply for a new threshold.": [
        ""
      ],
      "All measures": [
        ""
      ],
      "Hint the customer that all measure should be completed": [
        ""
      ],
      "Expiration": [
        ""
      ],
      "Predefined shortcuts": [
        ""
      ],
      "In a week": [
        ""
      ],
      "In a month": [
        ""
      ],
      "In a year": [
        ""
      ],
      "Never": [
        ""
      ],
      "Expiration date": [
        ""
      ],
      "Until when are the new rules active.": [
        ""
      ],
      "Measure taken automatically upon expiration of the current decision.": [
        ""
      ],
      "New decision": [
        ""
      ],
      "Show transfers": [
        ""
      ],
      "Case history for selected account": [
        "Buchungen auf öffentlich sichtbaren Konten"
      ],
      "ID:": [
        ""
      ],
      "Payto:": [
        ""
      ],
      "Under investigation": [
        ""
      ],
      "This account requires a manual review and is waiting for a decision to be made.": [
        ""
      ],
      "Collected information": [
        ""
      ],
      "Every attribute collection event (via form upload or KYC provider).": [
        ""
      ],
      "The event list is empty": [
        ""
      ],
      "Current account properties": [
        ""
      ],
      "No active decision found": [
        ""
      ],
      "There is no decision for this account yet. The account is limited by the default rules.": [
        ""
      ],
      "Previous AML decisions": [
        ""
      ],
      "No aml history found": [
        ""
      ],
      "Current active measures": [
        ""
      ],
      "Now": [
        ""
      ],
      "Public key": [
        ""
      ],
      "Request account activation": [
        ""
      ],
      "Lock account": [
        ""
      ],
      "Operation denied": [
        ""
      ],
      "This account signature is invalid, contact administrator or create a new one.": [
        ""
      ],
      "The designated AML account is not known, contact administrator or create a new one.": [
        ""
      ],
      "The designated AML account is not enabled, contact administrator or create a new one.": [
        ""
      ],
      "High risk accounts. Only with custom rules.": [
        ""
      ],
      "Account with custom rules.": [
        ""
      ],
      "High risk accounts": [
        ""
      ],
      "All account known.": [
        ""
      ],
      "High risk accounts under investigation. Only with custom rules.": [
        ""
      ],
      "Accounts under investigation. Only with custom rules.": [
        ""
      ],
      "High risk accounts under investigation.": [
        ""
      ],
      "Accounts under investigation.": [
        ""
      ],
      "High risk accounts without investigation. Only with custom rules.": [
        ""
      ],
      "Accounts without investigation. Only with custom rules.": [
        ""
      ],
      "High risk accounts without investigation.": [
        ""
      ],
      "Accounts without investigation.": [
        ""
      ],
      "Accounts": [
        "Betrag"
      ],
      "Export as file": [
        ""
      ],
      "Export completed": [
        ""
      ],
      "Click here to save the file in your computer.": [
        ""
      ],
      "No results": [
        ""
      ],
      "#": [
        ""
      ],
      "Account Identification": [
        ""
      ],
      "Open time": [
        ""
      ],
      "Status": [
        ""
      ],
      "From %1$s%2$sTo %3$s": [
        ""
      ],
      "Since %1$s": [
        ""
      ],
      "Not opened": [
        ""
      ],
      "High risk": [
        ""
      ],
      "With custom rules": [
        ""
      ],
      "Invalid account": [
        ""
      ],
      "Search by ID": [
        ""
      ],
      "Under investigation: ": [
        ""
      ],
      "High risk:": [
        ""
      ],
      "Open:": [
        ""
      ],
      "Dashboard": [
        ""
      ],
      "Statistics": [
        ""
      ],
      "Accounts open": [
        ""
      ],
      "New files last year": [
        ""
      ],
      "Closes files last year": [
        ""
      ],
      "MROS substantaiated": [
        ""
      ],
      "MROS simple": [
        ""
      ],
      "High risk files": [
        ""
      ],
      "PEP files": [
        ""
      ],
      "Under investiation": [
        ""
      ],
      "Number of accounts that are opened": [
        ""
      ],
      "Number of new GwG files in the last year": [
        ""
      ],
      "Number of GwG files closed in the last year": [
        ""
      ],
      "Number of MROS reports based on Art 9 Abs. 1 GwG (per year)": [
        ""
      ],
      "Number of MROS reports based on Art 305ter Abs. 2 StGB (per year)": [
        ""
      ],
      "Number of GwG files of high-risk customers": [
        ""
      ],
      "Number of GwG files managed with “increased risk” due to PEP status": [
        ""
      ],
      "Number of customers involved in proceedings for which Art 6 GwG did apply": [
        ""
      ],
      "previous": [
        ""
      ],
      "Decreased by": [
        ""
      ],
      "Increased by": [
        ""
      ],
      "Events": [
        ""
      ],
      "The events will be triggered only after the decision has been confirmed.": [
        ""
      ],
      "Calculated": [
        ""
      ],
      "Events based on properties and new attributes that should be triggered": [
        ""
      ],
      "Others": [
        ""
      ],
      "Events that can be triggered manually": [
        ""
      ],
      "Custom list": [
        ""
      ],
      "Events without built-in support.": [
        ""
      ],
      "Complete the form": [
        ""
      ],
      "These measures will ask the customer for information.": [
        ""
      ],
      "change form": [
        ""
      ],
      "Expiration date of the information filled in this form.": [
        ""
      ],
      "Include new information": [
        ""
      ],
      "Select a form to submit new information about the customer or continue to the next step": [
        ""
      ],
      "Form:": [
        ""
      ],
      "Justification": [
        ""
      ],
      "Keep investigation?": [
        ""
      ],
      "Account holder": [
        ""
      ],
      "Full name of the account holder": [
        ""
      ],
      "loading...": [
        ""
      ],
      "Update": [
        ""
      ],
      "Remove": [
        ""
      ],
      "Configure verification type: %1$s": [
        ""
      ],
      "Show complete form": [
        ""
      ],
      "Configure measure": [
        ""
      ],
      "Show as verification": [
        ""
      ],
      "There is already a measure with that name": [
        ""
      ],
      "Only required when no check is specified": [
        ""
      ],
      "Missing check or program": [
        ""
      ],
      "Without a check the program will run automatically": [
        ""
      ],
      "Field name": [
        ""
      ],
      "Type": [
        ""
      ],
      "string": [
        ""
      ],
      "number": [
        ""
      ],
      "boolean": [
        ""
      ],
      "json": [
        ""
      ],
      "Value": [
        ""
      ],
      "There are unsatisfied inputs: %1$s": [
        ""
      ],
      "There are missing inputs: %1$s": [
        ""
      ],
      "There are missing requirements: %1$s": [
        ""
      ],
      "Can't be empty": [
        ""
      ],
      "It should be a number": [
        ""
      ],
      "Not a number": [
        ""
      ],
      "It should be finite": [
        ""
      ],
      "It should be a safe integer": [
        ""
      ],
      "It should be either \"true\" or \"false\"": [
        ""
      ],
      "Couldn't parse as json string: %1$s": [
        ""
      ],
      "Description": [
        ""
      ],
      "Inputs": [
        ""
      ],
      "Output": [
        ""
      ],
      "Requires": [
        ""
      ],
      "Fallback": [
        ""
      ],
      "Name of the verification measure": [
        ""
      ],
      "Read only": [
        ""
      ],
      "Prevent the customer of changing the address": [
        ""
      ],
      "Configure verification measure: \"%1$s\"": [
        ""
      ],
      "Add custom measure": [
        ""
      ],
      "Custom measures": [
        ""
      ],
      "Server measures": [
        ""
      ],
      "New measures": [
        ""
      ],
      "Measures that are available/executed directly after the decision is confirmed.": [
        ""
      ],
      "Reset to default": [
        ""
      ],
      "Properties": [
        ""
      ],
      "Custom properties": [
        ""
      ],
      "Add more properties that not listed above.": [
        ""
      ],
      "New properties": [
        ""
      ],
      "Current note on the GWG File": [
        ""
      ],
      "Customer name or internal alias.": [
        ""
      ],
      "Is the account active for deposit and payments?": [
        ""
      ],
      "Is the account idle?": [
        ""
      ],
      "No operation for a period of time": [
        ""
      ],
      "Does account belong to a domestic PEP?": [
        ""
      ],
      "Does account belong to a foreign PEP?": [
        ""
      ],
      "Does account belong to a international organization PEP?": [
        ""
      ],
      "Does account belong to a high risk customer?": [
        ""
      ],
      "Does account belong to a person from a high risk country?": [
        ""
      ],
      "The MROS reporting state for the account.": [
        ""
      ],
      "None": [
        ""
      ],
      "Pending investigation": [
        ""
      ],
      "Investigated without suspicion": [
        ""
      ],
      "Reported simple suspicion": [
        ""
      ],
      "Reported substantaited suspicion": [
        ""
      ],
      "Informal reason why the AML investigation was triggered": [
        ""
      ],
      "Identifies the sanction list entry that the account matched against": [
        ""
      ],
      "Best match, does not mean it was a good match.": [
        ""
      ],
      "Score for how good the sanction list match was": [
        ""
      ],
      "Score for how much supporting data we had for the sanction list match": [
        ""
      ],
      "Suppress flagging this account when it creates a hit on a sanctions list, this is a false-positive": [
        ""
      ],
      "GLS Customer ID": [
        ""
      ],
      "Wrong credentials for \"%1$s\"": [
        ""
      ],
      "The account was not found": [
        ""
      ],
      "Officer disabled or more recent decision was already submitted.": [
        ""
      ],
      "Confirmation required": [
        ""
      ],
      "This decision declares that the MROS report have concluded. Please make that this is the case": [
        ""
      ],
      "I want to check first!": [
        ""
      ],
      "Confirm decision": [
        "Bestätigen"
      ],
      "Missing deadline": [
        ""
      ],
      "Deadline should specify when this rules ends and what is the next measures to apply after expiration.": [
        ""
      ],
      "Missing rules": [
        ""
      ],
      "Can't make a decision without rules.": [
        ""
      ],
      "Missing active measure": [
        ""
      ],
      "You should specify in the measure section.": [
        ""
      ],
      "No customer action required.": [
        ""
      ],
      "No active measure has been selected.": [
        ""
      ],
      "Missing properties": [
        ""
      ],
      "You should specify in the properties section.": [
        ""
      ],
      "Missing events": [
        ""
      ],
      "You should specify in the events section.": [
        ""
      ],
      "Missing justification": [
        ""
      ],
      "You must complete the justification section.": [
        ""
      ],
      "Invalid attributes": [
        ""
      ],
      "You should check form errors or submit a decision without attributes.": [
        ""
      ],
      "Clear": [
        ""
      ],
      "Send decision": [
        ""
      ],
      "Prev": [
        ""
      ],
      "Next": [
        ""
      ],
      "Attributes": [
        ""
      ],
      "Add more information about the customer": [
        ""
      ],
      "Rules": [
        ""
      ],
      "Set the limit of the operations": [
        ""
      ],
      "Trigger notifications.": [
        ""
      ],
      "Measures": [
        ""
      ],
      "Ask the customer to take action.": [
        ""
      ],
      "Describe the decision.": [
        ""
      ],
      "Flag the current account state.": [
        ""
      ],
      "Summary": [
        ""
      ],
      "Review and submit.": [
        ""
      ],
      "Decision for account:": [
        ""
      ],
      "Decision for wallet:": [
        ""
      ],
      "Decision for bank account:": [
        ""
      ],
      "Search account": [
        ""
      ],
      "Check account details": [
        ""
      ],
      "Account most recent decisions": [
        ""
      ],
      "When": [
        ""
      ],
      "LATEST": [
        ""
      ],
      "ACTIVE": [
        ""
      ],
      "Account not found": [
        ""
      ],
      "There is no history known for this account yet.": [
        ""
      ],
      "You can make a decision for this account anyway.": [
        ""
      ],
      "Search": [
        ""
      ],
      "IBAN": [
        ""
      ],
      "Taler Bank": [
        "Taler-Logo"
      ],
      "Wallet": [
        ""
      ],
      "Generic Payto:// URI": [
        ""
      ],
      "Account type": [
        ""
      ],
      "Should be 52 characters.": [
        ""
      ],
      "Failed to parse: %1$s": [
        ""
      ],
      "Address contains invalid characters.": [
        ""
      ],
      "Can't mix uppercased and lowercased characters.": [
        ""
      ],
      "The addr checksum doesn't match.": [
        ""
      ],
      "Missing separator or address type.": [
        ""
      ],
      "Address should have less than 90 characters.": [
        ""
      ],
      "Address should have more than 6 characters.": [
        ""
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "The string should start with payto://": [
        ""
      ],
      "After the target type it should follow a '/' with more information.": [
        ""
      ],
      "The target type is not supported, only x-taler-bank, taler-reserve or iban": [
        ""
      ],
      "IBAN is missing": [
        ""
      ],
      "Bitcoin address is missing": [
        ""
      ],
      "Bank host and account is missing": [
        ""
      ],
      "Exchange host and account is missing": [
        ""
      ],
      "Ethereum address is missing": [
        ""
      ],
      "Invalid IBAN: %1$s": [
        ""
      ],
      "Invalid BTC: %1$s": [
        ""
      ],
      "Invalid reserve: %1$s": [
        ""
      ],
      "Invalid host": [
        ""
      ],
      "Invalid address": [
        "payto-Adresse"
      ],
      "Payto URI": [
        ""
      ],
      "As defined by RFC 8905": [
        ""
      ],
      "payto://": [
        ""
      ],
      "International Bank Account Number": [
        ""
      ],
      "DE1231231231": [
        ""
      ],
      "Host": [
        ""
      ],
      "Exchange hostname": [
        ""
      ],
      "exchange.taler.net": [
        ""
      ],
      "Remove last '/'": [
        ""
      ],
      "Invalid value": [
        ""
      ],
      "ID": [
        ""
      ],
      "Wallet reserve public key": [
        ""
      ],
      "abcdef1235": [
        ""
      ],
      "Should be 52 characters": [
        ""
      ],
      "Account": [
        "Betrag"
      ],
      "Bank account identification": [
        "Kontostand"
      ],
      "DE123123123": [
        ""
      ],
      "Hostname": [
        ""
      ],
      "Without the scheme, may include subpath: bank.com, bank.com/path/": [
        ""
      ],
      "bank.demo.taler.net": [
        ""
      ],
      "Invalid hostname": [
        ""
      ],
      "Different form version": [
        ""
      ],
      "The collected information don't have the form version used.": [
        ""
      ],
      "The collected information used the form version \"%1$s \" but is not available.": [
        ""
      ],
      "Case details": [
        ""
      ],
      "Next event": [
        ""
      ],
      "Previous event": [
        ""
      ],
      "Direction": [
        ""
      ],
      "Credit": [
        ""
      ],
      "Debit": [
        ""
      ],
      "KYC Auth": [
        ""
      ],
      "Minimum amount": [
        ""
      ],
      "All amounts below the given threshold will be filtered.": [
        ""
      ],
      "Select transaction type": [
        ""
      ],
      "Please select a transaction type to show transactions.": [
        ""
      ],
      "Transfers history for account": [
        ""
      ],
      "Transfers history": [
        ""
      ],
      "No transfers yet.": [
        ""
      ],
      "There are no transfer for this account.": [
        ""
      ],
      "There are no transfer reported to the exchange with the current threshold.": [
        ""
      ],
      "Date": [
        "Datum"
      ],
      "Amount": [
        "Betrag"
      ],
      "sent": [
        ""
      ],
      "received": [
        ""
      ],
      "to": [
        ""
      ],
      "from": [
        ""
      ],
      "Welcome to exchange config!": [
        ""
      ],
      "Transfers": [
        ""
      ],
      "Profile": [
        ""
      ],
      "A list of all the pre-define measures in your that can used.": [
        ""
      ],
      "threshold\u0004%1$s every %2$s": [
        ""
      ],
      "operation type\u0004Withdraw": [
        "Abhebung bestätigen"
      ],
      "operation type\u0004Deposit": [
        ""
      ],
      "operation type\u0004Merge": [
        ""
      ],
      "operation type\u0004Aggregate": [
        ""
      ],
      "operation type\u0004Balance": [
        ""
      ],
      "operation type\u0004Refund": [
        ""
      ],
      "operation type\u0004Close": [
        ""
      ],
      "operation type\u0004Transaction": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "de",
  "completeness": 2
};

