'use strict';

import warnAboutDeprecatedESMImport from './warnAboutDeprecatedESMImport.js';
warnAboutDeprecatedESMImport('PathUtils');

import { PathUtils } from '../esm/history.js';
export default PathUtils;
