/*
 This file is part of GNU Taler
 (C) 2022-2025 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */
import {
  OfficerAccount,
  PaytoString,
  TalerExchangeResultByMethod2,
  TalerHttpError,
} from "@gnu-taler/taler-util";
// FIX default import https://github.com/microsoft/TypeScript/issues/49189
import { useExchangeApiContext } from "@gnu-taler/web-util/browser";
import _useSWR, { mutate, SWRHook } from "swr";
import { useOfficer } from "./officer.js";
const useSWR = _useSWR as unknown as SWRHook;

export function revalidateAccountInformation() {
  return mutate(
    (key) =>
      Array.isArray(key) &&
      key[key.length - 1] === "getAmlAttributesForAccount",
    undefined,
    { revalidate: true },
  );
}
export function useAccountInformation(paytoHash?: string) {
  const officer = useOfficer();
  const session =
    officer.state === "ready" && paytoHash !== undefined
      ? officer.account
      : undefined;

  const {
    lib: { exchange: api },
  } = useExchangeApiContext();

  async function fetcher([officer, account]: [OfficerAccount, PaytoString]) {
    return await api.getAmlAttributesForAccount(officer, account);
  }

  const { data, error } = useSWR<
    TalerExchangeResultByMethod2<"getAmlAttributesForAccount">,
    TalerHttpError
  >(!session ? undefined : [session, paytoHash], fetcher, {
    refreshInterval: 0,
    refreshWhenHidden: false,
    revalidateOnFocus: false,
    revalidateOnReconnect: false,
    refreshWhenOffline: false,
    errorRetryCount: 0,
    errorRetryInterval: 1,
    shouldRetryOnError: false,
    keepPreviousData: true,
  });

  if (data) return data;
  if (error) return error;
  return undefined;
}
