/*
 This file is part of GNU Taler
 (C) 2021-2024 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

/*eslint quote-props: ["error", "consistent"]*/
export const strings: {[s: string]: any} = {};

strings['en'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=(n != 1);",
        "lang": "en"
      },
      "Key figures": [
        ""
      ],
      "Critical errors": [
        ""
      ],
      "Operating status": [
        ""
      ],
      "Inconsistencies": [
        ""
      ],
      "Settings": [
        ""
      ],
      "Cancel": [
        ""
      ],
      "%1$s": [
        ""
      ],
      "Close": [
        ""
      ],
      "suppress": [
        ""
      ],
      "Suppress": [
        ""
      ],
      "There are no entries yet": [
        ""
      ],
      "Entity row  with id: %1$s has been suppressed": [
        ""
      ],
      "Failed to suppress row": [
        ""
      ],
      "Language": [
        ""
      ],
      "generate random secret key": [
        ""
      ],
      "Set default": [
        ""
      ],
      "Token required": [
        ""
      ],
      "Need the access token for the API.": [
        ""
      ],
      "Access Token": [
        ""
      ],
      "Confirm": [
        ""
      ],
      "Login required": [
        ""
      ],
      "Please enter your access token.": [
        ""
      ],
      "Checking the /config endpoint got authorization error": [
        ""
      ],
      "Could not find /config endpoint on this URL": [
        ""
      ],
      "Server response with an error code": [
        ""
      ],
      "Got message \"%1$s\" from %2$s": [
        ""
      ],
      "Response from server is unreadable, http status: %1$s": [
        ""
      ],
      "Unexpected Error": [
        ""
      ],
      "Incompatible version": [
        ""
      ],
      "Auditor backend server version %1$s is not compatible with the supported version %2$s": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=(n != 1);",
  "lang": "en",
  "completeness": 100
};

strings['de'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "de"
      },
      "Key figures": [
        ""
      ],
      "Critical errors": [
        ""
      ],
      "Operating status": [
        ""
      ],
      "Inconsistencies": [
        ""
      ],
      "Settings": [
        ""
      ],
      "Cancel": [
        ""
      ],
      "%1$s": [
        ""
      ],
      "Close": [
        ""
      ],
      "suppress": [
        ""
      ],
      "Suppress": [
        ""
      ],
      "There are no entries yet": [
        ""
      ],
      "Entity row  with id: %1$s has been suppressed": [
        ""
      ],
      "Failed to suppress row": [
        ""
      ],
      "Language": [
        ""
      ],
      "generate random secret key": [
        ""
      ],
      "Set default": [
        ""
      ],
      "Token required": [
        ""
      ],
      "Need the access token for the API.": [
        ""
      ],
      "Access Token": [
        ""
      ],
      "Confirm": [
        ""
      ],
      "Login required": [
        ""
      ],
      "Please enter your access token.": [
        ""
      ],
      "Checking the /config endpoint got authorization error": [
        ""
      ],
      "Could not find /config endpoint on this URL": [
        ""
      ],
      "Server response with an error code": [
        ""
      ],
      "Got message \"%1$s\" from %2$s": [
        ""
      ],
      "Response from server is unreadable, http status: %1$s": [
        ""
      ],
      "Unexpected Error": [
        ""
      ],
      "Incompatible version": [
        ""
      ],
      "Auditor backend server version %1$s is not compatible with the supported version %2$s": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "de",
  "completeness": 0
};

