#people-app {
	.avatar {
		display: inline-block;
		overflow: hidden;
		width: var(--avatar-size, 32px);
		height: var(--avatar-size, 32px);
		background-color: var(--avatar-color, var(--app-primary));
		border-radius: 50%;
		font-size: calc(var(--avatar-size, 32px) * 0.5);
		line-height: var(--avatar-size, 32px);
		object-fit: cover;
		text-align: center;
		text-transform: uppercase;
		white-space: nowrap;
	}
}
