export async function HashArgon2idImpl(
  password: Uint8Array,
  salt: Uint8Array,
  iterations: number,
  memorySize: number,
  hashLength: number,
): Promise<Uint8Array> {
  throw new Error("Method not implemented.");
}
