'use strict';

import warnAboutDeprecatedESMImport from './warnAboutDeprecatedESMImport.js';
warnAboutDeprecatedESMImport('DOMUtils');

import { DOMUtils } from '../esm/history.js';
export default DOMUtils;
