// flow-typed signature: 973c9500e4cab50d99a36b48129481e3
// flow-typed version: <<STUB>>/rollup-plugin-babel_v^4.0.3/flow_v0.82.0

/**
 * This is an autogenerated libdef stub for:
 *
 *   'rollup-plugin-babel'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module 'rollup-plugin-babel' {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module 'rollup-plugin-babel/dist/rollup-plugin-babel.cjs' {
  declare module.exports: any;
}

declare module 'rollup-plugin-babel/dist/rollup-plugin-babel.esm' {
  declare module.exports: any;
}

declare module 'rollup-plugin-babel/src/constants' {
  declare module.exports: any;
}

declare module 'rollup-plugin-babel/src/helperPlugin' {
  declare module.exports: any;
}

declare module 'rollup-plugin-babel/src/index' {
  declare module.exports: any;
}

declare module 'rollup-plugin-babel/src/preflightCheck' {
  declare module.exports: any;
}

declare module 'rollup-plugin-babel/src/utils' {
  declare module.exports: any;
}

// Filename aliases
declare module 'rollup-plugin-babel/dist/rollup-plugin-babel.cjs.js' {
  declare module.exports: $Exports<'rollup-plugin-babel/dist/rollup-plugin-babel.cjs'>;
}
declare module 'rollup-plugin-babel/dist/rollup-plugin-babel.esm.js' {
  declare module.exports: $Exports<'rollup-plugin-babel/dist/rollup-plugin-babel.esm'>;
}
declare module 'rollup-plugin-babel/src/constants.js' {
  declare module.exports: $Exports<'rollup-plugin-babel/src/constants'>;
}
declare module 'rollup-plugin-babel/src/helperPlugin.js' {
  declare module.exports: $Exports<'rollup-plugin-babel/src/helperPlugin'>;
}
declare module 'rollup-plugin-babel/src/index.js' {
  declare module.exports: $Exports<'rollup-plugin-babel/src/index'>;
}
declare module 'rollup-plugin-babel/src/preflightCheck.js' {
  declare module.exports: $Exports<'rollup-plugin-babel/src/preflightCheck'>;
}
declare module 'rollup-plugin-babel/src/utils.js' {
  declare module.exports: $Exports<'rollup-plugin-babel/src/utils'>;
}
