/*
 This file is part of GNU Taler
 (C) 2020-2025 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

/**
 * Imports.
 */
import { TalerErrorCode } from "@gnu-taler/taler-util";
import { WalletApiOperation } from "@gnu-taler/taler-wallet-core";
import { GlobalTestState } from "../harness/harness.js";
import { createSimpleTestkudosEnvironmentV3 } from "../harness/environments.js";

/**
 * Run test for basic, bank-integrated withdrawal.
 */
export async function runWithdrawalAbortBankTest(t: GlobalTestState) {
  // Set up test environment

  const { walletClient, bankClient, exchange } =
    await createSimpleTestkudosEnvironmentV3(t);

  // Create a withdrawal operation

  const user = await bankClient.createRandomBankUser();
  bankClient.setAuth(user);
  const wop = await bankClient.createWithdrawalOperation(
    user.username,
    "TESTKUDOS:10",
  );

  // Hand it to the wallet

  await walletClient.call(WalletApiOperation.GetWithdrawalDetailsForUri, {
    talerWithdrawUri: wop.taler_withdraw_uri,
  });

  await walletClient.call(WalletApiOperation.TestingWaitTransactionsFinal, {});

  // Abort it

  await bankClient.abortWithdrawalOperationV2(user.username, wop);

  // Withdraw

  // Difference:
  // -> with euFin, the wallet selects
  // -> with PyBank, the wallet stops _before_
  //
  // WHY ?!
  //
  const e = await t.assertThrowsTalerErrorAsyncLegacy(
    walletClient.call(WalletApiOperation.AcceptBankIntegratedWithdrawal, {
      exchangeBaseUrl: exchange.baseUrl,
      talerWithdrawUri: wop.taler_withdraw_uri,
    })
  );
  t.assertDeepEqual(
    e.errorDetail.code,
    TalerErrorCode.WALLET_WITHDRAWAL_OPERATION_ABORTED_BY_BANK,
  );

  await t.shutdown();
}

runWithdrawalAbortBankTest.suites = ["wallet"];
