/*
 This file is part of GNU Taler
 (C) 2022 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 GNU Taler; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

export const strings: {[s: string]: any} = {};
strings['uk'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=3; plural=n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2;",
        "lang": "uk"
      },
      "Balance": [
        "Баланс"
      ],
      "Backup": [
        "Резервна копія"
      ],
      "Notifications": [
        ""
      ],
      "QR Reader and Taler URI": [
        "Зчитувач QR та Taler URI"
      ],
      "Settings": [
        "Налаштування"
      ],
      "Dev tools": [
        "Інструменти відлагодження"
      ],
      "Contacts": [
        ""
      ],
      "Mailbox": [
        ""
      ],
      "Search transactions": [
        "Комісії за транзакцію"
      ],
      "%1$s": [
        "%1$s"
      ],
      "KYC requirement": [
        ""
      ],
      "PENDING OPERATIONS": [
        "ОЧІКУВАНІ ОПЕРАЦІЇ"
      ],
      "Error was thrown trying to: \"%1$s\"": [
        ""
      ],
      "Could not open the wallet database. Firefox is known to run into this problem under \"permanent private mode\".": [
        ""
      ],
      "Could not open the wallet database.": [
        "Не вдалося розібрати URI для оплати"
      ],
      "Unexpected request error, code: %1$s": [
        ""
      ],
      "more info": [
        ""
      ],
      "wrap text": [
        ""
      ],
      "copy content": [
        ""
      ],
      "less info": [
        ""
      ],
      "Retry operation": [
        "Операції гаманця"
      ],
      "Loading": [
        "Завантаження"
      ],
      "Could not load backup providers": [
        "Не вдалося завантажити зберігачів резервних копій"
      ],
      "No backup providers configured": [
        "Не налаштовано жодного зберігача резервних копій"
      ],
      "Add provider": [
        "Додати зберігача"
      ],
      "Sync all backups": [
        "Синхронізувати всі резервні копії"
      ],
      "Sync now": [
        "Синхронізувати зараз"
      ],
      "Last synced": [
        "Останній раз синхронізовано"
      ],
      "Not synced": [
        "Не синхронізовано"
      ],
      "Expires in": [
        "Термін дії закінчується в"
      ],
      "show less": [
        "показати qr"
      ],
      "show more": [
        "показати qr"
      ],
      "There was an error loading the provider detail for &quot;%1$s&quot;": [
        "Виникла помилка при завантаженні інформації зберігача &quot; %1$s&quot;"
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [
        "Зберігач з посиланням &quot;%1$s&quot; невідомий."
      ],
      "See providers": [
        "Подивитись зберігачів"
      ],
      "Last backup": [
        "Остання резервна копія"
      ],
      "Back up": [
        "Зробити резервну копію"
      ],
      "Provider fee": [
        "Комісія зберігача"
      ],
      "per year": [
        "на рік"
      ],
      "Extend": [
        "Подовжити"
      ],
      "terms has changed, extending the service will imply accepting the new terms of service": [
        "умови надання послуг змінились, продовження послуги означатиме прийняття нових умов"
      ],
      "old": [
        "старий"
      ],
      "new": [
        "новий"
      ],
      "fee": [
        "комісія"
      ],
      "storage": [
        "сховище"
      ],
      "Remove provider": [
        "Видалити зберігача"
      ],
      "Pay": [
        "Оплата"
      ],
      "Withdraw": [
        "Зняття коштів"
      ],
      "This provider has reported an error": [
        "Цей зберігач повідомив про помилку"
      ],
      "There is conflict with another backup from &quot;%1$s&quot;": [
        "Конфлікт з іншою резервною копією з %1$s"
      ],
      "Backup is not readable": [
        "Резервна копія пошкоджена або не може бути прочитана"
      ],
      "Unknown backup problem: %1$s": [
        "Невідома помилка резервного копіювання: %1$s"
      ],
      "service paid": [
        "послуга сплачена"
      ],
      "Backup valid until": [
        "Резервна копія дійсна до"
      ],
      "Cancel": [
        "Скасувати"
      ],
      "Open pay page": [
        "Відкрити сторінку оплати"
      ],
      "Open refund page": [
        "Відкрити сторінку повернень"
      ],
      "Open withdraw page": [
        "Відкрити сторінку зняття"
      ],
      "Your wallet is empty.": [
        ""
      ],
      "Get digital cash": [
        "Отримати е-готівку"
      ],
      "Try the demo bank and withdraw test money.": [
        ""
      ],
      "Could not load the balance": [
        "Не вдалося завантажити баланс депозиту"
      ],
      "Receive": [
        "Чек"
      ],
      "Send %1$s": [
        "Переказати %1$s"
      ],
      "This page has a withdrawal action.": [
        "Ця сторінка має дію зняття коштів."
      ],
      "This page has pay action.": [
        "Ця сторінка має дію оплати."
      ],
      "This page has a refund action.": [
        "Ця сторінка має дію повернення коштів."
      ],
      "This page has a add exchange action.": [
        "Ця сторінка має дію повернення коштів."
      ],
      "Open add exchange page": [
        "Додати обмінник"
      ],
      "This page has an add contact action.": [
        "Ця сторінка має дію повернення коштів."
      ],
      "Open add contact page": [
        "Відкрити сторінку оплати"
      ],
      "Taler Action": [
        "Taler Дія"
      ],
      "This page has a malformed taler uri.": [
        "Ця сторінка має некоректний URI Taler."
      ],
      "Dismiss": [
        "Закрити"
      ],
      "this popup is being closed and you are being redirected to %1$s": [
        "це спливаюче вікно закривається, і ви перенаправляєтеся на %1$s"
      ],
      "Could not load purchase proposal details": [
        "Не вдалося завантажити деталі пропозиції покупки"
      ],
      "Order Id": [
        "Номер замовлення"
      ],
      "Summary": [
        "Підсумок"
      ],
      "Amount": [
        "Сума"
      ],
      "Choices": [
        ""
      ],
      "This purchase have %1$s choices": [
        ""
      ],
      "Merchant name": [
        "Ім'я продавця"
      ],
      "Merchant jurisdiction": [
        "Юрисдикція продавця"
      ],
      "Merchant address": [
        "Адреса продавця"
      ],
      "Merchant logo": [
        "Логотип продавця"
      ],
      "Merchant website": [
        "Вебсайт продавця"
      ],
      "Merchant email": [
        "Електронна пошта продавця"
      ],
      "Merchant public key": [
        "Публічний ключ продавця"
      ],
      "Created at": [
        "Створено о"
      ],
      "Refund deadline": [
        "Термін повернення"
      ],
      "Auto refund": [
        "Автоматичне повернення"
      ],
      "Pay deadline": [
        "Термін оплати"
      ],
      "Fulfillment URL": [
        "URL виконання"
      ],
      "Fulfillment message": [
        "Повідомлення про виконання"
      ],
      "Max deposit fee": [
        "Максимальна комісія за депозит"
      ],
      "Exchanges": [
        "Обмінники"
      ],
      "Could not load the status of the term of service": [
        "Не вдалося завантажити список транзакцій"
      ],
      "Select one option": [
        "Виберіть один варіант"
      ],
      "Show terms of service": [
        "Показати умови отримання послуг"
      ],
      "I accept the exchange terms of service": [
        "Я приймаю Умови надання послуг, цього обмінника"
      ],
      "Review exchange terms of service": [
        "Переглянути умови надання послуг обмінника"
      ],
      "The exchange doesn't have a terms of service.": [
        "Обмінник не має умов надання послуг"
      ],
      "Skip it for now.": [
        ""
      ],
      "Format": [
        ""
      ],
      "Language": [
        "Поточна мова"
      ],
      "The exchange replied with a empty terms of service": [
        "Обмінник відповів порожніми умовами надання послуг"
      ],
      "No terms of service. The exchange replied with a empty document": [
        "Обмінник відповів порожніми умовами надання послуг"
      ],
      "No terms of service. The exchange replied with a empty text": [
        "Обмінник відповів порожніми умовами надання послуг"
      ],
      "Download Terms of Service": [
        "Завантажити Умови надання послуг"
      ],
      "Hide terms of service": [
        "Сховати Умови надання послуг"
      ],
      "Amount is not valid": [
        ""
      ],
      "Bank account": [
        "Банківський рахунок"
      ],
      "Bitcoin address": [
        "Адреса Bitcoin"
      ],
      "BIC/IBAN": [
        "IBAN"
      ],
      "IBAN": [
        "IBAN"
      ],
      "Bitcoin transfer details": [
        "Деталі переказу Bitcoin"
      ],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.": [
        "Для обміну потрібна транзакція з 3 виходами: один вихід - це рахунок обміну, а два інші - segwit вигадані адреси для метаданих з мінімальною сумою."
      ],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts": [
        "У гаманці bitcoincore використовуйте кнопку &apos;Додати отримувача&apos;, щоб додати двох додаткових отримувачів і скопіювати адреси та суми"
      ],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC": [
        "Переконайтеся, що сума показує %1$s BTC, інакше вам потрібно змінити базову одиницю на BTC"
      ],
      "Ethereum transfer details": [
        "Деталі банківського переказу"
      ],
      "You need to wire to the service provider account.": [
        ""
      ],
      "Make sure the amount show %1$s ETH": [
        ""
      ],
      "Bank transfer details": [
        "Деталі банківського переказу"
      ],
      "Account": [
        "Рахунок"
      ],
      "Bank host": [
        "Хост банку"
      ],
      "BIC": [
        ""
      ],
      "Exchange": [
        "Exchange"
      ],
      "Reserve Pub": [
        ""
      ],
      "Step 1:": [
        ""
      ],
      "Copy this code and paste it into the subject/purpose field in your banking app or bank website": [
        ""
      ],
      "Subject": [
        "Призначення"
      ],
      "Step 2:": [
        ""
      ],
      "If you don't already have it in your banking favourites list, then copy and paste this IBAN and the name into the receiver fields in your banking app or website": [
        ""
      ],
      "Receiver name": [
        "Ім'я отримувача"
      ],
      "Receiver postal code": [
        "Ім'я отримувача"
      ],
      "Receiver town": [
        "Ім'я отримувача"
      ],
      "Step 3:": [
        ""
      ],
      "Finish the wire transfer setting the amount in your banking app or website, then this withdrawal will proceed automatically.": [
        ""
      ],
      "Make sure ALL data is correct, including the subject; otherwise, the money will not arrive in this wallet. You can use the copy buttons (%1$s) to prevent typing errors or the \"payto://\" URI below to copy just one value.": [
        ""
      ],
      "Alternative if your bank already supports PayTo URI, you can use this %1$s link instead": [
        ""
      ],
      "Continue with banking app or website": [
        ""
      ],
      "Show QR code": [
        ""
      ],
      "Could not load transaction information": [
        "Не вдалося завантажити інформацію про транзакцію"
      ],
      "Bank account confirmation required for the transaction to complete.": [
        ""
      ],
      "KYC check required for the transaction to complete.": [
        ""
      ],
      "No additional information has been provided.": [
        ""
      ],
      "Follow this link to the%1$s %2$s": [
        ""
      ],
      "This transaction is pending to complete.": [
        "Ця транзакція не завершена"
      ],
      "Retry": [
        "Спробувати ще раз"
      ],
      "There was an error trying to complete the transaction.": [
        "Сталася помилка під час спроби завершити транзакцію"
      ],
      "This transaction was aborted.": [
        "Ця транзакція не завершена"
      ],
      "This transaction failed.": [
        "Ця транзакція не завершена"
      ],
      "Caution!": [
        "Увага!"
      ],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.": [
        "Якщо Ви вже переказали гроші на обмінник, Ви втратите можливість отримати з нього монети."
      ],
      "Confirm": [
        "Підтвердити"
      ],
      "Doing a cancellation while the transaction still active might result in lost coins. Do you still want to cancel the transaction?": [
        ""
      ],
      "No": [
        ""
      ],
      "Yes": [
        ""
      ],
      "Abort": [
        ""
      ],
      "Resume": [
        ""
      ],
      "Suspend": [
        ""
      ],
      "Delete": [
        ""
      ],
      "Withdrawal": [
        "Зняття"
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Select one of the accounts and use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Details": [
        "Деталі"
      ],
      "Withdrawal incomplete.": [
        "Зняття"
      ],
      "If you have already sent money to the service provider account it will wire it back at %1$s": [
        "Якщо Ви вже переказали гроші на обмінник, Ви втратите можливість отримати з нього монети."
      ],
      "Payment": [
        "Оплата"
      ],
      "Refunds": [
        "Повернення"
      ],
      "%1$s %2$s on %3$s .": [
        "%1$s %2$s на %3$s"
      ],
      "Refund is in progress.": [
        "Повернення коштів в процесі."
      ],
      "Merchant created a refund for this order but was not automatically picked up.": [
        "Продавець створив повернення для цього замовлення, але воно не було автоматично оброблене."
      ],
      "Offer": [
        "Пропозиція"
      ],
      "Accept": [
        "Прийняти"
      ],
      "Confirmation code": [
        ""
      ],
      "Merchant": [
        "Продавець"
      ],
      "Invoice ID": [
        "№ рахунку-фактури"
      ],
      "Deposit": [
        "Депозит"
      ],
      "Wire transfer deadline.": [
        "Деталі переказу Bitcoin"
      ],
      "Wire transfer still pending.": [
        ""
      ],
      "The service provider deadline to make the wire transfer is: %1$s": [
        ""
      ],
      "Wire transfer completed.": [
        ""
      ],
      " ": [
        ""
      ],
      "Transfer details": [
        "Деталі банківського переказу"
      ],
      "Wire transfer in progress.": [
        "Повернення коштів в процесі."
      ],
      "Refresh": [
        "Оновити"
      ],
      "Refund": [
        "Повернення"
      ],
      "Purchase summary": [
        "Інформація про покупку"
      ],
      "Credit": [
        "Кредит"
      ],
      "Invoice": [
        "Рахунок-фактура"
      ],
      "URI": [
        "URI"
      ],
      "Debit": [
        "Дебет"
      ],
      "Transfer": [
        "Переказати"
      ],
      "Lost": [
        ""
      ],
      "Reason": [
        ""
      ],
      "Denomination expired.": [
        "Номінал"
      ],
      "Denomination vanished.": [
        "Номінал"
      ],
      "Denomination is unoffered.": [
        "Номінал"
      ],
      "Fees": [
        ""
      ],
      "Total": [
        "Загальна сума"
      ],
      "Sent": [
        ""
      ],
      "Converted": [
        ""
      ],
      "Bank fee": [
        "Максимальна комісія"
      ],
      "Price": [
        "Ціна"
      ],
      "Subtotal": [
        "Загальна сума"
      ],
      "Refunded": [
        "Повернено"
      ],
      "Transfer identification": [
        "Операції переказів"
      ],
      "copy": [
        "копіювати"
      ],
      "hide qr": [
        "сховати qr"
      ],
      "forward to contact": [
        ""
      ],
      "show qr": [
        "показати qr"
      ],
      "Wire transfer need a confirmation. Go to the %1$s and check wire transfer operation to exchange account is complete.": [
        ""
      ],
      "bank site": [
        "Хост банку"
      ],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins.": [
        "Банк підтвердив банківський переказ. Очікуємо, що обмінник надішле монети"
      ],
      "Exchange is ready to send the coins, withdrawal in progress.": [
        "Обмінник готовий до зняття коштів"
      ],
      "Send &nbsp; %1$s": [
        "Надіслати &nbsp; %1$s"
      ],
      "Pay &nbsp; %1$s": [
        "Оплатити &nbsp; %1$s"
      ],
      "Merchant message": [
        "Повідомлення продавця"
      ],
      "Balance is not enough.": [
        "Ваш поточний баланс недостатній."
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which exchange and auditors the merchant trust.": [
        ""
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which wire methods the merchant accepts.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to pay for this contract which is restricted.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to spend right know. There are some coins that need to be refreshed.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s available.": [
        ""
      ],
      "Provider is missing the global fee configuration, this likely means it is misconfigured.": [
        ""
      ],
      "Balance looks like it should be enough, but doesn't cover all fees requested by the merchant and payment processor. Please ensure there is at least %1$s %2$s more balance in your wallet or ask your merchant to cover more of the fees.": [
        ""
      ],
      "Pay with a mobile phone": [
        "Оплатити мобільним телефоном"
      ],
      "Hide QR": [
        "Сховати QR"
      ],
      "Scan the QR code or &nbsp; %1$s": [
        "Скануйте QR-код або &nbsp; %1$s"
      ],
      "click here": [
        "натисніть, щоб показати"
      ],
      "Could not load the payment and balance status": [
        "Не вдалося завантажити статус оплати"
      ],
      "Purchase": [
        "Покупка"
      ],
      "This purchase is age restricted.": [
        ""
      ],
      "Expires at": [
        "Термін дії закінчується в"
      ],
      "Already paid, you are going to be redirected to %1$s": [
        "Вже оплачено, Вас буде перенаправлено на %1$s"
      ],
      "Already paid": [
        "Вже оплачено"
      ],
      "Already claimed": [
        "Вже отримано"
      ],
      "Could not load the refund status": [
        "Не вдалося завантажити статус повернення"
      ],
      "You&apos;ve ignored the refund.": [
        ""
      ],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [
        "Продавець &quot;%1$s&quot; пропонує вам повернення коштів."
      ],
      "Order amount": [
        "Сума замовлення"
      ],
      "Could not load the list of exchanges": [
        "Не вдалося завантажити список обмінників"
      ],
      "Could not load info from URI": [
        "Не вдалося отримати інформацію з URI"
      ],
      "Could not load the withdrawal details": [
        "Не вдалося завантажити деталі пропозиції покупки"
      ],
      "under %1$s": [
        "Переказати %1$s"
      ],
      "should be greater than zero": [
        ""
      ],
      "can't be greater than %1$s": [
        ""
      ],
      "Could not load exchange details info": [
        "Не вдалося завантажити комісії обмінника"
      ],
      "Close": [
        "Закрити"
      ],
      "There is no exchange available for currency %1$s": [
        "Недостатньо балансу для внесення депозиту у валюті %1$s"
      ],
      "You can add more exchanges from the settings.": [
        ""
      ],
      "Exchange %1$s is not available": [
        ""
      ],
      "Exchange status can view accessed from the settings.": [
        ""
      ],
      "Service fee description": [
        "Опис комісії за послуги"
      ],
      "Select %1$s exchange": [
        "Вибрати обмінник %1$s"
      ],
      "Reset": [
        "Скинути"
      ],
      "Use this exchange": [
        "Використовувати цей обмінник"
      ],
      "Auditors": [
        "Аудитори"
      ],
      "Doesn&apos;t have auditors": [
        "Немає аудиторів"
      ],
      "currency": [
        "валюта"
      ],
      "Coin operations": [
        "Операції з монетами"
      ],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.": [
        "Кожна операція в цьому розділі може відрізнятися за номіналом і дійсна протягом певного періоду часу. Обмінник стягуватиме зазначену суму кожного разу, коли монета використовується в такій операції."
      ],
      "Deposits": [
        "Депозити"
      ],
      "Denomination": [
        "Номінал"
      ],
      "Current": [
        "Валюта"
      ],
      "Selected": [
        ""
      ],
      "Until": [
        "До"
      ],
      "Withdrawals": [
        "Зняття коштів"
      ],
      "Transfer operations": [
        "Операції переказів"
      ],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.": [
        "Кожна операція в цьому розділі може відрізнятися за типом переказу і дійсна протягом певного періоду часу. Обмінник стягуватиме зазначену суму кожного разу, коли здійснюється переказ."
      ],
      "Operation": [
        "Операція"
      ],
      "Fee": [
        "Комісія"
      ],
      "Wallet operations": [
        "Операції гаманця"
      ],
      "Feature": [
        "Функція"
      ],
      "Currency": [
        "Валюта"
      ],
      "This operation has already been completed.": [
        "Ця транзакція не завершена"
      ],
      "This operation has already been aborted": [
        ""
      ],
      "This operation has started and should be completed in the bank.": [
        ""
      ],
      "You can confirm the operation in": [
        "Не вдалося завершити операцію отримання"
      ],
      "this page": [
        "Використовувати цей обмінник"
      ],
      "This operation has already been completed by another wallet.": [
        ""
      ],
      "This operation has already been used by another wallet.": [
        ""
      ],
      "It can be confirmed in": [
        "Обмінник не налаштовано"
      ],
      "Age restriction": [
        "Обмеження за віком"
      ],
      "Withdraw &nbsp; %1$s": [
        "Зняти &nbsp; %1$s"
      ],
      "Withdraw to a mobile phone": [
        "Зняти на мобільний телефон"
      ],
      "Short description of the invoice": [
        "Вкажіть адрес призначення грошей"
      ],
      "Create": [
        "Створити"
      ],
      "Short description of the transfer": [
        "Вкажіть адрес призначення грошей"
      ],
      "Could not load the invoice payment status": [
        "Не вдалося завантажити статус оплати"
      ],
      "Receive &nbsp; %1$s": [
        "Отримати &nbsp; %1$s"
      ],
      "Could not load known bank accounts": [
        "Не вдалося завантажити статус оплати"
      ],
      "Could not load supported wire methods": [
        "Не вдалося завантажити зберігачів резервних копій"
      ],
      "No wire methods supported for this currency": [
        ""
      ],
      "Try another wire method.": [
        "На інший гаманець"
      ],
      "Known accounts for %1$s": [
        "Додати банківський рахунок для %1$s"
      ],
      "To add a new account first select the account type.": [
        ""
      ],
      "Unable add this account": [
        "Не вдалося додати цей рахунок"
      ],
      "Add": [
        "Додати"
      ],
      "IBAN accounts": [
        "Додати рахунок"
      ],
      "Alias": [
        ""
      ],
      "Int. Account Number": [
        ""
      ],
      "Account name": [
        "Рахунок"
      ],
      "KYC": [
        ""
      ],
      "Taler accounts": [
        "Вибрати рахунок"
      ],
      "Host": [
        ""
      ],
      "Bitcoin accounts": [
        "Банківський рахунок"
      ],
      "Address": [
        "Рядки адреси"
      ],
      "Can't be empty": [
        ""
      ],
      "Bitcoin Account": [
        "Банківський рахунок"
      ],
      "Taler Bank": [
        "Taler Дія"
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "Required": [
        ""
      ],
      "International Bank Account Number": [
        ""
      ],
      "Could not load balance information": [
        "Не вдалося завантажити інформацію для відновлення резервної копії"
      ],
      "Could not load fee for amount %1$s": [
        "Не вдалося завантажити статус повернення"
      ],
      "A currency or an amount should be indicated": [
        "Необхідно вказати валюту або суму"
      ],
      "There is no enough balance to make a deposit for currency %1$s": [
        "Недостатньо балансу для внесення депозиту у валюті %1$s"
      ],
      "Send %1$s to your account": [
        "Надіслати %1$s на ваш рахунок"
      ],
      "There is no account to make a deposit for currency %1$s": [
        "Немає рахунку для внесення депозиту у валюті %1$s"
      ],
      "Add account": [
        "Додати рахунок"
      ],
      "Select account": [
        "Вибрати рахунок"
      ],
      "Manage accounts": [
        "Банківський рахунок"
      ],
      "Brut amount": [
        "Сума замовлення"
      ],
      "Deposit fee": [
        "Комісія за депозит"
      ],
      "Net amount": [
        "Сума замовлення"
      ],
      "Deposit&nbsp;%1$s %2$s": [
        "Депозит&nbsp;%1$s %2$s"
      ],
      "Need approval in the Bank": [
        ""
      ],
      "Waiting for wire transfer to complete": [
        ""
      ],
      "Exchange is waiting the wire transfer": [
        ""
      ],
      "Payment in progress": [
        "Повернення коштів в процесі."
      ],
      "Executing refund...": [
        "Автоматичне повернення"
      ],
      "Refreshing coins...": [
        ""
      ],
      "Deposit in progress": [
        "Повернення коштів в процесі."
      ],
      "Waiting to be paid": [
        ""
      ],
      "Receiving the transfer": [
        "Переказ електронних грошей"
      ],
      "Waiting to be received": [
        "До отримання"
      ],
      "Denomination expired": [
        "Номінал"
      ],
      "Denomination vanished": [
        "Номінал"
      ],
      "Denomination unoffered": [
        "Номінал"
      ],
      "ABORTED": [
        ""
      ],
      "FAILED": [
        ""
      ],
      "Could not load the list of transactions": [
        "Не вдалося завантажити список транзакцій"
      ],
      "unknown scope": [
        ""
      ],
      "Send": [
        "Здійснити переказ"
      ],
      "Your transaction history is empty for this currency.": [
        "Ваша історія транзакцій порожня для цієї валюти."
      ],
      "Review terms of service": [
        "Перегляньте умови обслуговування"
      ],
      "Provider URL": [
        "URL постачальника"
      ],
      "Please review and accept this provider&apos;s terms of service": [
        "Будь ласка, перегляньте та прийміть Умови надання послуг цього постачальника"
      ],
      "Pricing": [
        "Ціни"
      ],
      "free of charge": [
        "безкоштовно"
      ],
      "%1$s per year of service": [
        "%1$s на рік обслуговування"
      ],
      "Storage": [
        "Сховище"
      ],
      "%1$s megabytes of storage per year of service": [
        "%1$s мегабайт сховища на рік обслуговування"
      ],
      "Accept terms of service": [
        "Прийняти умови надання послуг"
      ],
      "Add backup provider": [
        "Додати постачальника резервного копіювання"
      ],
      "Backup providers may charge for their service": [
        "Постачальники резервного копіювання можуть стягувати плату за свої послуги"
      ],
      "URL": [
        "URL"
      ],
      "Name": [
        "Назва"
      ],
      "Next": [
        "Далі"
      ],
      "Navigator": [
        "Навігатор"
      ],
      "Automatically open wallet": [
        "Автоматично відкривати гаманець на основі вмісту сторінки"
      ],
      "Open the wallet when a payment action is found.": [
        ""
      ],
      "Version Info": [
        "Версія"
      ],
      "Web Extension": [
        "Веб-розширення"
      ],
      "The version of wallet core is not supported. (supported version: %1$s, wallet version: %2$s)": [
        ""
      ],
      "Exchange compatibility": [
        "Сумісність з обмінником"
      ],
      "Merchant compatibility": [
        "Сумісність з продавцем"
      ],
      "Bank compatibility": [
        "Сумісність з банком"
      ],
      "Wallet Core compatibility": [
        "Сумісність з банком"
      ],
      "Enable developer mode": [
        "Режим розробника"
      ],
      "Show more information and options in the UI": [
        ""
      ],
      "Display": [
        "Дисплей"
      ],
      "Current Language": [
        "Поточна мова"
      ],
      "Show backup feature": [
        ""
      ],
      "Backup integration still in beta.": [
        ""
      ],
      "Show P2P alias feature": [
        ""
      ],
      "P2P aliases integration still in beta.": [
        ""
      ],
      "Show suspend/resume transaction": [
        ""
      ],
      "Prevent transaction from doing network request.": [
        ""
      ],
      "Show refresh transaction type in the transaction list": [
        ""
      ],
      "Refresh transaction will be hidden by default if the refresh operation doesn't have fee.": [
        ""
      ],
      "Show more account types on deposit": [
        ""
      ],
      "Extends the UI to more payment target types.": [
        ""
      ],
      "Show JSON on error": [
        ""
      ],
      "Print more information about the error. Useful for debugging.": [
        "Більше опцій та інформації, корисної для відлагодження"
      ],
      "Allow HTTP connections": [
        ""
      ],
      "Using HTTP connection may be faster but unsafe (wallet restart required)": [
        ""
      ],
      "Lang selector": [
        ""
      ],
      "Allows to manually change the language of the UI. Otherwise it will be automatically selected by your browser configuration.": [
        ""
      ],
      "Edit exchange management": [
        "Ще немає обмінника"
      ],
      "Allows to see the list of exchange, remove, add and switch before withdrawal.": [
        ""
      ],
      "Select terms of service format": [
        "Прийняти умови надання послуг"
      ],
      "Allows to render the terms of service on different format selected by the user.": [
        ""
      ],
      "Show wallet activity": [
        ""
      ],
      "Show the wallet notification and observability event in the UI.": [
        ""
      ],
      "Enable v1 contract": [
        ""
      ],
      "v1 is under development and wallet-core converts v1 contract to v0. Enable this options for testing": [
        ""
      ],
      "GNU Taler Wallet installed!": [
        ""
      ],
      "You can open the wallet using the combination %1$s .": [
        "Ви можете відкрити гаманець GNU Taler за допомогою комбінації %1$s."
      ],
      "Also pinning the GNU Taler Wallet to your browser allows you to quick access without keyboard:": [
        "Також закріплення гаманця GNU Taler у вашому браузері Chrome дозволяє швидко отримати доступ без клавіатури:"
      ],
      "Click the puzzle icon": [
        "Натисніть на іконку пазлу"
      ],
      "Search for GNU Taler Wallet": [
        "Знайдіть гаманець GNU Taler"
      ],
      "Click the pin icon": [
        "Натисніть на іконку шпильки"
      ],
      "Next Steps": [
        "Наступні кроки"
      ],
      "Try the demo": [
        "Спробуйте демо"
      ],
      "Click here to open the wallet activity tab.": [
        ""
      ],
      "Events": [
        ""
      ],
      "Active tasks": [
        ""
      ],
      "All events": [
        ""
      ],
      "Balance change": [
        "Баланс"
      ],
      "Backup failed": [
        "Резервна копія дійсна до"
      ],
      "Transaction updated": [
        "Комісії за транзакцію"
      ],
      "Exchange updated": [
        "Exchange"
      ],
      "Idle": [
        ""
      ],
      "task.%1$s": [
        "%1$s"
      ],
      "wallet.%1$s(%2$s)": [
        ""
      ],
      "Bank account info changed": [
        "Банківський рахунок"
      ],
      "Contact added": [
        ""
      ],
      "Contact deleted": [
        ""
      ],
      "Mailbox message added": [
        ""
      ],
      "Mailbox message deleted": [
        ""
      ],
      "Type": [
        ""
      ],
      "Id": [
        ""
      ],
      "Since": [
        ""
      ],
      "Next try": [
        "Наступні кроки"
      ],
      "Error": [
        ""
      ],
      "Transaction": [
        "Комісії за транзакцію"
      ],
      "No exchange yet": [
        "Ще немає обмінника"
      ],
      "Status": [
        "статус"
      ],
      "Terms of Service": [
        "Умови обслуговування"
      ],
      "Last Update": [
        ""
      ],
      "Actions": [
        "Taler Дія"
      ],
      "ok": [
        "ок"
      ],
      "pending": [
        ""
      ],
      "proposed": [
        ""
      ],
      "unknown (exchange status should be updated)": [
        "невідомо (статус обмінника має бути оновлено)"
      ],
      "Reload": [
        "завантаження"
      ],
      "Purge": [
        ""
      ],
      "Make regional": [
        ""
      ],
      "Make global": [
        ""
      ],
      "Forget ToS": [
        "Забути"
      ],
      "Debug tools": [
        "Інструменти відлагодження"
      ],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?": [
        "Ви хочете НЕВІДВОРОТНО ЗНИЩИТИ все всередині вашого гаманця і ВТРАТИТИ ВСІ СВОЇ МОНЕТИ?"
      ],
      "reset": [
        "скинути"
      ],
      "TESTING: This may delete all your coin, proceed with caution": [
        "ТЕСТУВАННЯ: Це може видалити всі ваші монети, будьте обережні"
      ],
      "run gc": [
        "запустити gc"
      ],
      "import database": [
        "імпортувати базу даних"
      ],
      "export database": [
        "експортувати базу даних"
      ],
      "Clear exchange key cache": [
        "Ще немає обмінника"
      ],
      "Database exported at %1$s %2$s to download": [
        "База даних експортована о %1$s %2$s для завантаження"
      ],
      "Inject Taler support in all pages": [
        ""
      ],
      "Enabling this option will make `window.taler` be available in all sites": [
        ""
      ],
      "Exchange Entries": [
        "Обмінники"
      ],
      "Add an exchange": [
        "Додати обмінник"
      ],
      "Log levels": [
        ""
      ],
      "Coins": [
        "Монети"
      ],
      "usable coins": [
        "використовувані монети"
      ],
      "id": [
        "ідентифікатор"
      ],
      "denom": [
        "номінал"
      ],
      "status": [
        "статус"
      ],
      "from refresh?": [
        "з оновлення?"
      ],
      "age key count": [
        "кількість ключів за віком"
      ],
      "spent coins": [
        "витрачені монети"
      ],
      "click to show": [
        "натисніть, щоб показати"
      ],
      "URI is not valid. Taler URI should start with \"taler://\"": [
        "URI недійсний. Taler URI повинен починатися з `taler://`"
      ],
      "After the URI type it should follow a '/' with more information.": [
        ""
      ],
      "This URI type is not supported": [
        ""
      ],
      "This URI requires the bank host and operation id separated by /": [
        ""
      ],
      "This URI requires the merchant host, order id and session id separated by /": [
        ""
      ],
      "This URI requires the merchant host and order id separated by /": [
        ""
      ],
      "This URI requires the exchange host and key id separated by /": [
        ""
      ],
      "This URI requires the merchant host and template id separated by /": [
        ""
      ],
      "This URI requires the host key and provider list separated by /": [
        ""
      ],
      "This URI requires the experiment id": [
        ""
      ],
      "This URI requires the exchange host": [
        ""
      ],
      "This URI requires the alias type, the alias and the mailbox URI separated by /": [
        ""
      ],
      "This URI requires string after the first /": [
        ""
      ],
      "The bank host is invalid": [
        ""
      ],
      "The merchant host is invalid": [
        ""
      ],
      "The URI should end with /": [
        ""
      ],
      "The exchange host is invalid": [
        ""
      ],
      "The contact is invalid": [
        ""
      ],
      "The amount is invalid": [
        ""
      ],
      "A parameter is invalid": [
        ""
      ],
      "checking...": [
        ""
      ],
      "Unexpected error happen reading the camera: %1$s": [
        ""
      ],
      "Could not found a QR code in the file": [
        ""
      ],
      "Unexpected error happen reading the file: %1$s": [
        ""
      ],
      "Scan a QR code or enter taler:// URI below": [
        "Скануйте QR-код або введіть taler:// URI нижче"
      ],
      "Pay invoice": [
        "Електронний рахунок-фактура"
      ],
      "Withdrawal from bank": [
        "Зняття"
      ],
      "Claim refund": [
        "Вже повернено"
      ],
      "Accept payment": [
        "Прийняти"
      ],
      "Complete order": [
        ""
      ],
      "Restore wallet": [
        ""
      ],
      "Enable experiment": [
        ""
      ],
      "Withdraw from exchange": [
        "Зняти на мобільний телефон"
      ],
      "Add exchange": [
        "Додати обмінник"
      ],
      "Notify transaction": [
        ""
      ],
      "Add contact": [
        "Додати рахунок"
      ],
      "The exchange URL is invalid.": [
        ""
      ],
      "The exchange URL should end with '/'": [
        ""
      ],
      "Couldn't found an exchange in the URL specified.": [
        "не вдалося знайти жодного обмінника для валюти %1$s"
      ],
      "HTTP request failed to %1$s: %2$s": [
        ""
      ],
      "HTTP request failed to %1$s": [
        ""
      ],
      "The merchant URL is invalid.": [
        ""
      ],
      "The merchant URL should end with '/'": [
        ""
      ],
      "Couldn't found an merchant in the URL specified.": [
        ""
      ],
      "The bank URL is invalid.": [
        ""
      ],
      "The bank URL should end with '/'": [
        ""
      ],
      "Couldn't found an bank in the URL specified.": [
        ""
      ],
      "Could not load exchanges": [
        "Не вдалося завантажити комісії обмінника"
      ],
      "Choose a currency to proceed or add another exchange": [
        "Виберіть валюту для продовження або додайте інший обмінник"
      ],
      "Known currencies": [
        "Відомі валюти"
      ],
      "Specify the origin": [
        "Вкажіть джерело грошей"
      ],
      "Use previous origins:": [
        "Використовувати попередні джерела:"
      ],
      "From another bank account": [
        "З мого банківського рахунку"
      ],
      "From another wallet": [
        "З іншого гаманця"
      ],
      "From a %1$s URI or QR code": [
        ""
      ],
      "Enter URI here": [
        ""
      ],
      "Specify the destination": [
        "Вкажіть адрес призначення грошей"
      ],
      "Use previous destinations:": [
        "Використовувати попередні місця призначення:"
      ],
      "To another bank account": [
        "На мій банківський рахунок"
      ],
      "To another wallet": [
        "На інший гаманець"
      ],
      "To a %1$s URI or QR code": [
        ""
      ],
      "Afghanistan": [
        ""
      ],
      "Albania": [
        ""
      ],
      "Algeria": [
        ""
      ],
      "United States": [
        ""
      ],
      "Andorra": [
        ""
      ],
      "Angola": [
        ""
      ],
      "Antigua and Barbuda": [
        ""
      ],
      "Argentina": [
        ""
      ],
      "Armenia": [
        ""
      ],
      "Australia": [
        ""
      ],
      "Austria": [
        ""
      ],
      "Azerbaijan": [
        ""
      ],
      "Bahamas": [
        ""
      ],
      "Bahrain": [
        ""
      ],
      "Bangladesh": [
        ""
      ],
      "Barbados": [
        ""
      ],
      "Botswana": [
        ""
      ],
      "Belarus": [
        ""
      ],
      "Belgium": [
        ""
      ],
      "Belize": [
        ""
      ],
      "Benin": [
        ""
      ],
      "Bhutan": [
        ""
      ],
      "Bolivia": [
        ""
      ],
      "Bosnia and Herzegovina": [
        ""
      ],
      "Brazil": [
        ""
      ],
      "United Kingdom": [
        ""
      ],
      "Brunei": [
        ""
      ],
      "Bulgaria": [
        ""
      ],
      "Burkina Faso": [
        ""
      ],
      "Myanmar": [
        ""
      ],
      "Burundi": [
        ""
      ],
      "Cambodia": [
        ""
      ],
      "Cameroon": [
        ""
      ],
      "Canada": [
        ""
      ],
      "Cape Verde": [
        ""
      ],
      "Central African Republic": [
        ""
      ],
      "Chad": [
        ""
      ],
      "Chile": [
        ""
      ],
      "China": [
        ""
      ],
      "Colombia": [
        ""
      ],
      "Comoros": [
        ""
      ],
      "Democratic Republic of the Congo": [
        ""
      ],
      "Republic of the Congo": [
        ""
      ],
      "Costa Rica": [
        ""
      ],
      "Croatia": [
        ""
      ],
      "Cuba": [
        ""
      ],
      "Cyprus": [
        ""
      ],
      "Czech Republic": [
        "Публічний ключ продавця"
      ],
      "Denmark": [
        ""
      ],
      "Djibouti": [
        ""
      ],
      "Dominican Republic": [
        ""
      ],
      "Netherlands": [
        ""
      ],
      "Timor-Leste": [
        ""
      ],
      "Ecuador": [
        ""
      ],
      "Egypt": [
        ""
      ],
      "United Arab Emirates": [
        ""
      ],
      "Equatorial Guinea": [
        ""
      ],
      "Eritrea": [
        ""
      ],
      "Estonia": [
        ""
      ],
      "Ethiopia": [
        ""
      ],
      "Fiji": [
        ""
      ],
      "Philippines": [
        ""
      ],
      "Finland": [
        ""
      ],
      "France": [
        "Скасувати"
      ],
      "Gabon": [
        ""
      ],
      "Gambia": [
        ""
      ],
      "Georgia": [
        ""
      ],
      "Germany": [
        ""
      ],
      "Ghana": [
        ""
      ],
      "Greece": [
        ""
      ],
      "Grenada": [
        ""
      ],
      "Guatemala": [
        ""
      ],
      "Guinea-Bissau": [
        ""
      ],
      "Guinea": [
        ""
      ],
      "Guyana": [
        ""
      ],
      "Haiti": [
        ""
      ],
      "Honduras": [
        ""
      ],
      "Hungary": [
        ""
      ],
      "Kiribati": [
        ""
      ],
      "Iceland": [
        ""
      ],
      "India": [
        ""
      ],
      "Indonesia": [
        ""
      ],
      "Iran": [
        ""
      ],
      "Iraq": [
        ""
      ],
      "Ireland": [
        ""
      ],
      "Israel": [
        ""
      ],
      "Italy": [
        ""
      ],
      "Ivory Coast": [
        ""
      ],
      "Jamaica": [
        ""
      ],
      "Japan": [
        ""
      ],
      "Jordan": [
        ""
      ],
      "Kazakhstan": [
        ""
      ],
      "Kenya": [
        ""
      ],
      "Saint Kitts and Nevis": [
        ""
      ],
      "Kuwait": [
        ""
      ],
      "Kyrgyzstan": [
        ""
      ],
      "Laos": [
        ""
      ],
      "Latvia": [
        ""
      ],
      "Lebanon": [
        ""
      ],
      "Liberia": [
        ""
      ],
      "Libya": [
        ""
      ],
      "Liechtenstein": [
        ""
      ],
      "Lithuania": [
        ""
      ],
      "Luxembourg": [
        ""
      ],
      "North Macedonia": [
        ""
      ],
      "Madagascar": [
        ""
      ],
      "Malawi": [
        ""
      ],
      "Malaysia": [
        ""
      ],
      "Maldives": [
        ""
      ],
      "Mali": [
        ""
      ],
      "Malta": [
        ""
      ],
      "Marshall Islands": [
        ""
      ],
      "Mauritania": [
        ""
      ],
      "Mauritius": [
        ""
      ],
      "Mexico": [
        ""
      ],
      "Micronesia": [
        ""
      ],
      "Moldova": [
        ""
      ],
      "Monaco": [
        ""
      ],
      "Mongolia": [
        ""
      ],
      "Morocco": [
        ""
      ],
      "Lesotho": [
        ""
      ],
      "Mozambique": [
        ""
      ],
      "Namibia": [
        ""
      ],
      "Nauru": [
        ""
      ],
      "Nepal": [
        ""
      ],
      "New Zealand": [
        ""
      ],
      "Vanuatu": [
        ""
      ],
      "Nicaragua": [
        ""
      ],
      "Nigeria": [
        ""
      ],
      "Niger": [
        ""
      ],
      "North Korea": [
        ""
      ],
      "Norway": [
        ""
      ],
      "Oman": [
        ""
      ],
      "Pakistan": [
        ""
      ],
      "Palau": [
        ""
      ],
      "Panama": [
        ""
      ],
      "Papua New Guinea": [
        ""
      ],
      "Paraguay": [
        ""
      ],
      "Peru": [
        ""
      ],
      "Poland": [
        ""
      ],
      "Portugal": [
        ""
      ],
      "Qatar": [
        ""
      ],
      "Romania": [
        ""
      ],
      "Russia": [
        ""
      ],
      "Rwanda": [
        ""
      ],
      "Saint Lucia": [
        ""
      ],
      "El Salvador": [
        ""
      ],
      "Samoa": [
        ""
      ],
      "San Marino": [
        ""
      ],
      "São Tomé and Príncipe": [
        ""
      ],
      "Saudi Arabia": [
        ""
      ],
      "Senegal": [
        ""
      ],
      "Serbia": [
        ""
      ],
      "Seychelles": [
        ""
      ],
      "Sierra Leone": [
        ""
      ],
      "Singapore": [
        ""
      ],
      "Slovakia": [
        ""
      ],
      "Slovenia": [
        ""
      ],
      "Solomon Islands": [
        ""
      ],
      "Somalia": [
        ""
      ],
      "South Africa": [
        ""
      ],
      "South Korea": [
        ""
      ],
      "Spain": [
        ""
      ],
      "Sri Lanka": [
        ""
      ],
      "Sudan": [
        ""
      ],
      "Suriname": [
        ""
      ],
      "Eswatini": [
        ""
      ],
      "Sweden": [
        ""
      ],
      "Switzerland": [
        ""
      ],
      "Syria": [
        ""
      ],
      "Taiwan": [
        ""
      ],
      "Tajikistan": [
        ""
      ],
      "Tanzania": [
        ""
      ],
      "Thailand": [
        ""
      ],
      "Togo": [
        ""
      ],
      "Tonga": [
        ""
      ],
      "Trinidad and Tobago": [
        ""
      ],
      "Tunisia": [
        ""
      ],
      "Turkey": [
        ""
      ],
      "Tuvalu": [
        "значення"
      ],
      "Uganda": [
        ""
      ],
      "Ukraine": [
        ""
      ],
      "Uruguay": [
        ""
      ],
      "Uzbekistan": [
        ""
      ],
      "Venezuela": [
        ""
      ],
      "Vietnam": [
        ""
      ],
      "Yemen": [
        ""
      ],
      "Zambia": [
        ""
      ],
      "Zimbabwe": [
        ""
      ],
      "Afghan": [
        ""
      ],
      "Albanian": [
        ""
      ],
      "Algerian": [
        ""
      ],
      "American": [
        ""
      ],
      "Andorran": [
        ""
      ],
      "Angolan": [
        ""
      ],
      "Antiguan": [
        ""
      ],
      "Argentinean": [
        ""
      ],
      "Armenian": [
        ""
      ],
      "Australian": [
        ""
      ],
      "Austrian": [
        ""
      ],
      "Azerbaijani": [
        ""
      ],
      "Bahamian": [
        ""
      ],
      "Bahraini": [
        ""
      ],
      "Bangladeshi": [
        ""
      ],
      "Barbadian": [
        ""
      ],
      "Batswana": [
        ""
      ],
      "Belarusian": [
        ""
      ],
      "Belgian": [
        ""
      ],
      "Belizean": [
        ""
      ],
      "Beninese": [
        ""
      ],
      "Bhutanese": [
        ""
      ],
      "Bolivian": [
        ""
      ],
      "Bosnian": [
        ""
      ],
      "Brazilian": [
        ""
      ],
      "British": [
        ""
      ],
      "Bruneian": [
        ""
      ],
      "Bulgarian": [
        ""
      ],
      "Burkinabe": [
        ""
      ],
      "Burmese": [
        ""
      ],
      "Burundian": [
        ""
      ],
      "Cambodian": [
        ""
      ],
      "Cameroonian": [
        ""
      ],
      "Canadian": [
        ""
      ],
      "Cape Verdean": [
        ""
      ],
      "Central African": [
        ""
      ],
      "Chadian": [
        ""
      ],
      "Chilean": [
        ""
      ],
      "Chinese": [
        ""
      ],
      "Colombian": [
        ""
      ],
      "Comoran": [
        ""
      ],
      "Congolese": [
        ""
      ],
      "Costa Rican": [
        ""
      ],
      "Croatian": [
        ""
      ],
      "Cuban": [
        ""
      ],
      "Cypriot": [
        ""
      ],
      "Czech": [
        ""
      ],
      "Danish": [
        ""
      ],
      "Djiboutian": [
        ""
      ],
      "Dominican": [
        "Номінал"
      ],
      "Dutch": [
        ""
      ],
      "East Timorese": [
        ""
      ],
      "Ecuadorean": [
        ""
      ],
      "Egyptian": [
        ""
      ],
      "Emirati": [
        ""
      ],
      "Equatorial Guinean": [
        ""
      ],
      "Eritrean": [
        ""
      ],
      "Estonian": [
        ""
      ],
      "Ethiopian": [
        ""
      ],
      "Fijian": [
        ""
      ],
      "Filipino": [
        ""
      ],
      "Finnish": [
        ""
      ],
      "French": [
        ""
      ],
      "Gabonese": [
        ""
      ],
      "Gambian": [
        ""
      ],
      "Georgian": [
        ""
      ],
      "German": [
        ""
      ],
      "Ghanaian": [
        ""
      ],
      "Greek": [
        ""
      ],
      "Grenadian": [
        ""
      ],
      "Guatemalan": [
        ""
      ],
      "Guinea-Bissauan": [
        ""
      ],
      "Guinean": [
        ""
      ],
      "Guyanese": [
        ""
      ],
      "Haitian": [
        ""
      ],
      "Honduran": [
        ""
      ],
      "Hungarian": [
        ""
      ],
      "I-Kiribati": [
        ""
      ],
      "Icelander": [
        ""
      ],
      "Indian": [
        ""
      ],
      "Indonesian": [
        ""
      ],
      "Iranian": [
        ""
      ],
      "Iraqi": [
        ""
      ],
      "Irish": [
        ""
      ],
      "Israeli": [
        ""
      ],
      "Italian": [
        ""
      ],
      "Ivorian": [
        ""
      ],
      "Jamaican": [
        ""
      ],
      "Japanese": [
        ""
      ],
      "Jordanian": [
        ""
      ],
      "Kazakhstani": [
        ""
      ],
      "Kenyan": [
        ""
      ],
      "Kittian and Nevisian": [
        ""
      ],
      "Kuwaiti": [
        ""
      ],
      "Kyrgyz": [
        ""
      ],
      "Laotian": [
        ""
      ],
      "Latvian": [
        ""
      ],
      "Lebanese": [
        ""
      ],
      "Liberian": [
        ""
      ],
      "Libyan": [
        ""
      ],
      "Liechtensteiner": [
        ""
      ],
      "Lithuanian": [
        ""
      ],
      "Luxembourger": [
        ""
      ],
      "Macedonian": [
        ""
      ],
      "Malagasy": [
        ""
      ],
      "Malawian": [
        ""
      ],
      "Malaysian": [
        ""
      ],
      "Maldivian": [
        ""
      ],
      "Malian": [
        ""
      ],
      "Maltese": [
        ""
      ],
      "Marshallese": [
        ""
      ],
      "Mauritanian": [
        ""
      ],
      "Mauritian": [
        ""
      ],
      "Mexican": [
        ""
      ],
      "Micronesian": [
        ""
      ],
      "Moldovan": [
        ""
      ],
      "Monacan": [
        ""
      ],
      "Mongolian": [
        ""
      ],
      "Moroccan": [
        ""
      ],
      "Mosotho": [
        ""
      ],
      "Mozambican": [
        ""
      ],
      "Namibian": [
        ""
      ],
      "Nauruan": [
        ""
      ],
      "Nepalese": [
        ""
      ],
      "New Zealander": [
        ""
      ],
      "Ni-Vanuatu": [
        ""
      ],
      "Nicaraguan": [
        ""
      ],
      "Nigerian": [
        ""
      ],
      "Nigerien": [
        ""
      ],
      "North Korean": [
        ""
      ],
      "Norwegian": [
        ""
      ],
      "Omani": [
        ""
      ],
      "Pakistani": [
        ""
      ],
      "Palauan": [
        ""
      ],
      "Panamanian": [
        ""
      ],
      "Papua New Guinean": [
        ""
      ],
      "Paraguayan": [
        ""
      ],
      "Peruvian": [
        ""
      ],
      "Polish": [
        ""
      ],
      "Portuguese": [
        ""
      ],
      "Qatari": [
        ""
      ],
      "Romanian": [
        ""
      ],
      "Russian": [
        ""
      ],
      "Rwandan": [
        ""
      ],
      "Saint Lucian": [
        ""
      ],
      "Salvadoran": [
        ""
      ],
      "Samoan": [
        ""
      ],
      "San Marinese": [
        ""
      ],
      "Sao Tomean": [
        ""
      ],
      "Saudi": [
        ""
      ],
      "Senegalese": [
        ""
      ],
      "Serbian": [
        ""
      ],
      "Seychellois": [
        ""
      ],
      "Sierra Leonean": [
        ""
      ],
      "Singaporean": [
        ""
      ],
      "Slovakian": [
        ""
      ],
      "Slovenian": [
        ""
      ],
      "Solomon Islander": [
        ""
      ],
      "Somali": [
        ""
      ],
      "South African": [
        ""
      ],
      "South Korean": [
        ""
      ],
      "Spanish": [
        ""
      ],
      "Sri Lankan": [
        ""
      ],
      "Sudanese": [
        ""
      ],
      "Surinamer": [
        ""
      ],
      "Swazi": [
        ""
      ],
      "Swedish": [
        ""
      ],
      "Swiss": [
        ""
      ],
      "Syrian": [
        ""
      ],
      "Taiwanese": [
        ""
      ],
      "Tajik": [
        ""
      ],
      "Tanzanian": [
        ""
      ],
      "Thai": [
        ""
      ],
      "Togolese": [
        ""
      ],
      "Tongan": [
        ""
      ],
      "Trinidadian or Tobagonian": [
        ""
      ],
      "Tunisian": [
        ""
      ],
      "Turkish": [
        ""
      ],
      "Tuvaluan": [
        "значення"
      ],
      "Ugandan": [
        ""
      ],
      "Ukrainian": [
        ""
      ],
      "Uruguayan": [
        ""
      ],
      "Uzbekistani": [
        ""
      ],
      "Venezuelan": [
        ""
      ],
      "Vietnamese": [
        ""
      ],
      "Yemenite": [
        ""
      ],
      "Zambian": [
        ""
      ],
      "Zimbabwean": [
        ""
      ],
      "🇦🇫": [
        ""
      ],
      "🇦🇱": [
        ""
      ],
      "🇩🇿": [
        ""
      ],
      "🇺🇸": [
        ""
      ],
      "🇦🇩": [
        ""
      ],
      "🇦🇴": [
        ""
      ],
      "🇦🇬": [
        ""
      ],
      "🇦🇷": [
        ""
      ],
      "🇦🇲": [
        ""
      ],
      "🇦🇺": [
        ""
      ],
      "🇦🇹": [
        ""
      ],
      "🇦🇿": [
        ""
      ],
      "🇧🇸": [
        ""
      ],
      "🇧🇭": [
        ""
      ],
      "🇧🇩": [
        ""
      ],
      "🇧🇧": [
        ""
      ],
      "🇧🇼": [
        ""
      ],
      "🇧🇾": [
        ""
      ],
      "🇧🇪": [
        ""
      ],
      "🇧🇿": [
        ""
      ],
      "🇧🇯": [
        ""
      ],
      "🇧🇹": [
        ""
      ],
      "🇧🇴": [
        ""
      ],
      "🇧🇦": [
        ""
      ],
      "🇧🇷": [
        ""
      ],
      "🇬🇧": [
        ""
      ],
      "🇧🇳": [
        ""
      ],
      "🇧🇬": [
        ""
      ],
      "🇧🇫": [
        ""
      ],
      "🇲🇲": [
        ""
      ],
      "🇧🇮": [
        ""
      ],
      "🇰🇭": [
        ""
      ],
      "🇨🇲": [
        ""
      ],
      "🇨🇦": [
        ""
      ],
      "🇨🇻": [
        ""
      ],
      "🇨🇫": [
        ""
      ],
      "🇨🇱": [
        ""
      ],
      "🇨🇳": [
        ""
      ],
      "🇨🇴": [
        ""
      ],
      "🇨🇩": [
        ""
      ],
      "🇨🇬": [
        ""
      ],
      "🇨🇷": [
        ""
      ],
      "🇨🇺": [
        ""
      ],
      "🇨🇾": [
        ""
      ],
      "🇨🇿": [
        ""
      ],
      "🇩🇰": [
        ""
      ],
      "🇩🇯": [
        ""
      ],
      "🇩🇴": [
        ""
      ],
      "🇳🇱": [
        ""
      ],
      "🇹🇱": [
        ""
      ],
      "🇪🇨": [
        ""
      ],
      "🇪🇬": [
        ""
      ],
      "🇦🇪": [
        ""
      ],
      "🇬🇶": [
        ""
      ],
      "🇪🇷": [
        ""
      ],
      "🇪🇪": [
        ""
      ],
      "🇪🇹": [
        ""
      ],
      "🇫🇯": [
        ""
      ],
      "🇵🇭": [
        ""
      ],
      "🇫🇮": [
        ""
      ],
      "🇫🇷": [
        ""
      ],
      "🇬🇦": [
        ""
      ],
      "🇬🇲": [
        ""
      ],
      "🇬🇪": [
        ""
      ],
      "UAE Dirham": [
        ""
      ],
      "Afghani": [
        ""
      ],
      "Lek": [
        ""
      ],
      "Armenian Dram": [
        ""
      ],
      "Netherlands Antillean Guilder": [
        ""
      ],
      "Kwanza": [
        ""
      ],
      "Argentine Peso": [
        ""
      ],
      "Australian Dollar": [
        ""
      ],
      "Aruban Florin": [
        ""
      ],
      "Azerbaijan Manat": [
        ""
      ],
      "Convertible Mark": [
        ""
      ],
      "Barbados Dollar": [
        ""
      ],
      "Taka": [
        ""
      ],
      "Bulgarian Lev": [
        ""
      ],
      "Bahraini Dinar": [
        ""
      ],
      "Burundi Franc": [
        ""
      ],
      "Bermudian Dollar": [
        ""
      ],
      "Brunei Dollar": [
        ""
      ],
      "Boliviano": [
        ""
      ],
      "Mvdol": [
        ""
      ],
      "Brazilian Real": [
        ""
      ],
      "Bahamian Dollar": [
        ""
      ],
      "Ngultrum": [
        ""
      ],
      "Pula": [
        ""
      ],
      "Belarusian Ruble": [
        ""
      ],
      "Belize Dollar": [
        ""
      ],
      "Canadian Dollar": [
        ""
      ],
      "Congolese Franc": [
        ""
      ],
      "WIR Euro": [
        ""
      ],
      "Swiss Franc": [
        ""
      ],
      "WIR Franc": [
        ""
      ],
      "Unidad de Fomento": [
        ""
      ],
      "Chilean Peso": [
        ""
      ],
      "Yuan Renminbi": [
        ""
      ],
      "Colombian Peso": [
        ""
      ],
      "Unidad de Valor Real": [
        ""
      ],
      "Costa Rican Colon": [
        ""
      ],
      "Cuban Peso": [
        ""
      ],
      "Cabo Verde Escudo": [
        ""
      ],
      "Czech Koruna": [
        ""
      ],
      "Djibouti Franc": [
        ""
      ],
      "Danish Krone": [
        ""
      ],
      "Dominican Peso": [
        ""
      ],
      "Algerian Dinar": [
        ""
      ],
      "Egyptian Pound": [
        ""
      ],
      "Nakfa": [
        ""
      ],
      "Ethiopian Birr": [
        ""
      ],
      "Euro": [
        ""
      ],
      "Fiji Dollar": [
        ""
      ],
      "Falkland Islands Pound": [
        ""
      ],
      "Pound Sterling": [
        ""
      ],
      "Lari": [
        ""
      ],
      "Ghana Cedi": [
        ""
      ],
      "Gibraltar Pound": [
        ""
      ],
      "Dalasi": [
        ""
      ],
      "Guinean Franc": [
        ""
      ],
      "Quetzal": [
        ""
      ],
      "Guyana Dollar": [
        ""
      ],
      "Hong Kong Dollar": [
        ""
      ],
      "Lempira": [
        ""
      ],
      "Gourde": [
        ""
      ],
      "Forint": [
        ""
      ],
      "Rupiah": [
        ""
      ],
      "New Israeli Sheqel": [
        ""
      ],
      "Indian Rupee": [
        ""
      ],
      "Iraqi Dinar": [
        ""
      ],
      "Iranian Rial": [
        ""
      ],
      "Iceland Krona": [
        ""
      ],
      "Jamaican Dollar": [
        ""
      ],
      "Jordanian Dinar": [
        ""
      ],
      "Yen": [
        ""
      ],
      "Kenyan Shilling": [
        ""
      ],
      "Som": [
        ""
      ],
      "Riel": [
        ""
      ],
      "Comorian Franc ": [
        ""
      ],
      "North Korean Won": [
        ""
      ],
      "Won": [
        ""
      ],
      "Kuwaiti Dinar": [
        ""
      ],
      "Cayman Islands Dollar": [
        ""
      ],
      "Tenge": [
        ""
      ],
      "Lao Kip": [
        ""
      ],
      "Lebanese Pound": [
        ""
      ],
      "Sri Lanka Rupee": [
        ""
      ],
      "Liberian Dollar": [
        ""
      ],
      "Loti": [
        ""
      ],
      "Libyan Dinar": [
        ""
      ],
      "Moroccan Dirham": [
        ""
      ],
      "Moldovan Leu": [
        ""
      ],
      "Malagasy Ariary": [
        ""
      ],
      "Denar": [
        ""
      ],
      "Kyat": [
        ""
      ],
      "Tugrik": [
        ""
      ],
      "Pataca": [
        ""
      ],
      "Ouguiya": [
        ""
      ],
      "Mauritius Rupee": [
        ""
      ],
      "Rufiyaa": [
        ""
      ],
      "Malawi Kwacha": [
        ""
      ],
      "Mexican Peso": [
        ""
      ],
      "Mexican Unidad de Inversion (UDI)": [
        ""
      ],
      "Malaysian Ringgit": [
        ""
      ],
      "Mozambique Metical": [
        ""
      ],
      "Namibia Dollar": [
        ""
      ],
      "Naira": [
        ""
      ],
      "Cordoba Oro": [
        ""
      ],
      "Norwegian Krone": [
        ""
      ],
      "Nepalese Rupee": [
        ""
      ],
      "New Zealand Dollar": [
        ""
      ],
      "Rial Omani": [
        ""
      ],
      "Balboa": [
        ""
      ],
      "Sol": [
        "старий"
      ],
      "Kina": [
        ""
      ],
      "Philippine Peso": [
        ""
      ],
      "Pakistan Rupee": [
        ""
      ],
      "Zloty": [
        ""
      ],
      "Guarani": [
        ""
      ],
      "Qatari Rial": [
        ""
      ],
      "Romanian Leu": [
        ""
      ],
      "Serbian Dinar": [
        ""
      ],
      "Russian Ruble": [
        ""
      ],
      "Rwanda Franc": [
        ""
      ],
      "Saudi Riyal": [
        ""
      ],
      "Solomon Islands Dollar": [
        ""
      ],
      "Seychelles Rupee": [
        ""
      ],
      "Sudanese Pound": [
        ""
      ],
      "Swedish Krona": [
        ""
      ],
      "Singapore Dollar": [
        ""
      ],
      "Saint Helena Pound": [
        ""
      ],
      "Leone": [
        ""
      ],
      "Somali Shilling": [
        ""
      ],
      "Surinam Dollar": [
        ""
      ],
      "South Sudanese Pound": [
        ""
      ],
      "Dobra": [
        ""
      ],
      "El Salvador Colon": [
        ""
      ],
      "Syrian Pound": [
        ""
      ],
      "Lilangeni": [
        ""
      ],
      "Baht": [
        ""
      ],
      "Somoni": [
        ""
      ],
      "Turkmenistan New Manat": [
        ""
      ],
      "Tunisian Dinar": [
        ""
      ],
      "Pa’anga": [
        ""
      ],
      "Turkish Lira": [
        ""
      ],
      "Trinidad and Tobago Dollar": [
        ""
      ],
      "New Taiwan Dollar": [
        ""
      ],
      "Tanzanian Shilling": [
        ""
      ],
      "Hryvnia": [
        ""
      ],
      "Uganda Shilling": [
        ""
      ],
      "US Dollar": [
        ""
      ],
      "US Dollar (Next day)": [
        ""
      ],
      "Uruguay Peso en Unidades Indexadas (UI)": [
        ""
      ],
      "Peso Uruguayo": [
        ""
      ],
      "Unidad Previsional": [
        ""
      ],
      "Uzbekistan Sum": [
        ""
      ],
      "Bolívar Soberano": [
        ""
      ],
      "Dong": [
        ""
      ],
      "Vatu": [
        ""
      ],
      "Tala": [
        ""
      ],
      "CFA Franc BEAC": [
        ""
      ],
      "East Caribbean Dollar": [
        ""
      ],
      "CFA Franc BCEAO": [
        ""
      ],
      "CFP Franc": [
        ""
      ],
      "Sucre": [
        ""
      ],
      "Yemeni Rial": [
        ""
      ],
      "Rand": [
        ""
      ],
      "Zambian Kwacha": [
        ""
      ],
      "Zimbabwe Gold": [
        ""
      ],
      "Afar": [
        ""
      ],
      "Abkhazian": [
        ""
      ],
      "Achinese": [
        ""
      ],
      "Acoli": [
        ""
      ],
      "Adangme": [
        ""
      ],
      "Adyghe": [
        ""
      ],
      "Avestan": [
        ""
      ],
      "Tunisian Arabic": [
        ""
      ],
      "Afrikaans": [
        ""
      ],
      "Afrihili": [
        ""
      ],
      "Aghem": [
        ""
      ],
      "Ainu": [
        ""
      ],
      "Akan": [
        ""
      ],
      "Akkadian": [
        ""
      ],
      "Alabama": [
        ""
      ],
      "Aleut": [
        ""
      ],
      "Gheg Albanian": [
        ""
      ],
      "Southern Altai": [
        ""
      ],
      "Amarik": [
        ""
      ],
      "Aragonese": [
        ""
      ],
      "Old English": [
        ""
      ],
      "Angika": [
        ""
      ],
      "Modern Standard Arabic": [
        ""
      ],
      "Arabik": [
        ""
      ],
      "Aramaic": [
        ""
      ],
      "Mapuche": [
        ""
      ],
      "Araona": [
        ""
      ],
      "Arapaho": [
        ""
      ],
      "Algerian Arabic": [
        ""
      ],
      "Arawak": [
        ""
      ],
      "Moroccan Arabic": [
        ""
      ],
      "Egyptian Arabic": [
        ""
      ],
      "Asu": [
        ""
      ],
      "Assamese": [
        ""
      ],
      "American Sign Language": [
        ""
      ],
      "Asturian": [
        ""
      ],
      "Avaric": [
        ""
      ],
      "Kotava": [
        ""
      ],
      "Awadhi": [
        ""
      ],
      "Aymara": [
        ""
      ],
      "South Azerbaijani": [
        ""
      ],
      "Bashkir": [
        ""
      ],
      "Baluchi": [
        ""
      ],
      "Balinese": [
        ""
      ],
      "Bavarian": [
        ""
      ],
      "Basaa": [
        ""
      ],
      "Bamun": [
        ""
      ],
      "Batak Toba": [
        ""
      ],
      "Ghomala": [
        ""
      ],
      "Belarus kasa": [
        ""
      ],
      "Beja": [
        ""
      ],
      "Bemba": [
        ""
      ],
      "Betawi": [
        ""
      ],
      "Bena": [
        ""
      ],
      "Bafut": [
        ""
      ],
      "Badaga": [
        ""
      ],
      "Bɔlgeria kasa": [
        ""
      ],
      "Bhojpuri": [
        ""
      ],
      "Bislama": [
        ""
      ],
      "Bikol": [
        ""
      ],
      "Bini": [
        ""
      ],
      "Banjar": [
        ""
      ],
      "Kom": [
        ""
      ],
      "Siksika": [
        ""
      ],
      "Bambara": [
        ""
      ],
      "Bengali kasa": [
        ""
      ],
      "Tibetan": [
        ""
      ],
      "Bishnupriya": [
        ""
      ],
      "Bakhtiari": [
        ""
      ],
      "Braj": [
        ""
      ],
      "Breton": [
        ""
      ],
      "Brahui": [
        ""
      ],
      "Bodo": [
        ""
      ],
      "Akoose": [
        ""
      ],
      "Buriat": [
        ""
      ],
      "Buginese": [
        ""
      ],
      "Bulu": [
        ""
      ],
      "Blin": [
        ""
      ],
      "Medumba": [
        ""
      ],
      "Catalan": [
        ""
      ],
      "Caddo": [
        ""
      ],
      "Carib": [
        ""
      ],
      "Cayuga": [
        ""
      ],
      "Atsam": [
        ""
      ],
      "Cebuano": [
        ""
      ],
      "Chechen": [
        ""
      ],
      "Chiga": [
        ""
      ],
      "Chibcha": [
        ""
      ],
      "Chamorro": [
        ""
      ],
      "Chagatai": [
        ""
      ],
      "Chuukese": [
        ""
      ],
      "Mari": [
        ""
      ],
      "Chinook Jargon": [
        ""
      ],
      "Choctaw": [
        ""
      ],
      "Chipewyan": [
        ""
      ],
      "Cherokee": [
        ""
      ],
      "Cheyenne": [
        ""
      ],
      "Central Kurdish": [
        ""
      ],
      "Corsican": [
        ""
      ],
      "Coptic": [
        ""
      ],
      "Capiznon": [
        ""
      ],
      "Cree": [
        ""
      ],
      "Crimean Turkish": [
        ""
      ],
      "Kashubian": [
        ""
      ],
      "Kyɛk kasa": [
        ""
      ],
      "Church Slavic": [
        ""
      ],
      "Chuvash": [
        ""
      ],
      "Welsh": [
        ""
      ],
      "Dakota": [
        ""
      ],
      "Dargwa": [
        ""
      ],
      "Taita": [
        ""
      ],
      "Austrian German": [
        ""
      ],
      "Swiss High German": [
        ""
      ],
      "Gyaaman": [
        ""
      ],
      "Delaware": [
        ""
      ],
      "Slave": [
        ""
      ],
      "Dogrib": [
        ""
      ],
      "Dinka": [
        ""
      ],
      "Zarma": [
        ""
      ],
      "Dogri": [
        ""
      ],
      "Lower Sorbian": [
        ""
      ],
      "Central Dusun": [
        ""
      ],
      "Duala": [
        ""
      ],
      "Middle Dutch": [
        ""
      ],
      "Divehi": [
        ""
      ],
      "Jola-Fonyi": [
        ""
      ],
      "Dyula": [
        ""
      ],
      "Dzongkha": [
        ""
      ],
      "Dazaga": [
        ""
      ],
      "Embu": [
        ""
      ],
      "Ewe": [
        ""
      ],
      "Efik": [
        ""
      ],
      "Emilian": [
        ""
      ],
      "Ancient Egyptian": [
        ""
      ],
      "Ekajuk": [
        ""
      ],
      "Greek kasa": [
        ""
      ],
      "Elamite": [
        ""
      ],
      "Australian English": [
        ""
      ],
      "Borɔfo": [
        ""
      ],
      "Canadian English": [
        ""
      ],
      "British English": [
        ""
      ],
      "Middle English": [
        ""
      ],
      "American English": [
        ""
      ],
      "Esperanto": [
        "Операція"
      ],
      "Latin American Spanish": [
        ""
      ],
      "European Spanish": [
        ""
      ],
      "Mexican Spanish": [
        ""
      ],
      "Spain kasa": [
        ""
      ],
      "Central Yupik": [
        ""
      ],
      "Basque": [
        ""
      ],
      "Ewondo": [
        ""
      ],
      "Extremaduran": [
        ""
      ],
      "Fang": [
        ""
      ],
      "Pɛɛhyia kasa": [
        ""
      ],
      "Fanti": [
        ""
      ],
      "Fulah": [
        ""
      ],
      "Tornedalen Finnish": [
        ""
      ],
      "Faroese": [
        ""
      ],
      "Fon": [
        ""
      ],
      "Canadian French": [
        ""
      ],
      "Cajun French": [
        "Валюта"
      ],
      "Swiss French": [
        ""
      ],
      "Frɛnkye": [
        ""
      ],
      "Middle French": [
        ""
      ],
      "Old French": [
        ""
      ],
      "Arpitan": [
        ""
      ],
      "Northern Frisian": [
        ""
      ],
      "Eastern Frisian": [
        ""
      ],
      "Friulian": [
        ""
      ],
      "Western Frisian": [
        ""
      ],
      "Ga": [
        ""
      ],
      "Gagauz": [
        ""
      ],
      "Gan Chinese": [
        ""
      ],
      "Gayo": [
        ""
      ],
      "Gbaya": [
        ""
      ],
      "Zoroastrian Dari": [
        ""
      ],
      "Scottish Gaelic": [
        ""
      ],
      "Geez": [
        ""
      ],
      "Gilbertese": [
        ""
      ],
      "Galician": [
        ""
      ],
      "Gilaki": [
        ""
      ],
      "Middle High German": [
        ""
      ],
      "Old High German": [
        ""
      ],
      "Goan Konkani": [
        ""
      ],
      "Gondi": [
        ""
      ],
      "Gorontalo": [
        ""
      ],
      "Gothic": [
        ""
      ],
      "Grebo": [
        ""
      ],
      "Ancient Greek": [
        ""
      ],
      "Swiss German": [
        ""
      ],
      "Wayuu": [
        ""
      ],
      "Gujarati": [
        ""
      ],
      "Frafra": [
        ""
      ],
      "Gusii": [
        ""
      ],
      "Manx": [
        ""
      ],
      "Gwichʼin": [
        ""
      ],
      "Hausa": [
        ""
      ],
      "Haida": [
        ""
      ],
      "Hakka Chinese": [
        ""
      ],
      "Hawaiian": [
        ""
      ],
      "Hebrew": [
        ""
      ],
      "Fiji Hindi": [
        ""
      ],
      "Hindi": [
        ""
      ],
      "Hiligaynon": [
        ""
      ],
      "Hittite": [
        ""
      ],
      "Hmong": [
        ""
      ],
      "Hiri Motu": [
        ""
      ],
      "Upper Sorbian": [
        ""
      ],
      "Xiang Chinese": [
        ""
      ],
      "Hangri kasa": [
        ""
      ],
      "Hupa": [
        ""
      ],
      "Herero": [
        ""
      ],
      "Interlingua": [
        ""
      ],
      "Iban": [
        ""
      ],
      "Ibibio": [
        ""
      ],
      "Indonihyia kasa": [
        ""
      ],
      "Interlingue": [
        ""
      ],
      "Igbo": [
        ""
      ],
      "Sichuan Yi": [
        ""
      ],
      "Inupiaq": [
        ""
      ],
      "Iloko": [
        ""
      ],
      "Ingush": [
        ""
      ],
      "Ido": [
        ""
      ],
      "Icelandic": [
        ""
      ],
      "Italy kasa": [
        ""
      ],
      "Inuktitut": [
        ""
      ],
      "Ingrian": [
        ""
      ],
      "Gyapan kasa": [
        ""
      ],
      "Jamaican Creole English": [
        ""
      ],
      "Lojban": [
        ""
      ],
      "Ngomba": [
        ""
      ],
      "Machame": [
        ""
      ],
      "Judeo-Persian": [
        ""
      ],
      "Judeo-Arabic": [
        ""
      ],
      "Jutish": [
        ""
      ],
      "Gyabanis kasa": [
        ""
      ],
      "Kara-Kalpak": [
        ""
      ],
      "Kabyle": [
        ""
      ],
      "Kachin": [
        ""
      ],
      "Jju": [
        ""
      ],
      "Kamba": [
        ""
      ],
      "Kawi": [
        ""
      ],
      "Kabardian": [
        ""
      ],
      "Kanembu": [
        ""
      ],
      "Tyap": [
        ""
      ],
      "Makonde": [
        ""
      ],
      "Kabuverdianu": [
        ""
      ],
      "Kenyang": [
        ""
      ],
      "Koro": [
        ""
      ],
      "Kongo": [
        ""
      ],
      "Kaingang": [
        ""
      ],
      "Khasi": [
        ""
      ],
      "Khotanese": [
        ""
      ],
      "Koyra Chiini": [
        ""
      ],
      "Khowar": [
        ""
      ],
      "Kikuyu": [
        ""
      ],
      "Kirmanjki": [
        ""
      ],
      "Kuanyama": [
        ""
      ],
      "Kako": [
        ""
      ],
      "Kazakh": [
        ""
      ],
      "Kalaallisut": [
        ""
      ],
      "Kalenjin": [
        ""
      ],
      "Kimbundu": [
        ""
      ],
      "Kambodia kasa": [
        ""
      ],
      "Kannada": [
        ""
      ],
      "Komi-Permyak": [
        ""
      ],
      "Konkani": [
        ""
      ],
      "Korea kasa": [
        ""
      ],
      "Kosraean": [
        ""
      ],
      "Kpelle": [
        ""
      ],
      "Karachay-Balkar": [
        ""
      ],
      "Krio": [
        ""
      ],
      "Kinaray-a": [
        ""
      ],
      "Kanuri": [
        ""
      ],
      "Karelian": [
        ""
      ],
      "Kurukh": [
        ""
      ],
      "Shambala": [
        ""
      ],
      "Bafia": [
        ""
      ],
      "Colognian": [
        ""
      ],
      "Kashmiri": [
        ""
      ],
      "Kurdish": [
        ""
      ],
      "Kumyk": [
        ""
      ],
      "Kutenai": [
        ""
      ],
      "Komi": [
        ""
      ],
      "Cornish": [
        ""
      ],
      "Ladino": [
        "Завантаження"
      ],
      "Langi": [
        ""
      ],
      "Lahnda": [
        ""
      ],
      "Latin": [
        ""
      ],
      "Lamba": [
        ""
      ],
      "Luxembourgish": [
        ""
      ],
      "Lezghian": [
        ""
      ],
      "Lingua Franca Nova": [
        ""
      ],
      "Ganda": [
        ""
      ],
      "Ligurian": [
        ""
      ],
      "Limburgish": [
        ""
      ],
      "Livonian": [
        ""
      ],
      "Lakota": [
        ""
      ],
      "Lombard": [
        ""
      ],
      "Lingala": [
        ""
      ],
      "Lao": [
        ""
      ],
      "Mongo": [
        ""
      ],
      "Lozi": [
        ""
      ],
      "Latgalian": [
        ""
      ],
      "Luba-Lulua": [
        ""
      ],
      "Luiseno": [
        ""
      ],
      "Luba-Katanga": [
        ""
      ],
      "Lunda": [
        ""
      ],
      "Luo": [
        ""
      ],
      "Mizo": [
        ""
      ],
      "Luyia": [
        ""
      ],
      "Literary Chinese": [
        ""
      ],
      "Laz": [
        ""
      ],
      "Madurese": [
        "скинути"
      ],
      "Mafa": [
        ""
      ],
      "Magahi": [
        ""
      ],
      "Maithili": [
        ""
      ],
      "Makasar": [
        ""
      ],
      "Mandingo": [
        "Завантаження"
      ],
      "Masai": [
        ""
      ],
      "Maba": [
        ""
      ],
      "Moksha": [
        ""
      ],
      "Mandar": [
        ""
      ],
      "Mende": [
        ""
      ],
      "Meru": [
        ""
      ],
      "Morisyen": [
        ""
      ],
      "Middle Irish": [
        ""
      ],
      "Makhuwa-Meetto": [
        ""
      ],
      "Metaʼ": [
        ""
      ],
      "Micmac": [
        ""
      ],
      "Maori": [
        ""
      ],
      "Minangkabau": [
        ""
      ],
      "Malayalam": [
        ""
      ],
      "Manchu": [
        ""
      ],
      "Manipuri": [
        ""
      ],
      "Mohawk": [
        ""
      ],
      "Mossi": [
        ""
      ],
      "Western Mari": [
        ""
      ],
      "Marathi": [
        ""
      ],
      "Malay kasa": [
        ""
      ],
      "Mundang": [
        ""
      ],
      "Multiple Languages": [
        "Поточна мова"
      ],
      "Creek": [
        ""
      ],
      "Mirandese": [
        ""
      ],
      "Marwari": [
        ""
      ],
      "Mentawai": [
        ""
      ],
      "Bɛɛmis kasa": [
        ""
      ],
      "Myene": [
        ""
      ],
      "Erzya": [
        ""
      ],
      "Mazanderani": [
        ""
      ],
      "Min Nan Chinese": [
        ""
      ],
      "Neapolitan": [
        ""
      ],
      "Nama": [
        ""
      ],
      "Norwegian Bokmål": [
        ""
      ],
      "North Ndebele": [
        ""
      ],
      "Low German": [
        ""
      ],
      "Nɛpal kasa": [
        ""
      ],
      "Newari": [
        ""
      ],
      "Ndonga": [
        ""
      ],
      "Nias": [
        ""
      ],
      "Niuean": [
        ""
      ],
      "Ao Naga": [
        ""
      ],
      "Flemish": [
        ""
      ],
      "Dɛɛkye": [
        ""
      ],
      "Kwasio": [
        ""
      ],
      "Ngiemboon": [
        ""
      ],
      "Norwegian Nynorsk": [
        ""
      ],
      "Nogai": [
        ""
      ],
      "Old Norse": [
        ""
      ],
      "Novial": [
        ""
      ],
      "NʼKo": [
        ""
      ],
      "South Ndebele": [
        ""
      ],
      "Northern Sotho": [
        ""
      ],
      "Nuer": [
        ""
      ],
      "Navajo": [
        ""
      ],
      "Classical Newari": [
        ""
      ],
      "Nyamwezi": [
        ""
      ],
      "Nyankole": [
        ""
      ],
      "Nyanja": [
        ""
      ],
      "Nyoro": [
        ""
      ],
      "Nzima": [
        ""
      ],
      "Occitan": [
        ""
      ],
      "Ojibwa": [
        ""
      ],
      "Oromo": [
        ""
      ],
      "Oriya": [
        ""
      ],
      "Osage": [
        ""
      ],
      "Ossetic": [
        ""
      ],
      "Ottoman Turkish": [
        ""
      ],
      "Pangasinan": [
        ""
      ],
      "Pahlavi": [
        ""
      ],
      "Pampanga": [
        ""
      ],
      "Papiamento": [
        "Оплата"
      ],
      "Pungyabi kasa": [
        ""
      ],
      "Picard": [
        ""
      ],
      "Pennsylvania German": [
        ""
      ],
      "Plautdietsch": [
        ""
      ],
      "Old Persian": [
        ""
      ],
      "Palatine German": [
        ""
      ],
      "Phoenician": [
        ""
      ],
      "Pali": [
        ""
      ],
      "Pɔland kasa": [
        ""
      ],
      "Piedmontese": [
        ""
      ],
      "Pontic": [
        ""
      ],
      "Pohnpeian": [
        ""
      ],
      "Prussian": [
        ""
      ],
      "Old Provençal": [
        ""
      ],
      "Pashto": [
        ""
      ],
      "Brazilian Portuguese": [
        ""
      ],
      "Pɔɔtugal kasa": [
        ""
      ],
      "European Portuguese": [
        ""
      ],
      "Kʼicheʼ": [
        ""
      ],
      "Chimborazo Highland Quichua": [
        ""
      ],
      "Quechua": [
        ""
      ],
      "Rajasthani": [
        ""
      ],
      "Rapanui": [
        ""
      ],
      "Rarotongan": [
        ""
      ],
      "Romagnol": [
        ""
      ],
      "Riffian": [
        ""
      ],
      "Romansh": [
        ""
      ],
      "Rundi": [
        ""
      ],
      "Rombo": [
        ""
      ],
      "Moldavian": [
        ""
      ],
      "Romany": [
        ""
      ],
      "Root": [
        ""
      ],
      "Romenia kasa": [
        ""
      ],
      "Rotuman": [
        ""
      ],
      "Rusyn": [
        ""
      ],
      "Roviana": [
        ""
      ],
      "Aromanian": [
        ""
      ],
      "Rahyia kasa": [
        ""
      ],
      "Rwa": [
        ""
      ],
      "Rewanda kasa": [
        ""
      ],
      "Sandawe": [
        ""
      ],
      "Sakha": [
        ""
      ],
      "Samaritan Aramaic": [
        ""
      ],
      "Samburu": [
        ""
      ],
      "Sanskrit": [
        ""
      ],
      "Sasak": [
        ""
      ],
      "Santali": [
        ""
      ],
      "Saurashtra": [
        ""
      ],
      "Ngambay": [
        ""
      ],
      "Sangu": [
        ""
      ],
      "Sicilian": [
        ""
      ],
      "Scots": [
        ""
      ],
      "Sardinian": [
        ""
      ],
      "Sassarese Sardinian": [
        ""
      ],
      "Sindhi": [
        ""
      ],
      "Seneca": [
        ""
      ],
      "Sena": [
        ""
      ],
      "Seri": [
        ""
      ],
      "Selkup": [
        ""
      ],
      "Northern Sami": [
        ""
      ],
      "Koyraboro Senni": [
        ""
      ],
      "Old Irish": [
        ""
      ],
      "Sango": [
        ""
      ],
      "Samogitian": [
        ""
      ],
      "Tachelhit": [
        ""
      ],
      "Shan": [
        ""
      ],
      "Serbo-Croatian": [
        ""
      ],
      "Chadian Arabic": [
        ""
      ],
      "Sidamo": [
        ""
      ],
      "Sinhala": [
        ""
      ],
      "Slovak": [
        ""
      ],
      "Lower Silesian": [
        ""
      ],
      "Selayar": [
        ""
      ],
      "Southern Sami": [
        ""
      ],
      "Lule Sami": [
        ""
      ],
      "Inari Sami": [
        ""
      ],
      "Skolt Sami": [
        ""
      ],
      "Soninke": [
        ""
      ],
      "Shona": [
        ""
      ],
      "Sogdien": [
        ""
      ],
      "Somalia kasa": [
        ""
      ],
      "Sranan Tongo": [
        ""
      ],
      "Serer": [
        ""
      ],
      "Swati": [
        ""
      ],
      "Saho": [
        ""
      ],
      "Saterland Frisian": [
        ""
      ],
      "Southern Sotho": [
        ""
      ],
      "Sukuma": [
        ""
      ],
      "Susu": [
        ""
      ],
      "Sundanese": [
        ""
      ],
      "Sumerian": [
        ""
      ],
      "Sweden kasa": [
        ""
      ],
      "Comorian": [
        ""
      ],
      "Congo Swahili": [
        ""
      ],
      "Swahili": [
        ""
      ],
      "Classical Syriac": [
        ""
      ],
      "Syriac": [
        ""
      ],
      "Silesian": [
        ""
      ],
      "Tamil kasa": [
        ""
      ],
      "Tulu": [
        ""
      ],
      "Timne": [
        ""
      ],
      "Teso": [
        ""
      ],
      "Tereno": [
        ""
      ],
      "Telugu": [
        ""
      ],
      "Tetum": [
        ""
      ],
      "Taeland kasa": [
        ""
      ],
      "Tigre": [
        ""
      ],
      "Tigrinya": [
        ""
      ],
      "Tiv": [
        "Чайові"
      ],
      "Tokelau": [
        ""
      ],
      "Tsakhur": [
        ""
      ],
      "Turkmen": [
        ""
      ],
      "Klingon": [
        ""
      ],
      "Tlingit": [
        ""
      ],
      "Tagalog": [
        ""
      ],
      "Talysh": [
        ""
      ],
      "Tamashek": [
        ""
      ],
      "Tswana": [
        ""
      ],
      "Nyasa Tonga": [
        ""
      ],
      "Tok Pisin": [
        ""
      ],
      "Tɛɛki kasa": [
        ""
      ],
      "Turoyo": [
        ""
      ],
      "Taroko": [
        ""
      ],
      "Tsakonian": [
        ""
      ],
      "Tsimshian": [
        ""
      ],
      "Tsonga": [
        ""
      ],
      "Tatar": [
        ""
      ],
      "Muslim Tat": [
        ""
      ],
      "Tumbuka": [
        ""
      ],
      "Tasawaq": [
        ""
      ],
      "Twi": [
        "Чайові"
      ],
      "Tahitian": [
        ""
      ],
      "Tuvinian": [
        ""
      ],
      "Central Atlas Tamazight": [
        ""
      ],
      "Udmurt": [
        ""
      ],
      "Ugaritic": [
        ""
      ],
      "Uyghur": [
        ""
      ],
      "Ukren kasa": [
        ""
      ],
      "Umbundu": [
        ""
      ],
      "Unknown Language": [
        "Поточна мова"
      ],
      "Urdu kasa": [
        ""
      ],
      "Uzbek": [
        ""
      ],
      "Vai": [
        ""
      ],
      "Venetian": [
        ""
      ],
      "Veps": [
        ""
      ],
      "Venda": [
        ""
      ],
      "Viɛtnam kasa": [
        ""
      ],
      "West Flemish": [
        ""
      ],
      "Main-Franconian": [
        ""
      ],
      "Votic": [
        ""
      ],
      "Volapük": [
        ""
      ],
      "Võro": [
        ""
      ],
      "Vunjo": [
        ""
      ],
      "Walser": [
        ""
      ],
      "Wolaytta": [
        ""
      ],
      "Waray": [
        ""
      ],
      "Washo": [
        ""
      ],
      "Walloon": [
        ""
      ],
      "Warlpiri": [
        ""
      ],
      "Wolof": [
        ""
      ],
      "Wu Chinese": [
        ""
      ],
      "Kalmyk": [
        ""
      ],
      "Xhosa": [
        ""
      ],
      "Mingrelian": [
        ""
      ],
      "Soga": [
        ""
      ],
      "Yao": [
        ""
      ],
      "Yapese": [
        ""
      ],
      "Yangben": [
        ""
      ],
      "Yemba": [
        ""
      ],
      "Yiddish": [
        ""
      ],
      "Yoruba": [
        ""
      ],
      "Nheengatu": [
        ""
      ],
      "Cantonese": [
        ""
      ],
      "Zapotec": [
        ""
      ],
      "Zhuang": [
        ""
      ],
      "Blissymbols": [
        ""
      ],
      "Zeelandic": [
        ""
      ],
      "Zenaga": [
        ""
      ],
      "Standard Moroccan Tamazight": [
        ""
      ],
      "Simplified Chinese": [
        ""
      ],
      "Traditional Chinese": [
        ""
      ],
      "Kyaena kasa": [
        ""
      ],
      "Zuni": [
        ""
      ],
      "Zulu": [
        ""
      ],
      "No linguistic content": [
        ""
      ],
      "Zaza": [
        ""
      ],
      "Could not load user attention request": [
        "Не вдалося завантажити комісії обмінника"
      ],
      "No notification left": [
        ""
      ],
      "Ignore": [
        ""
      ],
      "Invalid dev experiment URI.": [
        ""
      ],
      "URI: %1$s": [
        "%1$s"
      ],
      "Experiment": [
        ""
      ],
      "Insert lost denomination": [
        "Номінал"
      ],
      "Apply": [
        ""
      ],
      "Pending refresh": [
        "Очікувані операції"
      ],
      "Could not load the transfer payment status": [
        "Не вдалося завантажити статус оплати"
      ],
      "Could not load the status of the order template": [
        "Не вдалося завантажити список транзакцій"
      ],
      "required": [
        ""
      ],
      "Review order": [
        ""
      ],
      "Missing URI": [
        ""
      ],
      "Check the link.": [
        "Натисніть на іконку пазлу"
      ],
      "Could not parse the recovery URI": [
        "Не вдалося розібрати URI для оплати"
      ],
      "Try another URI": [
        "Спробуйте інший"
      ],
      "Import backup, show info": [
        "Імпортувати резервну копію, показати інформацію"
      ],
      "Add new exchange": [
        "Додати новий обмінник"
      ],
      "Add exchange for %1$s": [
        "Додати обмінник для %1$s"
      ],
      "Enter the URL of an exchange you trust.": [
        "Введіть URL обмінника, якому ви довіряєте."
      ],
      "An exchange has been found! Review the information and click next": [
        "Знайдено обмінник! Перегляньте інформацію та натисніть \"Далі\""
      ],
      "This exchange is already in your list.": [
        "Обмінник готовий до зняття коштів"
      ],
      "Only exchange accessible through \"http\" and \"https\" are allowed.": [
        ""
      ],
      "This exchange protocol version is not supported: \" %1$s\".": [
        ""
      ],
      "This exchange currency \"%1$s\" doesn&apos;t match the expected currency %2$s.": [
        "Цей обмінник не відповідає очікуваній валюті %1$s"
      ],
      "No exchange found in that URL.": [
        "Не знайдено обмінника для %1$s"
      ],
      "loading": [
        "завантаження"
      ],
      "Version": [
        "Версія"
      ],
      "Can't use the URL: \"%1$s\"": [
        ""
      ],
      "Exchange URL": [
        "URL обмінника"
      ],
      "Add new contact": [
        "Додати інший рахунок"
      ],
      "Enter the details of a contact you trust.": [
        "Введіть URL обмінника, якому ви довіряєте."
      ],
      "Contact info valid.": [
        ""
      ],
      "This contact is already in your list.": [
        ""
      ],
      "Review contact": [
        ""
      ],
      "Contact alias": [
        ""
      ],
      "Alias type": [
        ""
      ],
      "Mailbox service": [
        ""
      ],
      "Mailbox address": [
        "Адреса Bitcoin"
      ],
      "Contact source": [
        ""
      ],
      "Could not load the list of contacts": [
        "Не вдалося завантажити список транзакцій"
      ],
      "Source": [
        ""
      ],
      "Send Cash": [
        "Переказати %1$s"
      ],
      "Request Cash": [
        ""
      ],
      "No contacts found.": [
        ""
      ],
      "Could not bank choices for account": [
        "Не вдалося завантажити інформацію для відновлення резервної копії"
      ],
      "Could not load the list of messages": [
        "Не вдалося завантажити список відомих обмінників"
      ],
      "Unexpected error when trying to initialize mailbox: %1$s": [
        ""
      ],
      "Unexpected error when trying to delete message: %1$s": [
        ""
      ],
      "Unexpected error when trying to fetch messages: %1$s": [
        ""
      ],
      "Date": [
        "Дата"
      ],
      "You may copy your mailbox URI from below and register it at %1$s with your alias(es) to make it easier for others to find you.": [
        ""
      ],
      "copy uri": [
        "копіювати"
      ],
      "hide uri": [
        "сховати qr"
      ],
      "show uri": [
        "показати qr"
      ],
      "Fetch messages": [
        "Повідомлення продавця"
      ],
      "Mailbox not initialized": [
        ""
      ],
      "Initialize mailbox": [
        ""
      ],
      "No messages yet.": [
        "Ще немає обмінника"
      ],
      "Taler URI handler": [
        ""
      ],
      "Could not found a handler for the Taler URI": [
        "не вдалося знайти жодного обмінника для валюти %1$s"
      ],
      "The uri read in the path parameter is not valid: \"%1$s\"": [
        ""
      ],
      "Digital cash payment": [
        "Оплата електронними грошима"
      ],
      "Digital cash refund": [
        "Повернення електронних грошей"
      ],
      "Digital cash withdrawal": [
        "Зняття електронних грошей"
      ],
      "Digital cash deposit": [
        "Депозит електронних грошей"
      ],
      "Digital cash invoice": [
        "Електронний рахунок-фактура"
      ],
      "Digital cash transfer": [
        "Переказ електронних грошей"
      ],
      "Digital cash recovery": [
        "Відновлення цифрового гаманця"
      ],
      "Development experiment": [
        "Режим розробника"
      ],
      "Return to wallet": [
        ""
      ],
      "Network is offline": [
        ""
      ],
      "Diagnostics timed out. Could not talk to the wallet backend.": [
        "Час діагностики вичерпано. Не вдалося зв'язатися з бекендом гаманця."
      ],
      "Problems detected:": [
        "Виявлено проблеми:"
      ],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).": [
        "Будь ласка, перевірте у налаштуваннях %1$s, що у вас увімкнено IndexedDB (перевірте назву налаштування %2$s)."
      ],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.": [
        "База даних вашого гаманця застаріла. Наразі автоматична міграція не підтримується. Будь ласка, перейдіть до %1$s, щоб скинути базу даних гаманця."
      ],
      "here": [
        ""
      ],
      "Running diagnostics": [
        "Запуск діагностики"
      ],
      "Edit": [
        "Редагувати"
      ],
      "List of products": [
        "Список продуктів"
      ],
      "free": [
        "безкоштовно"
      ],
      "Could not get provider information": [
        "Не вдалося отримати інформацію про постачальника"
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=3; plural=n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2;",
  "lang": "uk",
  "completeness": 22
};

strings['tr'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "tr"
      },
      "Balance": [
        "Bakiye"
      ],
      "Backup": [
        "Yedekle"
      ],
      "Notifications": [
        ""
      ],
      "QR Reader and Taler URI": [
        "QR Okuyucu ve Taler URI'si"
      ],
      "Settings": [
        "Ayarlar"
      ],
      "Dev tools": [
        "Hata ayıklama araçları"
      ],
      "Contacts": [
        ""
      ],
      "Mailbox": [
        ""
      ],
      "Search transactions": [
        "İşlem ücretleri"
      ],
      "%1$s": [
        "%1$s"
      ],
      "KYC requirement": [
        ""
      ],
      "PENDING OPERATIONS": [
        "BEKLEYEN İŞLEMLER"
      ],
      "Error was thrown trying to: \"%1$s\"": [
        ""
      ],
      "Could not open the wallet database. Firefox is known to run into this problem under \"permanent private mode\".": [
        ""
      ],
      "Could not open the wallet database.": [
        "Para çekme işlemi için ayrıntılar alınamadı:"
      ],
      "Unexpected request error, code: %1$s": [
        ""
      ],
      "more info": [
        ""
      ],
      "wrap text": [
        ""
      ],
      "copy content": [
        ""
      ],
      "less info": [
        ""
      ],
      "Retry operation": [
        "Bekleyen işlemler"
      ],
      "Loading": [
        "Yükleniyor"
      ],
      "Could not load backup providers": [
        "Yedekleme sağlayıcıları yüklenemedi"
      ],
      "No backup providers configured": [
        "Yapılandırılmış yedekleme sağlayıcısı yok"
      ],
      "Add provider": [
        "Sağlayıcı ekle"
      ],
      "Sync all backups": [
        "Tüm yedeklemeleri senkronize et"
      ],
      "Sync now": [
        "Şimdi senkronize et"
      ],
      "Last synced": [
        "Son Senkronizasyon"
      ],
      "Not synced": [
        "Senkronize Edilmedi"
      ],
      "Expires in": [
        "İçinde sona eriyor"
      ],
      "show less": [
        "qr'yi göster"
      ],
      "show more": [
        "qr'yi göster"
      ],
      "There was an error loading the provider detail for &quot;%1$s&quot;": [
        "&quot;%1$s&quot; için sağlayıcı ayrıntısı yüklenirken bir hata oluştu"
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [
        "&quot;%1$s&quot; URL'sine sahip bilinen bir sağlayıcı yok."
      ],
      "See providers": [
        "Sağlayıcıları gör"
      ],
      "Last backup": [
        "Son yedek"
      ],
      "Back up": [
        "Yedekle"
      ],
      "Provider fee": [
        "Sağlayıcı ücreti"
      ],
      "per year": [
        "yıllık"
      ],
      "Extend": [
        "Uzat"
      ],
      "terms has changed, extending the service will imply accepting the new terms of service": [
        "şartlar değişti; hizmetin uzatılması yeni hizmet şartlarının kabul edilmesi anlamına gelecektir"
      ],
      "old": [
        "eski"
      ],
      "new": [
        "yeni"
      ],
      "fee": [
        "ücret"
      ],
      "storage": [
        "depolama"
      ],
      "Remove provider": [
        "Sağlayıcıyı kaldır"
      ],
      "Pay": [
        "Ödeme gönderildi"
      ],
      "Withdraw": [
        "Para çek"
      ],
      "This provider has reported an error": [
        "Sağlayıcı bir hata bildirdi"
      ],
      "There is conflict with another backup from &quot;%1$s&quot;": [
        "%1$s başka bir yedekle çakışıyor"
      ],
      "Backup is not readable": [
        "Yedek okunamıyor"
      ],
      "Unknown backup problem: %1$s": [
        "Bilinmeyen yedekleme problemi: %1$s"
      ],
      "service paid": [
        "hizmet ödendi"
      ],
      "Backup valid until": [
        "Yedekleme geçerlilik süresi"
      ],
      "Cancel": [
        "İptal"
      ],
      "Open pay page": [
        "Ödeme sayfasını açın"
      ],
      "Open refund page": [
        "Geri ödeme sayfasını açın"
      ],
      "Open withdraw page": [
        "Para çekme sayfasını açın"
      ],
      "Your wallet is empty.": [
        "Cüzdanınızda kayıtlı bir haraket yok."
      ],
      "Get digital cash": [
        "Dijital para alın"
      ],
      "Try the demo bank and withdraw test money.": [
        ""
      ],
      "Could not load the balance": [
        "Bakiye sayfası yüklenemedi"
      ],
      "Receive": [
        "Fiş"
      ],
      "Send %1$s": [
        "%1$s gönder"
      ],
      "This page has a withdrawal action.": [
        "Bu sayfada para çekme eylemi var."
      ],
      "This page has pay action.": [
        "Bu sayfada ödeme eylemi var."
      ],
      "This page has a refund action.": [
        "Bu sayfada bir geri ödeme eylemi var."
      ],
      "This page has a add exchange action.": [
        "Bu sayfada bir geri ödeme eylemi var."
      ],
      "Open add exchange page": [
        "Geri ödeme sayfasını açın"
      ],
      "This page has an add contact action.": [
        "Bu sayfada bir geri ödeme eylemi var."
      ],
      "Open add contact page": [
        "Ödeme sayfasını açın"
      ],
      "Taler Action": [
        "Taler Eylemi"
      ],
      "This page has a malformed taler uri.": [
        "Bu sayfada hatalı biçimlendirilmiş taler uri var."
      ],
      "Dismiss": [
        "Reddet"
      ],
      "this popup is being closed and you are being redirected to %1$s": [
        "bu açılır pencere kapatılıyor ve %1$s adresine yönlendiriliyorsunuz"
      ],
      "Could not load purchase proposal details": [
        "Satın alma teklifi ayrıntıları yüklenemedi"
      ],
      "Order Id": [
        "Sipariş kimliği"
      ],
      "Summary": [
        "Özet"
      ],
      "Amount": [
        "Miktar"
      ],
      "Choices": [
        ""
      ],
      "This purchase have %1$s choices": [
        ""
      ],
      "Merchant name": [
        "Satıcı adı"
      ],
      "Merchant jurisdiction": [
        "Satıcının yetki alanı"
      ],
      "Merchant address": [
        "Satıcı adresi"
      ],
      "Merchant logo": [
        "Satıcı logosu"
      ],
      "Merchant website": [
        "Satıcı web sitesi"
      ],
      "Merchant email": [
        "Satıcı e-postası"
      ],
      "Merchant public key": [
        "Satıcı genel anahtarı"
      ],
      "Created at": [
        "Oluşturulma"
      ],
      "Refund deadline": [
        "Geri ödeme son tarihi"
      ],
      "Auto refund": [
        "Otomatik geri ödeme"
      ],
      "Pay deadline": [
        "Son ödeme tarihi"
      ],
      "Fulfillment URL": [
        "Gönderim URL'si"
      ],
      "Fulfillment message": [
        "Gönderim mesajı"
      ],
      "Max deposit fee": [
        "Azami para yatırma ücreti"
      ],
      "Exchanges": [
        "Borsalar"
      ],
      "Could not load the status of the term of service": [
        "Bakiye sayfası yüklenemedi"
      ],
      "Select one option": [
        ""
      ],
      "Show terms of service": [
        "Hizmet şartlarını göster"
      ],
      "I accept the exchange terms of service": [
        "Hizmet şartlarını kabul ediyorum"
      ],
      "Review exchange terms of service": [
        "Exchange'in hizmet şartlarını inceleyin"
      ],
      "The exchange doesn't have a terms of service.": [
        "Exchange'in hizmet şartları yok"
      ],
      "Skip it for now.": [
        ""
      ],
      "Format": [
        ""
      ],
      "Language": [
        ""
      ],
      "The exchange replied with a empty terms of service": [
        "Exchange'in hizmet şartları yok"
      ],
      "No terms of service. The exchange replied with a empty document": [
        ""
      ],
      "No terms of service. The exchange replied with a empty text": [
        ""
      ],
      "Download Terms of Service": [
        ""
      ],
      "Hide terms of service": [
        ""
      ],
      "Amount is not valid": [
        ""
      ],
      "Bank account": [
        "Banka hesabı"
      ],
      "Bitcoin address": [
        "Bitcoin adresi"
      ],
      "BIC/IBAN": [
        "IBAN"
      ],
      "IBAN": [
        "IBAN"
      ],
      "Bitcoin transfer details": [
        "Bitcoin transfer ayrıntıları"
      ],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.": [
        ""
      ],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts": [
        ""
      ],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC": [
        ""
      ],
      "Ethereum transfer details": [
        "Bitcoin transfer ayrıntıları"
      ],
      "You need to wire to the service provider account.": [
        ""
      ],
      "Make sure the amount show %1$s ETH": [
        ""
      ],
      "Bank transfer details": [
        ""
      ],
      "Account": [
        "Hesap"
      ],
      "Bank host": [
        ""
      ],
      "BIC": [
        ""
      ],
      "Exchange": [
        "Exchange"
      ],
      "Reserve Pub": [
        ""
      ],
      "Step 1:": [
        ""
      ],
      "Copy this code and paste it into the subject/purpose field in your banking app or bank website": [
        ""
      ],
      "Subject": [
        "Konu"
      ],
      "Step 2:": [
        ""
      ],
      "If you don't already have it in your banking favourites list, then copy and paste this IBAN and the name into the receiver fields in your banking app or website": [
        ""
      ],
      "Receiver name": [
        ""
      ],
      "Receiver postal code": [
        ""
      ],
      "Receiver town": [
        "Fiş"
      ],
      "Step 3:": [
        ""
      ],
      "Finish the wire transfer setting the amount in your banking app or website, then this withdrawal will proceed automatically.": [
        ""
      ],
      "Make sure ALL data is correct, including the subject; otherwise, the money will not arrive in this wallet. You can use the copy buttons (%1$s) to prevent typing errors or the \"payto://\" URI below to copy just one value.": [
        ""
      ],
      "Alternative if your bank already supports PayTo URI, you can use this %1$s link instead": [
        ""
      ],
      "Continue with banking app or website": [
        ""
      ],
      "Show QR code": [
        ""
      ],
      "Could not load transaction information": [
        "Yedekleme sağlayıcıları yüklenemedi"
      ],
      "Bank account confirmation required for the transaction to complete.": [
        ""
      ],
      "KYC check required for the transaction to complete.": [
        ""
      ],
      "No additional information has been provided.": [
        ""
      ],
      "Follow this link to the%1$s %2$s": [
        ""
      ],
      "This transaction is pending to complete.": [
        ""
      ],
      "Retry": [
        "Yeniden deneyin"
      ],
      "There was an error trying to complete the transaction.": [
        ""
      ],
      "This transaction was aborted.": [
        ""
      ],
      "This transaction failed.": [
        "İşlem ücretleri"
      ],
      "Caution!": [
        "Dikkat!"
      ],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.": [
        ""
      ],
      "Confirm": [
        "Onaylamak"
      ],
      "Doing a cancellation while the transaction still active might result in lost coins. Do you still want to cancel the transaction?": [
        ""
      ],
      "No": [
        ""
      ],
      "Yes": [
        ""
      ],
      "Abort": [
        ""
      ],
      "Resume": [
        ""
      ],
      "Suspend": [
        ""
      ],
      "Delete": [
        ""
      ],
      "Withdrawal": [
        "Çekildi"
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Select one of the accounts and use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Details": [
        "Detaylar"
      ],
      "Withdrawal incomplete.": [
        "Para çekme Ücreti"
      ],
      "If you have already sent money to the service provider account it will wire it back at %1$s": [
        ""
      ],
      "Payment": [
        "Ödeme gönderildi"
      ],
      "Refunds": [
        ""
      ],
      "%1$s %2$s on %3$s .": [
        ""
      ],
      "Refund is in progress.": [
        ""
      ],
      "Merchant created a refund for this order but was not automatically picked up.": [
        ""
      ],
      "Offer": [
        "Teklif"
      ],
      "Accept": [
        "İkramiye kabul edildi"
      ],
      "Confirmation code": [
        ""
      ],
      "Merchant": [
        "Satıcı"
      ],
      "Invoice ID": [
        "Fatura Kimliği"
      ],
      "Deposit": [
        "Yatır"
      ],
      "Wire transfer deadline.": [
        "Bitcoin transfer ayrıntıları"
      ],
      "Wire transfer still pending.": [
        ""
      ],
      "The service provider deadline to make the wire transfer is: %1$s": [
        ""
      ],
      "Wire transfer completed.": [
        ""
      ],
      " ": [
        ""
      ],
      "Transfer details": [
        "Bitcoin transfer ayrıntıları"
      ],
      "Wire transfer in progress.": [
        ""
      ],
      "Refresh": [
        "Ücreti yenile"
      ],
      "Refund": [
        ""
      ],
      "Purchase summary": [
        ""
      ],
      "Credit": [
        "Kredi"
      ],
      "Invoice": [
        "Fatura"
      ],
      "URI": [
        "URI"
      ],
      "Debit": [
        ""
      ],
      "Transfer": [
        "Transfer"
      ],
      "Lost": [
        ""
      ],
      "Reason": [
        ""
      ],
      "Denomination expired.": [
        "Bekleyen işlemler"
      ],
      "Denomination vanished.": [
        "Bekleyen işlemler"
      ],
      "Denomination is unoffered.": [
        "Bekleyen işlemler"
      ],
      "Fees": [
        ""
      ],
      "Total": [
        "Toplam"
      ],
      "Sent": [
        ""
      ],
      "Converted": [
        ""
      ],
      "Bank fee": [
        "Azami ücret"
      ],
      "Price": [
        "Fiyat"
      ],
      "Subtotal": [
        "Toplam"
      ],
      "Refunded": [
        "İade edildi"
      ],
      "Transfer identification": [
        "Bekleyen işlemler"
      ],
      "copy": [
        "kopyala"
      ],
      "hide qr": [
        "qr'yi gizle"
      ],
      "forward to contact": [
        ""
      ],
      "show qr": [
        "qr'yi göster"
      ],
      "Wire transfer need a confirmation. Go to the %1$s and check wire transfer operation to exchange account is complete.": [
        ""
      ],
      "bank site": [
        ""
      ],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins.": [
        ""
      ],
      "Exchange is ready to send the coins, withdrawal in progress.": [
        ""
      ],
      "Send &nbsp; %1$s": [
        "&nbsp;%1$s gönder"
      ],
      "Pay &nbsp; %1$s": [
        "%1$snbsp;öde"
      ],
      "Merchant message": [
        "Satıcı mesajı"
      ],
      "Balance is not enough.": [
        "Gösterecek bakiyeniz yok."
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which exchange and auditors the merchant trust.": [
        ""
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which wire methods the merchant accepts.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to pay for this contract which is restricted.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to spend right know. There are some coins that need to be refreshed.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s available.": [
        ""
      ],
      "Provider is missing the global fee configuration, this likely means it is misconfigured.": [
        ""
      ],
      "Balance looks like it should be enough, but doesn't cover all fees requested by the merchant and payment processor. Please ensure there is at least %1$s %2$s more balance in your wallet or ask your merchant to cover more of the fees.": [
        ""
      ],
      "Pay with a mobile phone": [
        "Cep telefonuyla ödeme yapın"
      ],
      "Hide QR": [
        "QR'yi gizle"
      ],
      "Scan the QR code or &nbsp; %1$s": [
        "QR kodunu tarayın veya&nbsp;%1$s"
      ],
      "click here": [
        "göstermek için tıklayın"
      ],
      "Could not load the payment and balance status": [
        "Ödeme durumu yüklenemedi"
      ],
      "Purchase": [
        "Satın al"
      ],
      "This purchase is age restricted.": [
        ""
      ],
      "Expires at": [
        "İçinde sona eriyor"
      ],
      "Already paid, you are going to be redirected to %1$s": [
        "Zaten ödeme yaptınız, %1$s adresine yönlendirileceksiniz"
      ],
      "Already paid": [
        "Zaten ödendi"
      ],
      "Already claimed": [
        "Zaten talep edilmiş"
      ],
      "Could not load the refund status": [
        "Bakiye sayfası yüklenemedi"
      ],
      "You&apos;ve ignored the refund.": [
        ""
      ],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [
        ""
      ],
      "Order amount": [
        "Ücreti yenile"
      ],
      "Could not load the list of exchanges": [
        "Bakiye sayfası yüklenemedi"
      ],
      "Could not load info from URI": [
        "Bakiye sayfası yüklenemedi"
      ],
      "Could not load the withdrawal details": [
        "Satın alma teklifi ayrıntıları yüklenemedi"
      ],
      "under %1$s": [
        "%1$s gönder"
      ],
      "should be greater than zero": [
        ""
      ],
      "can't be greater than %1$s": [
        ""
      ],
      "Could not load exchange details info": [
        "Bakiye sayfası yüklenemedi"
      ],
      "Close": [
        ""
      ],
      "There is no exchange available for currency %1$s": [
        ""
      ],
      "You can add more exchanges from the settings.": [
        ""
      ],
      "Exchange %1$s is not available": [
        ""
      ],
      "Exchange status can view accessed from the settings.": [
        ""
      ],
      "Service fee description": [
        ""
      ],
      "Select %1$s exchange": [
        "Özel exchange'i seçin"
      ],
      "Reset": [
        ""
      ],
      "Use this exchange": [
        "Özel exchange'i seçin"
      ],
      "Auditors": [
        "Denetçiler"
      ],
      "Doesn&apos;t have auditors": [
        ""
      ],
      "currency": [
        ""
      ],
      "Coin operations": [
        "Bekleyen işlemler"
      ],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.": [
        ""
      ],
      "Deposits": [
        "Depozito %1$s"
      ],
      "Denomination": [
        "Bekleyen işlemler"
      ],
      "Current": [
        ""
      ],
      "Selected": [
        ""
      ],
      "Until": [
        ""
      ],
      "Withdrawals": [
        "Çekildi"
      ],
      "Transfer operations": [
        "Bekleyen işlemler"
      ],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.": [
        ""
      ],
      "Operation": [
        ""
      ],
      "Fee": [
        "Ücretler"
      ],
      "Wallet operations": [
        "Bekleyen işlemler"
      ],
      "Feature": [
        ""
      ],
      "Currency": [
        ""
      ],
      "This operation has already been completed.": [
        ""
      ],
      "This operation has already been aborted": [
        ""
      ],
      "This operation has started and should be completed in the bank.": [
        ""
      ],
      "You can confirm the operation in": [
        "Para çekme işlemi için ayrıntılar alınamadı:"
      ],
      "this page": [
        "Özel exchange'i seçin"
      ],
      "This operation has already been completed by another wallet.": [
        ""
      ],
      "This operation has already been used by another wallet.": [
        ""
      ],
      "It can be confirmed in": [
        ""
      ],
      "Age restriction": [
        ""
      ],
      "Withdraw &nbsp; %1$s": [
        ""
      ],
      "Withdraw to a mobile phone": [
        ""
      ],
      "Short description of the invoice": [
        ""
      ],
      "Create": [
        ""
      ],
      "Short description of the transfer": [
        ""
      ],
      "Could not load the invoice payment status": [
        "Ödeme durumu yüklenemedi"
      ],
      "Receive &nbsp; %1$s": [
        ""
      ],
      "Could not load known bank accounts": [
        "Ödeme durumu yüklenemedi"
      ],
      "Could not load supported wire methods": [
        "Yedekleme sağlayıcıları yüklenemedi"
      ],
      "No wire methods supported for this currency": [
        ""
      ],
      "Try another wire method.": [
        ""
      ],
      "Known accounts for %1$s": [
        ""
      ],
      "To add a new account first select the account type.": [
        ""
      ],
      "Unable add this account": [
        ""
      ],
      "Add": [
        "Ekle"
      ],
      "IBAN accounts": [
        "Banka hesabı"
      ],
      "Alias": [
        ""
      ],
      "Int. Account Number": [
        ""
      ],
      "Account name": [
        "Hesap"
      ],
      "KYC": [
        ""
      ],
      "Taler accounts": [
        "Banka hesabı"
      ],
      "Host": [
        ""
      ],
      "Bitcoin accounts": [
        "Banka hesabı"
      ],
      "Address": [
        "Adres satırı"
      ],
      "Can't be empty": [
        ""
      ],
      "Bitcoin Account": [
        "Banka hesabı"
      ],
      "Taler Bank": [
        "Taler Eylemi"
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "Required": [
        ""
      ],
      "International Bank Account Number": [
        ""
      ],
      "Could not load balance information": [
        "Yedekleme sağlayıcıları yüklenemedi"
      ],
      "Could not load fee for amount %1$s": [
        "Bakiye sayfası yüklenemedi"
      ],
      "A currency or an amount should be indicated": [
        ""
      ],
      "There is no enough balance to make a deposit for currency %1$s": [
        ""
      ],
      "Send %1$s to your account": [
        ""
      ],
      "There is no account to make a deposit for currency %1$s": [
        ""
      ],
      "Add account": [
        ""
      ],
      "Select account": [
        ""
      ],
      "Manage accounts": [
        "Banka hesabı"
      ],
      "Brut amount": [
        "Ücreti yenile"
      ],
      "Deposit fee": [
        ""
      ],
      "Net amount": [
        "Depozito Ücreti"
      ],
      "Deposit&nbsp;%1$s %2$s": [
        "Test Havale Hesap #%1$s üzerinde %2$s"
      ],
      "Need approval in the Bank": [
        ""
      ],
      "Waiting for wire transfer to complete": [
        ""
      ],
      "Exchange is waiting the wire transfer": [
        ""
      ],
      "Payment in progress": [
        "Ödeme durduruldu"
      ],
      "Executing refund...": [
        "Otomatik geri ödeme"
      ],
      "Refreshing coins...": [
        ""
      ],
      "Deposit in progress": [
        ""
      ],
      "Waiting to be paid": [
        ""
      ],
      "Receiving the transfer": [
        ""
      ],
      "Waiting to be received": [
        "Alınacak"
      ],
      "Denomination expired": [
        "Bekleyen işlemler"
      ],
      "Denomination vanished": [
        "Bekleyen işlemler"
      ],
      "Denomination unoffered": [
        "Bekleyen işlemler"
      ],
      "ABORTED": [
        ""
      ],
      "FAILED": [
        ""
      ],
      "Could not load the list of transactions": [
        ""
      ],
      "unknown scope": [
        ""
      ],
      "Send": [
        "Göder"
      ],
      "Your transaction history is empty for this currency.": [
        ""
      ],
      "Review terms of service": [
        ""
      ],
      "Provider URL": [
        ""
      ],
      "Please review and accept this provider&apos;s terms of service": [
        "Hizmet şartlarını kabul ediyorum"
      ],
      "Pricing": [
        ""
      ],
      "free of charge": [
        ""
      ],
      "%1$s per year of service": [
        ""
      ],
      "Storage": [
        ""
      ],
      "%1$s megabytes of storage per year of service": [
        ""
      ],
      "Accept terms of service": [
        ""
      ],
      "Add backup provider": [
        ""
      ],
      "Backup providers may charge for their service": [
        ""
      ],
      "URL": [
        ""
      ],
      "Name": [
        ""
      ],
      "Next": [
        ""
      ],
      "Navigator": [
        ""
      ],
      "Automatically open wallet": [
        ""
      ],
      "Open the wallet when a payment action is found.": [
        ""
      ],
      "Version Info": [
        ""
      ],
      "Web Extension": [
        ""
      ],
      "The version of wallet core is not supported. (supported version: %1$s, wallet version: %2$s)": [
        ""
      ],
      "Exchange compatibility": [
        ""
      ],
      "Merchant compatibility": [
        ""
      ],
      "Bank compatibility": [
        ""
      ],
      "Wallet Core compatibility": [
        ""
      ],
      "Enable developer mode": [
        ""
      ],
      "Show more information and options in the UI": [
        ""
      ],
      "Display": [
        ""
      ],
      "Current Language": [
        ""
      ],
      "Show backup feature": [
        ""
      ],
      "Backup integration still in beta.": [
        ""
      ],
      "Show P2P alias feature": [
        ""
      ],
      "P2P aliases integration still in beta.": [
        ""
      ],
      "Show suspend/resume transaction": [
        ""
      ],
      "Prevent transaction from doing network request.": [
        ""
      ],
      "Show refresh transaction type in the transaction list": [
        ""
      ],
      "Refresh transaction will be hidden by default if the refresh operation doesn't have fee.": [
        ""
      ],
      "Show more account types on deposit": [
        ""
      ],
      "Extends the UI to more payment target types.": [
        ""
      ],
      "Show JSON on error": [
        ""
      ],
      "Print more information about the error. Useful for debugging.": [
        ""
      ],
      "Allow HTTP connections": [
        ""
      ],
      "Using HTTP connection may be faster but unsafe (wallet restart required)": [
        ""
      ],
      "Lang selector": [
        ""
      ],
      "Allows to manually change the language of the UI. Otherwise it will be automatically selected by your browser configuration.": [
        ""
      ],
      "Edit exchange management": [
        ""
      ],
      "Allows to see the list of exchange, remove, add and switch before withdrawal.": [
        ""
      ],
      "Select terms of service format": [
        "Hizmet şartlarını göster"
      ],
      "Allows to render the terms of service on different format selected by the user.": [
        ""
      ],
      "Show wallet activity": [
        ""
      ],
      "Show the wallet notification and observability event in the UI.": [
        ""
      ],
      "Enable v1 contract": [
        ""
      ],
      "v1 is under development and wallet-core converts v1 contract to v0. Enable this options for testing": [
        ""
      ],
      "GNU Taler Wallet installed!": [
        ""
      ],
      "You can open the wallet using the combination %1$s .": [
        ""
      ],
      "Also pinning the GNU Taler Wallet to your browser allows you to quick access without keyboard:": [
        ""
      ],
      "Click the puzzle icon": [
        ""
      ],
      "Search for GNU Taler Wallet": [
        ""
      ],
      "Click the pin icon": [
        ""
      ],
      "Next Steps": [
        ""
      ],
      "Try the demo": [
        ""
      ],
      "Click here to open the wallet activity tab.": [
        ""
      ],
      "Events": [
        ""
      ],
      "Active tasks": [
        ""
      ],
      "All events": [
        ""
      ],
      "Balance change": [
        "Bakiye"
      ],
      "Backup failed": [
        "Yedekleme geçerlilik süresi"
      ],
      "Transaction updated": [
        "İşlem ücretleri"
      ],
      "Exchange updated": [
        "Exchange"
      ],
      "Idle": [
        ""
      ],
      "task.%1$s": [
        "%1$s"
      ],
      "wallet.%1$s(%2$s)": [
        ""
      ],
      "Bank account info changed": [
        "Banka hesabı"
      ],
      "Contact added": [
        ""
      ],
      "Contact deleted": [
        ""
      ],
      "Mailbox message added": [
        ""
      ],
      "Mailbox message deleted": [
        ""
      ],
      "Type": [
        ""
      ],
      "Id": [
        ""
      ],
      "Since": [
        ""
      ],
      "Next try": [
        ""
      ],
      "Error": [
        ""
      ],
      "Transaction": [
        "İşlem ücretleri"
      ],
      "No exchange yet": [
        ""
      ],
      "Status": [
        "durum"
      ],
      "Terms of Service": [
        "Hizmet şartlarını göster"
      ],
      "Last Update": [
        ""
      ],
      "Actions": [
        "Taler Eylemi"
      ],
      "ok": [
        ""
      ],
      "pending": [
        ""
      ],
      "proposed": [
        ""
      ],
      "unknown (exchange status should be updated)": [
        ""
      ],
      "Reload": [
        ""
      ],
      "Purge": [
        ""
      ],
      "Make regional": [
        ""
      ],
      "Make global": [
        ""
      ],
      "Forget ToS": [
        "Unut"
      ],
      "Debug tools": [
        "Hata ayıklama araçları"
      ],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?": [
        "Cüzdanınızdaki her şeyi GERİ ALINAMAZ BİÇİMDE İMHA ETMEK ve TÜM PARALARINIZI KAYBETMEK mi istiyorsunuz?"
      ],
      "reset": [
        "sıfırla"
      ],
      "TESTING: This may delete all your coin, proceed with caution": [
        ""
      ],
      "run gc": [
        ""
      ],
      "import database": [
        "veritabanını içe aktar"
      ],
      "export database": [
        "veritabanını dışa aktar"
      ],
      "Clear exchange key cache": [
        ""
      ],
      "Database exported at %1$s %2$s to download": [
        "Veritabanı %1$s'de dışa aktarıldı-%2$s indirilecek"
      ],
      "Inject Taler support in all pages": [
        ""
      ],
      "Enabling this option will make `window.taler` be available in all sites": [
        ""
      ],
      "Exchange Entries": [
        "Borsalar"
      ],
      "Add an exchange": [
        ""
      ],
      "Log levels": [
        ""
      ],
      "Coins": [
        "Madeni paralar"
      ],
      "usable coins": [
        "kullanılabilir madeni paralar"
      ],
      "id": [
        "kimlik"
      ],
      "denom": [
        ""
      ],
      "status": [
        "durum"
      ],
      "from refresh?": [
        "yenilemeden mi?"
      ],
      "age key count": [
        ""
      ],
      "spent coins": [
        "harcanan madeni paralar"
      ],
      "click to show": [
        "göstermek için tıklayın"
      ],
      "URI is not valid. Taler URI should start with \"taler://\"": [
        ""
      ],
      "After the URI type it should follow a '/' with more information.": [
        ""
      ],
      "This URI type is not supported": [
        ""
      ],
      "This URI requires the bank host and operation id separated by /": [
        ""
      ],
      "This URI requires the merchant host, order id and session id separated by /": [
        ""
      ],
      "This URI requires the merchant host and order id separated by /": [
        ""
      ],
      "This URI requires the exchange host and key id separated by /": [
        ""
      ],
      "This URI requires the merchant host and template id separated by /": [
        ""
      ],
      "This URI requires the host key and provider list separated by /": [
        ""
      ],
      "This URI requires the experiment id": [
        ""
      ],
      "This URI requires the exchange host": [
        ""
      ],
      "This URI requires the alias type, the alias and the mailbox URI separated by /": [
        ""
      ],
      "This URI requires string after the first /": [
        ""
      ],
      "The bank host is invalid": [
        ""
      ],
      "The merchant host is invalid": [
        ""
      ],
      "The URI should end with /": [
        ""
      ],
      "The exchange host is invalid": [
        ""
      ],
      "The contact is invalid": [
        ""
      ],
      "The amount is invalid": [
        ""
      ],
      "A parameter is invalid": [
        ""
      ],
      "checking...": [
        ""
      ],
      "Unexpected error happen reading the camera: %1$s": [
        ""
      ],
      "Could not found a QR code in the file": [
        ""
      ],
      "Unexpected error happen reading the file: %1$s": [
        ""
      ],
      "Scan a QR code or enter taler:// URI below": [
        ""
      ],
      "Pay invoice": [
        "Fatura"
      ],
      "Withdrawal from bank": [
        "Çekildi"
      ],
      "Claim refund": [
        "Ödeme iadesi"
      ],
      "Accept payment": [
        "İkramiye kabul edildi"
      ],
      "Complete order": [
        ""
      ],
      "Restore wallet": [
        ""
      ],
      "Enable experiment": [
        ""
      ],
      "Withdraw from exchange": [
        "Para çekme Ücreti"
      ],
      "Add exchange": [
        ""
      ],
      "Notify transaction": [
        ""
      ],
      "Add contact": [
        ""
      ],
      "The exchange URL is invalid.": [
        ""
      ],
      "The exchange URL should end with '/'": [
        ""
      ],
      "Couldn't found an exchange in the URL specified.": [
        ""
      ],
      "HTTP request failed to %1$s: %2$s": [
        ""
      ],
      "HTTP request failed to %1$s": [
        ""
      ],
      "The merchant URL is invalid.": [
        ""
      ],
      "The merchant URL should end with '/'": [
        ""
      ],
      "Couldn't found an merchant in the URL specified.": [
        ""
      ],
      "The bank URL is invalid.": [
        ""
      ],
      "The bank URL should end with '/'": [
        ""
      ],
      "Couldn't found an bank in the URL specified.": [
        ""
      ],
      "Could not load exchanges": [
        "Bakiye sayfası yüklenemedi"
      ],
      "Choose a currency to proceed or add another exchange": [
        ""
      ],
      "Known currencies": [
        ""
      ],
      "Specify the origin": [
        ""
      ],
      "Use previous origins:": [
        ""
      ],
      "From another bank account": [
        "Banka hesabına havale yap"
      ],
      "From another wallet": [
        ""
      ],
      "From a %1$s URI or QR code": [
        ""
      ],
      "Enter URI here": [
        "URI'yi girin"
      ],
      "Specify the destination": [
        ""
      ],
      "Use previous destinations:": [
        ""
      ],
      "To another bank account": [
        "Banka hesabına havale yap"
      ],
      "To another wallet": [
        ""
      ],
      "To a %1$s URI or QR code": [
        ""
      ],
      "Afghanistan": [
        ""
      ],
      "Albania": [
        ""
      ],
      "Algeria": [
        ""
      ],
      "United States": [
        ""
      ],
      "Andorra": [
        ""
      ],
      "Angola": [
        ""
      ],
      "Antigua and Barbuda": [
        ""
      ],
      "Argentina": [
        ""
      ],
      "Armenia": [
        ""
      ],
      "Australia": [
        ""
      ],
      "Austria": [
        ""
      ],
      "Azerbaijan": [
        ""
      ],
      "Bahamas": [
        ""
      ],
      "Bahrain": [
        ""
      ],
      "Bangladesh": [
        ""
      ],
      "Barbados": [
        ""
      ],
      "Botswana": [
        ""
      ],
      "Belarus": [
        ""
      ],
      "Belgium": [
        ""
      ],
      "Belize": [
        ""
      ],
      "Benin": [
        ""
      ],
      "Bhutan": [
        ""
      ],
      "Bolivia": [
        ""
      ],
      "Bosnia and Herzegovina": [
        ""
      ],
      "Brazil": [
        ""
      ],
      "United Kingdom": [
        ""
      ],
      "Brunei": [
        ""
      ],
      "Bulgaria": [
        ""
      ],
      "Burkina Faso": [
        ""
      ],
      "Myanmar": [
        ""
      ],
      "Burundi": [
        ""
      ],
      "Cambodia": [
        ""
      ],
      "Cameroon": [
        ""
      ],
      "Canada": [
        ""
      ],
      "Cape Verde": [
        ""
      ],
      "Central African Republic": [
        ""
      ],
      "Chad": [
        ""
      ],
      "Chile": [
        ""
      ],
      "China": [
        ""
      ],
      "Colombia": [
        ""
      ],
      "Comoros": [
        ""
      ],
      "Democratic Republic of the Congo": [
        ""
      ],
      "Republic of the Congo": [
        ""
      ],
      "Costa Rica": [
        ""
      ],
      "Croatia": [
        ""
      ],
      "Cuba": [
        ""
      ],
      "Cyprus": [
        ""
      ],
      "Czech Republic": [
        "Satıcı genel anahtarı"
      ],
      "Denmark": [
        ""
      ],
      "Djibouti": [
        ""
      ],
      "Dominican Republic": [
        ""
      ],
      "Netherlands": [
        ""
      ],
      "Timor-Leste": [
        ""
      ],
      "Ecuador": [
        ""
      ],
      "Egypt": [
        ""
      ],
      "United Arab Emirates": [
        ""
      ],
      "Equatorial Guinea": [
        ""
      ],
      "Eritrea": [
        ""
      ],
      "Estonia": [
        ""
      ],
      "Ethiopia": [
        ""
      ],
      "Fiji": [
        ""
      ],
      "Philippines": [
        ""
      ],
      "Finland": [
        ""
      ],
      "France": [
        "İptal"
      ],
      "Gabon": [
        ""
      ],
      "Gambia": [
        ""
      ],
      "Georgia": [
        ""
      ],
      "Germany": [
        ""
      ],
      "Ghana": [
        ""
      ],
      "Greece": [
        ""
      ],
      "Grenada": [
        ""
      ],
      "Guatemala": [
        ""
      ],
      "Guinea-Bissau": [
        ""
      ],
      "Guinea": [
        ""
      ],
      "Guyana": [
        ""
      ],
      "Haiti": [
        ""
      ],
      "Honduras": [
        ""
      ],
      "Hungary": [
        ""
      ],
      "Kiribati": [
        ""
      ],
      "Iceland": [
        ""
      ],
      "India": [
        ""
      ],
      "Indonesia": [
        ""
      ],
      "Iran": [
        ""
      ],
      "Iraq": [
        ""
      ],
      "Ireland": [
        ""
      ],
      "Israel": [
        ""
      ],
      "Italy": [
        ""
      ],
      "Ivory Coast": [
        ""
      ],
      "Jamaica": [
        ""
      ],
      "Japan": [
        ""
      ],
      "Jordan": [
        ""
      ],
      "Kazakhstan": [
        ""
      ],
      "Kenya": [
        ""
      ],
      "Saint Kitts and Nevis": [
        ""
      ],
      "Kuwait": [
        ""
      ],
      "Kyrgyzstan": [
        ""
      ],
      "Laos": [
        ""
      ],
      "Latvia": [
        ""
      ],
      "Lebanon": [
        ""
      ],
      "Liberia": [
        ""
      ],
      "Libya": [
        ""
      ],
      "Liechtenstein": [
        ""
      ],
      "Lithuania": [
        ""
      ],
      "Luxembourg": [
        ""
      ],
      "North Macedonia": [
        ""
      ],
      "Madagascar": [
        ""
      ],
      "Malawi": [
        ""
      ],
      "Malaysia": [
        ""
      ],
      "Maldives": [
        ""
      ],
      "Mali": [
        ""
      ],
      "Malta": [
        ""
      ],
      "Marshall Islands": [
        ""
      ],
      "Mauritania": [
        ""
      ],
      "Mauritius": [
        ""
      ],
      "Mexico": [
        ""
      ],
      "Micronesia": [
        ""
      ],
      "Moldova": [
        ""
      ],
      "Monaco": [
        ""
      ],
      "Mongolia": [
        ""
      ],
      "Morocco": [
        ""
      ],
      "Lesotho": [
        ""
      ],
      "Mozambique": [
        ""
      ],
      "Namibia": [
        ""
      ],
      "Nauru": [
        ""
      ],
      "Nepal": [
        ""
      ],
      "New Zealand": [
        ""
      ],
      "Vanuatu": [
        ""
      ],
      "Nicaragua": [
        ""
      ],
      "Nigeria": [
        ""
      ],
      "Niger": [
        ""
      ],
      "North Korea": [
        ""
      ],
      "Norway": [
        ""
      ],
      "Oman": [
        ""
      ],
      "Pakistan": [
        ""
      ],
      "Palau": [
        ""
      ],
      "Panama": [
        ""
      ],
      "Papua New Guinea": [
        ""
      ],
      "Paraguay": [
        ""
      ],
      "Peru": [
        ""
      ],
      "Poland": [
        ""
      ],
      "Portugal": [
        ""
      ],
      "Qatar": [
        ""
      ],
      "Romania": [
        ""
      ],
      "Russia": [
        ""
      ],
      "Rwanda": [
        ""
      ],
      "Saint Lucia": [
        ""
      ],
      "El Salvador": [
        ""
      ],
      "Samoa": [
        ""
      ],
      "San Marino": [
        ""
      ],
      "São Tomé and Príncipe": [
        ""
      ],
      "Saudi Arabia": [
        ""
      ],
      "Senegal": [
        ""
      ],
      "Serbia": [
        ""
      ],
      "Seychelles": [
        ""
      ],
      "Sierra Leone": [
        ""
      ],
      "Singapore": [
        ""
      ],
      "Slovakia": [
        ""
      ],
      "Slovenia": [
        ""
      ],
      "Solomon Islands": [
        ""
      ],
      "Somalia": [
        ""
      ],
      "South Africa": [
        ""
      ],
      "South Korea": [
        ""
      ],
      "Spain": [
        ""
      ],
      "Sri Lanka": [
        ""
      ],
      "Sudan": [
        ""
      ],
      "Suriname": [
        ""
      ],
      "Eswatini": [
        ""
      ],
      "Sweden": [
        ""
      ],
      "Switzerland": [
        ""
      ],
      "Syria": [
        ""
      ],
      "Taiwan": [
        ""
      ],
      "Tajikistan": [
        ""
      ],
      "Tanzania": [
        ""
      ],
      "Thailand": [
        ""
      ],
      "Togo": [
        ""
      ],
      "Tonga": [
        ""
      ],
      "Trinidad and Tobago": [
        ""
      ],
      "Tunisia": [
        ""
      ],
      "Turkey": [
        ""
      ],
      "Tuvalu": [
        "değer"
      ],
      "Uganda": [
        ""
      ],
      "Ukraine": [
        ""
      ],
      "Uruguay": [
        ""
      ],
      "Uzbekistan": [
        ""
      ],
      "Venezuela": [
        ""
      ],
      "Vietnam": [
        ""
      ],
      "Yemen": [
        ""
      ],
      "Zambia": [
        ""
      ],
      "Zimbabwe": [
        ""
      ],
      "Afghan": [
        ""
      ],
      "Albanian": [
        ""
      ],
      "Algerian": [
        ""
      ],
      "American": [
        ""
      ],
      "Andorran": [
        ""
      ],
      "Angolan": [
        ""
      ],
      "Antiguan": [
        ""
      ],
      "Argentinean": [
        ""
      ],
      "Armenian": [
        ""
      ],
      "Australian": [
        ""
      ],
      "Austrian": [
        ""
      ],
      "Azerbaijani": [
        ""
      ],
      "Bahamian": [
        ""
      ],
      "Bahraini": [
        ""
      ],
      "Bangladeshi": [
        ""
      ],
      "Barbadian": [
        ""
      ],
      "Batswana": [
        ""
      ],
      "Belarusian": [
        ""
      ],
      "Belgian": [
        ""
      ],
      "Belizean": [
        ""
      ],
      "Beninese": [
        ""
      ],
      "Bhutanese": [
        ""
      ],
      "Bolivian": [
        ""
      ],
      "Bosnian": [
        ""
      ],
      "Brazilian": [
        ""
      ],
      "British": [
        ""
      ],
      "Bruneian": [
        ""
      ],
      "Bulgarian": [
        ""
      ],
      "Burkinabe": [
        ""
      ],
      "Burmese": [
        ""
      ],
      "Burundian": [
        ""
      ],
      "Cambodian": [
        ""
      ],
      "Cameroonian": [
        ""
      ],
      "Canadian": [
        ""
      ],
      "Cape Verdean": [
        ""
      ],
      "Central African": [
        ""
      ],
      "Chadian": [
        ""
      ],
      "Chilean": [
        ""
      ],
      "Chinese": [
        ""
      ],
      "Colombian": [
        ""
      ],
      "Comoran": [
        ""
      ],
      "Congolese": [
        ""
      ],
      "Costa Rican": [
        ""
      ],
      "Croatian": [
        ""
      ],
      "Cuban": [
        ""
      ],
      "Cypriot": [
        ""
      ],
      "Czech": [
        ""
      ],
      "Danish": [
        ""
      ],
      "Djiboutian": [
        ""
      ],
      "Dominican": [
        "Bekleyen işlemler"
      ],
      "Dutch": [
        ""
      ],
      "East Timorese": [
        ""
      ],
      "Ecuadorean": [
        ""
      ],
      "Egyptian": [
        ""
      ],
      "Emirati": [
        ""
      ],
      "Equatorial Guinean": [
        ""
      ],
      "Eritrean": [
        ""
      ],
      "Estonian": [
        ""
      ],
      "Ethiopian": [
        ""
      ],
      "Fijian": [
        ""
      ],
      "Filipino": [
        ""
      ],
      "Finnish": [
        ""
      ],
      "French": [
        ""
      ],
      "Gabonese": [
        ""
      ],
      "Gambian": [
        ""
      ],
      "Georgian": [
        ""
      ],
      "German": [
        ""
      ],
      "Ghanaian": [
        ""
      ],
      "Greek": [
        ""
      ],
      "Grenadian": [
        ""
      ],
      "Guatemalan": [
        ""
      ],
      "Guinea-Bissauan": [
        ""
      ],
      "Guinean": [
        ""
      ],
      "Guyanese": [
        ""
      ],
      "Haitian": [
        ""
      ],
      "Honduran": [
        ""
      ],
      "Hungarian": [
        ""
      ],
      "I-Kiribati": [
        ""
      ],
      "Icelander": [
        ""
      ],
      "Indian": [
        ""
      ],
      "Indonesian": [
        ""
      ],
      "Iranian": [
        ""
      ],
      "Iraqi": [
        ""
      ],
      "Irish": [
        ""
      ],
      "Israeli": [
        ""
      ],
      "Italian": [
        ""
      ],
      "Ivorian": [
        ""
      ],
      "Jamaican": [
        ""
      ],
      "Japanese": [
        ""
      ],
      "Jordanian": [
        ""
      ],
      "Kazakhstani": [
        ""
      ],
      "Kenyan": [
        ""
      ],
      "Kittian and Nevisian": [
        ""
      ],
      "Kuwaiti": [
        ""
      ],
      "Kyrgyz": [
        ""
      ],
      "Laotian": [
        ""
      ],
      "Latvian": [
        ""
      ],
      "Lebanese": [
        ""
      ],
      "Liberian": [
        ""
      ],
      "Libyan": [
        ""
      ],
      "Liechtensteiner": [
        ""
      ],
      "Lithuanian": [
        ""
      ],
      "Luxembourger": [
        ""
      ],
      "Macedonian": [
        ""
      ],
      "Malagasy": [
        ""
      ],
      "Malawian": [
        ""
      ],
      "Malaysian": [
        ""
      ],
      "Maldivian": [
        ""
      ],
      "Malian": [
        ""
      ],
      "Maltese": [
        ""
      ],
      "Marshallese": [
        ""
      ],
      "Mauritanian": [
        ""
      ],
      "Mauritian": [
        ""
      ],
      "Mexican": [
        ""
      ],
      "Micronesian": [
        ""
      ],
      "Moldovan": [
        ""
      ],
      "Monacan": [
        ""
      ],
      "Mongolian": [
        ""
      ],
      "Moroccan": [
        ""
      ],
      "Mosotho": [
        ""
      ],
      "Mozambican": [
        ""
      ],
      "Namibian": [
        ""
      ],
      "Nauruan": [
        ""
      ],
      "Nepalese": [
        ""
      ],
      "New Zealander": [
        ""
      ],
      "Ni-Vanuatu": [
        ""
      ],
      "Nicaraguan": [
        ""
      ],
      "Nigerian": [
        ""
      ],
      "Nigerien": [
        ""
      ],
      "North Korean": [
        ""
      ],
      "Norwegian": [
        ""
      ],
      "Omani": [
        ""
      ],
      "Pakistani": [
        ""
      ],
      "Palauan": [
        ""
      ],
      "Panamanian": [
        ""
      ],
      "Papua New Guinean": [
        ""
      ],
      "Paraguayan": [
        ""
      ],
      "Peruvian": [
        ""
      ],
      "Polish": [
        ""
      ],
      "Portuguese": [
        ""
      ],
      "Qatari": [
        ""
      ],
      "Romanian": [
        ""
      ],
      "Russian": [
        ""
      ],
      "Rwandan": [
        ""
      ],
      "Saint Lucian": [
        ""
      ],
      "Salvadoran": [
        ""
      ],
      "Samoan": [
        ""
      ],
      "San Marinese": [
        ""
      ],
      "Sao Tomean": [
        ""
      ],
      "Saudi": [
        ""
      ],
      "Senegalese": [
        ""
      ],
      "Serbian": [
        ""
      ],
      "Seychellois": [
        ""
      ],
      "Sierra Leonean": [
        ""
      ],
      "Singaporean": [
        ""
      ],
      "Slovakian": [
        ""
      ],
      "Slovenian": [
        ""
      ],
      "Solomon Islander": [
        ""
      ],
      "Somali": [
        ""
      ],
      "South African": [
        ""
      ],
      "South Korean": [
        ""
      ],
      "Spanish": [
        ""
      ],
      "Sri Lankan": [
        ""
      ],
      "Sudanese": [
        ""
      ],
      "Surinamer": [
        ""
      ],
      "Swazi": [
        ""
      ],
      "Swedish": [
        ""
      ],
      "Swiss": [
        ""
      ],
      "Syrian": [
        ""
      ],
      "Taiwanese": [
        ""
      ],
      "Tajik": [
        ""
      ],
      "Tanzanian": [
        ""
      ],
      "Thai": [
        ""
      ],
      "Togolese": [
        ""
      ],
      "Tongan": [
        ""
      ],
      "Trinidadian or Tobagonian": [
        ""
      ],
      "Tunisian": [
        ""
      ],
      "Turkish": [
        ""
      ],
      "Tuvaluan": [
        "değer"
      ],
      "Ugandan": [
        ""
      ],
      "Ukrainian": [
        ""
      ],
      "Uruguayan": [
        ""
      ],
      "Uzbekistani": [
        ""
      ],
      "Venezuelan": [
        ""
      ],
      "Vietnamese": [
        ""
      ],
      "Yemenite": [
        ""
      ],
      "Zambian": [
        ""
      ],
      "Zimbabwean": [
        ""
      ],
      "🇦🇫": [
        ""
      ],
      "🇦🇱": [
        ""
      ],
      "🇩🇿": [
        ""
      ],
      "🇺🇸": [
        ""
      ],
      "🇦🇩": [
        ""
      ],
      "🇦🇴": [
        ""
      ],
      "🇦🇬": [
        ""
      ],
      "🇦🇷": [
        ""
      ],
      "🇦🇲": [
        ""
      ],
      "🇦🇺": [
        ""
      ],
      "🇦🇹": [
        ""
      ],
      "🇦🇿": [
        ""
      ],
      "🇧🇸": [
        ""
      ],
      "🇧🇭": [
        ""
      ],
      "🇧🇩": [
        ""
      ],
      "🇧🇧": [
        ""
      ],
      "🇧🇼": [
        ""
      ],
      "🇧🇾": [
        ""
      ],
      "🇧🇪": [
        ""
      ],
      "🇧🇿": [
        ""
      ],
      "🇧🇯": [
        ""
      ],
      "🇧🇹": [
        ""
      ],
      "🇧🇴": [
        ""
      ],
      "🇧🇦": [
        ""
      ],
      "🇧🇷": [
        ""
      ],
      "🇬🇧": [
        ""
      ],
      "🇧🇳": [
        ""
      ],
      "🇧🇬": [
        ""
      ],
      "🇧🇫": [
        ""
      ],
      "🇲🇲": [
        ""
      ],
      "🇧🇮": [
        ""
      ],
      "🇰🇭": [
        ""
      ],
      "🇨🇲": [
        ""
      ],
      "🇨🇦": [
        ""
      ],
      "🇨🇻": [
        ""
      ],
      "🇨🇫": [
        ""
      ],
      "🇨🇱": [
        ""
      ],
      "🇨🇳": [
        ""
      ],
      "🇨🇴": [
        ""
      ],
      "🇨🇩": [
        ""
      ],
      "🇨🇬": [
        ""
      ],
      "🇨🇷": [
        ""
      ],
      "🇨🇺": [
        ""
      ],
      "🇨🇾": [
        ""
      ],
      "🇨🇿": [
        ""
      ],
      "🇩🇰": [
        ""
      ],
      "🇩🇯": [
        ""
      ],
      "🇩🇴": [
        ""
      ],
      "🇳🇱": [
        ""
      ],
      "🇹🇱": [
        ""
      ],
      "🇪🇨": [
        ""
      ],
      "🇪🇬": [
        ""
      ],
      "🇦🇪": [
        ""
      ],
      "🇬🇶": [
        ""
      ],
      "🇪🇷": [
        ""
      ],
      "🇪🇪": [
        ""
      ],
      "🇪🇹": [
        ""
      ],
      "🇫🇯": [
        ""
      ],
      "🇵🇭": [
        ""
      ],
      "🇫🇮": [
        ""
      ],
      "🇫🇷": [
        ""
      ],
      "🇬🇦": [
        ""
      ],
      "🇬🇲": [
        ""
      ],
      "🇬🇪": [
        ""
      ],
      "UAE Dirham": [
        ""
      ],
      "Afghani": [
        ""
      ],
      "Lek": [
        ""
      ],
      "Armenian Dram": [
        ""
      ],
      "Netherlands Antillean Guilder": [
        ""
      ],
      "Kwanza": [
        ""
      ],
      "Argentine Peso": [
        ""
      ],
      "Australian Dollar": [
        ""
      ],
      "Aruban Florin": [
        ""
      ],
      "Azerbaijan Manat": [
        ""
      ],
      "Convertible Mark": [
        ""
      ],
      "Barbados Dollar": [
        ""
      ],
      "Taka": [
        ""
      ],
      "Bulgarian Lev": [
        ""
      ],
      "Bahraini Dinar": [
        ""
      ],
      "Burundi Franc": [
        ""
      ],
      "Bermudian Dollar": [
        ""
      ],
      "Brunei Dollar": [
        ""
      ],
      "Boliviano": [
        ""
      ],
      "Mvdol": [
        ""
      ],
      "Brazilian Real": [
        ""
      ],
      "Bahamian Dollar": [
        ""
      ],
      "Ngultrum": [
        ""
      ],
      "Pula": [
        ""
      ],
      "Belarusian Ruble": [
        ""
      ],
      "Belize Dollar": [
        ""
      ],
      "Canadian Dollar": [
        ""
      ],
      "Congolese Franc": [
        ""
      ],
      "WIR Euro": [
        ""
      ],
      "Swiss Franc": [
        ""
      ],
      "WIR Franc": [
        ""
      ],
      "Unidad de Fomento": [
        ""
      ],
      "Chilean Peso": [
        ""
      ],
      "Yuan Renminbi": [
        ""
      ],
      "Colombian Peso": [
        ""
      ],
      "Unidad de Valor Real": [
        ""
      ],
      "Costa Rican Colon": [
        ""
      ],
      "Cuban Peso": [
        ""
      ],
      "Cabo Verde Escudo": [
        ""
      ],
      "Czech Koruna": [
        ""
      ],
      "Djibouti Franc": [
        ""
      ],
      "Danish Krone": [
        ""
      ],
      "Dominican Peso": [
        ""
      ],
      "Algerian Dinar": [
        ""
      ],
      "Egyptian Pound": [
        ""
      ],
      "Nakfa": [
        ""
      ],
      "Ethiopian Birr": [
        ""
      ],
      "Euro": [
        ""
      ],
      "Fiji Dollar": [
        ""
      ],
      "Falkland Islands Pound": [
        ""
      ],
      "Pound Sterling": [
        ""
      ],
      "Lari": [
        ""
      ],
      "Ghana Cedi": [
        ""
      ],
      "Gibraltar Pound": [
        ""
      ],
      "Dalasi": [
        ""
      ],
      "Guinean Franc": [
        ""
      ],
      "Quetzal": [
        ""
      ],
      "Guyana Dollar": [
        ""
      ],
      "Hong Kong Dollar": [
        ""
      ],
      "Lempira": [
        ""
      ],
      "Gourde": [
        ""
      ],
      "Forint": [
        ""
      ],
      "Rupiah": [
        ""
      ],
      "New Israeli Sheqel": [
        ""
      ],
      "Indian Rupee": [
        ""
      ],
      "Iraqi Dinar": [
        ""
      ],
      "Iranian Rial": [
        ""
      ],
      "Iceland Krona": [
        ""
      ],
      "Jamaican Dollar": [
        ""
      ],
      "Jordanian Dinar": [
        ""
      ],
      "Yen": [
        ""
      ],
      "Kenyan Shilling": [
        ""
      ],
      "Som": [
        ""
      ],
      "Riel": [
        ""
      ],
      "Comorian Franc ": [
        ""
      ],
      "North Korean Won": [
        ""
      ],
      "Won": [
        ""
      ],
      "Kuwaiti Dinar": [
        ""
      ],
      "Cayman Islands Dollar": [
        ""
      ],
      "Tenge": [
        ""
      ],
      "Lao Kip": [
        ""
      ],
      "Lebanese Pound": [
        ""
      ],
      "Sri Lanka Rupee": [
        ""
      ],
      "Liberian Dollar": [
        ""
      ],
      "Loti": [
        ""
      ],
      "Libyan Dinar": [
        ""
      ],
      "Moroccan Dirham": [
        ""
      ],
      "Moldovan Leu": [
        ""
      ],
      "Malagasy Ariary": [
        ""
      ],
      "Denar": [
        ""
      ],
      "Kyat": [
        ""
      ],
      "Tugrik": [
        ""
      ],
      "Pataca": [
        ""
      ],
      "Ouguiya": [
        ""
      ],
      "Mauritius Rupee": [
        ""
      ],
      "Rufiyaa": [
        ""
      ],
      "Malawi Kwacha": [
        ""
      ],
      "Mexican Peso": [
        ""
      ],
      "Mexican Unidad de Inversion (UDI)": [
        ""
      ],
      "Malaysian Ringgit": [
        ""
      ],
      "Mozambique Metical": [
        ""
      ],
      "Namibia Dollar": [
        ""
      ],
      "Naira": [
        ""
      ],
      "Cordoba Oro": [
        ""
      ],
      "Norwegian Krone": [
        ""
      ],
      "Nepalese Rupee": [
        ""
      ],
      "New Zealand Dollar": [
        ""
      ],
      "Rial Omani": [
        ""
      ],
      "Balboa": [
        ""
      ],
      "Sol": [
        "eski"
      ],
      "Kina": [
        ""
      ],
      "Philippine Peso": [
        ""
      ],
      "Pakistan Rupee": [
        ""
      ],
      "Zloty": [
        ""
      ],
      "Guarani": [
        ""
      ],
      "Qatari Rial": [
        ""
      ],
      "Romanian Leu": [
        ""
      ],
      "Serbian Dinar": [
        ""
      ],
      "Russian Ruble": [
        ""
      ],
      "Rwanda Franc": [
        ""
      ],
      "Saudi Riyal": [
        ""
      ],
      "Solomon Islands Dollar": [
        ""
      ],
      "Seychelles Rupee": [
        ""
      ],
      "Sudanese Pound": [
        ""
      ],
      "Swedish Krona": [
        ""
      ],
      "Singapore Dollar": [
        ""
      ],
      "Saint Helena Pound": [
        ""
      ],
      "Leone": [
        ""
      ],
      "Somali Shilling": [
        ""
      ],
      "Surinam Dollar": [
        ""
      ],
      "South Sudanese Pound": [
        ""
      ],
      "Dobra": [
        ""
      ],
      "El Salvador Colon": [
        ""
      ],
      "Syrian Pound": [
        ""
      ],
      "Lilangeni": [
        ""
      ],
      "Baht": [
        ""
      ],
      "Somoni": [
        ""
      ],
      "Turkmenistan New Manat": [
        ""
      ],
      "Tunisian Dinar": [
        ""
      ],
      "Pa’anga": [
        ""
      ],
      "Turkish Lira": [
        ""
      ],
      "Trinidad and Tobago Dollar": [
        ""
      ],
      "New Taiwan Dollar": [
        ""
      ],
      "Tanzanian Shilling": [
        ""
      ],
      "Hryvnia": [
        ""
      ],
      "Uganda Shilling": [
        ""
      ],
      "US Dollar": [
        ""
      ],
      "US Dollar (Next day)": [
        ""
      ],
      "Uruguay Peso en Unidades Indexadas (UI)": [
        ""
      ],
      "Peso Uruguayo": [
        ""
      ],
      "Unidad Previsional": [
        ""
      ],
      "Uzbekistan Sum": [
        ""
      ],
      "Bolívar Soberano": [
        ""
      ],
      "Dong": [
        ""
      ],
      "Vatu": [
        ""
      ],
      "Tala": [
        ""
      ],
      "CFA Franc BEAC": [
        ""
      ],
      "East Caribbean Dollar": [
        ""
      ],
      "CFA Franc BCEAO": [
        ""
      ],
      "CFP Franc": [
        ""
      ],
      "Sucre": [
        ""
      ],
      "Yemeni Rial": [
        ""
      ],
      "Rand": [
        ""
      ],
      "Zambian Kwacha": [
        ""
      ],
      "Zimbabwe Gold": [
        ""
      ],
      "Afar": [
        ""
      ],
      "Abkhazian": [
        ""
      ],
      "Achinese": [
        ""
      ],
      "Acoli": [
        ""
      ],
      "Adangme": [
        ""
      ],
      "Adyghe": [
        ""
      ],
      "Avestan": [
        ""
      ],
      "Tunisian Arabic": [
        ""
      ],
      "Afrikaans": [
        ""
      ],
      "Afrihili": [
        ""
      ],
      "Aghem": [
        ""
      ],
      "Ainu": [
        ""
      ],
      "Akan": [
        ""
      ],
      "Akkadian": [
        ""
      ],
      "Alabama": [
        ""
      ],
      "Aleut": [
        ""
      ],
      "Gheg Albanian": [
        ""
      ],
      "Southern Altai": [
        ""
      ],
      "Amarik": [
        ""
      ],
      "Aragonese": [
        ""
      ],
      "Old English": [
        ""
      ],
      "Angika": [
        ""
      ],
      "Modern Standard Arabic": [
        ""
      ],
      "Arabik": [
        ""
      ],
      "Aramaic": [
        ""
      ],
      "Mapuche": [
        ""
      ],
      "Araona": [
        ""
      ],
      "Arapaho": [
        ""
      ],
      "Algerian Arabic": [
        ""
      ],
      "Arawak": [
        ""
      ],
      "Moroccan Arabic": [
        ""
      ],
      "Egyptian Arabic": [
        ""
      ],
      "Asu": [
        ""
      ],
      "Assamese": [
        ""
      ],
      "American Sign Language": [
        ""
      ],
      "Asturian": [
        ""
      ],
      "Avaric": [
        ""
      ],
      "Kotava": [
        ""
      ],
      "Awadhi": [
        ""
      ],
      "Aymara": [
        ""
      ],
      "South Azerbaijani": [
        ""
      ],
      "Bashkir": [
        ""
      ],
      "Baluchi": [
        ""
      ],
      "Balinese": [
        ""
      ],
      "Bavarian": [
        ""
      ],
      "Basaa": [
        ""
      ],
      "Bamun": [
        ""
      ],
      "Batak Toba": [
        ""
      ],
      "Ghomala": [
        ""
      ],
      "Belarus kasa": [
        ""
      ],
      "Beja": [
        ""
      ],
      "Bemba": [
        ""
      ],
      "Betawi": [
        ""
      ],
      "Bena": [
        ""
      ],
      "Bafut": [
        ""
      ],
      "Badaga": [
        ""
      ],
      "Bɔlgeria kasa": [
        ""
      ],
      "Bhojpuri": [
        ""
      ],
      "Bislama": [
        ""
      ],
      "Bikol": [
        ""
      ],
      "Bini": [
        ""
      ],
      "Banjar": [
        ""
      ],
      "Kom": [
        ""
      ],
      "Siksika": [
        ""
      ],
      "Bambara": [
        ""
      ],
      "Bengali kasa": [
        ""
      ],
      "Tibetan": [
        ""
      ],
      "Bishnupriya": [
        ""
      ],
      "Bakhtiari": [
        ""
      ],
      "Braj": [
        ""
      ],
      "Breton": [
        ""
      ],
      "Brahui": [
        ""
      ],
      "Bodo": [
        ""
      ],
      "Akoose": [
        ""
      ],
      "Buriat": [
        ""
      ],
      "Buginese": [
        ""
      ],
      "Bulu": [
        ""
      ],
      "Blin": [
        ""
      ],
      "Medumba": [
        ""
      ],
      "Catalan": [
        ""
      ],
      "Caddo": [
        ""
      ],
      "Carib": [
        ""
      ],
      "Cayuga": [
        ""
      ],
      "Atsam": [
        ""
      ],
      "Cebuano": [
        ""
      ],
      "Chechen": [
        ""
      ],
      "Chiga": [
        ""
      ],
      "Chibcha": [
        ""
      ],
      "Chamorro": [
        ""
      ],
      "Chagatai": [
        ""
      ],
      "Chuukese": [
        ""
      ],
      "Mari": [
        ""
      ],
      "Chinook Jargon": [
        ""
      ],
      "Choctaw": [
        ""
      ],
      "Chipewyan": [
        ""
      ],
      "Cherokee": [
        ""
      ],
      "Cheyenne": [
        ""
      ],
      "Central Kurdish": [
        ""
      ],
      "Corsican": [
        ""
      ],
      "Coptic": [
        ""
      ],
      "Capiznon": [
        ""
      ],
      "Cree": [
        ""
      ],
      "Crimean Turkish": [
        ""
      ],
      "Kashubian": [
        ""
      ],
      "Kyɛk kasa": [
        ""
      ],
      "Church Slavic": [
        ""
      ],
      "Chuvash": [
        ""
      ],
      "Welsh": [
        ""
      ],
      "Dakota": [
        ""
      ],
      "Dargwa": [
        ""
      ],
      "Taita": [
        ""
      ],
      "Austrian German": [
        ""
      ],
      "Swiss High German": [
        ""
      ],
      "Gyaaman": [
        ""
      ],
      "Delaware": [
        ""
      ],
      "Slave": [
        ""
      ],
      "Dogrib": [
        ""
      ],
      "Dinka": [
        ""
      ],
      "Zarma": [
        ""
      ],
      "Dogri": [
        ""
      ],
      "Lower Sorbian": [
        ""
      ],
      "Central Dusun": [
        ""
      ],
      "Duala": [
        ""
      ],
      "Middle Dutch": [
        ""
      ],
      "Divehi": [
        ""
      ],
      "Jola-Fonyi": [
        ""
      ],
      "Dyula": [
        ""
      ],
      "Dzongkha": [
        ""
      ],
      "Dazaga": [
        ""
      ],
      "Embu": [
        ""
      ],
      "Ewe": [
        ""
      ],
      "Efik": [
        ""
      ],
      "Emilian": [
        ""
      ],
      "Ancient Egyptian": [
        ""
      ],
      "Ekajuk": [
        ""
      ],
      "Greek kasa": [
        ""
      ],
      "Elamite": [
        ""
      ],
      "Australian English": [
        ""
      ],
      "Borɔfo": [
        ""
      ],
      "Canadian English": [
        ""
      ],
      "British English": [
        ""
      ],
      "Middle English": [
        ""
      ],
      "American English": [
        ""
      ],
      "Esperanto": [
        "Bekleyen işlemler"
      ],
      "Latin American Spanish": [
        ""
      ],
      "European Spanish": [
        ""
      ],
      "Mexican Spanish": [
        ""
      ],
      "Spain kasa": [
        ""
      ],
      "Central Yupik": [
        ""
      ],
      "Basque": [
        ""
      ],
      "Ewondo": [
        ""
      ],
      "Extremaduran": [
        ""
      ],
      "Fang": [
        ""
      ],
      "Pɛɛhyia kasa": [
        ""
      ],
      "Fanti": [
        ""
      ],
      "Fulah": [
        ""
      ],
      "Tornedalen Finnish": [
        ""
      ],
      "Faroese": [
        ""
      ],
      "Fon": [
        ""
      ],
      "Canadian French": [
        ""
      ],
      "Cajun French": [
        ""
      ],
      "Swiss French": [
        ""
      ],
      "Frɛnkye": [
        ""
      ],
      "Middle French": [
        ""
      ],
      "Old French": [
        ""
      ],
      "Arpitan": [
        ""
      ],
      "Northern Frisian": [
        ""
      ],
      "Eastern Frisian": [
        ""
      ],
      "Friulian": [
        ""
      ],
      "Western Frisian": [
        ""
      ],
      "Ga": [
        ""
      ],
      "Gagauz": [
        ""
      ],
      "Gan Chinese": [
        ""
      ],
      "Gayo": [
        ""
      ],
      "Gbaya": [
        ""
      ],
      "Zoroastrian Dari": [
        ""
      ],
      "Scottish Gaelic": [
        ""
      ],
      "Geez": [
        ""
      ],
      "Gilbertese": [
        ""
      ],
      "Galician": [
        ""
      ],
      "Gilaki": [
        ""
      ],
      "Middle High German": [
        ""
      ],
      "Old High German": [
        ""
      ],
      "Goan Konkani": [
        ""
      ],
      "Gondi": [
        ""
      ],
      "Gorontalo": [
        ""
      ],
      "Gothic": [
        ""
      ],
      "Grebo": [
        ""
      ],
      "Ancient Greek": [
        ""
      ],
      "Swiss German": [
        ""
      ],
      "Wayuu": [
        ""
      ],
      "Gujarati": [
        ""
      ],
      "Frafra": [
        ""
      ],
      "Gusii": [
        ""
      ],
      "Manx": [
        ""
      ],
      "Gwichʼin": [
        ""
      ],
      "Hausa": [
        ""
      ],
      "Haida": [
        ""
      ],
      "Hakka Chinese": [
        ""
      ],
      "Hawaiian": [
        ""
      ],
      "Hebrew": [
        ""
      ],
      "Fiji Hindi": [
        ""
      ],
      "Hindi": [
        ""
      ],
      "Hiligaynon": [
        ""
      ],
      "Hittite": [
        ""
      ],
      "Hmong": [
        ""
      ],
      "Hiri Motu": [
        ""
      ],
      "Upper Sorbian": [
        ""
      ],
      "Xiang Chinese": [
        ""
      ],
      "Hangri kasa": [
        ""
      ],
      "Hupa": [
        ""
      ],
      "Herero": [
        ""
      ],
      "Interlingua": [
        ""
      ],
      "Iban": [
        ""
      ],
      "Ibibio": [
        ""
      ],
      "Indonihyia kasa": [
        ""
      ],
      "Interlingue": [
        ""
      ],
      "Igbo": [
        ""
      ],
      "Sichuan Yi": [
        ""
      ],
      "Inupiaq": [
        ""
      ],
      "Iloko": [
        ""
      ],
      "Ingush": [
        ""
      ],
      "Ido": [
        ""
      ],
      "Icelandic": [
        ""
      ],
      "Italy kasa": [
        ""
      ],
      "Inuktitut": [
        ""
      ],
      "Ingrian": [
        ""
      ],
      "Gyapan kasa": [
        ""
      ],
      "Jamaican Creole English": [
        ""
      ],
      "Lojban": [
        ""
      ],
      "Ngomba": [
        ""
      ],
      "Machame": [
        ""
      ],
      "Judeo-Persian": [
        ""
      ],
      "Judeo-Arabic": [
        ""
      ],
      "Jutish": [
        ""
      ],
      "Gyabanis kasa": [
        ""
      ],
      "Kara-Kalpak": [
        ""
      ],
      "Kabyle": [
        ""
      ],
      "Kachin": [
        ""
      ],
      "Jju": [
        ""
      ],
      "Kamba": [
        ""
      ],
      "Kawi": [
        ""
      ],
      "Kabardian": [
        ""
      ],
      "Kanembu": [
        ""
      ],
      "Tyap": [
        ""
      ],
      "Makonde": [
        ""
      ],
      "Kabuverdianu": [
        ""
      ],
      "Kenyang": [
        ""
      ],
      "Koro": [
        ""
      ],
      "Kongo": [
        ""
      ],
      "Kaingang": [
        ""
      ],
      "Khasi": [
        ""
      ],
      "Khotanese": [
        ""
      ],
      "Koyra Chiini": [
        ""
      ],
      "Khowar": [
        ""
      ],
      "Kikuyu": [
        ""
      ],
      "Kirmanjki": [
        ""
      ],
      "Kuanyama": [
        ""
      ],
      "Kako": [
        ""
      ],
      "Kazakh": [
        ""
      ],
      "Kalaallisut": [
        ""
      ],
      "Kalenjin": [
        ""
      ],
      "Kimbundu": [
        ""
      ],
      "Kambodia kasa": [
        ""
      ],
      "Kannada": [
        ""
      ],
      "Komi-Permyak": [
        ""
      ],
      "Konkani": [
        ""
      ],
      "Korea kasa": [
        ""
      ],
      "Kosraean": [
        ""
      ],
      "Kpelle": [
        ""
      ],
      "Karachay-Balkar": [
        ""
      ],
      "Krio": [
        ""
      ],
      "Kinaray-a": [
        ""
      ],
      "Kanuri": [
        ""
      ],
      "Karelian": [
        ""
      ],
      "Kurukh": [
        ""
      ],
      "Shambala": [
        ""
      ],
      "Bafia": [
        ""
      ],
      "Colognian": [
        ""
      ],
      "Kashmiri": [
        ""
      ],
      "Kurdish": [
        ""
      ],
      "Kumyk": [
        ""
      ],
      "Kutenai": [
        ""
      ],
      "Komi": [
        ""
      ],
      "Cornish": [
        ""
      ],
      "Ladino": [
        "Yükleniyor"
      ],
      "Langi": [
        ""
      ],
      "Lahnda": [
        ""
      ],
      "Latin": [
        ""
      ],
      "Lamba": [
        ""
      ],
      "Luxembourgish": [
        ""
      ],
      "Lezghian": [
        ""
      ],
      "Lingua Franca Nova": [
        ""
      ],
      "Ganda": [
        ""
      ],
      "Ligurian": [
        ""
      ],
      "Limburgish": [
        ""
      ],
      "Livonian": [
        ""
      ],
      "Lakota": [
        ""
      ],
      "Lombard": [
        ""
      ],
      "Lingala": [
        ""
      ],
      "Lao": [
        ""
      ],
      "Mongo": [
        ""
      ],
      "Lozi": [
        ""
      ],
      "Latgalian": [
        ""
      ],
      "Luba-Lulua": [
        ""
      ],
      "Luiseno": [
        ""
      ],
      "Luba-Katanga": [
        ""
      ],
      "Lunda": [
        ""
      ],
      "Luo": [
        ""
      ],
      "Mizo": [
        ""
      ],
      "Luyia": [
        ""
      ],
      "Literary Chinese": [
        ""
      ],
      "Laz": [
        ""
      ],
      "Madurese": [
        "sıfırla"
      ],
      "Mafa": [
        ""
      ],
      "Magahi": [
        ""
      ],
      "Maithili": [
        ""
      ],
      "Makasar": [
        ""
      ],
      "Mandingo": [
        "Yükleniyor"
      ],
      "Masai": [
        ""
      ],
      "Maba": [
        ""
      ],
      "Moksha": [
        ""
      ],
      "Mandar": [
        ""
      ],
      "Mende": [
        ""
      ],
      "Meru": [
        ""
      ],
      "Morisyen": [
        ""
      ],
      "Middle Irish": [
        ""
      ],
      "Makhuwa-Meetto": [
        ""
      ],
      "Metaʼ": [
        ""
      ],
      "Micmac": [
        ""
      ],
      "Maori": [
        ""
      ],
      "Minangkabau": [
        ""
      ],
      "Malayalam": [
        ""
      ],
      "Manchu": [
        ""
      ],
      "Manipuri": [
        ""
      ],
      "Mohawk": [
        ""
      ],
      "Mossi": [
        ""
      ],
      "Western Mari": [
        ""
      ],
      "Marathi": [
        ""
      ],
      "Malay kasa": [
        ""
      ],
      "Mundang": [
        ""
      ],
      "Multiple Languages": [
        ""
      ],
      "Creek": [
        ""
      ],
      "Mirandese": [
        ""
      ],
      "Marwari": [
        ""
      ],
      "Mentawai": [
        ""
      ],
      "Bɛɛmis kasa": [
        ""
      ],
      "Myene": [
        ""
      ],
      "Erzya": [
        ""
      ],
      "Mazanderani": [
        ""
      ],
      "Min Nan Chinese": [
        ""
      ],
      "Neapolitan": [
        ""
      ],
      "Nama": [
        ""
      ],
      "Norwegian Bokmål": [
        ""
      ],
      "North Ndebele": [
        ""
      ],
      "Low German": [
        ""
      ],
      "Nɛpal kasa": [
        ""
      ],
      "Newari": [
        ""
      ],
      "Ndonga": [
        ""
      ],
      "Nias": [
        ""
      ],
      "Niuean": [
        ""
      ],
      "Ao Naga": [
        ""
      ],
      "Flemish": [
        ""
      ],
      "Dɛɛkye": [
        ""
      ],
      "Kwasio": [
        ""
      ],
      "Ngiemboon": [
        ""
      ],
      "Norwegian Nynorsk": [
        ""
      ],
      "Nogai": [
        ""
      ],
      "Old Norse": [
        ""
      ],
      "Novial": [
        ""
      ],
      "NʼKo": [
        ""
      ],
      "South Ndebele": [
        ""
      ],
      "Northern Sotho": [
        ""
      ],
      "Nuer": [
        ""
      ],
      "Navajo": [
        ""
      ],
      "Classical Newari": [
        ""
      ],
      "Nyamwezi": [
        ""
      ],
      "Nyankole": [
        ""
      ],
      "Nyanja": [
        ""
      ],
      "Nyoro": [
        ""
      ],
      "Nzima": [
        ""
      ],
      "Occitan": [
        ""
      ],
      "Ojibwa": [
        ""
      ],
      "Oromo": [
        ""
      ],
      "Oriya": [
        ""
      ],
      "Osage": [
        ""
      ],
      "Ossetic": [
        ""
      ],
      "Ottoman Turkish": [
        ""
      ],
      "Pangasinan": [
        ""
      ],
      "Pahlavi": [
        ""
      ],
      "Pampanga": [
        ""
      ],
      "Papiamento": [
        "Ödeme gönderildi"
      ],
      "Pungyabi kasa": [
        ""
      ],
      "Picard": [
        ""
      ],
      "Pennsylvania German": [
        ""
      ],
      "Plautdietsch": [
        ""
      ],
      "Old Persian": [
        ""
      ],
      "Palatine German": [
        ""
      ],
      "Phoenician": [
        ""
      ],
      "Pali": [
        ""
      ],
      "Pɔland kasa": [
        ""
      ],
      "Piedmontese": [
        ""
      ],
      "Pontic": [
        ""
      ],
      "Pohnpeian": [
        ""
      ],
      "Prussian": [
        ""
      ],
      "Old Provençal": [
        ""
      ],
      "Pashto": [
        ""
      ],
      "Brazilian Portuguese": [
        ""
      ],
      "Pɔɔtugal kasa": [
        ""
      ],
      "European Portuguese": [
        ""
      ],
      "Kʼicheʼ": [
        ""
      ],
      "Chimborazo Highland Quichua": [
        ""
      ],
      "Quechua": [
        ""
      ],
      "Rajasthani": [
        ""
      ],
      "Rapanui": [
        ""
      ],
      "Rarotongan": [
        ""
      ],
      "Romagnol": [
        ""
      ],
      "Riffian": [
        ""
      ],
      "Romansh": [
        ""
      ],
      "Rundi": [
        ""
      ],
      "Rombo": [
        ""
      ],
      "Moldavian": [
        ""
      ],
      "Romany": [
        ""
      ],
      "Root": [
        ""
      ],
      "Romenia kasa": [
        ""
      ],
      "Rotuman": [
        ""
      ],
      "Rusyn": [
        ""
      ],
      "Roviana": [
        ""
      ],
      "Aromanian": [
        ""
      ],
      "Rahyia kasa": [
        ""
      ],
      "Rwa": [
        ""
      ],
      "Rewanda kasa": [
        ""
      ],
      "Sandawe": [
        ""
      ],
      "Sakha": [
        ""
      ],
      "Samaritan Aramaic": [
        ""
      ],
      "Samburu": [
        ""
      ],
      "Sanskrit": [
        ""
      ],
      "Sasak": [
        ""
      ],
      "Santali": [
        ""
      ],
      "Saurashtra": [
        ""
      ],
      "Ngambay": [
        ""
      ],
      "Sangu": [
        ""
      ],
      "Sicilian": [
        ""
      ],
      "Scots": [
        ""
      ],
      "Sardinian": [
        ""
      ],
      "Sassarese Sardinian": [
        ""
      ],
      "Sindhi": [
        ""
      ],
      "Seneca": [
        ""
      ],
      "Sena": [
        ""
      ],
      "Seri": [
        ""
      ],
      "Selkup": [
        ""
      ],
      "Northern Sami": [
        ""
      ],
      "Koyraboro Senni": [
        ""
      ],
      "Old Irish": [
        ""
      ],
      "Sango": [
        ""
      ],
      "Samogitian": [
        ""
      ],
      "Tachelhit": [
        ""
      ],
      "Shan": [
        ""
      ],
      "Serbo-Croatian": [
        ""
      ],
      "Chadian Arabic": [
        ""
      ],
      "Sidamo": [
        ""
      ],
      "Sinhala": [
        ""
      ],
      "Slovak": [
        ""
      ],
      "Lower Silesian": [
        ""
      ],
      "Selayar": [
        ""
      ],
      "Southern Sami": [
        ""
      ],
      "Lule Sami": [
        ""
      ],
      "Inari Sami": [
        ""
      ],
      "Skolt Sami": [
        ""
      ],
      "Soninke": [
        ""
      ],
      "Shona": [
        ""
      ],
      "Sogdien": [
        ""
      ],
      "Somalia kasa": [
        ""
      ],
      "Sranan Tongo": [
        ""
      ],
      "Serer": [
        ""
      ],
      "Swati": [
        ""
      ],
      "Saho": [
        ""
      ],
      "Saterland Frisian": [
        ""
      ],
      "Southern Sotho": [
        ""
      ],
      "Sukuma": [
        ""
      ],
      "Susu": [
        ""
      ],
      "Sundanese": [
        ""
      ],
      "Sumerian": [
        ""
      ],
      "Sweden kasa": [
        ""
      ],
      "Comorian": [
        ""
      ],
      "Congo Swahili": [
        ""
      ],
      "Swahili": [
        ""
      ],
      "Classical Syriac": [
        ""
      ],
      "Syriac": [
        ""
      ],
      "Silesian": [
        ""
      ],
      "Tamil kasa": [
        ""
      ],
      "Tulu": [
        ""
      ],
      "Timne": [
        ""
      ],
      "Teso": [
        ""
      ],
      "Tereno": [
        ""
      ],
      "Telugu": [
        ""
      ],
      "Tetum": [
        ""
      ],
      "Taeland kasa": [
        ""
      ],
      "Tigre": [
        ""
      ],
      "Tigrinya": [
        ""
      ],
      "Tiv": [
        "Bahşiş"
      ],
      "Tokelau": [
        ""
      ],
      "Tsakhur": [
        ""
      ],
      "Turkmen": [
        ""
      ],
      "Klingon": [
        ""
      ],
      "Tlingit": [
        ""
      ],
      "Tagalog": [
        ""
      ],
      "Talysh": [
        ""
      ],
      "Tamashek": [
        ""
      ],
      "Tswana": [
        ""
      ],
      "Nyasa Tonga": [
        ""
      ],
      "Tok Pisin": [
        ""
      ],
      "Tɛɛki kasa": [
        ""
      ],
      "Turoyo": [
        ""
      ],
      "Taroko": [
        ""
      ],
      "Tsakonian": [
        ""
      ],
      "Tsimshian": [
        ""
      ],
      "Tsonga": [
        ""
      ],
      "Tatar": [
        ""
      ],
      "Muslim Tat": [
        ""
      ],
      "Tumbuka": [
        ""
      ],
      "Tasawaq": [
        ""
      ],
      "Twi": [
        "Bahşiş"
      ],
      "Tahitian": [
        ""
      ],
      "Tuvinian": [
        ""
      ],
      "Central Atlas Tamazight": [
        ""
      ],
      "Udmurt": [
        ""
      ],
      "Ugaritic": [
        ""
      ],
      "Uyghur": [
        ""
      ],
      "Ukren kasa": [
        ""
      ],
      "Umbundu": [
        ""
      ],
      "Unknown Language": [
        ""
      ],
      "Urdu kasa": [
        ""
      ],
      "Uzbek": [
        ""
      ],
      "Vai": [
        ""
      ],
      "Venetian": [
        ""
      ],
      "Veps": [
        ""
      ],
      "Venda": [
        ""
      ],
      "Viɛtnam kasa": [
        ""
      ],
      "West Flemish": [
        ""
      ],
      "Main-Franconian": [
        ""
      ],
      "Votic": [
        ""
      ],
      "Volapük": [
        ""
      ],
      "Võro": [
        ""
      ],
      "Vunjo": [
        ""
      ],
      "Walser": [
        ""
      ],
      "Wolaytta": [
        ""
      ],
      "Waray": [
        ""
      ],
      "Washo": [
        ""
      ],
      "Walloon": [
        ""
      ],
      "Warlpiri": [
        ""
      ],
      "Wolof": [
        ""
      ],
      "Wu Chinese": [
        ""
      ],
      "Kalmyk": [
        ""
      ],
      "Xhosa": [
        ""
      ],
      "Mingrelian": [
        ""
      ],
      "Soga": [
        ""
      ],
      "Yao": [
        ""
      ],
      "Yapese": [
        ""
      ],
      "Yangben": [
        ""
      ],
      "Yemba": [
        ""
      ],
      "Yiddish": [
        ""
      ],
      "Yoruba": [
        ""
      ],
      "Nheengatu": [
        ""
      ],
      "Cantonese": [
        ""
      ],
      "Zapotec": [
        ""
      ],
      "Zhuang": [
        ""
      ],
      "Blissymbols": [
        ""
      ],
      "Zeelandic": [
        ""
      ],
      "Zenaga": [
        ""
      ],
      "Standard Moroccan Tamazight": [
        ""
      ],
      "Simplified Chinese": [
        ""
      ],
      "Traditional Chinese": [
        ""
      ],
      "Kyaena kasa": [
        ""
      ],
      "Zuni": [
        ""
      ],
      "Zulu": [
        ""
      ],
      "No linguistic content": [
        ""
      ],
      "Zaza": [
        ""
      ],
      "Could not load user attention request": [
        "Bakiye sayfası yüklenemedi"
      ],
      "No notification left": [
        ""
      ],
      "Ignore": [
        ""
      ],
      "Invalid dev experiment URI.": [
        ""
      ],
      "URI: %1$s": [
        "%1$s"
      ],
      "Experiment": [
        ""
      ],
      "Insert lost denomination": [
        "Bekleyen işlemler"
      ],
      "Apply": [
        ""
      ],
      "Pending refresh": [
        "Bekleyen işlemler"
      ],
      "Could not load the transfer payment status": [
        "Ödeme durumu yüklenemedi"
      ],
      "Could not load the status of the order template": [
        "Bakiye sayfası yüklenemedi"
      ],
      "required": [
        ""
      ],
      "Review order": [
        ""
      ],
      "Missing URI": [
        ""
      ],
      "Check the link.": [
        ""
      ],
      "Could not parse the recovery URI": [
        ""
      ],
      "Try another URI": [
        "URI'yi girin"
      ],
      "Import backup, show info": [
        ""
      ],
      "Add new exchange": [
        ""
      ],
      "Add exchange for %1$s": [
        ""
      ],
      "Enter the URL of an exchange you trust.": [
        ""
      ],
      "An exchange has been found! Review the information and click next": [
        ""
      ],
      "This exchange is already in your list.": [
        ""
      ],
      "Only exchange accessible through \"http\" and \"https\" are allowed.": [
        ""
      ],
      "This exchange protocol version is not supported: \" %1$s\".": [
        ""
      ],
      "This exchange currency \"%1$s\" doesn&apos;t match the expected currency %2$s.": [
        ""
      ],
      "No exchange found in that URL.": [
        ""
      ],
      "loading": [
        ""
      ],
      "Version": [
        ""
      ],
      "Can't use the URL: \"%1$s\"": [
        ""
      ],
      "Exchange URL": [
        ""
      ],
      "Add new contact": [
        ""
      ],
      "Enter the details of a contact you trust.": [
        ""
      ],
      "Contact info valid.": [
        ""
      ],
      "This contact is already in your list.": [
        ""
      ],
      "Review contact": [
        ""
      ],
      "Contact alias": [
        ""
      ],
      "Alias type": [
        ""
      ],
      "Mailbox service": [
        ""
      ],
      "Mailbox address": [
        "Bitcoin adresi"
      ],
      "Contact source": [
        ""
      ],
      "Could not load the list of contacts": [
        "Bakiye sayfası yüklenemedi"
      ],
      "Source": [
        ""
      ],
      "Send Cash": [
        "%1$s gönder"
      ],
      "Request Cash": [
        ""
      ],
      "No contacts found.": [
        ""
      ],
      "Could not bank choices for account": [
        "Yedekleme sağlayıcıları yüklenemedi"
      ],
      "Could not load the list of messages": [
        "Bakiye sayfası yüklenemedi"
      ],
      "Unexpected error when trying to initialize mailbox: %1$s": [
        ""
      ],
      "Unexpected error when trying to delete message: %1$s": [
        ""
      ],
      "Unexpected error when trying to fetch messages: %1$s": [
        ""
      ],
      "Date": [
        "Tarih"
      ],
      "You may copy your mailbox URI from below and register it at %1$s with your alias(es) to make it easier for others to find you.": [
        ""
      ],
      "copy uri": [
        "kopyala"
      ],
      "hide uri": [
        "qr'yi gizle"
      ],
      "show uri": [
        "qr'yi göster"
      ],
      "Fetch messages": [
        "Satıcı mesajı"
      ],
      "Mailbox not initialized": [
        ""
      ],
      "Initialize mailbox": [
        ""
      ],
      "No messages yet.": [
        ""
      ],
      "Taler URI handler": [
        ""
      ],
      "Could not found a handler for the Taler URI": [
        ""
      ],
      "The uri read in the path parameter is not valid: \"%1$s\"": [
        ""
      ],
      "Digital cash payment": [
        "Dijital ödeme"
      ],
      "Digital cash refund": [
        ""
      ],
      "Digital cash withdrawal": [
        ""
      ],
      "Digital cash deposit": [
        "Dijital para yatırma"
      ],
      "Digital cash invoice": [
        "Dijital para yatırma"
      ],
      "Digital cash transfer": [
        ""
      ],
      "Digital cash recovery": [
        "Dijital para yatırma"
      ],
      "Development experiment": [
        ""
      ],
      "Return to wallet": [
        ""
      ],
      "Network is offline": [
        ""
      ],
      "Diagnostics timed out. Could not talk to the wallet backend.": [
        "Tanılar zaman aşımına uğradı. Cüzdan arka ucuyla konuşulamadı."
      ],
      "Problems detected:": [
        "Tespit edilen sorunlar:"
      ],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).": [
        "Lütfen %1$s ayarlarınızda, IndexedDB'nin etkinleştirildiğinizi kontrol edin (%2$s tercih adını kontrol edin)."
      ],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.": [
        "Cüzdan veritabanınız eski. Şu anda otomatik aktarım desteklenmiyor. Cüzdan veritabanını sıfırlamak için lütfen %1$s gidin."
      ],
      "here": [
        ""
      ],
      "Running diagnostics": [
        "Tanılamayı çalıştır"
      ],
      "Edit": [
        "Lütfen bir exchange seçin. Detayları seçiminizden önce inceleyebilirsiniz."
      ],
      "List of products": [
        "Ürün listesi"
      ],
      "free": [
        "ücretsiz"
      ],
      "Could not get provider information": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "tr",
  "completeness": 14
};

strings['sv'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "sv"
      },
      "Balance": [
        "Balans"
      ],
      "Backup": [
        ""
      ],
      "Notifications": [
        ""
      ],
      "QR Reader and Taler URI": [
        ""
      ],
      "Settings": [
        ""
      ],
      "Dev tools": [
        ""
      ],
      "Contacts": [
        ""
      ],
      "Mailbox": [
        ""
      ],
      "Search transactions": [
        ""
      ],
      "%1$s": [
        "%1$s"
      ],
      "KYC requirement": [
        ""
      ],
      "PENDING OPERATIONS": [
        ""
      ],
      "Error was thrown trying to: \"%1$s\"": [
        ""
      ],
      "Could not open the wallet database. Firefox is known to run into this problem under \"permanent private mode\".": [
        ""
      ],
      "Could not open the wallet database.": [
        ""
      ],
      "Unexpected request error, code: %1$s": [
        ""
      ],
      "more info": [
        ""
      ],
      "wrap text": [
        ""
      ],
      "copy content": [
        ""
      ],
      "less info": [
        ""
      ],
      "Retry operation": [
        ""
      ],
      "Loading": [
        ""
      ],
      "Could not load backup providers": [
        ""
      ],
      "No backup providers configured": [
        ""
      ],
      "Add provider": [
        ""
      ],
      "Sync all backups": [
        ""
      ],
      "Sync now": [
        ""
      ],
      "Last synced": [
        ""
      ],
      "Not synced": [
        ""
      ],
      "Expires in": [
        ""
      ],
      "show less": [
        ""
      ],
      "show more": [
        "visa mer"
      ],
      "There was an error loading the provider detail for &quot;%1$s&quot;": [
        ""
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [
        ""
      ],
      "See providers": [
        ""
      ],
      "Last backup": [
        ""
      ],
      "Back up": [
        ""
      ],
      "Provider fee": [
        ""
      ],
      "per year": [
        ""
      ],
      "Extend": [
        ""
      ],
      "terms has changed, extending the service will imply accepting the new terms of service": [
        ""
      ],
      "old": [
        ""
      ],
      "new": [
        ""
      ],
      "fee": [
        ""
      ],
      "storage": [
        ""
      ],
      "Remove provider": [
        ""
      ],
      "Pay": [
        "Godkän betalning"
      ],
      "Withdraw": [
        "Utbetalnings avgift"
      ],
      "This provider has reported an error": [
        ""
      ],
      "There is conflict with another backup from &quot;%1$s&quot;": [
        ""
      ],
      "Backup is not readable": [
        ""
      ],
      "Unknown backup problem: %1$s": [
        ""
      ],
      "service paid": [
        ""
      ],
      "Backup valid until": [
        ""
      ],
      "Cancel": [
        "Avbryt"
      ],
      "Open pay page": [
        ""
      ],
      "Open refund page": [
        ""
      ],
      "Open withdraw page": [
        "Utbetalnings avgift"
      ],
      "Your wallet is empty.": [
        "plånboken"
      ],
      "Get digital cash": [
        "Utbetalnings avgifter:"
      ],
      "Try the demo bank and withdraw test money.": [
        ""
      ],
      "Could not load the balance": [
        ""
      ],
      "Receive": [
        ""
      ],
      "Send %1$s": [
        "Välj %1$s"
      ],
      "This page has a withdrawal action.": [
        ""
      ],
      "This page has pay action.": [
        ""
      ],
      "This page has a refund action.": [
        ""
      ],
      "This page has a add exchange action.": [
        ""
      ],
      "Open add exchange page": [
        "Accepterade tjänsteleverantörer:"
      ],
      "This page has an add contact action.": [
        ""
      ],
      "Open add contact page": [
        "Utbetalnings avgift"
      ],
      "Taler Action": [
        ""
      ],
      "This page has a malformed taler uri.": [
        ""
      ],
      "Dismiss": [
        ""
      ],
      "this popup is being closed and you are being redirected to %1$s": [
        ""
      ],
      "Could not load purchase proposal details": [
        ""
      ],
      "Order Id": [
        ""
      ],
      "Summary": [
        ""
      ],
      "Amount": [
        ""
      ],
      "Choices": [
        ""
      ],
      "This purchase have %1$s choices": [
        ""
      ],
      "Merchant name": [
        ""
      ],
      "Merchant jurisdiction": [
        ""
      ],
      "Merchant address": [
        ""
      ],
      "Merchant logo": [
        ""
      ],
      "Merchant website": [
        ""
      ],
      "Merchant email": [
        ""
      ],
      "Merchant public key": [
        ""
      ],
      "Created at": [
        ""
      ],
      "Refund deadline": [
        ""
      ],
      "Auto refund": [
        ""
      ],
      "Pay deadline": [
        ""
      ],
      "Fulfillment URL": [
        ""
      ],
      "Fulfillment message": [
        ""
      ],
      "Max deposit fee": [
        "Depostitions avgift"
      ],
      "Exchanges": [
        "Leverantör av betaltjänster"
      ],
      "Could not load the status of the term of service": [
        ""
      ],
      "Select one option": [
        ""
      ],
      "Show terms of service": [
        ""
      ],
      "I accept the exchange terms of service": [
        ""
      ],
      "Review exchange terms of service": [
        ""
      ],
      "The exchange doesn't have a terms of service.": [
        ""
      ],
      "Skip it for now.": [
        ""
      ],
      "Format": [
        ""
      ],
      "Language": [
        ""
      ],
      "The exchange replied with a empty terms of service": [
        ""
      ],
      "No terms of service. The exchange replied with a empty document": [
        ""
      ],
      "No terms of service. The exchange replied with a empty text": [
        ""
      ],
      "Download Terms of Service": [
        ""
      ],
      "Hide terms of service": [
        ""
      ],
      "Amount is not valid": [
        ""
      ],
      "Bank account": [
        "Bankkonto"
      ],
      "Bitcoin address": [
        ""
      ],
      "BIC/IBAN": [
        ""
      ],
      "IBAN": [
        ""
      ],
      "Bitcoin transfer details": [
        ""
      ],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.": [
        ""
      ],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts": [
        ""
      ],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC": [
        ""
      ],
      "Ethereum transfer details": [
        ""
      ],
      "You need to wire to the service provider account.": [
        ""
      ],
      "Make sure the amount show %1$s ETH": [
        ""
      ],
      "Bank transfer details": [
        ""
      ],
      "Account": [
        ""
      ],
      "Bank host": [
        ""
      ],
      "BIC": [
        ""
      ],
      "Exchange": [
        ""
      ],
      "Reserve Pub": [
        ""
      ],
      "Step 1:": [
        ""
      ],
      "Copy this code and paste it into the subject/purpose field in your banking app or bank website": [
        ""
      ],
      "Subject": [
        ""
      ],
      "Step 2:": [
        ""
      ],
      "If you don't already have it in your banking favourites list, then copy and paste this IBAN and the name into the receiver fields in your banking app or website": [
        ""
      ],
      "Receiver name": [
        ""
      ],
      "Receiver postal code": [
        ""
      ],
      "Receiver town": [
        ""
      ],
      "Step 3:": [
        ""
      ],
      "Finish the wire transfer setting the amount in your banking app or website, then this withdrawal will proceed automatically.": [
        ""
      ],
      "Make sure ALL data is correct, including the subject; otherwise, the money will not arrive in this wallet. You can use the copy buttons (%1$s) to prevent typing errors or the \"payto://\" URI below to copy just one value.": [
        ""
      ],
      "Alternative if your bank already supports PayTo URI, you can use this %1$s link instead": [
        ""
      ],
      "Continue with banking app or website": [
        ""
      ],
      "Show QR code": [
        ""
      ],
      "Could not load transaction information": [
        ""
      ],
      "Bank account confirmation required for the transaction to complete.": [
        ""
      ],
      "KYC check required for the transaction to complete.": [
        ""
      ],
      "No additional information has been provided.": [
        ""
      ],
      "Follow this link to the%1$s %2$s": [
        ""
      ],
      "This transaction is pending to complete.": [
        ""
      ],
      "Retry": [
        ""
      ],
      "There was an error trying to complete the transaction.": [
        ""
      ],
      "This transaction was aborted.": [
        ""
      ],
      "This transaction failed.": [
        ""
      ],
      "Caution!": [
        ""
      ],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.": [
        ""
      ],
      "Confirm": [
        "Bekräfta"
      ],
      "Doing a cancellation while the transaction still active might result in lost coins. Do you still want to cancel the transaction?": [
        ""
      ],
      "No": [
        ""
      ],
      "Yes": [
        ""
      ],
      "Abort": [
        ""
      ],
      "Resume": [
        ""
      ],
      "Suspend": [
        ""
      ],
      "Delete": [
        ""
      ],
      "Withdrawal": [
        "Utbetalnings avgift"
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Select one of the accounts and use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Details": [
        ""
      ],
      "Withdrawal incomplete.": [
        "Utbetalnings avgift"
      ],
      "If you have already sent money to the service provider account it will wire it back at %1$s": [
        ""
      ],
      "Payment": [
        "Godkän betalning"
      ],
      "Refunds": [
        ""
      ],
      "%1$s %2$s on %3$s .": [
        "%1$s %2$s över %3$s"
      ],
      "Refund is in progress.": [
        ""
      ],
      "Merchant created a refund for this order but was not automatically picked up.": [
        ""
      ],
      "Offer": [
        ""
      ],
      "Accept": [
        ""
      ],
      "Confirmation code": [
        ""
      ],
      "Merchant": [
        ""
      ],
      "Invoice ID": [
        ""
      ],
      "Deposit": [
        "Depostitions avgift"
      ],
      "Wire transfer deadline.": [
        ""
      ],
      "Wire transfer still pending.": [
        ""
      ],
      "The service provider deadline to make the wire transfer is: %1$s": [
        ""
      ],
      "Wire transfer completed.": [
        ""
      ],
      " ": [
        ""
      ],
      "Transfer details": [
        "visa mer"
      ],
      "Wire transfer in progress.": [
        ""
      ],
      "Refresh": [
        "Återhämtnings avgift"
      ],
      "Refund": [
        ""
      ],
      "Purchase summary": [
        ""
      ],
      "Credit": [
        ""
      ],
      "Invoice": [
        ""
      ],
      "URI": [
        ""
      ],
      "Debit": [
        ""
      ],
      "Transfer": [
        ""
      ],
      "Lost": [
        ""
      ],
      "Reason": [
        ""
      ],
      "Denomination expired.": [
        ""
      ],
      "Denomination vanished.": [
        ""
      ],
      "Denomination is unoffered.": [
        ""
      ],
      "Fees": [
        ""
      ],
      "Total": [
        ""
      ],
      "Sent": [
        ""
      ],
      "Converted": [
        ""
      ],
      "Bank fee": [
        "Tjänsteleverantörer i plånboken:"
      ],
      "Price": [
        ""
      ],
      "Subtotal": [
        ""
      ],
      "Refunded": [
        ""
      ],
      "Transfer identification": [
        ""
      ],
      "copy": [
        ""
      ],
      "hide qr": [
        ""
      ],
      "forward to contact": [
        ""
      ],
      "show qr": [
        ""
      ],
      "Wire transfer need a confirmation. Go to the %1$s and check wire transfer operation to exchange account is complete.": [
        ""
      ],
      "bank site": [
        ""
      ],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins.": [
        ""
      ],
      "Exchange is ready to send the coins, withdrawal in progress.": [
        "Tjänsteleverantörer i plånboken:"
      ],
      "Send &nbsp; %1$s": [
        ""
      ],
      "Pay &nbsp; %1$s": [
        ""
      ],
      "Merchant message": [
        ""
      ],
      "Balance is not enough.": [
        "Du har ingen balans att visa. Behöver du\n %1$s att börja?\n"
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which exchange and auditors the merchant trust.": [
        ""
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which wire methods the merchant accepts.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to pay for this contract which is restricted.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to spend right know. There are some coins that need to be refreshed.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s available.": [
        ""
      ],
      "Provider is missing the global fee configuration, this likely means it is misconfigured.": [
        ""
      ],
      "Balance looks like it should be enough, but doesn't cover all fees requested by the merchant and payment processor. Please ensure there is at least %1$s %2$s more balance in your wallet or ask your merchant to cover more of the fees.": [
        ""
      ],
      "Pay with a mobile phone": [
        ""
      ],
      "Hide QR": [
        ""
      ],
      "Scan the QR code or &nbsp; %1$s": [
        ""
      ],
      "click here": [
        ""
      ],
      "Could not load the payment and balance status": [
        ""
      ],
      "Purchase": [
        ""
      ],
      "This purchase is age restricted.": [
        ""
      ],
      "Expires at": [
        ""
      ],
      "Already paid, you are going to be redirected to %1$s": [
        ""
      ],
      "Already paid": [
        ""
      ],
      "Already claimed": [
        ""
      ],
      "Could not load the refund status": [
        ""
      ],
      "You&apos;ve ignored the refund.": [
        ""
      ],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [
        "Säljaren %1$s erbjuder följande:"
      ],
      "Order amount": [
        "Återhämtnings avgift"
      ],
      "Could not load the list of exchanges": [
        "Accepterade tjänsteleverantörer:"
      ],
      "Could not load info from URI": [
        ""
      ],
      "Could not load the withdrawal details": [
        ""
      ],
      "under %1$s": [
        "Välj %1$s"
      ],
      "should be greater than zero": [
        ""
      ],
      "can't be greater than %1$s": [
        ""
      ],
      "Could not load exchange details info": [
        "Accepterade tjänsteleverantörer:"
      ],
      "Close": [
        ""
      ],
      "There is no exchange available for currency %1$s": [
        ""
      ],
      "You can add more exchanges from the settings.": [
        ""
      ],
      "Exchange %1$s is not available": [
        ""
      ],
      "Exchange status can view accessed from the settings.": [
        ""
      ],
      "Service fee description": [
        ""
      ],
      "Select %1$s exchange": [
        "Välj %1$s"
      ],
      "Reset": [
        ""
      ],
      "Use this exchange": [
        "Accepterade tjänsteleverantörer:"
      ],
      "Auditors": [
        ""
      ],
      "Doesn&apos;t have auditors": [
        ""
      ],
      "currency": [
        ""
      ],
      "Coin operations": [
        ""
      ],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.": [
        ""
      ],
      "Deposits": [
        "Depostitions avgift"
      ],
      "Denomination": [
        ""
      ],
      "Current": [
        ""
      ],
      "Selected": [
        "Välj"
      ],
      "Until": [
        ""
      ],
      "Withdrawals": [
        "Utbetalnings avgift"
      ],
      "Transfer operations": [
        ""
      ],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.": [
        ""
      ],
      "Operation": [
        ""
      ],
      "Fee": [
        ""
      ],
      "Wallet operations": [
        ""
      ],
      "Feature": [
        ""
      ],
      "Currency": [
        ""
      ],
      "This operation has already been completed.": [
        ""
      ],
      "This operation has already been aborted": [
        ""
      ],
      "This operation has started and should be completed in the bank.": [
        ""
      ],
      "You can confirm the operation in": [
        ""
      ],
      "this page": [
        "Accepterade tjänsteleverantörer:"
      ],
      "This operation has already been completed by another wallet.": [
        ""
      ],
      "This operation has already been used by another wallet.": [
        ""
      ],
      "It can be confirmed in": [
        ""
      ],
      "Age restriction": [
        ""
      ],
      "Withdraw &nbsp; %1$s": [
        ""
      ],
      "Withdraw to a mobile phone": [
        ""
      ],
      "Short description of the invoice": [
        ""
      ],
      "Create": [
        ""
      ],
      "Short description of the transfer": [
        ""
      ],
      "Could not load the invoice payment status": [
        ""
      ],
      "Receive &nbsp; %1$s": [
        ""
      ],
      "Could not load known bank accounts": [
        ""
      ],
      "Could not load supported wire methods": [
        ""
      ],
      "No wire methods supported for this currency": [
        ""
      ],
      "Try another wire method.": [
        ""
      ],
      "Known accounts for %1$s": [
        "Accepterade tjänsteleverantörer:"
      ],
      "To add a new account first select the account type.": [
        ""
      ],
      "Unable add this account": [
        ""
      ],
      "Add": [
        ""
      ],
      "IBAN accounts": [
        "Övervisa till bank konto"
      ],
      "Alias": [
        ""
      ],
      "Int. Account Number": [
        ""
      ],
      "Account name": [
        "Övervisa till bank konto"
      ],
      "KYC": [
        ""
      ],
      "Taler accounts": [
        "Övervisa till bank konto"
      ],
      "Host": [
        ""
      ],
      "Bitcoin accounts": [
        "Bankkonto"
      ],
      "Address": [
        ""
      ],
      "Can't be empty": [
        ""
      ],
      "Bitcoin Account": [
        "Bankkonto"
      ],
      "Taler Bank": [
        ""
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "Required": [
        ""
      ],
      "International Bank Account Number": [
        ""
      ],
      "Could not load balance information": [
        ""
      ],
      "Could not load fee for amount %1$s": [
        ""
      ],
      "A currency or an amount should be indicated": [
        ""
      ],
      "There is no enough balance to make a deposit for currency %1$s": [
        ""
      ],
      "Send %1$s to your account": [
        ""
      ],
      "There is no account to make a deposit for currency %1$s": [
        ""
      ],
      "Add account": [
        "Övervisa till bank konto"
      ],
      "Select account": [
        "Övervisa till bank konto"
      ],
      "Manage accounts": [
        "Bankkonto"
      ],
      "Brut amount": [
        "Återhämtnings avgift"
      ],
      "Deposit fee": [
        "Depostitions avgift"
      ],
      "Net amount": [
        "Depostitions avgift"
      ],
      "Deposit&nbsp;%1$s %2$s": [
        "Depostitions avgift"
      ],
      "Need approval in the Bank": [
        ""
      ],
      "Waiting for wire transfer to complete": [
        ""
      ],
      "Exchange is waiting the wire transfer": [
        ""
      ],
      "Payment in progress": [
        ""
      ],
      "Executing refund...": [
        ""
      ],
      "Refreshing coins...": [
        ""
      ],
      "Deposit in progress": [
        "Depostitions avgift"
      ],
      "Waiting to be paid": [
        ""
      ],
      "Receiving the transfer": [
        ""
      ],
      "Waiting to be received": [
        ""
      ],
      "Denomination expired": [
        ""
      ],
      "Denomination vanished": [
        ""
      ],
      "Denomination unoffered": [
        ""
      ],
      "ABORTED": [
        ""
      ],
      "FAILED": [
        ""
      ],
      "Could not load the list of transactions": [
        ""
      ],
      "unknown scope": [
        ""
      ],
      "Send": [
        ""
      ],
      "Your transaction history is empty for this currency.": [
        ""
      ],
      "Review terms of service": [
        ""
      ],
      "Provider URL": [
        ""
      ],
      "Please review and accept this provider&apos;s terms of service": [
        ""
      ],
      "Pricing": [
        ""
      ],
      "free of charge": [
        ""
      ],
      "%1$s per year of service": [
        ""
      ],
      "Storage": [
        ""
      ],
      "%1$s megabytes of storage per year of service": [
        ""
      ],
      "Accept terms of service": [
        ""
      ],
      "Add backup provider": [
        ""
      ],
      "Backup providers may charge for their service": [
        ""
      ],
      "URL": [
        ""
      ],
      "Name": [
        ""
      ],
      "Next": [
        ""
      ],
      "Navigator": [
        ""
      ],
      "Automatically open wallet": [
        ""
      ],
      "Open the wallet when a payment action is found.": [
        ""
      ],
      "Version Info": [
        ""
      ],
      "Web Extension": [
        ""
      ],
      "The version of wallet core is not supported. (supported version: %1$s, wallet version: %2$s)": [
        ""
      ],
      "Exchange compatibility": [
        ""
      ],
      "Merchant compatibility": [
        ""
      ],
      "Bank compatibility": [
        ""
      ],
      "Wallet Core compatibility": [
        ""
      ],
      "Enable developer mode": [
        ""
      ],
      "Show more information and options in the UI": [
        ""
      ],
      "Display": [
        ""
      ],
      "Current Language": [
        ""
      ],
      "Show backup feature": [
        ""
      ],
      "Backup integration still in beta.": [
        ""
      ],
      "Show P2P alias feature": [
        ""
      ],
      "P2P aliases integration still in beta.": [
        ""
      ],
      "Show suspend/resume transaction": [
        ""
      ],
      "Prevent transaction from doing network request.": [
        ""
      ],
      "Show refresh transaction type in the transaction list": [
        ""
      ],
      "Refresh transaction will be hidden by default if the refresh operation doesn't have fee.": [
        ""
      ],
      "Show more account types on deposit": [
        ""
      ],
      "Extends the UI to more payment target types.": [
        ""
      ],
      "Show JSON on error": [
        ""
      ],
      "Print more information about the error. Useful for debugging.": [
        ""
      ],
      "Allow HTTP connections": [
        ""
      ],
      "Using HTTP connection may be faster but unsafe (wallet restart required)": [
        ""
      ],
      "Lang selector": [
        ""
      ],
      "Allows to manually change the language of the UI. Otherwise it will be automatically selected by your browser configuration.": [
        ""
      ],
      "Edit exchange management": [
        "Tjänsteleverantörer i plånboken:"
      ],
      "Allows to see the list of exchange, remove, add and switch before withdrawal.": [
        ""
      ],
      "Select terms of service format": [
        ""
      ],
      "Allows to render the terms of service on different format selected by the user.": [
        ""
      ],
      "Show wallet activity": [
        ""
      ],
      "Show the wallet notification and observability event in the UI.": [
        ""
      ],
      "Enable v1 contract": [
        ""
      ],
      "v1 is under development and wallet-core converts v1 contract to v0. Enable this options for testing": [
        ""
      ],
      "GNU Taler Wallet installed!": [
        ""
      ],
      "You can open the wallet using the combination %1$s .": [
        ""
      ],
      "Also pinning the GNU Taler Wallet to your browser allows you to quick access without keyboard:": [
        ""
      ],
      "Click the puzzle icon": [
        ""
      ],
      "Search for GNU Taler Wallet": [
        ""
      ],
      "Click the pin icon": [
        ""
      ],
      "Next Steps": [
        ""
      ],
      "Try the demo": [
        ""
      ],
      "Click here to open the wallet activity tab.": [
        ""
      ],
      "Events": [
        ""
      ],
      "Active tasks": [
        ""
      ],
      "All events": [
        ""
      ],
      "Balance change": [
        "Balans"
      ],
      "Backup failed": [
        ""
      ],
      "Transaction updated": [
        ""
      ],
      "Exchange updated": [
        "Tjänsteleverantörer i plånboken:"
      ],
      "Idle": [
        ""
      ],
      "task.%1$s": [
        "%1$s"
      ],
      "wallet.%1$s(%2$s)": [
        ""
      ],
      "Bank account info changed": [
        "Bankkonto"
      ],
      "Contact added": [
        ""
      ],
      "Contact deleted": [
        ""
      ],
      "Mailbox message added": [
        ""
      ],
      "Mailbox message deleted": [
        ""
      ],
      "Type": [
        ""
      ],
      "Id": [
        ""
      ],
      "Since": [
        ""
      ],
      "Next try": [
        ""
      ],
      "Error": [
        ""
      ],
      "Transaction": [
        ""
      ],
      "No exchange yet": [
        ""
      ],
      "Status": [
        ""
      ],
      "Terms of Service": [
        ""
      ],
      "Last Update": [
        ""
      ],
      "Actions": [
        ""
      ],
      "ok": [
        ""
      ],
      "pending": [
        ""
      ],
      "proposed": [
        ""
      ],
      "unknown (exchange status should be updated)": [
        ""
      ],
      "Reload": [
        ""
      ],
      "Purge": [
        ""
      ],
      "Make regional": [
        ""
      ],
      "Make global": [
        ""
      ],
      "Forget ToS": [
        ""
      ],
      "Debug tools": [
        ""
      ],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?": [
        ""
      ],
      "reset": [
        ""
      ],
      "TESTING: This may delete all your coin, proceed with caution": [
        ""
      ],
      "run gc": [
        ""
      ],
      "import database": [
        ""
      ],
      "export database": [
        ""
      ],
      "Clear exchange key cache": [
        ""
      ],
      "Database exported at %1$s %2$s to download": [
        ""
      ],
      "Inject Taler support in all pages": [
        ""
      ],
      "Enabling this option will make `window.taler` be available in all sites": [
        ""
      ],
      "Exchange Entries": [
        "Leverantör av betaltjänster"
      ],
      "Add an exchange": [
        "Accepterade tjänsteleverantörer:"
      ],
      "Log levels": [
        ""
      ],
      "Coins": [
        "# Mynt"
      ],
      "usable coins": [
        ""
      ],
      "id": [
        ""
      ],
      "denom": [
        ""
      ],
      "status": [
        ""
      ],
      "from refresh?": [
        ""
      ],
      "age key count": [
        "Övervisa till bank konto"
      ],
      "spent coins": [
        ""
      ],
      "click to show": [
        ""
      ],
      "URI is not valid. Taler URI should start with \"taler://\"": [
        ""
      ],
      "After the URI type it should follow a '/' with more information.": [
        ""
      ],
      "This URI type is not supported": [
        ""
      ],
      "This URI requires the bank host and operation id separated by /": [
        ""
      ],
      "This URI requires the merchant host, order id and session id separated by /": [
        ""
      ],
      "This URI requires the merchant host and order id separated by /": [
        ""
      ],
      "This URI requires the exchange host and key id separated by /": [
        ""
      ],
      "This URI requires the merchant host and template id separated by /": [
        ""
      ],
      "This URI requires the host key and provider list separated by /": [
        ""
      ],
      "This URI requires the experiment id": [
        ""
      ],
      "This URI requires the exchange host": [
        ""
      ],
      "This URI requires the alias type, the alias and the mailbox URI separated by /": [
        ""
      ],
      "This URI requires string after the first /": [
        ""
      ],
      "The bank host is invalid": [
        ""
      ],
      "The merchant host is invalid": [
        ""
      ],
      "The URI should end with /": [
        ""
      ],
      "The exchange host is invalid": [
        ""
      ],
      "The contact is invalid": [
        ""
      ],
      "The amount is invalid": [
        ""
      ],
      "A parameter is invalid": [
        ""
      ],
      "checking...": [
        ""
      ],
      "Unexpected error happen reading the camera: %1$s": [
        ""
      ],
      "Could not found a QR code in the file": [
        ""
      ],
      "Unexpected error happen reading the file: %1$s": [
        ""
      ],
      "Scan a QR code or enter taler:// URI below": [
        ""
      ],
      "Pay invoice": [
        ""
      ],
      "Withdrawal from bank": [
        "Utbetalnings avgift"
      ],
      "Claim refund": [
        ""
      ],
      "Accept payment": [
        ""
      ],
      "Complete order": [
        ""
      ],
      "Restore wallet": [
        ""
      ],
      "Enable experiment": [
        ""
      ],
      "Withdraw from exchange": [
        ""
      ],
      "Add exchange": [
        "Accepterade tjänsteleverantörer:"
      ],
      "Notify transaction": [
        ""
      ],
      "Add contact": [
        "Övervisa till bank konto"
      ],
      "The exchange URL is invalid.": [
        ""
      ],
      "The exchange URL should end with '/'": [
        ""
      ],
      "Couldn't found an exchange in the URL specified.": [
        ""
      ],
      "HTTP request failed to %1$s: %2$s": [
        ""
      ],
      "HTTP request failed to %1$s": [
        ""
      ],
      "The merchant URL is invalid.": [
        ""
      ],
      "The merchant URL should end with '/'": [
        ""
      ],
      "Couldn't found an merchant in the URL specified.": [
        ""
      ],
      "The bank URL is invalid.": [
        ""
      ],
      "The bank URL should end with '/'": [
        ""
      ],
      "Couldn't found an bank in the URL specified.": [
        ""
      ],
      "Could not load exchanges": [
        "Accepterade tjänsteleverantörer:"
      ],
      "Choose a currency to proceed or add another exchange": [
        ""
      ],
      "Known currencies": [
        ""
      ],
      "Specify the origin": [
        ""
      ],
      "Use previous origins:": [
        ""
      ],
      "From another bank account": [
        "Övervisa till bank konto"
      ],
      "From another wallet": [
        ""
      ],
      "From a %1$s URI or QR code": [
        ""
      ],
      "Enter URI here": [
        ""
      ],
      "Specify the destination": [
        ""
      ],
      "Use previous destinations:": [
        ""
      ],
      "To another bank account": [
        "Övervisa till bank konto"
      ],
      "To another wallet": [
        ""
      ],
      "To a %1$s URI or QR code": [
        ""
      ],
      "Afghanistan": [
        ""
      ],
      "Albania": [
        ""
      ],
      "Algeria": [
        ""
      ],
      "United States": [
        ""
      ],
      "Andorra": [
        ""
      ],
      "Angola": [
        ""
      ],
      "Antigua and Barbuda": [
        ""
      ],
      "Argentina": [
        ""
      ],
      "Armenia": [
        ""
      ],
      "Australia": [
        ""
      ],
      "Austria": [
        ""
      ],
      "Azerbaijan": [
        ""
      ],
      "Bahamas": [
        ""
      ],
      "Bahrain": [
        ""
      ],
      "Bangladesh": [
        ""
      ],
      "Barbados": [
        ""
      ],
      "Botswana": [
        ""
      ],
      "Belarus": [
        ""
      ],
      "Belgium": [
        ""
      ],
      "Belize": [
        ""
      ],
      "Benin": [
        ""
      ],
      "Bhutan": [
        ""
      ],
      "Bolivia": [
        ""
      ],
      "Bosnia and Herzegovina": [
        ""
      ],
      "Brazil": [
        ""
      ],
      "United Kingdom": [
        ""
      ],
      "Brunei": [
        ""
      ],
      "Bulgaria": [
        ""
      ],
      "Burkina Faso": [
        ""
      ],
      "Myanmar": [
        ""
      ],
      "Burundi": [
        ""
      ],
      "Cambodia": [
        ""
      ],
      "Cameroon": [
        ""
      ],
      "Canada": [
        ""
      ],
      "Cape Verde": [
        ""
      ],
      "Central African Republic": [
        ""
      ],
      "Chad": [
        ""
      ],
      "Chile": [
        ""
      ],
      "China": [
        ""
      ],
      "Colombia": [
        ""
      ],
      "Comoros": [
        ""
      ],
      "Democratic Republic of the Congo": [
        ""
      ],
      "Republic of the Congo": [
        ""
      ],
      "Costa Rica": [
        ""
      ],
      "Croatia": [
        ""
      ],
      "Cuba": [
        ""
      ],
      "Cyprus": [
        ""
      ],
      "Czech Republic": [
        ""
      ],
      "Denmark": [
        ""
      ],
      "Djibouti": [
        ""
      ],
      "Dominican Republic": [
        ""
      ],
      "Netherlands": [
        ""
      ],
      "Timor-Leste": [
        ""
      ],
      "Ecuador": [
        ""
      ],
      "Egypt": [
        ""
      ],
      "United Arab Emirates": [
        ""
      ],
      "Equatorial Guinea": [
        ""
      ],
      "Eritrea": [
        ""
      ],
      "Estonia": [
        ""
      ],
      "Ethiopia": [
        ""
      ],
      "Fiji": [
        ""
      ],
      "Philippines": [
        ""
      ],
      "Finland": [
        ""
      ],
      "France": [
        "Avbryt"
      ],
      "Gabon": [
        ""
      ],
      "Gambia": [
        ""
      ],
      "Georgia": [
        ""
      ],
      "Germany": [
        ""
      ],
      "Ghana": [
        ""
      ],
      "Greece": [
        ""
      ],
      "Grenada": [
        ""
      ],
      "Guatemala": [
        ""
      ],
      "Guinea-Bissau": [
        ""
      ],
      "Guinea": [
        ""
      ],
      "Guyana": [
        ""
      ],
      "Haiti": [
        ""
      ],
      "Honduras": [
        ""
      ],
      "Hungary": [
        ""
      ],
      "Kiribati": [
        ""
      ],
      "Iceland": [
        ""
      ],
      "India": [
        ""
      ],
      "Indonesia": [
        ""
      ],
      "Iran": [
        ""
      ],
      "Iraq": [
        ""
      ],
      "Ireland": [
        ""
      ],
      "Israel": [
        ""
      ],
      "Italy": [
        ""
      ],
      "Ivory Coast": [
        ""
      ],
      "Jamaica": [
        ""
      ],
      "Japan": [
        ""
      ],
      "Jordan": [
        ""
      ],
      "Kazakhstan": [
        ""
      ],
      "Kenya": [
        ""
      ],
      "Saint Kitts and Nevis": [
        ""
      ],
      "Kuwait": [
        ""
      ],
      "Kyrgyzstan": [
        ""
      ],
      "Laos": [
        ""
      ],
      "Latvia": [
        ""
      ],
      "Lebanon": [
        ""
      ],
      "Liberia": [
        ""
      ],
      "Libya": [
        ""
      ],
      "Liechtenstein": [
        ""
      ],
      "Lithuania": [
        ""
      ],
      "Luxembourg": [
        ""
      ],
      "North Macedonia": [
        ""
      ],
      "Madagascar": [
        ""
      ],
      "Malawi": [
        ""
      ],
      "Malaysia": [
        ""
      ],
      "Maldives": [
        ""
      ],
      "Mali": [
        ""
      ],
      "Malta": [
        ""
      ],
      "Marshall Islands": [
        ""
      ],
      "Mauritania": [
        ""
      ],
      "Mauritius": [
        ""
      ],
      "Mexico": [
        ""
      ],
      "Micronesia": [
        ""
      ],
      "Moldova": [
        ""
      ],
      "Monaco": [
        ""
      ],
      "Mongolia": [
        ""
      ],
      "Morocco": [
        ""
      ],
      "Lesotho": [
        ""
      ],
      "Mozambique": [
        ""
      ],
      "Namibia": [
        ""
      ],
      "Nauru": [
        ""
      ],
      "Nepal": [
        ""
      ],
      "New Zealand": [
        ""
      ],
      "Vanuatu": [
        ""
      ],
      "Nicaragua": [
        ""
      ],
      "Nigeria": [
        ""
      ],
      "Niger": [
        ""
      ],
      "North Korea": [
        ""
      ],
      "Norway": [
        ""
      ],
      "Oman": [
        ""
      ],
      "Pakistan": [
        ""
      ],
      "Palau": [
        ""
      ],
      "Panama": [
        ""
      ],
      "Papua New Guinea": [
        ""
      ],
      "Paraguay": [
        ""
      ],
      "Peru": [
        ""
      ],
      "Poland": [
        ""
      ],
      "Portugal": [
        ""
      ],
      "Qatar": [
        ""
      ],
      "Romania": [
        ""
      ],
      "Russia": [
        ""
      ],
      "Rwanda": [
        ""
      ],
      "Saint Lucia": [
        ""
      ],
      "El Salvador": [
        ""
      ],
      "Samoa": [
        ""
      ],
      "San Marino": [
        ""
      ],
      "São Tomé and Príncipe": [
        ""
      ],
      "Saudi Arabia": [
        ""
      ],
      "Senegal": [
        ""
      ],
      "Serbia": [
        ""
      ],
      "Seychelles": [
        ""
      ],
      "Sierra Leone": [
        ""
      ],
      "Singapore": [
        ""
      ],
      "Slovakia": [
        ""
      ],
      "Slovenia": [
        ""
      ],
      "Solomon Islands": [
        ""
      ],
      "Somalia": [
        ""
      ],
      "South Africa": [
        ""
      ],
      "South Korea": [
        ""
      ],
      "Spain": [
        ""
      ],
      "Sri Lanka": [
        ""
      ],
      "Sudan": [
        ""
      ],
      "Suriname": [
        ""
      ],
      "Eswatini": [
        ""
      ],
      "Sweden": [
        ""
      ],
      "Switzerland": [
        ""
      ],
      "Syria": [
        ""
      ],
      "Taiwan": [
        ""
      ],
      "Tajikistan": [
        ""
      ],
      "Tanzania": [
        ""
      ],
      "Thailand": [
        ""
      ],
      "Togo": [
        ""
      ],
      "Tonga": [
        ""
      ],
      "Trinidad and Tobago": [
        ""
      ],
      "Tunisia": [
        ""
      ],
      "Turkey": [
        ""
      ],
      "Tuvalu": [
        "Värde"
      ],
      "Uganda": [
        ""
      ],
      "Ukraine": [
        ""
      ],
      "Uruguay": [
        ""
      ],
      "Uzbekistan": [
        ""
      ],
      "Venezuela": [
        ""
      ],
      "Vietnam": [
        ""
      ],
      "Yemen": [
        ""
      ],
      "Zambia": [
        ""
      ],
      "Zimbabwe": [
        ""
      ],
      "Afghan": [
        ""
      ],
      "Albanian": [
        ""
      ],
      "Algerian": [
        ""
      ],
      "American": [
        ""
      ],
      "Andorran": [
        ""
      ],
      "Angolan": [
        ""
      ],
      "Antiguan": [
        ""
      ],
      "Argentinean": [
        ""
      ],
      "Armenian": [
        ""
      ],
      "Australian": [
        ""
      ],
      "Austrian": [
        ""
      ],
      "Azerbaijani": [
        ""
      ],
      "Bahamian": [
        ""
      ],
      "Bahraini": [
        ""
      ],
      "Bangladeshi": [
        ""
      ],
      "Barbadian": [
        ""
      ],
      "Batswana": [
        ""
      ],
      "Belarusian": [
        ""
      ],
      "Belgian": [
        ""
      ],
      "Belizean": [
        ""
      ],
      "Beninese": [
        ""
      ],
      "Bhutanese": [
        ""
      ],
      "Bolivian": [
        ""
      ],
      "Bosnian": [
        ""
      ],
      "Brazilian": [
        ""
      ],
      "British": [
        ""
      ],
      "Bruneian": [
        ""
      ],
      "Bulgarian": [
        ""
      ],
      "Burkinabe": [
        ""
      ],
      "Burmese": [
        ""
      ],
      "Burundian": [
        ""
      ],
      "Cambodian": [
        ""
      ],
      "Cameroonian": [
        ""
      ],
      "Canadian": [
        ""
      ],
      "Cape Verdean": [
        ""
      ],
      "Central African": [
        ""
      ],
      "Chadian": [
        ""
      ],
      "Chilean": [
        ""
      ],
      "Chinese": [
        ""
      ],
      "Colombian": [
        ""
      ],
      "Comoran": [
        ""
      ],
      "Congolese": [
        ""
      ],
      "Costa Rican": [
        ""
      ],
      "Croatian": [
        ""
      ],
      "Cuban": [
        ""
      ],
      "Cypriot": [
        ""
      ],
      "Czech": [
        ""
      ],
      "Danish": [
        ""
      ],
      "Djiboutian": [
        ""
      ],
      "Dominican": [
        ""
      ],
      "Dutch": [
        ""
      ],
      "East Timorese": [
        ""
      ],
      "Ecuadorean": [
        ""
      ],
      "Egyptian": [
        ""
      ],
      "Emirati": [
        ""
      ],
      "Equatorial Guinean": [
        ""
      ],
      "Eritrean": [
        ""
      ],
      "Estonian": [
        ""
      ],
      "Ethiopian": [
        ""
      ],
      "Fijian": [
        ""
      ],
      "Filipino": [
        ""
      ],
      "Finnish": [
        ""
      ],
      "French": [
        ""
      ],
      "Gabonese": [
        ""
      ],
      "Gambian": [
        ""
      ],
      "Georgian": [
        ""
      ],
      "German": [
        ""
      ],
      "Ghanaian": [
        ""
      ],
      "Greek": [
        ""
      ],
      "Grenadian": [
        ""
      ],
      "Guatemalan": [
        ""
      ],
      "Guinea-Bissauan": [
        ""
      ],
      "Guinean": [
        ""
      ],
      "Guyanese": [
        ""
      ],
      "Haitian": [
        ""
      ],
      "Honduran": [
        ""
      ],
      "Hungarian": [
        ""
      ],
      "I-Kiribati": [
        ""
      ],
      "Icelander": [
        ""
      ],
      "Indian": [
        ""
      ],
      "Indonesian": [
        ""
      ],
      "Iranian": [
        ""
      ],
      "Iraqi": [
        ""
      ],
      "Irish": [
        ""
      ],
      "Israeli": [
        ""
      ],
      "Italian": [
        ""
      ],
      "Ivorian": [
        ""
      ],
      "Jamaican": [
        ""
      ],
      "Japanese": [
        ""
      ],
      "Jordanian": [
        ""
      ],
      "Kazakhstani": [
        ""
      ],
      "Kenyan": [
        ""
      ],
      "Kittian and Nevisian": [
        ""
      ],
      "Kuwaiti": [
        ""
      ],
      "Kyrgyz": [
        ""
      ],
      "Laotian": [
        ""
      ],
      "Latvian": [
        ""
      ],
      "Lebanese": [
        ""
      ],
      "Liberian": [
        ""
      ],
      "Libyan": [
        ""
      ],
      "Liechtensteiner": [
        ""
      ],
      "Lithuanian": [
        ""
      ],
      "Luxembourger": [
        ""
      ],
      "Macedonian": [
        ""
      ],
      "Malagasy": [
        ""
      ],
      "Malawian": [
        ""
      ],
      "Malaysian": [
        ""
      ],
      "Maldivian": [
        ""
      ],
      "Malian": [
        ""
      ],
      "Maltese": [
        ""
      ],
      "Marshallese": [
        ""
      ],
      "Mauritanian": [
        ""
      ],
      "Mauritian": [
        ""
      ],
      "Mexican": [
        ""
      ],
      "Micronesian": [
        ""
      ],
      "Moldovan": [
        ""
      ],
      "Monacan": [
        ""
      ],
      "Mongolian": [
        ""
      ],
      "Moroccan": [
        ""
      ],
      "Mosotho": [
        ""
      ],
      "Mozambican": [
        ""
      ],
      "Namibian": [
        ""
      ],
      "Nauruan": [
        ""
      ],
      "Nepalese": [
        ""
      ],
      "New Zealander": [
        ""
      ],
      "Ni-Vanuatu": [
        ""
      ],
      "Nicaraguan": [
        ""
      ],
      "Nigerian": [
        ""
      ],
      "Nigerien": [
        ""
      ],
      "North Korean": [
        ""
      ],
      "Norwegian": [
        ""
      ],
      "Omani": [
        ""
      ],
      "Pakistani": [
        ""
      ],
      "Palauan": [
        ""
      ],
      "Panamanian": [
        ""
      ],
      "Papua New Guinean": [
        ""
      ],
      "Paraguayan": [
        ""
      ],
      "Peruvian": [
        ""
      ],
      "Polish": [
        ""
      ],
      "Portuguese": [
        ""
      ],
      "Qatari": [
        ""
      ],
      "Romanian": [
        ""
      ],
      "Russian": [
        ""
      ],
      "Rwandan": [
        ""
      ],
      "Saint Lucian": [
        ""
      ],
      "Salvadoran": [
        ""
      ],
      "Samoan": [
        ""
      ],
      "San Marinese": [
        ""
      ],
      "Sao Tomean": [
        ""
      ],
      "Saudi": [
        ""
      ],
      "Senegalese": [
        ""
      ],
      "Serbian": [
        ""
      ],
      "Seychellois": [
        ""
      ],
      "Sierra Leonean": [
        ""
      ],
      "Singaporean": [
        ""
      ],
      "Slovakian": [
        ""
      ],
      "Slovenian": [
        ""
      ],
      "Solomon Islander": [
        ""
      ],
      "Somali": [
        ""
      ],
      "South African": [
        ""
      ],
      "South Korean": [
        ""
      ],
      "Spanish": [
        ""
      ],
      "Sri Lankan": [
        ""
      ],
      "Sudanese": [
        ""
      ],
      "Surinamer": [
        ""
      ],
      "Swazi": [
        ""
      ],
      "Swedish": [
        ""
      ],
      "Swiss": [
        ""
      ],
      "Syrian": [
        ""
      ],
      "Taiwanese": [
        ""
      ],
      "Tajik": [
        ""
      ],
      "Tanzanian": [
        ""
      ],
      "Thai": [
        ""
      ],
      "Togolese": [
        ""
      ],
      "Tongan": [
        ""
      ],
      "Trinidadian or Tobagonian": [
        ""
      ],
      "Tunisian": [
        ""
      ],
      "Turkish": [
        ""
      ],
      "Tuvaluan": [
        "Värde"
      ],
      "Ugandan": [
        ""
      ],
      "Ukrainian": [
        ""
      ],
      "Uruguayan": [
        ""
      ],
      "Uzbekistani": [
        ""
      ],
      "Venezuelan": [
        ""
      ],
      "Vietnamese": [
        ""
      ],
      "Yemenite": [
        ""
      ],
      "Zambian": [
        ""
      ],
      "Zimbabwean": [
        ""
      ],
      "🇦🇫": [
        ""
      ],
      "🇦🇱": [
        ""
      ],
      "🇩🇿": [
        ""
      ],
      "🇺🇸": [
        ""
      ],
      "🇦🇩": [
        ""
      ],
      "🇦🇴": [
        ""
      ],
      "🇦🇬": [
        ""
      ],
      "🇦🇷": [
        ""
      ],
      "🇦🇲": [
        ""
      ],
      "🇦🇺": [
        ""
      ],
      "🇦🇹": [
        ""
      ],
      "🇦🇿": [
        ""
      ],
      "🇧🇸": [
        ""
      ],
      "🇧🇭": [
        ""
      ],
      "🇧🇩": [
        ""
      ],
      "🇧🇧": [
        ""
      ],
      "🇧🇼": [
        ""
      ],
      "🇧🇾": [
        ""
      ],
      "🇧🇪": [
        ""
      ],
      "🇧🇿": [
        ""
      ],
      "🇧🇯": [
        ""
      ],
      "🇧🇹": [
        ""
      ],
      "🇧🇴": [
        ""
      ],
      "🇧🇦": [
        ""
      ],
      "🇧🇷": [
        ""
      ],
      "🇬🇧": [
        ""
      ],
      "🇧🇳": [
        ""
      ],
      "🇧🇬": [
        ""
      ],
      "🇧🇫": [
        ""
      ],
      "🇲🇲": [
        ""
      ],
      "🇧🇮": [
        ""
      ],
      "🇰🇭": [
        ""
      ],
      "🇨🇲": [
        ""
      ],
      "🇨🇦": [
        ""
      ],
      "🇨🇻": [
        ""
      ],
      "🇨🇫": [
        ""
      ],
      "🇨🇱": [
        ""
      ],
      "🇨🇳": [
        ""
      ],
      "🇨🇴": [
        ""
      ],
      "🇨🇩": [
        ""
      ],
      "🇨🇬": [
        ""
      ],
      "🇨🇷": [
        ""
      ],
      "🇨🇺": [
        ""
      ],
      "🇨🇾": [
        ""
      ],
      "🇨🇿": [
        ""
      ],
      "🇩🇰": [
        ""
      ],
      "🇩🇯": [
        ""
      ],
      "🇩🇴": [
        ""
      ],
      "🇳🇱": [
        ""
      ],
      "🇹🇱": [
        ""
      ],
      "🇪🇨": [
        ""
      ],
      "🇪🇬": [
        ""
      ],
      "🇦🇪": [
        ""
      ],
      "🇬🇶": [
        ""
      ],
      "🇪🇷": [
        ""
      ],
      "🇪🇪": [
        ""
      ],
      "🇪🇹": [
        ""
      ],
      "🇫🇯": [
        ""
      ],
      "🇵🇭": [
        ""
      ],
      "🇫🇮": [
        ""
      ],
      "🇫🇷": [
        ""
      ],
      "🇬🇦": [
        ""
      ],
      "🇬🇲": [
        ""
      ],
      "🇬🇪": [
        ""
      ],
      "UAE Dirham": [
        ""
      ],
      "Afghani": [
        ""
      ],
      "Lek": [
        ""
      ],
      "Armenian Dram": [
        ""
      ],
      "Netherlands Antillean Guilder": [
        ""
      ],
      "Kwanza": [
        ""
      ],
      "Argentine Peso": [
        ""
      ],
      "Australian Dollar": [
        ""
      ],
      "Aruban Florin": [
        ""
      ],
      "Azerbaijan Manat": [
        ""
      ],
      "Convertible Mark": [
        ""
      ],
      "Barbados Dollar": [
        ""
      ],
      "Taka": [
        ""
      ],
      "Bulgarian Lev": [
        ""
      ],
      "Bahraini Dinar": [
        ""
      ],
      "Burundi Franc": [
        ""
      ],
      "Bermudian Dollar": [
        ""
      ],
      "Brunei Dollar": [
        ""
      ],
      "Boliviano": [
        ""
      ],
      "Mvdol": [
        ""
      ],
      "Brazilian Real": [
        ""
      ],
      "Bahamian Dollar": [
        ""
      ],
      "Ngultrum": [
        ""
      ],
      "Pula": [
        ""
      ],
      "Belarusian Ruble": [
        ""
      ],
      "Belize Dollar": [
        ""
      ],
      "Canadian Dollar": [
        ""
      ],
      "Congolese Franc": [
        ""
      ],
      "WIR Euro": [
        ""
      ],
      "Swiss Franc": [
        ""
      ],
      "WIR Franc": [
        ""
      ],
      "Unidad de Fomento": [
        ""
      ],
      "Chilean Peso": [
        ""
      ],
      "Yuan Renminbi": [
        ""
      ],
      "Colombian Peso": [
        ""
      ],
      "Unidad de Valor Real": [
        ""
      ],
      "Costa Rican Colon": [
        ""
      ],
      "Cuban Peso": [
        ""
      ],
      "Cabo Verde Escudo": [
        ""
      ],
      "Czech Koruna": [
        ""
      ],
      "Djibouti Franc": [
        ""
      ],
      "Danish Krone": [
        ""
      ],
      "Dominican Peso": [
        ""
      ],
      "Algerian Dinar": [
        ""
      ],
      "Egyptian Pound": [
        ""
      ],
      "Nakfa": [
        ""
      ],
      "Ethiopian Birr": [
        ""
      ],
      "Euro": [
        ""
      ],
      "Fiji Dollar": [
        ""
      ],
      "Falkland Islands Pound": [
        ""
      ],
      "Pound Sterling": [
        ""
      ],
      "Lari": [
        ""
      ],
      "Ghana Cedi": [
        ""
      ],
      "Gibraltar Pound": [
        ""
      ],
      "Dalasi": [
        ""
      ],
      "Guinean Franc": [
        ""
      ],
      "Quetzal": [
        ""
      ],
      "Guyana Dollar": [
        ""
      ],
      "Hong Kong Dollar": [
        ""
      ],
      "Lempira": [
        ""
      ],
      "Gourde": [
        ""
      ],
      "Forint": [
        ""
      ],
      "Rupiah": [
        ""
      ],
      "New Israeli Sheqel": [
        ""
      ],
      "Indian Rupee": [
        ""
      ],
      "Iraqi Dinar": [
        ""
      ],
      "Iranian Rial": [
        ""
      ],
      "Iceland Krona": [
        ""
      ],
      "Jamaican Dollar": [
        ""
      ],
      "Jordanian Dinar": [
        ""
      ],
      "Yen": [
        ""
      ],
      "Kenyan Shilling": [
        ""
      ],
      "Som": [
        ""
      ],
      "Riel": [
        ""
      ],
      "Comorian Franc ": [
        ""
      ],
      "North Korean Won": [
        ""
      ],
      "Won": [
        ""
      ],
      "Kuwaiti Dinar": [
        ""
      ],
      "Cayman Islands Dollar": [
        ""
      ],
      "Tenge": [
        ""
      ],
      "Lao Kip": [
        ""
      ],
      "Lebanese Pound": [
        ""
      ],
      "Sri Lanka Rupee": [
        ""
      ],
      "Liberian Dollar": [
        ""
      ],
      "Loti": [
        ""
      ],
      "Libyan Dinar": [
        ""
      ],
      "Moroccan Dirham": [
        ""
      ],
      "Moldovan Leu": [
        ""
      ],
      "Malagasy Ariary": [
        ""
      ],
      "Denar": [
        ""
      ],
      "Kyat": [
        ""
      ],
      "Tugrik": [
        ""
      ],
      "Pataca": [
        ""
      ],
      "Ouguiya": [
        ""
      ],
      "Mauritius Rupee": [
        ""
      ],
      "Rufiyaa": [
        ""
      ],
      "Malawi Kwacha": [
        ""
      ],
      "Mexican Peso": [
        ""
      ],
      "Mexican Unidad de Inversion (UDI)": [
        ""
      ],
      "Malaysian Ringgit": [
        ""
      ],
      "Mozambique Metical": [
        ""
      ],
      "Namibia Dollar": [
        ""
      ],
      "Naira": [
        ""
      ],
      "Cordoba Oro": [
        ""
      ],
      "Norwegian Krone": [
        ""
      ],
      "Nepalese Rupee": [
        ""
      ],
      "New Zealand Dollar": [
        ""
      ],
      "Rial Omani": [
        ""
      ],
      "Balboa": [
        ""
      ],
      "Sol": [
        ""
      ],
      "Kina": [
        ""
      ],
      "Philippine Peso": [
        ""
      ],
      "Pakistan Rupee": [
        ""
      ],
      "Zloty": [
        ""
      ],
      "Guarani": [
        ""
      ],
      "Qatari Rial": [
        ""
      ],
      "Romanian Leu": [
        ""
      ],
      "Serbian Dinar": [
        ""
      ],
      "Russian Ruble": [
        ""
      ],
      "Rwanda Franc": [
        ""
      ],
      "Saudi Riyal": [
        ""
      ],
      "Solomon Islands Dollar": [
        ""
      ],
      "Seychelles Rupee": [
        ""
      ],
      "Sudanese Pound": [
        ""
      ],
      "Swedish Krona": [
        ""
      ],
      "Singapore Dollar": [
        ""
      ],
      "Saint Helena Pound": [
        ""
      ],
      "Leone": [
        ""
      ],
      "Somali Shilling": [
        ""
      ],
      "Surinam Dollar": [
        ""
      ],
      "South Sudanese Pound": [
        ""
      ],
      "Dobra": [
        ""
      ],
      "El Salvador Colon": [
        ""
      ],
      "Syrian Pound": [
        ""
      ],
      "Lilangeni": [
        ""
      ],
      "Baht": [
        ""
      ],
      "Somoni": [
        ""
      ],
      "Turkmenistan New Manat": [
        ""
      ],
      "Tunisian Dinar": [
        ""
      ],
      "Pa’anga": [
        ""
      ],
      "Turkish Lira": [
        ""
      ],
      "Trinidad and Tobago Dollar": [
        ""
      ],
      "New Taiwan Dollar": [
        ""
      ],
      "Tanzanian Shilling": [
        ""
      ],
      "Hryvnia": [
        ""
      ],
      "Uganda Shilling": [
        ""
      ],
      "US Dollar": [
        ""
      ],
      "US Dollar (Next day)": [
        ""
      ],
      "Uruguay Peso en Unidades Indexadas (UI)": [
        ""
      ],
      "Peso Uruguayo": [
        ""
      ],
      "Unidad Previsional": [
        ""
      ],
      "Uzbekistan Sum": [
        ""
      ],
      "Bolívar Soberano": [
        ""
      ],
      "Dong": [
        ""
      ],
      "Vatu": [
        ""
      ],
      "Tala": [
        ""
      ],
      "CFA Franc BEAC": [
        ""
      ],
      "East Caribbean Dollar": [
        ""
      ],
      "CFA Franc BCEAO": [
        ""
      ],
      "CFP Franc": [
        ""
      ],
      "Sucre": [
        ""
      ],
      "Yemeni Rial": [
        ""
      ],
      "Rand": [
        ""
      ],
      "Zambian Kwacha": [
        ""
      ],
      "Zimbabwe Gold": [
        ""
      ],
      "Afar": [
        ""
      ],
      "Abkhazian": [
        ""
      ],
      "Achinese": [
        ""
      ],
      "Acoli": [
        ""
      ],
      "Adangme": [
        ""
      ],
      "Adyghe": [
        ""
      ],
      "Avestan": [
        ""
      ],
      "Tunisian Arabic": [
        ""
      ],
      "Afrikaans": [
        ""
      ],
      "Afrihili": [
        ""
      ],
      "Aghem": [
        ""
      ],
      "Ainu": [
        ""
      ],
      "Akan": [
        ""
      ],
      "Akkadian": [
        ""
      ],
      "Alabama": [
        ""
      ],
      "Aleut": [
        ""
      ],
      "Gheg Albanian": [
        ""
      ],
      "Southern Altai": [
        ""
      ],
      "Amarik": [
        ""
      ],
      "Aragonese": [
        ""
      ],
      "Old English": [
        ""
      ],
      "Angika": [
        ""
      ],
      "Modern Standard Arabic": [
        ""
      ],
      "Arabik": [
        ""
      ],
      "Aramaic": [
        ""
      ],
      "Mapuche": [
        ""
      ],
      "Araona": [
        ""
      ],
      "Arapaho": [
        ""
      ],
      "Algerian Arabic": [
        ""
      ],
      "Arawak": [
        ""
      ],
      "Moroccan Arabic": [
        ""
      ],
      "Egyptian Arabic": [
        ""
      ],
      "Asu": [
        ""
      ],
      "Assamese": [
        ""
      ],
      "American Sign Language": [
        ""
      ],
      "Asturian": [
        ""
      ],
      "Avaric": [
        ""
      ],
      "Kotava": [
        ""
      ],
      "Awadhi": [
        ""
      ],
      "Aymara": [
        ""
      ],
      "South Azerbaijani": [
        ""
      ],
      "Bashkir": [
        ""
      ],
      "Baluchi": [
        ""
      ],
      "Balinese": [
        ""
      ],
      "Bavarian": [
        ""
      ],
      "Basaa": [
        ""
      ],
      "Bamun": [
        ""
      ],
      "Batak Toba": [
        ""
      ],
      "Ghomala": [
        ""
      ],
      "Belarus kasa": [
        ""
      ],
      "Beja": [
        ""
      ],
      "Bemba": [
        ""
      ],
      "Betawi": [
        ""
      ],
      "Bena": [
        ""
      ],
      "Bafut": [
        ""
      ],
      "Badaga": [
        ""
      ],
      "Bɔlgeria kasa": [
        ""
      ],
      "Bhojpuri": [
        ""
      ],
      "Bislama": [
        ""
      ],
      "Bikol": [
        ""
      ],
      "Bini": [
        ""
      ],
      "Banjar": [
        ""
      ],
      "Kom": [
        ""
      ],
      "Siksika": [
        ""
      ],
      "Bambara": [
        ""
      ],
      "Bengali kasa": [
        ""
      ],
      "Tibetan": [
        ""
      ],
      "Bishnupriya": [
        ""
      ],
      "Bakhtiari": [
        ""
      ],
      "Braj": [
        ""
      ],
      "Breton": [
        ""
      ],
      "Brahui": [
        ""
      ],
      "Bodo": [
        ""
      ],
      "Akoose": [
        ""
      ],
      "Buriat": [
        ""
      ],
      "Buginese": [
        ""
      ],
      "Bulu": [
        ""
      ],
      "Blin": [
        ""
      ],
      "Medumba": [
        ""
      ],
      "Catalan": [
        ""
      ],
      "Caddo": [
        ""
      ],
      "Carib": [
        ""
      ],
      "Cayuga": [
        ""
      ],
      "Atsam": [
        ""
      ],
      "Cebuano": [
        ""
      ],
      "Chechen": [
        ""
      ],
      "Chiga": [
        ""
      ],
      "Chibcha": [
        ""
      ],
      "Chamorro": [
        ""
      ],
      "Chagatai": [
        ""
      ],
      "Chuukese": [
        ""
      ],
      "Mari": [
        ""
      ],
      "Chinook Jargon": [
        ""
      ],
      "Choctaw": [
        ""
      ],
      "Chipewyan": [
        ""
      ],
      "Cherokee": [
        ""
      ],
      "Cheyenne": [
        ""
      ],
      "Central Kurdish": [
        ""
      ],
      "Corsican": [
        ""
      ],
      "Coptic": [
        ""
      ],
      "Capiznon": [
        ""
      ],
      "Cree": [
        ""
      ],
      "Crimean Turkish": [
        ""
      ],
      "Kashubian": [
        ""
      ],
      "Kyɛk kasa": [
        ""
      ],
      "Church Slavic": [
        ""
      ],
      "Chuvash": [
        ""
      ],
      "Welsh": [
        ""
      ],
      "Dakota": [
        ""
      ],
      "Dargwa": [
        ""
      ],
      "Taita": [
        ""
      ],
      "Austrian German": [
        ""
      ],
      "Swiss High German": [
        ""
      ],
      "Gyaaman": [
        ""
      ],
      "Delaware": [
        ""
      ],
      "Slave": [
        ""
      ],
      "Dogrib": [
        ""
      ],
      "Dinka": [
        ""
      ],
      "Zarma": [
        ""
      ],
      "Dogri": [
        ""
      ],
      "Lower Sorbian": [
        ""
      ],
      "Central Dusun": [
        ""
      ],
      "Duala": [
        ""
      ],
      "Middle Dutch": [
        ""
      ],
      "Divehi": [
        ""
      ],
      "Jola-Fonyi": [
        ""
      ],
      "Dyula": [
        ""
      ],
      "Dzongkha": [
        ""
      ],
      "Dazaga": [
        ""
      ],
      "Embu": [
        ""
      ],
      "Ewe": [
        ""
      ],
      "Efik": [
        ""
      ],
      "Emilian": [
        ""
      ],
      "Ancient Egyptian": [
        ""
      ],
      "Ekajuk": [
        ""
      ],
      "Greek kasa": [
        ""
      ],
      "Elamite": [
        ""
      ],
      "Australian English": [
        ""
      ],
      "Borɔfo": [
        ""
      ],
      "Canadian English": [
        ""
      ],
      "British English": [
        ""
      ],
      "Middle English": [
        ""
      ],
      "American English": [
        ""
      ],
      "Esperanto": [
        ""
      ],
      "Latin American Spanish": [
        ""
      ],
      "European Spanish": [
        ""
      ],
      "Mexican Spanish": [
        ""
      ],
      "Spain kasa": [
        ""
      ],
      "Central Yupik": [
        ""
      ],
      "Basque": [
        ""
      ],
      "Ewondo": [
        ""
      ],
      "Extremaduran": [
        ""
      ],
      "Fang": [
        ""
      ],
      "Pɛɛhyia kasa": [
        ""
      ],
      "Fanti": [
        ""
      ],
      "Fulah": [
        ""
      ],
      "Tornedalen Finnish": [
        ""
      ],
      "Faroese": [
        ""
      ],
      "Fon": [
        ""
      ],
      "Canadian French": [
        ""
      ],
      "Cajun French": [
        ""
      ],
      "Swiss French": [
        ""
      ],
      "Frɛnkye": [
        ""
      ],
      "Middle French": [
        ""
      ],
      "Old French": [
        ""
      ],
      "Arpitan": [
        ""
      ],
      "Northern Frisian": [
        ""
      ],
      "Eastern Frisian": [
        ""
      ],
      "Friulian": [
        ""
      ],
      "Western Frisian": [
        ""
      ],
      "Ga": [
        ""
      ],
      "Gagauz": [
        ""
      ],
      "Gan Chinese": [
        ""
      ],
      "Gayo": [
        ""
      ],
      "Gbaya": [
        ""
      ],
      "Zoroastrian Dari": [
        ""
      ],
      "Scottish Gaelic": [
        ""
      ],
      "Geez": [
        ""
      ],
      "Gilbertese": [
        ""
      ],
      "Galician": [
        ""
      ],
      "Gilaki": [
        ""
      ],
      "Middle High German": [
        ""
      ],
      "Old High German": [
        ""
      ],
      "Goan Konkani": [
        ""
      ],
      "Gondi": [
        ""
      ],
      "Gorontalo": [
        ""
      ],
      "Gothic": [
        ""
      ],
      "Grebo": [
        ""
      ],
      "Ancient Greek": [
        ""
      ],
      "Swiss German": [
        ""
      ],
      "Wayuu": [
        ""
      ],
      "Gujarati": [
        ""
      ],
      "Frafra": [
        ""
      ],
      "Gusii": [
        ""
      ],
      "Manx": [
        ""
      ],
      "Gwichʼin": [
        ""
      ],
      "Hausa": [
        ""
      ],
      "Haida": [
        ""
      ],
      "Hakka Chinese": [
        ""
      ],
      "Hawaiian": [
        ""
      ],
      "Hebrew": [
        ""
      ],
      "Fiji Hindi": [
        ""
      ],
      "Hindi": [
        ""
      ],
      "Hiligaynon": [
        ""
      ],
      "Hittite": [
        ""
      ],
      "Hmong": [
        ""
      ],
      "Hiri Motu": [
        ""
      ],
      "Upper Sorbian": [
        ""
      ],
      "Xiang Chinese": [
        ""
      ],
      "Hangri kasa": [
        ""
      ],
      "Hupa": [
        ""
      ],
      "Herero": [
        ""
      ],
      "Interlingua": [
        ""
      ],
      "Iban": [
        ""
      ],
      "Ibibio": [
        ""
      ],
      "Indonihyia kasa": [
        ""
      ],
      "Interlingue": [
        ""
      ],
      "Igbo": [
        ""
      ],
      "Sichuan Yi": [
        ""
      ],
      "Inupiaq": [
        ""
      ],
      "Iloko": [
        ""
      ],
      "Ingush": [
        ""
      ],
      "Ido": [
        ""
      ],
      "Icelandic": [
        ""
      ],
      "Italy kasa": [
        ""
      ],
      "Inuktitut": [
        ""
      ],
      "Ingrian": [
        ""
      ],
      "Gyapan kasa": [
        ""
      ],
      "Jamaican Creole English": [
        ""
      ],
      "Lojban": [
        ""
      ],
      "Ngomba": [
        ""
      ],
      "Machame": [
        ""
      ],
      "Judeo-Persian": [
        ""
      ],
      "Judeo-Arabic": [
        ""
      ],
      "Jutish": [
        ""
      ],
      "Gyabanis kasa": [
        ""
      ],
      "Kara-Kalpak": [
        ""
      ],
      "Kabyle": [
        ""
      ],
      "Kachin": [
        ""
      ],
      "Jju": [
        ""
      ],
      "Kamba": [
        ""
      ],
      "Kawi": [
        ""
      ],
      "Kabardian": [
        ""
      ],
      "Kanembu": [
        ""
      ],
      "Tyap": [
        ""
      ],
      "Makonde": [
        ""
      ],
      "Kabuverdianu": [
        ""
      ],
      "Kenyang": [
        ""
      ],
      "Koro": [
        ""
      ],
      "Kongo": [
        ""
      ],
      "Kaingang": [
        ""
      ],
      "Khasi": [
        ""
      ],
      "Khotanese": [
        ""
      ],
      "Koyra Chiini": [
        ""
      ],
      "Khowar": [
        ""
      ],
      "Kikuyu": [
        ""
      ],
      "Kirmanjki": [
        ""
      ],
      "Kuanyama": [
        ""
      ],
      "Kako": [
        ""
      ],
      "Kazakh": [
        ""
      ],
      "Kalaallisut": [
        ""
      ],
      "Kalenjin": [
        ""
      ],
      "Kimbundu": [
        ""
      ],
      "Kambodia kasa": [
        ""
      ],
      "Kannada": [
        ""
      ],
      "Komi-Permyak": [
        ""
      ],
      "Konkani": [
        ""
      ],
      "Korea kasa": [
        ""
      ],
      "Kosraean": [
        ""
      ],
      "Kpelle": [
        ""
      ],
      "Karachay-Balkar": [
        ""
      ],
      "Krio": [
        ""
      ],
      "Kinaray-a": [
        ""
      ],
      "Kanuri": [
        ""
      ],
      "Karelian": [
        ""
      ],
      "Kurukh": [
        ""
      ],
      "Shambala": [
        ""
      ],
      "Bafia": [
        ""
      ],
      "Colognian": [
        ""
      ],
      "Kashmiri": [
        ""
      ],
      "Kurdish": [
        ""
      ],
      "Kumyk": [
        ""
      ],
      "Kutenai": [
        ""
      ],
      "Komi": [
        ""
      ],
      "Cornish": [
        ""
      ],
      "Ladino": [
        ""
      ],
      "Langi": [
        ""
      ],
      "Lahnda": [
        ""
      ],
      "Latin": [
        ""
      ],
      "Lamba": [
        ""
      ],
      "Luxembourgish": [
        ""
      ],
      "Lezghian": [
        ""
      ],
      "Lingua Franca Nova": [
        ""
      ],
      "Ganda": [
        ""
      ],
      "Ligurian": [
        ""
      ],
      "Limburgish": [
        ""
      ],
      "Livonian": [
        ""
      ],
      "Lakota": [
        ""
      ],
      "Lombard": [
        ""
      ],
      "Lingala": [
        ""
      ],
      "Lao": [
        ""
      ],
      "Mongo": [
        ""
      ],
      "Lozi": [
        ""
      ],
      "Latgalian": [
        ""
      ],
      "Luba-Lulua": [
        ""
      ],
      "Luiseno": [
        ""
      ],
      "Luba-Katanga": [
        ""
      ],
      "Lunda": [
        ""
      ],
      "Luo": [
        ""
      ],
      "Mizo": [
        ""
      ],
      "Luyia": [
        ""
      ],
      "Literary Chinese": [
        ""
      ],
      "Laz": [
        ""
      ],
      "Madurese": [
        ""
      ],
      "Mafa": [
        ""
      ],
      "Magahi": [
        ""
      ],
      "Maithili": [
        ""
      ],
      "Makasar": [
        ""
      ],
      "Mandingo": [
        ""
      ],
      "Masai": [
        ""
      ],
      "Maba": [
        ""
      ],
      "Moksha": [
        ""
      ],
      "Mandar": [
        ""
      ],
      "Mende": [
        ""
      ],
      "Meru": [
        ""
      ],
      "Morisyen": [
        ""
      ],
      "Middle Irish": [
        ""
      ],
      "Makhuwa-Meetto": [
        ""
      ],
      "Metaʼ": [
        ""
      ],
      "Micmac": [
        ""
      ],
      "Maori": [
        ""
      ],
      "Minangkabau": [
        ""
      ],
      "Malayalam": [
        ""
      ],
      "Manchu": [
        ""
      ],
      "Manipuri": [
        ""
      ],
      "Mohawk": [
        ""
      ],
      "Mossi": [
        ""
      ],
      "Western Mari": [
        ""
      ],
      "Marathi": [
        ""
      ],
      "Malay kasa": [
        ""
      ],
      "Mundang": [
        ""
      ],
      "Multiple Languages": [
        ""
      ],
      "Creek": [
        ""
      ],
      "Mirandese": [
        ""
      ],
      "Marwari": [
        ""
      ],
      "Mentawai": [
        ""
      ],
      "Bɛɛmis kasa": [
        ""
      ],
      "Myene": [
        ""
      ],
      "Erzya": [
        ""
      ],
      "Mazanderani": [
        ""
      ],
      "Min Nan Chinese": [
        ""
      ],
      "Neapolitan": [
        ""
      ],
      "Nama": [
        ""
      ],
      "Norwegian Bokmål": [
        ""
      ],
      "North Ndebele": [
        ""
      ],
      "Low German": [
        ""
      ],
      "Nɛpal kasa": [
        ""
      ],
      "Newari": [
        ""
      ],
      "Ndonga": [
        ""
      ],
      "Nias": [
        ""
      ],
      "Niuean": [
        ""
      ],
      "Ao Naga": [
        ""
      ],
      "Flemish": [
        ""
      ],
      "Dɛɛkye": [
        ""
      ],
      "Kwasio": [
        ""
      ],
      "Ngiemboon": [
        ""
      ],
      "Norwegian Nynorsk": [
        ""
      ],
      "Nogai": [
        ""
      ],
      "Old Norse": [
        ""
      ],
      "Novial": [
        ""
      ],
      "NʼKo": [
        ""
      ],
      "South Ndebele": [
        ""
      ],
      "Northern Sotho": [
        ""
      ],
      "Nuer": [
        ""
      ],
      "Navajo": [
        ""
      ],
      "Classical Newari": [
        ""
      ],
      "Nyamwezi": [
        ""
      ],
      "Nyankole": [
        ""
      ],
      "Nyanja": [
        ""
      ],
      "Nyoro": [
        ""
      ],
      "Nzima": [
        ""
      ],
      "Occitan": [
        ""
      ],
      "Ojibwa": [
        ""
      ],
      "Oromo": [
        ""
      ],
      "Oriya": [
        ""
      ],
      "Osage": [
        ""
      ],
      "Ossetic": [
        ""
      ],
      "Ottoman Turkish": [
        ""
      ],
      "Pangasinan": [
        ""
      ],
      "Pahlavi": [
        ""
      ],
      "Pampanga": [
        ""
      ],
      "Papiamento": [
        "Godkän betalning"
      ],
      "Pungyabi kasa": [
        ""
      ],
      "Picard": [
        ""
      ],
      "Pennsylvania German": [
        ""
      ],
      "Plautdietsch": [
        ""
      ],
      "Old Persian": [
        ""
      ],
      "Palatine German": [
        ""
      ],
      "Phoenician": [
        ""
      ],
      "Pali": [
        ""
      ],
      "Pɔland kasa": [
        ""
      ],
      "Piedmontese": [
        ""
      ],
      "Pontic": [
        ""
      ],
      "Pohnpeian": [
        ""
      ],
      "Prussian": [
        ""
      ],
      "Old Provençal": [
        ""
      ],
      "Pashto": [
        ""
      ],
      "Brazilian Portuguese": [
        ""
      ],
      "Pɔɔtugal kasa": [
        ""
      ],
      "European Portuguese": [
        ""
      ],
      "Kʼicheʼ": [
        ""
      ],
      "Chimborazo Highland Quichua": [
        ""
      ],
      "Quechua": [
        ""
      ],
      "Rajasthani": [
        ""
      ],
      "Rapanui": [
        ""
      ],
      "Rarotongan": [
        ""
      ],
      "Romagnol": [
        ""
      ],
      "Riffian": [
        ""
      ],
      "Romansh": [
        ""
      ],
      "Rundi": [
        ""
      ],
      "Rombo": [
        ""
      ],
      "Moldavian": [
        ""
      ],
      "Romany": [
        ""
      ],
      "Root": [
        ""
      ],
      "Romenia kasa": [
        ""
      ],
      "Rotuman": [
        ""
      ],
      "Rusyn": [
        ""
      ],
      "Roviana": [
        ""
      ],
      "Aromanian": [
        ""
      ],
      "Rahyia kasa": [
        ""
      ],
      "Rwa": [
        ""
      ],
      "Rewanda kasa": [
        ""
      ],
      "Sandawe": [
        ""
      ],
      "Sakha": [
        ""
      ],
      "Samaritan Aramaic": [
        ""
      ],
      "Samburu": [
        ""
      ],
      "Sanskrit": [
        ""
      ],
      "Sasak": [
        ""
      ],
      "Santali": [
        ""
      ],
      "Saurashtra": [
        ""
      ],
      "Ngambay": [
        ""
      ],
      "Sangu": [
        ""
      ],
      "Sicilian": [
        ""
      ],
      "Scots": [
        ""
      ],
      "Sardinian": [
        ""
      ],
      "Sassarese Sardinian": [
        ""
      ],
      "Sindhi": [
        ""
      ],
      "Seneca": [
        ""
      ],
      "Sena": [
        ""
      ],
      "Seri": [
        ""
      ],
      "Selkup": [
        ""
      ],
      "Northern Sami": [
        ""
      ],
      "Koyraboro Senni": [
        ""
      ],
      "Old Irish": [
        ""
      ],
      "Sango": [
        ""
      ],
      "Samogitian": [
        ""
      ],
      "Tachelhit": [
        ""
      ],
      "Shan": [
        ""
      ],
      "Serbo-Croatian": [
        ""
      ],
      "Chadian Arabic": [
        ""
      ],
      "Sidamo": [
        ""
      ],
      "Sinhala": [
        ""
      ],
      "Slovak": [
        ""
      ],
      "Lower Silesian": [
        ""
      ],
      "Selayar": [
        ""
      ],
      "Southern Sami": [
        ""
      ],
      "Lule Sami": [
        ""
      ],
      "Inari Sami": [
        ""
      ],
      "Skolt Sami": [
        ""
      ],
      "Soninke": [
        ""
      ],
      "Shona": [
        ""
      ],
      "Sogdien": [
        ""
      ],
      "Somalia kasa": [
        ""
      ],
      "Sranan Tongo": [
        ""
      ],
      "Serer": [
        ""
      ],
      "Swati": [
        ""
      ],
      "Saho": [
        ""
      ],
      "Saterland Frisian": [
        ""
      ],
      "Southern Sotho": [
        ""
      ],
      "Sukuma": [
        ""
      ],
      "Susu": [
        ""
      ],
      "Sundanese": [
        ""
      ],
      "Sumerian": [
        ""
      ],
      "Sweden kasa": [
        ""
      ],
      "Comorian": [
        ""
      ],
      "Congo Swahili": [
        ""
      ],
      "Swahili": [
        ""
      ],
      "Classical Syriac": [
        ""
      ],
      "Syriac": [
        ""
      ],
      "Silesian": [
        ""
      ],
      "Tamil kasa": [
        ""
      ],
      "Tulu": [
        ""
      ],
      "Timne": [
        ""
      ],
      "Teso": [
        ""
      ],
      "Tereno": [
        ""
      ],
      "Telugu": [
        ""
      ],
      "Tetum": [
        ""
      ],
      "Taeland kasa": [
        ""
      ],
      "Tigre": [
        ""
      ],
      "Tigrinya": [
        ""
      ],
      "Tiv": [
        ""
      ],
      "Tokelau": [
        ""
      ],
      "Tsakhur": [
        ""
      ],
      "Turkmen": [
        ""
      ],
      "Klingon": [
        ""
      ],
      "Tlingit": [
        ""
      ],
      "Tagalog": [
        ""
      ],
      "Talysh": [
        ""
      ],
      "Tamashek": [
        ""
      ],
      "Tswana": [
        ""
      ],
      "Nyasa Tonga": [
        ""
      ],
      "Tok Pisin": [
        ""
      ],
      "Tɛɛki kasa": [
        ""
      ],
      "Turoyo": [
        ""
      ],
      "Taroko": [
        ""
      ],
      "Tsakonian": [
        ""
      ],
      "Tsimshian": [
        ""
      ],
      "Tsonga": [
        ""
      ],
      "Tatar": [
        ""
      ],
      "Muslim Tat": [
        ""
      ],
      "Tumbuka": [
        ""
      ],
      "Tasawaq": [
        ""
      ],
      "Twi": [
        ""
      ],
      "Tahitian": [
        ""
      ],
      "Tuvinian": [
        ""
      ],
      "Central Atlas Tamazight": [
        ""
      ],
      "Udmurt": [
        ""
      ],
      "Ugaritic": [
        ""
      ],
      "Uyghur": [
        ""
      ],
      "Ukren kasa": [
        ""
      ],
      "Umbundu": [
        ""
      ],
      "Unknown Language": [
        ""
      ],
      "Urdu kasa": [
        ""
      ],
      "Uzbek": [
        ""
      ],
      "Vai": [
        ""
      ],
      "Venetian": [
        ""
      ],
      "Veps": [
        ""
      ],
      "Venda": [
        ""
      ],
      "Viɛtnam kasa": [
        ""
      ],
      "West Flemish": [
        ""
      ],
      "Main-Franconian": [
        ""
      ],
      "Votic": [
        ""
      ],
      "Volapük": [
        ""
      ],
      "Võro": [
        ""
      ],
      "Vunjo": [
        ""
      ],
      "Walser": [
        ""
      ],
      "Wolaytta": [
        ""
      ],
      "Waray": [
        ""
      ],
      "Washo": [
        ""
      ],
      "Walloon": [
        ""
      ],
      "Warlpiri": [
        ""
      ],
      "Wolof": [
        ""
      ],
      "Wu Chinese": [
        ""
      ],
      "Kalmyk": [
        ""
      ],
      "Xhosa": [
        ""
      ],
      "Mingrelian": [
        ""
      ],
      "Soga": [
        ""
      ],
      "Yao": [
        ""
      ],
      "Yapese": [
        ""
      ],
      "Yangben": [
        ""
      ],
      "Yemba": [
        ""
      ],
      "Yiddish": [
        ""
      ],
      "Yoruba": [
        ""
      ],
      "Nheengatu": [
        ""
      ],
      "Cantonese": [
        ""
      ],
      "Zapotec": [
        ""
      ],
      "Zhuang": [
        ""
      ],
      "Blissymbols": [
        ""
      ],
      "Zeelandic": [
        ""
      ],
      "Zenaga": [
        ""
      ],
      "Standard Moroccan Tamazight": [
        ""
      ],
      "Simplified Chinese": [
        ""
      ],
      "Traditional Chinese": [
        ""
      ],
      "Kyaena kasa": [
        ""
      ],
      "Zuni": [
        ""
      ],
      "Zulu": [
        ""
      ],
      "No linguistic content": [
        ""
      ],
      "Zaza": [
        ""
      ],
      "Could not load user attention request": [
        ""
      ],
      "No notification left": [
        ""
      ],
      "Ignore": [
        ""
      ],
      "Invalid dev experiment URI.": [
        ""
      ],
      "URI: %1$s": [
        "%1$s"
      ],
      "Experiment": [
        ""
      ],
      "Insert lost denomination": [
        ""
      ],
      "Apply": [
        ""
      ],
      "Pending refresh": [
        ""
      ],
      "Could not load the transfer payment status": [
        ""
      ],
      "Could not load the status of the order template": [
        ""
      ],
      "required": [
        ""
      ],
      "Review order": [
        ""
      ],
      "Missing URI": [
        ""
      ],
      "Check the link.": [
        ""
      ],
      "Could not parse the recovery URI": [
        ""
      ],
      "Try another URI": [
        ""
      ],
      "Import backup, show info": [
        ""
      ],
      "Add new exchange": [
        "Accepterade tjänsteleverantörer:"
      ],
      "Add exchange for %1$s": [
        "Accepterade tjänsteleverantörer:"
      ],
      "Enter the URL of an exchange you trust.": [
        ""
      ],
      "An exchange has been found! Review the information and click next": [
        ""
      ],
      "This exchange is already in your list.": [
        "Tjänsteleverantörer i plånboken:"
      ],
      "Only exchange accessible through \"http\" and \"https\" are allowed.": [
        ""
      ],
      "This exchange protocol version is not supported: \" %1$s\".": [
        ""
      ],
      "This exchange currency \"%1$s\" doesn&apos;t match the expected currency %2$s.": [
        ""
      ],
      "No exchange found in that URL.": [
        "Accepterade tjänsteleverantörer:"
      ],
      "loading": [
        ""
      ],
      "Version": [
        ""
      ],
      "Can't use the URL: \"%1$s\"": [
        ""
      ],
      "Exchange URL": [
        ""
      ],
      "Add new contact": [
        "Övervisa till bank konto"
      ],
      "Enter the details of a contact you trust.": [
        ""
      ],
      "Contact info valid.": [
        ""
      ],
      "This contact is already in your list.": [
        ""
      ],
      "Review contact": [
        ""
      ],
      "Contact alias": [
        ""
      ],
      "Alias type": [
        ""
      ],
      "Mailbox service": [
        ""
      ],
      "Mailbox address": [
        ""
      ],
      "Contact source": [
        ""
      ],
      "Could not load the list of contacts": [
        ""
      ],
      "Source": [
        ""
      ],
      "Send Cash": [
        "Välj %1$s"
      ],
      "Request Cash": [
        ""
      ],
      "No contacts found.": [
        ""
      ],
      "Could not bank choices for account": [
        "Övervisa till bank konto"
      ],
      "Could not load the list of messages": [
        ""
      ],
      "Unexpected error when trying to initialize mailbox: %1$s": [
        ""
      ],
      "Unexpected error when trying to delete message: %1$s": [
        ""
      ],
      "Unexpected error when trying to fetch messages: %1$s": [
        ""
      ],
      "Date": [
        ""
      ],
      "You may copy your mailbox URI from below and register it at %1$s with your alias(es) to make it easier for others to find you.": [
        ""
      ],
      "copy uri": [
        ""
      ],
      "hide uri": [
        ""
      ],
      "show uri": [
        ""
      ],
      "Fetch messages": [
        ""
      ],
      "Mailbox not initialized": [
        ""
      ],
      "Initialize mailbox": [
        ""
      ],
      "No messages yet.": [
        ""
      ],
      "Taler URI handler": [
        ""
      ],
      "Could not found a handler for the Taler URI": [
        ""
      ],
      "The uri read in the path parameter is not valid: \"%1$s\"": [
        ""
      ],
      "Digital cash payment": [
        ""
      ],
      "Digital cash refund": [
        ""
      ],
      "Digital cash withdrawal": [
        ""
      ],
      "Digital cash deposit": [
        ""
      ],
      "Digital cash invoice": [
        "Utbetalnings avgifter:"
      ],
      "Digital cash transfer": [
        ""
      ],
      "Digital cash recovery": [
        ""
      ],
      "Development experiment": [
        ""
      ],
      "Return to wallet": [
        ""
      ],
      "Network is offline": [
        ""
      ],
      "Diagnostics timed out. Could not talk to the wallet backend.": [
        ""
      ],
      "Problems detected:": [
        ""
      ],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).": [
        ""
      ],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.": [
        ""
      ],
      "here": [
        ""
      ],
      "Running diagnostics": [
        ""
      ],
      "Edit": [
        ""
      ],
      "List of products": [
        ""
      ],
      "free": [
        ""
      ],
      "Could not get provider information": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "sv",
  "completeness": 4
};

strings['ru'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=3; plural=n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2;",
        "lang": "ru"
      },
      "Balance": [
        "Баланс"
      ],
      "Backup": [
        "Резервная копия"
      ],
      "Notifications": [
        ""
      ],
      "QR Reader and Taler URI": [
        "Считыватель QR-кодов и URI Taler"
      ],
      "Settings": [
        "Настройки"
      ],
      "Dev tools": [
        "Инструменты отладки"
      ],
      "Contacts": [
        ""
      ],
      "Mailbox": [
        ""
      ],
      "Search transactions": [
        "Комиссия транзакции"
      ],
      "%1$s": [
        "%1$s"
      ],
      "KYC requirement": [
        ""
      ],
      "PENDING OPERATIONS": [
        "ОЖИДАЮЩИЕ ОПЕРАЦИИ"
      ],
      "Error was thrown trying to: \"%1$s\"": [
        ""
      ],
      "Could not open the wallet database. Firefox is known to run into this problem under \"permanent private mode\".": [
        ""
      ],
      "Could not open the wallet database.": [
        ""
      ],
      "Unexpected request error, code: %1$s": [
        ""
      ],
      "more info": [
        ""
      ],
      "wrap text": [
        ""
      ],
      "copy content": [
        ""
      ],
      "less info": [
        ""
      ],
      "Retry operation": [
        "Операции кошелька"
      ],
      "Loading": [
        "Загружаются"
      ],
      "Could not load backup providers": [
        "Не удалось загрузить поставщиков резервного копирования"
      ],
      "No backup providers configured": [
        "Поставщики резервного копирования не настроены"
      ],
      "Add provider": [
        "Добавить сервис"
      ],
      "Sync all backups": [
        "Синхронизация всех резервных копий"
      ],
      "Sync now": [
        "Синхронизировать сейчас"
      ],
      "Last synced": [
        "Последняя синхронизация"
      ],
      "Not synced": [
        "Не синхронизировано"
      ],
      "Expires in": [
        "Срок действия истекает в"
      ],
      "show less": [
        "показать qr"
      ],
      "show more": [
        "показать qr"
      ],
      "There was an error loading the provider detail for &quot;%1$s&quot;": [
        "Произошла ошибка при загрузке сведений о поставщике для &quot; %1$s&quot;"
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [
        "Нет провайдера с url &quot;%1$s&quot;."
      ],
      "See providers": [
        "Посмотреть провайдеров"
      ],
      "Last backup": [
        "Последняя резервная копия"
      ],
      "Back up": [
        "Создать резервную копию"
      ],
      "Provider fee": [
        "Комиссия провайдера"
      ],
      "per year": [
        "в год"
      ],
      "Extend": [
        "Расширить"
      ],
      "terms has changed, extending the service will imply accepting the new terms of service": [
        "изменились условия, продление сервиса будет означать принятие новых условий предоставления услуг"
      ],
      "old": [
        "старый"
      ],
      "new": [
        "новый"
      ],
      "fee": [
        "комиссия"
      ],
      "storage": [
        "хранение"
      ],
      "Remove provider": [
        "Удалить провадер"
      ],
      "Pay": [
        "Платёж"
      ],
      "Withdraw": [
        "Снять средства"
      ],
      "This provider has reported an error": [
        "Этот провайдер сообщил об ошибке"
      ],
      "There is conflict with another backup from &quot;%1$s&quot;": [
        "Возник конфликт с другой резервной копией из %1$s"
      ],
      "Backup is not readable": [
        "Резервная копия не читается"
      ],
      "Unknown backup problem: %1$s": [
        "Неизвестная проблема резервного копирования: %1$s"
      ],
      "service paid": [
        "Услуга платная"
      ],
      "Backup valid until": [
        "Резервная копия действительна до"
      ],
      "Cancel": [
        "Отмена"
      ],
      "Open pay page": [
        "Открыть страницу оплаты"
      ],
      "Open refund page": [
        "Открыть страницу возврата средств"
      ],
      "Open withdraw page": [
        "Открыть страницу вывода средств"
      ],
      "Your wallet is empty.": [
        ""
      ],
      "Get digital cash": [
        "Получите цифровую наличность"
      ],
      "Try the demo bank and withdraw test money.": [
        ""
      ],
      "Could not load the balance": [
        "Не удалось загрузить страницу баланса"
      ],
      "Receive": [
        "Чек"
      ],
      "Send %1$s": [
        "Отправить %1$s"
      ],
      "This page has a withdrawal action.": [
        "На этой странице есть действие по выводу средств."
      ],
      "This page has pay action.": [
        "На этой странице есть платное действие."
      ],
      "This page has a refund action.": [
        "На этой странице есть действие по возврату средств."
      ],
      "This page has a add exchange action.": [
        "На этой странице есть действие по возврату средств."
      ],
      "Open add exchange page": [
        "Добавить Обменник"
      ],
      "This page has an add contact action.": [
        "На этой странице есть действие по возврату средств."
      ],
      "Open add contact page": [
        "Открыть страницу оплаты"
      ],
      "Taler Action": [
        "Действие Талер"
      ],
      "This page has a malformed taler uri.": [
        "На этой странице неправильно сформирован Taler URI."
      ],
      "Dismiss": [
        "Закрыть"
      ],
      "this popup is being closed and you are being redirected to %1$s": [
        "Это всплывающее окно закрывается и вы перенаправляетесь на %1$s"
      ],
      "Could not load purchase proposal details": [
        "Не удалось загрузить сведения о предложении покупки"
      ],
      "Order Id": [
        "Номер заказа"
      ],
      "Summary": [
        "Вкратце"
      ],
      "Amount": [
        "Сумма"
      ],
      "Choices": [
        ""
      ],
      "This purchase have %1$s choices": [
        ""
      ],
      "Merchant name": [
        "Название продавца"
      ],
      "Merchant jurisdiction": [
        "Юрисдикция продавца"
      ],
      "Merchant address": [
        "Адрес продавца"
      ],
      "Merchant logo": [
        "Логотип продавца"
      ],
      "Merchant website": [
        "Сайт продавца"
      ],
      "Merchant email": [
        "Email продавца"
      ],
      "Merchant public key": [
        "Публичный ключ продавца"
      ],
      "Created at": [
        "Создано в"
      ],
      "Refund deadline": [
        "Крайний срок возврата средств"
      ],
      "Auto refund": [
        "Автоматический возврат средств"
      ],
      "Pay deadline": [
        "Крайний срок оплаты"
      ],
      "Fulfillment URL": [
        "URL-адрес выполнения"
      ],
      "Fulfillment message": [
        "Сообщение о выполнении"
      ],
      "Max deposit fee": [
        "Максимальная комиссия за депозит"
      ],
      "Exchanges": [
        "Обменники"
      ],
      "Could not load the status of the term of service": [
        "Не удалось загрузить список известных обменников"
      ],
      "Select one option": [
        "Выберете одну опцию"
      ],
      "Show terms of service": [
        "Показать Условия использования"
      ],
      "I accept the exchange terms of service": [
        "Я принимаю эти Условия использования"
      ],
      "Review exchange terms of service": [
        "Ознакомиться с Условиями использования"
      ],
      "The exchange doesn't have a terms of service.": [
        "Обменник не имеет условий использования"
      ],
      "Skip it for now.": [
        ""
      ],
      "Format": [
        ""
      ],
      "Language": [
        ""
      ],
      "The exchange replied with a empty terms of service": [
        "Биржа ответитила с пустыми условиями использования"
      ],
      "No terms of service. The exchange replied with a empty document": [
        "Биржа ответитила с пустыми условиями использования"
      ],
      "No terms of service. The exchange replied with a empty text": [
        "Биржа ответитила с пустыми условиями использования"
      ],
      "Download Terms of Service": [
        "Скачать Условия использования"
      ],
      "Hide terms of service": [
        "Скрыть Условия использования"
      ],
      "Amount is not valid": [
        ""
      ],
      "Bank account": [
        "Баковский счёт"
      ],
      "Bitcoin address": [
        "Биткоин адрес"
      ],
      "BIC/IBAN": [
        "IBAN"
      ],
      "IBAN": [
        "IBAN"
      ],
      "Bitcoin transfer details": [
        "Подробности перевода биткоина"
      ],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.": [
        "Обменнику нужна транзакция с 3 выходами, один выход - это счёт обменника, а два других - это сегвит фейк адрес для метаданных с минимальной суммой."
      ],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts": [
        ""
      ],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC": [
        "Убедитесь что сумма показывает %1$s BTC, в противном случае вам придётся изменить базовую единицу на BTC"
      ],
      "Ethereum transfer details": [
        "Подробности банковского перевода"
      ],
      "You need to wire to the service provider account.": [
        ""
      ],
      "Make sure the amount show %1$s ETH": [
        ""
      ],
      "Bank transfer details": [
        "Подробности банковского перевода"
      ],
      "Account": [
        "Счёт"
      ],
      "Bank host": [
        "Хост банка"
      ],
      "BIC": [
        ""
      ],
      "Exchange": [
        "Обменник"
      ],
      "Reserve Pub": [
        ""
      ],
      "Step 1:": [
        ""
      ],
      "Copy this code and paste it into the subject/purpose field in your banking app or bank website": [
        ""
      ],
      "Subject": [
        "Причина"
      ],
      "Step 2:": [
        ""
      ],
      "If you don't already have it in your banking favourites list, then copy and paste this IBAN and the name into the receiver fields in your banking app or website": [
        ""
      ],
      "Receiver name": [
        "Имя получателя"
      ],
      "Receiver postal code": [
        "Имя получателя"
      ],
      "Receiver town": [
        "Имя получателя"
      ],
      "Step 3:": [
        ""
      ],
      "Finish the wire transfer setting the amount in your banking app or website, then this withdrawal will proceed automatically.": [
        ""
      ],
      "Make sure ALL data is correct, including the subject; otherwise, the money will not arrive in this wallet. You can use the copy buttons (%1$s) to prevent typing errors or the \"payto://\" URI below to copy just one value.": [
        ""
      ],
      "Alternative if your bank already supports PayTo URI, you can use this %1$s link instead": [
        ""
      ],
      "Continue with banking app or website": [
        ""
      ],
      "Show QR code": [
        ""
      ],
      "Could not load transaction information": [
        "Не удалось загрузить информацию о транзакции"
      ],
      "Bank account confirmation required for the transaction to complete.": [
        ""
      ],
      "KYC check required for the transaction to complete.": [
        ""
      ],
      "No additional information has been provided.": [
        ""
      ],
      "Follow this link to the%1$s %2$s": [
        ""
      ],
      "This transaction is pending to complete.": [
        "Эта транзакция не завершена"
      ],
      "Retry": [
        "Повторить попытку"
      ],
      "There was an error trying to complete the transaction.": [
        "При попытке завершить транзакцию произошла ошибка"
      ],
      "This transaction was aborted.": [
        "Эта транзакция не завершена"
      ],
      "This transaction failed.": [
        "Эта транзакция не завершена"
      ],
      "Caution!": [
        "Внимание!"
      ],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.": [
        "Если вы уже перевели деньги на обменник вы потеряете шанс получить монеты с нее."
      ],
      "Confirm": [
        "Подтвердить"
      ],
      "Doing a cancellation while the transaction still active might result in lost coins. Do you still want to cancel the transaction?": [
        ""
      ],
      "No": [
        ""
      ],
      "Yes": [
        ""
      ],
      "Abort": [
        ""
      ],
      "Resume": [
        ""
      ],
      "Suspend": [
        ""
      ],
      "Delete": [
        ""
      ],
      "Withdrawal": [
        "Вывод"
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Select one of the accounts and use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Details": [
        "Подробности"
      ],
      "Withdrawal incomplete.": [
        "Вывод"
      ],
      "If you have already sent money to the service provider account it will wire it back at %1$s": [
        "Если вы уже перевели деньги на обменник вы потеряете шанс получить монеты с нее."
      ],
      "Payment": [
        "Платёж"
      ],
      "Refunds": [
        "Возвраты"
      ],
      "%1$s %2$s on %3$s .": [
        "%1$s %2$s на %3$s"
      ],
      "Refund is in progress.": [
        "Возврат средств в выполняется."
      ],
      "Merchant created a refund for this order but was not automatically picked up.": [
        "Продавец создал возврат средств за этот заказ, но не был автоматически забран."
      ],
      "Offer": [
        "Предложение"
      ],
      "Accept": [
        "Принять"
      ],
      "Confirmation code": [
        ""
      ],
      "Merchant": [
        "Продавец"
      ],
      "Invoice ID": [
        "№ счёта-фактуры"
      ],
      "Deposit": [
        "Депозит"
      ],
      "Wire transfer deadline.": [
        "Подробности перевода биткоина"
      ],
      "Wire transfer still pending.": [
        ""
      ],
      "The service provider deadline to make the wire transfer is: %1$s": [
        ""
      ],
      "Wire transfer completed.": [
        ""
      ],
      " ": [
        ""
      ],
      "Transfer details": [
        "Подробности банковского перевода"
      ],
      "Wire transfer in progress.": [
        "Возврат средств в выполняется."
      ],
      "Refresh": [
        "Обновить"
      ],
      "Refund": [
        "Возврат"
      ],
      "Purchase summary": [
        "Сводка о покупке"
      ],
      "Credit": [
        "Кредит"
      ],
      "Invoice": [
        "Счёт-фактура"
      ],
      "URI": [
        "URI"
      ],
      "Debit": [
        "Дебит"
      ],
      "Transfer": [
        "Перевести"
      ],
      "Lost": [
        ""
      ],
      "Reason": [
        ""
      ],
      "Denomination expired.": [
        "Деноминация"
      ],
      "Denomination vanished.": [
        "Деноминация"
      ],
      "Denomination is unoffered.": [
        "Деноминация"
      ],
      "Fees": [
        ""
      ],
      "Total": [
        "Всего"
      ],
      "Sent": [
        ""
      ],
      "Converted": [
        ""
      ],
      "Bank fee": [
        "максимальная комиссия"
      ],
      "Price": [
        "Цена"
      ],
      "Subtotal": [
        "Всего"
      ],
      "Refunded": [
        "Возвращено на счёт"
      ],
      "Transfer identification": [
        "Операции переводов"
      ],
      "copy": [
        "копировать"
      ],
      "hide qr": [
        "спрятать qr"
      ],
      "forward to contact": [
        ""
      ],
      "show qr": [
        "показать qr"
      ],
      "Wire transfer need a confirmation. Go to the %1$s and check wire transfer operation to exchange account is complete.": [
        ""
      ],
      "bank site": [
        "Хост банка"
      ],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins.": [
        "Банк подтвердил перевод. Ожидание пока обменик отправит монеты"
      ],
      "Exchange is ready to send the coins, withdrawal in progress.": [
        ""
      ],
      "Send &nbsp; %1$s": [
        "Отправить &nbsp; %1$s"
      ],
      "Pay &nbsp; %1$s": [
        "Заплатить &nbsp; %1$s"
      ],
      "Merchant message": [
        "Сообщение продавца"
      ],
      "Balance is not enough.": [
        "Недостаточно средств на балансе."
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which exchange and auditors the merchant trust.": [
        ""
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which wire methods the merchant accepts.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to pay for this contract which is restricted.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to spend right know. There are some coins that need to be refreshed.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s available.": [
        ""
      ],
      "Provider is missing the global fee configuration, this likely means it is misconfigured.": [
        ""
      ],
      "Balance looks like it should be enough, but doesn't cover all fees requested by the merchant and payment processor. Please ensure there is at least %1$s %2$s more balance in your wallet or ask your merchant to cover more of the fees.": [
        ""
      ],
      "Pay with a mobile phone": [
        "Оплата с помощью мобильного телефона"
      ],
      "Hide QR": [
        "Скрыть QR"
      ],
      "Scan the QR code or &nbsp; %1$s": [
        "Отсканируйте QR код или &nbsp; %1$s"
      ],
      "click here": [
        "кликите чтобы показать"
      ],
      "Could not load the payment and balance status": [
        "Не удалось загрузить статус оплаты"
      ],
      "Purchase": [
        "Покупка"
      ],
      "This purchase is age restricted.": [
        ""
      ],
      "Expires at": [
        "Срок действия истекает в"
      ],
      "Already paid, you are going to be redirected to %1$s": [
        "Уже оплачено, вы будете перенаправлены на %1$s"
      ],
      "Already paid": [
        "Уже оплачено"
      ],
      "Already claimed": [
        "Уже заявлено"
      ],
      "Could not load the refund status": [
        "Не удалось загрузить статус возврата"
      ],
      "You&apos;ve ignored the refund.": [
        ""
      ],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [
        "Продавец «%1$s»‎ предлагает вам возврат средств."
      ],
      "Order amount": [
        "Сумма заказа"
      ],
      "Could not load the list of exchanges": [
        "Не удалось загрузить список известных обменников"
      ],
      "Could not load info from URI": [
        "Не удалось получить информацию из URI"
      ],
      "Could not load the withdrawal details": [
        "Не удалось загрузить сведения о предложении покупки"
      ],
      "under %1$s": [
        "Отправить %1$s"
      ],
      "should be greater than zero": [
        ""
      ],
      "can't be greater than %1$s": [
        ""
      ],
      "Could not load exchange details info": [
        "Не удалось загрузить комиссию за обмен"
      ],
      "Close": [
        "Закрыть"
      ],
      "There is no exchange available for currency %1$s": [
        "Не удалось найти ни одного обменника для валюты %1$s"
      ],
      "You can add more exchanges from the settings.": [
        ""
      ],
      "Exchange %1$s is not available": [
        ""
      ],
      "Exchange status can view accessed from the settings.": [
        ""
      ],
      "Service fee description": [
        "Описание комиссии за услугу"
      ],
      "Select %1$s exchange": [
        "Выберите %1$s обменник"
      ],
      "Reset": [
        "Сбросить"
      ],
      "Use this exchange": [
        "Использовать этот обменник"
      ],
      "Auditors": [
        "Аудиторы"
      ],
      "Doesn&apos;t have auditors": [
        "Не имеет аудиторов"
      ],
      "currency": [
        "валюта"
      ],
      "Coin operations": [
        "Операции моент"
      ],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.": [
        "Каждая операция в этом разделе может отличаться номиналом и действительна в течение определенного периода времени. Биржа будет взимать указанную сумму каждый раз, когда монета используется в такой операции."
      ],
      "Deposits": [
        "Депозиты"
      ],
      "Denomination": [
        "Деноминация"
      ],
      "Current": [
        "Валюта"
      ],
      "Selected": [
        ""
      ],
      "Until": [
        "до"
      ],
      "Withdrawals": [
        "Выводы средств"
      ],
      "Transfer operations": [
        "Операции переводов"
      ],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.": [
        "Каждая операция в этом разделе может отличаться в зависимости от типа перевода и действительна в течение определенного периода времени. Обменник будет взимать указанную сумму каждый раз при совершении перевода."
      ],
      "Operation": [
        "Операция"
      ],
      "Fee": [
        "Комиссия"
      ],
      "Wallet operations": [
        "Операции кошелька"
      ],
      "Feature": [
        "Возможность"
      ],
      "Currency": [
        "Валюта"
      ],
      "This operation has already been completed.": [
        "Эта транзакция не завершена"
      ],
      "This operation has already been aborted": [
        ""
      ],
      "This operation has started and should be completed in the bank.": [
        ""
      ],
      "You can confirm the operation in": [
        ""
      ],
      "this page": [
        "Использовать этот обменник"
      ],
      "This operation has already been completed by another wallet.": [
        ""
      ],
      "This operation has already been used by another wallet.": [
        ""
      ],
      "It can be confirmed in": [
        ""
      ],
      "Age restriction": [
        "Ограничения возраста"
      ],
      "Withdraw &nbsp; %1$s": [
        "Вывести &nbsp; %1$s"
      ],
      "Withdraw to a mobile phone": [
        "Вывести на мобильный телефон"
      ],
      "Short description of the invoice": [
        ""
      ],
      "Create": [
        "Создать"
      ],
      "Short description of the transfer": [
        ""
      ],
      "Could not load the invoice payment status": [
        "Не удалось загрузить статус оплаты"
      ],
      "Receive &nbsp; %1$s": [
        "Получить &nbsp; %1$s"
      ],
      "Could not load known bank accounts": [
        "Не удалось загрузить статус оплаты"
      ],
      "Could not load supported wire methods": [
        "Не удалось загрузить поставщиков резервного копирования"
      ],
      "No wire methods supported for this currency": [
        ""
      ],
      "Try another wire method.": [
        "На другой кошелек"
      ],
      "Known accounts for %1$s": [
        ""
      ],
      "To add a new account first select the account type.": [
        ""
      ],
      "Unable add this account": [
        ""
      ],
      "Add": [
        "Добавить"
      ],
      "IBAN accounts": [
        "Добавить Счёт"
      ],
      "Alias": [
        ""
      ],
      "Int. Account Number": [
        ""
      ],
      "Account name": [
        "Счёт"
      ],
      "KYC": [
        ""
      ],
      "Taler accounts": [
        "Выберете счёт"
      ],
      "Host": [
        ""
      ],
      "Bitcoin accounts": [
        "Баковский счёт"
      ],
      "Address": [
        "Строки адреса"
      ],
      "Can't be empty": [
        ""
      ],
      "Bitcoin Account": [
        "Баковский счёт"
      ],
      "Taler Bank": [
        "Действие Талер"
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "Required": [
        ""
      ],
      "International Bank Account Number": [
        ""
      ],
      "Could not load balance information": [
        "Не удалось загрузить информацию о транзакции"
      ],
      "Could not load fee for amount %1$s": [
        "Не удалось загрузить статус возврата"
      ],
      "A currency or an amount should be indicated": [
        ""
      ],
      "There is no enough balance to make a deposit for currency %1$s": [
        ""
      ],
      "Send %1$s to your account": [
        ""
      ],
      "There is no account to make a deposit for currency %1$s": [
        ""
      ],
      "Add account": [
        "Добавить Счёт"
      ],
      "Select account": [
        "Выберете счёт"
      ],
      "Manage accounts": [
        "Баковский счёт"
      ],
      "Brut amount": [
        "Сумма заказа"
      ],
      "Deposit fee": [
        "Комиссия депозита"
      ],
      "Net amount": [
        "Сумма заказа"
      ],
      "Deposit&nbsp;%1$s %2$s": [
        ""
      ],
      "Need approval in the Bank": [
        ""
      ],
      "Waiting for wire transfer to complete": [
        ""
      ],
      "Exchange is waiting the wire transfer": [
        ""
      ],
      "Payment in progress": [
        "Возврат средств в выполняется."
      ],
      "Executing refund...": [
        "Автоматический возврат средств"
      ],
      "Refreshing coins...": [
        ""
      ],
      "Deposit in progress": [
        "Возврат средств в выполняется."
      ],
      "Waiting to be paid": [
        ""
      ],
      "Receiving the transfer": [
        ""
      ],
      "Waiting to be received": [
        "К получению"
      ],
      "Denomination expired": [
        "Деноминация"
      ],
      "Denomination vanished": [
        "Деноминация"
      ],
      "Denomination unoffered": [
        "Деноминация"
      ],
      "ABORTED": [
        ""
      ],
      "FAILED": [
        ""
      ],
      "Could not load the list of transactions": [
        ""
      ],
      "unknown scope": [
        ""
      ],
      "Send": [
        "Осуществить перевод"
      ],
      "Your transaction history is empty for this currency.": [
        ""
      ],
      "Review terms of service": [
        ""
      ],
      "Provider URL": [
        "URL провайдера"
      ],
      "Please review and accept this provider&apos;s terms of service": [
        ""
      ],
      "Pricing": [
        ""
      ],
      "free of charge": [
        "комиссия за пополнение"
      ],
      "%1$s per year of service": [
        ""
      ],
      "Storage": [
        "Хранилище"
      ],
      "%1$s megabytes of storage per year of service": [
        ""
      ],
      "Accept terms of service": [
        ""
      ],
      "Add backup provider": [
        "Добавить провайдера резервной копии"
      ],
      "Backup providers may charge for their service": [
        ""
      ],
      "URL": [
        "URL"
      ],
      "Name": [
        "Название"
      ],
      "Next": [
        "Далее"
      ],
      "Navigator": [
        "Навигатор"
      ],
      "Automatically open wallet": [
        ""
      ],
      "Open the wallet when a payment action is found.": [
        ""
      ],
      "Version Info": [
        "Версия"
      ],
      "Web Extension": [
        "Расширение браузера"
      ],
      "The version of wallet core is not supported. (supported version: %1$s, wallet version: %2$s)": [
        ""
      ],
      "Exchange compatibility": [
        ""
      ],
      "Merchant compatibility": [
        ""
      ],
      "Bank compatibility": [
        ""
      ],
      "Wallet Core compatibility": [
        ""
      ],
      "Enable developer mode": [
        "Режим разработчика"
      ],
      "Show more information and options in the UI": [
        ""
      ],
      "Display": [
        "Отбражение"
      ],
      "Current Language": [
        ""
      ],
      "Show backup feature": [
        ""
      ],
      "Backup integration still in beta.": [
        ""
      ],
      "Show P2P alias feature": [
        ""
      ],
      "P2P aliases integration still in beta.": [
        ""
      ],
      "Show suspend/resume transaction": [
        ""
      ],
      "Prevent transaction from doing network request.": [
        ""
      ],
      "Show refresh transaction type in the transaction list": [
        ""
      ],
      "Refresh transaction will be hidden by default if the refresh operation doesn't have fee.": [
        ""
      ],
      "Show more account types on deposit": [
        ""
      ],
      "Extends the UI to more payment target types.": [
        ""
      ],
      "Show JSON on error": [
        ""
      ],
      "Print more information about the error. Useful for debugging.": [
        ""
      ],
      "Allow HTTP connections": [
        ""
      ],
      "Using HTTP connection may be faster but unsafe (wallet restart required)": [
        ""
      ],
      "Lang selector": [
        ""
      ],
      "Allows to manually change the language of the UI. Otherwise it will be automatically selected by your browser configuration.": [
        ""
      ],
      "Edit exchange management": [
        ""
      ],
      "Allows to see the list of exchange, remove, add and switch before withdrawal.": [
        ""
      ],
      "Select terms of service format": [
        "Показать Условия использования"
      ],
      "Allows to render the terms of service on different format selected by the user.": [
        ""
      ],
      "Show wallet activity": [
        ""
      ],
      "Show the wallet notification and observability event in the UI.": [
        ""
      ],
      "Enable v1 contract": [
        ""
      ],
      "v1 is under development and wallet-core converts v1 contract to v0. Enable this options for testing": [
        ""
      ],
      "GNU Taler Wallet installed!": [
        ""
      ],
      "You can open the wallet using the combination %1$s .": [
        ""
      ],
      "Also pinning the GNU Taler Wallet to your browser allows you to quick access without keyboard:": [
        ""
      ],
      "Click the puzzle icon": [
        ""
      ],
      "Search for GNU Taler Wallet": [
        ""
      ],
      "Click the pin icon": [
        ""
      ],
      "Next Steps": [
        "Следующий шаг"
      ],
      "Try the demo": [
        "Попробовать демо"
      ],
      "Click here to open the wallet activity tab.": [
        ""
      ],
      "Events": [
        ""
      ],
      "Active tasks": [
        ""
      ],
      "All events": [
        ""
      ],
      "Balance change": [
        "Баланс"
      ],
      "Backup failed": [
        "Резервная копия действительна до"
      ],
      "Transaction updated": [
        "Комиссия транзакции"
      ],
      "Exchange updated": [
        "Обменник"
      ],
      "Idle": [
        ""
      ],
      "task.%1$s": [
        "%1$s"
      ],
      "wallet.%1$s(%2$s)": [
        ""
      ],
      "Bank account info changed": [
        "Баковский счёт"
      ],
      "Contact added": [
        ""
      ],
      "Contact deleted": [
        ""
      ],
      "Mailbox message added": [
        ""
      ],
      "Mailbox message deleted": [
        ""
      ],
      "Type": [
        ""
      ],
      "Id": [
        ""
      ],
      "Since": [
        ""
      ],
      "Next try": [
        "Следующий шаг"
      ],
      "Error": [
        ""
      ],
      "Transaction": [
        "Комиссия транзакции"
      ],
      "No exchange yet": [
        ""
      ],
      "Status": [
        "статус"
      ],
      "Terms of Service": [
        "Условия использования"
      ],
      "Last Update": [
        ""
      ],
      "Actions": [
        "Действие Талер"
      ],
      "ok": [
        "ok"
      ],
      "pending": [
        ""
      ],
      "proposed": [
        ""
      ],
      "unknown (exchange status should be updated)": [
        ""
      ],
      "Reload": [
        "загрузка"
      ],
      "Purge": [
        ""
      ],
      "Make regional": [
        ""
      ],
      "Make global": [
        ""
      ],
      "Forget ToS": [
        "Забыть"
      ],
      "Debug tools": [
        "Инструменты отладки"
      ],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?": [
        ""
      ],
      "reset": [
        "сбросить"
      ],
      "TESTING: This may delete all your coin, proceed with caution": [
        ""
      ],
      "run gc": [
        ""
      ],
      "import database": [
        "импортировать базу данных"
      ],
      "export database": [
        "экспортировать базу данных"
      ],
      "Clear exchange key cache": [
        ""
      ],
      "Database exported at %1$s %2$s to download": [
        ""
      ],
      "Inject Taler support in all pages": [
        ""
      ],
      "Enabling this option will make `window.taler` be available in all sites": [
        ""
      ],
      "Exchange Entries": [
        "Обменники"
      ],
      "Add an exchange": [
        ""
      ],
      "Log levels": [
        ""
      ],
      "Coins": [
        "Монеты"
      ],
      "usable coins": [
        ""
      ],
      "id": [
        ""
      ],
      "denom": [
        ""
      ],
      "status": [
        "статус"
      ],
      "from refresh?": [
        ""
      ],
      "age key count": [
        ""
      ],
      "spent coins": [
        ""
      ],
      "click to show": [
        "кликите чтобы показать"
      ],
      "URI is not valid. Taler URI should start with \"taler://\"": [
        ""
      ],
      "After the URI type it should follow a '/' with more information.": [
        ""
      ],
      "This URI type is not supported": [
        ""
      ],
      "This URI requires the bank host and operation id separated by /": [
        ""
      ],
      "This URI requires the merchant host, order id and session id separated by /": [
        ""
      ],
      "This URI requires the merchant host and order id separated by /": [
        ""
      ],
      "This URI requires the exchange host and key id separated by /": [
        ""
      ],
      "This URI requires the merchant host and template id separated by /": [
        ""
      ],
      "This URI requires the host key and provider list separated by /": [
        ""
      ],
      "This URI requires the experiment id": [
        ""
      ],
      "This URI requires the exchange host": [
        ""
      ],
      "This URI requires the alias type, the alias and the mailbox URI separated by /": [
        ""
      ],
      "This URI requires string after the first /": [
        ""
      ],
      "The bank host is invalid": [
        ""
      ],
      "The merchant host is invalid": [
        ""
      ],
      "The URI should end with /": [
        ""
      ],
      "The exchange host is invalid": [
        ""
      ],
      "The contact is invalid": [
        ""
      ],
      "The amount is invalid": [
        ""
      ],
      "A parameter is invalid": [
        ""
      ],
      "checking...": [
        ""
      ],
      "Unexpected error happen reading the camera: %1$s": [
        ""
      ],
      "Could not found a QR code in the file": [
        ""
      ],
      "Unexpected error happen reading the file: %1$s": [
        ""
      ],
      "Scan a QR code or enter taler:// URI below": [
        ""
      ],
      "Pay invoice": [
        "Цифровой счёт-фактура"
      ],
      "Withdrawal from bank": [
        "Вывод"
      ],
      "Claim refund": [
        "Уже возвращено"
      ],
      "Accept payment": [
        "Принять"
      ],
      "Complete order": [
        ""
      ],
      "Restore wallet": [
        ""
      ],
      "Enable experiment": [
        ""
      ],
      "Withdraw from exchange": [
        "Вывести на мобильный телефон"
      ],
      "Add exchange": [
        "Добавить Обменник"
      ],
      "Notify transaction": [
        ""
      ],
      "Add contact": [
        "Добавить Счёт"
      ],
      "The exchange URL is invalid.": [
        ""
      ],
      "The exchange URL should end with '/'": [
        ""
      ],
      "Couldn't found an exchange in the URL specified.": [
        "Не удалось найти ни одного обменника для валюты %1$s"
      ],
      "HTTP request failed to %1$s: %2$s": [
        ""
      ],
      "HTTP request failed to %1$s": [
        ""
      ],
      "The merchant URL is invalid.": [
        ""
      ],
      "The merchant URL should end with '/'": [
        ""
      ],
      "Couldn't found an merchant in the URL specified.": [
        ""
      ],
      "The bank URL is invalid.": [
        ""
      ],
      "The bank URL should end with '/'": [
        ""
      ],
      "Couldn't found an bank in the URL specified.": [
        ""
      ],
      "Could not load exchanges": [
        "Не удалось загрузить комиссию за обмен"
      ],
      "Choose a currency to proceed or add another exchange": [
        ""
      ],
      "Known currencies": [
        ""
      ],
      "Specify the origin": [
        ""
      ],
      "Use previous origins:": [
        ""
      ],
      "From another bank account": [
        "Из моего банковского счёта"
      ],
      "From another wallet": [
        ""
      ],
      "From a %1$s URI or QR code": [
        ""
      ],
      "Enter URI here": [
        ""
      ],
      "Specify the destination": [
        "Описание комиссии за услугу"
      ],
      "Use previous destinations:": [
        ""
      ],
      "To another bank account": [
        "На мой банковский счёт"
      ],
      "To another wallet": [
        "На другой кошелек"
      ],
      "To a %1$s URI or QR code": [
        ""
      ],
      "Afghanistan": [
        ""
      ],
      "Albania": [
        ""
      ],
      "Algeria": [
        ""
      ],
      "United States": [
        ""
      ],
      "Andorra": [
        ""
      ],
      "Angola": [
        ""
      ],
      "Antigua and Barbuda": [
        ""
      ],
      "Argentina": [
        ""
      ],
      "Armenia": [
        ""
      ],
      "Australia": [
        ""
      ],
      "Austria": [
        ""
      ],
      "Azerbaijan": [
        ""
      ],
      "Bahamas": [
        ""
      ],
      "Bahrain": [
        ""
      ],
      "Bangladesh": [
        ""
      ],
      "Barbados": [
        ""
      ],
      "Botswana": [
        ""
      ],
      "Belarus": [
        ""
      ],
      "Belgium": [
        ""
      ],
      "Belize": [
        ""
      ],
      "Benin": [
        ""
      ],
      "Bhutan": [
        ""
      ],
      "Bolivia": [
        ""
      ],
      "Bosnia and Herzegovina": [
        ""
      ],
      "Brazil": [
        ""
      ],
      "United Kingdom": [
        ""
      ],
      "Brunei": [
        ""
      ],
      "Bulgaria": [
        ""
      ],
      "Burkina Faso": [
        ""
      ],
      "Myanmar": [
        ""
      ],
      "Burundi": [
        ""
      ],
      "Cambodia": [
        ""
      ],
      "Cameroon": [
        ""
      ],
      "Canada": [
        ""
      ],
      "Cape Verde": [
        ""
      ],
      "Central African Republic": [
        ""
      ],
      "Chad": [
        ""
      ],
      "Chile": [
        ""
      ],
      "China": [
        ""
      ],
      "Colombia": [
        ""
      ],
      "Comoros": [
        ""
      ],
      "Democratic Republic of the Congo": [
        ""
      ],
      "Republic of the Congo": [
        ""
      ],
      "Costa Rica": [
        ""
      ],
      "Croatia": [
        ""
      ],
      "Cuba": [
        ""
      ],
      "Cyprus": [
        ""
      ],
      "Czech Republic": [
        "Публичный ключ продавца"
      ],
      "Denmark": [
        ""
      ],
      "Djibouti": [
        ""
      ],
      "Dominican Republic": [
        ""
      ],
      "Netherlands": [
        ""
      ],
      "Timor-Leste": [
        ""
      ],
      "Ecuador": [
        ""
      ],
      "Egypt": [
        ""
      ],
      "United Arab Emirates": [
        ""
      ],
      "Equatorial Guinea": [
        ""
      ],
      "Eritrea": [
        ""
      ],
      "Estonia": [
        ""
      ],
      "Ethiopia": [
        ""
      ],
      "Fiji": [
        ""
      ],
      "Philippines": [
        ""
      ],
      "Finland": [
        ""
      ],
      "France": [
        "Отмена"
      ],
      "Gabon": [
        ""
      ],
      "Gambia": [
        ""
      ],
      "Georgia": [
        ""
      ],
      "Germany": [
        ""
      ],
      "Ghana": [
        ""
      ],
      "Greece": [
        ""
      ],
      "Grenada": [
        ""
      ],
      "Guatemala": [
        ""
      ],
      "Guinea-Bissau": [
        ""
      ],
      "Guinea": [
        ""
      ],
      "Guyana": [
        ""
      ],
      "Haiti": [
        ""
      ],
      "Honduras": [
        ""
      ],
      "Hungary": [
        ""
      ],
      "Kiribati": [
        ""
      ],
      "Iceland": [
        ""
      ],
      "India": [
        ""
      ],
      "Indonesia": [
        ""
      ],
      "Iran": [
        ""
      ],
      "Iraq": [
        ""
      ],
      "Ireland": [
        ""
      ],
      "Israel": [
        ""
      ],
      "Italy": [
        ""
      ],
      "Ivory Coast": [
        ""
      ],
      "Jamaica": [
        ""
      ],
      "Japan": [
        ""
      ],
      "Jordan": [
        ""
      ],
      "Kazakhstan": [
        ""
      ],
      "Kenya": [
        ""
      ],
      "Saint Kitts and Nevis": [
        ""
      ],
      "Kuwait": [
        ""
      ],
      "Kyrgyzstan": [
        ""
      ],
      "Laos": [
        ""
      ],
      "Latvia": [
        ""
      ],
      "Lebanon": [
        ""
      ],
      "Liberia": [
        ""
      ],
      "Libya": [
        ""
      ],
      "Liechtenstein": [
        ""
      ],
      "Lithuania": [
        ""
      ],
      "Luxembourg": [
        ""
      ],
      "North Macedonia": [
        ""
      ],
      "Madagascar": [
        ""
      ],
      "Malawi": [
        ""
      ],
      "Malaysia": [
        ""
      ],
      "Maldives": [
        ""
      ],
      "Mali": [
        ""
      ],
      "Malta": [
        ""
      ],
      "Marshall Islands": [
        ""
      ],
      "Mauritania": [
        ""
      ],
      "Mauritius": [
        ""
      ],
      "Mexico": [
        ""
      ],
      "Micronesia": [
        ""
      ],
      "Moldova": [
        ""
      ],
      "Monaco": [
        ""
      ],
      "Mongolia": [
        ""
      ],
      "Morocco": [
        ""
      ],
      "Lesotho": [
        ""
      ],
      "Mozambique": [
        ""
      ],
      "Namibia": [
        ""
      ],
      "Nauru": [
        ""
      ],
      "Nepal": [
        ""
      ],
      "New Zealand": [
        ""
      ],
      "Vanuatu": [
        ""
      ],
      "Nicaragua": [
        ""
      ],
      "Nigeria": [
        ""
      ],
      "Niger": [
        ""
      ],
      "North Korea": [
        ""
      ],
      "Norway": [
        ""
      ],
      "Oman": [
        ""
      ],
      "Pakistan": [
        ""
      ],
      "Palau": [
        ""
      ],
      "Panama": [
        ""
      ],
      "Papua New Guinea": [
        ""
      ],
      "Paraguay": [
        ""
      ],
      "Peru": [
        ""
      ],
      "Poland": [
        ""
      ],
      "Portugal": [
        ""
      ],
      "Qatar": [
        ""
      ],
      "Romania": [
        ""
      ],
      "Russia": [
        ""
      ],
      "Rwanda": [
        ""
      ],
      "Saint Lucia": [
        ""
      ],
      "El Salvador": [
        ""
      ],
      "Samoa": [
        ""
      ],
      "San Marino": [
        ""
      ],
      "São Tomé and Príncipe": [
        ""
      ],
      "Saudi Arabia": [
        ""
      ],
      "Senegal": [
        ""
      ],
      "Serbia": [
        ""
      ],
      "Seychelles": [
        ""
      ],
      "Sierra Leone": [
        ""
      ],
      "Singapore": [
        ""
      ],
      "Slovakia": [
        ""
      ],
      "Slovenia": [
        ""
      ],
      "Solomon Islands": [
        ""
      ],
      "Somalia": [
        ""
      ],
      "South Africa": [
        ""
      ],
      "South Korea": [
        ""
      ],
      "Spain": [
        ""
      ],
      "Sri Lanka": [
        ""
      ],
      "Sudan": [
        ""
      ],
      "Suriname": [
        ""
      ],
      "Eswatini": [
        ""
      ],
      "Sweden": [
        ""
      ],
      "Switzerland": [
        ""
      ],
      "Syria": [
        ""
      ],
      "Taiwan": [
        ""
      ],
      "Tajikistan": [
        ""
      ],
      "Tanzania": [
        ""
      ],
      "Thailand": [
        ""
      ],
      "Togo": [
        ""
      ],
      "Tonga": [
        ""
      ],
      "Trinidad and Tobago": [
        ""
      ],
      "Tunisia": [
        ""
      ],
      "Turkey": [
        ""
      ],
      "Tuvalu": [
        "значение"
      ],
      "Uganda": [
        ""
      ],
      "Ukraine": [
        ""
      ],
      "Uruguay": [
        ""
      ],
      "Uzbekistan": [
        ""
      ],
      "Venezuela": [
        ""
      ],
      "Vietnam": [
        ""
      ],
      "Yemen": [
        ""
      ],
      "Zambia": [
        ""
      ],
      "Zimbabwe": [
        ""
      ],
      "Afghan": [
        ""
      ],
      "Albanian": [
        ""
      ],
      "Algerian": [
        ""
      ],
      "American": [
        ""
      ],
      "Andorran": [
        ""
      ],
      "Angolan": [
        ""
      ],
      "Antiguan": [
        ""
      ],
      "Argentinean": [
        ""
      ],
      "Armenian": [
        ""
      ],
      "Australian": [
        ""
      ],
      "Austrian": [
        ""
      ],
      "Azerbaijani": [
        ""
      ],
      "Bahamian": [
        ""
      ],
      "Bahraini": [
        ""
      ],
      "Bangladeshi": [
        ""
      ],
      "Barbadian": [
        ""
      ],
      "Batswana": [
        ""
      ],
      "Belarusian": [
        ""
      ],
      "Belgian": [
        ""
      ],
      "Belizean": [
        ""
      ],
      "Beninese": [
        ""
      ],
      "Bhutanese": [
        ""
      ],
      "Bolivian": [
        ""
      ],
      "Bosnian": [
        ""
      ],
      "Brazilian": [
        ""
      ],
      "British": [
        ""
      ],
      "Bruneian": [
        ""
      ],
      "Bulgarian": [
        ""
      ],
      "Burkinabe": [
        ""
      ],
      "Burmese": [
        ""
      ],
      "Burundian": [
        ""
      ],
      "Cambodian": [
        ""
      ],
      "Cameroonian": [
        ""
      ],
      "Canadian": [
        ""
      ],
      "Cape Verdean": [
        ""
      ],
      "Central African": [
        ""
      ],
      "Chadian": [
        ""
      ],
      "Chilean": [
        ""
      ],
      "Chinese": [
        ""
      ],
      "Colombian": [
        ""
      ],
      "Comoran": [
        ""
      ],
      "Congolese": [
        ""
      ],
      "Costa Rican": [
        ""
      ],
      "Croatian": [
        ""
      ],
      "Cuban": [
        ""
      ],
      "Cypriot": [
        ""
      ],
      "Czech": [
        ""
      ],
      "Danish": [
        ""
      ],
      "Djiboutian": [
        ""
      ],
      "Dominican": [
        "Деноминация"
      ],
      "Dutch": [
        ""
      ],
      "East Timorese": [
        ""
      ],
      "Ecuadorean": [
        ""
      ],
      "Egyptian": [
        ""
      ],
      "Emirati": [
        ""
      ],
      "Equatorial Guinean": [
        ""
      ],
      "Eritrean": [
        ""
      ],
      "Estonian": [
        ""
      ],
      "Ethiopian": [
        ""
      ],
      "Fijian": [
        ""
      ],
      "Filipino": [
        ""
      ],
      "Finnish": [
        ""
      ],
      "French": [
        ""
      ],
      "Gabonese": [
        ""
      ],
      "Gambian": [
        ""
      ],
      "Georgian": [
        ""
      ],
      "German": [
        ""
      ],
      "Ghanaian": [
        ""
      ],
      "Greek": [
        ""
      ],
      "Grenadian": [
        ""
      ],
      "Guatemalan": [
        ""
      ],
      "Guinea-Bissauan": [
        ""
      ],
      "Guinean": [
        ""
      ],
      "Guyanese": [
        ""
      ],
      "Haitian": [
        ""
      ],
      "Honduran": [
        ""
      ],
      "Hungarian": [
        ""
      ],
      "I-Kiribati": [
        ""
      ],
      "Icelander": [
        ""
      ],
      "Indian": [
        ""
      ],
      "Indonesian": [
        ""
      ],
      "Iranian": [
        ""
      ],
      "Iraqi": [
        ""
      ],
      "Irish": [
        ""
      ],
      "Israeli": [
        ""
      ],
      "Italian": [
        ""
      ],
      "Ivorian": [
        ""
      ],
      "Jamaican": [
        ""
      ],
      "Japanese": [
        ""
      ],
      "Jordanian": [
        ""
      ],
      "Kazakhstani": [
        ""
      ],
      "Kenyan": [
        ""
      ],
      "Kittian and Nevisian": [
        ""
      ],
      "Kuwaiti": [
        ""
      ],
      "Kyrgyz": [
        ""
      ],
      "Laotian": [
        ""
      ],
      "Latvian": [
        ""
      ],
      "Lebanese": [
        ""
      ],
      "Liberian": [
        ""
      ],
      "Libyan": [
        ""
      ],
      "Liechtensteiner": [
        ""
      ],
      "Lithuanian": [
        ""
      ],
      "Luxembourger": [
        ""
      ],
      "Macedonian": [
        ""
      ],
      "Malagasy": [
        ""
      ],
      "Malawian": [
        ""
      ],
      "Malaysian": [
        ""
      ],
      "Maldivian": [
        ""
      ],
      "Malian": [
        ""
      ],
      "Maltese": [
        ""
      ],
      "Marshallese": [
        ""
      ],
      "Mauritanian": [
        ""
      ],
      "Mauritian": [
        ""
      ],
      "Mexican": [
        ""
      ],
      "Micronesian": [
        ""
      ],
      "Moldovan": [
        ""
      ],
      "Monacan": [
        ""
      ],
      "Mongolian": [
        ""
      ],
      "Moroccan": [
        ""
      ],
      "Mosotho": [
        ""
      ],
      "Mozambican": [
        ""
      ],
      "Namibian": [
        ""
      ],
      "Nauruan": [
        ""
      ],
      "Nepalese": [
        ""
      ],
      "New Zealander": [
        ""
      ],
      "Ni-Vanuatu": [
        ""
      ],
      "Nicaraguan": [
        ""
      ],
      "Nigerian": [
        ""
      ],
      "Nigerien": [
        ""
      ],
      "North Korean": [
        ""
      ],
      "Norwegian": [
        ""
      ],
      "Omani": [
        ""
      ],
      "Pakistani": [
        ""
      ],
      "Palauan": [
        ""
      ],
      "Panamanian": [
        ""
      ],
      "Papua New Guinean": [
        ""
      ],
      "Paraguayan": [
        ""
      ],
      "Peruvian": [
        ""
      ],
      "Polish": [
        ""
      ],
      "Portuguese": [
        ""
      ],
      "Qatari": [
        ""
      ],
      "Romanian": [
        ""
      ],
      "Russian": [
        ""
      ],
      "Rwandan": [
        ""
      ],
      "Saint Lucian": [
        ""
      ],
      "Salvadoran": [
        ""
      ],
      "Samoan": [
        ""
      ],
      "San Marinese": [
        ""
      ],
      "Sao Tomean": [
        ""
      ],
      "Saudi": [
        ""
      ],
      "Senegalese": [
        ""
      ],
      "Serbian": [
        ""
      ],
      "Seychellois": [
        ""
      ],
      "Sierra Leonean": [
        ""
      ],
      "Singaporean": [
        ""
      ],
      "Slovakian": [
        ""
      ],
      "Slovenian": [
        ""
      ],
      "Solomon Islander": [
        ""
      ],
      "Somali": [
        ""
      ],
      "South African": [
        ""
      ],
      "South Korean": [
        ""
      ],
      "Spanish": [
        ""
      ],
      "Sri Lankan": [
        ""
      ],
      "Sudanese": [
        ""
      ],
      "Surinamer": [
        ""
      ],
      "Swazi": [
        ""
      ],
      "Swedish": [
        ""
      ],
      "Swiss": [
        ""
      ],
      "Syrian": [
        ""
      ],
      "Taiwanese": [
        ""
      ],
      "Tajik": [
        ""
      ],
      "Tanzanian": [
        ""
      ],
      "Thai": [
        ""
      ],
      "Togolese": [
        ""
      ],
      "Tongan": [
        ""
      ],
      "Trinidadian or Tobagonian": [
        ""
      ],
      "Tunisian": [
        ""
      ],
      "Turkish": [
        ""
      ],
      "Tuvaluan": [
        "значение"
      ],
      "Ugandan": [
        ""
      ],
      "Ukrainian": [
        ""
      ],
      "Uruguayan": [
        ""
      ],
      "Uzbekistani": [
        ""
      ],
      "Venezuelan": [
        ""
      ],
      "Vietnamese": [
        ""
      ],
      "Yemenite": [
        ""
      ],
      "Zambian": [
        ""
      ],
      "Zimbabwean": [
        ""
      ],
      "🇦🇫": [
        ""
      ],
      "🇦🇱": [
        ""
      ],
      "🇩🇿": [
        ""
      ],
      "🇺🇸": [
        ""
      ],
      "🇦🇩": [
        ""
      ],
      "🇦🇴": [
        ""
      ],
      "🇦🇬": [
        ""
      ],
      "🇦🇷": [
        ""
      ],
      "🇦🇲": [
        ""
      ],
      "🇦🇺": [
        ""
      ],
      "🇦🇹": [
        ""
      ],
      "🇦🇿": [
        ""
      ],
      "🇧🇸": [
        ""
      ],
      "🇧🇭": [
        ""
      ],
      "🇧🇩": [
        ""
      ],
      "🇧🇧": [
        ""
      ],
      "🇧🇼": [
        ""
      ],
      "🇧🇾": [
        ""
      ],
      "🇧🇪": [
        ""
      ],
      "🇧🇿": [
        ""
      ],
      "🇧🇯": [
        ""
      ],
      "🇧🇹": [
        ""
      ],
      "🇧🇴": [
        ""
      ],
      "🇧🇦": [
        ""
      ],
      "🇧🇷": [
        ""
      ],
      "🇬🇧": [
        ""
      ],
      "🇧🇳": [
        ""
      ],
      "🇧🇬": [
        ""
      ],
      "🇧🇫": [
        ""
      ],
      "🇲🇲": [
        ""
      ],
      "🇧🇮": [
        ""
      ],
      "🇰🇭": [
        ""
      ],
      "🇨🇲": [
        ""
      ],
      "🇨🇦": [
        ""
      ],
      "🇨🇻": [
        ""
      ],
      "🇨🇫": [
        ""
      ],
      "🇨🇱": [
        ""
      ],
      "🇨🇳": [
        ""
      ],
      "🇨🇴": [
        ""
      ],
      "🇨🇩": [
        ""
      ],
      "🇨🇬": [
        ""
      ],
      "🇨🇷": [
        ""
      ],
      "🇨🇺": [
        ""
      ],
      "🇨🇾": [
        ""
      ],
      "🇨🇿": [
        ""
      ],
      "🇩🇰": [
        ""
      ],
      "🇩🇯": [
        ""
      ],
      "🇩🇴": [
        ""
      ],
      "🇳🇱": [
        ""
      ],
      "🇹🇱": [
        ""
      ],
      "🇪🇨": [
        ""
      ],
      "🇪🇬": [
        ""
      ],
      "🇦🇪": [
        ""
      ],
      "🇬🇶": [
        ""
      ],
      "🇪🇷": [
        ""
      ],
      "🇪🇪": [
        ""
      ],
      "🇪🇹": [
        ""
      ],
      "🇫🇯": [
        ""
      ],
      "🇵🇭": [
        ""
      ],
      "🇫🇮": [
        ""
      ],
      "🇫🇷": [
        ""
      ],
      "🇬🇦": [
        ""
      ],
      "🇬🇲": [
        ""
      ],
      "🇬🇪": [
        ""
      ],
      "UAE Dirham": [
        ""
      ],
      "Afghani": [
        ""
      ],
      "Lek": [
        ""
      ],
      "Armenian Dram": [
        ""
      ],
      "Netherlands Antillean Guilder": [
        ""
      ],
      "Kwanza": [
        ""
      ],
      "Argentine Peso": [
        ""
      ],
      "Australian Dollar": [
        ""
      ],
      "Aruban Florin": [
        ""
      ],
      "Azerbaijan Manat": [
        ""
      ],
      "Convertible Mark": [
        ""
      ],
      "Barbados Dollar": [
        ""
      ],
      "Taka": [
        ""
      ],
      "Bulgarian Lev": [
        ""
      ],
      "Bahraini Dinar": [
        ""
      ],
      "Burundi Franc": [
        ""
      ],
      "Bermudian Dollar": [
        ""
      ],
      "Brunei Dollar": [
        ""
      ],
      "Boliviano": [
        ""
      ],
      "Mvdol": [
        ""
      ],
      "Brazilian Real": [
        ""
      ],
      "Bahamian Dollar": [
        ""
      ],
      "Ngultrum": [
        ""
      ],
      "Pula": [
        ""
      ],
      "Belarusian Ruble": [
        ""
      ],
      "Belize Dollar": [
        ""
      ],
      "Canadian Dollar": [
        ""
      ],
      "Congolese Franc": [
        ""
      ],
      "WIR Euro": [
        ""
      ],
      "Swiss Franc": [
        ""
      ],
      "WIR Franc": [
        ""
      ],
      "Unidad de Fomento": [
        ""
      ],
      "Chilean Peso": [
        ""
      ],
      "Yuan Renminbi": [
        ""
      ],
      "Colombian Peso": [
        ""
      ],
      "Unidad de Valor Real": [
        ""
      ],
      "Costa Rican Colon": [
        ""
      ],
      "Cuban Peso": [
        ""
      ],
      "Cabo Verde Escudo": [
        ""
      ],
      "Czech Koruna": [
        ""
      ],
      "Djibouti Franc": [
        ""
      ],
      "Danish Krone": [
        ""
      ],
      "Dominican Peso": [
        ""
      ],
      "Algerian Dinar": [
        ""
      ],
      "Egyptian Pound": [
        ""
      ],
      "Nakfa": [
        ""
      ],
      "Ethiopian Birr": [
        ""
      ],
      "Euro": [
        ""
      ],
      "Fiji Dollar": [
        ""
      ],
      "Falkland Islands Pound": [
        ""
      ],
      "Pound Sterling": [
        ""
      ],
      "Lari": [
        ""
      ],
      "Ghana Cedi": [
        ""
      ],
      "Gibraltar Pound": [
        ""
      ],
      "Dalasi": [
        ""
      ],
      "Guinean Franc": [
        ""
      ],
      "Quetzal": [
        ""
      ],
      "Guyana Dollar": [
        ""
      ],
      "Hong Kong Dollar": [
        ""
      ],
      "Lempira": [
        ""
      ],
      "Gourde": [
        ""
      ],
      "Forint": [
        ""
      ],
      "Rupiah": [
        ""
      ],
      "New Israeli Sheqel": [
        ""
      ],
      "Indian Rupee": [
        ""
      ],
      "Iraqi Dinar": [
        ""
      ],
      "Iranian Rial": [
        ""
      ],
      "Iceland Krona": [
        ""
      ],
      "Jamaican Dollar": [
        ""
      ],
      "Jordanian Dinar": [
        ""
      ],
      "Yen": [
        ""
      ],
      "Kenyan Shilling": [
        ""
      ],
      "Som": [
        ""
      ],
      "Riel": [
        ""
      ],
      "Comorian Franc ": [
        ""
      ],
      "North Korean Won": [
        ""
      ],
      "Won": [
        ""
      ],
      "Kuwaiti Dinar": [
        ""
      ],
      "Cayman Islands Dollar": [
        ""
      ],
      "Tenge": [
        ""
      ],
      "Lao Kip": [
        ""
      ],
      "Lebanese Pound": [
        ""
      ],
      "Sri Lanka Rupee": [
        ""
      ],
      "Liberian Dollar": [
        ""
      ],
      "Loti": [
        ""
      ],
      "Libyan Dinar": [
        ""
      ],
      "Moroccan Dirham": [
        ""
      ],
      "Moldovan Leu": [
        ""
      ],
      "Malagasy Ariary": [
        ""
      ],
      "Denar": [
        ""
      ],
      "Kyat": [
        ""
      ],
      "Tugrik": [
        ""
      ],
      "Pataca": [
        ""
      ],
      "Ouguiya": [
        ""
      ],
      "Mauritius Rupee": [
        ""
      ],
      "Rufiyaa": [
        ""
      ],
      "Malawi Kwacha": [
        ""
      ],
      "Mexican Peso": [
        ""
      ],
      "Mexican Unidad de Inversion (UDI)": [
        ""
      ],
      "Malaysian Ringgit": [
        ""
      ],
      "Mozambique Metical": [
        ""
      ],
      "Namibia Dollar": [
        ""
      ],
      "Naira": [
        ""
      ],
      "Cordoba Oro": [
        ""
      ],
      "Norwegian Krone": [
        ""
      ],
      "Nepalese Rupee": [
        ""
      ],
      "New Zealand Dollar": [
        ""
      ],
      "Rial Omani": [
        ""
      ],
      "Balboa": [
        ""
      ],
      "Sol": [
        "старый"
      ],
      "Kina": [
        ""
      ],
      "Philippine Peso": [
        ""
      ],
      "Pakistan Rupee": [
        ""
      ],
      "Zloty": [
        ""
      ],
      "Guarani": [
        ""
      ],
      "Qatari Rial": [
        ""
      ],
      "Romanian Leu": [
        ""
      ],
      "Serbian Dinar": [
        ""
      ],
      "Russian Ruble": [
        ""
      ],
      "Rwanda Franc": [
        ""
      ],
      "Saudi Riyal": [
        ""
      ],
      "Solomon Islands Dollar": [
        ""
      ],
      "Seychelles Rupee": [
        ""
      ],
      "Sudanese Pound": [
        ""
      ],
      "Swedish Krona": [
        ""
      ],
      "Singapore Dollar": [
        ""
      ],
      "Saint Helena Pound": [
        ""
      ],
      "Leone": [
        ""
      ],
      "Somali Shilling": [
        ""
      ],
      "Surinam Dollar": [
        ""
      ],
      "South Sudanese Pound": [
        ""
      ],
      "Dobra": [
        ""
      ],
      "El Salvador Colon": [
        ""
      ],
      "Syrian Pound": [
        ""
      ],
      "Lilangeni": [
        ""
      ],
      "Baht": [
        ""
      ],
      "Somoni": [
        ""
      ],
      "Turkmenistan New Manat": [
        ""
      ],
      "Tunisian Dinar": [
        ""
      ],
      "Pa’anga": [
        ""
      ],
      "Turkish Lira": [
        ""
      ],
      "Trinidad and Tobago Dollar": [
        ""
      ],
      "New Taiwan Dollar": [
        ""
      ],
      "Tanzanian Shilling": [
        ""
      ],
      "Hryvnia": [
        ""
      ],
      "Uganda Shilling": [
        ""
      ],
      "US Dollar": [
        ""
      ],
      "US Dollar (Next day)": [
        ""
      ],
      "Uruguay Peso en Unidades Indexadas (UI)": [
        ""
      ],
      "Peso Uruguayo": [
        ""
      ],
      "Unidad Previsional": [
        ""
      ],
      "Uzbekistan Sum": [
        ""
      ],
      "Bolívar Soberano": [
        ""
      ],
      "Dong": [
        ""
      ],
      "Vatu": [
        ""
      ],
      "Tala": [
        ""
      ],
      "CFA Franc BEAC": [
        ""
      ],
      "East Caribbean Dollar": [
        ""
      ],
      "CFA Franc BCEAO": [
        ""
      ],
      "CFP Franc": [
        ""
      ],
      "Sucre": [
        ""
      ],
      "Yemeni Rial": [
        ""
      ],
      "Rand": [
        ""
      ],
      "Zambian Kwacha": [
        ""
      ],
      "Zimbabwe Gold": [
        ""
      ],
      "Afar": [
        ""
      ],
      "Abkhazian": [
        ""
      ],
      "Achinese": [
        ""
      ],
      "Acoli": [
        ""
      ],
      "Adangme": [
        ""
      ],
      "Adyghe": [
        ""
      ],
      "Avestan": [
        ""
      ],
      "Tunisian Arabic": [
        ""
      ],
      "Afrikaans": [
        ""
      ],
      "Afrihili": [
        ""
      ],
      "Aghem": [
        ""
      ],
      "Ainu": [
        ""
      ],
      "Akan": [
        ""
      ],
      "Akkadian": [
        ""
      ],
      "Alabama": [
        ""
      ],
      "Aleut": [
        ""
      ],
      "Gheg Albanian": [
        ""
      ],
      "Southern Altai": [
        ""
      ],
      "Amarik": [
        ""
      ],
      "Aragonese": [
        ""
      ],
      "Old English": [
        ""
      ],
      "Angika": [
        ""
      ],
      "Modern Standard Arabic": [
        ""
      ],
      "Arabik": [
        ""
      ],
      "Aramaic": [
        ""
      ],
      "Mapuche": [
        ""
      ],
      "Araona": [
        ""
      ],
      "Arapaho": [
        ""
      ],
      "Algerian Arabic": [
        ""
      ],
      "Arawak": [
        ""
      ],
      "Moroccan Arabic": [
        ""
      ],
      "Egyptian Arabic": [
        ""
      ],
      "Asu": [
        ""
      ],
      "Assamese": [
        ""
      ],
      "American Sign Language": [
        ""
      ],
      "Asturian": [
        ""
      ],
      "Avaric": [
        ""
      ],
      "Kotava": [
        ""
      ],
      "Awadhi": [
        ""
      ],
      "Aymara": [
        ""
      ],
      "South Azerbaijani": [
        ""
      ],
      "Bashkir": [
        ""
      ],
      "Baluchi": [
        ""
      ],
      "Balinese": [
        ""
      ],
      "Bavarian": [
        ""
      ],
      "Basaa": [
        ""
      ],
      "Bamun": [
        ""
      ],
      "Batak Toba": [
        ""
      ],
      "Ghomala": [
        ""
      ],
      "Belarus kasa": [
        ""
      ],
      "Beja": [
        ""
      ],
      "Bemba": [
        ""
      ],
      "Betawi": [
        ""
      ],
      "Bena": [
        ""
      ],
      "Bafut": [
        ""
      ],
      "Badaga": [
        ""
      ],
      "Bɔlgeria kasa": [
        ""
      ],
      "Bhojpuri": [
        ""
      ],
      "Bislama": [
        ""
      ],
      "Bikol": [
        ""
      ],
      "Bini": [
        ""
      ],
      "Banjar": [
        ""
      ],
      "Kom": [
        ""
      ],
      "Siksika": [
        ""
      ],
      "Bambara": [
        ""
      ],
      "Bengali kasa": [
        ""
      ],
      "Tibetan": [
        ""
      ],
      "Bishnupriya": [
        ""
      ],
      "Bakhtiari": [
        ""
      ],
      "Braj": [
        ""
      ],
      "Breton": [
        ""
      ],
      "Brahui": [
        ""
      ],
      "Bodo": [
        ""
      ],
      "Akoose": [
        ""
      ],
      "Buriat": [
        ""
      ],
      "Buginese": [
        ""
      ],
      "Bulu": [
        ""
      ],
      "Blin": [
        ""
      ],
      "Medumba": [
        ""
      ],
      "Catalan": [
        ""
      ],
      "Caddo": [
        ""
      ],
      "Carib": [
        ""
      ],
      "Cayuga": [
        ""
      ],
      "Atsam": [
        ""
      ],
      "Cebuano": [
        ""
      ],
      "Chechen": [
        ""
      ],
      "Chiga": [
        ""
      ],
      "Chibcha": [
        ""
      ],
      "Chamorro": [
        ""
      ],
      "Chagatai": [
        ""
      ],
      "Chuukese": [
        ""
      ],
      "Mari": [
        ""
      ],
      "Chinook Jargon": [
        ""
      ],
      "Choctaw": [
        ""
      ],
      "Chipewyan": [
        ""
      ],
      "Cherokee": [
        ""
      ],
      "Cheyenne": [
        ""
      ],
      "Central Kurdish": [
        ""
      ],
      "Corsican": [
        ""
      ],
      "Coptic": [
        ""
      ],
      "Capiznon": [
        ""
      ],
      "Cree": [
        ""
      ],
      "Crimean Turkish": [
        ""
      ],
      "Kashubian": [
        ""
      ],
      "Kyɛk kasa": [
        ""
      ],
      "Church Slavic": [
        ""
      ],
      "Chuvash": [
        ""
      ],
      "Welsh": [
        ""
      ],
      "Dakota": [
        ""
      ],
      "Dargwa": [
        ""
      ],
      "Taita": [
        ""
      ],
      "Austrian German": [
        ""
      ],
      "Swiss High German": [
        ""
      ],
      "Gyaaman": [
        ""
      ],
      "Delaware": [
        ""
      ],
      "Slave": [
        ""
      ],
      "Dogrib": [
        ""
      ],
      "Dinka": [
        ""
      ],
      "Zarma": [
        ""
      ],
      "Dogri": [
        ""
      ],
      "Lower Sorbian": [
        ""
      ],
      "Central Dusun": [
        ""
      ],
      "Duala": [
        ""
      ],
      "Middle Dutch": [
        ""
      ],
      "Divehi": [
        ""
      ],
      "Jola-Fonyi": [
        ""
      ],
      "Dyula": [
        ""
      ],
      "Dzongkha": [
        ""
      ],
      "Dazaga": [
        ""
      ],
      "Embu": [
        ""
      ],
      "Ewe": [
        ""
      ],
      "Efik": [
        ""
      ],
      "Emilian": [
        ""
      ],
      "Ancient Egyptian": [
        ""
      ],
      "Ekajuk": [
        ""
      ],
      "Greek kasa": [
        ""
      ],
      "Elamite": [
        ""
      ],
      "Australian English": [
        ""
      ],
      "Borɔfo": [
        ""
      ],
      "Canadian English": [
        ""
      ],
      "British English": [
        ""
      ],
      "Middle English": [
        ""
      ],
      "American English": [
        ""
      ],
      "Esperanto": [
        "Операция"
      ],
      "Latin American Spanish": [
        ""
      ],
      "European Spanish": [
        ""
      ],
      "Mexican Spanish": [
        ""
      ],
      "Spain kasa": [
        ""
      ],
      "Central Yupik": [
        ""
      ],
      "Basque": [
        ""
      ],
      "Ewondo": [
        ""
      ],
      "Extremaduran": [
        ""
      ],
      "Fang": [
        ""
      ],
      "Pɛɛhyia kasa": [
        ""
      ],
      "Fanti": [
        ""
      ],
      "Fulah": [
        ""
      ],
      "Tornedalen Finnish": [
        ""
      ],
      "Faroese": [
        ""
      ],
      "Fon": [
        ""
      ],
      "Canadian French": [
        ""
      ],
      "Cajun French": [
        "Валюта"
      ],
      "Swiss French": [
        ""
      ],
      "Frɛnkye": [
        ""
      ],
      "Middle French": [
        ""
      ],
      "Old French": [
        ""
      ],
      "Arpitan": [
        ""
      ],
      "Northern Frisian": [
        ""
      ],
      "Eastern Frisian": [
        ""
      ],
      "Friulian": [
        ""
      ],
      "Western Frisian": [
        ""
      ],
      "Ga": [
        ""
      ],
      "Gagauz": [
        ""
      ],
      "Gan Chinese": [
        ""
      ],
      "Gayo": [
        ""
      ],
      "Gbaya": [
        ""
      ],
      "Zoroastrian Dari": [
        ""
      ],
      "Scottish Gaelic": [
        ""
      ],
      "Geez": [
        ""
      ],
      "Gilbertese": [
        ""
      ],
      "Galician": [
        ""
      ],
      "Gilaki": [
        ""
      ],
      "Middle High German": [
        ""
      ],
      "Old High German": [
        ""
      ],
      "Goan Konkani": [
        ""
      ],
      "Gondi": [
        ""
      ],
      "Gorontalo": [
        ""
      ],
      "Gothic": [
        ""
      ],
      "Grebo": [
        ""
      ],
      "Ancient Greek": [
        ""
      ],
      "Swiss German": [
        ""
      ],
      "Wayuu": [
        ""
      ],
      "Gujarati": [
        ""
      ],
      "Frafra": [
        ""
      ],
      "Gusii": [
        ""
      ],
      "Manx": [
        ""
      ],
      "Gwichʼin": [
        ""
      ],
      "Hausa": [
        ""
      ],
      "Haida": [
        ""
      ],
      "Hakka Chinese": [
        ""
      ],
      "Hawaiian": [
        ""
      ],
      "Hebrew": [
        ""
      ],
      "Fiji Hindi": [
        ""
      ],
      "Hindi": [
        ""
      ],
      "Hiligaynon": [
        ""
      ],
      "Hittite": [
        ""
      ],
      "Hmong": [
        ""
      ],
      "Hiri Motu": [
        ""
      ],
      "Upper Sorbian": [
        ""
      ],
      "Xiang Chinese": [
        ""
      ],
      "Hangri kasa": [
        ""
      ],
      "Hupa": [
        ""
      ],
      "Herero": [
        ""
      ],
      "Interlingua": [
        ""
      ],
      "Iban": [
        ""
      ],
      "Ibibio": [
        ""
      ],
      "Indonihyia kasa": [
        ""
      ],
      "Interlingue": [
        ""
      ],
      "Igbo": [
        ""
      ],
      "Sichuan Yi": [
        ""
      ],
      "Inupiaq": [
        ""
      ],
      "Iloko": [
        ""
      ],
      "Ingush": [
        ""
      ],
      "Ido": [
        ""
      ],
      "Icelandic": [
        ""
      ],
      "Italy kasa": [
        ""
      ],
      "Inuktitut": [
        ""
      ],
      "Ingrian": [
        ""
      ],
      "Gyapan kasa": [
        ""
      ],
      "Jamaican Creole English": [
        ""
      ],
      "Lojban": [
        ""
      ],
      "Ngomba": [
        ""
      ],
      "Machame": [
        ""
      ],
      "Judeo-Persian": [
        ""
      ],
      "Judeo-Arabic": [
        ""
      ],
      "Jutish": [
        ""
      ],
      "Gyabanis kasa": [
        ""
      ],
      "Kara-Kalpak": [
        ""
      ],
      "Kabyle": [
        ""
      ],
      "Kachin": [
        ""
      ],
      "Jju": [
        ""
      ],
      "Kamba": [
        ""
      ],
      "Kawi": [
        ""
      ],
      "Kabardian": [
        ""
      ],
      "Kanembu": [
        ""
      ],
      "Tyap": [
        ""
      ],
      "Makonde": [
        ""
      ],
      "Kabuverdianu": [
        ""
      ],
      "Kenyang": [
        ""
      ],
      "Koro": [
        ""
      ],
      "Kongo": [
        ""
      ],
      "Kaingang": [
        ""
      ],
      "Khasi": [
        ""
      ],
      "Khotanese": [
        ""
      ],
      "Koyra Chiini": [
        ""
      ],
      "Khowar": [
        ""
      ],
      "Kikuyu": [
        ""
      ],
      "Kirmanjki": [
        ""
      ],
      "Kuanyama": [
        ""
      ],
      "Kako": [
        ""
      ],
      "Kazakh": [
        ""
      ],
      "Kalaallisut": [
        ""
      ],
      "Kalenjin": [
        ""
      ],
      "Kimbundu": [
        ""
      ],
      "Kambodia kasa": [
        ""
      ],
      "Kannada": [
        ""
      ],
      "Komi-Permyak": [
        ""
      ],
      "Konkani": [
        ""
      ],
      "Korea kasa": [
        ""
      ],
      "Kosraean": [
        ""
      ],
      "Kpelle": [
        ""
      ],
      "Karachay-Balkar": [
        ""
      ],
      "Krio": [
        ""
      ],
      "Kinaray-a": [
        ""
      ],
      "Kanuri": [
        ""
      ],
      "Karelian": [
        ""
      ],
      "Kurukh": [
        ""
      ],
      "Shambala": [
        ""
      ],
      "Bafia": [
        ""
      ],
      "Colognian": [
        ""
      ],
      "Kashmiri": [
        ""
      ],
      "Kurdish": [
        ""
      ],
      "Kumyk": [
        ""
      ],
      "Kutenai": [
        ""
      ],
      "Komi": [
        ""
      ],
      "Cornish": [
        ""
      ],
      "Ladino": [
        "Загружаются"
      ],
      "Langi": [
        ""
      ],
      "Lahnda": [
        ""
      ],
      "Latin": [
        ""
      ],
      "Lamba": [
        ""
      ],
      "Luxembourgish": [
        ""
      ],
      "Lezghian": [
        ""
      ],
      "Lingua Franca Nova": [
        ""
      ],
      "Ganda": [
        ""
      ],
      "Ligurian": [
        ""
      ],
      "Limburgish": [
        ""
      ],
      "Livonian": [
        ""
      ],
      "Lakota": [
        ""
      ],
      "Lombard": [
        ""
      ],
      "Lingala": [
        ""
      ],
      "Lao": [
        ""
      ],
      "Mongo": [
        ""
      ],
      "Lozi": [
        ""
      ],
      "Latgalian": [
        ""
      ],
      "Luba-Lulua": [
        ""
      ],
      "Luiseno": [
        ""
      ],
      "Luba-Katanga": [
        ""
      ],
      "Lunda": [
        ""
      ],
      "Luo": [
        ""
      ],
      "Mizo": [
        ""
      ],
      "Luyia": [
        ""
      ],
      "Literary Chinese": [
        ""
      ],
      "Laz": [
        ""
      ],
      "Madurese": [
        "сбросить"
      ],
      "Mafa": [
        ""
      ],
      "Magahi": [
        ""
      ],
      "Maithili": [
        ""
      ],
      "Makasar": [
        ""
      ],
      "Mandingo": [
        "Загружаются"
      ],
      "Masai": [
        ""
      ],
      "Maba": [
        ""
      ],
      "Moksha": [
        ""
      ],
      "Mandar": [
        ""
      ],
      "Mende": [
        ""
      ],
      "Meru": [
        ""
      ],
      "Morisyen": [
        ""
      ],
      "Middle Irish": [
        ""
      ],
      "Makhuwa-Meetto": [
        ""
      ],
      "Metaʼ": [
        ""
      ],
      "Micmac": [
        ""
      ],
      "Maori": [
        ""
      ],
      "Minangkabau": [
        ""
      ],
      "Malayalam": [
        ""
      ],
      "Manchu": [
        ""
      ],
      "Manipuri": [
        ""
      ],
      "Mohawk": [
        ""
      ],
      "Mossi": [
        ""
      ],
      "Western Mari": [
        ""
      ],
      "Marathi": [
        ""
      ],
      "Malay kasa": [
        ""
      ],
      "Mundang": [
        ""
      ],
      "Multiple Languages": [
        ""
      ],
      "Creek": [
        ""
      ],
      "Mirandese": [
        ""
      ],
      "Marwari": [
        ""
      ],
      "Mentawai": [
        ""
      ],
      "Bɛɛmis kasa": [
        ""
      ],
      "Myene": [
        ""
      ],
      "Erzya": [
        ""
      ],
      "Mazanderani": [
        ""
      ],
      "Min Nan Chinese": [
        ""
      ],
      "Neapolitan": [
        ""
      ],
      "Nama": [
        ""
      ],
      "Norwegian Bokmål": [
        ""
      ],
      "North Ndebele": [
        ""
      ],
      "Low German": [
        ""
      ],
      "Nɛpal kasa": [
        ""
      ],
      "Newari": [
        ""
      ],
      "Ndonga": [
        ""
      ],
      "Nias": [
        ""
      ],
      "Niuean": [
        ""
      ],
      "Ao Naga": [
        ""
      ],
      "Flemish": [
        ""
      ],
      "Dɛɛkye": [
        ""
      ],
      "Kwasio": [
        ""
      ],
      "Ngiemboon": [
        ""
      ],
      "Norwegian Nynorsk": [
        ""
      ],
      "Nogai": [
        ""
      ],
      "Old Norse": [
        ""
      ],
      "Novial": [
        ""
      ],
      "NʼKo": [
        ""
      ],
      "South Ndebele": [
        ""
      ],
      "Northern Sotho": [
        ""
      ],
      "Nuer": [
        ""
      ],
      "Navajo": [
        ""
      ],
      "Classical Newari": [
        ""
      ],
      "Nyamwezi": [
        ""
      ],
      "Nyankole": [
        ""
      ],
      "Nyanja": [
        ""
      ],
      "Nyoro": [
        ""
      ],
      "Nzima": [
        ""
      ],
      "Occitan": [
        ""
      ],
      "Ojibwa": [
        ""
      ],
      "Oromo": [
        ""
      ],
      "Oriya": [
        ""
      ],
      "Osage": [
        ""
      ],
      "Ossetic": [
        ""
      ],
      "Ottoman Turkish": [
        ""
      ],
      "Pangasinan": [
        ""
      ],
      "Pahlavi": [
        ""
      ],
      "Pampanga": [
        ""
      ],
      "Papiamento": [
        "Платёж"
      ],
      "Pungyabi kasa": [
        ""
      ],
      "Picard": [
        ""
      ],
      "Pennsylvania German": [
        ""
      ],
      "Plautdietsch": [
        ""
      ],
      "Old Persian": [
        ""
      ],
      "Palatine German": [
        ""
      ],
      "Phoenician": [
        ""
      ],
      "Pali": [
        ""
      ],
      "Pɔland kasa": [
        ""
      ],
      "Piedmontese": [
        ""
      ],
      "Pontic": [
        ""
      ],
      "Pohnpeian": [
        ""
      ],
      "Prussian": [
        ""
      ],
      "Old Provençal": [
        ""
      ],
      "Pashto": [
        ""
      ],
      "Brazilian Portuguese": [
        ""
      ],
      "Pɔɔtugal kasa": [
        ""
      ],
      "European Portuguese": [
        ""
      ],
      "Kʼicheʼ": [
        ""
      ],
      "Chimborazo Highland Quichua": [
        ""
      ],
      "Quechua": [
        ""
      ],
      "Rajasthani": [
        ""
      ],
      "Rapanui": [
        ""
      ],
      "Rarotongan": [
        ""
      ],
      "Romagnol": [
        ""
      ],
      "Riffian": [
        ""
      ],
      "Romansh": [
        ""
      ],
      "Rundi": [
        ""
      ],
      "Rombo": [
        ""
      ],
      "Moldavian": [
        ""
      ],
      "Romany": [
        ""
      ],
      "Root": [
        ""
      ],
      "Romenia kasa": [
        ""
      ],
      "Rotuman": [
        ""
      ],
      "Rusyn": [
        ""
      ],
      "Roviana": [
        ""
      ],
      "Aromanian": [
        ""
      ],
      "Rahyia kasa": [
        ""
      ],
      "Rwa": [
        ""
      ],
      "Rewanda kasa": [
        ""
      ],
      "Sandawe": [
        ""
      ],
      "Sakha": [
        ""
      ],
      "Samaritan Aramaic": [
        ""
      ],
      "Samburu": [
        ""
      ],
      "Sanskrit": [
        ""
      ],
      "Sasak": [
        ""
      ],
      "Santali": [
        ""
      ],
      "Saurashtra": [
        ""
      ],
      "Ngambay": [
        ""
      ],
      "Sangu": [
        ""
      ],
      "Sicilian": [
        ""
      ],
      "Scots": [
        ""
      ],
      "Sardinian": [
        ""
      ],
      "Sassarese Sardinian": [
        ""
      ],
      "Sindhi": [
        ""
      ],
      "Seneca": [
        ""
      ],
      "Sena": [
        ""
      ],
      "Seri": [
        ""
      ],
      "Selkup": [
        ""
      ],
      "Northern Sami": [
        ""
      ],
      "Koyraboro Senni": [
        ""
      ],
      "Old Irish": [
        ""
      ],
      "Sango": [
        ""
      ],
      "Samogitian": [
        ""
      ],
      "Tachelhit": [
        ""
      ],
      "Shan": [
        ""
      ],
      "Serbo-Croatian": [
        ""
      ],
      "Chadian Arabic": [
        ""
      ],
      "Sidamo": [
        ""
      ],
      "Sinhala": [
        ""
      ],
      "Slovak": [
        ""
      ],
      "Lower Silesian": [
        ""
      ],
      "Selayar": [
        ""
      ],
      "Southern Sami": [
        ""
      ],
      "Lule Sami": [
        ""
      ],
      "Inari Sami": [
        ""
      ],
      "Skolt Sami": [
        ""
      ],
      "Soninke": [
        ""
      ],
      "Shona": [
        ""
      ],
      "Sogdien": [
        ""
      ],
      "Somalia kasa": [
        ""
      ],
      "Sranan Tongo": [
        ""
      ],
      "Serer": [
        ""
      ],
      "Swati": [
        ""
      ],
      "Saho": [
        ""
      ],
      "Saterland Frisian": [
        ""
      ],
      "Southern Sotho": [
        ""
      ],
      "Sukuma": [
        ""
      ],
      "Susu": [
        ""
      ],
      "Sundanese": [
        ""
      ],
      "Sumerian": [
        ""
      ],
      "Sweden kasa": [
        ""
      ],
      "Comorian": [
        ""
      ],
      "Congo Swahili": [
        ""
      ],
      "Swahili": [
        ""
      ],
      "Classical Syriac": [
        ""
      ],
      "Syriac": [
        ""
      ],
      "Silesian": [
        ""
      ],
      "Tamil kasa": [
        ""
      ],
      "Tulu": [
        ""
      ],
      "Timne": [
        ""
      ],
      "Teso": [
        ""
      ],
      "Tereno": [
        ""
      ],
      "Telugu": [
        ""
      ],
      "Tetum": [
        ""
      ],
      "Taeland kasa": [
        ""
      ],
      "Tigre": [
        ""
      ],
      "Tigrinya": [
        ""
      ],
      "Tiv": [
        "Чаевые"
      ],
      "Tokelau": [
        ""
      ],
      "Tsakhur": [
        ""
      ],
      "Turkmen": [
        ""
      ],
      "Klingon": [
        ""
      ],
      "Tlingit": [
        ""
      ],
      "Tagalog": [
        ""
      ],
      "Talysh": [
        ""
      ],
      "Tamashek": [
        ""
      ],
      "Tswana": [
        ""
      ],
      "Nyasa Tonga": [
        ""
      ],
      "Tok Pisin": [
        ""
      ],
      "Tɛɛki kasa": [
        ""
      ],
      "Turoyo": [
        ""
      ],
      "Taroko": [
        ""
      ],
      "Tsakonian": [
        ""
      ],
      "Tsimshian": [
        ""
      ],
      "Tsonga": [
        ""
      ],
      "Tatar": [
        ""
      ],
      "Muslim Tat": [
        ""
      ],
      "Tumbuka": [
        ""
      ],
      "Tasawaq": [
        ""
      ],
      "Twi": [
        "Чаевые"
      ],
      "Tahitian": [
        ""
      ],
      "Tuvinian": [
        ""
      ],
      "Central Atlas Tamazight": [
        ""
      ],
      "Udmurt": [
        ""
      ],
      "Ugaritic": [
        ""
      ],
      "Uyghur": [
        ""
      ],
      "Ukren kasa": [
        ""
      ],
      "Umbundu": [
        ""
      ],
      "Unknown Language": [
        ""
      ],
      "Urdu kasa": [
        ""
      ],
      "Uzbek": [
        ""
      ],
      "Vai": [
        ""
      ],
      "Venetian": [
        ""
      ],
      "Veps": [
        ""
      ],
      "Venda": [
        ""
      ],
      "Viɛtnam kasa": [
        ""
      ],
      "West Flemish": [
        ""
      ],
      "Main-Franconian": [
        ""
      ],
      "Votic": [
        ""
      ],
      "Volapük": [
        ""
      ],
      "Võro": [
        ""
      ],
      "Vunjo": [
        ""
      ],
      "Walser": [
        ""
      ],
      "Wolaytta": [
        ""
      ],
      "Waray": [
        ""
      ],
      "Washo": [
        ""
      ],
      "Walloon": [
        ""
      ],
      "Warlpiri": [
        ""
      ],
      "Wolof": [
        ""
      ],
      "Wu Chinese": [
        ""
      ],
      "Kalmyk": [
        ""
      ],
      "Xhosa": [
        ""
      ],
      "Mingrelian": [
        ""
      ],
      "Soga": [
        ""
      ],
      "Yao": [
        ""
      ],
      "Yapese": [
        ""
      ],
      "Yangben": [
        ""
      ],
      "Yemba": [
        ""
      ],
      "Yiddish": [
        ""
      ],
      "Yoruba": [
        ""
      ],
      "Nheengatu": [
        ""
      ],
      "Cantonese": [
        ""
      ],
      "Zapotec": [
        ""
      ],
      "Zhuang": [
        ""
      ],
      "Blissymbols": [
        ""
      ],
      "Zeelandic": [
        ""
      ],
      "Zenaga": [
        ""
      ],
      "Standard Moroccan Tamazight": [
        ""
      ],
      "Simplified Chinese": [
        ""
      ],
      "Traditional Chinese": [
        ""
      ],
      "Kyaena kasa": [
        ""
      ],
      "Zuni": [
        ""
      ],
      "Zulu": [
        ""
      ],
      "No linguistic content": [
        ""
      ],
      "Zaza": [
        ""
      ],
      "Could not load user attention request": [
        "Не удалось загрузить комиссию за обмен"
      ],
      "No notification left": [
        ""
      ],
      "Ignore": [
        ""
      ],
      "Invalid dev experiment URI.": [
        ""
      ],
      "URI: %1$s": [
        "%1$s"
      ],
      "Experiment": [
        ""
      ],
      "Insert lost denomination": [
        "Деноминация"
      ],
      "Apply": [
        ""
      ],
      "Pending refresh": [
        ""
      ],
      "Could not load the transfer payment status": [
        "Не удалось загрузить статус оплаты"
      ],
      "Could not load the status of the order template": [
        "Не удалось загрузить список известных обменников"
      ],
      "required": [
        ""
      ],
      "Review order": [
        ""
      ],
      "Missing URI": [
        ""
      ],
      "Check the link.": [
        ""
      ],
      "Could not parse the recovery URI": [
        "Не удалось получить информацию из URI"
      ],
      "Try another URI": [
        "На другой кошелек"
      ],
      "Import backup, show info": [
        "Импорт резервной копии, отображение информации"
      ],
      "Add new exchange": [
        "Добавить новый Обменник"
      ],
      "Add exchange for %1$s": [
        ""
      ],
      "Enter the URL of an exchange you trust.": [
        ""
      ],
      "An exchange has been found! Review the information and click next": [
        ""
      ],
      "This exchange is already in your list.": [
        ""
      ],
      "Only exchange accessible through \"http\" and \"https\" are allowed.": [
        ""
      ],
      "This exchange protocol version is not supported: \" %1$s\".": [
        ""
      ],
      "This exchange currency \"%1$s\" doesn&apos;t match the expected currency %2$s.": [
        ""
      ],
      "No exchange found in that URL.": [
        ""
      ],
      "loading": [
        "загрузка"
      ],
      "Version": [
        "Версия"
      ],
      "Can't use the URL: \"%1$s\"": [
        ""
      ],
      "Exchange URL": [
        "URL обменника"
      ],
      "Add new contact": [
        "Добавить другой счёт"
      ],
      "Enter the details of a contact you trust.": [
        ""
      ],
      "Contact info valid.": [
        ""
      ],
      "This contact is already in your list.": [
        ""
      ],
      "Review contact": [
        ""
      ],
      "Contact alias": [
        ""
      ],
      "Alias type": [
        ""
      ],
      "Mailbox service": [
        ""
      ],
      "Mailbox address": [
        "Биткоин адрес"
      ],
      "Contact source": [
        ""
      ],
      "Could not load the list of contacts": [
        "Не удалось загрузить список известных обменников"
      ],
      "Source": [
        ""
      ],
      "Send Cash": [
        "Отправить %1$s"
      ],
      "Request Cash": [
        ""
      ],
      "No contacts found.": [
        ""
      ],
      "Could not bank choices for account": [
        "Добавить другой счёт"
      ],
      "Could not load the list of messages": [
        "Не удалось загрузить список известных обменников"
      ],
      "Unexpected error when trying to initialize mailbox: %1$s": [
        ""
      ],
      "Unexpected error when trying to delete message: %1$s": [
        ""
      ],
      "Unexpected error when trying to fetch messages: %1$s": [
        ""
      ],
      "Date": [
        "Дата"
      ],
      "You may copy your mailbox URI from below and register it at %1$s with your alias(es) to make it easier for others to find you.": [
        ""
      ],
      "copy uri": [
        "копировать"
      ],
      "hide uri": [
        "спрятать qr"
      ],
      "show uri": [
        "показать qr"
      ],
      "Fetch messages": [
        "Сообщение продавца"
      ],
      "Mailbox not initialized": [
        ""
      ],
      "Initialize mailbox": [
        ""
      ],
      "No messages yet.": [
        ""
      ],
      "Taler URI handler": [
        ""
      ],
      "Could not found a handler for the Taler URI": [
        "Не удалось найти ни одного обменника для валюты %1$s"
      ],
      "The uri read in the path parameter is not valid: \"%1$s\"": [
        ""
      ],
      "Digital cash payment": [
        "Оплата цифровой наличкой"
      ],
      "Digital cash refund": [
        "Возврат цифровой налички"
      ],
      "Digital cash withdrawal": [
        "Вывод цифровых наличных"
      ],
      "Digital cash deposit": [
        "Депозит цифровой налички"
      ],
      "Digital cash invoice": [
        "Цифровой счёт-фактура"
      ],
      "Digital cash transfer": [
        ""
      ],
      "Digital cash recovery": [
        "Возврат цифровой налички"
      ],
      "Development experiment": [
        "Режим разработчика"
      ],
      "Return to wallet": [
        ""
      ],
      "Network is offline": [
        ""
      ],
      "Diagnostics timed out. Could not talk to the wallet backend.": [
        ""
      ],
      "Problems detected:": [
        ""
      ],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).": [
        ""
      ],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.": [
        ""
      ],
      "here": [
        ""
      ],
      "Running diagnostics": [
        ""
      ],
      "Edit": [
        "Изменить"
      ],
      "List of products": [
        "Список продуктов"
      ],
      "free": [
        "комиссия"
      ],
      "Could not get provider information": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=3; plural=n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2;",
  "lang": "ru",
  "completeness": 18
};

strings['nl'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "nl"
      },
      "Balance": [
        ""
      ],
      "Backup": [
        ""
      ],
      "Notifications": [
        ""
      ],
      "QR Reader and Taler URI": [
        ""
      ],
      "Settings": [
        ""
      ],
      "Dev tools": [
        ""
      ],
      "Contacts": [
        ""
      ],
      "Mailbox": [
        ""
      ],
      "Search transactions": [
        ""
      ],
      "%1$s": [
        ""
      ],
      "KYC requirement": [
        ""
      ],
      "PENDING OPERATIONS": [
        ""
      ],
      "Error was thrown trying to: \"%1$s\"": [
        ""
      ],
      "Could not open the wallet database. Firefox is known to run into this problem under \"permanent private mode\".": [
        ""
      ],
      "Could not open the wallet database.": [
        ""
      ],
      "Unexpected request error, code: %1$s": [
        ""
      ],
      "more info": [
        ""
      ],
      "wrap text": [
        ""
      ],
      "copy content": [
        ""
      ],
      "less info": [
        ""
      ],
      "Retry operation": [
        ""
      ],
      "Loading": [
        ""
      ],
      "Could not load backup providers": [
        ""
      ],
      "No backup providers configured": [
        ""
      ],
      "Add provider": [
        ""
      ],
      "Sync all backups": [
        ""
      ],
      "Sync now": [
        ""
      ],
      "Last synced": [
        ""
      ],
      "Not synced": [
        ""
      ],
      "Expires in": [
        ""
      ],
      "show less": [
        ""
      ],
      "show more": [
        ""
      ],
      "There was an error loading the provider detail for &quot;%1$s&quot;": [
        ""
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [
        ""
      ],
      "See providers": [
        ""
      ],
      "Last backup": [
        ""
      ],
      "Back up": [
        ""
      ],
      "Provider fee": [
        ""
      ],
      "per year": [
        ""
      ],
      "Extend": [
        ""
      ],
      "terms has changed, extending the service will imply accepting the new terms of service": [
        ""
      ],
      "old": [
        ""
      ],
      "new": [
        ""
      ],
      "fee": [
        ""
      ],
      "storage": [
        ""
      ],
      "Remove provider": [
        ""
      ],
      "Pay": [
        ""
      ],
      "Withdraw": [
        ""
      ],
      "This provider has reported an error": [
        ""
      ],
      "There is conflict with another backup from &quot;%1$s&quot;": [
        ""
      ],
      "Backup is not readable": [
        ""
      ],
      "Unknown backup problem: %1$s": [
        ""
      ],
      "service paid": [
        ""
      ],
      "Backup valid until": [
        ""
      ],
      "Cancel": [
        ""
      ],
      "Open pay page": [
        ""
      ],
      "Open refund page": [
        ""
      ],
      "Open withdraw page": [
        ""
      ],
      "Your wallet is empty.": [
        ""
      ],
      "Get digital cash": [
        ""
      ],
      "Try the demo bank and withdraw test money.": [
        ""
      ],
      "Could not load the balance": [
        ""
      ],
      "Receive": [
        ""
      ],
      "Send %1$s": [
        ""
      ],
      "This page has a withdrawal action.": [
        ""
      ],
      "This page has pay action.": [
        ""
      ],
      "This page has a refund action.": [
        ""
      ],
      "This page has a add exchange action.": [
        ""
      ],
      "Open add exchange page": [
        ""
      ],
      "This page has an add contact action.": [
        ""
      ],
      "Open add contact page": [
        ""
      ],
      "Taler Action": [
        ""
      ],
      "This page has a malformed taler uri.": [
        ""
      ],
      "Dismiss": [
        ""
      ],
      "this popup is being closed and you are being redirected to %1$s": [
        ""
      ],
      "Could not load purchase proposal details": [
        ""
      ],
      "Order Id": [
        ""
      ],
      "Summary": [
        ""
      ],
      "Amount": [
        ""
      ],
      "Choices": [
        ""
      ],
      "This purchase have %1$s choices": [
        ""
      ],
      "Merchant name": [
        ""
      ],
      "Merchant jurisdiction": [
        ""
      ],
      "Merchant address": [
        ""
      ],
      "Merchant logo": [
        ""
      ],
      "Merchant website": [
        ""
      ],
      "Merchant email": [
        ""
      ],
      "Merchant public key": [
        ""
      ],
      "Created at": [
        ""
      ],
      "Refund deadline": [
        ""
      ],
      "Auto refund": [
        ""
      ],
      "Pay deadline": [
        ""
      ],
      "Fulfillment URL": [
        ""
      ],
      "Fulfillment message": [
        ""
      ],
      "Max deposit fee": [
        ""
      ],
      "Exchanges": [
        ""
      ],
      "Could not load the status of the term of service": [
        ""
      ],
      "Select one option": [
        ""
      ],
      "Show terms of service": [
        ""
      ],
      "I accept the exchange terms of service": [
        ""
      ],
      "Review exchange terms of service": [
        ""
      ],
      "The exchange doesn't have a terms of service.": [
        ""
      ],
      "Skip it for now.": [
        ""
      ],
      "Format": [
        ""
      ],
      "Language": [
        ""
      ],
      "The exchange replied with a empty terms of service": [
        ""
      ],
      "No terms of service. The exchange replied with a empty document": [
        ""
      ],
      "No terms of service. The exchange replied with a empty text": [
        ""
      ],
      "Download Terms of Service": [
        ""
      ],
      "Hide terms of service": [
        ""
      ],
      "Amount is not valid": [
        ""
      ],
      "Bank account": [
        ""
      ],
      "Bitcoin address": [
        ""
      ],
      "BIC/IBAN": [
        ""
      ],
      "IBAN": [
        ""
      ],
      "Bitcoin transfer details": [
        ""
      ],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.": [
        ""
      ],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts": [
        ""
      ],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC": [
        ""
      ],
      "Ethereum transfer details": [
        ""
      ],
      "You need to wire to the service provider account.": [
        ""
      ],
      "Make sure the amount show %1$s ETH": [
        ""
      ],
      "Bank transfer details": [
        ""
      ],
      "Account": [
        ""
      ],
      "Bank host": [
        ""
      ],
      "BIC": [
        ""
      ],
      "Exchange": [
        "Taler Exchange"
      ],
      "Reserve Pub": [
        ""
      ],
      "Step 1:": [
        ""
      ],
      "Copy this code and paste it into the subject/purpose field in your banking app or bank website": [
        ""
      ],
      "Subject": [
        ""
      ],
      "Step 2:": [
        ""
      ],
      "If you don't already have it in your banking favourites list, then copy and paste this IBAN and the name into the receiver fields in your banking app or website": [
        ""
      ],
      "Receiver name": [
        ""
      ],
      "Receiver postal code": [
        ""
      ],
      "Receiver town": [
        ""
      ],
      "Step 3:": [
        ""
      ],
      "Finish the wire transfer setting the amount in your banking app or website, then this withdrawal will proceed automatically.": [
        ""
      ],
      "Make sure ALL data is correct, including the subject; otherwise, the money will not arrive in this wallet. You can use the copy buttons (%1$s) to prevent typing errors or the \"payto://\" URI below to copy just one value.": [
        ""
      ],
      "Alternative if your bank already supports PayTo URI, you can use this %1$s link instead": [
        ""
      ],
      "Continue with banking app or website": [
        ""
      ],
      "Show QR code": [
        ""
      ],
      "Could not load transaction information": [
        ""
      ],
      "Bank account confirmation required for the transaction to complete.": [
        ""
      ],
      "KYC check required for the transaction to complete.": [
        ""
      ],
      "No additional information has been provided.": [
        ""
      ],
      "Follow this link to the%1$s %2$s": [
        ""
      ],
      "This transaction is pending to complete.": [
        ""
      ],
      "Retry": [
        ""
      ],
      "There was an error trying to complete the transaction.": [
        ""
      ],
      "This transaction was aborted.": [
        ""
      ],
      "This transaction failed.": [
        ""
      ],
      "Caution!": [
        ""
      ],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.": [
        ""
      ],
      "Confirm": [
        ""
      ],
      "Doing a cancellation while the transaction still active might result in lost coins. Do you still want to cancel the transaction?": [
        ""
      ],
      "No": [
        ""
      ],
      "Yes": [
        ""
      ],
      "Abort": [
        ""
      ],
      "Resume": [
        ""
      ],
      "Suspend": [
        ""
      ],
      "Delete": [
        ""
      ],
      "Withdrawal": [
        ""
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Select one of the accounts and use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Details": [
        ""
      ],
      "Withdrawal incomplete.": [
        ""
      ],
      "If you have already sent money to the service provider account it will wire it back at %1$s": [
        ""
      ],
      "Payment": [
        ""
      ],
      "Refunds": [
        ""
      ],
      "%1$s %2$s on %3$s .": [
        ""
      ],
      "Refund is in progress.": [
        ""
      ],
      "Merchant created a refund for this order but was not automatically picked up.": [
        ""
      ],
      "Offer": [
        ""
      ],
      "Accept": [
        ""
      ],
      "Confirmation code": [
        ""
      ],
      "Merchant": [
        ""
      ],
      "Invoice ID": [
        ""
      ],
      "Deposit": [
        ""
      ],
      "Wire transfer deadline.": [
        ""
      ],
      "Wire transfer still pending.": [
        ""
      ],
      "The service provider deadline to make the wire transfer is: %1$s": [
        ""
      ],
      "Wire transfer completed.": [
        ""
      ],
      " ": [
        ""
      ],
      "Transfer details": [
        ""
      ],
      "Wire transfer in progress.": [
        ""
      ],
      "Refresh": [
        ""
      ],
      "Refund": [
        ""
      ],
      "Purchase summary": [
        ""
      ],
      "Credit": [
        ""
      ],
      "Invoice": [
        ""
      ],
      "URI": [
        ""
      ],
      "Debit": [
        ""
      ],
      "Transfer": [
        ""
      ],
      "Lost": [
        ""
      ],
      "Reason": [
        ""
      ],
      "Denomination expired.": [
        ""
      ],
      "Denomination vanished.": [
        ""
      ],
      "Denomination is unoffered.": [
        ""
      ],
      "Fees": [
        ""
      ],
      "Total": [
        ""
      ],
      "Sent": [
        ""
      ],
      "Converted": [
        ""
      ],
      "Bank fee": [
        ""
      ],
      "Price": [
        ""
      ],
      "Subtotal": [
        ""
      ],
      "Refunded": [
        ""
      ],
      "Transfer identification": [
        ""
      ],
      "copy": [
        ""
      ],
      "hide qr": [
        ""
      ],
      "forward to contact": [
        ""
      ],
      "show qr": [
        ""
      ],
      "Wire transfer need a confirmation. Go to the %1$s and check wire transfer operation to exchange account is complete.": [
        ""
      ],
      "bank site": [
        ""
      ],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins.": [
        ""
      ],
      "Exchange is ready to send the coins, withdrawal in progress.": [
        ""
      ],
      "Send &nbsp; %1$s": [
        ""
      ],
      "Pay &nbsp; %1$s": [
        ""
      ],
      "Merchant message": [
        ""
      ],
      "Balance is not enough.": [
        ""
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which exchange and auditors the merchant trust.": [
        ""
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which wire methods the merchant accepts.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to pay for this contract which is restricted.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to spend right know. There are some coins that need to be refreshed.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s available.": [
        ""
      ],
      "Provider is missing the global fee configuration, this likely means it is misconfigured.": [
        ""
      ],
      "Balance looks like it should be enough, but doesn't cover all fees requested by the merchant and payment processor. Please ensure there is at least %1$s %2$s more balance in your wallet or ask your merchant to cover more of the fees.": [
        ""
      ],
      "Pay with a mobile phone": [
        ""
      ],
      "Hide QR": [
        ""
      ],
      "Scan the QR code or &nbsp; %1$s": [
        ""
      ],
      "click here": [
        ""
      ],
      "Could not load the payment and balance status": [
        ""
      ],
      "Purchase": [
        ""
      ],
      "This purchase is age restricted.": [
        ""
      ],
      "Expires at": [
        ""
      ],
      "Already paid, you are going to be redirected to %1$s": [
        ""
      ],
      "Already paid": [
        ""
      ],
      "Already claimed": [
        ""
      ],
      "Could not load the refund status": [
        ""
      ],
      "You&apos;ve ignored the refund.": [
        ""
      ],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [
        ""
      ],
      "Order amount": [
        ""
      ],
      "Could not load the list of exchanges": [
        ""
      ],
      "Could not load info from URI": [
        ""
      ],
      "Could not load the withdrawal details": [
        ""
      ],
      "under %1$s": [
        ""
      ],
      "should be greater than zero": [
        ""
      ],
      "can't be greater than %1$s": [
        ""
      ],
      "Could not load exchange details info": [
        ""
      ],
      "Close": [
        ""
      ],
      "There is no exchange available for currency %1$s": [
        ""
      ],
      "You can add more exchanges from the settings.": [
        ""
      ],
      "Exchange %1$s is not available": [
        ""
      ],
      "Exchange status can view accessed from the settings.": [
        ""
      ],
      "Service fee description": [
        ""
      ],
      "Select %1$s exchange": [
        ""
      ],
      "Reset": [
        ""
      ],
      "Use this exchange": [
        ""
      ],
      "Auditors": [
        ""
      ],
      "Doesn&apos;t have auditors": [
        ""
      ],
      "currency": [
        ""
      ],
      "Coin operations": [
        ""
      ],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.": [
        ""
      ],
      "Deposits": [
        ""
      ],
      "Denomination": [
        ""
      ],
      "Current": [
        ""
      ],
      "Selected": [
        ""
      ],
      "Until": [
        ""
      ],
      "Withdrawals": [
        ""
      ],
      "Transfer operations": [
        ""
      ],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.": [
        ""
      ],
      "Operation": [
        ""
      ],
      "Fee": [
        ""
      ],
      "Wallet operations": [
        ""
      ],
      "Feature": [
        ""
      ],
      "Currency": [
        ""
      ],
      "This operation has already been completed.": [
        ""
      ],
      "This operation has already been aborted": [
        ""
      ],
      "This operation has started and should be completed in the bank.": [
        ""
      ],
      "You can confirm the operation in": [
        ""
      ],
      "this page": [
        ""
      ],
      "This operation has already been completed by another wallet.": [
        ""
      ],
      "This operation has already been used by another wallet.": [
        ""
      ],
      "It can be confirmed in": [
        ""
      ],
      "Age restriction": [
        ""
      ],
      "Withdraw &nbsp; %1$s": [
        ""
      ],
      "Withdraw to a mobile phone": [
        ""
      ],
      "Short description of the invoice": [
        ""
      ],
      "Create": [
        ""
      ],
      "Short description of the transfer": [
        ""
      ],
      "Could not load the invoice payment status": [
        ""
      ],
      "Receive &nbsp; %1$s": [
        ""
      ],
      "Could not load known bank accounts": [
        ""
      ],
      "Could not load supported wire methods": [
        ""
      ],
      "No wire methods supported for this currency": [
        ""
      ],
      "Try another wire method.": [
        ""
      ],
      "Known accounts for %1$s": [
        ""
      ],
      "To add a new account first select the account type.": [
        ""
      ],
      "Unable add this account": [
        ""
      ],
      "Add": [
        ""
      ],
      "IBAN accounts": [
        ""
      ],
      "Alias": [
        ""
      ],
      "Int. Account Number": [
        ""
      ],
      "Account name": [
        ""
      ],
      "KYC": [
        ""
      ],
      "Taler accounts": [
        ""
      ],
      "Host": [
        ""
      ],
      "Bitcoin accounts": [
        ""
      ],
      "Address": [
        ""
      ],
      "Can't be empty": [
        ""
      ],
      "Bitcoin Account": [
        ""
      ],
      "Taler Bank": [
        ""
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "Required": [
        ""
      ],
      "International Bank Account Number": [
        ""
      ],
      "Could not load balance information": [
        ""
      ],
      "Could not load fee for amount %1$s": [
        ""
      ],
      "A currency or an amount should be indicated": [
        ""
      ],
      "There is no enough balance to make a deposit for currency %1$s": [
        ""
      ],
      "Send %1$s to your account": [
        ""
      ],
      "There is no account to make a deposit for currency %1$s": [
        ""
      ],
      "Add account": [
        ""
      ],
      "Select account": [
        ""
      ],
      "Manage accounts": [
        ""
      ],
      "Brut amount": [
        ""
      ],
      "Deposit fee": [
        ""
      ],
      "Net amount": [
        ""
      ],
      "Deposit&nbsp;%1$s %2$s": [
        ""
      ],
      "Need approval in the Bank": [
        ""
      ],
      "Waiting for wire transfer to complete": [
        ""
      ],
      "Exchange is waiting the wire transfer": [
        ""
      ],
      "Payment in progress": [
        ""
      ],
      "Executing refund...": [
        ""
      ],
      "Refreshing coins...": [
        ""
      ],
      "Deposit in progress": [
        ""
      ],
      "Waiting to be paid": [
        ""
      ],
      "Receiving the transfer": [
        ""
      ],
      "Waiting to be received": [
        ""
      ],
      "Denomination expired": [
        ""
      ],
      "Denomination vanished": [
        ""
      ],
      "Denomination unoffered": [
        ""
      ],
      "ABORTED": [
        ""
      ],
      "FAILED": [
        ""
      ],
      "Could not load the list of transactions": [
        ""
      ],
      "unknown scope": [
        ""
      ],
      "Send": [
        ""
      ],
      "Your transaction history is empty for this currency.": [
        ""
      ],
      "Review terms of service": [
        ""
      ],
      "Provider URL": [
        ""
      ],
      "Please review and accept this provider&apos;s terms of service": [
        ""
      ],
      "Pricing": [
        ""
      ],
      "free of charge": [
        ""
      ],
      "%1$s per year of service": [
        ""
      ],
      "Storage": [
        ""
      ],
      "%1$s megabytes of storage per year of service": [
        ""
      ],
      "Accept terms of service": [
        ""
      ],
      "Add backup provider": [
        ""
      ],
      "Backup providers may charge for their service": [
        ""
      ],
      "URL": [
        ""
      ],
      "Name": [
        ""
      ],
      "Next": [
        ""
      ],
      "Navigator": [
        ""
      ],
      "Automatically open wallet": [
        ""
      ],
      "Open the wallet when a payment action is found.": [
        ""
      ],
      "Version Info": [
        ""
      ],
      "Web Extension": [
        ""
      ],
      "The version of wallet core is not supported. (supported version: %1$s, wallet version: %2$s)": [
        ""
      ],
      "Exchange compatibility": [
        ""
      ],
      "Merchant compatibility": [
        ""
      ],
      "Bank compatibility": [
        ""
      ],
      "Wallet Core compatibility": [
        ""
      ],
      "Enable developer mode": [
        ""
      ],
      "Show more information and options in the UI": [
        ""
      ],
      "Display": [
        ""
      ],
      "Current Language": [
        ""
      ],
      "Show backup feature": [
        ""
      ],
      "Backup integration still in beta.": [
        ""
      ],
      "Show P2P alias feature": [
        ""
      ],
      "P2P aliases integration still in beta.": [
        ""
      ],
      "Show suspend/resume transaction": [
        ""
      ],
      "Prevent transaction from doing network request.": [
        ""
      ],
      "Show refresh transaction type in the transaction list": [
        ""
      ],
      "Refresh transaction will be hidden by default if the refresh operation doesn't have fee.": [
        ""
      ],
      "Show more account types on deposit": [
        ""
      ],
      "Extends the UI to more payment target types.": [
        ""
      ],
      "Show JSON on error": [
        ""
      ],
      "Print more information about the error. Useful for debugging.": [
        ""
      ],
      "Allow HTTP connections": [
        ""
      ],
      "Using HTTP connection may be faster but unsafe (wallet restart required)": [
        ""
      ],
      "Lang selector": [
        ""
      ],
      "Allows to manually change the language of the UI. Otherwise it will be automatically selected by your browser configuration.": [
        ""
      ],
      "Edit exchange management": [
        ""
      ],
      "Allows to see the list of exchange, remove, add and switch before withdrawal.": [
        ""
      ],
      "Select terms of service format": [
        ""
      ],
      "Allows to render the terms of service on different format selected by the user.": [
        ""
      ],
      "Show wallet activity": [
        ""
      ],
      "Show the wallet notification and observability event in the UI.": [
        ""
      ],
      "Enable v1 contract": [
        ""
      ],
      "v1 is under development and wallet-core converts v1 contract to v0. Enable this options for testing": [
        ""
      ],
      "GNU Taler Wallet installed!": [
        ""
      ],
      "You can open the wallet using the combination %1$s .": [
        ""
      ],
      "Also pinning the GNU Taler Wallet to your browser allows you to quick access without keyboard:": [
        ""
      ],
      "Click the puzzle icon": [
        ""
      ],
      "Search for GNU Taler Wallet": [
        ""
      ],
      "Click the pin icon": [
        ""
      ],
      "Next Steps": [
        ""
      ],
      "Try the demo": [
        ""
      ],
      "Click here to open the wallet activity tab.": [
        ""
      ],
      "Events": [
        ""
      ],
      "Active tasks": [
        ""
      ],
      "All events": [
        ""
      ],
      "Balance change": [
        ""
      ],
      "Backup failed": [
        ""
      ],
      "Transaction updated": [
        ""
      ],
      "Exchange updated": [
        "Taler Exchange"
      ],
      "Idle": [
        ""
      ],
      "task.%1$s": [
        ""
      ],
      "wallet.%1$s(%2$s)": [
        ""
      ],
      "Bank account info changed": [
        ""
      ],
      "Contact added": [
        ""
      ],
      "Contact deleted": [
        ""
      ],
      "Mailbox message added": [
        ""
      ],
      "Mailbox message deleted": [
        ""
      ],
      "Type": [
        ""
      ],
      "Id": [
        ""
      ],
      "Since": [
        ""
      ],
      "Next try": [
        ""
      ],
      "Error": [
        ""
      ],
      "Transaction": [
        ""
      ],
      "No exchange yet": [
        ""
      ],
      "Status": [
        ""
      ],
      "Terms of Service": [
        ""
      ],
      "Last Update": [
        ""
      ],
      "Actions": [
        ""
      ],
      "ok": [
        ""
      ],
      "pending": [
        ""
      ],
      "proposed": [
        ""
      ],
      "unknown (exchange status should be updated)": [
        ""
      ],
      "Reload": [
        ""
      ],
      "Purge": [
        ""
      ],
      "Make regional": [
        ""
      ],
      "Make global": [
        ""
      ],
      "Forget ToS": [
        ""
      ],
      "Debug tools": [
        ""
      ],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?": [
        ""
      ],
      "reset": [
        ""
      ],
      "TESTING: This may delete all your coin, proceed with caution": [
        ""
      ],
      "run gc": [
        ""
      ],
      "import database": [
        ""
      ],
      "export database": [
        ""
      ],
      "Clear exchange key cache": [
        ""
      ],
      "Database exported at %1$s %2$s to download": [
        ""
      ],
      "Inject Taler support in all pages": [
        ""
      ],
      "Enabling this option will make `window.taler` be available in all sites": [
        ""
      ],
      "Exchange Entries": [
        "Taler Exchange"
      ],
      "Add an exchange": [
        ""
      ],
      "Log levels": [
        ""
      ],
      "Coins": [
        ""
      ],
      "usable coins": [
        ""
      ],
      "id": [
        ""
      ],
      "denom": [
        ""
      ],
      "status": [
        ""
      ],
      "from refresh?": [
        ""
      ],
      "age key count": [
        ""
      ],
      "spent coins": [
        ""
      ],
      "click to show": [
        ""
      ],
      "URI is not valid. Taler URI should start with \"taler://\"": [
        ""
      ],
      "After the URI type it should follow a '/' with more information.": [
        ""
      ],
      "This URI type is not supported": [
        ""
      ],
      "This URI requires the bank host and operation id separated by /": [
        ""
      ],
      "This URI requires the merchant host, order id and session id separated by /": [
        ""
      ],
      "This URI requires the merchant host and order id separated by /": [
        ""
      ],
      "This URI requires the exchange host and key id separated by /": [
        ""
      ],
      "This URI requires the merchant host and template id separated by /": [
        ""
      ],
      "This URI requires the host key and provider list separated by /": [
        ""
      ],
      "This URI requires the experiment id": [
        ""
      ],
      "This URI requires the exchange host": [
        ""
      ],
      "This URI requires the alias type, the alias and the mailbox URI separated by /": [
        ""
      ],
      "This URI requires string after the first /": [
        ""
      ],
      "The bank host is invalid": [
        ""
      ],
      "The merchant host is invalid": [
        ""
      ],
      "The URI should end with /": [
        ""
      ],
      "The exchange host is invalid": [
        ""
      ],
      "The contact is invalid": [
        ""
      ],
      "The amount is invalid": [
        ""
      ],
      "A parameter is invalid": [
        ""
      ],
      "checking...": [
        ""
      ],
      "Unexpected error happen reading the camera: %1$s": [
        ""
      ],
      "Could not found a QR code in the file": [
        ""
      ],
      "Unexpected error happen reading the file: %1$s": [
        ""
      ],
      "Scan a QR code or enter taler:// URI below": [
        ""
      ],
      "Pay invoice": [
        ""
      ],
      "Withdrawal from bank": [
        ""
      ],
      "Claim refund": [
        ""
      ],
      "Accept payment": [
        ""
      ],
      "Complete order": [
        ""
      ],
      "Restore wallet": [
        ""
      ],
      "Enable experiment": [
        ""
      ],
      "Withdraw from exchange": [
        ""
      ],
      "Add exchange": [
        ""
      ],
      "Notify transaction": [
        ""
      ],
      "Add contact": [
        ""
      ],
      "The exchange URL is invalid.": [
        ""
      ],
      "The exchange URL should end with '/'": [
        ""
      ],
      "Couldn't found an exchange in the URL specified.": [
        ""
      ],
      "HTTP request failed to %1$s: %2$s": [
        ""
      ],
      "HTTP request failed to %1$s": [
        ""
      ],
      "The merchant URL is invalid.": [
        ""
      ],
      "The merchant URL should end with '/'": [
        ""
      ],
      "Couldn't found an merchant in the URL specified.": [
        ""
      ],
      "The bank URL is invalid.": [
        ""
      ],
      "The bank URL should end with '/'": [
        ""
      ],
      "Couldn't found an bank in the URL specified.": [
        ""
      ],
      "Could not load exchanges": [
        ""
      ],
      "Choose a currency to proceed or add another exchange": [
        ""
      ],
      "Known currencies": [
        ""
      ],
      "Specify the origin": [
        ""
      ],
      "Use previous origins:": [
        ""
      ],
      "From another bank account": [
        ""
      ],
      "From another wallet": [
        ""
      ],
      "From a %1$s URI or QR code": [
        ""
      ],
      "Enter URI here": [
        ""
      ],
      "Specify the destination": [
        ""
      ],
      "Use previous destinations:": [
        ""
      ],
      "To another bank account": [
        ""
      ],
      "To another wallet": [
        ""
      ],
      "To a %1$s URI or QR code": [
        ""
      ],
      "Afghanistan": [
        ""
      ],
      "Albania": [
        ""
      ],
      "Algeria": [
        ""
      ],
      "United States": [
        ""
      ],
      "Andorra": [
        ""
      ],
      "Angola": [
        ""
      ],
      "Antigua and Barbuda": [
        ""
      ],
      "Argentina": [
        ""
      ],
      "Armenia": [
        ""
      ],
      "Australia": [
        ""
      ],
      "Austria": [
        ""
      ],
      "Azerbaijan": [
        ""
      ],
      "Bahamas": [
        ""
      ],
      "Bahrain": [
        ""
      ],
      "Bangladesh": [
        ""
      ],
      "Barbados": [
        ""
      ],
      "Botswana": [
        ""
      ],
      "Belarus": [
        ""
      ],
      "Belgium": [
        ""
      ],
      "Belize": [
        ""
      ],
      "Benin": [
        ""
      ],
      "Bhutan": [
        ""
      ],
      "Bolivia": [
        ""
      ],
      "Bosnia and Herzegovina": [
        ""
      ],
      "Brazil": [
        ""
      ],
      "United Kingdom": [
        ""
      ],
      "Brunei": [
        ""
      ],
      "Bulgaria": [
        ""
      ],
      "Burkina Faso": [
        ""
      ],
      "Myanmar": [
        ""
      ],
      "Burundi": [
        ""
      ],
      "Cambodia": [
        ""
      ],
      "Cameroon": [
        ""
      ],
      "Canada": [
        ""
      ],
      "Cape Verde": [
        ""
      ],
      "Central African Republic": [
        ""
      ],
      "Chad": [
        ""
      ],
      "Chile": [
        ""
      ],
      "China": [
        ""
      ],
      "Colombia": [
        ""
      ],
      "Comoros": [
        ""
      ],
      "Democratic Republic of the Congo": [
        ""
      ],
      "Republic of the Congo": [
        ""
      ],
      "Costa Rica": [
        ""
      ],
      "Croatia": [
        ""
      ],
      "Cuba": [
        ""
      ],
      "Cyprus": [
        ""
      ],
      "Czech Republic": [
        ""
      ],
      "Denmark": [
        ""
      ],
      "Djibouti": [
        ""
      ],
      "Dominican Republic": [
        ""
      ],
      "Netherlands": [
        ""
      ],
      "Timor-Leste": [
        ""
      ],
      "Ecuador": [
        ""
      ],
      "Egypt": [
        ""
      ],
      "United Arab Emirates": [
        ""
      ],
      "Equatorial Guinea": [
        ""
      ],
      "Eritrea": [
        ""
      ],
      "Estonia": [
        ""
      ],
      "Ethiopia": [
        ""
      ],
      "Fiji": [
        ""
      ],
      "Philippines": [
        ""
      ],
      "Finland": [
        ""
      ],
      "France": [
        ""
      ],
      "Gabon": [
        ""
      ],
      "Gambia": [
        ""
      ],
      "Georgia": [
        ""
      ],
      "Germany": [
        ""
      ],
      "Ghana": [
        ""
      ],
      "Greece": [
        ""
      ],
      "Grenada": [
        ""
      ],
      "Guatemala": [
        ""
      ],
      "Guinea-Bissau": [
        ""
      ],
      "Guinea": [
        ""
      ],
      "Guyana": [
        ""
      ],
      "Haiti": [
        ""
      ],
      "Honduras": [
        ""
      ],
      "Hungary": [
        ""
      ],
      "Kiribati": [
        ""
      ],
      "Iceland": [
        ""
      ],
      "India": [
        ""
      ],
      "Indonesia": [
        ""
      ],
      "Iran": [
        ""
      ],
      "Iraq": [
        ""
      ],
      "Ireland": [
        ""
      ],
      "Israel": [
        ""
      ],
      "Italy": [
        ""
      ],
      "Ivory Coast": [
        ""
      ],
      "Jamaica": [
        ""
      ],
      "Japan": [
        ""
      ],
      "Jordan": [
        ""
      ],
      "Kazakhstan": [
        ""
      ],
      "Kenya": [
        ""
      ],
      "Saint Kitts and Nevis": [
        ""
      ],
      "Kuwait": [
        ""
      ],
      "Kyrgyzstan": [
        ""
      ],
      "Laos": [
        ""
      ],
      "Latvia": [
        ""
      ],
      "Lebanon": [
        ""
      ],
      "Liberia": [
        ""
      ],
      "Libya": [
        ""
      ],
      "Liechtenstein": [
        ""
      ],
      "Lithuania": [
        ""
      ],
      "Luxembourg": [
        ""
      ],
      "North Macedonia": [
        ""
      ],
      "Madagascar": [
        ""
      ],
      "Malawi": [
        ""
      ],
      "Malaysia": [
        ""
      ],
      "Maldives": [
        ""
      ],
      "Mali": [
        ""
      ],
      "Malta": [
        ""
      ],
      "Marshall Islands": [
        ""
      ],
      "Mauritania": [
        ""
      ],
      "Mauritius": [
        ""
      ],
      "Mexico": [
        ""
      ],
      "Micronesia": [
        ""
      ],
      "Moldova": [
        ""
      ],
      "Monaco": [
        ""
      ],
      "Mongolia": [
        ""
      ],
      "Morocco": [
        ""
      ],
      "Lesotho": [
        ""
      ],
      "Mozambique": [
        ""
      ],
      "Namibia": [
        ""
      ],
      "Nauru": [
        ""
      ],
      "Nepal": [
        ""
      ],
      "New Zealand": [
        ""
      ],
      "Vanuatu": [
        ""
      ],
      "Nicaragua": [
        ""
      ],
      "Nigeria": [
        ""
      ],
      "Niger": [
        ""
      ],
      "North Korea": [
        ""
      ],
      "Norway": [
        ""
      ],
      "Oman": [
        ""
      ],
      "Pakistan": [
        ""
      ],
      "Palau": [
        ""
      ],
      "Panama": [
        ""
      ],
      "Papua New Guinea": [
        ""
      ],
      "Paraguay": [
        ""
      ],
      "Peru": [
        ""
      ],
      "Poland": [
        ""
      ],
      "Portugal": [
        ""
      ],
      "Qatar": [
        ""
      ],
      "Romania": [
        ""
      ],
      "Russia": [
        ""
      ],
      "Rwanda": [
        ""
      ],
      "Saint Lucia": [
        ""
      ],
      "El Salvador": [
        ""
      ],
      "Samoa": [
        ""
      ],
      "San Marino": [
        ""
      ],
      "São Tomé and Príncipe": [
        ""
      ],
      "Saudi Arabia": [
        ""
      ],
      "Senegal": [
        ""
      ],
      "Serbia": [
        ""
      ],
      "Seychelles": [
        ""
      ],
      "Sierra Leone": [
        ""
      ],
      "Singapore": [
        ""
      ],
      "Slovakia": [
        ""
      ],
      "Slovenia": [
        ""
      ],
      "Solomon Islands": [
        ""
      ],
      "Somalia": [
        ""
      ],
      "South Africa": [
        ""
      ],
      "South Korea": [
        ""
      ],
      "Spain": [
        ""
      ],
      "Sri Lanka": [
        ""
      ],
      "Sudan": [
        ""
      ],
      "Suriname": [
        ""
      ],
      "Eswatini": [
        ""
      ],
      "Sweden": [
        ""
      ],
      "Switzerland": [
        ""
      ],
      "Syria": [
        ""
      ],
      "Taiwan": [
        ""
      ],
      "Tajikistan": [
        ""
      ],
      "Tanzania": [
        ""
      ],
      "Thailand": [
        ""
      ],
      "Togo": [
        ""
      ],
      "Tonga": [
        ""
      ],
      "Trinidad and Tobago": [
        ""
      ],
      "Tunisia": [
        ""
      ],
      "Turkey": [
        ""
      ],
      "Tuvalu": [
        ""
      ],
      "Uganda": [
        ""
      ],
      "Ukraine": [
        ""
      ],
      "Uruguay": [
        ""
      ],
      "Uzbekistan": [
        ""
      ],
      "Venezuela": [
        ""
      ],
      "Vietnam": [
        ""
      ],
      "Yemen": [
        ""
      ],
      "Zambia": [
        ""
      ],
      "Zimbabwe": [
        ""
      ],
      "Afghan": [
        ""
      ],
      "Albanian": [
        ""
      ],
      "Algerian": [
        ""
      ],
      "American": [
        ""
      ],
      "Andorran": [
        ""
      ],
      "Angolan": [
        ""
      ],
      "Antiguan": [
        ""
      ],
      "Argentinean": [
        ""
      ],
      "Armenian": [
        ""
      ],
      "Australian": [
        ""
      ],
      "Austrian": [
        ""
      ],
      "Azerbaijani": [
        ""
      ],
      "Bahamian": [
        ""
      ],
      "Bahraini": [
        ""
      ],
      "Bangladeshi": [
        ""
      ],
      "Barbadian": [
        ""
      ],
      "Batswana": [
        ""
      ],
      "Belarusian": [
        ""
      ],
      "Belgian": [
        ""
      ],
      "Belizean": [
        ""
      ],
      "Beninese": [
        ""
      ],
      "Bhutanese": [
        ""
      ],
      "Bolivian": [
        ""
      ],
      "Bosnian": [
        ""
      ],
      "Brazilian": [
        ""
      ],
      "British": [
        ""
      ],
      "Bruneian": [
        ""
      ],
      "Bulgarian": [
        ""
      ],
      "Burkinabe": [
        ""
      ],
      "Burmese": [
        ""
      ],
      "Burundian": [
        ""
      ],
      "Cambodian": [
        ""
      ],
      "Cameroonian": [
        ""
      ],
      "Canadian": [
        ""
      ],
      "Cape Verdean": [
        ""
      ],
      "Central African": [
        ""
      ],
      "Chadian": [
        ""
      ],
      "Chilean": [
        ""
      ],
      "Chinese": [
        ""
      ],
      "Colombian": [
        ""
      ],
      "Comoran": [
        ""
      ],
      "Congolese": [
        ""
      ],
      "Costa Rican": [
        ""
      ],
      "Croatian": [
        ""
      ],
      "Cuban": [
        ""
      ],
      "Cypriot": [
        ""
      ],
      "Czech": [
        ""
      ],
      "Danish": [
        ""
      ],
      "Djiboutian": [
        ""
      ],
      "Dominican": [
        ""
      ],
      "Dutch": [
        ""
      ],
      "East Timorese": [
        ""
      ],
      "Ecuadorean": [
        ""
      ],
      "Egyptian": [
        ""
      ],
      "Emirati": [
        ""
      ],
      "Equatorial Guinean": [
        ""
      ],
      "Eritrean": [
        ""
      ],
      "Estonian": [
        ""
      ],
      "Ethiopian": [
        ""
      ],
      "Fijian": [
        ""
      ],
      "Filipino": [
        ""
      ],
      "Finnish": [
        ""
      ],
      "French": [
        ""
      ],
      "Gabonese": [
        ""
      ],
      "Gambian": [
        ""
      ],
      "Georgian": [
        ""
      ],
      "German": [
        ""
      ],
      "Ghanaian": [
        ""
      ],
      "Greek": [
        ""
      ],
      "Grenadian": [
        ""
      ],
      "Guatemalan": [
        ""
      ],
      "Guinea-Bissauan": [
        ""
      ],
      "Guinean": [
        ""
      ],
      "Guyanese": [
        ""
      ],
      "Haitian": [
        ""
      ],
      "Honduran": [
        ""
      ],
      "Hungarian": [
        ""
      ],
      "I-Kiribati": [
        ""
      ],
      "Icelander": [
        ""
      ],
      "Indian": [
        ""
      ],
      "Indonesian": [
        ""
      ],
      "Iranian": [
        ""
      ],
      "Iraqi": [
        ""
      ],
      "Irish": [
        ""
      ],
      "Israeli": [
        ""
      ],
      "Italian": [
        ""
      ],
      "Ivorian": [
        ""
      ],
      "Jamaican": [
        ""
      ],
      "Japanese": [
        ""
      ],
      "Jordanian": [
        ""
      ],
      "Kazakhstani": [
        ""
      ],
      "Kenyan": [
        ""
      ],
      "Kittian and Nevisian": [
        ""
      ],
      "Kuwaiti": [
        ""
      ],
      "Kyrgyz": [
        ""
      ],
      "Laotian": [
        ""
      ],
      "Latvian": [
        ""
      ],
      "Lebanese": [
        ""
      ],
      "Liberian": [
        ""
      ],
      "Libyan": [
        ""
      ],
      "Liechtensteiner": [
        ""
      ],
      "Lithuanian": [
        ""
      ],
      "Luxembourger": [
        ""
      ],
      "Macedonian": [
        ""
      ],
      "Malagasy": [
        ""
      ],
      "Malawian": [
        ""
      ],
      "Malaysian": [
        ""
      ],
      "Maldivian": [
        ""
      ],
      "Malian": [
        ""
      ],
      "Maltese": [
        ""
      ],
      "Marshallese": [
        ""
      ],
      "Mauritanian": [
        ""
      ],
      "Mauritian": [
        ""
      ],
      "Mexican": [
        ""
      ],
      "Micronesian": [
        ""
      ],
      "Moldovan": [
        ""
      ],
      "Monacan": [
        ""
      ],
      "Mongolian": [
        ""
      ],
      "Moroccan": [
        ""
      ],
      "Mosotho": [
        ""
      ],
      "Mozambican": [
        ""
      ],
      "Namibian": [
        ""
      ],
      "Nauruan": [
        ""
      ],
      "Nepalese": [
        ""
      ],
      "New Zealander": [
        ""
      ],
      "Ni-Vanuatu": [
        ""
      ],
      "Nicaraguan": [
        ""
      ],
      "Nigerian": [
        ""
      ],
      "Nigerien": [
        ""
      ],
      "North Korean": [
        ""
      ],
      "Norwegian": [
        ""
      ],
      "Omani": [
        ""
      ],
      "Pakistani": [
        ""
      ],
      "Palauan": [
        ""
      ],
      "Panamanian": [
        ""
      ],
      "Papua New Guinean": [
        ""
      ],
      "Paraguayan": [
        ""
      ],
      "Peruvian": [
        ""
      ],
      "Polish": [
        ""
      ],
      "Portuguese": [
        ""
      ],
      "Qatari": [
        ""
      ],
      "Romanian": [
        ""
      ],
      "Russian": [
        ""
      ],
      "Rwandan": [
        ""
      ],
      "Saint Lucian": [
        ""
      ],
      "Salvadoran": [
        ""
      ],
      "Samoan": [
        ""
      ],
      "San Marinese": [
        ""
      ],
      "Sao Tomean": [
        ""
      ],
      "Saudi": [
        ""
      ],
      "Senegalese": [
        ""
      ],
      "Serbian": [
        ""
      ],
      "Seychellois": [
        ""
      ],
      "Sierra Leonean": [
        ""
      ],
      "Singaporean": [
        ""
      ],
      "Slovakian": [
        ""
      ],
      "Slovenian": [
        ""
      ],
      "Solomon Islander": [
        ""
      ],
      "Somali": [
        ""
      ],
      "South African": [
        ""
      ],
      "South Korean": [
        ""
      ],
      "Spanish": [
        ""
      ],
      "Sri Lankan": [
        ""
      ],
      "Sudanese": [
        ""
      ],
      "Surinamer": [
        ""
      ],
      "Swazi": [
        ""
      ],
      "Swedish": [
        ""
      ],
      "Swiss": [
        ""
      ],
      "Syrian": [
        ""
      ],
      "Taiwanese": [
        ""
      ],
      "Tajik": [
        ""
      ],
      "Tanzanian": [
        ""
      ],
      "Thai": [
        ""
      ],
      "Togolese": [
        ""
      ],
      "Tongan": [
        ""
      ],
      "Trinidadian or Tobagonian": [
        ""
      ],
      "Tunisian": [
        ""
      ],
      "Turkish": [
        ""
      ],
      "Tuvaluan": [
        ""
      ],
      "Ugandan": [
        ""
      ],
      "Ukrainian": [
        ""
      ],
      "Uruguayan": [
        ""
      ],
      "Uzbekistani": [
        ""
      ],
      "Venezuelan": [
        ""
      ],
      "Vietnamese": [
        ""
      ],
      "Yemenite": [
        ""
      ],
      "Zambian": [
        ""
      ],
      "Zimbabwean": [
        ""
      ],
      "🇦🇫": [
        ""
      ],
      "🇦🇱": [
        ""
      ],
      "🇩🇿": [
        ""
      ],
      "🇺🇸": [
        ""
      ],
      "🇦🇩": [
        ""
      ],
      "🇦🇴": [
        ""
      ],
      "🇦🇬": [
        ""
      ],
      "🇦🇷": [
        ""
      ],
      "🇦🇲": [
        ""
      ],
      "🇦🇺": [
        ""
      ],
      "🇦🇹": [
        ""
      ],
      "🇦🇿": [
        ""
      ],
      "🇧🇸": [
        ""
      ],
      "🇧🇭": [
        ""
      ],
      "🇧🇩": [
        ""
      ],
      "🇧🇧": [
        ""
      ],
      "🇧🇼": [
        ""
      ],
      "🇧🇾": [
        ""
      ],
      "🇧🇪": [
        ""
      ],
      "🇧🇿": [
        ""
      ],
      "🇧🇯": [
        ""
      ],
      "🇧🇹": [
        ""
      ],
      "🇧🇴": [
        ""
      ],
      "🇧🇦": [
        ""
      ],
      "🇧🇷": [
        ""
      ],
      "🇬🇧": [
        ""
      ],
      "🇧🇳": [
        ""
      ],
      "🇧🇬": [
        ""
      ],
      "🇧🇫": [
        ""
      ],
      "🇲🇲": [
        ""
      ],
      "🇧🇮": [
        ""
      ],
      "🇰🇭": [
        ""
      ],
      "🇨🇲": [
        ""
      ],
      "🇨🇦": [
        ""
      ],
      "🇨🇻": [
        ""
      ],
      "🇨🇫": [
        ""
      ],
      "🇨🇱": [
        ""
      ],
      "🇨🇳": [
        ""
      ],
      "🇨🇴": [
        ""
      ],
      "🇨🇩": [
        ""
      ],
      "🇨🇬": [
        ""
      ],
      "🇨🇷": [
        ""
      ],
      "🇨🇺": [
        ""
      ],
      "🇨🇾": [
        ""
      ],
      "🇨🇿": [
        ""
      ],
      "🇩🇰": [
        ""
      ],
      "🇩🇯": [
        ""
      ],
      "🇩🇴": [
        ""
      ],
      "🇳🇱": [
        ""
      ],
      "🇹🇱": [
        ""
      ],
      "🇪🇨": [
        ""
      ],
      "🇪🇬": [
        ""
      ],
      "🇦🇪": [
        ""
      ],
      "🇬🇶": [
        ""
      ],
      "🇪🇷": [
        ""
      ],
      "🇪🇪": [
        ""
      ],
      "🇪🇹": [
        ""
      ],
      "🇫🇯": [
        ""
      ],
      "🇵🇭": [
        ""
      ],
      "🇫🇮": [
        ""
      ],
      "🇫🇷": [
        ""
      ],
      "🇬🇦": [
        ""
      ],
      "🇬🇲": [
        ""
      ],
      "🇬🇪": [
        ""
      ],
      "UAE Dirham": [
        ""
      ],
      "Afghani": [
        ""
      ],
      "Lek": [
        ""
      ],
      "Armenian Dram": [
        ""
      ],
      "Netherlands Antillean Guilder": [
        ""
      ],
      "Kwanza": [
        ""
      ],
      "Argentine Peso": [
        ""
      ],
      "Australian Dollar": [
        ""
      ],
      "Aruban Florin": [
        ""
      ],
      "Azerbaijan Manat": [
        ""
      ],
      "Convertible Mark": [
        ""
      ],
      "Barbados Dollar": [
        ""
      ],
      "Taka": [
        ""
      ],
      "Bulgarian Lev": [
        ""
      ],
      "Bahraini Dinar": [
        ""
      ],
      "Burundi Franc": [
        ""
      ],
      "Bermudian Dollar": [
        ""
      ],
      "Brunei Dollar": [
        ""
      ],
      "Boliviano": [
        ""
      ],
      "Mvdol": [
        ""
      ],
      "Brazilian Real": [
        ""
      ],
      "Bahamian Dollar": [
        ""
      ],
      "Ngultrum": [
        ""
      ],
      "Pula": [
        ""
      ],
      "Belarusian Ruble": [
        ""
      ],
      "Belize Dollar": [
        ""
      ],
      "Canadian Dollar": [
        ""
      ],
      "Congolese Franc": [
        ""
      ],
      "WIR Euro": [
        ""
      ],
      "Swiss Franc": [
        ""
      ],
      "WIR Franc": [
        ""
      ],
      "Unidad de Fomento": [
        ""
      ],
      "Chilean Peso": [
        ""
      ],
      "Yuan Renminbi": [
        ""
      ],
      "Colombian Peso": [
        ""
      ],
      "Unidad de Valor Real": [
        ""
      ],
      "Costa Rican Colon": [
        ""
      ],
      "Cuban Peso": [
        ""
      ],
      "Cabo Verde Escudo": [
        ""
      ],
      "Czech Koruna": [
        ""
      ],
      "Djibouti Franc": [
        ""
      ],
      "Danish Krone": [
        ""
      ],
      "Dominican Peso": [
        ""
      ],
      "Algerian Dinar": [
        ""
      ],
      "Egyptian Pound": [
        ""
      ],
      "Nakfa": [
        ""
      ],
      "Ethiopian Birr": [
        ""
      ],
      "Euro": [
        ""
      ],
      "Fiji Dollar": [
        ""
      ],
      "Falkland Islands Pound": [
        ""
      ],
      "Pound Sterling": [
        ""
      ],
      "Lari": [
        ""
      ],
      "Ghana Cedi": [
        ""
      ],
      "Gibraltar Pound": [
        ""
      ],
      "Dalasi": [
        ""
      ],
      "Guinean Franc": [
        ""
      ],
      "Quetzal": [
        ""
      ],
      "Guyana Dollar": [
        ""
      ],
      "Hong Kong Dollar": [
        ""
      ],
      "Lempira": [
        ""
      ],
      "Gourde": [
        ""
      ],
      "Forint": [
        ""
      ],
      "Rupiah": [
        ""
      ],
      "New Israeli Sheqel": [
        ""
      ],
      "Indian Rupee": [
        ""
      ],
      "Iraqi Dinar": [
        ""
      ],
      "Iranian Rial": [
        ""
      ],
      "Iceland Krona": [
        ""
      ],
      "Jamaican Dollar": [
        ""
      ],
      "Jordanian Dinar": [
        ""
      ],
      "Yen": [
        ""
      ],
      "Kenyan Shilling": [
        ""
      ],
      "Som": [
        ""
      ],
      "Riel": [
        ""
      ],
      "Comorian Franc ": [
        ""
      ],
      "North Korean Won": [
        ""
      ],
      "Won": [
        ""
      ],
      "Kuwaiti Dinar": [
        ""
      ],
      "Cayman Islands Dollar": [
        ""
      ],
      "Tenge": [
        ""
      ],
      "Lao Kip": [
        ""
      ],
      "Lebanese Pound": [
        ""
      ],
      "Sri Lanka Rupee": [
        ""
      ],
      "Liberian Dollar": [
        ""
      ],
      "Loti": [
        ""
      ],
      "Libyan Dinar": [
        ""
      ],
      "Moroccan Dirham": [
        ""
      ],
      "Moldovan Leu": [
        ""
      ],
      "Malagasy Ariary": [
        ""
      ],
      "Denar": [
        ""
      ],
      "Kyat": [
        ""
      ],
      "Tugrik": [
        ""
      ],
      "Pataca": [
        ""
      ],
      "Ouguiya": [
        ""
      ],
      "Mauritius Rupee": [
        ""
      ],
      "Rufiyaa": [
        ""
      ],
      "Malawi Kwacha": [
        ""
      ],
      "Mexican Peso": [
        ""
      ],
      "Mexican Unidad de Inversion (UDI)": [
        ""
      ],
      "Malaysian Ringgit": [
        ""
      ],
      "Mozambique Metical": [
        ""
      ],
      "Namibia Dollar": [
        ""
      ],
      "Naira": [
        ""
      ],
      "Cordoba Oro": [
        ""
      ],
      "Norwegian Krone": [
        ""
      ],
      "Nepalese Rupee": [
        ""
      ],
      "New Zealand Dollar": [
        ""
      ],
      "Rial Omani": [
        ""
      ],
      "Balboa": [
        ""
      ],
      "Sol": [
        ""
      ],
      "Kina": [
        ""
      ],
      "Philippine Peso": [
        ""
      ],
      "Pakistan Rupee": [
        ""
      ],
      "Zloty": [
        ""
      ],
      "Guarani": [
        ""
      ],
      "Qatari Rial": [
        ""
      ],
      "Romanian Leu": [
        ""
      ],
      "Serbian Dinar": [
        ""
      ],
      "Russian Ruble": [
        ""
      ],
      "Rwanda Franc": [
        ""
      ],
      "Saudi Riyal": [
        ""
      ],
      "Solomon Islands Dollar": [
        ""
      ],
      "Seychelles Rupee": [
        ""
      ],
      "Sudanese Pound": [
        ""
      ],
      "Swedish Krona": [
        ""
      ],
      "Singapore Dollar": [
        ""
      ],
      "Saint Helena Pound": [
        ""
      ],
      "Leone": [
        ""
      ],
      "Somali Shilling": [
        ""
      ],
      "Surinam Dollar": [
        ""
      ],
      "South Sudanese Pound": [
        ""
      ],
      "Dobra": [
        ""
      ],
      "El Salvador Colon": [
        ""
      ],
      "Syrian Pound": [
        ""
      ],
      "Lilangeni": [
        ""
      ],
      "Baht": [
        ""
      ],
      "Somoni": [
        ""
      ],
      "Turkmenistan New Manat": [
        ""
      ],
      "Tunisian Dinar": [
        ""
      ],
      "Pa’anga": [
        ""
      ],
      "Turkish Lira": [
        ""
      ],
      "Trinidad and Tobago Dollar": [
        ""
      ],
      "New Taiwan Dollar": [
        ""
      ],
      "Tanzanian Shilling": [
        ""
      ],
      "Hryvnia": [
        ""
      ],
      "Uganda Shilling": [
        ""
      ],
      "US Dollar": [
        ""
      ],
      "US Dollar (Next day)": [
        ""
      ],
      "Uruguay Peso en Unidades Indexadas (UI)": [
        ""
      ],
      "Peso Uruguayo": [
        ""
      ],
      "Unidad Previsional": [
        ""
      ],
      "Uzbekistan Sum": [
        ""
      ],
      "Bolívar Soberano": [
        ""
      ],
      "Dong": [
        ""
      ],
      "Vatu": [
        ""
      ],
      "Tala": [
        ""
      ],
      "CFA Franc BEAC": [
        ""
      ],
      "East Caribbean Dollar": [
        ""
      ],
      "CFA Franc BCEAO": [
        ""
      ],
      "CFP Franc": [
        ""
      ],
      "Sucre": [
        ""
      ],
      "Yemeni Rial": [
        ""
      ],
      "Rand": [
        ""
      ],
      "Zambian Kwacha": [
        ""
      ],
      "Zimbabwe Gold": [
        ""
      ],
      "Afar": [
        ""
      ],
      "Abkhazian": [
        ""
      ],
      "Achinese": [
        ""
      ],
      "Acoli": [
        ""
      ],
      "Adangme": [
        ""
      ],
      "Adyghe": [
        ""
      ],
      "Avestan": [
        ""
      ],
      "Tunisian Arabic": [
        ""
      ],
      "Afrikaans": [
        ""
      ],
      "Afrihili": [
        ""
      ],
      "Aghem": [
        ""
      ],
      "Ainu": [
        ""
      ],
      "Akan": [
        ""
      ],
      "Akkadian": [
        ""
      ],
      "Alabama": [
        ""
      ],
      "Aleut": [
        ""
      ],
      "Gheg Albanian": [
        ""
      ],
      "Southern Altai": [
        ""
      ],
      "Amarik": [
        ""
      ],
      "Aragonese": [
        ""
      ],
      "Old English": [
        ""
      ],
      "Angika": [
        ""
      ],
      "Modern Standard Arabic": [
        ""
      ],
      "Arabik": [
        ""
      ],
      "Aramaic": [
        ""
      ],
      "Mapuche": [
        ""
      ],
      "Araona": [
        ""
      ],
      "Arapaho": [
        ""
      ],
      "Algerian Arabic": [
        ""
      ],
      "Arawak": [
        ""
      ],
      "Moroccan Arabic": [
        ""
      ],
      "Egyptian Arabic": [
        ""
      ],
      "Asu": [
        ""
      ],
      "Assamese": [
        ""
      ],
      "American Sign Language": [
        ""
      ],
      "Asturian": [
        ""
      ],
      "Avaric": [
        ""
      ],
      "Kotava": [
        ""
      ],
      "Awadhi": [
        ""
      ],
      "Aymara": [
        ""
      ],
      "South Azerbaijani": [
        ""
      ],
      "Bashkir": [
        ""
      ],
      "Baluchi": [
        ""
      ],
      "Balinese": [
        ""
      ],
      "Bavarian": [
        ""
      ],
      "Basaa": [
        ""
      ],
      "Bamun": [
        ""
      ],
      "Batak Toba": [
        ""
      ],
      "Ghomala": [
        ""
      ],
      "Belarus kasa": [
        ""
      ],
      "Beja": [
        ""
      ],
      "Bemba": [
        ""
      ],
      "Betawi": [
        ""
      ],
      "Bena": [
        ""
      ],
      "Bafut": [
        ""
      ],
      "Badaga": [
        ""
      ],
      "Bɔlgeria kasa": [
        ""
      ],
      "Bhojpuri": [
        ""
      ],
      "Bislama": [
        ""
      ],
      "Bikol": [
        ""
      ],
      "Bini": [
        ""
      ],
      "Banjar": [
        ""
      ],
      "Kom": [
        ""
      ],
      "Siksika": [
        ""
      ],
      "Bambara": [
        ""
      ],
      "Bengali kasa": [
        ""
      ],
      "Tibetan": [
        ""
      ],
      "Bishnupriya": [
        ""
      ],
      "Bakhtiari": [
        ""
      ],
      "Braj": [
        ""
      ],
      "Breton": [
        ""
      ],
      "Brahui": [
        ""
      ],
      "Bodo": [
        ""
      ],
      "Akoose": [
        ""
      ],
      "Buriat": [
        ""
      ],
      "Buginese": [
        ""
      ],
      "Bulu": [
        ""
      ],
      "Blin": [
        ""
      ],
      "Medumba": [
        ""
      ],
      "Catalan": [
        ""
      ],
      "Caddo": [
        ""
      ],
      "Carib": [
        ""
      ],
      "Cayuga": [
        ""
      ],
      "Atsam": [
        ""
      ],
      "Cebuano": [
        ""
      ],
      "Chechen": [
        ""
      ],
      "Chiga": [
        ""
      ],
      "Chibcha": [
        ""
      ],
      "Chamorro": [
        ""
      ],
      "Chagatai": [
        ""
      ],
      "Chuukese": [
        ""
      ],
      "Mari": [
        ""
      ],
      "Chinook Jargon": [
        ""
      ],
      "Choctaw": [
        ""
      ],
      "Chipewyan": [
        ""
      ],
      "Cherokee": [
        ""
      ],
      "Cheyenne": [
        ""
      ],
      "Central Kurdish": [
        ""
      ],
      "Corsican": [
        ""
      ],
      "Coptic": [
        ""
      ],
      "Capiznon": [
        ""
      ],
      "Cree": [
        ""
      ],
      "Crimean Turkish": [
        ""
      ],
      "Kashubian": [
        ""
      ],
      "Kyɛk kasa": [
        ""
      ],
      "Church Slavic": [
        ""
      ],
      "Chuvash": [
        ""
      ],
      "Welsh": [
        ""
      ],
      "Dakota": [
        ""
      ],
      "Dargwa": [
        ""
      ],
      "Taita": [
        ""
      ],
      "Austrian German": [
        ""
      ],
      "Swiss High German": [
        ""
      ],
      "Gyaaman": [
        ""
      ],
      "Delaware": [
        ""
      ],
      "Slave": [
        ""
      ],
      "Dogrib": [
        ""
      ],
      "Dinka": [
        ""
      ],
      "Zarma": [
        ""
      ],
      "Dogri": [
        ""
      ],
      "Lower Sorbian": [
        ""
      ],
      "Central Dusun": [
        ""
      ],
      "Duala": [
        ""
      ],
      "Middle Dutch": [
        ""
      ],
      "Divehi": [
        ""
      ],
      "Jola-Fonyi": [
        ""
      ],
      "Dyula": [
        ""
      ],
      "Dzongkha": [
        ""
      ],
      "Dazaga": [
        ""
      ],
      "Embu": [
        ""
      ],
      "Ewe": [
        ""
      ],
      "Efik": [
        ""
      ],
      "Emilian": [
        ""
      ],
      "Ancient Egyptian": [
        ""
      ],
      "Ekajuk": [
        ""
      ],
      "Greek kasa": [
        ""
      ],
      "Elamite": [
        ""
      ],
      "Australian English": [
        ""
      ],
      "Borɔfo": [
        ""
      ],
      "Canadian English": [
        ""
      ],
      "British English": [
        ""
      ],
      "Middle English": [
        ""
      ],
      "American English": [
        ""
      ],
      "Esperanto": [
        ""
      ],
      "Latin American Spanish": [
        ""
      ],
      "European Spanish": [
        ""
      ],
      "Mexican Spanish": [
        ""
      ],
      "Spain kasa": [
        ""
      ],
      "Central Yupik": [
        ""
      ],
      "Basque": [
        ""
      ],
      "Ewondo": [
        ""
      ],
      "Extremaduran": [
        ""
      ],
      "Fang": [
        ""
      ],
      "Pɛɛhyia kasa": [
        ""
      ],
      "Fanti": [
        ""
      ],
      "Fulah": [
        ""
      ],
      "Tornedalen Finnish": [
        ""
      ],
      "Faroese": [
        ""
      ],
      "Fon": [
        ""
      ],
      "Canadian French": [
        ""
      ],
      "Cajun French": [
        ""
      ],
      "Swiss French": [
        ""
      ],
      "Frɛnkye": [
        ""
      ],
      "Middle French": [
        ""
      ],
      "Old French": [
        ""
      ],
      "Arpitan": [
        ""
      ],
      "Northern Frisian": [
        ""
      ],
      "Eastern Frisian": [
        ""
      ],
      "Friulian": [
        ""
      ],
      "Western Frisian": [
        ""
      ],
      "Ga": [
        ""
      ],
      "Gagauz": [
        ""
      ],
      "Gan Chinese": [
        ""
      ],
      "Gayo": [
        ""
      ],
      "Gbaya": [
        ""
      ],
      "Zoroastrian Dari": [
        ""
      ],
      "Scottish Gaelic": [
        ""
      ],
      "Geez": [
        ""
      ],
      "Gilbertese": [
        ""
      ],
      "Galician": [
        ""
      ],
      "Gilaki": [
        ""
      ],
      "Middle High German": [
        ""
      ],
      "Old High German": [
        ""
      ],
      "Goan Konkani": [
        ""
      ],
      "Gondi": [
        ""
      ],
      "Gorontalo": [
        ""
      ],
      "Gothic": [
        ""
      ],
      "Grebo": [
        ""
      ],
      "Ancient Greek": [
        ""
      ],
      "Swiss German": [
        ""
      ],
      "Wayuu": [
        ""
      ],
      "Gujarati": [
        ""
      ],
      "Frafra": [
        ""
      ],
      "Gusii": [
        ""
      ],
      "Manx": [
        ""
      ],
      "Gwichʼin": [
        ""
      ],
      "Hausa": [
        ""
      ],
      "Haida": [
        ""
      ],
      "Hakka Chinese": [
        ""
      ],
      "Hawaiian": [
        ""
      ],
      "Hebrew": [
        ""
      ],
      "Fiji Hindi": [
        ""
      ],
      "Hindi": [
        ""
      ],
      "Hiligaynon": [
        ""
      ],
      "Hittite": [
        ""
      ],
      "Hmong": [
        ""
      ],
      "Hiri Motu": [
        ""
      ],
      "Upper Sorbian": [
        ""
      ],
      "Xiang Chinese": [
        ""
      ],
      "Hangri kasa": [
        ""
      ],
      "Hupa": [
        ""
      ],
      "Herero": [
        ""
      ],
      "Interlingua": [
        ""
      ],
      "Iban": [
        ""
      ],
      "Ibibio": [
        ""
      ],
      "Indonihyia kasa": [
        ""
      ],
      "Interlingue": [
        ""
      ],
      "Igbo": [
        ""
      ],
      "Sichuan Yi": [
        ""
      ],
      "Inupiaq": [
        ""
      ],
      "Iloko": [
        ""
      ],
      "Ingush": [
        ""
      ],
      "Ido": [
        ""
      ],
      "Icelandic": [
        ""
      ],
      "Italy kasa": [
        ""
      ],
      "Inuktitut": [
        ""
      ],
      "Ingrian": [
        ""
      ],
      "Gyapan kasa": [
        ""
      ],
      "Jamaican Creole English": [
        ""
      ],
      "Lojban": [
        ""
      ],
      "Ngomba": [
        ""
      ],
      "Machame": [
        ""
      ],
      "Judeo-Persian": [
        ""
      ],
      "Judeo-Arabic": [
        ""
      ],
      "Jutish": [
        ""
      ],
      "Gyabanis kasa": [
        ""
      ],
      "Kara-Kalpak": [
        ""
      ],
      "Kabyle": [
        ""
      ],
      "Kachin": [
        ""
      ],
      "Jju": [
        ""
      ],
      "Kamba": [
        ""
      ],
      "Kawi": [
        ""
      ],
      "Kabardian": [
        ""
      ],
      "Kanembu": [
        ""
      ],
      "Tyap": [
        ""
      ],
      "Makonde": [
        ""
      ],
      "Kabuverdianu": [
        ""
      ],
      "Kenyang": [
        ""
      ],
      "Koro": [
        ""
      ],
      "Kongo": [
        ""
      ],
      "Kaingang": [
        ""
      ],
      "Khasi": [
        ""
      ],
      "Khotanese": [
        ""
      ],
      "Koyra Chiini": [
        ""
      ],
      "Khowar": [
        ""
      ],
      "Kikuyu": [
        ""
      ],
      "Kirmanjki": [
        ""
      ],
      "Kuanyama": [
        ""
      ],
      "Kako": [
        ""
      ],
      "Kazakh": [
        ""
      ],
      "Kalaallisut": [
        ""
      ],
      "Kalenjin": [
        ""
      ],
      "Kimbundu": [
        ""
      ],
      "Kambodia kasa": [
        ""
      ],
      "Kannada": [
        ""
      ],
      "Komi-Permyak": [
        ""
      ],
      "Konkani": [
        ""
      ],
      "Korea kasa": [
        ""
      ],
      "Kosraean": [
        ""
      ],
      "Kpelle": [
        ""
      ],
      "Karachay-Balkar": [
        ""
      ],
      "Krio": [
        ""
      ],
      "Kinaray-a": [
        ""
      ],
      "Kanuri": [
        ""
      ],
      "Karelian": [
        ""
      ],
      "Kurukh": [
        ""
      ],
      "Shambala": [
        ""
      ],
      "Bafia": [
        ""
      ],
      "Colognian": [
        ""
      ],
      "Kashmiri": [
        ""
      ],
      "Kurdish": [
        ""
      ],
      "Kumyk": [
        ""
      ],
      "Kutenai": [
        ""
      ],
      "Komi": [
        ""
      ],
      "Cornish": [
        ""
      ],
      "Ladino": [
        ""
      ],
      "Langi": [
        ""
      ],
      "Lahnda": [
        ""
      ],
      "Latin": [
        ""
      ],
      "Lamba": [
        ""
      ],
      "Luxembourgish": [
        ""
      ],
      "Lezghian": [
        ""
      ],
      "Lingua Franca Nova": [
        ""
      ],
      "Ganda": [
        ""
      ],
      "Ligurian": [
        ""
      ],
      "Limburgish": [
        ""
      ],
      "Livonian": [
        ""
      ],
      "Lakota": [
        ""
      ],
      "Lombard": [
        ""
      ],
      "Lingala": [
        ""
      ],
      "Lao": [
        ""
      ],
      "Mongo": [
        ""
      ],
      "Lozi": [
        ""
      ],
      "Latgalian": [
        ""
      ],
      "Luba-Lulua": [
        ""
      ],
      "Luiseno": [
        ""
      ],
      "Luba-Katanga": [
        ""
      ],
      "Lunda": [
        ""
      ],
      "Luo": [
        ""
      ],
      "Mizo": [
        ""
      ],
      "Luyia": [
        ""
      ],
      "Literary Chinese": [
        ""
      ],
      "Laz": [
        ""
      ],
      "Madurese": [
        ""
      ],
      "Mafa": [
        ""
      ],
      "Magahi": [
        ""
      ],
      "Maithili": [
        ""
      ],
      "Makasar": [
        ""
      ],
      "Mandingo": [
        ""
      ],
      "Masai": [
        ""
      ],
      "Maba": [
        ""
      ],
      "Moksha": [
        ""
      ],
      "Mandar": [
        ""
      ],
      "Mende": [
        ""
      ],
      "Meru": [
        ""
      ],
      "Morisyen": [
        ""
      ],
      "Middle Irish": [
        ""
      ],
      "Makhuwa-Meetto": [
        ""
      ],
      "Metaʼ": [
        ""
      ],
      "Micmac": [
        ""
      ],
      "Maori": [
        ""
      ],
      "Minangkabau": [
        ""
      ],
      "Malayalam": [
        ""
      ],
      "Manchu": [
        ""
      ],
      "Manipuri": [
        ""
      ],
      "Mohawk": [
        ""
      ],
      "Mossi": [
        ""
      ],
      "Western Mari": [
        ""
      ],
      "Marathi": [
        ""
      ],
      "Malay kasa": [
        ""
      ],
      "Mundang": [
        ""
      ],
      "Multiple Languages": [
        ""
      ],
      "Creek": [
        ""
      ],
      "Mirandese": [
        ""
      ],
      "Marwari": [
        ""
      ],
      "Mentawai": [
        ""
      ],
      "Bɛɛmis kasa": [
        ""
      ],
      "Myene": [
        ""
      ],
      "Erzya": [
        ""
      ],
      "Mazanderani": [
        ""
      ],
      "Min Nan Chinese": [
        ""
      ],
      "Neapolitan": [
        ""
      ],
      "Nama": [
        ""
      ],
      "Norwegian Bokmål": [
        ""
      ],
      "North Ndebele": [
        ""
      ],
      "Low German": [
        ""
      ],
      "Nɛpal kasa": [
        ""
      ],
      "Newari": [
        ""
      ],
      "Ndonga": [
        ""
      ],
      "Nias": [
        ""
      ],
      "Niuean": [
        ""
      ],
      "Ao Naga": [
        ""
      ],
      "Flemish": [
        ""
      ],
      "Dɛɛkye": [
        ""
      ],
      "Kwasio": [
        ""
      ],
      "Ngiemboon": [
        ""
      ],
      "Norwegian Nynorsk": [
        ""
      ],
      "Nogai": [
        ""
      ],
      "Old Norse": [
        ""
      ],
      "Novial": [
        ""
      ],
      "NʼKo": [
        ""
      ],
      "South Ndebele": [
        ""
      ],
      "Northern Sotho": [
        ""
      ],
      "Nuer": [
        ""
      ],
      "Navajo": [
        ""
      ],
      "Classical Newari": [
        ""
      ],
      "Nyamwezi": [
        ""
      ],
      "Nyankole": [
        ""
      ],
      "Nyanja": [
        ""
      ],
      "Nyoro": [
        ""
      ],
      "Nzima": [
        ""
      ],
      "Occitan": [
        ""
      ],
      "Ojibwa": [
        ""
      ],
      "Oromo": [
        ""
      ],
      "Oriya": [
        ""
      ],
      "Osage": [
        ""
      ],
      "Ossetic": [
        ""
      ],
      "Ottoman Turkish": [
        ""
      ],
      "Pangasinan": [
        ""
      ],
      "Pahlavi": [
        ""
      ],
      "Pampanga": [
        ""
      ],
      "Papiamento": [
        ""
      ],
      "Pungyabi kasa": [
        ""
      ],
      "Picard": [
        ""
      ],
      "Pennsylvania German": [
        ""
      ],
      "Plautdietsch": [
        ""
      ],
      "Old Persian": [
        ""
      ],
      "Palatine German": [
        ""
      ],
      "Phoenician": [
        ""
      ],
      "Pali": [
        ""
      ],
      "Pɔland kasa": [
        ""
      ],
      "Piedmontese": [
        ""
      ],
      "Pontic": [
        ""
      ],
      "Pohnpeian": [
        ""
      ],
      "Prussian": [
        ""
      ],
      "Old Provençal": [
        ""
      ],
      "Pashto": [
        ""
      ],
      "Brazilian Portuguese": [
        ""
      ],
      "Pɔɔtugal kasa": [
        ""
      ],
      "European Portuguese": [
        ""
      ],
      "Kʼicheʼ": [
        ""
      ],
      "Chimborazo Highland Quichua": [
        ""
      ],
      "Quechua": [
        ""
      ],
      "Rajasthani": [
        ""
      ],
      "Rapanui": [
        ""
      ],
      "Rarotongan": [
        ""
      ],
      "Romagnol": [
        ""
      ],
      "Riffian": [
        ""
      ],
      "Romansh": [
        ""
      ],
      "Rundi": [
        ""
      ],
      "Rombo": [
        ""
      ],
      "Moldavian": [
        ""
      ],
      "Romany": [
        ""
      ],
      "Root": [
        ""
      ],
      "Romenia kasa": [
        ""
      ],
      "Rotuman": [
        ""
      ],
      "Rusyn": [
        ""
      ],
      "Roviana": [
        ""
      ],
      "Aromanian": [
        ""
      ],
      "Rahyia kasa": [
        ""
      ],
      "Rwa": [
        ""
      ],
      "Rewanda kasa": [
        ""
      ],
      "Sandawe": [
        ""
      ],
      "Sakha": [
        ""
      ],
      "Samaritan Aramaic": [
        ""
      ],
      "Samburu": [
        ""
      ],
      "Sanskrit": [
        ""
      ],
      "Sasak": [
        ""
      ],
      "Santali": [
        ""
      ],
      "Saurashtra": [
        ""
      ],
      "Ngambay": [
        ""
      ],
      "Sangu": [
        ""
      ],
      "Sicilian": [
        ""
      ],
      "Scots": [
        ""
      ],
      "Sardinian": [
        ""
      ],
      "Sassarese Sardinian": [
        ""
      ],
      "Sindhi": [
        ""
      ],
      "Seneca": [
        ""
      ],
      "Sena": [
        ""
      ],
      "Seri": [
        ""
      ],
      "Selkup": [
        ""
      ],
      "Northern Sami": [
        ""
      ],
      "Koyraboro Senni": [
        ""
      ],
      "Old Irish": [
        ""
      ],
      "Sango": [
        ""
      ],
      "Samogitian": [
        ""
      ],
      "Tachelhit": [
        ""
      ],
      "Shan": [
        ""
      ],
      "Serbo-Croatian": [
        ""
      ],
      "Chadian Arabic": [
        ""
      ],
      "Sidamo": [
        ""
      ],
      "Sinhala": [
        ""
      ],
      "Slovak": [
        ""
      ],
      "Lower Silesian": [
        ""
      ],
      "Selayar": [
        ""
      ],
      "Southern Sami": [
        ""
      ],
      "Lule Sami": [
        ""
      ],
      "Inari Sami": [
        ""
      ],
      "Skolt Sami": [
        ""
      ],
      "Soninke": [
        ""
      ],
      "Shona": [
        ""
      ],
      "Sogdien": [
        ""
      ],
      "Somalia kasa": [
        ""
      ],
      "Sranan Tongo": [
        ""
      ],
      "Serer": [
        ""
      ],
      "Swati": [
        ""
      ],
      "Saho": [
        ""
      ],
      "Saterland Frisian": [
        ""
      ],
      "Southern Sotho": [
        ""
      ],
      "Sukuma": [
        ""
      ],
      "Susu": [
        ""
      ],
      "Sundanese": [
        ""
      ],
      "Sumerian": [
        ""
      ],
      "Sweden kasa": [
        ""
      ],
      "Comorian": [
        ""
      ],
      "Congo Swahili": [
        ""
      ],
      "Swahili": [
        ""
      ],
      "Classical Syriac": [
        ""
      ],
      "Syriac": [
        ""
      ],
      "Silesian": [
        ""
      ],
      "Tamil kasa": [
        ""
      ],
      "Tulu": [
        ""
      ],
      "Timne": [
        ""
      ],
      "Teso": [
        ""
      ],
      "Tereno": [
        ""
      ],
      "Telugu": [
        ""
      ],
      "Tetum": [
        ""
      ],
      "Taeland kasa": [
        ""
      ],
      "Tigre": [
        ""
      ],
      "Tigrinya": [
        ""
      ],
      "Tiv": [
        ""
      ],
      "Tokelau": [
        ""
      ],
      "Tsakhur": [
        ""
      ],
      "Turkmen": [
        ""
      ],
      "Klingon": [
        ""
      ],
      "Tlingit": [
        ""
      ],
      "Tagalog": [
        ""
      ],
      "Talysh": [
        ""
      ],
      "Tamashek": [
        ""
      ],
      "Tswana": [
        ""
      ],
      "Nyasa Tonga": [
        ""
      ],
      "Tok Pisin": [
        ""
      ],
      "Tɛɛki kasa": [
        ""
      ],
      "Turoyo": [
        ""
      ],
      "Taroko": [
        ""
      ],
      "Tsakonian": [
        ""
      ],
      "Tsimshian": [
        ""
      ],
      "Tsonga": [
        ""
      ],
      "Tatar": [
        ""
      ],
      "Muslim Tat": [
        ""
      ],
      "Tumbuka": [
        ""
      ],
      "Tasawaq": [
        ""
      ],
      "Twi": [
        ""
      ],
      "Tahitian": [
        ""
      ],
      "Tuvinian": [
        ""
      ],
      "Central Atlas Tamazight": [
        ""
      ],
      "Udmurt": [
        ""
      ],
      "Ugaritic": [
        ""
      ],
      "Uyghur": [
        ""
      ],
      "Ukren kasa": [
        ""
      ],
      "Umbundu": [
        ""
      ],
      "Unknown Language": [
        ""
      ],
      "Urdu kasa": [
        ""
      ],
      "Uzbek": [
        ""
      ],
      "Vai": [
        ""
      ],
      "Venetian": [
        ""
      ],
      "Veps": [
        ""
      ],
      "Venda": [
        ""
      ],
      "Viɛtnam kasa": [
        ""
      ],
      "West Flemish": [
        ""
      ],
      "Main-Franconian": [
        ""
      ],
      "Votic": [
        ""
      ],
      "Volapük": [
        ""
      ],
      "Võro": [
        ""
      ],
      "Vunjo": [
        ""
      ],
      "Walser": [
        ""
      ],
      "Wolaytta": [
        ""
      ],
      "Waray": [
        ""
      ],
      "Washo": [
        ""
      ],
      "Walloon": [
        ""
      ],
      "Warlpiri": [
        ""
      ],
      "Wolof": [
        ""
      ],
      "Wu Chinese": [
        ""
      ],
      "Kalmyk": [
        ""
      ],
      "Xhosa": [
        ""
      ],
      "Mingrelian": [
        ""
      ],
      "Soga": [
        ""
      ],
      "Yao": [
        ""
      ],
      "Yapese": [
        ""
      ],
      "Yangben": [
        ""
      ],
      "Yemba": [
        ""
      ],
      "Yiddish": [
        ""
      ],
      "Yoruba": [
        ""
      ],
      "Nheengatu": [
        ""
      ],
      "Cantonese": [
        ""
      ],
      "Zapotec": [
        ""
      ],
      "Zhuang": [
        ""
      ],
      "Blissymbols": [
        ""
      ],
      "Zeelandic": [
        ""
      ],
      "Zenaga": [
        ""
      ],
      "Standard Moroccan Tamazight": [
        ""
      ],
      "Simplified Chinese": [
        ""
      ],
      "Traditional Chinese": [
        ""
      ],
      "Kyaena kasa": [
        ""
      ],
      "Zuni": [
        ""
      ],
      "Zulu": [
        ""
      ],
      "No linguistic content": [
        ""
      ],
      "Zaza": [
        ""
      ],
      "Could not load user attention request": [
        ""
      ],
      "No notification left": [
        ""
      ],
      "Ignore": [
        ""
      ],
      "Invalid dev experiment URI.": [
        ""
      ],
      "URI: %1$s": [
        ""
      ],
      "Experiment": [
        ""
      ],
      "Insert lost denomination": [
        ""
      ],
      "Apply": [
        ""
      ],
      "Pending refresh": [
        ""
      ],
      "Could not load the transfer payment status": [
        ""
      ],
      "Could not load the status of the order template": [
        ""
      ],
      "required": [
        ""
      ],
      "Review order": [
        ""
      ],
      "Missing URI": [
        ""
      ],
      "Check the link.": [
        ""
      ],
      "Could not parse the recovery URI": [
        ""
      ],
      "Try another URI": [
        ""
      ],
      "Import backup, show info": [
        ""
      ],
      "Add new exchange": [
        ""
      ],
      "Add exchange for %1$s": [
        ""
      ],
      "Enter the URL of an exchange you trust.": [
        ""
      ],
      "An exchange has been found! Review the information and click next": [
        ""
      ],
      "This exchange is already in your list.": [
        ""
      ],
      "Only exchange accessible through \"http\" and \"https\" are allowed.": [
        ""
      ],
      "This exchange protocol version is not supported: \" %1$s\".": [
        ""
      ],
      "This exchange currency \"%1$s\" doesn&apos;t match the expected currency %2$s.": [
        ""
      ],
      "No exchange found in that URL.": [
        ""
      ],
      "loading": [
        ""
      ],
      "Version": [
        ""
      ],
      "Can't use the URL: \"%1$s\"": [
        ""
      ],
      "Exchange URL": [
        ""
      ],
      "Add new contact": [
        ""
      ],
      "Enter the details of a contact you trust.": [
        ""
      ],
      "Contact info valid.": [
        ""
      ],
      "This contact is already in your list.": [
        ""
      ],
      "Review contact": [
        ""
      ],
      "Contact alias": [
        ""
      ],
      "Alias type": [
        ""
      ],
      "Mailbox service": [
        ""
      ],
      "Mailbox address": [
        ""
      ],
      "Contact source": [
        ""
      ],
      "Could not load the list of contacts": [
        ""
      ],
      "Source": [
        ""
      ],
      "Send Cash": [
        ""
      ],
      "Request Cash": [
        ""
      ],
      "No contacts found.": [
        ""
      ],
      "Could not bank choices for account": [
        ""
      ],
      "Could not load the list of messages": [
        ""
      ],
      "Unexpected error when trying to initialize mailbox: %1$s": [
        ""
      ],
      "Unexpected error when trying to delete message: %1$s": [
        ""
      ],
      "Unexpected error when trying to fetch messages: %1$s": [
        ""
      ],
      "Date": [
        ""
      ],
      "You may copy your mailbox URI from below and register it at %1$s with your alias(es) to make it easier for others to find you.": [
        ""
      ],
      "copy uri": [
        ""
      ],
      "hide uri": [
        ""
      ],
      "show uri": [
        ""
      ],
      "Fetch messages": [
        ""
      ],
      "Mailbox not initialized": [
        ""
      ],
      "Initialize mailbox": [
        ""
      ],
      "No messages yet.": [
        ""
      ],
      "Taler URI handler": [
        ""
      ],
      "Could not found a handler for the Taler URI": [
        ""
      ],
      "The uri read in the path parameter is not valid: \"%1$s\"": [
        ""
      ],
      "Digital cash payment": [
        ""
      ],
      "Digital cash refund": [
        ""
      ],
      "Digital cash withdrawal": [
        ""
      ],
      "Digital cash deposit": [
        ""
      ],
      "Digital cash invoice": [
        ""
      ],
      "Digital cash transfer": [
        ""
      ],
      "Digital cash recovery": [
        ""
      ],
      "Development experiment": [
        ""
      ],
      "Return to wallet": [
        ""
      ],
      "Network is offline": [
        ""
      ],
      "Diagnostics timed out. Could not talk to the wallet backend.": [
        ""
      ],
      "Problems detected:": [
        ""
      ],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).": [
        ""
      ],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.": [
        ""
      ],
      "here": [
        ""
      ],
      "Running diagnostics": [
        ""
      ],
      "Edit": [
        ""
      ],
      "List of products": [
        ""
      ],
      "free": [
        ""
      ],
      "Could not get provider information": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "nl",
  "completeness": 0
};

strings['ja'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "ja"
      },
      "Balance": [
        "残高"
      ],
      "Backup": [
        "バックアップ"
      ],
      "Notifications": [
        ""
      ],
      "QR Reader and Taler URI": [
        ""
      ],
      "Settings": [
        "設定"
      ],
      "Dev tools": [
        ""
      ],
      "Contacts": [
        ""
      ],
      "Mailbox": [
        ""
      ],
      "Search transactions": [
        ""
      ],
      "%1$s": [
        ""
      ],
      "KYC requirement": [
        ""
      ],
      "PENDING OPERATIONS": [
        ""
      ],
      "Error was thrown trying to: \"%1$s\"": [
        ""
      ],
      "Could not open the wallet database. Firefox is known to run into this problem under \"permanent private mode\".": [
        ""
      ],
      "Could not open the wallet database.": [
        ""
      ],
      "Unexpected request error, code: %1$s": [
        ""
      ],
      "more info": [
        ""
      ],
      "wrap text": [
        ""
      ],
      "copy content": [
        ""
      ],
      "less info": [
        ""
      ],
      "Retry operation": [
        ""
      ],
      "Loading": [
        ""
      ],
      "Could not load backup providers": [
        ""
      ],
      "No backup providers configured": [
        ""
      ],
      "Add provider": [
        ""
      ],
      "Sync all backups": [
        ""
      ],
      "Sync now": [
        ""
      ],
      "Last synced": [
        ""
      ],
      "Not synced": [
        ""
      ],
      "Expires in": [
        ""
      ],
      "show less": [
        ""
      ],
      "show more": [
        ""
      ],
      "There was an error loading the provider detail for &quot;%1$s&quot;": [
        ""
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [
        ""
      ],
      "See providers": [
        ""
      ],
      "Last backup": [
        ""
      ],
      "Back up": [
        ""
      ],
      "Provider fee": [
        ""
      ],
      "per year": [
        ""
      ],
      "Extend": [
        ""
      ],
      "terms has changed, extending the service will imply accepting the new terms of service": [
        ""
      ],
      "old": [
        ""
      ],
      "new": [
        ""
      ],
      "fee": [
        ""
      ],
      "storage": [
        ""
      ],
      "Remove provider": [
        ""
      ],
      "Pay": [
        ""
      ],
      "Withdraw": [
        "撤退"
      ],
      "This provider has reported an error": [
        ""
      ],
      "There is conflict with another backup from &quot;%1$s&quot;": [
        ""
      ],
      "Backup is not readable": [
        ""
      ],
      "Unknown backup problem: %1$s": [
        ""
      ],
      "service paid": [
        ""
      ],
      "Backup valid until": [
        ""
      ],
      "Cancel": [
        ""
      ],
      "Open pay page": [
        ""
      ],
      "Open refund page": [
        ""
      ],
      "Open withdraw page": [
        ""
      ],
      "Your wallet is empty.": [
        ""
      ],
      "Get digital cash": [
        ""
      ],
      "Try the demo bank and withdraw test money.": [
        ""
      ],
      "Could not load the balance": [
        ""
      ],
      "Receive": [
        ""
      ],
      "Send %1$s": [
        ""
      ],
      "This page has a withdrawal action.": [
        ""
      ],
      "This page has pay action.": [
        ""
      ],
      "This page has a refund action.": [
        ""
      ],
      "This page has a add exchange action.": [
        ""
      ],
      "Open add exchange page": [
        ""
      ],
      "This page has an add contact action.": [
        ""
      ],
      "Open add contact page": [
        ""
      ],
      "Taler Action": [
        ""
      ],
      "This page has a malformed taler uri.": [
        ""
      ],
      "Dismiss": [
        ""
      ],
      "this popup is being closed and you are being redirected to %1$s": [
        ""
      ],
      "Could not load purchase proposal details": [
        ""
      ],
      "Order Id": [
        ""
      ],
      "Summary": [
        ""
      ],
      "Amount": [
        ""
      ],
      "Choices": [
        ""
      ],
      "This purchase have %1$s choices": [
        ""
      ],
      "Merchant name": [
        ""
      ],
      "Merchant jurisdiction": [
        ""
      ],
      "Merchant address": [
        ""
      ],
      "Merchant logo": [
        ""
      ],
      "Merchant website": [
        ""
      ],
      "Merchant email": [
        ""
      ],
      "Merchant public key": [
        ""
      ],
      "Created at": [
        ""
      ],
      "Refund deadline": [
        ""
      ],
      "Auto refund": [
        ""
      ],
      "Pay deadline": [
        ""
      ],
      "Fulfillment URL": [
        ""
      ],
      "Fulfillment message": [
        ""
      ],
      "Max deposit fee": [
        ""
      ],
      "Exchanges": [
        ""
      ],
      "Could not load the status of the term of service": [
        ""
      ],
      "Select one option": [
        ""
      ],
      "Show terms of service": [
        ""
      ],
      "I accept the exchange terms of service": [
        ""
      ],
      "Review exchange terms of service": [
        ""
      ],
      "The exchange doesn't have a terms of service.": [
        ""
      ],
      "Skip it for now.": [
        ""
      ],
      "Format": [
        ""
      ],
      "Language": [
        ""
      ],
      "The exchange replied with a empty terms of service": [
        ""
      ],
      "No terms of service. The exchange replied with a empty document": [
        ""
      ],
      "No terms of service. The exchange replied with a empty text": [
        ""
      ],
      "Download Terms of Service": [
        ""
      ],
      "Hide terms of service": [
        ""
      ],
      "Amount is not valid": [
        ""
      ],
      "Bank account": [
        ""
      ],
      "Bitcoin address": [
        ""
      ],
      "BIC/IBAN": [
        ""
      ],
      "IBAN": [
        ""
      ],
      "Bitcoin transfer details": [
        ""
      ],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.": [
        ""
      ],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts": [
        ""
      ],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC": [
        ""
      ],
      "Ethereum transfer details": [
        ""
      ],
      "You need to wire to the service provider account.": [
        ""
      ],
      "Make sure the amount show %1$s ETH": [
        ""
      ],
      "Bank transfer details": [
        ""
      ],
      "Account": [
        ""
      ],
      "Bank host": [
        ""
      ],
      "BIC": [
        ""
      ],
      "Exchange": [
        ""
      ],
      "Reserve Pub": [
        ""
      ],
      "Step 1:": [
        ""
      ],
      "Copy this code and paste it into the subject/purpose field in your banking app or bank website": [
        ""
      ],
      "Subject": [
        ""
      ],
      "Step 2:": [
        ""
      ],
      "If you don't already have it in your banking favourites list, then copy and paste this IBAN and the name into the receiver fields in your banking app or website": [
        ""
      ],
      "Receiver name": [
        ""
      ],
      "Receiver postal code": [
        ""
      ],
      "Receiver town": [
        ""
      ],
      "Step 3:": [
        ""
      ],
      "Finish the wire transfer setting the amount in your banking app or website, then this withdrawal will proceed automatically.": [
        ""
      ],
      "Make sure ALL data is correct, including the subject; otherwise, the money will not arrive in this wallet. You can use the copy buttons (%1$s) to prevent typing errors or the \"payto://\" URI below to copy just one value.": [
        ""
      ],
      "Alternative if your bank already supports PayTo URI, you can use this %1$s link instead": [
        ""
      ],
      "Continue with banking app or website": [
        ""
      ],
      "Show QR code": [
        ""
      ],
      "Could not load transaction information": [
        ""
      ],
      "Bank account confirmation required for the transaction to complete.": [
        ""
      ],
      "KYC check required for the transaction to complete.": [
        ""
      ],
      "No additional information has been provided.": [
        ""
      ],
      "Follow this link to the%1$s %2$s": [
        ""
      ],
      "This transaction is pending to complete.": [
        ""
      ],
      "Retry": [
        ""
      ],
      "There was an error trying to complete the transaction.": [
        ""
      ],
      "This transaction was aborted.": [
        ""
      ],
      "This transaction failed.": [
        ""
      ],
      "Caution!": [
        ""
      ],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.": [
        ""
      ],
      "Confirm": [
        ""
      ],
      "Doing a cancellation while the transaction still active might result in lost coins. Do you still want to cancel the transaction?": [
        ""
      ],
      "No": [
        ""
      ],
      "Yes": [
        ""
      ],
      "Abort": [
        ""
      ],
      "Resume": [
        ""
      ],
      "Suspend": [
        ""
      ],
      "Delete": [
        ""
      ],
      "Withdrawal": [
        ""
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Select one of the accounts and use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Details": [
        ""
      ],
      "Withdrawal incomplete.": [
        "撤退"
      ],
      "If you have already sent money to the service provider account it will wire it back at %1$s": [
        ""
      ],
      "Payment": [
        ""
      ],
      "Refunds": [
        ""
      ],
      "%1$s %2$s on %3$s .": [
        ""
      ],
      "Refund is in progress.": [
        ""
      ],
      "Merchant created a refund for this order but was not automatically picked up.": [
        ""
      ],
      "Offer": [
        ""
      ],
      "Accept": [
        ""
      ],
      "Confirmation code": [
        ""
      ],
      "Merchant": [
        ""
      ],
      "Invoice ID": [
        ""
      ],
      "Deposit": [
        ""
      ],
      "Wire transfer deadline.": [
        ""
      ],
      "Wire transfer still pending.": [
        ""
      ],
      "The service provider deadline to make the wire transfer is: %1$s": [
        ""
      ],
      "Wire transfer completed.": [
        ""
      ],
      " ": [
        ""
      ],
      "Transfer details": [
        ""
      ],
      "Wire transfer in progress.": [
        ""
      ],
      "Refresh": [
        ""
      ],
      "Refund": [
        ""
      ],
      "Purchase summary": [
        ""
      ],
      "Credit": [
        ""
      ],
      "Invoice": [
        ""
      ],
      "URI": [
        ""
      ],
      "Debit": [
        ""
      ],
      "Transfer": [
        ""
      ],
      "Lost": [
        ""
      ],
      "Reason": [
        ""
      ],
      "Denomination expired.": [
        ""
      ],
      "Denomination vanished.": [
        ""
      ],
      "Denomination is unoffered.": [
        ""
      ],
      "Fees": [
        ""
      ],
      "Total": [
        ""
      ],
      "Sent": [
        ""
      ],
      "Converted": [
        ""
      ],
      "Bank fee": [
        ""
      ],
      "Price": [
        ""
      ],
      "Subtotal": [
        ""
      ],
      "Refunded": [
        ""
      ],
      "Transfer identification": [
        ""
      ],
      "copy": [
        ""
      ],
      "hide qr": [
        ""
      ],
      "forward to contact": [
        ""
      ],
      "show qr": [
        ""
      ],
      "Wire transfer need a confirmation. Go to the %1$s and check wire transfer operation to exchange account is complete.": [
        ""
      ],
      "bank site": [
        ""
      ],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins.": [
        ""
      ],
      "Exchange is ready to send the coins, withdrawal in progress.": [
        ""
      ],
      "Send &nbsp; %1$s": [
        ""
      ],
      "Pay &nbsp; %1$s": [
        ""
      ],
      "Merchant message": [
        ""
      ],
      "Balance is not enough.": [
        "表示するバランスがありません"
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which exchange and auditors the merchant trust.": [
        ""
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which wire methods the merchant accepts.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to pay for this contract which is restricted.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to spend right know. There are some coins that need to be refreshed.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s available.": [
        ""
      ],
      "Provider is missing the global fee configuration, this likely means it is misconfigured.": [
        ""
      ],
      "Balance looks like it should be enough, but doesn't cover all fees requested by the merchant and payment processor. Please ensure there is at least %1$s %2$s more balance in your wallet or ask your merchant to cover more of the fees.": [
        ""
      ],
      "Pay with a mobile phone": [
        ""
      ],
      "Hide QR": [
        ""
      ],
      "Scan the QR code or &nbsp; %1$s": [
        ""
      ],
      "click here": [
        ""
      ],
      "Could not load the payment and balance status": [
        ""
      ],
      "Purchase": [
        ""
      ],
      "This purchase is age restricted.": [
        ""
      ],
      "Expires at": [
        ""
      ],
      "Already paid, you are going to be redirected to %1$s": [
        ""
      ],
      "Already paid": [
        ""
      ],
      "Already claimed": [
        ""
      ],
      "Could not load the refund status": [
        ""
      ],
      "You&apos;ve ignored the refund.": [
        ""
      ],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [
        ""
      ],
      "Order amount": [
        ""
      ],
      "Could not load the list of exchanges": [
        ""
      ],
      "Could not load info from URI": [
        ""
      ],
      "Could not load the withdrawal details": [
        ""
      ],
      "under %1$s": [
        ""
      ],
      "should be greater than zero": [
        ""
      ],
      "can't be greater than %1$s": [
        ""
      ],
      "Could not load exchange details info": [
        ""
      ],
      "Close": [
        ""
      ],
      "There is no exchange available for currency %1$s": [
        ""
      ],
      "You can add more exchanges from the settings.": [
        ""
      ],
      "Exchange %1$s is not available": [
        ""
      ],
      "Exchange status can view accessed from the settings.": [
        ""
      ],
      "Service fee description": [
        ""
      ],
      "Select %1$s exchange": [
        ""
      ],
      "Reset": [
        ""
      ],
      "Use this exchange": [
        ""
      ],
      "Auditors": [
        ""
      ],
      "Doesn&apos;t have auditors": [
        ""
      ],
      "currency": [
        ""
      ],
      "Coin operations": [
        ""
      ],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.": [
        ""
      ],
      "Deposits": [
        ""
      ],
      "Denomination": [
        ""
      ],
      "Current": [
        ""
      ],
      "Selected": [
        ""
      ],
      "Until": [
        ""
      ],
      "Withdrawals": [
        "撤退"
      ],
      "Transfer operations": [
        ""
      ],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.": [
        ""
      ],
      "Operation": [
        ""
      ],
      "Fee": [
        ""
      ],
      "Wallet operations": [
        ""
      ],
      "Feature": [
        ""
      ],
      "Currency": [
        ""
      ],
      "This operation has already been completed.": [
        ""
      ],
      "This operation has already been aborted": [
        ""
      ],
      "This operation has started and should be completed in the bank.": [
        ""
      ],
      "You can confirm the operation in": [
        ""
      ],
      "this page": [
        ""
      ],
      "This operation has already been completed by another wallet.": [
        ""
      ],
      "This operation has already been used by another wallet.": [
        ""
      ],
      "It can be confirmed in": [
        ""
      ],
      "Age restriction": [
        ""
      ],
      "Withdraw &nbsp; %1$s": [
        ""
      ],
      "Withdraw to a mobile phone": [
        ""
      ],
      "Short description of the invoice": [
        ""
      ],
      "Create": [
        ""
      ],
      "Short description of the transfer": [
        ""
      ],
      "Could not load the invoice payment status": [
        ""
      ],
      "Receive &nbsp; %1$s": [
        ""
      ],
      "Could not load known bank accounts": [
        ""
      ],
      "Could not load supported wire methods": [
        ""
      ],
      "No wire methods supported for this currency": [
        ""
      ],
      "Try another wire method.": [
        ""
      ],
      "Known accounts for %1$s": [
        ""
      ],
      "To add a new account first select the account type.": [
        ""
      ],
      "Unable add this account": [
        ""
      ],
      "Add": [
        ""
      ],
      "IBAN accounts": [
        ""
      ],
      "Alias": [
        ""
      ],
      "Int. Account Number": [
        ""
      ],
      "Account name": [
        ""
      ],
      "KYC": [
        ""
      ],
      "Taler accounts": [
        ""
      ],
      "Host": [
        ""
      ],
      "Bitcoin accounts": [
        ""
      ],
      "Address": [
        ""
      ],
      "Can't be empty": [
        ""
      ],
      "Bitcoin Account": [
        ""
      ],
      "Taler Bank": [
        ""
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "Required": [
        ""
      ],
      "International Bank Account Number": [
        ""
      ],
      "Could not load balance information": [
        ""
      ],
      "Could not load fee for amount %1$s": [
        ""
      ],
      "A currency or an amount should be indicated": [
        ""
      ],
      "There is no enough balance to make a deposit for currency %1$s": [
        ""
      ],
      "Send %1$s to your account": [
        ""
      ],
      "There is no account to make a deposit for currency %1$s": [
        ""
      ],
      "Add account": [
        ""
      ],
      "Select account": [
        ""
      ],
      "Manage accounts": [
        ""
      ],
      "Brut amount": [
        ""
      ],
      "Deposit fee": [
        ""
      ],
      "Net amount": [
        ""
      ],
      "Deposit&nbsp;%1$s %2$s": [
        ""
      ],
      "Need approval in the Bank": [
        ""
      ],
      "Waiting for wire transfer to complete": [
        ""
      ],
      "Exchange is waiting the wire transfer": [
        ""
      ],
      "Payment in progress": [
        ""
      ],
      "Executing refund...": [
        ""
      ],
      "Refreshing coins...": [
        ""
      ],
      "Deposit in progress": [
        ""
      ],
      "Waiting to be paid": [
        ""
      ],
      "Receiving the transfer": [
        ""
      ],
      "Waiting to be received": [
        ""
      ],
      "Denomination expired": [
        ""
      ],
      "Denomination vanished": [
        ""
      ],
      "Denomination unoffered": [
        ""
      ],
      "ABORTED": [
        ""
      ],
      "FAILED": [
        ""
      ],
      "Could not load the list of transactions": [
        ""
      ],
      "unknown scope": [
        ""
      ],
      "Send": [
        ""
      ],
      "Your transaction history is empty for this currency.": [
        ""
      ],
      "Review terms of service": [
        ""
      ],
      "Provider URL": [
        ""
      ],
      "Please review and accept this provider&apos;s terms of service": [
        ""
      ],
      "Pricing": [
        ""
      ],
      "free of charge": [
        ""
      ],
      "%1$s per year of service": [
        ""
      ],
      "Storage": [
        ""
      ],
      "%1$s megabytes of storage per year of service": [
        ""
      ],
      "Accept terms of service": [
        ""
      ],
      "Add backup provider": [
        ""
      ],
      "Backup providers may charge for their service": [
        ""
      ],
      "URL": [
        ""
      ],
      "Name": [
        ""
      ],
      "Next": [
        ""
      ],
      "Navigator": [
        ""
      ],
      "Automatically open wallet": [
        ""
      ],
      "Open the wallet when a payment action is found.": [
        ""
      ],
      "Version Info": [
        ""
      ],
      "Web Extension": [
        ""
      ],
      "The version of wallet core is not supported. (supported version: %1$s, wallet version: %2$s)": [
        ""
      ],
      "Exchange compatibility": [
        ""
      ],
      "Merchant compatibility": [
        ""
      ],
      "Bank compatibility": [
        ""
      ],
      "Wallet Core compatibility": [
        ""
      ],
      "Enable developer mode": [
        ""
      ],
      "Show more information and options in the UI": [
        ""
      ],
      "Display": [
        ""
      ],
      "Current Language": [
        ""
      ],
      "Show backup feature": [
        ""
      ],
      "Backup integration still in beta.": [
        ""
      ],
      "Show P2P alias feature": [
        ""
      ],
      "P2P aliases integration still in beta.": [
        ""
      ],
      "Show suspend/resume transaction": [
        ""
      ],
      "Prevent transaction from doing network request.": [
        ""
      ],
      "Show refresh transaction type in the transaction list": [
        ""
      ],
      "Refresh transaction will be hidden by default if the refresh operation doesn't have fee.": [
        ""
      ],
      "Show more account types on deposit": [
        ""
      ],
      "Extends the UI to more payment target types.": [
        ""
      ],
      "Show JSON on error": [
        ""
      ],
      "Print more information about the error. Useful for debugging.": [
        ""
      ],
      "Allow HTTP connections": [
        ""
      ],
      "Using HTTP connection may be faster but unsafe (wallet restart required)": [
        ""
      ],
      "Lang selector": [
        ""
      ],
      "Allows to manually change the language of the UI. Otherwise it will be automatically selected by your browser configuration.": [
        ""
      ],
      "Edit exchange management": [
        ""
      ],
      "Allows to see the list of exchange, remove, add and switch before withdrawal.": [
        ""
      ],
      "Select terms of service format": [
        ""
      ],
      "Allows to render the terms of service on different format selected by the user.": [
        ""
      ],
      "Show wallet activity": [
        ""
      ],
      "Show the wallet notification and observability event in the UI.": [
        ""
      ],
      "Enable v1 contract": [
        ""
      ],
      "v1 is under development and wallet-core converts v1 contract to v0. Enable this options for testing": [
        ""
      ],
      "GNU Taler Wallet installed!": [
        ""
      ],
      "You can open the wallet using the combination %1$s .": [
        ""
      ],
      "Also pinning the GNU Taler Wallet to your browser allows you to quick access without keyboard:": [
        ""
      ],
      "Click the puzzle icon": [
        ""
      ],
      "Search for GNU Taler Wallet": [
        ""
      ],
      "Click the pin icon": [
        ""
      ],
      "Next Steps": [
        ""
      ],
      "Try the demo": [
        ""
      ],
      "Click here to open the wallet activity tab.": [
        ""
      ],
      "Events": [
        ""
      ],
      "Active tasks": [
        ""
      ],
      "All events": [
        ""
      ],
      "Balance change": [
        "残高"
      ],
      "Backup failed": [
        "バックアップ"
      ],
      "Transaction updated": [
        ""
      ],
      "Exchange updated": [
        ""
      ],
      "Idle": [
        ""
      ],
      "task.%1$s": [
        ""
      ],
      "wallet.%1$s(%2$s)": [
        ""
      ],
      "Bank account info changed": [
        ""
      ],
      "Contact added": [
        ""
      ],
      "Contact deleted": [
        ""
      ],
      "Mailbox message added": [
        ""
      ],
      "Mailbox message deleted": [
        ""
      ],
      "Type": [
        ""
      ],
      "Id": [
        ""
      ],
      "Since": [
        ""
      ],
      "Next try": [
        ""
      ],
      "Error": [
        ""
      ],
      "Transaction": [
        ""
      ],
      "No exchange yet": [
        ""
      ],
      "Status": [
        ""
      ],
      "Terms of Service": [
        ""
      ],
      "Last Update": [
        ""
      ],
      "Actions": [
        ""
      ],
      "ok": [
        ""
      ],
      "pending": [
        ""
      ],
      "proposed": [
        ""
      ],
      "unknown (exchange status should be updated)": [
        ""
      ],
      "Reload": [
        ""
      ],
      "Purge": [
        ""
      ],
      "Make regional": [
        ""
      ],
      "Make global": [
        ""
      ],
      "Forget ToS": [
        ""
      ],
      "Debug tools": [
        ""
      ],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?": [
        ""
      ],
      "reset": [
        ""
      ],
      "TESTING: This may delete all your coin, proceed with caution": [
        ""
      ],
      "run gc": [
        ""
      ],
      "import database": [
        ""
      ],
      "export database": [
        ""
      ],
      "Clear exchange key cache": [
        ""
      ],
      "Database exported at %1$s %2$s to download": [
        ""
      ],
      "Inject Taler support in all pages": [
        ""
      ],
      "Enabling this option will make `window.taler` be available in all sites": [
        ""
      ],
      "Exchange Entries": [
        ""
      ],
      "Add an exchange": [
        ""
      ],
      "Log levels": [
        ""
      ],
      "Coins": [
        ""
      ],
      "usable coins": [
        ""
      ],
      "id": [
        ""
      ],
      "denom": [
        ""
      ],
      "status": [
        ""
      ],
      "from refresh?": [
        ""
      ],
      "age key count": [
        ""
      ],
      "spent coins": [
        ""
      ],
      "click to show": [
        ""
      ],
      "URI is not valid. Taler URI should start with \"taler://\"": [
        ""
      ],
      "After the URI type it should follow a '/' with more information.": [
        ""
      ],
      "This URI type is not supported": [
        ""
      ],
      "This URI requires the bank host and operation id separated by /": [
        ""
      ],
      "This URI requires the merchant host, order id and session id separated by /": [
        ""
      ],
      "This URI requires the merchant host and order id separated by /": [
        ""
      ],
      "This URI requires the exchange host and key id separated by /": [
        ""
      ],
      "This URI requires the merchant host and template id separated by /": [
        ""
      ],
      "This URI requires the host key and provider list separated by /": [
        ""
      ],
      "This URI requires the experiment id": [
        ""
      ],
      "This URI requires the exchange host": [
        ""
      ],
      "This URI requires the alias type, the alias and the mailbox URI separated by /": [
        ""
      ],
      "This URI requires string after the first /": [
        ""
      ],
      "The bank host is invalid": [
        ""
      ],
      "The merchant host is invalid": [
        ""
      ],
      "The URI should end with /": [
        ""
      ],
      "The exchange host is invalid": [
        ""
      ],
      "The contact is invalid": [
        ""
      ],
      "The amount is invalid": [
        ""
      ],
      "A parameter is invalid": [
        ""
      ],
      "checking...": [
        ""
      ],
      "Unexpected error happen reading the camera: %1$s": [
        ""
      ],
      "Could not found a QR code in the file": [
        ""
      ],
      "Unexpected error happen reading the file: %1$s": [
        ""
      ],
      "Scan a QR code or enter taler:// URI below": [
        ""
      ],
      "Pay invoice": [
        ""
      ],
      "Withdrawal from bank": [
        "撤退"
      ],
      "Claim refund": [
        ""
      ],
      "Accept payment": [
        ""
      ],
      "Complete order": [
        ""
      ],
      "Restore wallet": [
        ""
      ],
      "Enable experiment": [
        ""
      ],
      "Withdraw from exchange": [
        ""
      ],
      "Add exchange": [
        ""
      ],
      "Notify transaction": [
        ""
      ],
      "Add contact": [
        ""
      ],
      "The exchange URL is invalid.": [
        ""
      ],
      "The exchange URL should end with '/'": [
        ""
      ],
      "Couldn't found an exchange in the URL specified.": [
        ""
      ],
      "HTTP request failed to %1$s: %2$s": [
        ""
      ],
      "HTTP request failed to %1$s": [
        ""
      ],
      "The merchant URL is invalid.": [
        ""
      ],
      "The merchant URL should end with '/'": [
        ""
      ],
      "Couldn't found an merchant in the URL specified.": [
        ""
      ],
      "The bank URL is invalid.": [
        ""
      ],
      "The bank URL should end with '/'": [
        ""
      ],
      "Couldn't found an bank in the URL specified.": [
        ""
      ],
      "Could not load exchanges": [
        ""
      ],
      "Choose a currency to proceed or add another exchange": [
        ""
      ],
      "Known currencies": [
        ""
      ],
      "Specify the origin": [
        ""
      ],
      "Use previous origins:": [
        ""
      ],
      "From another bank account": [
        ""
      ],
      "From another wallet": [
        ""
      ],
      "From a %1$s URI or QR code": [
        ""
      ],
      "Enter URI here": [
        ""
      ],
      "Specify the destination": [
        ""
      ],
      "Use previous destinations:": [
        ""
      ],
      "To another bank account": [
        ""
      ],
      "To another wallet": [
        ""
      ],
      "To a %1$s URI or QR code": [
        ""
      ],
      "Afghanistan": [
        ""
      ],
      "Albania": [
        ""
      ],
      "Algeria": [
        ""
      ],
      "United States": [
        ""
      ],
      "Andorra": [
        ""
      ],
      "Angola": [
        ""
      ],
      "Antigua and Barbuda": [
        ""
      ],
      "Argentina": [
        ""
      ],
      "Armenia": [
        ""
      ],
      "Australia": [
        ""
      ],
      "Austria": [
        ""
      ],
      "Azerbaijan": [
        ""
      ],
      "Bahamas": [
        ""
      ],
      "Bahrain": [
        ""
      ],
      "Bangladesh": [
        ""
      ],
      "Barbados": [
        ""
      ],
      "Botswana": [
        ""
      ],
      "Belarus": [
        ""
      ],
      "Belgium": [
        ""
      ],
      "Belize": [
        ""
      ],
      "Benin": [
        ""
      ],
      "Bhutan": [
        ""
      ],
      "Bolivia": [
        ""
      ],
      "Bosnia and Herzegovina": [
        ""
      ],
      "Brazil": [
        ""
      ],
      "United Kingdom": [
        ""
      ],
      "Brunei": [
        ""
      ],
      "Bulgaria": [
        ""
      ],
      "Burkina Faso": [
        ""
      ],
      "Myanmar": [
        ""
      ],
      "Burundi": [
        ""
      ],
      "Cambodia": [
        ""
      ],
      "Cameroon": [
        ""
      ],
      "Canada": [
        ""
      ],
      "Cape Verde": [
        ""
      ],
      "Central African Republic": [
        ""
      ],
      "Chad": [
        ""
      ],
      "Chile": [
        ""
      ],
      "China": [
        ""
      ],
      "Colombia": [
        ""
      ],
      "Comoros": [
        ""
      ],
      "Democratic Republic of the Congo": [
        ""
      ],
      "Republic of the Congo": [
        ""
      ],
      "Costa Rica": [
        ""
      ],
      "Croatia": [
        ""
      ],
      "Cuba": [
        ""
      ],
      "Cyprus": [
        ""
      ],
      "Czech Republic": [
        ""
      ],
      "Denmark": [
        ""
      ],
      "Djibouti": [
        ""
      ],
      "Dominican Republic": [
        ""
      ],
      "Netherlands": [
        ""
      ],
      "Timor-Leste": [
        ""
      ],
      "Ecuador": [
        ""
      ],
      "Egypt": [
        ""
      ],
      "United Arab Emirates": [
        ""
      ],
      "Equatorial Guinea": [
        ""
      ],
      "Eritrea": [
        ""
      ],
      "Estonia": [
        ""
      ],
      "Ethiopia": [
        ""
      ],
      "Fiji": [
        ""
      ],
      "Philippines": [
        ""
      ],
      "Finland": [
        ""
      ],
      "France": [
        "残高"
      ],
      "Gabon": [
        ""
      ],
      "Gambia": [
        ""
      ],
      "Georgia": [
        ""
      ],
      "Germany": [
        ""
      ],
      "Ghana": [
        ""
      ],
      "Greece": [
        ""
      ],
      "Grenada": [
        ""
      ],
      "Guatemala": [
        ""
      ],
      "Guinea-Bissau": [
        ""
      ],
      "Guinea": [
        ""
      ],
      "Guyana": [
        ""
      ],
      "Haiti": [
        ""
      ],
      "Honduras": [
        ""
      ],
      "Hungary": [
        ""
      ],
      "Kiribati": [
        ""
      ],
      "Iceland": [
        ""
      ],
      "India": [
        ""
      ],
      "Indonesia": [
        ""
      ],
      "Iran": [
        ""
      ],
      "Iraq": [
        ""
      ],
      "Ireland": [
        ""
      ],
      "Israel": [
        ""
      ],
      "Italy": [
        ""
      ],
      "Ivory Coast": [
        ""
      ],
      "Jamaica": [
        ""
      ],
      "Japan": [
        ""
      ],
      "Jordan": [
        ""
      ],
      "Kazakhstan": [
        ""
      ],
      "Kenya": [
        ""
      ],
      "Saint Kitts and Nevis": [
        ""
      ],
      "Kuwait": [
        ""
      ],
      "Kyrgyzstan": [
        ""
      ],
      "Laos": [
        ""
      ],
      "Latvia": [
        ""
      ],
      "Lebanon": [
        ""
      ],
      "Liberia": [
        ""
      ],
      "Libya": [
        ""
      ],
      "Liechtenstein": [
        ""
      ],
      "Lithuania": [
        ""
      ],
      "Luxembourg": [
        ""
      ],
      "North Macedonia": [
        ""
      ],
      "Madagascar": [
        ""
      ],
      "Malawi": [
        ""
      ],
      "Malaysia": [
        ""
      ],
      "Maldives": [
        ""
      ],
      "Mali": [
        ""
      ],
      "Malta": [
        ""
      ],
      "Marshall Islands": [
        ""
      ],
      "Mauritania": [
        ""
      ],
      "Mauritius": [
        ""
      ],
      "Mexico": [
        ""
      ],
      "Micronesia": [
        ""
      ],
      "Moldova": [
        ""
      ],
      "Monaco": [
        ""
      ],
      "Mongolia": [
        ""
      ],
      "Morocco": [
        ""
      ],
      "Lesotho": [
        ""
      ],
      "Mozambique": [
        ""
      ],
      "Namibia": [
        ""
      ],
      "Nauru": [
        ""
      ],
      "Nepal": [
        ""
      ],
      "New Zealand": [
        ""
      ],
      "Vanuatu": [
        ""
      ],
      "Nicaragua": [
        ""
      ],
      "Nigeria": [
        ""
      ],
      "Niger": [
        ""
      ],
      "North Korea": [
        ""
      ],
      "Norway": [
        ""
      ],
      "Oman": [
        ""
      ],
      "Pakistan": [
        ""
      ],
      "Palau": [
        ""
      ],
      "Panama": [
        ""
      ],
      "Papua New Guinea": [
        ""
      ],
      "Paraguay": [
        ""
      ],
      "Peru": [
        ""
      ],
      "Poland": [
        ""
      ],
      "Portugal": [
        ""
      ],
      "Qatar": [
        ""
      ],
      "Romania": [
        ""
      ],
      "Russia": [
        ""
      ],
      "Rwanda": [
        ""
      ],
      "Saint Lucia": [
        ""
      ],
      "El Salvador": [
        ""
      ],
      "Samoa": [
        ""
      ],
      "San Marino": [
        ""
      ],
      "São Tomé and Príncipe": [
        ""
      ],
      "Saudi Arabia": [
        ""
      ],
      "Senegal": [
        ""
      ],
      "Serbia": [
        ""
      ],
      "Seychelles": [
        ""
      ],
      "Sierra Leone": [
        ""
      ],
      "Singapore": [
        ""
      ],
      "Slovakia": [
        ""
      ],
      "Slovenia": [
        ""
      ],
      "Solomon Islands": [
        ""
      ],
      "Somalia": [
        ""
      ],
      "South Africa": [
        ""
      ],
      "South Korea": [
        ""
      ],
      "Spain": [
        ""
      ],
      "Sri Lanka": [
        ""
      ],
      "Sudan": [
        ""
      ],
      "Suriname": [
        ""
      ],
      "Eswatini": [
        ""
      ],
      "Sweden": [
        ""
      ],
      "Switzerland": [
        ""
      ],
      "Syria": [
        ""
      ],
      "Taiwan": [
        ""
      ],
      "Tajikistan": [
        ""
      ],
      "Tanzania": [
        ""
      ],
      "Thailand": [
        ""
      ],
      "Togo": [
        ""
      ],
      "Tonga": [
        ""
      ],
      "Trinidad and Tobago": [
        ""
      ],
      "Tunisia": [
        ""
      ],
      "Turkey": [
        ""
      ],
      "Tuvalu": [
        ""
      ],
      "Uganda": [
        ""
      ],
      "Ukraine": [
        ""
      ],
      "Uruguay": [
        ""
      ],
      "Uzbekistan": [
        ""
      ],
      "Venezuela": [
        ""
      ],
      "Vietnam": [
        ""
      ],
      "Yemen": [
        ""
      ],
      "Zambia": [
        ""
      ],
      "Zimbabwe": [
        ""
      ],
      "Afghan": [
        ""
      ],
      "Albanian": [
        ""
      ],
      "Algerian": [
        ""
      ],
      "American": [
        ""
      ],
      "Andorran": [
        ""
      ],
      "Angolan": [
        ""
      ],
      "Antiguan": [
        ""
      ],
      "Argentinean": [
        ""
      ],
      "Armenian": [
        ""
      ],
      "Australian": [
        ""
      ],
      "Austrian": [
        ""
      ],
      "Azerbaijani": [
        ""
      ],
      "Bahamian": [
        ""
      ],
      "Bahraini": [
        ""
      ],
      "Bangladeshi": [
        ""
      ],
      "Barbadian": [
        ""
      ],
      "Batswana": [
        ""
      ],
      "Belarusian": [
        ""
      ],
      "Belgian": [
        ""
      ],
      "Belizean": [
        ""
      ],
      "Beninese": [
        ""
      ],
      "Bhutanese": [
        ""
      ],
      "Bolivian": [
        ""
      ],
      "Bosnian": [
        ""
      ],
      "Brazilian": [
        ""
      ],
      "British": [
        ""
      ],
      "Bruneian": [
        ""
      ],
      "Bulgarian": [
        ""
      ],
      "Burkinabe": [
        ""
      ],
      "Burmese": [
        ""
      ],
      "Burundian": [
        ""
      ],
      "Cambodian": [
        ""
      ],
      "Cameroonian": [
        ""
      ],
      "Canadian": [
        ""
      ],
      "Cape Verdean": [
        ""
      ],
      "Central African": [
        ""
      ],
      "Chadian": [
        ""
      ],
      "Chilean": [
        ""
      ],
      "Chinese": [
        ""
      ],
      "Colombian": [
        ""
      ],
      "Comoran": [
        ""
      ],
      "Congolese": [
        ""
      ],
      "Costa Rican": [
        ""
      ],
      "Croatian": [
        ""
      ],
      "Cuban": [
        ""
      ],
      "Cypriot": [
        ""
      ],
      "Czech": [
        ""
      ],
      "Danish": [
        ""
      ],
      "Djiboutian": [
        ""
      ],
      "Dominican": [
        ""
      ],
      "Dutch": [
        ""
      ],
      "East Timorese": [
        ""
      ],
      "Ecuadorean": [
        ""
      ],
      "Egyptian": [
        ""
      ],
      "Emirati": [
        ""
      ],
      "Equatorial Guinean": [
        ""
      ],
      "Eritrean": [
        ""
      ],
      "Estonian": [
        ""
      ],
      "Ethiopian": [
        ""
      ],
      "Fijian": [
        ""
      ],
      "Filipino": [
        ""
      ],
      "Finnish": [
        ""
      ],
      "French": [
        ""
      ],
      "Gabonese": [
        ""
      ],
      "Gambian": [
        ""
      ],
      "Georgian": [
        ""
      ],
      "German": [
        ""
      ],
      "Ghanaian": [
        ""
      ],
      "Greek": [
        ""
      ],
      "Grenadian": [
        ""
      ],
      "Guatemalan": [
        ""
      ],
      "Guinea-Bissauan": [
        ""
      ],
      "Guinean": [
        ""
      ],
      "Guyanese": [
        ""
      ],
      "Haitian": [
        ""
      ],
      "Honduran": [
        ""
      ],
      "Hungarian": [
        ""
      ],
      "I-Kiribati": [
        ""
      ],
      "Icelander": [
        ""
      ],
      "Indian": [
        ""
      ],
      "Indonesian": [
        ""
      ],
      "Iranian": [
        ""
      ],
      "Iraqi": [
        ""
      ],
      "Irish": [
        ""
      ],
      "Israeli": [
        ""
      ],
      "Italian": [
        ""
      ],
      "Ivorian": [
        ""
      ],
      "Jamaican": [
        ""
      ],
      "Japanese": [
        ""
      ],
      "Jordanian": [
        ""
      ],
      "Kazakhstani": [
        ""
      ],
      "Kenyan": [
        ""
      ],
      "Kittian and Nevisian": [
        ""
      ],
      "Kuwaiti": [
        ""
      ],
      "Kyrgyz": [
        ""
      ],
      "Laotian": [
        ""
      ],
      "Latvian": [
        ""
      ],
      "Lebanese": [
        ""
      ],
      "Liberian": [
        ""
      ],
      "Libyan": [
        ""
      ],
      "Liechtensteiner": [
        ""
      ],
      "Lithuanian": [
        ""
      ],
      "Luxembourger": [
        ""
      ],
      "Macedonian": [
        ""
      ],
      "Malagasy": [
        ""
      ],
      "Malawian": [
        ""
      ],
      "Malaysian": [
        ""
      ],
      "Maldivian": [
        ""
      ],
      "Malian": [
        ""
      ],
      "Maltese": [
        ""
      ],
      "Marshallese": [
        ""
      ],
      "Mauritanian": [
        ""
      ],
      "Mauritian": [
        ""
      ],
      "Mexican": [
        ""
      ],
      "Micronesian": [
        ""
      ],
      "Moldovan": [
        ""
      ],
      "Monacan": [
        ""
      ],
      "Mongolian": [
        ""
      ],
      "Moroccan": [
        ""
      ],
      "Mosotho": [
        ""
      ],
      "Mozambican": [
        ""
      ],
      "Namibian": [
        ""
      ],
      "Nauruan": [
        ""
      ],
      "Nepalese": [
        ""
      ],
      "New Zealander": [
        ""
      ],
      "Ni-Vanuatu": [
        ""
      ],
      "Nicaraguan": [
        ""
      ],
      "Nigerian": [
        ""
      ],
      "Nigerien": [
        ""
      ],
      "North Korean": [
        ""
      ],
      "Norwegian": [
        ""
      ],
      "Omani": [
        ""
      ],
      "Pakistani": [
        ""
      ],
      "Palauan": [
        ""
      ],
      "Panamanian": [
        ""
      ],
      "Papua New Guinean": [
        ""
      ],
      "Paraguayan": [
        ""
      ],
      "Peruvian": [
        ""
      ],
      "Polish": [
        ""
      ],
      "Portuguese": [
        ""
      ],
      "Qatari": [
        ""
      ],
      "Romanian": [
        ""
      ],
      "Russian": [
        ""
      ],
      "Rwandan": [
        ""
      ],
      "Saint Lucian": [
        ""
      ],
      "Salvadoran": [
        ""
      ],
      "Samoan": [
        ""
      ],
      "San Marinese": [
        ""
      ],
      "Sao Tomean": [
        ""
      ],
      "Saudi": [
        ""
      ],
      "Senegalese": [
        ""
      ],
      "Serbian": [
        ""
      ],
      "Seychellois": [
        ""
      ],
      "Sierra Leonean": [
        ""
      ],
      "Singaporean": [
        ""
      ],
      "Slovakian": [
        ""
      ],
      "Slovenian": [
        ""
      ],
      "Solomon Islander": [
        ""
      ],
      "Somali": [
        ""
      ],
      "South African": [
        ""
      ],
      "South Korean": [
        ""
      ],
      "Spanish": [
        ""
      ],
      "Sri Lankan": [
        ""
      ],
      "Sudanese": [
        ""
      ],
      "Surinamer": [
        ""
      ],
      "Swazi": [
        ""
      ],
      "Swedish": [
        ""
      ],
      "Swiss": [
        ""
      ],
      "Syrian": [
        ""
      ],
      "Taiwanese": [
        ""
      ],
      "Tajik": [
        ""
      ],
      "Tanzanian": [
        ""
      ],
      "Thai": [
        ""
      ],
      "Togolese": [
        ""
      ],
      "Tongan": [
        ""
      ],
      "Trinidadian or Tobagonian": [
        ""
      ],
      "Tunisian": [
        ""
      ],
      "Turkish": [
        ""
      ],
      "Tuvaluan": [
        ""
      ],
      "Ugandan": [
        ""
      ],
      "Ukrainian": [
        ""
      ],
      "Uruguayan": [
        ""
      ],
      "Uzbekistani": [
        ""
      ],
      "Venezuelan": [
        ""
      ],
      "Vietnamese": [
        ""
      ],
      "Yemenite": [
        ""
      ],
      "Zambian": [
        ""
      ],
      "Zimbabwean": [
        ""
      ],
      "🇦🇫": [
        ""
      ],
      "🇦🇱": [
        ""
      ],
      "🇩🇿": [
        ""
      ],
      "🇺🇸": [
        ""
      ],
      "🇦🇩": [
        ""
      ],
      "🇦🇴": [
        ""
      ],
      "🇦🇬": [
        ""
      ],
      "🇦🇷": [
        ""
      ],
      "🇦🇲": [
        ""
      ],
      "🇦🇺": [
        ""
      ],
      "🇦🇹": [
        ""
      ],
      "🇦🇿": [
        ""
      ],
      "🇧🇸": [
        ""
      ],
      "🇧🇭": [
        ""
      ],
      "🇧🇩": [
        ""
      ],
      "🇧🇧": [
        ""
      ],
      "🇧🇼": [
        ""
      ],
      "🇧🇾": [
        ""
      ],
      "🇧🇪": [
        ""
      ],
      "🇧🇿": [
        ""
      ],
      "🇧🇯": [
        ""
      ],
      "🇧🇹": [
        ""
      ],
      "🇧🇴": [
        ""
      ],
      "🇧🇦": [
        ""
      ],
      "🇧🇷": [
        ""
      ],
      "🇬🇧": [
        ""
      ],
      "🇧🇳": [
        ""
      ],
      "🇧🇬": [
        ""
      ],
      "🇧🇫": [
        ""
      ],
      "🇲🇲": [
        ""
      ],
      "🇧🇮": [
        ""
      ],
      "🇰🇭": [
        ""
      ],
      "🇨🇲": [
        ""
      ],
      "🇨🇦": [
        ""
      ],
      "🇨🇻": [
        ""
      ],
      "🇨🇫": [
        ""
      ],
      "🇨🇱": [
        ""
      ],
      "🇨🇳": [
        ""
      ],
      "🇨🇴": [
        ""
      ],
      "🇨🇩": [
        ""
      ],
      "🇨🇬": [
        ""
      ],
      "🇨🇷": [
        ""
      ],
      "🇨🇺": [
        ""
      ],
      "🇨🇾": [
        ""
      ],
      "🇨🇿": [
        ""
      ],
      "🇩🇰": [
        ""
      ],
      "🇩🇯": [
        ""
      ],
      "🇩🇴": [
        ""
      ],
      "🇳🇱": [
        ""
      ],
      "🇹🇱": [
        ""
      ],
      "🇪🇨": [
        ""
      ],
      "🇪🇬": [
        ""
      ],
      "🇦🇪": [
        ""
      ],
      "🇬🇶": [
        ""
      ],
      "🇪🇷": [
        ""
      ],
      "🇪🇪": [
        ""
      ],
      "🇪🇹": [
        ""
      ],
      "🇫🇯": [
        ""
      ],
      "🇵🇭": [
        ""
      ],
      "🇫🇮": [
        ""
      ],
      "🇫🇷": [
        ""
      ],
      "🇬🇦": [
        ""
      ],
      "🇬🇲": [
        ""
      ],
      "🇬🇪": [
        ""
      ],
      "UAE Dirham": [
        ""
      ],
      "Afghani": [
        ""
      ],
      "Lek": [
        ""
      ],
      "Armenian Dram": [
        ""
      ],
      "Netherlands Antillean Guilder": [
        ""
      ],
      "Kwanza": [
        ""
      ],
      "Argentine Peso": [
        ""
      ],
      "Australian Dollar": [
        ""
      ],
      "Aruban Florin": [
        ""
      ],
      "Azerbaijan Manat": [
        ""
      ],
      "Convertible Mark": [
        ""
      ],
      "Barbados Dollar": [
        ""
      ],
      "Taka": [
        ""
      ],
      "Bulgarian Lev": [
        ""
      ],
      "Bahraini Dinar": [
        ""
      ],
      "Burundi Franc": [
        ""
      ],
      "Bermudian Dollar": [
        ""
      ],
      "Brunei Dollar": [
        ""
      ],
      "Boliviano": [
        ""
      ],
      "Mvdol": [
        ""
      ],
      "Brazilian Real": [
        ""
      ],
      "Bahamian Dollar": [
        ""
      ],
      "Ngultrum": [
        ""
      ],
      "Pula": [
        ""
      ],
      "Belarusian Ruble": [
        ""
      ],
      "Belize Dollar": [
        ""
      ],
      "Canadian Dollar": [
        ""
      ],
      "Congolese Franc": [
        ""
      ],
      "WIR Euro": [
        ""
      ],
      "Swiss Franc": [
        ""
      ],
      "WIR Franc": [
        ""
      ],
      "Unidad de Fomento": [
        ""
      ],
      "Chilean Peso": [
        ""
      ],
      "Yuan Renminbi": [
        ""
      ],
      "Colombian Peso": [
        ""
      ],
      "Unidad de Valor Real": [
        ""
      ],
      "Costa Rican Colon": [
        ""
      ],
      "Cuban Peso": [
        ""
      ],
      "Cabo Verde Escudo": [
        ""
      ],
      "Czech Koruna": [
        ""
      ],
      "Djibouti Franc": [
        ""
      ],
      "Danish Krone": [
        ""
      ],
      "Dominican Peso": [
        ""
      ],
      "Algerian Dinar": [
        ""
      ],
      "Egyptian Pound": [
        ""
      ],
      "Nakfa": [
        ""
      ],
      "Ethiopian Birr": [
        ""
      ],
      "Euro": [
        ""
      ],
      "Fiji Dollar": [
        ""
      ],
      "Falkland Islands Pound": [
        ""
      ],
      "Pound Sterling": [
        ""
      ],
      "Lari": [
        ""
      ],
      "Ghana Cedi": [
        ""
      ],
      "Gibraltar Pound": [
        ""
      ],
      "Dalasi": [
        ""
      ],
      "Guinean Franc": [
        ""
      ],
      "Quetzal": [
        ""
      ],
      "Guyana Dollar": [
        ""
      ],
      "Hong Kong Dollar": [
        ""
      ],
      "Lempira": [
        ""
      ],
      "Gourde": [
        ""
      ],
      "Forint": [
        ""
      ],
      "Rupiah": [
        ""
      ],
      "New Israeli Sheqel": [
        ""
      ],
      "Indian Rupee": [
        ""
      ],
      "Iraqi Dinar": [
        ""
      ],
      "Iranian Rial": [
        ""
      ],
      "Iceland Krona": [
        ""
      ],
      "Jamaican Dollar": [
        ""
      ],
      "Jordanian Dinar": [
        ""
      ],
      "Yen": [
        ""
      ],
      "Kenyan Shilling": [
        ""
      ],
      "Som": [
        ""
      ],
      "Riel": [
        ""
      ],
      "Comorian Franc ": [
        ""
      ],
      "North Korean Won": [
        ""
      ],
      "Won": [
        ""
      ],
      "Kuwaiti Dinar": [
        ""
      ],
      "Cayman Islands Dollar": [
        ""
      ],
      "Tenge": [
        ""
      ],
      "Lao Kip": [
        ""
      ],
      "Lebanese Pound": [
        ""
      ],
      "Sri Lanka Rupee": [
        ""
      ],
      "Liberian Dollar": [
        ""
      ],
      "Loti": [
        ""
      ],
      "Libyan Dinar": [
        ""
      ],
      "Moroccan Dirham": [
        ""
      ],
      "Moldovan Leu": [
        ""
      ],
      "Malagasy Ariary": [
        ""
      ],
      "Denar": [
        ""
      ],
      "Kyat": [
        ""
      ],
      "Tugrik": [
        ""
      ],
      "Pataca": [
        ""
      ],
      "Ouguiya": [
        ""
      ],
      "Mauritius Rupee": [
        ""
      ],
      "Rufiyaa": [
        ""
      ],
      "Malawi Kwacha": [
        ""
      ],
      "Mexican Peso": [
        ""
      ],
      "Mexican Unidad de Inversion (UDI)": [
        ""
      ],
      "Malaysian Ringgit": [
        ""
      ],
      "Mozambique Metical": [
        ""
      ],
      "Namibia Dollar": [
        ""
      ],
      "Naira": [
        ""
      ],
      "Cordoba Oro": [
        ""
      ],
      "Norwegian Krone": [
        ""
      ],
      "Nepalese Rupee": [
        ""
      ],
      "New Zealand Dollar": [
        ""
      ],
      "Rial Omani": [
        ""
      ],
      "Balboa": [
        ""
      ],
      "Sol": [
        ""
      ],
      "Kina": [
        ""
      ],
      "Philippine Peso": [
        ""
      ],
      "Pakistan Rupee": [
        ""
      ],
      "Zloty": [
        ""
      ],
      "Guarani": [
        ""
      ],
      "Qatari Rial": [
        ""
      ],
      "Romanian Leu": [
        ""
      ],
      "Serbian Dinar": [
        ""
      ],
      "Russian Ruble": [
        ""
      ],
      "Rwanda Franc": [
        ""
      ],
      "Saudi Riyal": [
        ""
      ],
      "Solomon Islands Dollar": [
        ""
      ],
      "Seychelles Rupee": [
        ""
      ],
      "Sudanese Pound": [
        ""
      ],
      "Swedish Krona": [
        ""
      ],
      "Singapore Dollar": [
        ""
      ],
      "Saint Helena Pound": [
        ""
      ],
      "Leone": [
        ""
      ],
      "Somali Shilling": [
        ""
      ],
      "Surinam Dollar": [
        ""
      ],
      "South Sudanese Pound": [
        ""
      ],
      "Dobra": [
        ""
      ],
      "El Salvador Colon": [
        ""
      ],
      "Syrian Pound": [
        ""
      ],
      "Lilangeni": [
        ""
      ],
      "Baht": [
        ""
      ],
      "Somoni": [
        ""
      ],
      "Turkmenistan New Manat": [
        ""
      ],
      "Tunisian Dinar": [
        ""
      ],
      "Pa’anga": [
        ""
      ],
      "Turkish Lira": [
        ""
      ],
      "Trinidad and Tobago Dollar": [
        ""
      ],
      "New Taiwan Dollar": [
        ""
      ],
      "Tanzanian Shilling": [
        ""
      ],
      "Hryvnia": [
        ""
      ],
      "Uganda Shilling": [
        ""
      ],
      "US Dollar": [
        ""
      ],
      "US Dollar (Next day)": [
        ""
      ],
      "Uruguay Peso en Unidades Indexadas (UI)": [
        ""
      ],
      "Peso Uruguayo": [
        ""
      ],
      "Unidad Previsional": [
        ""
      ],
      "Uzbekistan Sum": [
        ""
      ],
      "Bolívar Soberano": [
        ""
      ],
      "Dong": [
        ""
      ],
      "Vatu": [
        ""
      ],
      "Tala": [
        ""
      ],
      "CFA Franc BEAC": [
        ""
      ],
      "East Caribbean Dollar": [
        ""
      ],
      "CFA Franc BCEAO": [
        ""
      ],
      "CFP Franc": [
        ""
      ],
      "Sucre": [
        ""
      ],
      "Yemeni Rial": [
        ""
      ],
      "Rand": [
        ""
      ],
      "Zambian Kwacha": [
        ""
      ],
      "Zimbabwe Gold": [
        ""
      ],
      "Afar": [
        ""
      ],
      "Abkhazian": [
        ""
      ],
      "Achinese": [
        ""
      ],
      "Acoli": [
        ""
      ],
      "Adangme": [
        ""
      ],
      "Adyghe": [
        ""
      ],
      "Avestan": [
        ""
      ],
      "Tunisian Arabic": [
        ""
      ],
      "Afrikaans": [
        ""
      ],
      "Afrihili": [
        ""
      ],
      "Aghem": [
        ""
      ],
      "Ainu": [
        ""
      ],
      "Akan": [
        ""
      ],
      "Akkadian": [
        ""
      ],
      "Alabama": [
        ""
      ],
      "Aleut": [
        ""
      ],
      "Gheg Albanian": [
        ""
      ],
      "Southern Altai": [
        ""
      ],
      "Amarik": [
        ""
      ],
      "Aragonese": [
        ""
      ],
      "Old English": [
        ""
      ],
      "Angika": [
        ""
      ],
      "Modern Standard Arabic": [
        ""
      ],
      "Arabik": [
        ""
      ],
      "Aramaic": [
        ""
      ],
      "Mapuche": [
        ""
      ],
      "Araona": [
        ""
      ],
      "Arapaho": [
        ""
      ],
      "Algerian Arabic": [
        ""
      ],
      "Arawak": [
        ""
      ],
      "Moroccan Arabic": [
        ""
      ],
      "Egyptian Arabic": [
        ""
      ],
      "Asu": [
        ""
      ],
      "Assamese": [
        ""
      ],
      "American Sign Language": [
        ""
      ],
      "Asturian": [
        ""
      ],
      "Avaric": [
        ""
      ],
      "Kotava": [
        ""
      ],
      "Awadhi": [
        ""
      ],
      "Aymara": [
        ""
      ],
      "South Azerbaijani": [
        ""
      ],
      "Bashkir": [
        ""
      ],
      "Baluchi": [
        ""
      ],
      "Balinese": [
        ""
      ],
      "Bavarian": [
        ""
      ],
      "Basaa": [
        ""
      ],
      "Bamun": [
        ""
      ],
      "Batak Toba": [
        ""
      ],
      "Ghomala": [
        ""
      ],
      "Belarus kasa": [
        ""
      ],
      "Beja": [
        ""
      ],
      "Bemba": [
        ""
      ],
      "Betawi": [
        ""
      ],
      "Bena": [
        ""
      ],
      "Bafut": [
        ""
      ],
      "Badaga": [
        ""
      ],
      "Bɔlgeria kasa": [
        ""
      ],
      "Bhojpuri": [
        ""
      ],
      "Bislama": [
        ""
      ],
      "Bikol": [
        ""
      ],
      "Bini": [
        ""
      ],
      "Banjar": [
        ""
      ],
      "Kom": [
        ""
      ],
      "Siksika": [
        ""
      ],
      "Bambara": [
        ""
      ],
      "Bengali kasa": [
        ""
      ],
      "Tibetan": [
        ""
      ],
      "Bishnupriya": [
        ""
      ],
      "Bakhtiari": [
        ""
      ],
      "Braj": [
        ""
      ],
      "Breton": [
        ""
      ],
      "Brahui": [
        ""
      ],
      "Bodo": [
        ""
      ],
      "Akoose": [
        ""
      ],
      "Buriat": [
        ""
      ],
      "Buginese": [
        ""
      ],
      "Bulu": [
        ""
      ],
      "Blin": [
        ""
      ],
      "Medumba": [
        ""
      ],
      "Catalan": [
        ""
      ],
      "Caddo": [
        ""
      ],
      "Carib": [
        ""
      ],
      "Cayuga": [
        ""
      ],
      "Atsam": [
        ""
      ],
      "Cebuano": [
        ""
      ],
      "Chechen": [
        ""
      ],
      "Chiga": [
        ""
      ],
      "Chibcha": [
        ""
      ],
      "Chamorro": [
        ""
      ],
      "Chagatai": [
        ""
      ],
      "Chuukese": [
        ""
      ],
      "Mari": [
        ""
      ],
      "Chinook Jargon": [
        ""
      ],
      "Choctaw": [
        ""
      ],
      "Chipewyan": [
        ""
      ],
      "Cherokee": [
        ""
      ],
      "Cheyenne": [
        ""
      ],
      "Central Kurdish": [
        ""
      ],
      "Corsican": [
        ""
      ],
      "Coptic": [
        ""
      ],
      "Capiznon": [
        ""
      ],
      "Cree": [
        ""
      ],
      "Crimean Turkish": [
        ""
      ],
      "Kashubian": [
        ""
      ],
      "Kyɛk kasa": [
        ""
      ],
      "Church Slavic": [
        ""
      ],
      "Chuvash": [
        ""
      ],
      "Welsh": [
        ""
      ],
      "Dakota": [
        ""
      ],
      "Dargwa": [
        ""
      ],
      "Taita": [
        ""
      ],
      "Austrian German": [
        ""
      ],
      "Swiss High German": [
        ""
      ],
      "Gyaaman": [
        ""
      ],
      "Delaware": [
        ""
      ],
      "Slave": [
        ""
      ],
      "Dogrib": [
        ""
      ],
      "Dinka": [
        ""
      ],
      "Zarma": [
        ""
      ],
      "Dogri": [
        ""
      ],
      "Lower Sorbian": [
        ""
      ],
      "Central Dusun": [
        ""
      ],
      "Duala": [
        ""
      ],
      "Middle Dutch": [
        ""
      ],
      "Divehi": [
        ""
      ],
      "Jola-Fonyi": [
        ""
      ],
      "Dyula": [
        ""
      ],
      "Dzongkha": [
        ""
      ],
      "Dazaga": [
        ""
      ],
      "Embu": [
        ""
      ],
      "Ewe": [
        ""
      ],
      "Efik": [
        ""
      ],
      "Emilian": [
        ""
      ],
      "Ancient Egyptian": [
        ""
      ],
      "Ekajuk": [
        ""
      ],
      "Greek kasa": [
        ""
      ],
      "Elamite": [
        ""
      ],
      "Australian English": [
        ""
      ],
      "Borɔfo": [
        ""
      ],
      "Canadian English": [
        ""
      ],
      "British English": [
        ""
      ],
      "Middle English": [
        ""
      ],
      "American English": [
        ""
      ],
      "Esperanto": [
        ""
      ],
      "Latin American Spanish": [
        ""
      ],
      "European Spanish": [
        ""
      ],
      "Mexican Spanish": [
        ""
      ],
      "Spain kasa": [
        ""
      ],
      "Central Yupik": [
        ""
      ],
      "Basque": [
        ""
      ],
      "Ewondo": [
        ""
      ],
      "Extremaduran": [
        ""
      ],
      "Fang": [
        ""
      ],
      "Pɛɛhyia kasa": [
        ""
      ],
      "Fanti": [
        ""
      ],
      "Fulah": [
        ""
      ],
      "Tornedalen Finnish": [
        ""
      ],
      "Faroese": [
        ""
      ],
      "Fon": [
        ""
      ],
      "Canadian French": [
        ""
      ],
      "Cajun French": [
        ""
      ],
      "Swiss French": [
        ""
      ],
      "Frɛnkye": [
        ""
      ],
      "Middle French": [
        ""
      ],
      "Old French": [
        ""
      ],
      "Arpitan": [
        ""
      ],
      "Northern Frisian": [
        ""
      ],
      "Eastern Frisian": [
        ""
      ],
      "Friulian": [
        ""
      ],
      "Western Frisian": [
        ""
      ],
      "Ga": [
        ""
      ],
      "Gagauz": [
        ""
      ],
      "Gan Chinese": [
        ""
      ],
      "Gayo": [
        ""
      ],
      "Gbaya": [
        ""
      ],
      "Zoroastrian Dari": [
        ""
      ],
      "Scottish Gaelic": [
        ""
      ],
      "Geez": [
        ""
      ],
      "Gilbertese": [
        ""
      ],
      "Galician": [
        ""
      ],
      "Gilaki": [
        ""
      ],
      "Middle High German": [
        ""
      ],
      "Old High German": [
        ""
      ],
      "Goan Konkani": [
        ""
      ],
      "Gondi": [
        ""
      ],
      "Gorontalo": [
        ""
      ],
      "Gothic": [
        ""
      ],
      "Grebo": [
        ""
      ],
      "Ancient Greek": [
        ""
      ],
      "Swiss German": [
        ""
      ],
      "Wayuu": [
        ""
      ],
      "Gujarati": [
        ""
      ],
      "Frafra": [
        ""
      ],
      "Gusii": [
        ""
      ],
      "Manx": [
        ""
      ],
      "Gwichʼin": [
        ""
      ],
      "Hausa": [
        ""
      ],
      "Haida": [
        ""
      ],
      "Hakka Chinese": [
        ""
      ],
      "Hawaiian": [
        ""
      ],
      "Hebrew": [
        ""
      ],
      "Fiji Hindi": [
        ""
      ],
      "Hindi": [
        ""
      ],
      "Hiligaynon": [
        ""
      ],
      "Hittite": [
        ""
      ],
      "Hmong": [
        ""
      ],
      "Hiri Motu": [
        ""
      ],
      "Upper Sorbian": [
        ""
      ],
      "Xiang Chinese": [
        ""
      ],
      "Hangri kasa": [
        ""
      ],
      "Hupa": [
        ""
      ],
      "Herero": [
        ""
      ],
      "Interlingua": [
        ""
      ],
      "Iban": [
        ""
      ],
      "Ibibio": [
        ""
      ],
      "Indonihyia kasa": [
        ""
      ],
      "Interlingue": [
        ""
      ],
      "Igbo": [
        ""
      ],
      "Sichuan Yi": [
        ""
      ],
      "Inupiaq": [
        ""
      ],
      "Iloko": [
        ""
      ],
      "Ingush": [
        ""
      ],
      "Ido": [
        ""
      ],
      "Icelandic": [
        ""
      ],
      "Italy kasa": [
        ""
      ],
      "Inuktitut": [
        ""
      ],
      "Ingrian": [
        ""
      ],
      "Gyapan kasa": [
        ""
      ],
      "Jamaican Creole English": [
        ""
      ],
      "Lojban": [
        ""
      ],
      "Ngomba": [
        ""
      ],
      "Machame": [
        ""
      ],
      "Judeo-Persian": [
        ""
      ],
      "Judeo-Arabic": [
        ""
      ],
      "Jutish": [
        ""
      ],
      "Gyabanis kasa": [
        ""
      ],
      "Kara-Kalpak": [
        ""
      ],
      "Kabyle": [
        ""
      ],
      "Kachin": [
        ""
      ],
      "Jju": [
        ""
      ],
      "Kamba": [
        ""
      ],
      "Kawi": [
        ""
      ],
      "Kabardian": [
        ""
      ],
      "Kanembu": [
        ""
      ],
      "Tyap": [
        ""
      ],
      "Makonde": [
        ""
      ],
      "Kabuverdianu": [
        ""
      ],
      "Kenyang": [
        ""
      ],
      "Koro": [
        ""
      ],
      "Kongo": [
        ""
      ],
      "Kaingang": [
        ""
      ],
      "Khasi": [
        ""
      ],
      "Khotanese": [
        ""
      ],
      "Koyra Chiini": [
        ""
      ],
      "Khowar": [
        ""
      ],
      "Kikuyu": [
        ""
      ],
      "Kirmanjki": [
        ""
      ],
      "Kuanyama": [
        ""
      ],
      "Kako": [
        ""
      ],
      "Kazakh": [
        ""
      ],
      "Kalaallisut": [
        ""
      ],
      "Kalenjin": [
        ""
      ],
      "Kimbundu": [
        ""
      ],
      "Kambodia kasa": [
        ""
      ],
      "Kannada": [
        ""
      ],
      "Komi-Permyak": [
        ""
      ],
      "Konkani": [
        ""
      ],
      "Korea kasa": [
        ""
      ],
      "Kosraean": [
        ""
      ],
      "Kpelle": [
        ""
      ],
      "Karachay-Balkar": [
        ""
      ],
      "Krio": [
        ""
      ],
      "Kinaray-a": [
        ""
      ],
      "Kanuri": [
        ""
      ],
      "Karelian": [
        ""
      ],
      "Kurukh": [
        ""
      ],
      "Shambala": [
        ""
      ],
      "Bafia": [
        ""
      ],
      "Colognian": [
        ""
      ],
      "Kashmiri": [
        ""
      ],
      "Kurdish": [
        ""
      ],
      "Kumyk": [
        ""
      ],
      "Kutenai": [
        ""
      ],
      "Komi": [
        ""
      ],
      "Cornish": [
        ""
      ],
      "Ladino": [
        ""
      ],
      "Langi": [
        ""
      ],
      "Lahnda": [
        ""
      ],
      "Latin": [
        ""
      ],
      "Lamba": [
        ""
      ],
      "Luxembourgish": [
        ""
      ],
      "Lezghian": [
        ""
      ],
      "Lingua Franca Nova": [
        ""
      ],
      "Ganda": [
        ""
      ],
      "Ligurian": [
        ""
      ],
      "Limburgish": [
        ""
      ],
      "Livonian": [
        ""
      ],
      "Lakota": [
        ""
      ],
      "Lombard": [
        ""
      ],
      "Lingala": [
        ""
      ],
      "Lao": [
        ""
      ],
      "Mongo": [
        ""
      ],
      "Lozi": [
        ""
      ],
      "Latgalian": [
        ""
      ],
      "Luba-Lulua": [
        ""
      ],
      "Luiseno": [
        ""
      ],
      "Luba-Katanga": [
        ""
      ],
      "Lunda": [
        ""
      ],
      "Luo": [
        ""
      ],
      "Mizo": [
        ""
      ],
      "Luyia": [
        ""
      ],
      "Literary Chinese": [
        ""
      ],
      "Laz": [
        ""
      ],
      "Madurese": [
        ""
      ],
      "Mafa": [
        ""
      ],
      "Magahi": [
        ""
      ],
      "Maithili": [
        ""
      ],
      "Makasar": [
        ""
      ],
      "Mandingo": [
        ""
      ],
      "Masai": [
        ""
      ],
      "Maba": [
        ""
      ],
      "Moksha": [
        ""
      ],
      "Mandar": [
        ""
      ],
      "Mende": [
        ""
      ],
      "Meru": [
        ""
      ],
      "Morisyen": [
        ""
      ],
      "Middle Irish": [
        ""
      ],
      "Makhuwa-Meetto": [
        ""
      ],
      "Metaʼ": [
        ""
      ],
      "Micmac": [
        ""
      ],
      "Maori": [
        ""
      ],
      "Minangkabau": [
        ""
      ],
      "Malayalam": [
        ""
      ],
      "Manchu": [
        ""
      ],
      "Manipuri": [
        ""
      ],
      "Mohawk": [
        ""
      ],
      "Mossi": [
        ""
      ],
      "Western Mari": [
        ""
      ],
      "Marathi": [
        ""
      ],
      "Malay kasa": [
        ""
      ],
      "Mundang": [
        ""
      ],
      "Multiple Languages": [
        ""
      ],
      "Creek": [
        ""
      ],
      "Mirandese": [
        ""
      ],
      "Marwari": [
        ""
      ],
      "Mentawai": [
        ""
      ],
      "Bɛɛmis kasa": [
        ""
      ],
      "Myene": [
        ""
      ],
      "Erzya": [
        ""
      ],
      "Mazanderani": [
        ""
      ],
      "Min Nan Chinese": [
        ""
      ],
      "Neapolitan": [
        ""
      ],
      "Nama": [
        ""
      ],
      "Norwegian Bokmål": [
        ""
      ],
      "North Ndebele": [
        ""
      ],
      "Low German": [
        ""
      ],
      "Nɛpal kasa": [
        ""
      ],
      "Newari": [
        ""
      ],
      "Ndonga": [
        ""
      ],
      "Nias": [
        ""
      ],
      "Niuean": [
        ""
      ],
      "Ao Naga": [
        ""
      ],
      "Flemish": [
        ""
      ],
      "Dɛɛkye": [
        ""
      ],
      "Kwasio": [
        ""
      ],
      "Ngiemboon": [
        ""
      ],
      "Norwegian Nynorsk": [
        ""
      ],
      "Nogai": [
        ""
      ],
      "Old Norse": [
        ""
      ],
      "Novial": [
        ""
      ],
      "NʼKo": [
        ""
      ],
      "South Ndebele": [
        ""
      ],
      "Northern Sotho": [
        ""
      ],
      "Nuer": [
        ""
      ],
      "Navajo": [
        ""
      ],
      "Classical Newari": [
        ""
      ],
      "Nyamwezi": [
        ""
      ],
      "Nyankole": [
        ""
      ],
      "Nyanja": [
        ""
      ],
      "Nyoro": [
        ""
      ],
      "Nzima": [
        ""
      ],
      "Occitan": [
        ""
      ],
      "Ojibwa": [
        ""
      ],
      "Oromo": [
        ""
      ],
      "Oriya": [
        ""
      ],
      "Osage": [
        ""
      ],
      "Ossetic": [
        ""
      ],
      "Ottoman Turkish": [
        ""
      ],
      "Pangasinan": [
        ""
      ],
      "Pahlavi": [
        ""
      ],
      "Pampanga": [
        ""
      ],
      "Papiamento": [
        ""
      ],
      "Pungyabi kasa": [
        ""
      ],
      "Picard": [
        ""
      ],
      "Pennsylvania German": [
        ""
      ],
      "Plautdietsch": [
        ""
      ],
      "Old Persian": [
        ""
      ],
      "Palatine German": [
        ""
      ],
      "Phoenician": [
        ""
      ],
      "Pali": [
        ""
      ],
      "Pɔland kasa": [
        ""
      ],
      "Piedmontese": [
        ""
      ],
      "Pontic": [
        ""
      ],
      "Pohnpeian": [
        ""
      ],
      "Prussian": [
        ""
      ],
      "Old Provençal": [
        ""
      ],
      "Pashto": [
        ""
      ],
      "Brazilian Portuguese": [
        ""
      ],
      "Pɔɔtugal kasa": [
        ""
      ],
      "European Portuguese": [
        ""
      ],
      "Kʼicheʼ": [
        ""
      ],
      "Chimborazo Highland Quichua": [
        ""
      ],
      "Quechua": [
        ""
      ],
      "Rajasthani": [
        ""
      ],
      "Rapanui": [
        ""
      ],
      "Rarotongan": [
        ""
      ],
      "Romagnol": [
        ""
      ],
      "Riffian": [
        ""
      ],
      "Romansh": [
        ""
      ],
      "Rundi": [
        ""
      ],
      "Rombo": [
        ""
      ],
      "Moldavian": [
        ""
      ],
      "Romany": [
        ""
      ],
      "Root": [
        ""
      ],
      "Romenia kasa": [
        ""
      ],
      "Rotuman": [
        ""
      ],
      "Rusyn": [
        ""
      ],
      "Roviana": [
        ""
      ],
      "Aromanian": [
        ""
      ],
      "Rahyia kasa": [
        ""
      ],
      "Rwa": [
        ""
      ],
      "Rewanda kasa": [
        ""
      ],
      "Sandawe": [
        ""
      ],
      "Sakha": [
        ""
      ],
      "Samaritan Aramaic": [
        ""
      ],
      "Samburu": [
        ""
      ],
      "Sanskrit": [
        ""
      ],
      "Sasak": [
        ""
      ],
      "Santali": [
        ""
      ],
      "Saurashtra": [
        ""
      ],
      "Ngambay": [
        ""
      ],
      "Sangu": [
        ""
      ],
      "Sicilian": [
        ""
      ],
      "Scots": [
        ""
      ],
      "Sardinian": [
        ""
      ],
      "Sassarese Sardinian": [
        ""
      ],
      "Sindhi": [
        ""
      ],
      "Seneca": [
        ""
      ],
      "Sena": [
        ""
      ],
      "Seri": [
        ""
      ],
      "Selkup": [
        ""
      ],
      "Northern Sami": [
        ""
      ],
      "Koyraboro Senni": [
        ""
      ],
      "Old Irish": [
        ""
      ],
      "Sango": [
        ""
      ],
      "Samogitian": [
        ""
      ],
      "Tachelhit": [
        ""
      ],
      "Shan": [
        ""
      ],
      "Serbo-Croatian": [
        ""
      ],
      "Chadian Arabic": [
        ""
      ],
      "Sidamo": [
        ""
      ],
      "Sinhala": [
        ""
      ],
      "Slovak": [
        ""
      ],
      "Lower Silesian": [
        ""
      ],
      "Selayar": [
        ""
      ],
      "Southern Sami": [
        ""
      ],
      "Lule Sami": [
        ""
      ],
      "Inari Sami": [
        ""
      ],
      "Skolt Sami": [
        ""
      ],
      "Soninke": [
        ""
      ],
      "Shona": [
        ""
      ],
      "Sogdien": [
        ""
      ],
      "Somalia kasa": [
        ""
      ],
      "Sranan Tongo": [
        ""
      ],
      "Serer": [
        ""
      ],
      "Swati": [
        ""
      ],
      "Saho": [
        ""
      ],
      "Saterland Frisian": [
        ""
      ],
      "Southern Sotho": [
        ""
      ],
      "Sukuma": [
        ""
      ],
      "Susu": [
        ""
      ],
      "Sundanese": [
        ""
      ],
      "Sumerian": [
        ""
      ],
      "Sweden kasa": [
        ""
      ],
      "Comorian": [
        ""
      ],
      "Congo Swahili": [
        ""
      ],
      "Swahili": [
        ""
      ],
      "Classical Syriac": [
        ""
      ],
      "Syriac": [
        ""
      ],
      "Silesian": [
        ""
      ],
      "Tamil kasa": [
        ""
      ],
      "Tulu": [
        ""
      ],
      "Timne": [
        ""
      ],
      "Teso": [
        ""
      ],
      "Tereno": [
        ""
      ],
      "Telugu": [
        ""
      ],
      "Tetum": [
        ""
      ],
      "Taeland kasa": [
        ""
      ],
      "Tigre": [
        ""
      ],
      "Tigrinya": [
        ""
      ],
      "Tiv": [
        ""
      ],
      "Tokelau": [
        ""
      ],
      "Tsakhur": [
        ""
      ],
      "Turkmen": [
        ""
      ],
      "Klingon": [
        ""
      ],
      "Tlingit": [
        ""
      ],
      "Tagalog": [
        ""
      ],
      "Talysh": [
        ""
      ],
      "Tamashek": [
        ""
      ],
      "Tswana": [
        ""
      ],
      "Nyasa Tonga": [
        ""
      ],
      "Tok Pisin": [
        ""
      ],
      "Tɛɛki kasa": [
        ""
      ],
      "Turoyo": [
        ""
      ],
      "Taroko": [
        ""
      ],
      "Tsakonian": [
        ""
      ],
      "Tsimshian": [
        ""
      ],
      "Tsonga": [
        ""
      ],
      "Tatar": [
        ""
      ],
      "Muslim Tat": [
        ""
      ],
      "Tumbuka": [
        ""
      ],
      "Tasawaq": [
        ""
      ],
      "Twi": [
        ""
      ],
      "Tahitian": [
        ""
      ],
      "Tuvinian": [
        ""
      ],
      "Central Atlas Tamazight": [
        ""
      ],
      "Udmurt": [
        ""
      ],
      "Ugaritic": [
        ""
      ],
      "Uyghur": [
        ""
      ],
      "Ukren kasa": [
        ""
      ],
      "Umbundu": [
        ""
      ],
      "Unknown Language": [
        ""
      ],
      "Urdu kasa": [
        ""
      ],
      "Uzbek": [
        ""
      ],
      "Vai": [
        ""
      ],
      "Venetian": [
        ""
      ],
      "Veps": [
        ""
      ],
      "Venda": [
        ""
      ],
      "Viɛtnam kasa": [
        ""
      ],
      "West Flemish": [
        ""
      ],
      "Main-Franconian": [
        ""
      ],
      "Votic": [
        ""
      ],
      "Volapük": [
        ""
      ],
      "Võro": [
        ""
      ],
      "Vunjo": [
        ""
      ],
      "Walser": [
        ""
      ],
      "Wolaytta": [
        ""
      ],
      "Waray": [
        ""
      ],
      "Washo": [
        ""
      ],
      "Walloon": [
        ""
      ],
      "Warlpiri": [
        ""
      ],
      "Wolof": [
        ""
      ],
      "Wu Chinese": [
        ""
      ],
      "Kalmyk": [
        ""
      ],
      "Xhosa": [
        ""
      ],
      "Mingrelian": [
        ""
      ],
      "Soga": [
        ""
      ],
      "Yao": [
        ""
      ],
      "Yapese": [
        ""
      ],
      "Yangben": [
        ""
      ],
      "Yemba": [
        ""
      ],
      "Yiddish": [
        ""
      ],
      "Yoruba": [
        ""
      ],
      "Nheengatu": [
        ""
      ],
      "Cantonese": [
        ""
      ],
      "Zapotec": [
        ""
      ],
      "Zhuang": [
        ""
      ],
      "Blissymbols": [
        ""
      ],
      "Zeelandic": [
        ""
      ],
      "Zenaga": [
        ""
      ],
      "Standard Moroccan Tamazight": [
        ""
      ],
      "Simplified Chinese": [
        ""
      ],
      "Traditional Chinese": [
        ""
      ],
      "Kyaena kasa": [
        ""
      ],
      "Zuni": [
        ""
      ],
      "Zulu": [
        ""
      ],
      "No linguistic content": [
        ""
      ],
      "Zaza": [
        ""
      ],
      "Could not load user attention request": [
        ""
      ],
      "No notification left": [
        ""
      ],
      "Ignore": [
        ""
      ],
      "Invalid dev experiment URI.": [
        ""
      ],
      "URI: %1$s": [
        ""
      ],
      "Experiment": [
        ""
      ],
      "Insert lost denomination": [
        ""
      ],
      "Apply": [
        ""
      ],
      "Pending refresh": [
        ""
      ],
      "Could not load the transfer payment status": [
        ""
      ],
      "Could not load the status of the order template": [
        ""
      ],
      "required": [
        ""
      ],
      "Review order": [
        ""
      ],
      "Missing URI": [
        ""
      ],
      "Check the link.": [
        ""
      ],
      "Could not parse the recovery URI": [
        ""
      ],
      "Try another URI": [
        ""
      ],
      "Import backup, show info": [
        ""
      ],
      "Add new exchange": [
        ""
      ],
      "Add exchange for %1$s": [
        ""
      ],
      "Enter the URL of an exchange you trust.": [
        ""
      ],
      "An exchange has been found! Review the information and click next": [
        ""
      ],
      "This exchange is already in your list.": [
        ""
      ],
      "Only exchange accessible through \"http\" and \"https\" are allowed.": [
        ""
      ],
      "This exchange protocol version is not supported: \" %1$s\".": [
        ""
      ],
      "This exchange currency \"%1$s\" doesn&apos;t match the expected currency %2$s.": [
        ""
      ],
      "No exchange found in that URL.": [
        ""
      ],
      "loading": [
        ""
      ],
      "Version": [
        ""
      ],
      "Can't use the URL: \"%1$s\"": [
        ""
      ],
      "Exchange URL": [
        ""
      ],
      "Add new contact": [
        ""
      ],
      "Enter the details of a contact you trust.": [
        ""
      ],
      "Contact info valid.": [
        ""
      ],
      "This contact is already in your list.": [
        ""
      ],
      "Review contact": [
        ""
      ],
      "Contact alias": [
        ""
      ],
      "Alias type": [
        ""
      ],
      "Mailbox service": [
        ""
      ],
      "Mailbox address": [
        ""
      ],
      "Contact source": [
        ""
      ],
      "Could not load the list of contacts": [
        ""
      ],
      "Source": [
        ""
      ],
      "Send Cash": [
        ""
      ],
      "Request Cash": [
        ""
      ],
      "No contacts found.": [
        ""
      ],
      "Could not bank choices for account": [
        ""
      ],
      "Could not load the list of messages": [
        ""
      ],
      "Unexpected error when trying to initialize mailbox: %1$s": [
        ""
      ],
      "Unexpected error when trying to delete message: %1$s": [
        ""
      ],
      "Unexpected error when trying to fetch messages: %1$s": [
        ""
      ],
      "Date": [
        ""
      ],
      "You may copy your mailbox URI from below and register it at %1$s with your alias(es) to make it easier for others to find you.": [
        ""
      ],
      "copy uri": [
        ""
      ],
      "hide uri": [
        ""
      ],
      "show uri": [
        ""
      ],
      "Fetch messages": [
        ""
      ],
      "Mailbox not initialized": [
        ""
      ],
      "Initialize mailbox": [
        ""
      ],
      "No messages yet.": [
        ""
      ],
      "Taler URI handler": [
        ""
      ],
      "Could not found a handler for the Taler URI": [
        ""
      ],
      "The uri read in the path parameter is not valid: \"%1$s\"": [
        ""
      ],
      "Digital cash payment": [
        ""
      ],
      "Digital cash refund": [
        ""
      ],
      "Digital cash withdrawal": [
        ""
      ],
      "Digital cash deposit": [
        ""
      ],
      "Digital cash invoice": [
        ""
      ],
      "Digital cash transfer": [
        ""
      ],
      "Digital cash recovery": [
        ""
      ],
      "Development experiment": [
        ""
      ],
      "Return to wallet": [
        ""
      ],
      "Network is offline": [
        ""
      ],
      "Diagnostics timed out. Could not talk to the wallet backend.": [
        ""
      ],
      "Problems detected:": [
        ""
      ],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).": [
        ""
      ],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.": [
        ""
      ],
      "here": [
        ""
      ],
      "Running diagnostics": [
        ""
      ],
      "Edit": [
        ""
      ],
      "List of products": [
        ""
      ],
      "free": [
        ""
      ],
      "Could not get provider information": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "ja",
  "completeness": 0
};

strings['it'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "it"
      },
      "Balance": [
        "Saldo"
      ],
      "Backup": [
        ""
      ],
      "Notifications": [
        ""
      ],
      "QR Reader and Taler URI": [
        "Lettore QR e Taler URI"
      ],
      "Settings": [
        "Impostazioni"
      ],
      "Dev tools": [
        ""
      ],
      "Contacts": [
        ""
      ],
      "Mailbox": [
        ""
      ],
      "Search transactions": [
        ""
      ],
      "%1$s": [
        ""
      ],
      "KYC requirement": [
        ""
      ],
      "PENDING OPERATIONS": [
        ""
      ],
      "Error was thrown trying to: \"%1$s\"": [
        ""
      ],
      "Could not open the wallet database. Firefox is known to run into this problem under \"permanent private mode\".": [
        ""
      ],
      "Could not open the wallet database.": [
        ""
      ],
      "Unexpected request error, code: %1$s": [
        ""
      ],
      "more info": [
        ""
      ],
      "wrap text": [
        ""
      ],
      "copy content": [
        ""
      ],
      "less info": [
        ""
      ],
      "Retry operation": [
        ""
      ],
      "Loading": [
        ""
      ],
      "Could not load backup providers": [
        ""
      ],
      "No backup providers configured": [
        ""
      ],
      "Add provider": [
        ""
      ],
      "Sync all backups": [
        ""
      ],
      "Sync now": [
        ""
      ],
      "Last synced": [
        ""
      ],
      "Not synced": [
        ""
      ],
      "Expires in": [
        ""
      ],
      "show less": [
        ""
      ],
      "show more": [
        ""
      ],
      "There was an error loading the provider detail for &quot;%1$s&quot;": [
        ""
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [
        ""
      ],
      "See providers": [
        ""
      ],
      "Last backup": [
        ""
      ],
      "Back up": [
        ""
      ],
      "Provider fee": [
        ""
      ],
      "per year": [
        ""
      ],
      "Extend": [
        ""
      ],
      "terms has changed, extending the service will imply accepting the new terms of service": [
        ""
      ],
      "old": [
        ""
      ],
      "new": [
        ""
      ],
      "fee": [
        ""
      ],
      "storage": [
        ""
      ],
      "Remove provider": [
        ""
      ],
      "Pay": [
        ""
      ],
      "Withdraw": [
        "Prelevare"
      ],
      "This provider has reported an error": [
        ""
      ],
      "There is conflict with another backup from &quot;%1$s&quot;": [
        ""
      ],
      "Backup is not readable": [
        ""
      ],
      "Unknown backup problem: %1$s": [
        ""
      ],
      "service paid": [
        ""
      ],
      "Backup valid until": [
        ""
      ],
      "Cancel": [
        ""
      ],
      "Open pay page": [
        ""
      ],
      "Open refund page": [
        ""
      ],
      "Open withdraw page": [
        ""
      ],
      "Your wallet is empty.": [
        ""
      ],
      "Get digital cash": [
        ""
      ],
      "Try the demo bank and withdraw test money.": [
        ""
      ],
      "Could not load the balance": [
        ""
      ],
      "Receive": [
        ""
      ],
      "Send %1$s": [
        ""
      ],
      "This page has a withdrawal action.": [
        ""
      ],
      "This page has pay action.": [
        ""
      ],
      "This page has a refund action.": [
        ""
      ],
      "This page has a add exchange action.": [
        ""
      ],
      "Open add exchange page": [
        ""
      ],
      "This page has an add contact action.": [
        ""
      ],
      "Open add contact page": [
        ""
      ],
      "Taler Action": [
        ""
      ],
      "This page has a malformed taler uri.": [
        ""
      ],
      "Dismiss": [
        ""
      ],
      "this popup is being closed and you are being redirected to %1$s": [
        ""
      ],
      "Could not load purchase proposal details": [
        ""
      ],
      "Order Id": [
        ""
      ],
      "Summary": [
        ""
      ],
      "Amount": [
        "Importo"
      ],
      "Choices": [
        ""
      ],
      "This purchase have %1$s choices": [
        ""
      ],
      "Merchant name": [
        ""
      ],
      "Merchant jurisdiction": [
        ""
      ],
      "Merchant address": [
        ""
      ],
      "Merchant logo": [
        ""
      ],
      "Merchant website": [
        ""
      ],
      "Merchant email": [
        ""
      ],
      "Merchant public key": [
        ""
      ],
      "Created at": [
        ""
      ],
      "Refund deadline": [
        ""
      ],
      "Auto refund": [
        ""
      ],
      "Pay deadline": [
        ""
      ],
      "Fulfillment URL": [
        ""
      ],
      "Fulfillment message": [
        ""
      ],
      "Max deposit fee": [
        ""
      ],
      "Exchanges": [
        ""
      ],
      "Could not load the status of the term of service": [
        ""
      ],
      "Select one option": [
        ""
      ],
      "Show terms of service": [
        ""
      ],
      "I accept the exchange terms of service": [
        ""
      ],
      "Review exchange terms of service": [
        ""
      ],
      "The exchange doesn't have a terms of service.": [
        ""
      ],
      "Skip it for now.": [
        ""
      ],
      "Format": [
        ""
      ],
      "Language": [
        ""
      ],
      "The exchange replied with a empty terms of service": [
        ""
      ],
      "No terms of service. The exchange replied with a empty document": [
        ""
      ],
      "No terms of service. The exchange replied with a empty text": [
        ""
      ],
      "Download Terms of Service": [
        ""
      ],
      "Hide terms of service": [
        ""
      ],
      "Amount is not valid": [
        ""
      ],
      "Bank account": [
        ""
      ],
      "Bitcoin address": [
        ""
      ],
      "BIC/IBAN": [
        ""
      ],
      "IBAN": [
        ""
      ],
      "Bitcoin transfer details": [
        ""
      ],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.": [
        ""
      ],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts": [
        ""
      ],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC": [
        ""
      ],
      "Ethereum transfer details": [
        ""
      ],
      "You need to wire to the service provider account.": [
        ""
      ],
      "Make sure the amount show %1$s ETH": [
        ""
      ],
      "Bank transfer details": [
        ""
      ],
      "Account": [
        "Conto"
      ],
      "Bank host": [
        ""
      ],
      "BIC": [
        ""
      ],
      "Exchange": [
        "Cambio"
      ],
      "Reserve Pub": [
        ""
      ],
      "Step 1:": [
        ""
      ],
      "Copy this code and paste it into the subject/purpose field in your banking app or bank website": [
        ""
      ],
      "Subject": [
        "Soggetto"
      ],
      "Step 2:": [
        ""
      ],
      "If you don't already have it in your banking favourites list, then copy and paste this IBAN and the name into the receiver fields in your banking app or website": [
        ""
      ],
      "Receiver name": [
        ""
      ],
      "Receiver postal code": [
        ""
      ],
      "Receiver town": [
        ""
      ],
      "Step 3:": [
        ""
      ],
      "Finish the wire transfer setting the amount in your banking app or website, then this withdrawal will proceed automatically.": [
        ""
      ],
      "Make sure ALL data is correct, including the subject; otherwise, the money will not arrive in this wallet. You can use the copy buttons (%1$s) to prevent typing errors or the \"payto://\" URI below to copy just one value.": [
        ""
      ],
      "Alternative if your bank already supports PayTo URI, you can use this %1$s link instead": [
        ""
      ],
      "Continue with banking app or website": [
        ""
      ],
      "Show QR code": [
        ""
      ],
      "Could not load transaction information": [
        ""
      ],
      "Bank account confirmation required for the transaction to complete.": [
        ""
      ],
      "KYC check required for the transaction to complete.": [
        ""
      ],
      "No additional information has been provided.": [
        ""
      ],
      "Follow this link to the%1$s %2$s": [
        ""
      ],
      "This transaction is pending to complete.": [
        ""
      ],
      "Retry": [
        ""
      ],
      "There was an error trying to complete the transaction.": [
        ""
      ],
      "This transaction was aborted.": [
        ""
      ],
      "This transaction failed.": [
        ""
      ],
      "Caution!": [
        ""
      ],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.": [
        ""
      ],
      "Confirm": [
        "Confermare"
      ],
      "Doing a cancellation while the transaction still active might result in lost coins. Do you still want to cancel the transaction?": [
        ""
      ],
      "No": [
        ""
      ],
      "Yes": [
        ""
      ],
      "Abort": [
        ""
      ],
      "Resume": [
        ""
      ],
      "Suspend": [
        ""
      ],
      "Delete": [
        ""
      ],
      "Withdrawal": [
        ""
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Select one of the accounts and use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Details": [
        ""
      ],
      "Withdrawal incomplete.": [
        ""
      ],
      "If you have already sent money to the service provider account it will wire it back at %1$s": [
        ""
      ],
      "Payment": [
        ""
      ],
      "Refunds": [
        ""
      ],
      "%1$s %2$s on %3$s .": [
        ""
      ],
      "Refund is in progress.": [
        ""
      ],
      "Merchant created a refund for this order but was not automatically picked up.": [
        ""
      ],
      "Offer": [
        ""
      ],
      "Accept": [
        ""
      ],
      "Confirmation code": [
        ""
      ],
      "Merchant": [
        ""
      ],
      "Invoice ID": [
        ""
      ],
      "Deposit": [
        ""
      ],
      "Wire transfer deadline.": [
        ""
      ],
      "Wire transfer still pending.": [
        ""
      ],
      "The service provider deadline to make the wire transfer is: %1$s": [
        ""
      ],
      "Wire transfer completed.": [
        ""
      ],
      " ": [
        ""
      ],
      "Transfer details": [
        ""
      ],
      "Wire transfer in progress.": [
        ""
      ],
      "Refresh": [
        ""
      ],
      "Refund": [
        ""
      ],
      "Purchase summary": [
        ""
      ],
      "Credit": [
        ""
      ],
      "Invoice": [
        ""
      ],
      "URI": [
        ""
      ],
      "Debit": [
        ""
      ],
      "Transfer": [
        ""
      ],
      "Lost": [
        ""
      ],
      "Reason": [
        ""
      ],
      "Denomination expired.": [
        ""
      ],
      "Denomination vanished.": [
        ""
      ],
      "Denomination is unoffered.": [
        ""
      ],
      "Fees": [
        ""
      ],
      "Total": [
        ""
      ],
      "Sent": [
        ""
      ],
      "Converted": [
        ""
      ],
      "Bank fee": [
        ""
      ],
      "Price": [
        ""
      ],
      "Subtotal": [
        ""
      ],
      "Refunded": [
        "Rimborsato"
      ],
      "Transfer identification": [
        ""
      ],
      "copy": [
        ""
      ],
      "hide qr": [
        ""
      ],
      "forward to contact": [
        ""
      ],
      "show qr": [
        ""
      ],
      "Wire transfer need a confirmation. Go to the %1$s and check wire transfer operation to exchange account is complete.": [
        ""
      ],
      "bank site": [
        ""
      ],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins.": [
        ""
      ],
      "Exchange is ready to send the coins, withdrawal in progress.": [
        ""
      ],
      "Send &nbsp; %1$s": [
        ""
      ],
      "Pay &nbsp; %1$s": [
        ""
      ],
      "Merchant message": [
        ""
      ],
      "Balance is not enough.": [
        ""
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which exchange and auditors the merchant trust.": [
        ""
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which wire methods the merchant accepts.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to pay for this contract which is restricted.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to spend right know. There are some coins that need to be refreshed.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s available.": [
        ""
      ],
      "Provider is missing the global fee configuration, this likely means it is misconfigured.": [
        ""
      ],
      "Balance looks like it should be enough, but doesn't cover all fees requested by the merchant and payment processor. Please ensure there is at least %1$s %2$s more balance in your wallet or ask your merchant to cover more of the fees.": [
        ""
      ],
      "Pay with a mobile phone": [
        ""
      ],
      "Hide QR": [
        ""
      ],
      "Scan the QR code or &nbsp; %1$s": [
        ""
      ],
      "click here": [
        ""
      ],
      "Could not load the payment and balance status": [
        ""
      ],
      "Purchase": [
        ""
      ],
      "This purchase is age restricted.": [
        ""
      ],
      "Expires at": [
        ""
      ],
      "Already paid, you are going to be redirected to %1$s": [
        ""
      ],
      "Already paid": [
        ""
      ],
      "Already claimed": [
        ""
      ],
      "Could not load the refund status": [
        ""
      ],
      "You&apos;ve ignored the refund.": [
        ""
      ],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [
        ""
      ],
      "Order amount": [
        ""
      ],
      "Could not load the list of exchanges": [
        ""
      ],
      "Could not load info from URI": [
        ""
      ],
      "Could not load the withdrawal details": [
        ""
      ],
      "under %1$s": [
        ""
      ],
      "should be greater than zero": [
        ""
      ],
      "can't be greater than %1$s": [
        ""
      ],
      "Could not load exchange details info": [
        ""
      ],
      "Close": [
        ""
      ],
      "There is no exchange available for currency %1$s": [
        ""
      ],
      "You can add more exchanges from the settings.": [
        ""
      ],
      "Exchange %1$s is not available": [
        ""
      ],
      "Exchange status can view accessed from the settings.": [
        ""
      ],
      "Service fee description": [
        ""
      ],
      "Select %1$s exchange": [
        ""
      ],
      "Reset": [
        ""
      ],
      "Use this exchange": [
        ""
      ],
      "Auditors": [
        ""
      ],
      "Doesn&apos;t have auditors": [
        ""
      ],
      "currency": [
        ""
      ],
      "Coin operations": [
        ""
      ],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.": [
        ""
      ],
      "Deposits": [
        ""
      ],
      "Denomination": [
        ""
      ],
      "Current": [
        ""
      ],
      "Selected": [
        ""
      ],
      "Until": [
        ""
      ],
      "Withdrawals": [
        ""
      ],
      "Transfer operations": [
        ""
      ],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.": [
        ""
      ],
      "Operation": [
        ""
      ],
      "Fee": [
        ""
      ],
      "Wallet operations": [
        ""
      ],
      "Feature": [
        ""
      ],
      "Currency": [
        ""
      ],
      "This operation has already been completed.": [
        ""
      ],
      "This operation has already been aborted": [
        ""
      ],
      "This operation has started and should be completed in the bank.": [
        ""
      ],
      "You can confirm the operation in": [
        ""
      ],
      "this page": [
        ""
      ],
      "This operation has already been completed by another wallet.": [
        ""
      ],
      "This operation has already been used by another wallet.": [
        ""
      ],
      "It can be confirmed in": [
        ""
      ],
      "Age restriction": [
        ""
      ],
      "Withdraw &nbsp; %1$s": [
        ""
      ],
      "Withdraw to a mobile phone": [
        ""
      ],
      "Short description of the invoice": [
        ""
      ],
      "Create": [
        ""
      ],
      "Short description of the transfer": [
        ""
      ],
      "Could not load the invoice payment status": [
        ""
      ],
      "Receive &nbsp; %1$s": [
        ""
      ],
      "Could not load known bank accounts": [
        ""
      ],
      "Could not load supported wire methods": [
        ""
      ],
      "No wire methods supported for this currency": [
        ""
      ],
      "Try another wire method.": [
        ""
      ],
      "Known accounts for %1$s": [
        ""
      ],
      "To add a new account first select the account type.": [
        ""
      ],
      "Unable add this account": [
        ""
      ],
      "Add": [
        ""
      ],
      "IBAN accounts": [
        "Conto"
      ],
      "Alias": [
        ""
      ],
      "Int. Account Number": [
        ""
      ],
      "Account name": [
        "Conto"
      ],
      "KYC": [
        ""
      ],
      "Taler accounts": [
        ""
      ],
      "Host": [
        ""
      ],
      "Bitcoin accounts": [
        ""
      ],
      "Address": [
        ""
      ],
      "Can't be empty": [
        ""
      ],
      "Bitcoin Account": [
        "Conto"
      ],
      "Taler Bank": [
        ""
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "Required": [
        ""
      ],
      "International Bank Account Number": [
        ""
      ],
      "Could not load balance information": [
        ""
      ],
      "Could not load fee for amount %1$s": [
        ""
      ],
      "A currency or an amount should be indicated": [
        ""
      ],
      "There is no enough balance to make a deposit for currency %1$s": [
        ""
      ],
      "Send %1$s to your account": [
        ""
      ],
      "There is no account to make a deposit for currency %1$s": [
        ""
      ],
      "Add account": [
        ""
      ],
      "Select account": [
        ""
      ],
      "Manage accounts": [
        ""
      ],
      "Brut amount": [
        ""
      ],
      "Deposit fee": [
        ""
      ],
      "Net amount": [
        "Importo"
      ],
      "Deposit&nbsp;%1$s %2$s": [
        ""
      ],
      "Need approval in the Bank": [
        ""
      ],
      "Waiting for wire transfer to complete": [
        ""
      ],
      "Exchange is waiting the wire transfer": [
        ""
      ],
      "Payment in progress": [
        ""
      ],
      "Executing refund...": [
        ""
      ],
      "Refreshing coins...": [
        ""
      ],
      "Deposit in progress": [
        ""
      ],
      "Waiting to be paid": [
        ""
      ],
      "Receiving the transfer": [
        ""
      ],
      "Waiting to be received": [
        ""
      ],
      "Denomination expired": [
        ""
      ],
      "Denomination vanished": [
        ""
      ],
      "Denomination unoffered": [
        ""
      ],
      "ABORTED": [
        ""
      ],
      "FAILED": [
        ""
      ],
      "Could not load the list of transactions": [
        ""
      ],
      "unknown scope": [
        ""
      ],
      "Send": [
        ""
      ],
      "Your transaction history is empty for this currency.": [
        ""
      ],
      "Review terms of service": [
        ""
      ],
      "Provider URL": [
        ""
      ],
      "Please review and accept this provider&apos;s terms of service": [
        ""
      ],
      "Pricing": [
        ""
      ],
      "free of charge": [
        ""
      ],
      "%1$s per year of service": [
        ""
      ],
      "Storage": [
        ""
      ],
      "%1$s megabytes of storage per year of service": [
        ""
      ],
      "Accept terms of service": [
        ""
      ],
      "Add backup provider": [
        ""
      ],
      "Backup providers may charge for their service": [
        ""
      ],
      "URL": [
        ""
      ],
      "Name": [
        ""
      ],
      "Next": [
        ""
      ],
      "Navigator": [
        ""
      ],
      "Automatically open wallet": [
        ""
      ],
      "Open the wallet when a payment action is found.": [
        ""
      ],
      "Version Info": [
        ""
      ],
      "Web Extension": [
        ""
      ],
      "The version of wallet core is not supported. (supported version: %1$s, wallet version: %2$s)": [
        ""
      ],
      "Exchange compatibility": [
        ""
      ],
      "Merchant compatibility": [
        ""
      ],
      "Bank compatibility": [
        ""
      ],
      "Wallet Core compatibility": [
        ""
      ],
      "Enable developer mode": [
        ""
      ],
      "Show more information and options in the UI": [
        ""
      ],
      "Display": [
        ""
      ],
      "Current Language": [
        ""
      ],
      "Show backup feature": [
        ""
      ],
      "Backup integration still in beta.": [
        ""
      ],
      "Show P2P alias feature": [
        ""
      ],
      "P2P aliases integration still in beta.": [
        ""
      ],
      "Show suspend/resume transaction": [
        ""
      ],
      "Prevent transaction from doing network request.": [
        ""
      ],
      "Show refresh transaction type in the transaction list": [
        ""
      ],
      "Refresh transaction will be hidden by default if the refresh operation doesn't have fee.": [
        ""
      ],
      "Show more account types on deposit": [
        ""
      ],
      "Extends the UI to more payment target types.": [
        ""
      ],
      "Show JSON on error": [
        ""
      ],
      "Print more information about the error. Useful for debugging.": [
        ""
      ],
      "Allow HTTP connections": [
        ""
      ],
      "Using HTTP connection may be faster but unsafe (wallet restart required)": [
        ""
      ],
      "Lang selector": [
        ""
      ],
      "Allows to manually change the language of the UI. Otherwise it will be automatically selected by your browser configuration.": [
        ""
      ],
      "Edit exchange management": [
        ""
      ],
      "Allows to see the list of exchange, remove, add and switch before withdrawal.": [
        ""
      ],
      "Select terms of service format": [
        ""
      ],
      "Allows to render the terms of service on different format selected by the user.": [
        ""
      ],
      "Show wallet activity": [
        ""
      ],
      "Show the wallet notification and observability event in the UI.": [
        ""
      ],
      "Enable v1 contract": [
        ""
      ],
      "v1 is under development and wallet-core converts v1 contract to v0. Enable this options for testing": [
        ""
      ],
      "GNU Taler Wallet installed!": [
        ""
      ],
      "You can open the wallet using the combination %1$s .": [
        ""
      ],
      "Also pinning the GNU Taler Wallet to your browser allows you to quick access without keyboard:": [
        ""
      ],
      "Click the puzzle icon": [
        ""
      ],
      "Search for GNU Taler Wallet": [
        ""
      ],
      "Click the pin icon": [
        ""
      ],
      "Next Steps": [
        ""
      ],
      "Try the demo": [
        ""
      ],
      "Click here to open the wallet activity tab.": [
        ""
      ],
      "Events": [
        ""
      ],
      "Active tasks": [
        ""
      ],
      "All events": [
        ""
      ],
      "Balance change": [
        "Saldo"
      ],
      "Backup failed": [
        ""
      ],
      "Transaction updated": [
        ""
      ],
      "Exchange updated": [
        "Cambio"
      ],
      "Idle": [
        ""
      ],
      "task.%1$s": [
        ""
      ],
      "wallet.%1$s(%2$s)": [
        ""
      ],
      "Bank account info changed": [
        ""
      ],
      "Contact added": [
        ""
      ],
      "Contact deleted": [
        ""
      ],
      "Mailbox message added": [
        ""
      ],
      "Mailbox message deleted": [
        ""
      ],
      "Type": [
        ""
      ],
      "Id": [
        ""
      ],
      "Since": [
        ""
      ],
      "Next try": [
        ""
      ],
      "Error": [
        ""
      ],
      "Transaction": [
        ""
      ],
      "No exchange yet": [
        ""
      ],
      "Status": [
        ""
      ],
      "Terms of Service": [
        ""
      ],
      "Last Update": [
        ""
      ],
      "Actions": [
        ""
      ],
      "ok": [
        "ok"
      ],
      "pending": [
        ""
      ],
      "proposed": [
        ""
      ],
      "unknown (exchange status should be updated)": [
        ""
      ],
      "Reload": [
        ""
      ],
      "Purge": [
        ""
      ],
      "Make regional": [
        ""
      ],
      "Make global": [
        ""
      ],
      "Forget ToS": [
        ""
      ],
      "Debug tools": [
        ""
      ],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?": [
        ""
      ],
      "reset": [
        ""
      ],
      "TESTING: This may delete all your coin, proceed with caution": [
        ""
      ],
      "run gc": [
        ""
      ],
      "import database": [
        ""
      ],
      "export database": [
        ""
      ],
      "Clear exchange key cache": [
        ""
      ],
      "Database exported at %1$s %2$s to download": [
        ""
      ],
      "Inject Taler support in all pages": [
        ""
      ],
      "Enabling this option will make `window.taler` be available in all sites": [
        ""
      ],
      "Exchange Entries": [
        "Cambio"
      ],
      "Add an exchange": [
        ""
      ],
      "Log levels": [
        ""
      ],
      "Coins": [
        ""
      ],
      "usable coins": [
        ""
      ],
      "id": [
        ""
      ],
      "denom": [
        ""
      ],
      "status": [
        ""
      ],
      "from refresh?": [
        ""
      ],
      "age key count": [
        ""
      ],
      "spent coins": [
        ""
      ],
      "click to show": [
        ""
      ],
      "URI is not valid. Taler URI should start with \"taler://\"": [
        ""
      ],
      "After the URI type it should follow a '/' with more information.": [
        ""
      ],
      "This URI type is not supported": [
        ""
      ],
      "This URI requires the bank host and operation id separated by /": [
        ""
      ],
      "This URI requires the merchant host, order id and session id separated by /": [
        ""
      ],
      "This URI requires the merchant host and order id separated by /": [
        ""
      ],
      "This URI requires the exchange host and key id separated by /": [
        ""
      ],
      "This URI requires the merchant host and template id separated by /": [
        ""
      ],
      "This URI requires the host key and provider list separated by /": [
        ""
      ],
      "This URI requires the experiment id": [
        ""
      ],
      "This URI requires the exchange host": [
        ""
      ],
      "This URI requires the alias type, the alias and the mailbox URI separated by /": [
        ""
      ],
      "This URI requires string after the first /": [
        ""
      ],
      "The bank host is invalid": [
        ""
      ],
      "The merchant host is invalid": [
        ""
      ],
      "The URI should end with /": [
        ""
      ],
      "The exchange host is invalid": [
        ""
      ],
      "The contact is invalid": [
        ""
      ],
      "The amount is invalid": [
        ""
      ],
      "A parameter is invalid": [
        ""
      ],
      "checking...": [
        ""
      ],
      "Unexpected error happen reading the camera: %1$s": [
        ""
      ],
      "Could not found a QR code in the file": [
        ""
      ],
      "Unexpected error happen reading the file: %1$s": [
        ""
      ],
      "Scan a QR code or enter taler:// URI below": [
        ""
      ],
      "Pay invoice": [
        ""
      ],
      "Withdrawal from bank": [
        ""
      ],
      "Claim refund": [
        ""
      ],
      "Accept payment": [
        ""
      ],
      "Complete order": [
        ""
      ],
      "Restore wallet": [
        ""
      ],
      "Enable experiment": [
        ""
      ],
      "Withdraw from exchange": [
        ""
      ],
      "Add exchange": [
        ""
      ],
      "Notify transaction": [
        ""
      ],
      "Add contact": [
        ""
      ],
      "The exchange URL is invalid.": [
        ""
      ],
      "The exchange URL should end with '/'": [
        ""
      ],
      "Couldn't found an exchange in the URL specified.": [
        ""
      ],
      "HTTP request failed to %1$s: %2$s": [
        ""
      ],
      "HTTP request failed to %1$s": [
        ""
      ],
      "The merchant URL is invalid.": [
        ""
      ],
      "The merchant URL should end with '/'": [
        ""
      ],
      "Couldn't found an merchant in the URL specified.": [
        ""
      ],
      "The bank URL is invalid.": [
        ""
      ],
      "The bank URL should end with '/'": [
        ""
      ],
      "Couldn't found an bank in the URL specified.": [
        ""
      ],
      "Could not load exchanges": [
        ""
      ],
      "Choose a currency to proceed or add another exchange": [
        ""
      ],
      "Known currencies": [
        ""
      ],
      "Specify the origin": [
        ""
      ],
      "Use previous origins:": [
        ""
      ],
      "From another bank account": [
        ""
      ],
      "From another wallet": [
        ""
      ],
      "From a %1$s URI or QR code": [
        ""
      ],
      "Enter URI here": [
        ""
      ],
      "Specify the destination": [
        ""
      ],
      "Use previous destinations:": [
        ""
      ],
      "To another bank account": [
        ""
      ],
      "To another wallet": [
        ""
      ],
      "To a %1$s URI or QR code": [
        ""
      ],
      "Afghanistan": [
        ""
      ],
      "Albania": [
        ""
      ],
      "Algeria": [
        ""
      ],
      "United States": [
        ""
      ],
      "Andorra": [
        ""
      ],
      "Angola": [
        ""
      ],
      "Antigua and Barbuda": [
        ""
      ],
      "Argentina": [
        ""
      ],
      "Armenia": [
        ""
      ],
      "Australia": [
        ""
      ],
      "Austria": [
        ""
      ],
      "Azerbaijan": [
        ""
      ],
      "Bahamas": [
        ""
      ],
      "Bahrain": [
        ""
      ],
      "Bangladesh": [
        ""
      ],
      "Barbados": [
        ""
      ],
      "Botswana": [
        ""
      ],
      "Belarus": [
        ""
      ],
      "Belgium": [
        ""
      ],
      "Belize": [
        ""
      ],
      "Benin": [
        ""
      ],
      "Bhutan": [
        ""
      ],
      "Bolivia": [
        ""
      ],
      "Bosnia and Herzegovina": [
        ""
      ],
      "Brazil": [
        ""
      ],
      "United Kingdom": [
        ""
      ],
      "Brunei": [
        ""
      ],
      "Bulgaria": [
        ""
      ],
      "Burkina Faso": [
        ""
      ],
      "Myanmar": [
        ""
      ],
      "Burundi": [
        ""
      ],
      "Cambodia": [
        ""
      ],
      "Cameroon": [
        ""
      ],
      "Canada": [
        ""
      ],
      "Cape Verde": [
        ""
      ],
      "Central African Republic": [
        ""
      ],
      "Chad": [
        ""
      ],
      "Chile": [
        ""
      ],
      "China": [
        ""
      ],
      "Colombia": [
        ""
      ],
      "Comoros": [
        ""
      ],
      "Democratic Republic of the Congo": [
        ""
      ],
      "Republic of the Congo": [
        ""
      ],
      "Costa Rica": [
        ""
      ],
      "Croatia": [
        ""
      ],
      "Cuba": [
        ""
      ],
      "Cyprus": [
        ""
      ],
      "Czech Republic": [
        ""
      ],
      "Denmark": [
        ""
      ],
      "Djibouti": [
        ""
      ],
      "Dominican Republic": [
        ""
      ],
      "Netherlands": [
        ""
      ],
      "Timor-Leste": [
        ""
      ],
      "Ecuador": [
        ""
      ],
      "Egypt": [
        ""
      ],
      "United Arab Emirates": [
        ""
      ],
      "Equatorial Guinea": [
        ""
      ],
      "Eritrea": [
        ""
      ],
      "Estonia": [
        ""
      ],
      "Ethiopia": [
        ""
      ],
      "Fiji": [
        ""
      ],
      "Philippines": [
        ""
      ],
      "Finland": [
        ""
      ],
      "France": [
        "Saldo"
      ],
      "Gabon": [
        ""
      ],
      "Gambia": [
        ""
      ],
      "Georgia": [
        ""
      ],
      "Germany": [
        ""
      ],
      "Ghana": [
        ""
      ],
      "Greece": [
        ""
      ],
      "Grenada": [
        ""
      ],
      "Guatemala": [
        ""
      ],
      "Guinea-Bissau": [
        ""
      ],
      "Guinea": [
        ""
      ],
      "Guyana": [
        ""
      ],
      "Haiti": [
        ""
      ],
      "Honduras": [
        ""
      ],
      "Hungary": [
        ""
      ],
      "Kiribati": [
        ""
      ],
      "Iceland": [
        ""
      ],
      "India": [
        ""
      ],
      "Indonesia": [
        ""
      ],
      "Iran": [
        ""
      ],
      "Iraq": [
        ""
      ],
      "Ireland": [
        ""
      ],
      "Israel": [
        ""
      ],
      "Italy": [
        ""
      ],
      "Ivory Coast": [
        ""
      ],
      "Jamaica": [
        ""
      ],
      "Japan": [
        ""
      ],
      "Jordan": [
        ""
      ],
      "Kazakhstan": [
        ""
      ],
      "Kenya": [
        ""
      ],
      "Saint Kitts and Nevis": [
        ""
      ],
      "Kuwait": [
        ""
      ],
      "Kyrgyzstan": [
        ""
      ],
      "Laos": [
        ""
      ],
      "Latvia": [
        ""
      ],
      "Lebanon": [
        ""
      ],
      "Liberia": [
        ""
      ],
      "Libya": [
        ""
      ],
      "Liechtenstein": [
        ""
      ],
      "Lithuania": [
        ""
      ],
      "Luxembourg": [
        ""
      ],
      "North Macedonia": [
        ""
      ],
      "Madagascar": [
        ""
      ],
      "Malawi": [
        ""
      ],
      "Malaysia": [
        ""
      ],
      "Maldives": [
        ""
      ],
      "Mali": [
        ""
      ],
      "Malta": [
        ""
      ],
      "Marshall Islands": [
        ""
      ],
      "Mauritania": [
        ""
      ],
      "Mauritius": [
        ""
      ],
      "Mexico": [
        ""
      ],
      "Micronesia": [
        ""
      ],
      "Moldova": [
        ""
      ],
      "Monaco": [
        ""
      ],
      "Mongolia": [
        ""
      ],
      "Morocco": [
        ""
      ],
      "Lesotho": [
        ""
      ],
      "Mozambique": [
        ""
      ],
      "Namibia": [
        ""
      ],
      "Nauru": [
        ""
      ],
      "Nepal": [
        ""
      ],
      "New Zealand": [
        ""
      ],
      "Vanuatu": [
        ""
      ],
      "Nicaragua": [
        ""
      ],
      "Nigeria": [
        ""
      ],
      "Niger": [
        ""
      ],
      "North Korea": [
        ""
      ],
      "Norway": [
        ""
      ],
      "Oman": [
        ""
      ],
      "Pakistan": [
        ""
      ],
      "Palau": [
        ""
      ],
      "Panama": [
        ""
      ],
      "Papua New Guinea": [
        ""
      ],
      "Paraguay": [
        ""
      ],
      "Peru": [
        ""
      ],
      "Poland": [
        ""
      ],
      "Portugal": [
        ""
      ],
      "Qatar": [
        ""
      ],
      "Romania": [
        ""
      ],
      "Russia": [
        ""
      ],
      "Rwanda": [
        ""
      ],
      "Saint Lucia": [
        ""
      ],
      "El Salvador": [
        ""
      ],
      "Samoa": [
        ""
      ],
      "San Marino": [
        ""
      ],
      "São Tomé and Príncipe": [
        ""
      ],
      "Saudi Arabia": [
        ""
      ],
      "Senegal": [
        ""
      ],
      "Serbia": [
        ""
      ],
      "Seychelles": [
        ""
      ],
      "Sierra Leone": [
        ""
      ],
      "Singapore": [
        ""
      ],
      "Slovakia": [
        ""
      ],
      "Slovenia": [
        ""
      ],
      "Solomon Islands": [
        ""
      ],
      "Somalia": [
        ""
      ],
      "South Africa": [
        ""
      ],
      "South Korea": [
        ""
      ],
      "Spain": [
        ""
      ],
      "Sri Lanka": [
        ""
      ],
      "Sudan": [
        ""
      ],
      "Suriname": [
        ""
      ],
      "Eswatini": [
        ""
      ],
      "Sweden": [
        ""
      ],
      "Switzerland": [
        ""
      ],
      "Syria": [
        ""
      ],
      "Taiwan": [
        ""
      ],
      "Tajikistan": [
        ""
      ],
      "Tanzania": [
        ""
      ],
      "Thailand": [
        ""
      ],
      "Togo": [
        ""
      ],
      "Tonga": [
        ""
      ],
      "Trinidad and Tobago": [
        ""
      ],
      "Tunisia": [
        ""
      ],
      "Turkey": [
        ""
      ],
      "Tuvalu": [
        ""
      ],
      "Uganda": [
        ""
      ],
      "Ukraine": [
        ""
      ],
      "Uruguay": [
        ""
      ],
      "Uzbekistan": [
        ""
      ],
      "Venezuela": [
        ""
      ],
      "Vietnam": [
        ""
      ],
      "Yemen": [
        ""
      ],
      "Zambia": [
        ""
      ],
      "Zimbabwe": [
        ""
      ],
      "Afghan": [
        ""
      ],
      "Albanian": [
        ""
      ],
      "Algerian": [
        ""
      ],
      "American": [
        ""
      ],
      "Andorran": [
        ""
      ],
      "Angolan": [
        ""
      ],
      "Antiguan": [
        ""
      ],
      "Argentinean": [
        ""
      ],
      "Armenian": [
        ""
      ],
      "Australian": [
        ""
      ],
      "Austrian": [
        ""
      ],
      "Azerbaijani": [
        ""
      ],
      "Bahamian": [
        ""
      ],
      "Bahraini": [
        ""
      ],
      "Bangladeshi": [
        ""
      ],
      "Barbadian": [
        ""
      ],
      "Batswana": [
        ""
      ],
      "Belarusian": [
        ""
      ],
      "Belgian": [
        ""
      ],
      "Belizean": [
        ""
      ],
      "Beninese": [
        ""
      ],
      "Bhutanese": [
        ""
      ],
      "Bolivian": [
        ""
      ],
      "Bosnian": [
        ""
      ],
      "Brazilian": [
        ""
      ],
      "British": [
        ""
      ],
      "Bruneian": [
        ""
      ],
      "Bulgarian": [
        ""
      ],
      "Burkinabe": [
        ""
      ],
      "Burmese": [
        ""
      ],
      "Burundian": [
        ""
      ],
      "Cambodian": [
        ""
      ],
      "Cameroonian": [
        ""
      ],
      "Canadian": [
        ""
      ],
      "Cape Verdean": [
        ""
      ],
      "Central African": [
        ""
      ],
      "Chadian": [
        ""
      ],
      "Chilean": [
        ""
      ],
      "Chinese": [
        ""
      ],
      "Colombian": [
        ""
      ],
      "Comoran": [
        ""
      ],
      "Congolese": [
        ""
      ],
      "Costa Rican": [
        ""
      ],
      "Croatian": [
        ""
      ],
      "Cuban": [
        ""
      ],
      "Cypriot": [
        ""
      ],
      "Czech": [
        ""
      ],
      "Danish": [
        ""
      ],
      "Djiboutian": [
        ""
      ],
      "Dominican": [
        ""
      ],
      "Dutch": [
        ""
      ],
      "East Timorese": [
        ""
      ],
      "Ecuadorean": [
        ""
      ],
      "Egyptian": [
        ""
      ],
      "Emirati": [
        ""
      ],
      "Equatorial Guinean": [
        ""
      ],
      "Eritrean": [
        ""
      ],
      "Estonian": [
        ""
      ],
      "Ethiopian": [
        ""
      ],
      "Fijian": [
        ""
      ],
      "Filipino": [
        ""
      ],
      "Finnish": [
        ""
      ],
      "French": [
        ""
      ],
      "Gabonese": [
        ""
      ],
      "Gambian": [
        ""
      ],
      "Georgian": [
        ""
      ],
      "German": [
        ""
      ],
      "Ghanaian": [
        ""
      ],
      "Greek": [
        ""
      ],
      "Grenadian": [
        ""
      ],
      "Guatemalan": [
        ""
      ],
      "Guinea-Bissauan": [
        ""
      ],
      "Guinean": [
        ""
      ],
      "Guyanese": [
        ""
      ],
      "Haitian": [
        ""
      ],
      "Honduran": [
        ""
      ],
      "Hungarian": [
        ""
      ],
      "I-Kiribati": [
        ""
      ],
      "Icelander": [
        ""
      ],
      "Indian": [
        ""
      ],
      "Indonesian": [
        ""
      ],
      "Iranian": [
        ""
      ],
      "Iraqi": [
        ""
      ],
      "Irish": [
        ""
      ],
      "Israeli": [
        ""
      ],
      "Italian": [
        ""
      ],
      "Ivorian": [
        ""
      ],
      "Jamaican": [
        ""
      ],
      "Japanese": [
        ""
      ],
      "Jordanian": [
        ""
      ],
      "Kazakhstani": [
        ""
      ],
      "Kenyan": [
        ""
      ],
      "Kittian and Nevisian": [
        ""
      ],
      "Kuwaiti": [
        ""
      ],
      "Kyrgyz": [
        ""
      ],
      "Laotian": [
        ""
      ],
      "Latvian": [
        ""
      ],
      "Lebanese": [
        ""
      ],
      "Liberian": [
        ""
      ],
      "Libyan": [
        ""
      ],
      "Liechtensteiner": [
        ""
      ],
      "Lithuanian": [
        ""
      ],
      "Luxembourger": [
        ""
      ],
      "Macedonian": [
        ""
      ],
      "Malagasy": [
        ""
      ],
      "Malawian": [
        ""
      ],
      "Malaysian": [
        ""
      ],
      "Maldivian": [
        ""
      ],
      "Malian": [
        ""
      ],
      "Maltese": [
        ""
      ],
      "Marshallese": [
        ""
      ],
      "Mauritanian": [
        ""
      ],
      "Mauritian": [
        ""
      ],
      "Mexican": [
        ""
      ],
      "Micronesian": [
        ""
      ],
      "Moldovan": [
        ""
      ],
      "Monacan": [
        ""
      ],
      "Mongolian": [
        ""
      ],
      "Moroccan": [
        ""
      ],
      "Mosotho": [
        ""
      ],
      "Mozambican": [
        ""
      ],
      "Namibian": [
        ""
      ],
      "Nauruan": [
        ""
      ],
      "Nepalese": [
        ""
      ],
      "New Zealander": [
        ""
      ],
      "Ni-Vanuatu": [
        ""
      ],
      "Nicaraguan": [
        ""
      ],
      "Nigerian": [
        ""
      ],
      "Nigerien": [
        ""
      ],
      "North Korean": [
        ""
      ],
      "Norwegian": [
        ""
      ],
      "Omani": [
        ""
      ],
      "Pakistani": [
        ""
      ],
      "Palauan": [
        ""
      ],
      "Panamanian": [
        ""
      ],
      "Papua New Guinean": [
        ""
      ],
      "Paraguayan": [
        ""
      ],
      "Peruvian": [
        ""
      ],
      "Polish": [
        ""
      ],
      "Portuguese": [
        ""
      ],
      "Qatari": [
        ""
      ],
      "Romanian": [
        ""
      ],
      "Russian": [
        ""
      ],
      "Rwandan": [
        ""
      ],
      "Saint Lucian": [
        ""
      ],
      "Salvadoran": [
        ""
      ],
      "Samoan": [
        ""
      ],
      "San Marinese": [
        ""
      ],
      "Sao Tomean": [
        ""
      ],
      "Saudi": [
        ""
      ],
      "Senegalese": [
        ""
      ],
      "Serbian": [
        ""
      ],
      "Seychellois": [
        ""
      ],
      "Sierra Leonean": [
        ""
      ],
      "Singaporean": [
        ""
      ],
      "Slovakian": [
        ""
      ],
      "Slovenian": [
        ""
      ],
      "Solomon Islander": [
        ""
      ],
      "Somali": [
        ""
      ],
      "South African": [
        ""
      ],
      "South Korean": [
        ""
      ],
      "Spanish": [
        ""
      ],
      "Sri Lankan": [
        ""
      ],
      "Sudanese": [
        ""
      ],
      "Surinamer": [
        ""
      ],
      "Swazi": [
        ""
      ],
      "Swedish": [
        ""
      ],
      "Swiss": [
        ""
      ],
      "Syrian": [
        ""
      ],
      "Taiwanese": [
        ""
      ],
      "Tajik": [
        ""
      ],
      "Tanzanian": [
        ""
      ],
      "Thai": [
        ""
      ],
      "Togolese": [
        ""
      ],
      "Tongan": [
        ""
      ],
      "Trinidadian or Tobagonian": [
        ""
      ],
      "Tunisian": [
        ""
      ],
      "Turkish": [
        ""
      ],
      "Tuvaluan": [
        ""
      ],
      "Ugandan": [
        ""
      ],
      "Ukrainian": [
        ""
      ],
      "Uruguayan": [
        ""
      ],
      "Uzbekistani": [
        ""
      ],
      "Venezuelan": [
        ""
      ],
      "Vietnamese": [
        ""
      ],
      "Yemenite": [
        ""
      ],
      "Zambian": [
        ""
      ],
      "Zimbabwean": [
        ""
      ],
      "🇦🇫": [
        ""
      ],
      "🇦🇱": [
        ""
      ],
      "🇩🇿": [
        ""
      ],
      "🇺🇸": [
        ""
      ],
      "🇦🇩": [
        ""
      ],
      "🇦🇴": [
        ""
      ],
      "🇦🇬": [
        ""
      ],
      "🇦🇷": [
        ""
      ],
      "🇦🇲": [
        ""
      ],
      "🇦🇺": [
        ""
      ],
      "🇦🇹": [
        ""
      ],
      "🇦🇿": [
        ""
      ],
      "🇧🇸": [
        ""
      ],
      "🇧🇭": [
        ""
      ],
      "🇧🇩": [
        ""
      ],
      "🇧🇧": [
        ""
      ],
      "🇧🇼": [
        ""
      ],
      "🇧🇾": [
        ""
      ],
      "🇧🇪": [
        ""
      ],
      "🇧🇿": [
        ""
      ],
      "🇧🇯": [
        ""
      ],
      "🇧🇹": [
        ""
      ],
      "🇧🇴": [
        ""
      ],
      "🇧🇦": [
        ""
      ],
      "🇧🇷": [
        ""
      ],
      "🇬🇧": [
        ""
      ],
      "🇧🇳": [
        ""
      ],
      "🇧🇬": [
        ""
      ],
      "🇧🇫": [
        ""
      ],
      "🇲🇲": [
        ""
      ],
      "🇧🇮": [
        ""
      ],
      "🇰🇭": [
        ""
      ],
      "🇨🇲": [
        ""
      ],
      "🇨🇦": [
        ""
      ],
      "🇨🇻": [
        ""
      ],
      "🇨🇫": [
        ""
      ],
      "🇨🇱": [
        ""
      ],
      "🇨🇳": [
        ""
      ],
      "🇨🇴": [
        ""
      ],
      "🇨🇩": [
        ""
      ],
      "🇨🇬": [
        ""
      ],
      "🇨🇷": [
        ""
      ],
      "🇨🇺": [
        ""
      ],
      "🇨🇾": [
        ""
      ],
      "🇨🇿": [
        ""
      ],
      "🇩🇰": [
        ""
      ],
      "🇩🇯": [
        ""
      ],
      "🇩🇴": [
        ""
      ],
      "🇳🇱": [
        ""
      ],
      "🇹🇱": [
        ""
      ],
      "🇪🇨": [
        ""
      ],
      "🇪🇬": [
        ""
      ],
      "🇦🇪": [
        ""
      ],
      "🇬🇶": [
        ""
      ],
      "🇪🇷": [
        ""
      ],
      "🇪🇪": [
        ""
      ],
      "🇪🇹": [
        ""
      ],
      "🇫🇯": [
        ""
      ],
      "🇵🇭": [
        ""
      ],
      "🇫🇮": [
        ""
      ],
      "🇫🇷": [
        ""
      ],
      "🇬🇦": [
        ""
      ],
      "🇬🇲": [
        ""
      ],
      "🇬🇪": [
        ""
      ],
      "UAE Dirham": [
        ""
      ],
      "Afghani": [
        ""
      ],
      "Lek": [
        ""
      ],
      "Armenian Dram": [
        ""
      ],
      "Netherlands Antillean Guilder": [
        ""
      ],
      "Kwanza": [
        ""
      ],
      "Argentine Peso": [
        ""
      ],
      "Australian Dollar": [
        ""
      ],
      "Aruban Florin": [
        ""
      ],
      "Azerbaijan Manat": [
        ""
      ],
      "Convertible Mark": [
        ""
      ],
      "Barbados Dollar": [
        ""
      ],
      "Taka": [
        ""
      ],
      "Bulgarian Lev": [
        ""
      ],
      "Bahraini Dinar": [
        ""
      ],
      "Burundi Franc": [
        ""
      ],
      "Bermudian Dollar": [
        ""
      ],
      "Brunei Dollar": [
        ""
      ],
      "Boliviano": [
        ""
      ],
      "Mvdol": [
        ""
      ],
      "Brazilian Real": [
        ""
      ],
      "Bahamian Dollar": [
        ""
      ],
      "Ngultrum": [
        ""
      ],
      "Pula": [
        ""
      ],
      "Belarusian Ruble": [
        ""
      ],
      "Belize Dollar": [
        ""
      ],
      "Canadian Dollar": [
        ""
      ],
      "Congolese Franc": [
        ""
      ],
      "WIR Euro": [
        ""
      ],
      "Swiss Franc": [
        ""
      ],
      "WIR Franc": [
        ""
      ],
      "Unidad de Fomento": [
        ""
      ],
      "Chilean Peso": [
        ""
      ],
      "Yuan Renminbi": [
        ""
      ],
      "Colombian Peso": [
        ""
      ],
      "Unidad de Valor Real": [
        ""
      ],
      "Costa Rican Colon": [
        ""
      ],
      "Cuban Peso": [
        ""
      ],
      "Cabo Verde Escudo": [
        ""
      ],
      "Czech Koruna": [
        ""
      ],
      "Djibouti Franc": [
        ""
      ],
      "Danish Krone": [
        ""
      ],
      "Dominican Peso": [
        ""
      ],
      "Algerian Dinar": [
        ""
      ],
      "Egyptian Pound": [
        ""
      ],
      "Nakfa": [
        ""
      ],
      "Ethiopian Birr": [
        ""
      ],
      "Euro": [
        ""
      ],
      "Fiji Dollar": [
        ""
      ],
      "Falkland Islands Pound": [
        ""
      ],
      "Pound Sterling": [
        ""
      ],
      "Lari": [
        ""
      ],
      "Ghana Cedi": [
        ""
      ],
      "Gibraltar Pound": [
        ""
      ],
      "Dalasi": [
        ""
      ],
      "Guinean Franc": [
        ""
      ],
      "Quetzal": [
        ""
      ],
      "Guyana Dollar": [
        ""
      ],
      "Hong Kong Dollar": [
        ""
      ],
      "Lempira": [
        ""
      ],
      "Gourde": [
        ""
      ],
      "Forint": [
        ""
      ],
      "Rupiah": [
        ""
      ],
      "New Israeli Sheqel": [
        ""
      ],
      "Indian Rupee": [
        ""
      ],
      "Iraqi Dinar": [
        ""
      ],
      "Iranian Rial": [
        ""
      ],
      "Iceland Krona": [
        ""
      ],
      "Jamaican Dollar": [
        ""
      ],
      "Jordanian Dinar": [
        ""
      ],
      "Yen": [
        ""
      ],
      "Kenyan Shilling": [
        ""
      ],
      "Som": [
        ""
      ],
      "Riel": [
        ""
      ],
      "Comorian Franc ": [
        ""
      ],
      "North Korean Won": [
        ""
      ],
      "Won": [
        ""
      ],
      "Kuwaiti Dinar": [
        ""
      ],
      "Cayman Islands Dollar": [
        ""
      ],
      "Tenge": [
        ""
      ],
      "Lao Kip": [
        ""
      ],
      "Lebanese Pound": [
        ""
      ],
      "Sri Lanka Rupee": [
        ""
      ],
      "Liberian Dollar": [
        ""
      ],
      "Loti": [
        ""
      ],
      "Libyan Dinar": [
        ""
      ],
      "Moroccan Dirham": [
        ""
      ],
      "Moldovan Leu": [
        ""
      ],
      "Malagasy Ariary": [
        ""
      ],
      "Denar": [
        ""
      ],
      "Kyat": [
        ""
      ],
      "Tugrik": [
        ""
      ],
      "Pataca": [
        ""
      ],
      "Ouguiya": [
        ""
      ],
      "Mauritius Rupee": [
        ""
      ],
      "Rufiyaa": [
        ""
      ],
      "Malawi Kwacha": [
        ""
      ],
      "Mexican Peso": [
        ""
      ],
      "Mexican Unidad de Inversion (UDI)": [
        ""
      ],
      "Malaysian Ringgit": [
        ""
      ],
      "Mozambique Metical": [
        ""
      ],
      "Namibia Dollar": [
        ""
      ],
      "Naira": [
        ""
      ],
      "Cordoba Oro": [
        ""
      ],
      "Norwegian Krone": [
        ""
      ],
      "Nepalese Rupee": [
        ""
      ],
      "New Zealand Dollar": [
        ""
      ],
      "Rial Omani": [
        ""
      ],
      "Balboa": [
        ""
      ],
      "Sol": [
        ""
      ],
      "Kina": [
        ""
      ],
      "Philippine Peso": [
        ""
      ],
      "Pakistan Rupee": [
        ""
      ],
      "Zloty": [
        ""
      ],
      "Guarani": [
        ""
      ],
      "Qatari Rial": [
        ""
      ],
      "Romanian Leu": [
        ""
      ],
      "Serbian Dinar": [
        ""
      ],
      "Russian Ruble": [
        ""
      ],
      "Rwanda Franc": [
        ""
      ],
      "Saudi Riyal": [
        ""
      ],
      "Solomon Islands Dollar": [
        ""
      ],
      "Seychelles Rupee": [
        ""
      ],
      "Sudanese Pound": [
        ""
      ],
      "Swedish Krona": [
        ""
      ],
      "Singapore Dollar": [
        ""
      ],
      "Saint Helena Pound": [
        ""
      ],
      "Leone": [
        ""
      ],
      "Somali Shilling": [
        ""
      ],
      "Surinam Dollar": [
        ""
      ],
      "South Sudanese Pound": [
        ""
      ],
      "Dobra": [
        ""
      ],
      "El Salvador Colon": [
        ""
      ],
      "Syrian Pound": [
        ""
      ],
      "Lilangeni": [
        ""
      ],
      "Baht": [
        ""
      ],
      "Somoni": [
        ""
      ],
      "Turkmenistan New Manat": [
        ""
      ],
      "Tunisian Dinar": [
        ""
      ],
      "Pa’anga": [
        ""
      ],
      "Turkish Lira": [
        ""
      ],
      "Trinidad and Tobago Dollar": [
        ""
      ],
      "New Taiwan Dollar": [
        ""
      ],
      "Tanzanian Shilling": [
        ""
      ],
      "Hryvnia": [
        ""
      ],
      "Uganda Shilling": [
        ""
      ],
      "US Dollar": [
        ""
      ],
      "US Dollar (Next day)": [
        ""
      ],
      "Uruguay Peso en Unidades Indexadas (UI)": [
        ""
      ],
      "Peso Uruguayo": [
        ""
      ],
      "Unidad Previsional": [
        ""
      ],
      "Uzbekistan Sum": [
        ""
      ],
      "Bolívar Soberano": [
        ""
      ],
      "Dong": [
        ""
      ],
      "Vatu": [
        ""
      ],
      "Tala": [
        ""
      ],
      "CFA Franc BEAC": [
        ""
      ],
      "East Caribbean Dollar": [
        ""
      ],
      "CFA Franc BCEAO": [
        ""
      ],
      "CFP Franc": [
        ""
      ],
      "Sucre": [
        ""
      ],
      "Yemeni Rial": [
        ""
      ],
      "Rand": [
        ""
      ],
      "Zambian Kwacha": [
        ""
      ],
      "Zimbabwe Gold": [
        ""
      ],
      "Afar": [
        ""
      ],
      "Abkhazian": [
        ""
      ],
      "Achinese": [
        ""
      ],
      "Acoli": [
        ""
      ],
      "Adangme": [
        ""
      ],
      "Adyghe": [
        ""
      ],
      "Avestan": [
        ""
      ],
      "Tunisian Arabic": [
        ""
      ],
      "Afrikaans": [
        ""
      ],
      "Afrihili": [
        ""
      ],
      "Aghem": [
        ""
      ],
      "Ainu": [
        ""
      ],
      "Akan": [
        ""
      ],
      "Akkadian": [
        ""
      ],
      "Alabama": [
        ""
      ],
      "Aleut": [
        ""
      ],
      "Gheg Albanian": [
        ""
      ],
      "Southern Altai": [
        ""
      ],
      "Amarik": [
        ""
      ],
      "Aragonese": [
        ""
      ],
      "Old English": [
        ""
      ],
      "Angika": [
        ""
      ],
      "Modern Standard Arabic": [
        ""
      ],
      "Arabik": [
        ""
      ],
      "Aramaic": [
        ""
      ],
      "Mapuche": [
        ""
      ],
      "Araona": [
        ""
      ],
      "Arapaho": [
        ""
      ],
      "Algerian Arabic": [
        ""
      ],
      "Arawak": [
        ""
      ],
      "Moroccan Arabic": [
        ""
      ],
      "Egyptian Arabic": [
        ""
      ],
      "Asu": [
        ""
      ],
      "Assamese": [
        ""
      ],
      "American Sign Language": [
        ""
      ],
      "Asturian": [
        ""
      ],
      "Avaric": [
        ""
      ],
      "Kotava": [
        ""
      ],
      "Awadhi": [
        ""
      ],
      "Aymara": [
        ""
      ],
      "South Azerbaijani": [
        ""
      ],
      "Bashkir": [
        ""
      ],
      "Baluchi": [
        ""
      ],
      "Balinese": [
        ""
      ],
      "Bavarian": [
        ""
      ],
      "Basaa": [
        ""
      ],
      "Bamun": [
        ""
      ],
      "Batak Toba": [
        ""
      ],
      "Ghomala": [
        ""
      ],
      "Belarus kasa": [
        ""
      ],
      "Beja": [
        ""
      ],
      "Bemba": [
        ""
      ],
      "Betawi": [
        ""
      ],
      "Bena": [
        ""
      ],
      "Bafut": [
        ""
      ],
      "Badaga": [
        ""
      ],
      "Bɔlgeria kasa": [
        ""
      ],
      "Bhojpuri": [
        ""
      ],
      "Bislama": [
        ""
      ],
      "Bikol": [
        ""
      ],
      "Bini": [
        ""
      ],
      "Banjar": [
        ""
      ],
      "Kom": [
        ""
      ],
      "Siksika": [
        ""
      ],
      "Bambara": [
        ""
      ],
      "Bengali kasa": [
        ""
      ],
      "Tibetan": [
        ""
      ],
      "Bishnupriya": [
        ""
      ],
      "Bakhtiari": [
        ""
      ],
      "Braj": [
        ""
      ],
      "Breton": [
        ""
      ],
      "Brahui": [
        ""
      ],
      "Bodo": [
        ""
      ],
      "Akoose": [
        ""
      ],
      "Buriat": [
        ""
      ],
      "Buginese": [
        ""
      ],
      "Bulu": [
        ""
      ],
      "Blin": [
        ""
      ],
      "Medumba": [
        ""
      ],
      "Catalan": [
        ""
      ],
      "Caddo": [
        ""
      ],
      "Carib": [
        ""
      ],
      "Cayuga": [
        ""
      ],
      "Atsam": [
        ""
      ],
      "Cebuano": [
        ""
      ],
      "Chechen": [
        ""
      ],
      "Chiga": [
        ""
      ],
      "Chibcha": [
        ""
      ],
      "Chamorro": [
        ""
      ],
      "Chagatai": [
        ""
      ],
      "Chuukese": [
        ""
      ],
      "Mari": [
        ""
      ],
      "Chinook Jargon": [
        ""
      ],
      "Choctaw": [
        ""
      ],
      "Chipewyan": [
        ""
      ],
      "Cherokee": [
        ""
      ],
      "Cheyenne": [
        ""
      ],
      "Central Kurdish": [
        ""
      ],
      "Corsican": [
        ""
      ],
      "Coptic": [
        ""
      ],
      "Capiznon": [
        ""
      ],
      "Cree": [
        ""
      ],
      "Crimean Turkish": [
        ""
      ],
      "Kashubian": [
        ""
      ],
      "Kyɛk kasa": [
        ""
      ],
      "Church Slavic": [
        ""
      ],
      "Chuvash": [
        ""
      ],
      "Welsh": [
        ""
      ],
      "Dakota": [
        ""
      ],
      "Dargwa": [
        ""
      ],
      "Taita": [
        ""
      ],
      "Austrian German": [
        ""
      ],
      "Swiss High German": [
        ""
      ],
      "Gyaaman": [
        ""
      ],
      "Delaware": [
        ""
      ],
      "Slave": [
        ""
      ],
      "Dogrib": [
        ""
      ],
      "Dinka": [
        ""
      ],
      "Zarma": [
        ""
      ],
      "Dogri": [
        ""
      ],
      "Lower Sorbian": [
        ""
      ],
      "Central Dusun": [
        ""
      ],
      "Duala": [
        ""
      ],
      "Middle Dutch": [
        ""
      ],
      "Divehi": [
        ""
      ],
      "Jola-Fonyi": [
        ""
      ],
      "Dyula": [
        ""
      ],
      "Dzongkha": [
        ""
      ],
      "Dazaga": [
        ""
      ],
      "Embu": [
        ""
      ],
      "Ewe": [
        ""
      ],
      "Efik": [
        ""
      ],
      "Emilian": [
        ""
      ],
      "Ancient Egyptian": [
        ""
      ],
      "Ekajuk": [
        ""
      ],
      "Greek kasa": [
        ""
      ],
      "Elamite": [
        ""
      ],
      "Australian English": [
        ""
      ],
      "Borɔfo": [
        ""
      ],
      "Canadian English": [
        ""
      ],
      "British English": [
        ""
      ],
      "Middle English": [
        ""
      ],
      "American English": [
        ""
      ],
      "Esperanto": [
        ""
      ],
      "Latin American Spanish": [
        ""
      ],
      "European Spanish": [
        ""
      ],
      "Mexican Spanish": [
        ""
      ],
      "Spain kasa": [
        ""
      ],
      "Central Yupik": [
        ""
      ],
      "Basque": [
        ""
      ],
      "Ewondo": [
        ""
      ],
      "Extremaduran": [
        ""
      ],
      "Fang": [
        ""
      ],
      "Pɛɛhyia kasa": [
        ""
      ],
      "Fanti": [
        ""
      ],
      "Fulah": [
        ""
      ],
      "Tornedalen Finnish": [
        ""
      ],
      "Faroese": [
        ""
      ],
      "Fon": [
        ""
      ],
      "Canadian French": [
        ""
      ],
      "Cajun French": [
        ""
      ],
      "Swiss French": [
        ""
      ],
      "Frɛnkye": [
        ""
      ],
      "Middle French": [
        ""
      ],
      "Old French": [
        ""
      ],
      "Arpitan": [
        ""
      ],
      "Northern Frisian": [
        ""
      ],
      "Eastern Frisian": [
        ""
      ],
      "Friulian": [
        ""
      ],
      "Western Frisian": [
        ""
      ],
      "Ga": [
        ""
      ],
      "Gagauz": [
        ""
      ],
      "Gan Chinese": [
        ""
      ],
      "Gayo": [
        ""
      ],
      "Gbaya": [
        ""
      ],
      "Zoroastrian Dari": [
        ""
      ],
      "Scottish Gaelic": [
        ""
      ],
      "Geez": [
        ""
      ],
      "Gilbertese": [
        ""
      ],
      "Galician": [
        ""
      ],
      "Gilaki": [
        ""
      ],
      "Middle High German": [
        ""
      ],
      "Old High German": [
        ""
      ],
      "Goan Konkani": [
        ""
      ],
      "Gondi": [
        ""
      ],
      "Gorontalo": [
        ""
      ],
      "Gothic": [
        ""
      ],
      "Grebo": [
        ""
      ],
      "Ancient Greek": [
        ""
      ],
      "Swiss German": [
        ""
      ],
      "Wayuu": [
        ""
      ],
      "Gujarati": [
        ""
      ],
      "Frafra": [
        ""
      ],
      "Gusii": [
        ""
      ],
      "Manx": [
        ""
      ],
      "Gwichʼin": [
        ""
      ],
      "Hausa": [
        ""
      ],
      "Haida": [
        ""
      ],
      "Hakka Chinese": [
        ""
      ],
      "Hawaiian": [
        ""
      ],
      "Hebrew": [
        ""
      ],
      "Fiji Hindi": [
        ""
      ],
      "Hindi": [
        ""
      ],
      "Hiligaynon": [
        ""
      ],
      "Hittite": [
        ""
      ],
      "Hmong": [
        ""
      ],
      "Hiri Motu": [
        ""
      ],
      "Upper Sorbian": [
        ""
      ],
      "Xiang Chinese": [
        ""
      ],
      "Hangri kasa": [
        ""
      ],
      "Hupa": [
        ""
      ],
      "Herero": [
        ""
      ],
      "Interlingua": [
        ""
      ],
      "Iban": [
        ""
      ],
      "Ibibio": [
        ""
      ],
      "Indonihyia kasa": [
        ""
      ],
      "Interlingue": [
        ""
      ],
      "Igbo": [
        ""
      ],
      "Sichuan Yi": [
        ""
      ],
      "Inupiaq": [
        ""
      ],
      "Iloko": [
        ""
      ],
      "Ingush": [
        ""
      ],
      "Ido": [
        ""
      ],
      "Icelandic": [
        ""
      ],
      "Italy kasa": [
        ""
      ],
      "Inuktitut": [
        ""
      ],
      "Ingrian": [
        ""
      ],
      "Gyapan kasa": [
        ""
      ],
      "Jamaican Creole English": [
        ""
      ],
      "Lojban": [
        ""
      ],
      "Ngomba": [
        ""
      ],
      "Machame": [
        ""
      ],
      "Judeo-Persian": [
        ""
      ],
      "Judeo-Arabic": [
        ""
      ],
      "Jutish": [
        ""
      ],
      "Gyabanis kasa": [
        ""
      ],
      "Kara-Kalpak": [
        ""
      ],
      "Kabyle": [
        ""
      ],
      "Kachin": [
        ""
      ],
      "Jju": [
        ""
      ],
      "Kamba": [
        ""
      ],
      "Kawi": [
        ""
      ],
      "Kabardian": [
        ""
      ],
      "Kanembu": [
        ""
      ],
      "Tyap": [
        ""
      ],
      "Makonde": [
        ""
      ],
      "Kabuverdianu": [
        ""
      ],
      "Kenyang": [
        ""
      ],
      "Koro": [
        ""
      ],
      "Kongo": [
        ""
      ],
      "Kaingang": [
        ""
      ],
      "Khasi": [
        ""
      ],
      "Khotanese": [
        ""
      ],
      "Koyra Chiini": [
        ""
      ],
      "Khowar": [
        ""
      ],
      "Kikuyu": [
        ""
      ],
      "Kirmanjki": [
        ""
      ],
      "Kuanyama": [
        ""
      ],
      "Kako": [
        ""
      ],
      "Kazakh": [
        ""
      ],
      "Kalaallisut": [
        ""
      ],
      "Kalenjin": [
        ""
      ],
      "Kimbundu": [
        ""
      ],
      "Kambodia kasa": [
        ""
      ],
      "Kannada": [
        ""
      ],
      "Komi-Permyak": [
        ""
      ],
      "Konkani": [
        ""
      ],
      "Korea kasa": [
        ""
      ],
      "Kosraean": [
        ""
      ],
      "Kpelle": [
        ""
      ],
      "Karachay-Balkar": [
        ""
      ],
      "Krio": [
        ""
      ],
      "Kinaray-a": [
        ""
      ],
      "Kanuri": [
        ""
      ],
      "Karelian": [
        ""
      ],
      "Kurukh": [
        ""
      ],
      "Shambala": [
        ""
      ],
      "Bafia": [
        ""
      ],
      "Colognian": [
        ""
      ],
      "Kashmiri": [
        ""
      ],
      "Kurdish": [
        ""
      ],
      "Kumyk": [
        ""
      ],
      "Kutenai": [
        ""
      ],
      "Komi": [
        ""
      ],
      "Cornish": [
        ""
      ],
      "Ladino": [
        ""
      ],
      "Langi": [
        ""
      ],
      "Lahnda": [
        ""
      ],
      "Latin": [
        ""
      ],
      "Lamba": [
        ""
      ],
      "Luxembourgish": [
        ""
      ],
      "Lezghian": [
        ""
      ],
      "Lingua Franca Nova": [
        ""
      ],
      "Ganda": [
        ""
      ],
      "Ligurian": [
        ""
      ],
      "Limburgish": [
        ""
      ],
      "Livonian": [
        ""
      ],
      "Lakota": [
        ""
      ],
      "Lombard": [
        ""
      ],
      "Lingala": [
        ""
      ],
      "Lao": [
        ""
      ],
      "Mongo": [
        ""
      ],
      "Lozi": [
        ""
      ],
      "Latgalian": [
        ""
      ],
      "Luba-Lulua": [
        ""
      ],
      "Luiseno": [
        ""
      ],
      "Luba-Katanga": [
        ""
      ],
      "Lunda": [
        ""
      ],
      "Luo": [
        ""
      ],
      "Mizo": [
        ""
      ],
      "Luyia": [
        ""
      ],
      "Literary Chinese": [
        ""
      ],
      "Laz": [
        ""
      ],
      "Madurese": [
        ""
      ],
      "Mafa": [
        ""
      ],
      "Magahi": [
        ""
      ],
      "Maithili": [
        ""
      ],
      "Makasar": [
        ""
      ],
      "Mandingo": [
        ""
      ],
      "Masai": [
        ""
      ],
      "Maba": [
        ""
      ],
      "Moksha": [
        ""
      ],
      "Mandar": [
        ""
      ],
      "Mende": [
        ""
      ],
      "Meru": [
        ""
      ],
      "Morisyen": [
        ""
      ],
      "Middle Irish": [
        ""
      ],
      "Makhuwa-Meetto": [
        ""
      ],
      "Metaʼ": [
        ""
      ],
      "Micmac": [
        ""
      ],
      "Maori": [
        ""
      ],
      "Minangkabau": [
        ""
      ],
      "Malayalam": [
        ""
      ],
      "Manchu": [
        ""
      ],
      "Manipuri": [
        ""
      ],
      "Mohawk": [
        ""
      ],
      "Mossi": [
        ""
      ],
      "Western Mari": [
        ""
      ],
      "Marathi": [
        ""
      ],
      "Malay kasa": [
        ""
      ],
      "Mundang": [
        ""
      ],
      "Multiple Languages": [
        ""
      ],
      "Creek": [
        ""
      ],
      "Mirandese": [
        ""
      ],
      "Marwari": [
        ""
      ],
      "Mentawai": [
        ""
      ],
      "Bɛɛmis kasa": [
        ""
      ],
      "Myene": [
        ""
      ],
      "Erzya": [
        ""
      ],
      "Mazanderani": [
        ""
      ],
      "Min Nan Chinese": [
        ""
      ],
      "Neapolitan": [
        ""
      ],
      "Nama": [
        ""
      ],
      "Norwegian Bokmål": [
        ""
      ],
      "North Ndebele": [
        ""
      ],
      "Low German": [
        ""
      ],
      "Nɛpal kasa": [
        ""
      ],
      "Newari": [
        ""
      ],
      "Ndonga": [
        ""
      ],
      "Nias": [
        ""
      ],
      "Niuean": [
        ""
      ],
      "Ao Naga": [
        ""
      ],
      "Flemish": [
        ""
      ],
      "Dɛɛkye": [
        ""
      ],
      "Kwasio": [
        ""
      ],
      "Ngiemboon": [
        ""
      ],
      "Norwegian Nynorsk": [
        ""
      ],
      "Nogai": [
        ""
      ],
      "Old Norse": [
        ""
      ],
      "Novial": [
        ""
      ],
      "NʼKo": [
        ""
      ],
      "South Ndebele": [
        ""
      ],
      "Northern Sotho": [
        ""
      ],
      "Nuer": [
        ""
      ],
      "Navajo": [
        ""
      ],
      "Classical Newari": [
        ""
      ],
      "Nyamwezi": [
        ""
      ],
      "Nyankole": [
        ""
      ],
      "Nyanja": [
        ""
      ],
      "Nyoro": [
        ""
      ],
      "Nzima": [
        ""
      ],
      "Occitan": [
        ""
      ],
      "Ojibwa": [
        ""
      ],
      "Oromo": [
        ""
      ],
      "Oriya": [
        ""
      ],
      "Osage": [
        ""
      ],
      "Ossetic": [
        ""
      ],
      "Ottoman Turkish": [
        ""
      ],
      "Pangasinan": [
        ""
      ],
      "Pahlavi": [
        ""
      ],
      "Pampanga": [
        ""
      ],
      "Papiamento": [
        ""
      ],
      "Pungyabi kasa": [
        ""
      ],
      "Picard": [
        ""
      ],
      "Pennsylvania German": [
        ""
      ],
      "Plautdietsch": [
        ""
      ],
      "Old Persian": [
        ""
      ],
      "Palatine German": [
        ""
      ],
      "Phoenician": [
        ""
      ],
      "Pali": [
        ""
      ],
      "Pɔland kasa": [
        ""
      ],
      "Piedmontese": [
        ""
      ],
      "Pontic": [
        ""
      ],
      "Pohnpeian": [
        ""
      ],
      "Prussian": [
        ""
      ],
      "Old Provençal": [
        ""
      ],
      "Pashto": [
        ""
      ],
      "Brazilian Portuguese": [
        ""
      ],
      "Pɔɔtugal kasa": [
        ""
      ],
      "European Portuguese": [
        ""
      ],
      "Kʼicheʼ": [
        ""
      ],
      "Chimborazo Highland Quichua": [
        ""
      ],
      "Quechua": [
        ""
      ],
      "Rajasthani": [
        ""
      ],
      "Rapanui": [
        ""
      ],
      "Rarotongan": [
        ""
      ],
      "Romagnol": [
        ""
      ],
      "Riffian": [
        ""
      ],
      "Romansh": [
        ""
      ],
      "Rundi": [
        ""
      ],
      "Rombo": [
        ""
      ],
      "Moldavian": [
        ""
      ],
      "Romany": [
        ""
      ],
      "Root": [
        ""
      ],
      "Romenia kasa": [
        ""
      ],
      "Rotuman": [
        ""
      ],
      "Rusyn": [
        ""
      ],
      "Roviana": [
        ""
      ],
      "Aromanian": [
        ""
      ],
      "Rahyia kasa": [
        ""
      ],
      "Rwa": [
        ""
      ],
      "Rewanda kasa": [
        ""
      ],
      "Sandawe": [
        ""
      ],
      "Sakha": [
        ""
      ],
      "Samaritan Aramaic": [
        ""
      ],
      "Samburu": [
        ""
      ],
      "Sanskrit": [
        ""
      ],
      "Sasak": [
        ""
      ],
      "Santali": [
        ""
      ],
      "Saurashtra": [
        ""
      ],
      "Ngambay": [
        ""
      ],
      "Sangu": [
        ""
      ],
      "Sicilian": [
        ""
      ],
      "Scots": [
        ""
      ],
      "Sardinian": [
        ""
      ],
      "Sassarese Sardinian": [
        ""
      ],
      "Sindhi": [
        ""
      ],
      "Seneca": [
        ""
      ],
      "Sena": [
        ""
      ],
      "Seri": [
        ""
      ],
      "Selkup": [
        ""
      ],
      "Northern Sami": [
        ""
      ],
      "Koyraboro Senni": [
        ""
      ],
      "Old Irish": [
        ""
      ],
      "Sango": [
        ""
      ],
      "Samogitian": [
        ""
      ],
      "Tachelhit": [
        ""
      ],
      "Shan": [
        ""
      ],
      "Serbo-Croatian": [
        ""
      ],
      "Chadian Arabic": [
        ""
      ],
      "Sidamo": [
        ""
      ],
      "Sinhala": [
        ""
      ],
      "Slovak": [
        ""
      ],
      "Lower Silesian": [
        ""
      ],
      "Selayar": [
        ""
      ],
      "Southern Sami": [
        ""
      ],
      "Lule Sami": [
        ""
      ],
      "Inari Sami": [
        ""
      ],
      "Skolt Sami": [
        ""
      ],
      "Soninke": [
        ""
      ],
      "Shona": [
        ""
      ],
      "Sogdien": [
        ""
      ],
      "Somalia kasa": [
        ""
      ],
      "Sranan Tongo": [
        ""
      ],
      "Serer": [
        ""
      ],
      "Swati": [
        ""
      ],
      "Saho": [
        ""
      ],
      "Saterland Frisian": [
        ""
      ],
      "Southern Sotho": [
        ""
      ],
      "Sukuma": [
        ""
      ],
      "Susu": [
        ""
      ],
      "Sundanese": [
        ""
      ],
      "Sumerian": [
        ""
      ],
      "Sweden kasa": [
        ""
      ],
      "Comorian": [
        ""
      ],
      "Congo Swahili": [
        ""
      ],
      "Swahili": [
        ""
      ],
      "Classical Syriac": [
        ""
      ],
      "Syriac": [
        ""
      ],
      "Silesian": [
        ""
      ],
      "Tamil kasa": [
        ""
      ],
      "Tulu": [
        ""
      ],
      "Timne": [
        ""
      ],
      "Teso": [
        ""
      ],
      "Tereno": [
        ""
      ],
      "Telugu": [
        ""
      ],
      "Tetum": [
        ""
      ],
      "Taeland kasa": [
        ""
      ],
      "Tigre": [
        ""
      ],
      "Tigrinya": [
        ""
      ],
      "Tiv": [
        ""
      ],
      "Tokelau": [
        ""
      ],
      "Tsakhur": [
        ""
      ],
      "Turkmen": [
        ""
      ],
      "Klingon": [
        ""
      ],
      "Tlingit": [
        ""
      ],
      "Tagalog": [
        ""
      ],
      "Talysh": [
        ""
      ],
      "Tamashek": [
        ""
      ],
      "Tswana": [
        ""
      ],
      "Nyasa Tonga": [
        ""
      ],
      "Tok Pisin": [
        ""
      ],
      "Tɛɛki kasa": [
        ""
      ],
      "Turoyo": [
        ""
      ],
      "Taroko": [
        ""
      ],
      "Tsakonian": [
        ""
      ],
      "Tsimshian": [
        ""
      ],
      "Tsonga": [
        ""
      ],
      "Tatar": [
        ""
      ],
      "Muslim Tat": [
        ""
      ],
      "Tumbuka": [
        ""
      ],
      "Tasawaq": [
        ""
      ],
      "Twi": [
        ""
      ],
      "Tahitian": [
        ""
      ],
      "Tuvinian": [
        ""
      ],
      "Central Atlas Tamazight": [
        ""
      ],
      "Udmurt": [
        ""
      ],
      "Ugaritic": [
        ""
      ],
      "Uyghur": [
        ""
      ],
      "Ukren kasa": [
        ""
      ],
      "Umbundu": [
        ""
      ],
      "Unknown Language": [
        ""
      ],
      "Urdu kasa": [
        ""
      ],
      "Uzbek": [
        ""
      ],
      "Vai": [
        ""
      ],
      "Venetian": [
        ""
      ],
      "Veps": [
        ""
      ],
      "Venda": [
        ""
      ],
      "Viɛtnam kasa": [
        ""
      ],
      "West Flemish": [
        ""
      ],
      "Main-Franconian": [
        ""
      ],
      "Votic": [
        ""
      ],
      "Volapük": [
        ""
      ],
      "Võro": [
        ""
      ],
      "Vunjo": [
        ""
      ],
      "Walser": [
        ""
      ],
      "Wolaytta": [
        ""
      ],
      "Waray": [
        ""
      ],
      "Washo": [
        ""
      ],
      "Walloon": [
        ""
      ],
      "Warlpiri": [
        ""
      ],
      "Wolof": [
        ""
      ],
      "Wu Chinese": [
        ""
      ],
      "Kalmyk": [
        ""
      ],
      "Xhosa": [
        ""
      ],
      "Mingrelian": [
        ""
      ],
      "Soga": [
        ""
      ],
      "Yao": [
        ""
      ],
      "Yapese": [
        ""
      ],
      "Yangben": [
        ""
      ],
      "Yemba": [
        ""
      ],
      "Yiddish": [
        ""
      ],
      "Yoruba": [
        ""
      ],
      "Nheengatu": [
        ""
      ],
      "Cantonese": [
        ""
      ],
      "Zapotec": [
        ""
      ],
      "Zhuang": [
        ""
      ],
      "Blissymbols": [
        ""
      ],
      "Zeelandic": [
        ""
      ],
      "Zenaga": [
        ""
      ],
      "Standard Moroccan Tamazight": [
        ""
      ],
      "Simplified Chinese": [
        ""
      ],
      "Traditional Chinese": [
        ""
      ],
      "Kyaena kasa": [
        ""
      ],
      "Zuni": [
        ""
      ],
      "Zulu": [
        ""
      ],
      "No linguistic content": [
        ""
      ],
      "Zaza": [
        ""
      ],
      "Could not load user attention request": [
        ""
      ],
      "No notification left": [
        ""
      ],
      "Ignore": [
        ""
      ],
      "Invalid dev experiment URI.": [
        ""
      ],
      "URI: %1$s": [
        ""
      ],
      "Experiment": [
        ""
      ],
      "Insert lost denomination": [
        ""
      ],
      "Apply": [
        ""
      ],
      "Pending refresh": [
        ""
      ],
      "Could not load the transfer payment status": [
        ""
      ],
      "Could not load the status of the order template": [
        ""
      ],
      "required": [
        ""
      ],
      "Review order": [
        ""
      ],
      "Missing URI": [
        ""
      ],
      "Check the link.": [
        ""
      ],
      "Could not parse the recovery URI": [
        ""
      ],
      "Try another URI": [
        ""
      ],
      "Import backup, show info": [
        ""
      ],
      "Add new exchange": [
        ""
      ],
      "Add exchange for %1$s": [
        ""
      ],
      "Enter the URL of an exchange you trust.": [
        ""
      ],
      "An exchange has been found! Review the information and click next": [
        ""
      ],
      "This exchange is already in your list.": [
        ""
      ],
      "Only exchange accessible through \"http\" and \"https\" are allowed.": [
        ""
      ],
      "This exchange protocol version is not supported: \" %1$s\".": [
        ""
      ],
      "This exchange currency \"%1$s\" doesn&apos;t match the expected currency %2$s.": [
        ""
      ],
      "No exchange found in that URL.": [
        ""
      ],
      "loading": [
        ""
      ],
      "Version": [
        ""
      ],
      "Can't use the URL: \"%1$s\"": [
        ""
      ],
      "Exchange URL": [
        ""
      ],
      "Add new contact": [
        ""
      ],
      "Enter the details of a contact you trust.": [
        ""
      ],
      "Contact info valid.": [
        ""
      ],
      "This contact is already in your list.": [
        ""
      ],
      "Review contact": [
        ""
      ],
      "Contact alias": [
        ""
      ],
      "Alias type": [
        ""
      ],
      "Mailbox service": [
        ""
      ],
      "Mailbox address": [
        ""
      ],
      "Contact source": [
        ""
      ],
      "Could not load the list of contacts": [
        ""
      ],
      "Source": [
        ""
      ],
      "Send Cash": [
        ""
      ],
      "Request Cash": [
        ""
      ],
      "No contacts found.": [
        ""
      ],
      "Could not bank choices for account": [
        ""
      ],
      "Could not load the list of messages": [
        ""
      ],
      "Unexpected error when trying to initialize mailbox: %1$s": [
        ""
      ],
      "Unexpected error when trying to delete message: %1$s": [
        ""
      ],
      "Unexpected error when trying to fetch messages: %1$s": [
        ""
      ],
      "Date": [
        "Data"
      ],
      "You may copy your mailbox URI from below and register it at %1$s with your alias(es) to make it easier for others to find you.": [
        ""
      ],
      "copy uri": [
        ""
      ],
      "hide uri": [
        ""
      ],
      "show uri": [
        ""
      ],
      "Fetch messages": [
        ""
      ],
      "Mailbox not initialized": [
        ""
      ],
      "Initialize mailbox": [
        ""
      ],
      "No messages yet.": [
        ""
      ],
      "Taler URI handler": [
        ""
      ],
      "Could not found a handler for the Taler URI": [
        ""
      ],
      "The uri read in the path parameter is not valid: \"%1$s\"": [
        ""
      ],
      "Digital cash payment": [
        ""
      ],
      "Digital cash refund": [
        ""
      ],
      "Digital cash withdrawal": [
        ""
      ],
      "Digital cash deposit": [
        ""
      ],
      "Digital cash invoice": [
        ""
      ],
      "Digital cash transfer": [
        ""
      ],
      "Digital cash recovery": [
        ""
      ],
      "Development experiment": [
        ""
      ],
      "Return to wallet": [
        ""
      ],
      "Network is offline": [
        ""
      ],
      "Diagnostics timed out. Could not talk to the wallet backend.": [
        ""
      ],
      "Problems detected:": [
        ""
      ],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).": [
        ""
      ],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.": [
        ""
      ],
      "here": [
        ""
      ],
      "Running diagnostics": [
        ""
      ],
      "Edit": [
        ""
      ],
      "List of products": [
        ""
      ],
      "free": [
        ""
      ],
      "Could not get provider information": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "it",
  "completeness": 1
};

strings['he'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=4; plural=(n == 1) ? 0 : ((n == 2) ? 1 : ((n > 10 && n % 10 == 0) ? 2 : 3));",
        "lang": "he"
      },
      "Balance": [
        ""
      ],
      "Backup": [
        ""
      ],
      "Notifications": [
        ""
      ],
      "QR Reader and Taler URI": [
        ""
      ],
      "Settings": [
        ""
      ],
      "Dev tools": [
        ""
      ],
      "Contacts": [
        ""
      ],
      "Mailbox": [
        ""
      ],
      "Search transactions": [
        ""
      ],
      "%1$s": [
        ""
      ],
      "KYC requirement": [
        ""
      ],
      "PENDING OPERATIONS": [
        ""
      ],
      "Error was thrown trying to: \"%1$s\"": [
        ""
      ],
      "Could not open the wallet database. Firefox is known to run into this problem under \"permanent private mode\".": [
        ""
      ],
      "Could not open the wallet database.": [
        ""
      ],
      "Unexpected request error, code: %1$s": [
        ""
      ],
      "more info": [
        ""
      ],
      "wrap text": [
        ""
      ],
      "copy content": [
        ""
      ],
      "less info": [
        ""
      ],
      "Retry operation": [
        ""
      ],
      "Loading": [
        ""
      ],
      "Could not load backup providers": [
        ""
      ],
      "No backup providers configured": [
        ""
      ],
      "Add provider": [
        ""
      ],
      "Sync all backups": [
        ""
      ],
      "Sync now": [
        ""
      ],
      "Last synced": [
        ""
      ],
      "Not synced": [
        ""
      ],
      "Expires in": [
        ""
      ],
      "show less": [
        ""
      ],
      "show more": [
        ""
      ],
      "There was an error loading the provider detail for &quot;%1$s&quot;": [
        ""
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [
        ""
      ],
      "See providers": [
        ""
      ],
      "Last backup": [
        ""
      ],
      "Back up": [
        ""
      ],
      "Provider fee": [
        ""
      ],
      "per year": [
        ""
      ],
      "Extend": [
        ""
      ],
      "terms has changed, extending the service will imply accepting the new terms of service": [
        ""
      ],
      "old": [
        ""
      ],
      "new": [
        ""
      ],
      "fee": [
        ""
      ],
      "storage": [
        ""
      ],
      "Remove provider": [
        ""
      ],
      "Pay": [
        ""
      ],
      "Withdraw": [
        ""
      ],
      "This provider has reported an error": [
        ""
      ],
      "There is conflict with another backup from &quot;%1$s&quot;": [
        ""
      ],
      "Backup is not readable": [
        ""
      ],
      "Unknown backup problem: %1$s": [
        ""
      ],
      "service paid": [
        ""
      ],
      "Backup valid until": [
        ""
      ],
      "Cancel": [
        ""
      ],
      "Open pay page": [
        ""
      ],
      "Open refund page": [
        ""
      ],
      "Open withdraw page": [
        ""
      ],
      "Your wallet is empty.": [
        ""
      ],
      "Get digital cash": [
        ""
      ],
      "Try the demo bank and withdraw test money.": [
        ""
      ],
      "Could not load the balance": [
        ""
      ],
      "Receive": [
        ""
      ],
      "Send %1$s": [
        ""
      ],
      "This page has a withdrawal action.": [
        ""
      ],
      "This page has pay action.": [
        ""
      ],
      "This page has a refund action.": [
        ""
      ],
      "This page has a add exchange action.": [
        ""
      ],
      "Open add exchange page": [
        ""
      ],
      "This page has an add contact action.": [
        ""
      ],
      "Open add contact page": [
        ""
      ],
      "Taler Action": [
        ""
      ],
      "This page has a malformed taler uri.": [
        ""
      ],
      "Dismiss": [
        ""
      ],
      "this popup is being closed and you are being redirected to %1$s": [
        ""
      ],
      "Could not load purchase proposal details": [
        ""
      ],
      "Order Id": [
        ""
      ],
      "Summary": [
        ""
      ],
      "Amount": [
        ""
      ],
      "Choices": [
        ""
      ],
      "This purchase have %1$s choices": [
        ""
      ],
      "Merchant name": [
        ""
      ],
      "Merchant jurisdiction": [
        ""
      ],
      "Merchant address": [
        ""
      ],
      "Merchant logo": [
        ""
      ],
      "Merchant website": [
        ""
      ],
      "Merchant email": [
        ""
      ],
      "Merchant public key": [
        ""
      ],
      "Created at": [
        ""
      ],
      "Refund deadline": [
        ""
      ],
      "Auto refund": [
        ""
      ],
      "Pay deadline": [
        ""
      ],
      "Fulfillment URL": [
        ""
      ],
      "Fulfillment message": [
        ""
      ],
      "Max deposit fee": [
        ""
      ],
      "Exchanges": [
        ""
      ],
      "Could not load the status of the term of service": [
        ""
      ],
      "Select one option": [
        ""
      ],
      "Show terms of service": [
        ""
      ],
      "I accept the exchange terms of service": [
        ""
      ],
      "Review exchange terms of service": [
        ""
      ],
      "The exchange doesn't have a terms of service.": [
        ""
      ],
      "Skip it for now.": [
        ""
      ],
      "Format": [
        ""
      ],
      "Language": [
        ""
      ],
      "The exchange replied with a empty terms of service": [
        ""
      ],
      "No terms of service. The exchange replied with a empty document": [
        ""
      ],
      "No terms of service. The exchange replied with a empty text": [
        ""
      ],
      "Download Terms of Service": [
        ""
      ],
      "Hide terms of service": [
        ""
      ],
      "Amount is not valid": [
        ""
      ],
      "Bank account": [
        ""
      ],
      "Bitcoin address": [
        ""
      ],
      "BIC/IBAN": [
        ""
      ],
      "IBAN": [
        ""
      ],
      "Bitcoin transfer details": [
        ""
      ],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.": [
        ""
      ],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts": [
        ""
      ],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC": [
        ""
      ],
      "Ethereum transfer details": [
        ""
      ],
      "You need to wire to the service provider account.": [
        ""
      ],
      "Make sure the amount show %1$s ETH": [
        ""
      ],
      "Bank transfer details": [
        ""
      ],
      "Account": [
        ""
      ],
      "Bank host": [
        ""
      ],
      "BIC": [
        ""
      ],
      "Exchange": [
        ""
      ],
      "Reserve Pub": [
        ""
      ],
      "Step 1:": [
        ""
      ],
      "Copy this code and paste it into the subject/purpose field in your banking app or bank website": [
        ""
      ],
      "Subject": [
        ""
      ],
      "Step 2:": [
        ""
      ],
      "If you don't already have it in your banking favourites list, then copy and paste this IBAN and the name into the receiver fields in your banking app or website": [
        ""
      ],
      "Receiver name": [
        ""
      ],
      "Receiver postal code": [
        ""
      ],
      "Receiver town": [
        ""
      ],
      "Step 3:": [
        ""
      ],
      "Finish the wire transfer setting the amount in your banking app or website, then this withdrawal will proceed automatically.": [
        ""
      ],
      "Make sure ALL data is correct, including the subject; otherwise, the money will not arrive in this wallet. You can use the copy buttons (%1$s) to prevent typing errors or the \"payto://\" URI below to copy just one value.": [
        ""
      ],
      "Alternative if your bank already supports PayTo URI, you can use this %1$s link instead": [
        ""
      ],
      "Continue with banking app or website": [
        ""
      ],
      "Show QR code": [
        ""
      ],
      "Could not load transaction information": [
        ""
      ],
      "Bank account confirmation required for the transaction to complete.": [
        ""
      ],
      "KYC check required for the transaction to complete.": [
        ""
      ],
      "No additional information has been provided.": [
        ""
      ],
      "Follow this link to the%1$s %2$s": [
        ""
      ],
      "This transaction is pending to complete.": [
        ""
      ],
      "Retry": [
        ""
      ],
      "There was an error trying to complete the transaction.": [
        ""
      ],
      "This transaction was aborted.": [
        ""
      ],
      "This transaction failed.": [
        ""
      ],
      "Caution!": [
        ""
      ],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.": [
        ""
      ],
      "Confirm": [
        ""
      ],
      "Doing a cancellation while the transaction still active might result in lost coins. Do you still want to cancel the transaction?": [
        ""
      ],
      "No": [
        ""
      ],
      "Yes": [
        ""
      ],
      "Abort": [
        ""
      ],
      "Resume": [
        ""
      ],
      "Suspend": [
        ""
      ],
      "Delete": [
        ""
      ],
      "Withdrawal": [
        ""
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Select one of the accounts and use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Details": [
        ""
      ],
      "Withdrawal incomplete.": [
        ""
      ],
      "If you have already sent money to the service provider account it will wire it back at %1$s": [
        ""
      ],
      "Payment": [
        ""
      ],
      "Refunds": [
        ""
      ],
      "%1$s %2$s on %3$s .": [
        ""
      ],
      "Refund is in progress.": [
        ""
      ],
      "Merchant created a refund for this order but was not automatically picked up.": [
        ""
      ],
      "Offer": [
        ""
      ],
      "Accept": [
        ""
      ],
      "Confirmation code": [
        ""
      ],
      "Merchant": [
        ""
      ],
      "Invoice ID": [
        ""
      ],
      "Deposit": [
        ""
      ],
      "Wire transfer deadline.": [
        ""
      ],
      "Wire transfer still pending.": [
        ""
      ],
      "The service provider deadline to make the wire transfer is: %1$s": [
        ""
      ],
      "Wire transfer completed.": [
        ""
      ],
      " ": [
        ""
      ],
      "Transfer details": [
        ""
      ],
      "Wire transfer in progress.": [
        ""
      ],
      "Refresh": [
        ""
      ],
      "Refund": [
        ""
      ],
      "Purchase summary": [
        ""
      ],
      "Credit": [
        ""
      ],
      "Invoice": [
        ""
      ],
      "URI": [
        ""
      ],
      "Debit": [
        ""
      ],
      "Transfer": [
        ""
      ],
      "Lost": [
        ""
      ],
      "Reason": [
        ""
      ],
      "Denomination expired.": [
        ""
      ],
      "Denomination vanished.": [
        ""
      ],
      "Denomination is unoffered.": [
        ""
      ],
      "Fees": [
        ""
      ],
      "Total": [
        ""
      ],
      "Sent": [
        ""
      ],
      "Converted": [
        ""
      ],
      "Bank fee": [
        ""
      ],
      "Price": [
        ""
      ],
      "Subtotal": [
        ""
      ],
      "Refunded": [
        ""
      ],
      "Transfer identification": [
        ""
      ],
      "copy": [
        ""
      ],
      "hide qr": [
        ""
      ],
      "forward to contact": [
        ""
      ],
      "show qr": [
        ""
      ],
      "Wire transfer need a confirmation. Go to the %1$s and check wire transfer operation to exchange account is complete.": [
        ""
      ],
      "bank site": [
        ""
      ],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins.": [
        ""
      ],
      "Exchange is ready to send the coins, withdrawal in progress.": [
        ""
      ],
      "Send &nbsp; %1$s": [
        ""
      ],
      "Pay &nbsp; %1$s": [
        ""
      ],
      "Merchant message": [
        ""
      ],
      "Balance is not enough.": [
        ""
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which exchange and auditors the merchant trust.": [
        ""
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which wire methods the merchant accepts.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to pay for this contract which is restricted.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to spend right know. There are some coins that need to be refreshed.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s available.": [
        ""
      ],
      "Provider is missing the global fee configuration, this likely means it is misconfigured.": [
        ""
      ],
      "Balance looks like it should be enough, but doesn't cover all fees requested by the merchant and payment processor. Please ensure there is at least %1$s %2$s more balance in your wallet or ask your merchant to cover more of the fees.": [
        ""
      ],
      "Pay with a mobile phone": [
        ""
      ],
      "Hide QR": [
        ""
      ],
      "Scan the QR code or &nbsp; %1$s": [
        ""
      ],
      "click here": [
        ""
      ],
      "Could not load the payment and balance status": [
        ""
      ],
      "Purchase": [
        ""
      ],
      "This purchase is age restricted.": [
        ""
      ],
      "Expires at": [
        ""
      ],
      "Already paid, you are going to be redirected to %1$s": [
        ""
      ],
      "Already paid": [
        ""
      ],
      "Already claimed": [
        ""
      ],
      "Could not load the refund status": [
        ""
      ],
      "You&apos;ve ignored the refund.": [
        ""
      ],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [
        ""
      ],
      "Order amount": [
        ""
      ],
      "Could not load the list of exchanges": [
        ""
      ],
      "Could not load info from URI": [
        ""
      ],
      "Could not load the withdrawal details": [
        ""
      ],
      "under %1$s": [
        ""
      ],
      "should be greater than zero": [
        ""
      ],
      "can't be greater than %1$s": [
        ""
      ],
      "Could not load exchange details info": [
        ""
      ],
      "Close": [
        ""
      ],
      "There is no exchange available for currency %1$s": [
        ""
      ],
      "You can add more exchanges from the settings.": [
        ""
      ],
      "Exchange %1$s is not available": [
        ""
      ],
      "Exchange status can view accessed from the settings.": [
        ""
      ],
      "Service fee description": [
        ""
      ],
      "Select %1$s exchange": [
        ""
      ],
      "Reset": [
        ""
      ],
      "Use this exchange": [
        ""
      ],
      "Auditors": [
        ""
      ],
      "Doesn&apos;t have auditors": [
        ""
      ],
      "currency": [
        ""
      ],
      "Coin operations": [
        ""
      ],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.": [
        ""
      ],
      "Deposits": [
        ""
      ],
      "Denomination": [
        ""
      ],
      "Current": [
        ""
      ],
      "Selected": [
        ""
      ],
      "Until": [
        ""
      ],
      "Withdrawals": [
        ""
      ],
      "Transfer operations": [
        ""
      ],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.": [
        ""
      ],
      "Operation": [
        ""
      ],
      "Fee": [
        ""
      ],
      "Wallet operations": [
        ""
      ],
      "Feature": [
        ""
      ],
      "Currency": [
        ""
      ],
      "This operation has already been completed.": [
        ""
      ],
      "This operation has already been aborted": [
        ""
      ],
      "This operation has started and should be completed in the bank.": [
        ""
      ],
      "You can confirm the operation in": [
        ""
      ],
      "this page": [
        ""
      ],
      "This operation has already been completed by another wallet.": [
        ""
      ],
      "This operation has already been used by another wallet.": [
        ""
      ],
      "It can be confirmed in": [
        ""
      ],
      "Age restriction": [
        ""
      ],
      "Withdraw &nbsp; %1$s": [
        ""
      ],
      "Withdraw to a mobile phone": [
        ""
      ],
      "Short description of the invoice": [
        ""
      ],
      "Create": [
        ""
      ],
      "Short description of the transfer": [
        ""
      ],
      "Could not load the invoice payment status": [
        ""
      ],
      "Receive &nbsp; %1$s": [
        ""
      ],
      "Could not load known bank accounts": [
        ""
      ],
      "Could not load supported wire methods": [
        ""
      ],
      "No wire methods supported for this currency": [
        ""
      ],
      "Try another wire method.": [
        ""
      ],
      "Known accounts for %1$s": [
        ""
      ],
      "To add a new account first select the account type.": [
        ""
      ],
      "Unable add this account": [
        ""
      ],
      "Add": [
        ""
      ],
      "IBAN accounts": [
        ""
      ],
      "Alias": [
        ""
      ],
      "Int. Account Number": [
        ""
      ],
      "Account name": [
        ""
      ],
      "KYC": [
        ""
      ],
      "Taler accounts": [
        ""
      ],
      "Host": [
        ""
      ],
      "Bitcoin accounts": [
        ""
      ],
      "Address": [
        ""
      ],
      "Can't be empty": [
        ""
      ],
      "Bitcoin Account": [
        ""
      ],
      "Taler Bank": [
        ""
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "Required": [
        ""
      ],
      "International Bank Account Number": [
        ""
      ],
      "Could not load balance information": [
        ""
      ],
      "Could not load fee for amount %1$s": [
        ""
      ],
      "A currency or an amount should be indicated": [
        ""
      ],
      "There is no enough balance to make a deposit for currency %1$s": [
        ""
      ],
      "Send %1$s to your account": [
        ""
      ],
      "There is no account to make a deposit for currency %1$s": [
        ""
      ],
      "Add account": [
        ""
      ],
      "Select account": [
        ""
      ],
      "Manage accounts": [
        ""
      ],
      "Brut amount": [
        ""
      ],
      "Deposit fee": [
        ""
      ],
      "Net amount": [
        ""
      ],
      "Deposit&nbsp;%1$s %2$s": [
        ""
      ],
      "Need approval in the Bank": [
        ""
      ],
      "Waiting for wire transfer to complete": [
        ""
      ],
      "Exchange is waiting the wire transfer": [
        ""
      ],
      "Payment in progress": [
        ""
      ],
      "Executing refund...": [
        ""
      ],
      "Refreshing coins...": [
        ""
      ],
      "Deposit in progress": [
        ""
      ],
      "Waiting to be paid": [
        ""
      ],
      "Receiving the transfer": [
        ""
      ],
      "Waiting to be received": [
        ""
      ],
      "Denomination expired": [
        ""
      ],
      "Denomination vanished": [
        ""
      ],
      "Denomination unoffered": [
        ""
      ],
      "ABORTED": [
        ""
      ],
      "FAILED": [
        ""
      ],
      "Could not load the list of transactions": [
        ""
      ],
      "unknown scope": [
        ""
      ],
      "Send": [
        ""
      ],
      "Your transaction history is empty for this currency.": [
        ""
      ],
      "Review terms of service": [
        ""
      ],
      "Provider URL": [
        ""
      ],
      "Please review and accept this provider&apos;s terms of service": [
        ""
      ],
      "Pricing": [
        ""
      ],
      "free of charge": [
        ""
      ],
      "%1$s per year of service": [
        ""
      ],
      "Storage": [
        ""
      ],
      "%1$s megabytes of storage per year of service": [
        ""
      ],
      "Accept terms of service": [
        ""
      ],
      "Add backup provider": [
        ""
      ],
      "Backup providers may charge for their service": [
        ""
      ],
      "URL": [
        ""
      ],
      "Name": [
        ""
      ],
      "Next": [
        ""
      ],
      "Navigator": [
        ""
      ],
      "Automatically open wallet": [
        ""
      ],
      "Open the wallet when a payment action is found.": [
        ""
      ],
      "Version Info": [
        ""
      ],
      "Web Extension": [
        ""
      ],
      "The version of wallet core is not supported. (supported version: %1$s, wallet version: %2$s)": [
        ""
      ],
      "Exchange compatibility": [
        ""
      ],
      "Merchant compatibility": [
        ""
      ],
      "Bank compatibility": [
        ""
      ],
      "Wallet Core compatibility": [
        ""
      ],
      "Enable developer mode": [
        ""
      ],
      "Show more information and options in the UI": [
        ""
      ],
      "Display": [
        ""
      ],
      "Current Language": [
        ""
      ],
      "Show backup feature": [
        ""
      ],
      "Backup integration still in beta.": [
        ""
      ],
      "Show P2P alias feature": [
        ""
      ],
      "P2P aliases integration still in beta.": [
        ""
      ],
      "Show suspend/resume transaction": [
        ""
      ],
      "Prevent transaction from doing network request.": [
        ""
      ],
      "Show refresh transaction type in the transaction list": [
        ""
      ],
      "Refresh transaction will be hidden by default if the refresh operation doesn't have fee.": [
        ""
      ],
      "Show more account types on deposit": [
        ""
      ],
      "Extends the UI to more payment target types.": [
        ""
      ],
      "Show JSON on error": [
        ""
      ],
      "Print more information about the error. Useful for debugging.": [
        ""
      ],
      "Allow HTTP connections": [
        ""
      ],
      "Using HTTP connection may be faster but unsafe (wallet restart required)": [
        ""
      ],
      "Lang selector": [
        ""
      ],
      "Allows to manually change the language of the UI. Otherwise it will be automatically selected by your browser configuration.": [
        ""
      ],
      "Edit exchange management": [
        ""
      ],
      "Allows to see the list of exchange, remove, add and switch before withdrawal.": [
        ""
      ],
      "Select terms of service format": [
        ""
      ],
      "Allows to render the terms of service on different format selected by the user.": [
        ""
      ],
      "Show wallet activity": [
        ""
      ],
      "Show the wallet notification and observability event in the UI.": [
        ""
      ],
      "Enable v1 contract": [
        ""
      ],
      "v1 is under development and wallet-core converts v1 contract to v0. Enable this options for testing": [
        ""
      ],
      "GNU Taler Wallet installed!": [
        ""
      ],
      "You can open the wallet using the combination %1$s .": [
        ""
      ],
      "Also pinning the GNU Taler Wallet to your browser allows you to quick access without keyboard:": [
        ""
      ],
      "Click the puzzle icon": [
        ""
      ],
      "Search for GNU Taler Wallet": [
        ""
      ],
      "Click the pin icon": [
        ""
      ],
      "Next Steps": [
        ""
      ],
      "Try the demo": [
        ""
      ],
      "Click here to open the wallet activity tab.": [
        ""
      ],
      "Events": [
        ""
      ],
      "Active tasks": [
        ""
      ],
      "All events": [
        ""
      ],
      "Balance change": [
        ""
      ],
      "Backup failed": [
        ""
      ],
      "Transaction updated": [
        ""
      ],
      "Exchange updated": [
        ""
      ],
      "Idle": [
        ""
      ],
      "task.%1$s": [
        ""
      ],
      "wallet.%1$s(%2$s)": [
        ""
      ],
      "Bank account info changed": [
        ""
      ],
      "Contact added": [
        ""
      ],
      "Contact deleted": [
        ""
      ],
      "Mailbox message added": [
        ""
      ],
      "Mailbox message deleted": [
        ""
      ],
      "Type": [
        ""
      ],
      "Id": [
        ""
      ],
      "Since": [
        ""
      ],
      "Next try": [
        ""
      ],
      "Error": [
        ""
      ],
      "Transaction": [
        ""
      ],
      "No exchange yet": [
        ""
      ],
      "Status": [
        ""
      ],
      "Terms of Service": [
        ""
      ],
      "Last Update": [
        ""
      ],
      "Actions": [
        ""
      ],
      "ok": [
        ""
      ],
      "pending": [
        ""
      ],
      "proposed": [
        ""
      ],
      "unknown (exchange status should be updated)": [
        ""
      ],
      "Reload": [
        ""
      ],
      "Purge": [
        ""
      ],
      "Make regional": [
        ""
      ],
      "Make global": [
        ""
      ],
      "Forget ToS": [
        ""
      ],
      "Debug tools": [
        ""
      ],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?": [
        ""
      ],
      "reset": [
        ""
      ],
      "TESTING: This may delete all your coin, proceed with caution": [
        ""
      ],
      "run gc": [
        ""
      ],
      "import database": [
        ""
      ],
      "export database": [
        ""
      ],
      "Clear exchange key cache": [
        ""
      ],
      "Database exported at %1$s %2$s to download": [
        ""
      ],
      "Inject Taler support in all pages": [
        ""
      ],
      "Enabling this option will make `window.taler` be available in all sites": [
        ""
      ],
      "Exchange Entries": [
        ""
      ],
      "Add an exchange": [
        ""
      ],
      "Log levels": [
        ""
      ],
      "Coins": [
        ""
      ],
      "usable coins": [
        ""
      ],
      "id": [
        ""
      ],
      "denom": [
        ""
      ],
      "status": [
        ""
      ],
      "from refresh?": [
        ""
      ],
      "age key count": [
        ""
      ],
      "spent coins": [
        ""
      ],
      "click to show": [
        ""
      ],
      "URI is not valid. Taler URI should start with \"taler://\"": [
        ""
      ],
      "After the URI type it should follow a '/' with more information.": [
        ""
      ],
      "This URI type is not supported": [
        ""
      ],
      "This URI requires the bank host and operation id separated by /": [
        ""
      ],
      "This URI requires the merchant host, order id and session id separated by /": [
        ""
      ],
      "This URI requires the merchant host and order id separated by /": [
        ""
      ],
      "This URI requires the exchange host and key id separated by /": [
        ""
      ],
      "This URI requires the merchant host and template id separated by /": [
        ""
      ],
      "This URI requires the host key and provider list separated by /": [
        ""
      ],
      "This URI requires the experiment id": [
        ""
      ],
      "This URI requires the exchange host": [
        ""
      ],
      "This URI requires the alias type, the alias and the mailbox URI separated by /": [
        ""
      ],
      "This URI requires string after the first /": [
        ""
      ],
      "The bank host is invalid": [
        ""
      ],
      "The merchant host is invalid": [
        ""
      ],
      "The URI should end with /": [
        ""
      ],
      "The exchange host is invalid": [
        ""
      ],
      "The contact is invalid": [
        ""
      ],
      "The amount is invalid": [
        ""
      ],
      "A parameter is invalid": [
        ""
      ],
      "checking...": [
        ""
      ],
      "Unexpected error happen reading the camera: %1$s": [
        ""
      ],
      "Could not found a QR code in the file": [
        ""
      ],
      "Unexpected error happen reading the file: %1$s": [
        ""
      ],
      "Scan a QR code or enter taler:// URI below": [
        ""
      ],
      "Pay invoice": [
        ""
      ],
      "Withdrawal from bank": [
        ""
      ],
      "Claim refund": [
        ""
      ],
      "Accept payment": [
        ""
      ],
      "Complete order": [
        ""
      ],
      "Restore wallet": [
        ""
      ],
      "Enable experiment": [
        ""
      ],
      "Withdraw from exchange": [
        ""
      ],
      "Add exchange": [
        ""
      ],
      "Notify transaction": [
        ""
      ],
      "Add contact": [
        ""
      ],
      "The exchange URL is invalid.": [
        ""
      ],
      "The exchange URL should end with '/'": [
        ""
      ],
      "Couldn't found an exchange in the URL specified.": [
        ""
      ],
      "HTTP request failed to %1$s: %2$s": [
        ""
      ],
      "HTTP request failed to %1$s": [
        ""
      ],
      "The merchant URL is invalid.": [
        ""
      ],
      "The merchant URL should end with '/'": [
        ""
      ],
      "Couldn't found an merchant in the URL specified.": [
        ""
      ],
      "The bank URL is invalid.": [
        ""
      ],
      "The bank URL should end with '/'": [
        ""
      ],
      "Couldn't found an bank in the URL specified.": [
        ""
      ],
      "Could not load exchanges": [
        ""
      ],
      "Choose a currency to proceed or add another exchange": [
        ""
      ],
      "Known currencies": [
        ""
      ],
      "Specify the origin": [
        ""
      ],
      "Use previous origins:": [
        ""
      ],
      "From another bank account": [
        ""
      ],
      "From another wallet": [
        ""
      ],
      "From a %1$s URI or QR code": [
        ""
      ],
      "Enter URI here": [
        ""
      ],
      "Specify the destination": [
        ""
      ],
      "Use previous destinations:": [
        ""
      ],
      "To another bank account": [
        ""
      ],
      "To another wallet": [
        ""
      ],
      "To a %1$s URI or QR code": [
        ""
      ],
      "Afghanistan": [
        ""
      ],
      "Albania": [
        ""
      ],
      "Algeria": [
        ""
      ],
      "United States": [
        ""
      ],
      "Andorra": [
        ""
      ],
      "Angola": [
        ""
      ],
      "Antigua and Barbuda": [
        ""
      ],
      "Argentina": [
        ""
      ],
      "Armenia": [
        ""
      ],
      "Australia": [
        ""
      ],
      "Austria": [
        ""
      ],
      "Azerbaijan": [
        ""
      ],
      "Bahamas": [
        ""
      ],
      "Bahrain": [
        ""
      ],
      "Bangladesh": [
        ""
      ],
      "Barbados": [
        ""
      ],
      "Botswana": [
        ""
      ],
      "Belarus": [
        ""
      ],
      "Belgium": [
        ""
      ],
      "Belize": [
        ""
      ],
      "Benin": [
        ""
      ],
      "Bhutan": [
        ""
      ],
      "Bolivia": [
        ""
      ],
      "Bosnia and Herzegovina": [
        ""
      ],
      "Brazil": [
        ""
      ],
      "United Kingdom": [
        ""
      ],
      "Brunei": [
        ""
      ],
      "Bulgaria": [
        ""
      ],
      "Burkina Faso": [
        ""
      ],
      "Myanmar": [
        ""
      ],
      "Burundi": [
        ""
      ],
      "Cambodia": [
        ""
      ],
      "Cameroon": [
        ""
      ],
      "Canada": [
        ""
      ],
      "Cape Verde": [
        ""
      ],
      "Central African Republic": [
        ""
      ],
      "Chad": [
        ""
      ],
      "Chile": [
        ""
      ],
      "China": [
        ""
      ],
      "Colombia": [
        ""
      ],
      "Comoros": [
        ""
      ],
      "Democratic Republic of the Congo": [
        ""
      ],
      "Republic of the Congo": [
        ""
      ],
      "Costa Rica": [
        ""
      ],
      "Croatia": [
        ""
      ],
      "Cuba": [
        ""
      ],
      "Cyprus": [
        ""
      ],
      "Czech Republic": [
        ""
      ],
      "Denmark": [
        ""
      ],
      "Djibouti": [
        ""
      ],
      "Dominican Republic": [
        ""
      ],
      "Netherlands": [
        ""
      ],
      "Timor-Leste": [
        ""
      ],
      "Ecuador": [
        ""
      ],
      "Egypt": [
        ""
      ],
      "United Arab Emirates": [
        ""
      ],
      "Equatorial Guinea": [
        ""
      ],
      "Eritrea": [
        ""
      ],
      "Estonia": [
        ""
      ],
      "Ethiopia": [
        ""
      ],
      "Fiji": [
        ""
      ],
      "Philippines": [
        ""
      ],
      "Finland": [
        ""
      ],
      "France": [
        ""
      ],
      "Gabon": [
        ""
      ],
      "Gambia": [
        ""
      ],
      "Georgia": [
        ""
      ],
      "Germany": [
        ""
      ],
      "Ghana": [
        ""
      ],
      "Greece": [
        ""
      ],
      "Grenada": [
        ""
      ],
      "Guatemala": [
        ""
      ],
      "Guinea-Bissau": [
        ""
      ],
      "Guinea": [
        ""
      ],
      "Guyana": [
        ""
      ],
      "Haiti": [
        ""
      ],
      "Honduras": [
        ""
      ],
      "Hungary": [
        ""
      ],
      "Kiribati": [
        ""
      ],
      "Iceland": [
        ""
      ],
      "India": [
        ""
      ],
      "Indonesia": [
        ""
      ],
      "Iran": [
        ""
      ],
      "Iraq": [
        ""
      ],
      "Ireland": [
        ""
      ],
      "Israel": [
        ""
      ],
      "Italy": [
        ""
      ],
      "Ivory Coast": [
        ""
      ],
      "Jamaica": [
        ""
      ],
      "Japan": [
        ""
      ],
      "Jordan": [
        ""
      ],
      "Kazakhstan": [
        ""
      ],
      "Kenya": [
        ""
      ],
      "Saint Kitts and Nevis": [
        ""
      ],
      "Kuwait": [
        ""
      ],
      "Kyrgyzstan": [
        ""
      ],
      "Laos": [
        ""
      ],
      "Latvia": [
        ""
      ],
      "Lebanon": [
        ""
      ],
      "Liberia": [
        ""
      ],
      "Libya": [
        ""
      ],
      "Liechtenstein": [
        ""
      ],
      "Lithuania": [
        ""
      ],
      "Luxembourg": [
        ""
      ],
      "North Macedonia": [
        ""
      ],
      "Madagascar": [
        ""
      ],
      "Malawi": [
        ""
      ],
      "Malaysia": [
        ""
      ],
      "Maldives": [
        ""
      ],
      "Mali": [
        ""
      ],
      "Malta": [
        ""
      ],
      "Marshall Islands": [
        ""
      ],
      "Mauritania": [
        ""
      ],
      "Mauritius": [
        ""
      ],
      "Mexico": [
        ""
      ],
      "Micronesia": [
        ""
      ],
      "Moldova": [
        ""
      ],
      "Monaco": [
        ""
      ],
      "Mongolia": [
        ""
      ],
      "Morocco": [
        ""
      ],
      "Lesotho": [
        ""
      ],
      "Mozambique": [
        ""
      ],
      "Namibia": [
        ""
      ],
      "Nauru": [
        ""
      ],
      "Nepal": [
        ""
      ],
      "New Zealand": [
        ""
      ],
      "Vanuatu": [
        ""
      ],
      "Nicaragua": [
        ""
      ],
      "Nigeria": [
        ""
      ],
      "Niger": [
        ""
      ],
      "North Korea": [
        ""
      ],
      "Norway": [
        ""
      ],
      "Oman": [
        ""
      ],
      "Pakistan": [
        ""
      ],
      "Palau": [
        ""
      ],
      "Panama": [
        ""
      ],
      "Papua New Guinea": [
        ""
      ],
      "Paraguay": [
        ""
      ],
      "Peru": [
        ""
      ],
      "Poland": [
        ""
      ],
      "Portugal": [
        ""
      ],
      "Qatar": [
        ""
      ],
      "Romania": [
        ""
      ],
      "Russia": [
        ""
      ],
      "Rwanda": [
        ""
      ],
      "Saint Lucia": [
        ""
      ],
      "El Salvador": [
        ""
      ],
      "Samoa": [
        ""
      ],
      "San Marino": [
        ""
      ],
      "São Tomé and Príncipe": [
        ""
      ],
      "Saudi Arabia": [
        ""
      ],
      "Senegal": [
        ""
      ],
      "Serbia": [
        ""
      ],
      "Seychelles": [
        ""
      ],
      "Sierra Leone": [
        ""
      ],
      "Singapore": [
        ""
      ],
      "Slovakia": [
        ""
      ],
      "Slovenia": [
        ""
      ],
      "Solomon Islands": [
        ""
      ],
      "Somalia": [
        ""
      ],
      "South Africa": [
        ""
      ],
      "South Korea": [
        ""
      ],
      "Spain": [
        ""
      ],
      "Sri Lanka": [
        ""
      ],
      "Sudan": [
        ""
      ],
      "Suriname": [
        ""
      ],
      "Eswatini": [
        ""
      ],
      "Sweden": [
        ""
      ],
      "Switzerland": [
        ""
      ],
      "Syria": [
        ""
      ],
      "Taiwan": [
        ""
      ],
      "Tajikistan": [
        ""
      ],
      "Tanzania": [
        ""
      ],
      "Thailand": [
        ""
      ],
      "Togo": [
        ""
      ],
      "Tonga": [
        ""
      ],
      "Trinidad and Tobago": [
        ""
      ],
      "Tunisia": [
        ""
      ],
      "Turkey": [
        ""
      ],
      "Tuvalu": [
        ""
      ],
      "Uganda": [
        ""
      ],
      "Ukraine": [
        ""
      ],
      "Uruguay": [
        ""
      ],
      "Uzbekistan": [
        ""
      ],
      "Venezuela": [
        ""
      ],
      "Vietnam": [
        ""
      ],
      "Yemen": [
        ""
      ],
      "Zambia": [
        ""
      ],
      "Zimbabwe": [
        ""
      ],
      "Afghan": [
        ""
      ],
      "Albanian": [
        ""
      ],
      "Algerian": [
        ""
      ],
      "American": [
        ""
      ],
      "Andorran": [
        ""
      ],
      "Angolan": [
        ""
      ],
      "Antiguan": [
        ""
      ],
      "Argentinean": [
        ""
      ],
      "Armenian": [
        ""
      ],
      "Australian": [
        ""
      ],
      "Austrian": [
        ""
      ],
      "Azerbaijani": [
        ""
      ],
      "Bahamian": [
        ""
      ],
      "Bahraini": [
        ""
      ],
      "Bangladeshi": [
        ""
      ],
      "Barbadian": [
        ""
      ],
      "Batswana": [
        ""
      ],
      "Belarusian": [
        ""
      ],
      "Belgian": [
        ""
      ],
      "Belizean": [
        ""
      ],
      "Beninese": [
        ""
      ],
      "Bhutanese": [
        ""
      ],
      "Bolivian": [
        ""
      ],
      "Bosnian": [
        ""
      ],
      "Brazilian": [
        ""
      ],
      "British": [
        ""
      ],
      "Bruneian": [
        ""
      ],
      "Bulgarian": [
        ""
      ],
      "Burkinabe": [
        ""
      ],
      "Burmese": [
        ""
      ],
      "Burundian": [
        ""
      ],
      "Cambodian": [
        ""
      ],
      "Cameroonian": [
        ""
      ],
      "Canadian": [
        ""
      ],
      "Cape Verdean": [
        ""
      ],
      "Central African": [
        ""
      ],
      "Chadian": [
        ""
      ],
      "Chilean": [
        ""
      ],
      "Chinese": [
        ""
      ],
      "Colombian": [
        ""
      ],
      "Comoran": [
        ""
      ],
      "Congolese": [
        ""
      ],
      "Costa Rican": [
        ""
      ],
      "Croatian": [
        ""
      ],
      "Cuban": [
        ""
      ],
      "Cypriot": [
        ""
      ],
      "Czech": [
        ""
      ],
      "Danish": [
        ""
      ],
      "Djiboutian": [
        ""
      ],
      "Dominican": [
        ""
      ],
      "Dutch": [
        ""
      ],
      "East Timorese": [
        ""
      ],
      "Ecuadorean": [
        ""
      ],
      "Egyptian": [
        ""
      ],
      "Emirati": [
        ""
      ],
      "Equatorial Guinean": [
        ""
      ],
      "Eritrean": [
        ""
      ],
      "Estonian": [
        ""
      ],
      "Ethiopian": [
        ""
      ],
      "Fijian": [
        ""
      ],
      "Filipino": [
        ""
      ],
      "Finnish": [
        ""
      ],
      "French": [
        ""
      ],
      "Gabonese": [
        ""
      ],
      "Gambian": [
        ""
      ],
      "Georgian": [
        ""
      ],
      "German": [
        ""
      ],
      "Ghanaian": [
        ""
      ],
      "Greek": [
        ""
      ],
      "Grenadian": [
        ""
      ],
      "Guatemalan": [
        ""
      ],
      "Guinea-Bissauan": [
        ""
      ],
      "Guinean": [
        ""
      ],
      "Guyanese": [
        ""
      ],
      "Haitian": [
        ""
      ],
      "Honduran": [
        ""
      ],
      "Hungarian": [
        ""
      ],
      "I-Kiribati": [
        ""
      ],
      "Icelander": [
        ""
      ],
      "Indian": [
        ""
      ],
      "Indonesian": [
        ""
      ],
      "Iranian": [
        ""
      ],
      "Iraqi": [
        ""
      ],
      "Irish": [
        ""
      ],
      "Israeli": [
        ""
      ],
      "Italian": [
        ""
      ],
      "Ivorian": [
        ""
      ],
      "Jamaican": [
        ""
      ],
      "Japanese": [
        ""
      ],
      "Jordanian": [
        ""
      ],
      "Kazakhstani": [
        ""
      ],
      "Kenyan": [
        ""
      ],
      "Kittian and Nevisian": [
        ""
      ],
      "Kuwaiti": [
        ""
      ],
      "Kyrgyz": [
        ""
      ],
      "Laotian": [
        ""
      ],
      "Latvian": [
        ""
      ],
      "Lebanese": [
        ""
      ],
      "Liberian": [
        ""
      ],
      "Libyan": [
        ""
      ],
      "Liechtensteiner": [
        ""
      ],
      "Lithuanian": [
        ""
      ],
      "Luxembourger": [
        ""
      ],
      "Macedonian": [
        ""
      ],
      "Malagasy": [
        ""
      ],
      "Malawian": [
        ""
      ],
      "Malaysian": [
        ""
      ],
      "Maldivian": [
        ""
      ],
      "Malian": [
        ""
      ],
      "Maltese": [
        ""
      ],
      "Marshallese": [
        ""
      ],
      "Mauritanian": [
        ""
      ],
      "Mauritian": [
        ""
      ],
      "Mexican": [
        ""
      ],
      "Micronesian": [
        ""
      ],
      "Moldovan": [
        ""
      ],
      "Monacan": [
        ""
      ],
      "Mongolian": [
        ""
      ],
      "Moroccan": [
        ""
      ],
      "Mosotho": [
        ""
      ],
      "Mozambican": [
        ""
      ],
      "Namibian": [
        ""
      ],
      "Nauruan": [
        ""
      ],
      "Nepalese": [
        ""
      ],
      "New Zealander": [
        ""
      ],
      "Ni-Vanuatu": [
        ""
      ],
      "Nicaraguan": [
        ""
      ],
      "Nigerian": [
        ""
      ],
      "Nigerien": [
        ""
      ],
      "North Korean": [
        ""
      ],
      "Norwegian": [
        ""
      ],
      "Omani": [
        ""
      ],
      "Pakistani": [
        ""
      ],
      "Palauan": [
        ""
      ],
      "Panamanian": [
        ""
      ],
      "Papua New Guinean": [
        ""
      ],
      "Paraguayan": [
        ""
      ],
      "Peruvian": [
        ""
      ],
      "Polish": [
        ""
      ],
      "Portuguese": [
        ""
      ],
      "Qatari": [
        ""
      ],
      "Romanian": [
        ""
      ],
      "Russian": [
        ""
      ],
      "Rwandan": [
        ""
      ],
      "Saint Lucian": [
        ""
      ],
      "Salvadoran": [
        ""
      ],
      "Samoan": [
        ""
      ],
      "San Marinese": [
        ""
      ],
      "Sao Tomean": [
        ""
      ],
      "Saudi": [
        ""
      ],
      "Senegalese": [
        ""
      ],
      "Serbian": [
        ""
      ],
      "Seychellois": [
        ""
      ],
      "Sierra Leonean": [
        ""
      ],
      "Singaporean": [
        ""
      ],
      "Slovakian": [
        ""
      ],
      "Slovenian": [
        ""
      ],
      "Solomon Islander": [
        ""
      ],
      "Somali": [
        ""
      ],
      "South African": [
        ""
      ],
      "South Korean": [
        ""
      ],
      "Spanish": [
        ""
      ],
      "Sri Lankan": [
        ""
      ],
      "Sudanese": [
        ""
      ],
      "Surinamer": [
        ""
      ],
      "Swazi": [
        ""
      ],
      "Swedish": [
        ""
      ],
      "Swiss": [
        ""
      ],
      "Syrian": [
        ""
      ],
      "Taiwanese": [
        ""
      ],
      "Tajik": [
        ""
      ],
      "Tanzanian": [
        ""
      ],
      "Thai": [
        ""
      ],
      "Togolese": [
        ""
      ],
      "Tongan": [
        ""
      ],
      "Trinidadian or Tobagonian": [
        ""
      ],
      "Tunisian": [
        ""
      ],
      "Turkish": [
        ""
      ],
      "Tuvaluan": [
        ""
      ],
      "Ugandan": [
        ""
      ],
      "Ukrainian": [
        ""
      ],
      "Uruguayan": [
        ""
      ],
      "Uzbekistani": [
        ""
      ],
      "Venezuelan": [
        ""
      ],
      "Vietnamese": [
        ""
      ],
      "Yemenite": [
        ""
      ],
      "Zambian": [
        ""
      ],
      "Zimbabwean": [
        ""
      ],
      "🇦🇫": [
        ""
      ],
      "🇦🇱": [
        ""
      ],
      "🇩🇿": [
        ""
      ],
      "🇺🇸": [
        ""
      ],
      "🇦🇩": [
        ""
      ],
      "🇦🇴": [
        ""
      ],
      "🇦🇬": [
        ""
      ],
      "🇦🇷": [
        ""
      ],
      "🇦🇲": [
        ""
      ],
      "🇦🇺": [
        ""
      ],
      "🇦🇹": [
        ""
      ],
      "🇦🇿": [
        ""
      ],
      "🇧🇸": [
        ""
      ],
      "🇧🇭": [
        ""
      ],
      "🇧🇩": [
        ""
      ],
      "🇧🇧": [
        ""
      ],
      "🇧🇼": [
        ""
      ],
      "🇧🇾": [
        ""
      ],
      "🇧🇪": [
        ""
      ],
      "🇧🇿": [
        ""
      ],
      "🇧🇯": [
        ""
      ],
      "🇧🇹": [
        ""
      ],
      "🇧🇴": [
        ""
      ],
      "🇧🇦": [
        ""
      ],
      "🇧🇷": [
        ""
      ],
      "🇬🇧": [
        ""
      ],
      "🇧🇳": [
        ""
      ],
      "🇧🇬": [
        ""
      ],
      "🇧🇫": [
        ""
      ],
      "🇲🇲": [
        ""
      ],
      "🇧🇮": [
        ""
      ],
      "🇰🇭": [
        ""
      ],
      "🇨🇲": [
        ""
      ],
      "🇨🇦": [
        ""
      ],
      "🇨🇻": [
        ""
      ],
      "🇨🇫": [
        ""
      ],
      "🇨🇱": [
        ""
      ],
      "🇨🇳": [
        ""
      ],
      "🇨🇴": [
        ""
      ],
      "🇨🇩": [
        ""
      ],
      "🇨🇬": [
        ""
      ],
      "🇨🇷": [
        ""
      ],
      "🇨🇺": [
        ""
      ],
      "🇨🇾": [
        ""
      ],
      "🇨🇿": [
        ""
      ],
      "🇩🇰": [
        ""
      ],
      "🇩🇯": [
        ""
      ],
      "🇩🇴": [
        ""
      ],
      "🇳🇱": [
        ""
      ],
      "🇹🇱": [
        ""
      ],
      "🇪🇨": [
        ""
      ],
      "🇪🇬": [
        ""
      ],
      "🇦🇪": [
        ""
      ],
      "🇬🇶": [
        ""
      ],
      "🇪🇷": [
        ""
      ],
      "🇪🇪": [
        ""
      ],
      "🇪🇹": [
        ""
      ],
      "🇫🇯": [
        ""
      ],
      "🇵🇭": [
        ""
      ],
      "🇫🇮": [
        ""
      ],
      "🇫🇷": [
        ""
      ],
      "🇬🇦": [
        ""
      ],
      "🇬🇲": [
        ""
      ],
      "🇬🇪": [
        ""
      ],
      "UAE Dirham": [
        ""
      ],
      "Afghani": [
        ""
      ],
      "Lek": [
        ""
      ],
      "Armenian Dram": [
        ""
      ],
      "Netherlands Antillean Guilder": [
        ""
      ],
      "Kwanza": [
        ""
      ],
      "Argentine Peso": [
        ""
      ],
      "Australian Dollar": [
        ""
      ],
      "Aruban Florin": [
        ""
      ],
      "Azerbaijan Manat": [
        ""
      ],
      "Convertible Mark": [
        ""
      ],
      "Barbados Dollar": [
        ""
      ],
      "Taka": [
        ""
      ],
      "Bulgarian Lev": [
        ""
      ],
      "Bahraini Dinar": [
        ""
      ],
      "Burundi Franc": [
        ""
      ],
      "Bermudian Dollar": [
        ""
      ],
      "Brunei Dollar": [
        ""
      ],
      "Boliviano": [
        ""
      ],
      "Mvdol": [
        ""
      ],
      "Brazilian Real": [
        ""
      ],
      "Bahamian Dollar": [
        ""
      ],
      "Ngultrum": [
        ""
      ],
      "Pula": [
        ""
      ],
      "Belarusian Ruble": [
        ""
      ],
      "Belize Dollar": [
        ""
      ],
      "Canadian Dollar": [
        ""
      ],
      "Congolese Franc": [
        ""
      ],
      "WIR Euro": [
        ""
      ],
      "Swiss Franc": [
        ""
      ],
      "WIR Franc": [
        ""
      ],
      "Unidad de Fomento": [
        ""
      ],
      "Chilean Peso": [
        ""
      ],
      "Yuan Renminbi": [
        ""
      ],
      "Colombian Peso": [
        ""
      ],
      "Unidad de Valor Real": [
        ""
      ],
      "Costa Rican Colon": [
        ""
      ],
      "Cuban Peso": [
        ""
      ],
      "Cabo Verde Escudo": [
        ""
      ],
      "Czech Koruna": [
        ""
      ],
      "Djibouti Franc": [
        ""
      ],
      "Danish Krone": [
        ""
      ],
      "Dominican Peso": [
        ""
      ],
      "Algerian Dinar": [
        ""
      ],
      "Egyptian Pound": [
        ""
      ],
      "Nakfa": [
        ""
      ],
      "Ethiopian Birr": [
        ""
      ],
      "Euro": [
        ""
      ],
      "Fiji Dollar": [
        ""
      ],
      "Falkland Islands Pound": [
        ""
      ],
      "Pound Sterling": [
        ""
      ],
      "Lari": [
        ""
      ],
      "Ghana Cedi": [
        ""
      ],
      "Gibraltar Pound": [
        ""
      ],
      "Dalasi": [
        ""
      ],
      "Guinean Franc": [
        ""
      ],
      "Quetzal": [
        ""
      ],
      "Guyana Dollar": [
        ""
      ],
      "Hong Kong Dollar": [
        ""
      ],
      "Lempira": [
        ""
      ],
      "Gourde": [
        ""
      ],
      "Forint": [
        ""
      ],
      "Rupiah": [
        ""
      ],
      "New Israeli Sheqel": [
        ""
      ],
      "Indian Rupee": [
        ""
      ],
      "Iraqi Dinar": [
        ""
      ],
      "Iranian Rial": [
        ""
      ],
      "Iceland Krona": [
        ""
      ],
      "Jamaican Dollar": [
        ""
      ],
      "Jordanian Dinar": [
        ""
      ],
      "Yen": [
        ""
      ],
      "Kenyan Shilling": [
        ""
      ],
      "Som": [
        ""
      ],
      "Riel": [
        ""
      ],
      "Comorian Franc ": [
        ""
      ],
      "North Korean Won": [
        ""
      ],
      "Won": [
        ""
      ],
      "Kuwaiti Dinar": [
        ""
      ],
      "Cayman Islands Dollar": [
        ""
      ],
      "Tenge": [
        ""
      ],
      "Lao Kip": [
        ""
      ],
      "Lebanese Pound": [
        ""
      ],
      "Sri Lanka Rupee": [
        ""
      ],
      "Liberian Dollar": [
        ""
      ],
      "Loti": [
        ""
      ],
      "Libyan Dinar": [
        ""
      ],
      "Moroccan Dirham": [
        ""
      ],
      "Moldovan Leu": [
        ""
      ],
      "Malagasy Ariary": [
        ""
      ],
      "Denar": [
        ""
      ],
      "Kyat": [
        ""
      ],
      "Tugrik": [
        ""
      ],
      "Pataca": [
        ""
      ],
      "Ouguiya": [
        ""
      ],
      "Mauritius Rupee": [
        ""
      ],
      "Rufiyaa": [
        ""
      ],
      "Malawi Kwacha": [
        ""
      ],
      "Mexican Peso": [
        ""
      ],
      "Mexican Unidad de Inversion (UDI)": [
        ""
      ],
      "Malaysian Ringgit": [
        ""
      ],
      "Mozambique Metical": [
        ""
      ],
      "Namibia Dollar": [
        ""
      ],
      "Naira": [
        ""
      ],
      "Cordoba Oro": [
        ""
      ],
      "Norwegian Krone": [
        ""
      ],
      "Nepalese Rupee": [
        ""
      ],
      "New Zealand Dollar": [
        ""
      ],
      "Rial Omani": [
        ""
      ],
      "Balboa": [
        ""
      ],
      "Sol": [
        ""
      ],
      "Kina": [
        ""
      ],
      "Philippine Peso": [
        ""
      ],
      "Pakistan Rupee": [
        ""
      ],
      "Zloty": [
        ""
      ],
      "Guarani": [
        ""
      ],
      "Qatari Rial": [
        ""
      ],
      "Romanian Leu": [
        ""
      ],
      "Serbian Dinar": [
        ""
      ],
      "Russian Ruble": [
        ""
      ],
      "Rwanda Franc": [
        ""
      ],
      "Saudi Riyal": [
        ""
      ],
      "Solomon Islands Dollar": [
        ""
      ],
      "Seychelles Rupee": [
        ""
      ],
      "Sudanese Pound": [
        ""
      ],
      "Swedish Krona": [
        ""
      ],
      "Singapore Dollar": [
        ""
      ],
      "Saint Helena Pound": [
        ""
      ],
      "Leone": [
        ""
      ],
      "Somali Shilling": [
        ""
      ],
      "Surinam Dollar": [
        ""
      ],
      "South Sudanese Pound": [
        ""
      ],
      "Dobra": [
        ""
      ],
      "El Salvador Colon": [
        ""
      ],
      "Syrian Pound": [
        ""
      ],
      "Lilangeni": [
        ""
      ],
      "Baht": [
        ""
      ],
      "Somoni": [
        ""
      ],
      "Turkmenistan New Manat": [
        ""
      ],
      "Tunisian Dinar": [
        ""
      ],
      "Pa’anga": [
        ""
      ],
      "Turkish Lira": [
        ""
      ],
      "Trinidad and Tobago Dollar": [
        ""
      ],
      "New Taiwan Dollar": [
        ""
      ],
      "Tanzanian Shilling": [
        ""
      ],
      "Hryvnia": [
        ""
      ],
      "Uganda Shilling": [
        ""
      ],
      "US Dollar": [
        ""
      ],
      "US Dollar (Next day)": [
        ""
      ],
      "Uruguay Peso en Unidades Indexadas (UI)": [
        ""
      ],
      "Peso Uruguayo": [
        ""
      ],
      "Unidad Previsional": [
        ""
      ],
      "Uzbekistan Sum": [
        ""
      ],
      "Bolívar Soberano": [
        ""
      ],
      "Dong": [
        ""
      ],
      "Vatu": [
        ""
      ],
      "Tala": [
        ""
      ],
      "CFA Franc BEAC": [
        ""
      ],
      "East Caribbean Dollar": [
        ""
      ],
      "CFA Franc BCEAO": [
        ""
      ],
      "CFP Franc": [
        ""
      ],
      "Sucre": [
        ""
      ],
      "Yemeni Rial": [
        ""
      ],
      "Rand": [
        ""
      ],
      "Zambian Kwacha": [
        ""
      ],
      "Zimbabwe Gold": [
        ""
      ],
      "Afar": [
        ""
      ],
      "Abkhazian": [
        ""
      ],
      "Achinese": [
        ""
      ],
      "Acoli": [
        ""
      ],
      "Adangme": [
        ""
      ],
      "Adyghe": [
        ""
      ],
      "Avestan": [
        ""
      ],
      "Tunisian Arabic": [
        ""
      ],
      "Afrikaans": [
        ""
      ],
      "Afrihili": [
        ""
      ],
      "Aghem": [
        ""
      ],
      "Ainu": [
        ""
      ],
      "Akan": [
        ""
      ],
      "Akkadian": [
        ""
      ],
      "Alabama": [
        ""
      ],
      "Aleut": [
        ""
      ],
      "Gheg Albanian": [
        ""
      ],
      "Southern Altai": [
        ""
      ],
      "Amarik": [
        ""
      ],
      "Aragonese": [
        ""
      ],
      "Old English": [
        ""
      ],
      "Angika": [
        ""
      ],
      "Modern Standard Arabic": [
        ""
      ],
      "Arabik": [
        ""
      ],
      "Aramaic": [
        ""
      ],
      "Mapuche": [
        ""
      ],
      "Araona": [
        ""
      ],
      "Arapaho": [
        ""
      ],
      "Algerian Arabic": [
        ""
      ],
      "Arawak": [
        ""
      ],
      "Moroccan Arabic": [
        ""
      ],
      "Egyptian Arabic": [
        ""
      ],
      "Asu": [
        ""
      ],
      "Assamese": [
        ""
      ],
      "American Sign Language": [
        ""
      ],
      "Asturian": [
        ""
      ],
      "Avaric": [
        ""
      ],
      "Kotava": [
        ""
      ],
      "Awadhi": [
        ""
      ],
      "Aymara": [
        ""
      ],
      "South Azerbaijani": [
        ""
      ],
      "Bashkir": [
        ""
      ],
      "Baluchi": [
        ""
      ],
      "Balinese": [
        ""
      ],
      "Bavarian": [
        ""
      ],
      "Basaa": [
        ""
      ],
      "Bamun": [
        ""
      ],
      "Batak Toba": [
        ""
      ],
      "Ghomala": [
        ""
      ],
      "Belarus kasa": [
        ""
      ],
      "Beja": [
        ""
      ],
      "Bemba": [
        ""
      ],
      "Betawi": [
        ""
      ],
      "Bena": [
        ""
      ],
      "Bafut": [
        ""
      ],
      "Badaga": [
        ""
      ],
      "Bɔlgeria kasa": [
        ""
      ],
      "Bhojpuri": [
        ""
      ],
      "Bislama": [
        ""
      ],
      "Bikol": [
        ""
      ],
      "Bini": [
        ""
      ],
      "Banjar": [
        ""
      ],
      "Kom": [
        ""
      ],
      "Siksika": [
        ""
      ],
      "Bambara": [
        ""
      ],
      "Bengali kasa": [
        ""
      ],
      "Tibetan": [
        ""
      ],
      "Bishnupriya": [
        ""
      ],
      "Bakhtiari": [
        ""
      ],
      "Braj": [
        ""
      ],
      "Breton": [
        ""
      ],
      "Brahui": [
        ""
      ],
      "Bodo": [
        ""
      ],
      "Akoose": [
        ""
      ],
      "Buriat": [
        ""
      ],
      "Buginese": [
        ""
      ],
      "Bulu": [
        ""
      ],
      "Blin": [
        ""
      ],
      "Medumba": [
        ""
      ],
      "Catalan": [
        ""
      ],
      "Caddo": [
        ""
      ],
      "Carib": [
        ""
      ],
      "Cayuga": [
        ""
      ],
      "Atsam": [
        ""
      ],
      "Cebuano": [
        ""
      ],
      "Chechen": [
        ""
      ],
      "Chiga": [
        ""
      ],
      "Chibcha": [
        ""
      ],
      "Chamorro": [
        ""
      ],
      "Chagatai": [
        ""
      ],
      "Chuukese": [
        ""
      ],
      "Mari": [
        ""
      ],
      "Chinook Jargon": [
        ""
      ],
      "Choctaw": [
        ""
      ],
      "Chipewyan": [
        ""
      ],
      "Cherokee": [
        ""
      ],
      "Cheyenne": [
        ""
      ],
      "Central Kurdish": [
        ""
      ],
      "Corsican": [
        ""
      ],
      "Coptic": [
        ""
      ],
      "Capiznon": [
        ""
      ],
      "Cree": [
        ""
      ],
      "Crimean Turkish": [
        ""
      ],
      "Kashubian": [
        ""
      ],
      "Kyɛk kasa": [
        ""
      ],
      "Church Slavic": [
        ""
      ],
      "Chuvash": [
        ""
      ],
      "Welsh": [
        ""
      ],
      "Dakota": [
        ""
      ],
      "Dargwa": [
        ""
      ],
      "Taita": [
        ""
      ],
      "Austrian German": [
        ""
      ],
      "Swiss High German": [
        ""
      ],
      "Gyaaman": [
        ""
      ],
      "Delaware": [
        ""
      ],
      "Slave": [
        ""
      ],
      "Dogrib": [
        ""
      ],
      "Dinka": [
        ""
      ],
      "Zarma": [
        ""
      ],
      "Dogri": [
        ""
      ],
      "Lower Sorbian": [
        ""
      ],
      "Central Dusun": [
        ""
      ],
      "Duala": [
        ""
      ],
      "Middle Dutch": [
        ""
      ],
      "Divehi": [
        ""
      ],
      "Jola-Fonyi": [
        ""
      ],
      "Dyula": [
        ""
      ],
      "Dzongkha": [
        ""
      ],
      "Dazaga": [
        ""
      ],
      "Embu": [
        ""
      ],
      "Ewe": [
        ""
      ],
      "Efik": [
        ""
      ],
      "Emilian": [
        ""
      ],
      "Ancient Egyptian": [
        ""
      ],
      "Ekajuk": [
        ""
      ],
      "Greek kasa": [
        ""
      ],
      "Elamite": [
        ""
      ],
      "Australian English": [
        ""
      ],
      "Borɔfo": [
        ""
      ],
      "Canadian English": [
        ""
      ],
      "British English": [
        ""
      ],
      "Middle English": [
        ""
      ],
      "American English": [
        ""
      ],
      "Esperanto": [
        ""
      ],
      "Latin American Spanish": [
        ""
      ],
      "European Spanish": [
        ""
      ],
      "Mexican Spanish": [
        ""
      ],
      "Spain kasa": [
        ""
      ],
      "Central Yupik": [
        ""
      ],
      "Basque": [
        ""
      ],
      "Ewondo": [
        ""
      ],
      "Extremaduran": [
        ""
      ],
      "Fang": [
        ""
      ],
      "Pɛɛhyia kasa": [
        ""
      ],
      "Fanti": [
        ""
      ],
      "Fulah": [
        ""
      ],
      "Tornedalen Finnish": [
        ""
      ],
      "Faroese": [
        ""
      ],
      "Fon": [
        ""
      ],
      "Canadian French": [
        ""
      ],
      "Cajun French": [
        ""
      ],
      "Swiss French": [
        ""
      ],
      "Frɛnkye": [
        ""
      ],
      "Middle French": [
        ""
      ],
      "Old French": [
        ""
      ],
      "Arpitan": [
        ""
      ],
      "Northern Frisian": [
        ""
      ],
      "Eastern Frisian": [
        ""
      ],
      "Friulian": [
        ""
      ],
      "Western Frisian": [
        ""
      ],
      "Ga": [
        ""
      ],
      "Gagauz": [
        ""
      ],
      "Gan Chinese": [
        ""
      ],
      "Gayo": [
        ""
      ],
      "Gbaya": [
        ""
      ],
      "Zoroastrian Dari": [
        ""
      ],
      "Scottish Gaelic": [
        ""
      ],
      "Geez": [
        ""
      ],
      "Gilbertese": [
        ""
      ],
      "Galician": [
        ""
      ],
      "Gilaki": [
        ""
      ],
      "Middle High German": [
        ""
      ],
      "Old High German": [
        ""
      ],
      "Goan Konkani": [
        ""
      ],
      "Gondi": [
        ""
      ],
      "Gorontalo": [
        ""
      ],
      "Gothic": [
        ""
      ],
      "Grebo": [
        ""
      ],
      "Ancient Greek": [
        ""
      ],
      "Swiss German": [
        ""
      ],
      "Wayuu": [
        ""
      ],
      "Gujarati": [
        ""
      ],
      "Frafra": [
        ""
      ],
      "Gusii": [
        ""
      ],
      "Manx": [
        ""
      ],
      "Gwichʼin": [
        ""
      ],
      "Hausa": [
        ""
      ],
      "Haida": [
        ""
      ],
      "Hakka Chinese": [
        ""
      ],
      "Hawaiian": [
        ""
      ],
      "Hebrew": [
        ""
      ],
      "Fiji Hindi": [
        ""
      ],
      "Hindi": [
        ""
      ],
      "Hiligaynon": [
        ""
      ],
      "Hittite": [
        ""
      ],
      "Hmong": [
        ""
      ],
      "Hiri Motu": [
        ""
      ],
      "Upper Sorbian": [
        ""
      ],
      "Xiang Chinese": [
        ""
      ],
      "Hangri kasa": [
        ""
      ],
      "Hupa": [
        ""
      ],
      "Herero": [
        ""
      ],
      "Interlingua": [
        ""
      ],
      "Iban": [
        ""
      ],
      "Ibibio": [
        ""
      ],
      "Indonihyia kasa": [
        ""
      ],
      "Interlingue": [
        ""
      ],
      "Igbo": [
        ""
      ],
      "Sichuan Yi": [
        ""
      ],
      "Inupiaq": [
        ""
      ],
      "Iloko": [
        ""
      ],
      "Ingush": [
        ""
      ],
      "Ido": [
        ""
      ],
      "Icelandic": [
        ""
      ],
      "Italy kasa": [
        ""
      ],
      "Inuktitut": [
        ""
      ],
      "Ingrian": [
        ""
      ],
      "Gyapan kasa": [
        ""
      ],
      "Jamaican Creole English": [
        ""
      ],
      "Lojban": [
        ""
      ],
      "Ngomba": [
        ""
      ],
      "Machame": [
        ""
      ],
      "Judeo-Persian": [
        ""
      ],
      "Judeo-Arabic": [
        ""
      ],
      "Jutish": [
        ""
      ],
      "Gyabanis kasa": [
        ""
      ],
      "Kara-Kalpak": [
        ""
      ],
      "Kabyle": [
        ""
      ],
      "Kachin": [
        ""
      ],
      "Jju": [
        ""
      ],
      "Kamba": [
        ""
      ],
      "Kawi": [
        ""
      ],
      "Kabardian": [
        ""
      ],
      "Kanembu": [
        ""
      ],
      "Tyap": [
        ""
      ],
      "Makonde": [
        ""
      ],
      "Kabuverdianu": [
        ""
      ],
      "Kenyang": [
        ""
      ],
      "Koro": [
        ""
      ],
      "Kongo": [
        ""
      ],
      "Kaingang": [
        ""
      ],
      "Khasi": [
        ""
      ],
      "Khotanese": [
        ""
      ],
      "Koyra Chiini": [
        ""
      ],
      "Khowar": [
        ""
      ],
      "Kikuyu": [
        ""
      ],
      "Kirmanjki": [
        ""
      ],
      "Kuanyama": [
        ""
      ],
      "Kako": [
        ""
      ],
      "Kazakh": [
        ""
      ],
      "Kalaallisut": [
        ""
      ],
      "Kalenjin": [
        ""
      ],
      "Kimbundu": [
        ""
      ],
      "Kambodia kasa": [
        ""
      ],
      "Kannada": [
        ""
      ],
      "Komi-Permyak": [
        ""
      ],
      "Konkani": [
        ""
      ],
      "Korea kasa": [
        ""
      ],
      "Kosraean": [
        ""
      ],
      "Kpelle": [
        ""
      ],
      "Karachay-Balkar": [
        ""
      ],
      "Krio": [
        ""
      ],
      "Kinaray-a": [
        ""
      ],
      "Kanuri": [
        ""
      ],
      "Karelian": [
        ""
      ],
      "Kurukh": [
        ""
      ],
      "Shambala": [
        ""
      ],
      "Bafia": [
        ""
      ],
      "Colognian": [
        ""
      ],
      "Kashmiri": [
        ""
      ],
      "Kurdish": [
        ""
      ],
      "Kumyk": [
        ""
      ],
      "Kutenai": [
        ""
      ],
      "Komi": [
        ""
      ],
      "Cornish": [
        ""
      ],
      "Ladino": [
        ""
      ],
      "Langi": [
        ""
      ],
      "Lahnda": [
        ""
      ],
      "Latin": [
        ""
      ],
      "Lamba": [
        ""
      ],
      "Luxembourgish": [
        ""
      ],
      "Lezghian": [
        ""
      ],
      "Lingua Franca Nova": [
        ""
      ],
      "Ganda": [
        ""
      ],
      "Ligurian": [
        ""
      ],
      "Limburgish": [
        ""
      ],
      "Livonian": [
        ""
      ],
      "Lakota": [
        ""
      ],
      "Lombard": [
        ""
      ],
      "Lingala": [
        ""
      ],
      "Lao": [
        ""
      ],
      "Mongo": [
        ""
      ],
      "Lozi": [
        ""
      ],
      "Latgalian": [
        ""
      ],
      "Luba-Lulua": [
        ""
      ],
      "Luiseno": [
        ""
      ],
      "Luba-Katanga": [
        ""
      ],
      "Lunda": [
        ""
      ],
      "Luo": [
        ""
      ],
      "Mizo": [
        ""
      ],
      "Luyia": [
        ""
      ],
      "Literary Chinese": [
        ""
      ],
      "Laz": [
        ""
      ],
      "Madurese": [
        ""
      ],
      "Mafa": [
        ""
      ],
      "Magahi": [
        ""
      ],
      "Maithili": [
        ""
      ],
      "Makasar": [
        ""
      ],
      "Mandingo": [
        ""
      ],
      "Masai": [
        ""
      ],
      "Maba": [
        ""
      ],
      "Moksha": [
        ""
      ],
      "Mandar": [
        ""
      ],
      "Mende": [
        ""
      ],
      "Meru": [
        ""
      ],
      "Morisyen": [
        ""
      ],
      "Middle Irish": [
        ""
      ],
      "Makhuwa-Meetto": [
        ""
      ],
      "Metaʼ": [
        ""
      ],
      "Micmac": [
        ""
      ],
      "Maori": [
        ""
      ],
      "Minangkabau": [
        ""
      ],
      "Malayalam": [
        ""
      ],
      "Manchu": [
        ""
      ],
      "Manipuri": [
        ""
      ],
      "Mohawk": [
        ""
      ],
      "Mossi": [
        ""
      ],
      "Western Mari": [
        ""
      ],
      "Marathi": [
        ""
      ],
      "Malay kasa": [
        ""
      ],
      "Mundang": [
        ""
      ],
      "Multiple Languages": [
        ""
      ],
      "Creek": [
        ""
      ],
      "Mirandese": [
        ""
      ],
      "Marwari": [
        ""
      ],
      "Mentawai": [
        ""
      ],
      "Bɛɛmis kasa": [
        ""
      ],
      "Myene": [
        ""
      ],
      "Erzya": [
        ""
      ],
      "Mazanderani": [
        ""
      ],
      "Min Nan Chinese": [
        ""
      ],
      "Neapolitan": [
        ""
      ],
      "Nama": [
        ""
      ],
      "Norwegian Bokmål": [
        ""
      ],
      "North Ndebele": [
        ""
      ],
      "Low German": [
        ""
      ],
      "Nɛpal kasa": [
        ""
      ],
      "Newari": [
        ""
      ],
      "Ndonga": [
        ""
      ],
      "Nias": [
        ""
      ],
      "Niuean": [
        ""
      ],
      "Ao Naga": [
        ""
      ],
      "Flemish": [
        ""
      ],
      "Dɛɛkye": [
        ""
      ],
      "Kwasio": [
        ""
      ],
      "Ngiemboon": [
        ""
      ],
      "Norwegian Nynorsk": [
        ""
      ],
      "Nogai": [
        ""
      ],
      "Old Norse": [
        ""
      ],
      "Novial": [
        ""
      ],
      "NʼKo": [
        ""
      ],
      "South Ndebele": [
        ""
      ],
      "Northern Sotho": [
        ""
      ],
      "Nuer": [
        ""
      ],
      "Navajo": [
        ""
      ],
      "Classical Newari": [
        ""
      ],
      "Nyamwezi": [
        ""
      ],
      "Nyankole": [
        ""
      ],
      "Nyanja": [
        ""
      ],
      "Nyoro": [
        ""
      ],
      "Nzima": [
        ""
      ],
      "Occitan": [
        ""
      ],
      "Ojibwa": [
        ""
      ],
      "Oromo": [
        ""
      ],
      "Oriya": [
        ""
      ],
      "Osage": [
        ""
      ],
      "Ossetic": [
        ""
      ],
      "Ottoman Turkish": [
        ""
      ],
      "Pangasinan": [
        ""
      ],
      "Pahlavi": [
        ""
      ],
      "Pampanga": [
        ""
      ],
      "Papiamento": [
        ""
      ],
      "Pungyabi kasa": [
        ""
      ],
      "Picard": [
        ""
      ],
      "Pennsylvania German": [
        ""
      ],
      "Plautdietsch": [
        ""
      ],
      "Old Persian": [
        ""
      ],
      "Palatine German": [
        ""
      ],
      "Phoenician": [
        ""
      ],
      "Pali": [
        ""
      ],
      "Pɔland kasa": [
        ""
      ],
      "Piedmontese": [
        ""
      ],
      "Pontic": [
        ""
      ],
      "Pohnpeian": [
        ""
      ],
      "Prussian": [
        ""
      ],
      "Old Provençal": [
        ""
      ],
      "Pashto": [
        ""
      ],
      "Brazilian Portuguese": [
        ""
      ],
      "Pɔɔtugal kasa": [
        ""
      ],
      "European Portuguese": [
        ""
      ],
      "Kʼicheʼ": [
        ""
      ],
      "Chimborazo Highland Quichua": [
        ""
      ],
      "Quechua": [
        ""
      ],
      "Rajasthani": [
        ""
      ],
      "Rapanui": [
        ""
      ],
      "Rarotongan": [
        ""
      ],
      "Romagnol": [
        ""
      ],
      "Riffian": [
        ""
      ],
      "Romansh": [
        ""
      ],
      "Rundi": [
        ""
      ],
      "Rombo": [
        ""
      ],
      "Moldavian": [
        ""
      ],
      "Romany": [
        ""
      ],
      "Root": [
        ""
      ],
      "Romenia kasa": [
        ""
      ],
      "Rotuman": [
        ""
      ],
      "Rusyn": [
        ""
      ],
      "Roviana": [
        ""
      ],
      "Aromanian": [
        ""
      ],
      "Rahyia kasa": [
        ""
      ],
      "Rwa": [
        ""
      ],
      "Rewanda kasa": [
        ""
      ],
      "Sandawe": [
        ""
      ],
      "Sakha": [
        ""
      ],
      "Samaritan Aramaic": [
        ""
      ],
      "Samburu": [
        ""
      ],
      "Sanskrit": [
        ""
      ],
      "Sasak": [
        ""
      ],
      "Santali": [
        ""
      ],
      "Saurashtra": [
        ""
      ],
      "Ngambay": [
        ""
      ],
      "Sangu": [
        ""
      ],
      "Sicilian": [
        ""
      ],
      "Scots": [
        ""
      ],
      "Sardinian": [
        ""
      ],
      "Sassarese Sardinian": [
        ""
      ],
      "Sindhi": [
        ""
      ],
      "Seneca": [
        ""
      ],
      "Sena": [
        ""
      ],
      "Seri": [
        ""
      ],
      "Selkup": [
        ""
      ],
      "Northern Sami": [
        ""
      ],
      "Koyraboro Senni": [
        ""
      ],
      "Old Irish": [
        ""
      ],
      "Sango": [
        ""
      ],
      "Samogitian": [
        ""
      ],
      "Tachelhit": [
        ""
      ],
      "Shan": [
        ""
      ],
      "Serbo-Croatian": [
        ""
      ],
      "Chadian Arabic": [
        ""
      ],
      "Sidamo": [
        ""
      ],
      "Sinhala": [
        ""
      ],
      "Slovak": [
        ""
      ],
      "Lower Silesian": [
        ""
      ],
      "Selayar": [
        ""
      ],
      "Southern Sami": [
        ""
      ],
      "Lule Sami": [
        ""
      ],
      "Inari Sami": [
        ""
      ],
      "Skolt Sami": [
        ""
      ],
      "Soninke": [
        ""
      ],
      "Shona": [
        ""
      ],
      "Sogdien": [
        ""
      ],
      "Somalia kasa": [
        ""
      ],
      "Sranan Tongo": [
        ""
      ],
      "Serer": [
        ""
      ],
      "Swati": [
        ""
      ],
      "Saho": [
        ""
      ],
      "Saterland Frisian": [
        ""
      ],
      "Southern Sotho": [
        ""
      ],
      "Sukuma": [
        ""
      ],
      "Susu": [
        ""
      ],
      "Sundanese": [
        ""
      ],
      "Sumerian": [
        ""
      ],
      "Sweden kasa": [
        ""
      ],
      "Comorian": [
        ""
      ],
      "Congo Swahili": [
        ""
      ],
      "Swahili": [
        ""
      ],
      "Classical Syriac": [
        ""
      ],
      "Syriac": [
        ""
      ],
      "Silesian": [
        ""
      ],
      "Tamil kasa": [
        ""
      ],
      "Tulu": [
        ""
      ],
      "Timne": [
        ""
      ],
      "Teso": [
        ""
      ],
      "Tereno": [
        ""
      ],
      "Telugu": [
        ""
      ],
      "Tetum": [
        ""
      ],
      "Taeland kasa": [
        ""
      ],
      "Tigre": [
        ""
      ],
      "Tigrinya": [
        ""
      ],
      "Tiv": [
        ""
      ],
      "Tokelau": [
        ""
      ],
      "Tsakhur": [
        ""
      ],
      "Turkmen": [
        ""
      ],
      "Klingon": [
        ""
      ],
      "Tlingit": [
        ""
      ],
      "Tagalog": [
        ""
      ],
      "Talysh": [
        ""
      ],
      "Tamashek": [
        ""
      ],
      "Tswana": [
        ""
      ],
      "Nyasa Tonga": [
        ""
      ],
      "Tok Pisin": [
        ""
      ],
      "Tɛɛki kasa": [
        ""
      ],
      "Turoyo": [
        ""
      ],
      "Taroko": [
        ""
      ],
      "Tsakonian": [
        ""
      ],
      "Tsimshian": [
        ""
      ],
      "Tsonga": [
        ""
      ],
      "Tatar": [
        ""
      ],
      "Muslim Tat": [
        ""
      ],
      "Tumbuka": [
        ""
      ],
      "Tasawaq": [
        ""
      ],
      "Twi": [
        ""
      ],
      "Tahitian": [
        ""
      ],
      "Tuvinian": [
        ""
      ],
      "Central Atlas Tamazight": [
        ""
      ],
      "Udmurt": [
        ""
      ],
      "Ugaritic": [
        ""
      ],
      "Uyghur": [
        ""
      ],
      "Ukren kasa": [
        ""
      ],
      "Umbundu": [
        ""
      ],
      "Unknown Language": [
        ""
      ],
      "Urdu kasa": [
        ""
      ],
      "Uzbek": [
        ""
      ],
      "Vai": [
        ""
      ],
      "Venetian": [
        ""
      ],
      "Veps": [
        ""
      ],
      "Venda": [
        ""
      ],
      "Viɛtnam kasa": [
        ""
      ],
      "West Flemish": [
        ""
      ],
      "Main-Franconian": [
        ""
      ],
      "Votic": [
        ""
      ],
      "Volapük": [
        ""
      ],
      "Võro": [
        ""
      ],
      "Vunjo": [
        ""
      ],
      "Walser": [
        ""
      ],
      "Wolaytta": [
        ""
      ],
      "Waray": [
        ""
      ],
      "Washo": [
        ""
      ],
      "Walloon": [
        ""
      ],
      "Warlpiri": [
        ""
      ],
      "Wolof": [
        ""
      ],
      "Wu Chinese": [
        ""
      ],
      "Kalmyk": [
        ""
      ],
      "Xhosa": [
        ""
      ],
      "Mingrelian": [
        ""
      ],
      "Soga": [
        ""
      ],
      "Yao": [
        ""
      ],
      "Yapese": [
        ""
      ],
      "Yangben": [
        ""
      ],
      "Yemba": [
        ""
      ],
      "Yiddish": [
        ""
      ],
      "Yoruba": [
        ""
      ],
      "Nheengatu": [
        ""
      ],
      "Cantonese": [
        ""
      ],
      "Zapotec": [
        ""
      ],
      "Zhuang": [
        ""
      ],
      "Blissymbols": [
        ""
      ],
      "Zeelandic": [
        ""
      ],
      "Zenaga": [
        ""
      ],
      "Standard Moroccan Tamazight": [
        ""
      ],
      "Simplified Chinese": [
        ""
      ],
      "Traditional Chinese": [
        ""
      ],
      "Kyaena kasa": [
        ""
      ],
      "Zuni": [
        ""
      ],
      "Zulu": [
        ""
      ],
      "No linguistic content": [
        ""
      ],
      "Zaza": [
        ""
      ],
      "Could not load user attention request": [
        ""
      ],
      "No notification left": [
        ""
      ],
      "Ignore": [
        ""
      ],
      "Invalid dev experiment URI.": [
        ""
      ],
      "URI: %1$s": [
        ""
      ],
      "Experiment": [
        ""
      ],
      "Insert lost denomination": [
        ""
      ],
      "Apply": [
        ""
      ],
      "Pending refresh": [
        ""
      ],
      "Could not load the transfer payment status": [
        ""
      ],
      "Could not load the status of the order template": [
        ""
      ],
      "required": [
        ""
      ],
      "Review order": [
        ""
      ],
      "Missing URI": [
        ""
      ],
      "Check the link.": [
        ""
      ],
      "Could not parse the recovery URI": [
        ""
      ],
      "Try another URI": [
        ""
      ],
      "Import backup, show info": [
        ""
      ],
      "Add new exchange": [
        ""
      ],
      "Add exchange for %1$s": [
        ""
      ],
      "Enter the URL of an exchange you trust.": [
        ""
      ],
      "An exchange has been found! Review the information and click next": [
        ""
      ],
      "This exchange is already in your list.": [
        ""
      ],
      "Only exchange accessible through \"http\" and \"https\" are allowed.": [
        ""
      ],
      "This exchange protocol version is not supported: \" %1$s\".": [
        ""
      ],
      "This exchange currency \"%1$s\" doesn&apos;t match the expected currency %2$s.": [
        ""
      ],
      "No exchange found in that URL.": [
        ""
      ],
      "loading": [
        ""
      ],
      "Version": [
        ""
      ],
      "Can't use the URL: \"%1$s\"": [
        ""
      ],
      "Exchange URL": [
        ""
      ],
      "Add new contact": [
        ""
      ],
      "Enter the details of a contact you trust.": [
        ""
      ],
      "Contact info valid.": [
        ""
      ],
      "This contact is already in your list.": [
        ""
      ],
      "Review contact": [
        ""
      ],
      "Contact alias": [
        ""
      ],
      "Alias type": [
        ""
      ],
      "Mailbox service": [
        ""
      ],
      "Mailbox address": [
        ""
      ],
      "Contact source": [
        ""
      ],
      "Could not load the list of contacts": [
        ""
      ],
      "Source": [
        ""
      ],
      "Send Cash": [
        ""
      ],
      "Request Cash": [
        ""
      ],
      "No contacts found.": [
        ""
      ],
      "Could not bank choices for account": [
        ""
      ],
      "Could not load the list of messages": [
        ""
      ],
      "Unexpected error when trying to initialize mailbox: %1$s": [
        ""
      ],
      "Unexpected error when trying to delete message: %1$s": [
        ""
      ],
      "Unexpected error when trying to fetch messages: %1$s": [
        ""
      ],
      "Date": [
        ""
      ],
      "You may copy your mailbox URI from below and register it at %1$s with your alias(es) to make it easier for others to find you.": [
        ""
      ],
      "copy uri": [
        ""
      ],
      "hide uri": [
        ""
      ],
      "show uri": [
        ""
      ],
      "Fetch messages": [
        ""
      ],
      "Mailbox not initialized": [
        ""
      ],
      "Initialize mailbox": [
        ""
      ],
      "No messages yet.": [
        ""
      ],
      "Taler URI handler": [
        ""
      ],
      "Could not found a handler for the Taler URI": [
        ""
      ],
      "The uri read in the path parameter is not valid: \"%1$s\"": [
        ""
      ],
      "Digital cash payment": [
        ""
      ],
      "Digital cash refund": [
        ""
      ],
      "Digital cash withdrawal": [
        ""
      ],
      "Digital cash deposit": [
        ""
      ],
      "Digital cash invoice": [
        ""
      ],
      "Digital cash transfer": [
        ""
      ],
      "Digital cash recovery": [
        ""
      ],
      "Development experiment": [
        ""
      ],
      "Return to wallet": [
        ""
      ],
      "Network is offline": [
        ""
      ],
      "Diagnostics timed out. Could not talk to the wallet backend.": [
        ""
      ],
      "Problems detected:": [
        ""
      ],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).": [
        ""
      ],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.": [
        ""
      ],
      "here": [
        ""
      ],
      "Running diagnostics": [
        ""
      ],
      "Edit": [
        ""
      ],
      "List of products": [
        ""
      ],
      "free": [
        ""
      ],
      "Could not get provider information": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=4; plural=(n == 1) ? 0 : ((n == 2) ? 1 : ((n > 10 && n % 10 == 0) ? 2 : 3));",
  "lang": "he",
  "completeness": 0
};

strings['fr'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=(n!=1);",
        "lang": "fr"
      },
      "Balance": [
        "Solde"
      ],
      "Backup": [
        "Sauvegarde"
      ],
      "Notifications": [
        ""
      ],
      "QR Reader and Taler URI": [
        "Lecteur de QR-code et d'URI Taler"
      ],
      "Settings": [
        "Configuration"
      ],
      "Dev tools": [
        "Outils de débogage"
      ],
      "Contacts": [
        ""
      ],
      "Mailbox": [
        ""
      ],
      "Search transactions": [
        "Frais de transaction"
      ],
      "%1$s": [
        "%1$s"
      ],
      "KYC requirement": [
        ""
      ],
      "PENDING OPERATIONS": [
        "OPERATIONS EN ATTENTE"
      ],
      "Error was thrown trying to: \"%1$s\"": [
        ""
      ],
      "Could not open the wallet database. Firefox is known to run into this problem under \"permanent private mode\".": [
        ""
      ],
      "Could not open the wallet database.": [
        "Impossible d'analyser l'URI payto"
      ],
      "Unexpected request error, code: %1$s": [
        ""
      ],
      "more info": [
        ""
      ],
      "wrap text": [
        ""
      ],
      "copy content": [
        ""
      ],
      "less info": [
        ""
      ],
      "Retry operation": [
        "Opérations du portefeuille"
      ],
      "Loading": [
        "Chargement"
      ],
      "Could not load backup providers": [
        "Impossible de charger les prestataires de sauvegarde"
      ],
      "No backup providers configured": [
        "Aucun prestataire de sauvegarde n'est configuré"
      ],
      "Add provider": [
        "Ajouter un prestataire"
      ],
      "Sync all backups": [
        "Synchroniser les sauvegardes"
      ],
      "Sync now": [
        "Synchroniser maintenant"
      ],
      "Last synced": [
        "Dernière synchronisation"
      ],
      "Not synced": [
        "Pas synchronisé"
      ],
      "Expires in": [
        "Expire dans"
      ],
      "show less": [
        "afficher le code QR"
      ],
      "show more": [
        "afficher le code QR"
      ],
      "There was an error loading the provider detail for &quot;%1$s&quot;": [
        "Une erreur s'est produite lors du chargement du détail du prestataire pour &quot; %1$s&quot;"
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [
        "Aucun prestataire n'est connu avec l'URL &quot;%1$s&quot;."
      ],
      "See providers": [
        "Voir les prestataires"
      ],
      "Last backup": [
        "Dernière sauvegarde"
      ],
      "Back up": [
        "Sauvegarde"
      ],
      "Provider fee": [
        "Frais du prestataire"
      ],
      "per year": [
        "par année"
      ],
      "Extend": [
        "Étendre"
      ],
      "terms has changed, extending the service will imply accepting the new terms of service": [
        "Les conditions ont changé, l'extension du service implique l'acceptation des nouvelles conditions d'utilisation"
      ],
      "old": [
        "ancien"
      ],
      "new": [
        "nouveau"
      ],
      "fee": [
        "frais"
      ],
      "storage": [
        "stockage"
      ],
      "Remove provider": [
        "Supprimer le prestataire"
      ],
      "Pay": [
        "Paiement"
      ],
      "Withdraw": [
        "Retrait"
      ],
      "This provider has reported an error": [
        "Ce prestataire a signalé une erreur"
      ],
      "There is conflict with another backup from &quot;%1$s&quot;": [
        "Il y a un conflit avec une autre sauvegarde du %1$s"
      ],
      "Backup is not readable": [
        "Sauvegarde non lisible"
      ],
      "Unknown backup problem: %1$s": [
        "Problème inconnu de sauvegarde : %1$s"
      ],
      "service paid": [
        "service payé"
      ],
      "Backup valid until": [
        "Sauvegarde valide jusqu'à"
      ],
      "Cancel": [
        "Annuler"
      ],
      "Open pay page": [
        "Ouvrir la page de paiement"
      ],
      "Open refund page": [
        "Ouvrir la page de remboursement"
      ],
      "Open withdraw page": [
        "Ouvrir la page de retrait"
      ],
      "Your wallet is empty.": [
        ""
      ],
      "Get digital cash": [
        "Obtenir de l'argent numérique"
      ],
      "Try the demo bank and withdraw test money.": [
        ""
      ],
      "Could not load the balance": [
        "Impossible de charger le solde du dépôt"
      ],
      "Receive": [
        "Reçu"
      ],
      "Send %1$s": [
        "Envoyé %1$s"
      ],
      "This page has a withdrawal action.": [
        "Cette page comporte une action de retrait."
      ],
      "This page has pay action.": [
        "Cette page comporte une action de paiement."
      ],
      "This page has a refund action.": [
        "Cette page comporte une action de remboursement."
      ],
      "This page has a add exchange action.": [
        "Cette page comporte une action de remboursement."
      ],
      "Open add exchange page": [
        "Ajouter un prestataire de services de paiement"
      ],
      "This page has an add contact action.": [
        "Cette page comporte une action de remboursement."
      ],
      "Open add contact page": [
        "Ouvrir la page de paiement"
      ],
      "Taler Action": [
        "Action de Taler"
      ],
      "This page has a malformed taler uri.": [
        "Cette page contient une URI Taler mal formée."
      ],
      "Dismiss": [
        "Rejeter"
      ],
      "this popup is being closed and you are being redirected to %1$s": [
        "ce popup est fermé et vous êtes redirigé vers %1$s"
      ],
      "Could not load purchase proposal details": [
        "Impossible de charger les détails de la proposition d'achat"
      ],
      "Order Id": [
        "Identifiant de la commande"
      ],
      "Summary": [
        "Résumé"
      ],
      "Amount": [
        "Montant"
      ],
      "Choices": [
        ""
      ],
      "This purchase have %1$s choices": [
        ""
      ],
      "Merchant name": [
        "Nom du commerçant"
      ],
      "Merchant jurisdiction": [
        "Juridiction du commerçant"
      ],
      "Merchant address": [
        "Adresse du commerçant"
      ],
      "Merchant logo": [
        "Logo du commerçant"
      ],
      "Merchant website": [
        "Site web du commerçant"
      ],
      "Merchant email": [
        "Email du commerçant"
      ],
      "Merchant public key": [
        "Clé publique du commerçant"
      ],
      "Created at": [
        "Créé à"
      ],
      "Refund deadline": [
        "Date limite de remboursement"
      ],
      "Auto refund": [
        "Remboursement automatique"
      ],
      "Pay deadline": [
        "Délai de paiement"
      ],
      "Fulfillment URL": [
        "URL du service de traitement des commandes"
      ],
      "Fulfillment message": [
        "Message après exécution"
      ],
      "Max deposit fee": [
        "Frais de dépôt maximum"
      ],
      "Exchanges": [
        "Prestataire de paiement"
      ],
      "Could not load the status of the term of service": [
        "Impossible de charger la liste des transactions"
      ],
      "Select one option": [
        "Sélectionnez une option"
      ],
      "Show terms of service": [
        "Afficher les Conditions Générales d'Utilisation"
      ],
      "I accept the exchange terms of service": [
        "J'accepte les Conditions Générales d'Utilisation de l'exchange"
      ],
      "Review exchange terms of service": [
        "Consulter les conditions d'utilisation de l'exchange"
      ],
      "The exchange doesn't have a terms of service.": [
        "Exchange n&apos;a pas de conditions d&apos;utilisation"
      ],
      "Skip it for now.": [
        ""
      ],
      "Format": [
        ""
      ],
      "Language": [
        "Langage actuel"
      ],
      "The exchange replied with a empty terms of service": [
        "L'exchange a répondu avec des conditions d'utilisations vides"
      ],
      "No terms of service. The exchange replied with a empty document": [
        "L'exchange a répondu avec des conditions d'utilisations vides"
      ],
      "No terms of service. The exchange replied with a empty text": [
        "L'exchange a répondu avec des conditions d'utilisations vides"
      ],
      "Download Terms of Service": [
        "Télécharger les conditions d'utilisation"
      ],
      "Hide terms of service": [
        "Cacher les conditions d'utilisation"
      ],
      "Amount is not valid": [
        ""
      ],
      "Bank account": [
        "Compte bancaire"
      ],
      "Bitcoin address": [
        "Adresse Bitcoin"
      ],
      "BIC/IBAN": [
        "IBAN"
      ],
      "IBAN": [
        "IBAN"
      ],
      "Bitcoin transfer details": [
        "Details du transfer de Bitcoins"
      ],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.": [
        "L'exchange a besoin d'une transaction avec 3 sorties, une sortie est le compte de l'exchange et les deux autres sont de fausses adresses segwit pour les métadonnées avec un montant minimal."
      ],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts": [
        "Dans le portefeuille bitcoincore, utilisez le bouton &apos;Add Recipient&apos ; pour ajouter deux destinataires supplémentaires et copier les adresses et les montants"
      ],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC": [
        "Assurez-vous que le montant indique %1$s BTC, sinon vous devez changer l'unité pour le BTC"
      ],
      "Ethereum transfer details": [
        "Détails du virement bancaire"
      ],
      "You need to wire to the service provider account.": [
        ""
      ],
      "Make sure the amount show %1$s ETH": [
        ""
      ],
      "Bank transfer details": [
        "Détails du virement bancaire"
      ],
      "Account": [
        "Compte"
      ],
      "Bank host": [
        "Serveur de la banque"
      ],
      "BIC": [
        ""
      ],
      "Exchange": [
        "Exchange"
      ],
      "Reserve Pub": [
        ""
      ],
      "Step 1:": [
        ""
      ],
      "Copy this code and paste it into the subject/purpose field in your banking app or bank website": [
        ""
      ],
      "Subject": [
        "Référence"
      ],
      "Step 2:": [
        ""
      ],
      "If you don't already have it in your banking favourites list, then copy and paste this IBAN and the name into the receiver fields in your banking app or website": [
        ""
      ],
      "Receiver name": [
        "Nom du bénéficiaire"
      ],
      "Receiver postal code": [
        "Nom du bénéficiaire"
      ],
      "Receiver town": [
        "Nom du bénéficiaire"
      ],
      "Step 3:": [
        ""
      ],
      "Finish the wire transfer setting the amount in your banking app or website, then this withdrawal will proceed automatically.": [
        ""
      ],
      "Make sure ALL data is correct, including the subject; otherwise, the money will not arrive in this wallet. You can use the copy buttons (%1$s) to prevent typing errors or the \"payto://\" URI below to copy just one value.": [
        ""
      ],
      "Alternative if your bank already supports PayTo URI, you can use this %1$s link instead": [
        ""
      ],
      "Continue with banking app or website": [
        ""
      ],
      "Show QR code": [
        ""
      ],
      "Could not load transaction information": [
        "Impossible de charger les informations relatives à la transaction"
      ],
      "Bank account confirmation required for the transaction to complete.": [
        ""
      ],
      "KYC check required for the transaction to complete.": [
        ""
      ],
      "No additional information has been provided.": [
        ""
      ],
      "Follow this link to the%1$s %2$s": [
        ""
      ],
      "This transaction is pending to complete.": [
        "Cette transaction n’est pas finalisée"
      ],
      "Retry": [
        "Réessayer"
      ],
      "There was an error trying to complete the transaction.": [
        "Une erreur s'est produite lors de l'exécution de la transaction"
      ],
      "This transaction was aborted.": [
        "Cette transaction n’est pas finalisée"
      ],
      "This transaction failed.": [
        "Cette transaction n’est pas finalisée"
      ],
      "Caution!": [
        "Attention !"
      ],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.": [
        "Si vous avez déjà transféré de l'argent à l'exchange, vous ne pourrez pas récupérer les pièces."
      ],
      "Confirm": [
        "Confirmer"
      ],
      "Doing a cancellation while the transaction still active might result in lost coins. Do you still want to cancel the transaction?": [
        ""
      ],
      "No": [
        ""
      ],
      "Yes": [
        ""
      ],
      "Abort": [
        ""
      ],
      "Resume": [
        ""
      ],
      "Suspend": [
        ""
      ],
      "Delete": [
        ""
      ],
      "Withdrawal": [
        "Retrait"
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Select one of the accounts and use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Details": [
        "Détails"
      ],
      "Withdrawal incomplete.": [
        "Retrait"
      ],
      "If you have already sent money to the service provider account it will wire it back at %1$s": [
        "Si vous avez déjà transféré de l'argent à l'exchange, vous ne pourrez pas récupérer les pièces."
      ],
      "Payment": [
        "Paiement"
      ],
      "Refunds": [
        "Remboursements"
      ],
      "%1$s %2$s on %3$s .": [
        "%1$s %2$s sur %3$s"
      ],
      "Refund is in progress.": [
        "Le remboursement est en cours."
      ],
      "Merchant created a refund for this order but was not automatically picked up.": [
        "Le commerçant a créé un remboursement pour cette commande, mais il n'a pas été prélevé automatiquement."
      ],
      "Offer": [
        "Offre"
      ],
      "Accept": [
        "Accepter"
      ],
      "Confirmation code": [
        ""
      ],
      "Merchant": [
        "Commerçant"
      ],
      "Invoice ID": [
        "Identifiant de la facture"
      ],
      "Deposit": [
        "Dépôt"
      ],
      "Wire transfer deadline.": [
        "Details du transfer de Bitcoins"
      ],
      "Wire transfer still pending.": [
        ""
      ],
      "The service provider deadline to make the wire transfer is: %1$s": [
        ""
      ],
      "Wire transfer completed.": [
        ""
      ],
      " ": [
        ""
      ],
      "Transfer details": [
        "Détails du virement bancaire"
      ],
      "Wire transfer in progress.": [
        "Le remboursement est en cours."
      ],
      "Refresh": [
        "Actualiser"
      ],
      "Refund": [
        "Remboursement"
      ],
      "Purchase summary": [
        "Récapitulatif des achats"
      ],
      "Credit": [
        "Crédit"
      ],
      "Invoice": [
        "Facture"
      ],
      "URI": [
        "URI"
      ],
      "Debit": [
        "Débit"
      ],
      "Transfer": [
        "Transfert"
      ],
      "Lost": [
        ""
      ],
      "Reason": [
        ""
      ],
      "Denomination expired.": [
        "Dénomination"
      ],
      "Denomination vanished.": [
        "Dénomination"
      ],
      "Denomination is unoffered.": [
        "Dénomination"
      ],
      "Fees": [
        ""
      ],
      "Total": [
        "Total"
      ],
      "Sent": [
        ""
      ],
      "Converted": [
        ""
      ],
      "Bank fee": [
        "Frais maximums"
      ],
      "Price": [
        "Prix"
      ],
      "Subtotal": [
        "Total"
      ],
      "Refunded": [
        "Remboursée"
      ],
      "Transfer identification": [
        "Opérations de transfert"
      ],
      "copy": [
        "copier"
      ],
      "hide qr": [
        "cacher le code QR"
      ],
      "forward to contact": [
        ""
      ],
      "show qr": [
        "afficher le code QR"
      ],
      "Wire transfer need a confirmation. Go to the %1$s and check wire transfer operation to exchange account is complete.": [
        ""
      ],
      "bank site": [
        "Serveur de la banque"
      ],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins.": [
        "La banque a confirmé le virement bancaire. En attente de l'envoi des pièces par l'exchange"
      ],
      "Exchange is ready to send the coins, withdrawal in progress.": [
        "L'exchange est prêt pour le retrait"
      ],
      "Send &nbsp; %1$s": [
        "Envoyer &nbsp; %1$s"
      ],
      "Pay &nbsp; %1$s": [
        "Payer &nbsp; %1$s"
      ],
      "Merchant message": [
        "Message du commerçant"
      ],
      "Balance is not enough.": [
        "Votre solde actuel est insuffisant."
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which exchange and auditors the merchant trust.": [
        ""
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which wire methods the merchant accepts.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to pay for this contract which is restricted.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to spend right know. There are some coins that need to be refreshed.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s available.": [
        ""
      ],
      "Provider is missing the global fee configuration, this likely means it is misconfigured.": [
        ""
      ],
      "Balance looks like it should be enough, but doesn't cover all fees requested by the merchant and payment processor. Please ensure there is at least %1$s %2$s more balance in your wallet or ask your merchant to cover more of the fees.": [
        ""
      ],
      "Pay with a mobile phone": [
        "Payer avec un téléphone portable"
      ],
      "Hide QR": [
        "Cacher le code QR"
      ],
      "Scan the QR code or &nbsp; %1$s": [
        "Scannez le code QR ou &nbsp; %1$s"
      ],
      "click here": [
        "Cliquez pour afficher"
      ],
      "Could not load the payment and balance status": [
        "Impossible de charger l’état du paiement"
      ],
      "Purchase": [
        "Achat"
      ],
      "This purchase is age restricted.": [
        ""
      ],
      "Expires at": [
        "Expire dans"
      ],
      "Already paid, you are going to be redirected to %1$s": [
        "Déjà payé, vous allez être redirigé vers %1$s"
      ],
      "Already paid": [
        "Déjà payé"
      ],
      "Already claimed": [
        "Déjà réclamé"
      ],
      "Could not load the refund status": [
        "Impossible de charger l’état du remboursement"
      ],
      "You&apos;ve ignored the refund.": [
        "Vous avez ignoré le pourboire."
      ],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [
        "Le commerçant &quot;%1$s&quot; vous propose un remboursement."
      ],
      "Order amount": [
        "Montant de la commande"
      ],
      "Could not load the list of exchanges": [
        "Impossible de charger la liste de l'exchange"
      ],
      "Could not load info from URI": [
        "Impossible d'obtenir les informations de l'URI"
      ],
      "Could not load the withdrawal details": [
        "Impossible de charger les détails de la proposition d'achat"
      ],
      "under %1$s": [
        "Envoyé %1$s"
      ],
      "should be greater than zero": [
        ""
      ],
      "can't be greater than %1$s": [
        ""
      ],
      "Could not load exchange details info": [
        "Impossible de charger les frais de l'exchange"
      ],
      "Close": [
        "Fermer"
      ],
      "There is no exchange available for currency %1$s": [
        "Votre solde est insuffisant pour effectuer un dépôt dans la devise %1$s"
      ],
      "You can add more exchanges from the settings.": [
        ""
      ],
      "Exchange %1$s is not available": [
        ""
      ],
      "Exchange status can view accessed from the settings.": [
        ""
      ],
      "Service fee description": [
        "Description des frais de service"
      ],
      "Select %1$s exchange": [
        "Sélectionnez le prestataire de services de paiement %1$s"
      ],
      "Reset": [
        "Réinitialisation"
      ],
      "Use this exchange": [
        "Utiliser ce prestataire de services de paiement"
      ],
      "Auditors": [
        "Auditeurs"
      ],
      "Doesn&apos;t have auditors": [
        "N'a pas d'auditeurs"
      ],
      "currency": [
        "devise"
      ],
      "Coin operations": [
        "Opérations sur les pièces"
      ],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.": [
        "Chaque opération de cette section peut être différente par la valeur de la dénomination et est valide pour une certaine période. L'exchange prélèvera le montant indiqué chaque fois qu'une pièce est utilisée dans le cadre de cette opération."
      ],
      "Deposits": [
        "Dépôts"
      ],
      "Denomination": [
        "Dénomination"
      ],
      "Current": [
        "Devise"
      ],
      "Selected": [
        ""
      ],
      "Until": [
        "﻿Jusqu'à"
      ],
      "Withdrawals": [
        "Retraits"
      ],
      "Transfer operations": [
        "Opérations de transfert"
      ],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.": [
        "Chaque opération décrite dans cette section peut être différente selon le type de transfert et est valable pour une période donnée. L'exchange facturera le montant indiqué à chaque fois qu'un transfert sera effectué."
      ],
      "Operation": [
        "Opération"
      ],
      "Fee": [
        "Frais"
      ],
      "Wallet operations": [
        "Opérations du portefeuille"
      ],
      "Feature": [
        "Fonctionnalité"
      ],
      "Currency": [
        "Devise"
      ],
      "This operation has already been completed.": [
        "Cette transaction n’est pas finalisée"
      ],
      "This operation has already been aborted": [
        ""
      ],
      "This operation has started and should be completed in the bank.": [
        ""
      ],
      "You can confirm the operation in": [
        "Impossible de terminer le retrait"
      ],
      "this page": [
        "Ouvrir la page de conseils"
      ],
      "This operation has already been completed by another wallet.": [
        ""
      ],
      "This operation has already been used by another wallet.": [
        ""
      ],
      "It can be confirmed in": [
        "Aucun exchange de configuré"
      ],
      "Age restriction": [
        "Restriction d'âge"
      ],
      "Withdraw &nbsp; %1$s": [
        "Retirer&nbsp;%1$s"
      ],
      "Withdraw to a mobile phone": [
        "Retrait vers un téléphone portable"
      ],
      "Short description of the invoice": [
        "Préciser la destination de l'argent"
      ],
      "Create": [
        "Créer"
      ],
      "Short description of the transfer": [
        "Préciser la destination de l'argent"
      ],
      "Could not load the invoice payment status": [
        "Impossible de charger l’état du paiement"
      ],
      "Receive &nbsp; %1$s": [
        "Recevoir &nbsp; %1$s"
      ],
      "Could not load known bank accounts": [
        "Impossible de charger l’état du paiement"
      ],
      "Could not load supported wire methods": [
        "Impossible de charger les prestataires de sauvegarde"
      ],
      "No wire methods supported for this currency": [
        ""
      ],
      "Try another wire method.": [
        "Vers un autre portefeuille"
      ],
      "Known accounts for %1$s": [
        "Ajouter un compte bancaire pour %1$s"
      ],
      "To add a new account first select the account type.": [
        ""
      ],
      "Unable add this account": [
        "Impossible d'ajouter ce compte"
      ],
      "Add": [
        "Ajouter"
      ],
      "IBAN accounts": [
        "Ajouter un compte"
      ],
      "Alias": [
        ""
      ],
      "Int. Account Number": [
        ""
      ],
      "Account name": [
        "Compte"
      ],
      "KYC": [
        ""
      ],
      "Taler accounts": [
        "Sélectionner un compte"
      ],
      "Host": [
        ""
      ],
      "Bitcoin accounts": [
        "Compte bancaire"
      ],
      "Address": [
        "Lignes de l'adresse"
      ],
      "Can't be empty": [
        ""
      ],
      "Bitcoin Account": [
        "Compte bancaire"
      ],
      "Taler Bank": [
        "Action de Taler"
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "Required": [
        ""
      ],
      "International Bank Account Number": [
        ""
      ],
      "Could not load balance information": [
        "Impossible de charger les informations de récupération de sauvegarde"
      ],
      "Could not load fee for amount %1$s": [
        "Impossible de charger l’état du remboursement"
      ],
      "A currency or an amount should be indicated": [
        "Une devise ou un montant doit être indiqué"
      ],
      "There is no enough balance to make a deposit for currency %1$s": [
        "Votre solde est insuffisant pour effectuer un dépôt dans la devise %1$s"
      ],
      "Send %1$s to your account": [
        "Envoyer %1$s sur votre compte"
      ],
      "There is no account to make a deposit for currency %1$s": [
        "Il n'y a pas de compte vers lequel effectuer un dépôt dans la devise %1$s"
      ],
      "Add account": [
        "Ajouter un compte"
      ],
      "Select account": [
        "Sélectionner un compte"
      ],
      "Manage accounts": [
        "Compte bancaire"
      ],
      "Brut amount": [
        "Montant de la commande"
      ],
      "Deposit fee": [
        "Frais de dépôt"
      ],
      "Net amount": [
        "Montant de la commande"
      ],
      "Deposit&nbsp;%1$s %2$s": [
        "Dépôt&nbsp;%1$s %2$s"
      ],
      "Need approval in the Bank": [
        ""
      ],
      "Waiting for wire transfer to complete": [
        ""
      ],
      "Exchange is waiting the wire transfer": [
        ""
      ],
      "Payment in progress": [
        "Le remboursement est en cours."
      ],
      "Executing refund...": [
        "Remboursement automatique"
      ],
      "Refreshing coins...": [
        ""
      ],
      "Deposit in progress": [
        "Le remboursement est en cours."
      ],
      "Waiting to be paid": [
        ""
      ],
      "Receiving the transfer": [
        "Transfert d'argent numérique"
      ],
      "Waiting to be received": [
        "À recevoir"
      ],
      "Denomination expired": [
        "Dénomination"
      ],
      "Denomination vanished": [
        "Dénomination"
      ],
      "Denomination unoffered": [
        "Dénomination"
      ],
      "ABORTED": [
        ""
      ],
      "FAILED": [
        ""
      ],
      "Could not load the list of transactions": [
        "Impossible de charger la liste des transactions"
      ],
      "unknown scope": [
        ""
      ],
      "Send": [
        "Envoyer"
      ],
      "Your transaction history is empty for this currency.": [
        "L'historique de vos transactions est vide pour cette devise."
      ],
      "Review terms of service": [
        "Consulter les conditions d'utilisation"
      ],
      "Provider URL": [
        "URL du prestataire"
      ],
      "Please review and accept this provider&apos;s terms of service": [
        "Veuillez consulter et accepter les conditions générales d'utilisation de ce prestataire de service"
      ],
      "Pricing": [
        "Tarification"
      ],
      "free of charge": [
        "gratuit"
      ],
      "%1$s per year of service": [
        "%1$s par année de service"
      ],
      "Storage": [
        "Stockage"
      ],
      "%1$s megabytes of storage per year of service": [
        "%1$s mégaoctets de stockage par année de service"
      ],
      "Accept terms of service": [
        "Accepter les conditions générales d'utilisation"
      ],
      "Add backup provider": [
        "Ajouter un prestataire de service de sauvegarde"
      ],
      "Backup providers may charge for their service": [
        "Les prestataires de service de sauvegarde peuvent facturer leur service"
      ],
      "URL": [
        "URL"
      ],
      "Name": [
        "Nom"
      ],
      "Next": [
        "Suivant"
      ],
      "Navigator": [
        "Navigateur"
      ],
      "Automatically open wallet": [
        "Ouvrir automatiquement le portefeuille en fonction du contenu de la page"
      ],
      "Open the wallet when a payment action is found.": [
        ""
      ],
      "Version Info": [
        "Version"
      ],
      "Web Extension": [
        "Extension Web"
      ],
      "The version of wallet core is not supported. (supported version: %1$s, wallet version: %2$s)": [
        ""
      ],
      "Exchange compatibility": [
        "Compatibilité de l'exchange"
      ],
      "Merchant compatibility": [
        "Compatibilité du système pour les commerçants"
      ],
      "Bank compatibility": [
        "Compatibilité du système bancaire"
      ],
      "Wallet Core compatibility": [
        "Compatibilité du système bancaire"
      ],
      "Enable developer mode": [
        "Mode dévelopeur"
      ],
      "Show more information and options in the UI": [
        ""
      ],
      "Display": [
        "Affichage"
      ],
      "Current Language": [
        "Langage actuel"
      ],
      "Show backup feature": [
        ""
      ],
      "Backup integration still in beta.": [
        ""
      ],
      "Show P2P alias feature": [
        ""
      ],
      "P2P aliases integration still in beta.": [
        ""
      ],
      "Show suspend/resume transaction": [
        ""
      ],
      "Prevent transaction from doing network request.": [
        ""
      ],
      "Show refresh transaction type in the transaction list": [
        ""
      ],
      "Refresh transaction will be hidden by default if the refresh operation doesn't have fee.": [
        ""
      ],
      "Show more account types on deposit": [
        ""
      ],
      "Extends the UI to more payment target types.": [
        ""
      ],
      "Show JSON on error": [
        ""
      ],
      "Print more information about the error. Useful for debugging.": [
        "Plus d'options et d'informations utiles pour le débogage"
      ],
      "Allow HTTP connections": [
        ""
      ],
      "Using HTTP connection may be faster but unsafe (wallet restart required)": [
        ""
      ],
      "Lang selector": [
        ""
      ],
      "Allows to manually change the language of the UI. Otherwise it will be automatically selected by your browser configuration.": [
        ""
      ],
      "Edit exchange management": [
        "Pas encore d'exchange"
      ],
      "Allows to see the list of exchange, remove, add and switch before withdrawal.": [
        ""
      ],
      "Select terms of service format": [
        "Accepter les conditions générales d'utilisation"
      ],
      "Allows to render the terms of service on different format selected by the user.": [
        ""
      ],
      "Show wallet activity": [
        ""
      ],
      "Show the wallet notification and observability event in the UI.": [
        ""
      ],
      "Enable v1 contract": [
        ""
      ],
      "v1 is under development and wallet-core converts v1 contract to v0. Enable this options for testing": [
        ""
      ],
      "GNU Taler Wallet installed!": [
        ""
      ],
      "You can open the wallet using the combination %1$s .": [
        "Vous pouvez ouvrir le portefeuille GNU Taler en utilisant la combinaison %1$s ."
      ],
      "Also pinning the GNU Taler Wallet to your browser allows you to quick access without keyboard:": [
        "Vous pouvez également épingler le portefeuille GNU Taler à votre navigateur Chrome, ce qui vous permet d'y accéder rapidement sans clavier :"
      ],
      "Click the puzzle icon": [
        "Cliquez sur l'icône du puzzle"
      ],
      "Search for GNU Taler Wallet": [
        "Rechercher le portefeuille GNU Taler"
      ],
      "Click the pin icon": [
        "Cliquez sur l'icône de l'épingle"
      ],
      "Next Steps": [
        "Prochaines Etapes"
      ],
      "Try the demo": [
        "Essayez la démonstration"
      ],
      "Click here to open the wallet activity tab.": [
        ""
      ],
      "Events": [
        ""
      ],
      "Active tasks": [
        ""
      ],
      "All events": [
        ""
      ],
      "Balance change": [
        "Solde"
      ],
      "Backup failed": [
        "Sauvegarde valide jusqu'à"
      ],
      "Transaction updated": [
        "Frais de transaction"
      ],
      "Exchange updated": [
        "Exchange"
      ],
      "Idle": [
        ""
      ],
      "task.%1$s": [
        "%1$s"
      ],
      "wallet.%1$s(%2$s)": [
        ""
      ],
      "Bank account info changed": [
        "Compte bancaire"
      ],
      "Contact added": [
        ""
      ],
      "Contact deleted": [
        ""
      ],
      "Mailbox message added": [
        ""
      ],
      "Mailbox message deleted": [
        ""
      ],
      "Type": [
        ""
      ],
      "Id": [
        ""
      ],
      "Since": [
        ""
      ],
      "Next try": [
        "Prochaines Etapes"
      ],
      "Error": [
        ""
      ],
      "Transaction": [
        "Frais de transaction"
      ],
      "No exchange yet": [
        "Pas encore d'exchange"
      ],
      "Status": [
        "état"
      ],
      "Terms of Service": [
        "Conditions d'utilisation"
      ],
      "Last Update": [
        ""
      ],
      "Actions": [
        "Action de Taler"
      ],
      "ok": [
        "ok"
      ],
      "pending": [
        ""
      ],
      "proposed": [
        ""
      ],
      "unknown (exchange status should be updated)": [
        "inconnu (le statut de l'échange doit être mis à jour)"
      ],
      "Reload": [
        "chargement"
      ],
      "Purge": [
        ""
      ],
      "Make regional": [
        ""
      ],
      "Make global": [
        ""
      ],
      "Forget ToS": [
        "Oublier"
      ],
      "Debug tools": [
        "Outils de débogage"
      ],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?": [
        "Voulez-vous DÉTRUIRE IRRÉVOCABLEMENT tout ce qui se trouve dans votre portefeuille et PERDRE TOUTES VOS PIÈCES ?"
      ],
      "reset": [
        "réinitialiser"
      ],
      "TESTING: This may delete all your coin, proceed with caution": [
        "TEST : Cela peut supprimer toutes vos pièces, procédez avec prudence"
      ],
      "run gc": [
        "exécuter gc"
      ],
      "import database": [
        "Importer une base de données"
      ],
      "export database": [
        "exporter la base de donnée"
      ],
      "Clear exchange key cache": [
        "Pas encore d'exchange"
      ],
      "Database exported at %1$s %2$s to download": [
        "Base de données exportée à %1$s %2$s pour être téléchargée"
      ],
      "Inject Taler support in all pages": [
        ""
      ],
      "Enabling this option will make `window.taler` be available in all sites": [
        ""
      ],
      "Exchange Entries": [
        "Prestataire de paiement"
      ],
      "Add an exchange": [
        "Ajouter un exchange"
      ],
      "Log levels": [
        ""
      ],
      "Coins": [
        "Pièces"
      ],
      "usable coins": [
        "pièces utilisables"
      ],
      "id": [
        "N°"
      ],
      "denom": [
        "dénomination"
      ],
      "status": [
        "état"
      ],
      "from refresh?": [
        "depuis une actualisation ?"
      ],
      "age key count": [
        "nombre de clés d'age"
      ],
      "spent coins": [
        "pièces dépensées"
      ],
      "click to show": [
        "Cliquez pour afficher"
      ],
      "URI is not valid. Taler URI should start with \"taler://\"": [
        "L'URI n'est pas valide. Une URI Tale dois démarrer par `taler://`"
      ],
      "After the URI type it should follow a '/' with more information.": [
        ""
      ],
      "This URI type is not supported": [
        ""
      ],
      "This URI requires the bank host and operation id separated by /": [
        ""
      ],
      "This URI requires the merchant host, order id and session id separated by /": [
        ""
      ],
      "This URI requires the merchant host and order id separated by /": [
        ""
      ],
      "This URI requires the exchange host and key id separated by /": [
        ""
      ],
      "This URI requires the merchant host and template id separated by /": [
        ""
      ],
      "This URI requires the host key and provider list separated by /": [
        ""
      ],
      "This URI requires the experiment id": [
        ""
      ],
      "This URI requires the exchange host": [
        ""
      ],
      "This URI requires the alias type, the alias and the mailbox URI separated by /": [
        ""
      ],
      "This URI requires string after the first /": [
        ""
      ],
      "The bank host is invalid": [
        ""
      ],
      "The merchant host is invalid": [
        "Le commerçant vous offre un pourboire"
      ],
      "The URI should end with /": [
        ""
      ],
      "The exchange host is invalid": [
        ""
      ],
      "The contact is invalid": [
        ""
      ],
      "The amount is invalid": [
        ""
      ],
      "A parameter is invalid": [
        ""
      ],
      "checking...": [
        ""
      ],
      "Unexpected error happen reading the camera: %1$s": [
        ""
      ],
      "Could not found a QR code in the file": [
        ""
      ],
      "Unexpected error happen reading the file: %1$s": [
        ""
      ],
      "Scan a QR code or enter taler:// URI below": [
        "Scannez un code QR ou entrez une URI de la forme taler :// ci-dessous"
      ],
      "Pay invoice": [
        "Facture numérique"
      ],
      "Withdrawal from bank": [
        "Retrait"
      ],
      "Claim refund": [
        "Déjà remboursé"
      ],
      "Accept payment": [
        "Accepter"
      ],
      "Complete order": [
        ""
      ],
      "Restore wallet": [
        ""
      ],
      "Enable experiment": [
        ""
      ],
      "Withdraw from exchange": [
        "Retrait vers un téléphone portable"
      ],
      "Add exchange": [
        "Ajouter un prestataire de services de paiement"
      ],
      "Notify transaction": [
        ""
      ],
      "Add contact": [
        "Ajouter un compte"
      ],
      "The exchange URL is invalid.": [
        ""
      ],
      "The exchange URL should end with '/'": [
        ""
      ],
      "Couldn't found an exchange in the URL specified.": [
        "Impossible de trouver un exchange pour la devise %1$s"
      ],
      "HTTP request failed to %1$s: %2$s": [
        ""
      ],
      "HTTP request failed to %1$s": [
        ""
      ],
      "The merchant URL is invalid.": [
        "Le commerçant vous offre un pourboire"
      ],
      "The merchant URL should end with '/'": [
        ""
      ],
      "Couldn't found an merchant in the URL specified.": [
        ""
      ],
      "The bank URL is invalid.": [
        ""
      ],
      "The bank URL should end with '/'": [
        ""
      ],
      "Couldn't found an bank in the URL specified.": [
        ""
      ],
      "Could not load exchanges": [
        "Impossible de charger les frais de l'exchange"
      ],
      "Choose a currency to proceed or add another exchange": [
        "Choisissez une devise pour continuer ou ajoutez un autre exchange"
      ],
      "Known currencies": [
        "Devises connues"
      ],
      "Specify the origin": [
        "Précisez l'origine de l'argent"
      ],
      "Use previous origins:": [
        "Utilisez les origines précédentes :"
      ],
      "From another bank account": [
        "Depuis mon compte en banque"
      ],
      "From another wallet": [
        "Depuis un autre portefeuille"
      ],
      "From a %1$s URI or QR code": [
        ""
      ],
      "Enter URI here": [
        ""
      ],
      "Specify the destination": [
        "Préciser la destination de l'argent"
      ],
      "Use previous destinations:": [
        "Utilisez les destinations précédentes :"
      ],
      "To another bank account": [
        "Vers mon compte en banque"
      ],
      "To another wallet": [
        "Vers un autre portefeuille"
      ],
      "To a %1$s URI or QR code": [
        ""
      ],
      "Afghanistan": [
        ""
      ],
      "Albania": [
        ""
      ],
      "Algeria": [
        ""
      ],
      "United States": [
        ""
      ],
      "Andorra": [
        ""
      ],
      "Angola": [
        ""
      ],
      "Antigua and Barbuda": [
        ""
      ],
      "Argentina": [
        ""
      ],
      "Armenia": [
        ""
      ],
      "Australia": [
        ""
      ],
      "Austria": [
        ""
      ],
      "Azerbaijan": [
        ""
      ],
      "Bahamas": [
        ""
      ],
      "Bahrain": [
        ""
      ],
      "Bangladesh": [
        ""
      ],
      "Barbados": [
        ""
      ],
      "Botswana": [
        ""
      ],
      "Belarus": [
        ""
      ],
      "Belgium": [
        ""
      ],
      "Belize": [
        ""
      ],
      "Benin": [
        ""
      ],
      "Bhutan": [
        ""
      ],
      "Bolivia": [
        ""
      ],
      "Bosnia and Herzegovina": [
        ""
      ],
      "Brazil": [
        ""
      ],
      "United Kingdom": [
        ""
      ],
      "Brunei": [
        ""
      ],
      "Bulgaria": [
        ""
      ],
      "Burkina Faso": [
        ""
      ],
      "Myanmar": [
        ""
      ],
      "Burundi": [
        ""
      ],
      "Cambodia": [
        ""
      ],
      "Cameroon": [
        ""
      ],
      "Canada": [
        ""
      ],
      "Cape Verde": [
        ""
      ],
      "Central African Republic": [
        ""
      ],
      "Chad": [
        ""
      ],
      "Chile": [
        ""
      ],
      "China": [
        ""
      ],
      "Colombia": [
        ""
      ],
      "Comoros": [
        ""
      ],
      "Democratic Republic of the Congo": [
        ""
      ],
      "Republic of the Congo": [
        ""
      ],
      "Costa Rica": [
        ""
      ],
      "Croatia": [
        ""
      ],
      "Cuba": [
        ""
      ],
      "Cyprus": [
        ""
      ],
      "Czech Republic": [
        "Clé publique du commerçant"
      ],
      "Denmark": [
        ""
      ],
      "Djibouti": [
        ""
      ],
      "Dominican Republic": [
        ""
      ],
      "Netherlands": [
        ""
      ],
      "Timor-Leste": [
        ""
      ],
      "Ecuador": [
        ""
      ],
      "Egypt": [
        ""
      ],
      "United Arab Emirates": [
        ""
      ],
      "Equatorial Guinea": [
        ""
      ],
      "Eritrea": [
        ""
      ],
      "Estonia": [
        ""
      ],
      "Ethiopia": [
        ""
      ],
      "Fiji": [
        ""
      ],
      "Philippines": [
        ""
      ],
      "Finland": [
        ""
      ],
      "France": [
        "Annuler"
      ],
      "Gabon": [
        ""
      ],
      "Gambia": [
        ""
      ],
      "Georgia": [
        ""
      ],
      "Germany": [
        ""
      ],
      "Ghana": [
        ""
      ],
      "Greece": [
        ""
      ],
      "Grenada": [
        ""
      ],
      "Guatemala": [
        ""
      ],
      "Guinea-Bissau": [
        ""
      ],
      "Guinea": [
        ""
      ],
      "Guyana": [
        ""
      ],
      "Haiti": [
        ""
      ],
      "Honduras": [
        ""
      ],
      "Hungary": [
        ""
      ],
      "Kiribati": [
        ""
      ],
      "Iceland": [
        ""
      ],
      "India": [
        ""
      ],
      "Indonesia": [
        ""
      ],
      "Iran": [
        ""
      ],
      "Iraq": [
        ""
      ],
      "Ireland": [
        ""
      ],
      "Israel": [
        ""
      ],
      "Italy": [
        ""
      ],
      "Ivory Coast": [
        ""
      ],
      "Jamaica": [
        ""
      ],
      "Japan": [
        ""
      ],
      "Jordan": [
        ""
      ],
      "Kazakhstan": [
        ""
      ],
      "Kenya": [
        ""
      ],
      "Saint Kitts and Nevis": [
        ""
      ],
      "Kuwait": [
        ""
      ],
      "Kyrgyzstan": [
        ""
      ],
      "Laos": [
        ""
      ],
      "Latvia": [
        ""
      ],
      "Lebanon": [
        ""
      ],
      "Liberia": [
        ""
      ],
      "Libya": [
        ""
      ],
      "Liechtenstein": [
        ""
      ],
      "Lithuania": [
        ""
      ],
      "Luxembourg": [
        ""
      ],
      "North Macedonia": [
        ""
      ],
      "Madagascar": [
        ""
      ],
      "Malawi": [
        ""
      ],
      "Malaysia": [
        ""
      ],
      "Maldives": [
        ""
      ],
      "Mali": [
        ""
      ],
      "Malta": [
        ""
      ],
      "Marshall Islands": [
        ""
      ],
      "Mauritania": [
        ""
      ],
      "Mauritius": [
        ""
      ],
      "Mexico": [
        ""
      ],
      "Micronesia": [
        ""
      ],
      "Moldova": [
        ""
      ],
      "Monaco": [
        ""
      ],
      "Mongolia": [
        ""
      ],
      "Morocco": [
        ""
      ],
      "Lesotho": [
        ""
      ],
      "Mozambique": [
        ""
      ],
      "Namibia": [
        ""
      ],
      "Nauru": [
        ""
      ],
      "Nepal": [
        ""
      ],
      "New Zealand": [
        ""
      ],
      "Vanuatu": [
        ""
      ],
      "Nicaragua": [
        ""
      ],
      "Nigeria": [
        ""
      ],
      "Niger": [
        ""
      ],
      "North Korea": [
        ""
      ],
      "Norway": [
        ""
      ],
      "Oman": [
        ""
      ],
      "Pakistan": [
        ""
      ],
      "Palau": [
        ""
      ],
      "Panama": [
        ""
      ],
      "Papua New Guinea": [
        ""
      ],
      "Paraguay": [
        ""
      ],
      "Peru": [
        ""
      ],
      "Poland": [
        ""
      ],
      "Portugal": [
        ""
      ],
      "Qatar": [
        ""
      ],
      "Romania": [
        ""
      ],
      "Russia": [
        ""
      ],
      "Rwanda": [
        ""
      ],
      "Saint Lucia": [
        ""
      ],
      "El Salvador": [
        ""
      ],
      "Samoa": [
        ""
      ],
      "San Marino": [
        ""
      ],
      "São Tomé and Príncipe": [
        ""
      ],
      "Saudi Arabia": [
        ""
      ],
      "Senegal": [
        ""
      ],
      "Serbia": [
        ""
      ],
      "Seychelles": [
        ""
      ],
      "Sierra Leone": [
        ""
      ],
      "Singapore": [
        ""
      ],
      "Slovakia": [
        ""
      ],
      "Slovenia": [
        ""
      ],
      "Solomon Islands": [
        ""
      ],
      "Somalia": [
        ""
      ],
      "South Africa": [
        ""
      ],
      "South Korea": [
        ""
      ],
      "Spain": [
        ""
      ],
      "Sri Lanka": [
        ""
      ],
      "Sudan": [
        ""
      ],
      "Suriname": [
        ""
      ],
      "Eswatini": [
        ""
      ],
      "Sweden": [
        ""
      ],
      "Switzerland": [
        ""
      ],
      "Syria": [
        ""
      ],
      "Taiwan": [
        ""
      ],
      "Tajikistan": [
        ""
      ],
      "Tanzania": [
        ""
      ],
      "Thailand": [
        ""
      ],
      "Togo": [
        ""
      ],
      "Tonga": [
        ""
      ],
      "Trinidad and Tobago": [
        ""
      ],
      "Tunisia": [
        ""
      ],
      "Turkey": [
        ""
      ],
      "Tuvalu": [
        "valeur"
      ],
      "Uganda": [
        ""
      ],
      "Ukraine": [
        ""
      ],
      "Uruguay": [
        ""
      ],
      "Uzbekistan": [
        ""
      ],
      "Venezuela": [
        ""
      ],
      "Vietnam": [
        ""
      ],
      "Yemen": [
        ""
      ],
      "Zambia": [
        ""
      ],
      "Zimbabwe": [
        ""
      ],
      "Afghan": [
        ""
      ],
      "Albanian": [
        ""
      ],
      "Algerian": [
        ""
      ],
      "American": [
        ""
      ],
      "Andorran": [
        ""
      ],
      "Angolan": [
        ""
      ],
      "Antiguan": [
        ""
      ],
      "Argentinean": [
        ""
      ],
      "Armenian": [
        ""
      ],
      "Australian": [
        ""
      ],
      "Austrian": [
        ""
      ],
      "Azerbaijani": [
        ""
      ],
      "Bahamian": [
        ""
      ],
      "Bahraini": [
        ""
      ],
      "Bangladeshi": [
        ""
      ],
      "Barbadian": [
        ""
      ],
      "Batswana": [
        ""
      ],
      "Belarusian": [
        ""
      ],
      "Belgian": [
        ""
      ],
      "Belizean": [
        ""
      ],
      "Beninese": [
        ""
      ],
      "Bhutanese": [
        ""
      ],
      "Bolivian": [
        ""
      ],
      "Bosnian": [
        ""
      ],
      "Brazilian": [
        ""
      ],
      "British": [
        ""
      ],
      "Bruneian": [
        ""
      ],
      "Bulgarian": [
        ""
      ],
      "Burkinabe": [
        ""
      ],
      "Burmese": [
        ""
      ],
      "Burundian": [
        ""
      ],
      "Cambodian": [
        ""
      ],
      "Cameroonian": [
        ""
      ],
      "Canadian": [
        ""
      ],
      "Cape Verdean": [
        ""
      ],
      "Central African": [
        ""
      ],
      "Chadian": [
        ""
      ],
      "Chilean": [
        ""
      ],
      "Chinese": [
        ""
      ],
      "Colombian": [
        ""
      ],
      "Comoran": [
        ""
      ],
      "Congolese": [
        ""
      ],
      "Costa Rican": [
        ""
      ],
      "Croatian": [
        ""
      ],
      "Cuban": [
        ""
      ],
      "Cypriot": [
        ""
      ],
      "Czech": [
        ""
      ],
      "Danish": [
        ""
      ],
      "Djiboutian": [
        ""
      ],
      "Dominican": [
        "Dénomination"
      ],
      "Dutch": [
        ""
      ],
      "East Timorese": [
        ""
      ],
      "Ecuadorean": [
        ""
      ],
      "Egyptian": [
        ""
      ],
      "Emirati": [
        ""
      ],
      "Equatorial Guinean": [
        ""
      ],
      "Eritrean": [
        ""
      ],
      "Estonian": [
        ""
      ],
      "Ethiopian": [
        ""
      ],
      "Fijian": [
        ""
      ],
      "Filipino": [
        ""
      ],
      "Finnish": [
        ""
      ],
      "French": [
        ""
      ],
      "Gabonese": [
        ""
      ],
      "Gambian": [
        ""
      ],
      "Georgian": [
        ""
      ],
      "German": [
        ""
      ],
      "Ghanaian": [
        ""
      ],
      "Greek": [
        ""
      ],
      "Grenadian": [
        ""
      ],
      "Guatemalan": [
        ""
      ],
      "Guinea-Bissauan": [
        ""
      ],
      "Guinean": [
        ""
      ],
      "Guyanese": [
        ""
      ],
      "Haitian": [
        ""
      ],
      "Honduran": [
        ""
      ],
      "Hungarian": [
        ""
      ],
      "I-Kiribati": [
        ""
      ],
      "Icelander": [
        ""
      ],
      "Indian": [
        ""
      ],
      "Indonesian": [
        ""
      ],
      "Iranian": [
        ""
      ],
      "Iraqi": [
        ""
      ],
      "Irish": [
        ""
      ],
      "Israeli": [
        ""
      ],
      "Italian": [
        ""
      ],
      "Ivorian": [
        ""
      ],
      "Jamaican": [
        ""
      ],
      "Japanese": [
        ""
      ],
      "Jordanian": [
        ""
      ],
      "Kazakhstani": [
        ""
      ],
      "Kenyan": [
        ""
      ],
      "Kittian and Nevisian": [
        ""
      ],
      "Kuwaiti": [
        ""
      ],
      "Kyrgyz": [
        ""
      ],
      "Laotian": [
        ""
      ],
      "Latvian": [
        ""
      ],
      "Lebanese": [
        ""
      ],
      "Liberian": [
        ""
      ],
      "Libyan": [
        ""
      ],
      "Liechtensteiner": [
        ""
      ],
      "Lithuanian": [
        ""
      ],
      "Luxembourger": [
        ""
      ],
      "Macedonian": [
        ""
      ],
      "Malagasy": [
        ""
      ],
      "Malawian": [
        ""
      ],
      "Malaysian": [
        ""
      ],
      "Maldivian": [
        ""
      ],
      "Malian": [
        ""
      ],
      "Maltese": [
        ""
      ],
      "Marshallese": [
        ""
      ],
      "Mauritanian": [
        ""
      ],
      "Mauritian": [
        ""
      ],
      "Mexican": [
        ""
      ],
      "Micronesian": [
        ""
      ],
      "Moldovan": [
        ""
      ],
      "Monacan": [
        ""
      ],
      "Mongolian": [
        ""
      ],
      "Moroccan": [
        ""
      ],
      "Mosotho": [
        ""
      ],
      "Mozambican": [
        ""
      ],
      "Namibian": [
        ""
      ],
      "Nauruan": [
        ""
      ],
      "Nepalese": [
        ""
      ],
      "New Zealander": [
        ""
      ],
      "Ni-Vanuatu": [
        ""
      ],
      "Nicaraguan": [
        ""
      ],
      "Nigerian": [
        ""
      ],
      "Nigerien": [
        ""
      ],
      "North Korean": [
        ""
      ],
      "Norwegian": [
        ""
      ],
      "Omani": [
        ""
      ],
      "Pakistani": [
        ""
      ],
      "Palauan": [
        ""
      ],
      "Panamanian": [
        ""
      ],
      "Papua New Guinean": [
        ""
      ],
      "Paraguayan": [
        ""
      ],
      "Peruvian": [
        ""
      ],
      "Polish": [
        ""
      ],
      "Portuguese": [
        ""
      ],
      "Qatari": [
        ""
      ],
      "Romanian": [
        ""
      ],
      "Russian": [
        ""
      ],
      "Rwandan": [
        ""
      ],
      "Saint Lucian": [
        ""
      ],
      "Salvadoran": [
        ""
      ],
      "Samoan": [
        ""
      ],
      "San Marinese": [
        ""
      ],
      "Sao Tomean": [
        ""
      ],
      "Saudi": [
        ""
      ],
      "Senegalese": [
        ""
      ],
      "Serbian": [
        ""
      ],
      "Seychellois": [
        ""
      ],
      "Sierra Leonean": [
        ""
      ],
      "Singaporean": [
        ""
      ],
      "Slovakian": [
        ""
      ],
      "Slovenian": [
        ""
      ],
      "Solomon Islander": [
        ""
      ],
      "Somali": [
        ""
      ],
      "South African": [
        ""
      ],
      "South Korean": [
        ""
      ],
      "Spanish": [
        ""
      ],
      "Sri Lankan": [
        ""
      ],
      "Sudanese": [
        ""
      ],
      "Surinamer": [
        ""
      ],
      "Swazi": [
        ""
      ],
      "Swedish": [
        ""
      ],
      "Swiss": [
        ""
      ],
      "Syrian": [
        ""
      ],
      "Taiwanese": [
        ""
      ],
      "Tajik": [
        ""
      ],
      "Tanzanian": [
        ""
      ],
      "Thai": [
        ""
      ],
      "Togolese": [
        ""
      ],
      "Tongan": [
        ""
      ],
      "Trinidadian or Tobagonian": [
        ""
      ],
      "Tunisian": [
        ""
      ],
      "Turkish": [
        ""
      ],
      "Tuvaluan": [
        "valeur"
      ],
      "Ugandan": [
        ""
      ],
      "Ukrainian": [
        ""
      ],
      "Uruguayan": [
        ""
      ],
      "Uzbekistani": [
        ""
      ],
      "Venezuelan": [
        ""
      ],
      "Vietnamese": [
        ""
      ],
      "Yemenite": [
        ""
      ],
      "Zambian": [
        ""
      ],
      "Zimbabwean": [
        ""
      ],
      "🇦🇫": [
        ""
      ],
      "🇦🇱": [
        ""
      ],
      "🇩🇿": [
        ""
      ],
      "🇺🇸": [
        ""
      ],
      "🇦🇩": [
        ""
      ],
      "🇦🇴": [
        ""
      ],
      "🇦🇬": [
        ""
      ],
      "🇦🇷": [
        ""
      ],
      "🇦🇲": [
        ""
      ],
      "🇦🇺": [
        ""
      ],
      "🇦🇹": [
        ""
      ],
      "🇦🇿": [
        ""
      ],
      "🇧🇸": [
        ""
      ],
      "🇧🇭": [
        ""
      ],
      "🇧🇩": [
        ""
      ],
      "🇧🇧": [
        ""
      ],
      "🇧🇼": [
        ""
      ],
      "🇧🇾": [
        ""
      ],
      "🇧🇪": [
        ""
      ],
      "🇧🇿": [
        ""
      ],
      "🇧🇯": [
        ""
      ],
      "🇧🇹": [
        ""
      ],
      "🇧🇴": [
        ""
      ],
      "🇧🇦": [
        ""
      ],
      "🇧🇷": [
        ""
      ],
      "🇬🇧": [
        ""
      ],
      "🇧🇳": [
        ""
      ],
      "🇧🇬": [
        ""
      ],
      "🇧🇫": [
        ""
      ],
      "🇲🇲": [
        ""
      ],
      "🇧🇮": [
        ""
      ],
      "🇰🇭": [
        ""
      ],
      "🇨🇲": [
        ""
      ],
      "🇨🇦": [
        ""
      ],
      "🇨🇻": [
        ""
      ],
      "🇨🇫": [
        ""
      ],
      "🇨🇱": [
        ""
      ],
      "🇨🇳": [
        ""
      ],
      "🇨🇴": [
        ""
      ],
      "🇨🇩": [
        ""
      ],
      "🇨🇬": [
        ""
      ],
      "🇨🇷": [
        ""
      ],
      "🇨🇺": [
        ""
      ],
      "🇨🇾": [
        ""
      ],
      "🇨🇿": [
        ""
      ],
      "🇩🇰": [
        ""
      ],
      "🇩🇯": [
        ""
      ],
      "🇩🇴": [
        ""
      ],
      "🇳🇱": [
        ""
      ],
      "🇹🇱": [
        ""
      ],
      "🇪🇨": [
        ""
      ],
      "🇪🇬": [
        ""
      ],
      "🇦🇪": [
        ""
      ],
      "🇬🇶": [
        ""
      ],
      "🇪🇷": [
        ""
      ],
      "🇪🇪": [
        ""
      ],
      "🇪🇹": [
        ""
      ],
      "🇫🇯": [
        ""
      ],
      "🇵🇭": [
        ""
      ],
      "🇫🇮": [
        ""
      ],
      "🇫🇷": [
        ""
      ],
      "🇬🇦": [
        ""
      ],
      "🇬🇲": [
        ""
      ],
      "🇬🇪": [
        ""
      ],
      "UAE Dirham": [
        ""
      ],
      "Afghani": [
        ""
      ],
      "Lek": [
        ""
      ],
      "Armenian Dram": [
        ""
      ],
      "Netherlands Antillean Guilder": [
        ""
      ],
      "Kwanza": [
        ""
      ],
      "Argentine Peso": [
        ""
      ],
      "Australian Dollar": [
        ""
      ],
      "Aruban Florin": [
        ""
      ],
      "Azerbaijan Manat": [
        ""
      ],
      "Convertible Mark": [
        ""
      ],
      "Barbados Dollar": [
        ""
      ],
      "Taka": [
        ""
      ],
      "Bulgarian Lev": [
        ""
      ],
      "Bahraini Dinar": [
        ""
      ],
      "Burundi Franc": [
        ""
      ],
      "Bermudian Dollar": [
        ""
      ],
      "Brunei Dollar": [
        ""
      ],
      "Boliviano": [
        ""
      ],
      "Mvdol": [
        ""
      ],
      "Brazilian Real": [
        ""
      ],
      "Bahamian Dollar": [
        ""
      ],
      "Ngultrum": [
        ""
      ],
      "Pula": [
        ""
      ],
      "Belarusian Ruble": [
        ""
      ],
      "Belize Dollar": [
        ""
      ],
      "Canadian Dollar": [
        ""
      ],
      "Congolese Franc": [
        ""
      ],
      "WIR Euro": [
        ""
      ],
      "Swiss Franc": [
        ""
      ],
      "WIR Franc": [
        ""
      ],
      "Unidad de Fomento": [
        ""
      ],
      "Chilean Peso": [
        ""
      ],
      "Yuan Renminbi": [
        ""
      ],
      "Colombian Peso": [
        ""
      ],
      "Unidad de Valor Real": [
        ""
      ],
      "Costa Rican Colon": [
        ""
      ],
      "Cuban Peso": [
        ""
      ],
      "Cabo Verde Escudo": [
        ""
      ],
      "Czech Koruna": [
        ""
      ],
      "Djibouti Franc": [
        ""
      ],
      "Danish Krone": [
        ""
      ],
      "Dominican Peso": [
        ""
      ],
      "Algerian Dinar": [
        ""
      ],
      "Egyptian Pound": [
        ""
      ],
      "Nakfa": [
        ""
      ],
      "Ethiopian Birr": [
        ""
      ],
      "Euro": [
        ""
      ],
      "Fiji Dollar": [
        ""
      ],
      "Falkland Islands Pound": [
        ""
      ],
      "Pound Sterling": [
        ""
      ],
      "Lari": [
        ""
      ],
      "Ghana Cedi": [
        ""
      ],
      "Gibraltar Pound": [
        ""
      ],
      "Dalasi": [
        ""
      ],
      "Guinean Franc": [
        ""
      ],
      "Quetzal": [
        ""
      ],
      "Guyana Dollar": [
        ""
      ],
      "Hong Kong Dollar": [
        ""
      ],
      "Lempira": [
        ""
      ],
      "Gourde": [
        ""
      ],
      "Forint": [
        ""
      ],
      "Rupiah": [
        ""
      ],
      "New Israeli Sheqel": [
        ""
      ],
      "Indian Rupee": [
        ""
      ],
      "Iraqi Dinar": [
        ""
      ],
      "Iranian Rial": [
        ""
      ],
      "Iceland Krona": [
        ""
      ],
      "Jamaican Dollar": [
        ""
      ],
      "Jordanian Dinar": [
        ""
      ],
      "Yen": [
        ""
      ],
      "Kenyan Shilling": [
        ""
      ],
      "Som": [
        ""
      ],
      "Riel": [
        ""
      ],
      "Comorian Franc ": [
        ""
      ],
      "North Korean Won": [
        ""
      ],
      "Won": [
        ""
      ],
      "Kuwaiti Dinar": [
        ""
      ],
      "Cayman Islands Dollar": [
        ""
      ],
      "Tenge": [
        ""
      ],
      "Lao Kip": [
        ""
      ],
      "Lebanese Pound": [
        ""
      ],
      "Sri Lanka Rupee": [
        ""
      ],
      "Liberian Dollar": [
        ""
      ],
      "Loti": [
        ""
      ],
      "Libyan Dinar": [
        ""
      ],
      "Moroccan Dirham": [
        ""
      ],
      "Moldovan Leu": [
        ""
      ],
      "Malagasy Ariary": [
        ""
      ],
      "Denar": [
        ""
      ],
      "Kyat": [
        ""
      ],
      "Tugrik": [
        ""
      ],
      "Pataca": [
        ""
      ],
      "Ouguiya": [
        ""
      ],
      "Mauritius Rupee": [
        ""
      ],
      "Rufiyaa": [
        ""
      ],
      "Malawi Kwacha": [
        ""
      ],
      "Mexican Peso": [
        ""
      ],
      "Mexican Unidad de Inversion (UDI)": [
        ""
      ],
      "Malaysian Ringgit": [
        ""
      ],
      "Mozambique Metical": [
        ""
      ],
      "Namibia Dollar": [
        ""
      ],
      "Naira": [
        ""
      ],
      "Cordoba Oro": [
        ""
      ],
      "Norwegian Krone": [
        ""
      ],
      "Nepalese Rupee": [
        ""
      ],
      "New Zealand Dollar": [
        ""
      ],
      "Rial Omani": [
        ""
      ],
      "Balboa": [
        ""
      ],
      "Sol": [
        "ancien"
      ],
      "Kina": [
        ""
      ],
      "Philippine Peso": [
        ""
      ],
      "Pakistan Rupee": [
        ""
      ],
      "Zloty": [
        ""
      ],
      "Guarani": [
        ""
      ],
      "Qatari Rial": [
        ""
      ],
      "Romanian Leu": [
        ""
      ],
      "Serbian Dinar": [
        ""
      ],
      "Russian Ruble": [
        ""
      ],
      "Rwanda Franc": [
        ""
      ],
      "Saudi Riyal": [
        ""
      ],
      "Solomon Islands Dollar": [
        ""
      ],
      "Seychelles Rupee": [
        ""
      ],
      "Sudanese Pound": [
        ""
      ],
      "Swedish Krona": [
        ""
      ],
      "Singapore Dollar": [
        ""
      ],
      "Saint Helena Pound": [
        ""
      ],
      "Leone": [
        ""
      ],
      "Somali Shilling": [
        ""
      ],
      "Surinam Dollar": [
        ""
      ],
      "South Sudanese Pound": [
        ""
      ],
      "Dobra": [
        ""
      ],
      "El Salvador Colon": [
        ""
      ],
      "Syrian Pound": [
        ""
      ],
      "Lilangeni": [
        ""
      ],
      "Baht": [
        ""
      ],
      "Somoni": [
        ""
      ],
      "Turkmenistan New Manat": [
        ""
      ],
      "Tunisian Dinar": [
        ""
      ],
      "Pa’anga": [
        ""
      ],
      "Turkish Lira": [
        ""
      ],
      "Trinidad and Tobago Dollar": [
        ""
      ],
      "New Taiwan Dollar": [
        ""
      ],
      "Tanzanian Shilling": [
        ""
      ],
      "Hryvnia": [
        ""
      ],
      "Uganda Shilling": [
        ""
      ],
      "US Dollar": [
        ""
      ],
      "US Dollar (Next day)": [
        ""
      ],
      "Uruguay Peso en Unidades Indexadas (UI)": [
        ""
      ],
      "Peso Uruguayo": [
        ""
      ],
      "Unidad Previsional": [
        ""
      ],
      "Uzbekistan Sum": [
        ""
      ],
      "Bolívar Soberano": [
        ""
      ],
      "Dong": [
        ""
      ],
      "Vatu": [
        ""
      ],
      "Tala": [
        ""
      ],
      "CFA Franc BEAC": [
        ""
      ],
      "East Caribbean Dollar": [
        ""
      ],
      "CFA Franc BCEAO": [
        ""
      ],
      "CFP Franc": [
        ""
      ],
      "Sucre": [
        ""
      ],
      "Yemeni Rial": [
        ""
      ],
      "Rand": [
        ""
      ],
      "Zambian Kwacha": [
        ""
      ],
      "Zimbabwe Gold": [
        ""
      ],
      "Afar": [
        ""
      ],
      "Abkhazian": [
        ""
      ],
      "Achinese": [
        ""
      ],
      "Acoli": [
        ""
      ],
      "Adangme": [
        ""
      ],
      "Adyghe": [
        ""
      ],
      "Avestan": [
        ""
      ],
      "Tunisian Arabic": [
        ""
      ],
      "Afrikaans": [
        ""
      ],
      "Afrihili": [
        ""
      ],
      "Aghem": [
        ""
      ],
      "Ainu": [
        ""
      ],
      "Akan": [
        ""
      ],
      "Akkadian": [
        ""
      ],
      "Alabama": [
        ""
      ],
      "Aleut": [
        ""
      ],
      "Gheg Albanian": [
        ""
      ],
      "Southern Altai": [
        ""
      ],
      "Amarik": [
        ""
      ],
      "Aragonese": [
        ""
      ],
      "Old English": [
        ""
      ],
      "Angika": [
        ""
      ],
      "Modern Standard Arabic": [
        ""
      ],
      "Arabik": [
        ""
      ],
      "Aramaic": [
        ""
      ],
      "Mapuche": [
        ""
      ],
      "Araona": [
        ""
      ],
      "Arapaho": [
        ""
      ],
      "Algerian Arabic": [
        ""
      ],
      "Arawak": [
        ""
      ],
      "Moroccan Arabic": [
        ""
      ],
      "Egyptian Arabic": [
        ""
      ],
      "Asu": [
        ""
      ],
      "Assamese": [
        ""
      ],
      "American Sign Language": [
        ""
      ],
      "Asturian": [
        ""
      ],
      "Avaric": [
        ""
      ],
      "Kotava": [
        ""
      ],
      "Awadhi": [
        ""
      ],
      "Aymara": [
        ""
      ],
      "South Azerbaijani": [
        ""
      ],
      "Bashkir": [
        ""
      ],
      "Baluchi": [
        ""
      ],
      "Balinese": [
        ""
      ],
      "Bavarian": [
        ""
      ],
      "Basaa": [
        ""
      ],
      "Bamun": [
        ""
      ],
      "Batak Toba": [
        ""
      ],
      "Ghomala": [
        ""
      ],
      "Belarus kasa": [
        ""
      ],
      "Beja": [
        ""
      ],
      "Bemba": [
        ""
      ],
      "Betawi": [
        ""
      ],
      "Bena": [
        ""
      ],
      "Bafut": [
        ""
      ],
      "Badaga": [
        ""
      ],
      "Bɔlgeria kasa": [
        ""
      ],
      "Bhojpuri": [
        ""
      ],
      "Bislama": [
        ""
      ],
      "Bikol": [
        ""
      ],
      "Bini": [
        ""
      ],
      "Banjar": [
        ""
      ],
      "Kom": [
        ""
      ],
      "Siksika": [
        ""
      ],
      "Bambara": [
        ""
      ],
      "Bengali kasa": [
        ""
      ],
      "Tibetan": [
        ""
      ],
      "Bishnupriya": [
        ""
      ],
      "Bakhtiari": [
        ""
      ],
      "Braj": [
        ""
      ],
      "Breton": [
        ""
      ],
      "Brahui": [
        ""
      ],
      "Bodo": [
        ""
      ],
      "Akoose": [
        ""
      ],
      "Buriat": [
        ""
      ],
      "Buginese": [
        ""
      ],
      "Bulu": [
        ""
      ],
      "Blin": [
        ""
      ],
      "Medumba": [
        ""
      ],
      "Catalan": [
        ""
      ],
      "Caddo": [
        ""
      ],
      "Carib": [
        ""
      ],
      "Cayuga": [
        ""
      ],
      "Atsam": [
        ""
      ],
      "Cebuano": [
        ""
      ],
      "Chechen": [
        ""
      ],
      "Chiga": [
        ""
      ],
      "Chibcha": [
        ""
      ],
      "Chamorro": [
        ""
      ],
      "Chagatai": [
        ""
      ],
      "Chuukese": [
        ""
      ],
      "Mari": [
        ""
      ],
      "Chinook Jargon": [
        ""
      ],
      "Choctaw": [
        ""
      ],
      "Chipewyan": [
        ""
      ],
      "Cherokee": [
        ""
      ],
      "Cheyenne": [
        ""
      ],
      "Central Kurdish": [
        ""
      ],
      "Corsican": [
        ""
      ],
      "Coptic": [
        ""
      ],
      "Capiznon": [
        ""
      ],
      "Cree": [
        ""
      ],
      "Crimean Turkish": [
        ""
      ],
      "Kashubian": [
        ""
      ],
      "Kyɛk kasa": [
        ""
      ],
      "Church Slavic": [
        ""
      ],
      "Chuvash": [
        ""
      ],
      "Welsh": [
        ""
      ],
      "Dakota": [
        ""
      ],
      "Dargwa": [
        ""
      ],
      "Taita": [
        ""
      ],
      "Austrian German": [
        ""
      ],
      "Swiss High German": [
        ""
      ],
      "Gyaaman": [
        ""
      ],
      "Delaware": [
        ""
      ],
      "Slave": [
        ""
      ],
      "Dogrib": [
        ""
      ],
      "Dinka": [
        ""
      ],
      "Zarma": [
        ""
      ],
      "Dogri": [
        ""
      ],
      "Lower Sorbian": [
        ""
      ],
      "Central Dusun": [
        ""
      ],
      "Duala": [
        ""
      ],
      "Middle Dutch": [
        ""
      ],
      "Divehi": [
        ""
      ],
      "Jola-Fonyi": [
        ""
      ],
      "Dyula": [
        ""
      ],
      "Dzongkha": [
        ""
      ],
      "Dazaga": [
        ""
      ],
      "Embu": [
        ""
      ],
      "Ewe": [
        ""
      ],
      "Efik": [
        ""
      ],
      "Emilian": [
        ""
      ],
      "Ancient Egyptian": [
        ""
      ],
      "Ekajuk": [
        ""
      ],
      "Greek kasa": [
        ""
      ],
      "Elamite": [
        ""
      ],
      "Australian English": [
        ""
      ],
      "Borɔfo": [
        ""
      ],
      "Canadian English": [
        ""
      ],
      "British English": [
        ""
      ],
      "Middle English": [
        ""
      ],
      "American English": [
        ""
      ],
      "Esperanto": [
        "Opération"
      ],
      "Latin American Spanish": [
        ""
      ],
      "European Spanish": [
        ""
      ],
      "Mexican Spanish": [
        ""
      ],
      "Spain kasa": [
        ""
      ],
      "Central Yupik": [
        ""
      ],
      "Basque": [
        ""
      ],
      "Ewondo": [
        ""
      ],
      "Extremaduran": [
        ""
      ],
      "Fang": [
        ""
      ],
      "Pɛɛhyia kasa": [
        ""
      ],
      "Fanti": [
        ""
      ],
      "Fulah": [
        ""
      ],
      "Tornedalen Finnish": [
        ""
      ],
      "Faroese": [
        ""
      ],
      "Fon": [
        ""
      ],
      "Canadian French": [
        ""
      ],
      "Cajun French": [
        "Devise"
      ],
      "Swiss French": [
        ""
      ],
      "Frɛnkye": [
        ""
      ],
      "Middle French": [
        ""
      ],
      "Old French": [
        ""
      ],
      "Arpitan": [
        ""
      ],
      "Northern Frisian": [
        ""
      ],
      "Eastern Frisian": [
        ""
      ],
      "Friulian": [
        ""
      ],
      "Western Frisian": [
        ""
      ],
      "Ga": [
        ""
      ],
      "Gagauz": [
        ""
      ],
      "Gan Chinese": [
        ""
      ],
      "Gayo": [
        ""
      ],
      "Gbaya": [
        ""
      ],
      "Zoroastrian Dari": [
        ""
      ],
      "Scottish Gaelic": [
        ""
      ],
      "Geez": [
        ""
      ],
      "Gilbertese": [
        ""
      ],
      "Galician": [
        ""
      ],
      "Gilaki": [
        ""
      ],
      "Middle High German": [
        ""
      ],
      "Old High German": [
        ""
      ],
      "Goan Konkani": [
        ""
      ],
      "Gondi": [
        ""
      ],
      "Gorontalo": [
        ""
      ],
      "Gothic": [
        ""
      ],
      "Grebo": [
        ""
      ],
      "Ancient Greek": [
        ""
      ],
      "Swiss German": [
        ""
      ],
      "Wayuu": [
        ""
      ],
      "Gujarati": [
        ""
      ],
      "Frafra": [
        ""
      ],
      "Gusii": [
        ""
      ],
      "Manx": [
        ""
      ],
      "Gwichʼin": [
        ""
      ],
      "Hausa": [
        ""
      ],
      "Haida": [
        ""
      ],
      "Hakka Chinese": [
        ""
      ],
      "Hawaiian": [
        ""
      ],
      "Hebrew": [
        ""
      ],
      "Fiji Hindi": [
        ""
      ],
      "Hindi": [
        ""
      ],
      "Hiligaynon": [
        ""
      ],
      "Hittite": [
        ""
      ],
      "Hmong": [
        ""
      ],
      "Hiri Motu": [
        ""
      ],
      "Upper Sorbian": [
        ""
      ],
      "Xiang Chinese": [
        ""
      ],
      "Hangri kasa": [
        ""
      ],
      "Hupa": [
        ""
      ],
      "Herero": [
        ""
      ],
      "Interlingua": [
        ""
      ],
      "Iban": [
        ""
      ],
      "Ibibio": [
        ""
      ],
      "Indonihyia kasa": [
        ""
      ],
      "Interlingue": [
        ""
      ],
      "Igbo": [
        ""
      ],
      "Sichuan Yi": [
        ""
      ],
      "Inupiaq": [
        ""
      ],
      "Iloko": [
        ""
      ],
      "Ingush": [
        ""
      ],
      "Ido": [
        ""
      ],
      "Icelandic": [
        ""
      ],
      "Italy kasa": [
        ""
      ],
      "Inuktitut": [
        ""
      ],
      "Ingrian": [
        ""
      ],
      "Gyapan kasa": [
        ""
      ],
      "Jamaican Creole English": [
        ""
      ],
      "Lojban": [
        ""
      ],
      "Ngomba": [
        ""
      ],
      "Machame": [
        ""
      ],
      "Judeo-Persian": [
        ""
      ],
      "Judeo-Arabic": [
        ""
      ],
      "Jutish": [
        ""
      ],
      "Gyabanis kasa": [
        ""
      ],
      "Kara-Kalpak": [
        ""
      ],
      "Kabyle": [
        ""
      ],
      "Kachin": [
        ""
      ],
      "Jju": [
        ""
      ],
      "Kamba": [
        ""
      ],
      "Kawi": [
        ""
      ],
      "Kabardian": [
        ""
      ],
      "Kanembu": [
        ""
      ],
      "Tyap": [
        ""
      ],
      "Makonde": [
        ""
      ],
      "Kabuverdianu": [
        ""
      ],
      "Kenyang": [
        ""
      ],
      "Koro": [
        ""
      ],
      "Kongo": [
        ""
      ],
      "Kaingang": [
        ""
      ],
      "Khasi": [
        ""
      ],
      "Khotanese": [
        ""
      ],
      "Koyra Chiini": [
        ""
      ],
      "Khowar": [
        ""
      ],
      "Kikuyu": [
        ""
      ],
      "Kirmanjki": [
        ""
      ],
      "Kuanyama": [
        ""
      ],
      "Kako": [
        ""
      ],
      "Kazakh": [
        ""
      ],
      "Kalaallisut": [
        ""
      ],
      "Kalenjin": [
        ""
      ],
      "Kimbundu": [
        ""
      ],
      "Kambodia kasa": [
        ""
      ],
      "Kannada": [
        ""
      ],
      "Komi-Permyak": [
        ""
      ],
      "Konkani": [
        ""
      ],
      "Korea kasa": [
        ""
      ],
      "Kosraean": [
        ""
      ],
      "Kpelle": [
        ""
      ],
      "Karachay-Balkar": [
        ""
      ],
      "Krio": [
        ""
      ],
      "Kinaray-a": [
        ""
      ],
      "Kanuri": [
        ""
      ],
      "Karelian": [
        ""
      ],
      "Kurukh": [
        ""
      ],
      "Shambala": [
        ""
      ],
      "Bafia": [
        ""
      ],
      "Colognian": [
        ""
      ],
      "Kashmiri": [
        ""
      ],
      "Kurdish": [
        ""
      ],
      "Kumyk": [
        ""
      ],
      "Kutenai": [
        ""
      ],
      "Komi": [
        ""
      ],
      "Cornish": [
        ""
      ],
      "Ladino": [
        "Chargement"
      ],
      "Langi": [
        ""
      ],
      "Lahnda": [
        ""
      ],
      "Latin": [
        ""
      ],
      "Lamba": [
        ""
      ],
      "Luxembourgish": [
        ""
      ],
      "Lezghian": [
        ""
      ],
      "Lingua Franca Nova": [
        ""
      ],
      "Ganda": [
        ""
      ],
      "Ligurian": [
        ""
      ],
      "Limburgish": [
        ""
      ],
      "Livonian": [
        ""
      ],
      "Lakota": [
        ""
      ],
      "Lombard": [
        ""
      ],
      "Lingala": [
        ""
      ],
      "Lao": [
        ""
      ],
      "Mongo": [
        ""
      ],
      "Lozi": [
        ""
      ],
      "Latgalian": [
        ""
      ],
      "Luba-Lulua": [
        ""
      ],
      "Luiseno": [
        ""
      ],
      "Luba-Katanga": [
        ""
      ],
      "Lunda": [
        ""
      ],
      "Luo": [
        ""
      ],
      "Mizo": [
        ""
      ],
      "Luyia": [
        ""
      ],
      "Literary Chinese": [
        ""
      ],
      "Laz": [
        ""
      ],
      "Madurese": [
        "réinitialiser"
      ],
      "Mafa": [
        ""
      ],
      "Magahi": [
        ""
      ],
      "Maithili": [
        ""
      ],
      "Makasar": [
        ""
      ],
      "Mandingo": [
        "Chargement"
      ],
      "Masai": [
        ""
      ],
      "Maba": [
        ""
      ],
      "Moksha": [
        ""
      ],
      "Mandar": [
        ""
      ],
      "Mende": [
        ""
      ],
      "Meru": [
        ""
      ],
      "Morisyen": [
        ""
      ],
      "Middle Irish": [
        ""
      ],
      "Makhuwa-Meetto": [
        ""
      ],
      "Metaʼ": [
        ""
      ],
      "Micmac": [
        ""
      ],
      "Maori": [
        ""
      ],
      "Minangkabau": [
        ""
      ],
      "Malayalam": [
        ""
      ],
      "Manchu": [
        ""
      ],
      "Manipuri": [
        ""
      ],
      "Mohawk": [
        ""
      ],
      "Mossi": [
        ""
      ],
      "Western Mari": [
        ""
      ],
      "Marathi": [
        ""
      ],
      "Malay kasa": [
        ""
      ],
      "Mundang": [
        ""
      ],
      "Multiple Languages": [
        "Langage actuel"
      ],
      "Creek": [
        ""
      ],
      "Mirandese": [
        ""
      ],
      "Marwari": [
        ""
      ],
      "Mentawai": [
        ""
      ],
      "Bɛɛmis kasa": [
        ""
      ],
      "Myene": [
        ""
      ],
      "Erzya": [
        ""
      ],
      "Mazanderani": [
        ""
      ],
      "Min Nan Chinese": [
        ""
      ],
      "Neapolitan": [
        ""
      ],
      "Nama": [
        ""
      ],
      "Norwegian Bokmål": [
        ""
      ],
      "North Ndebele": [
        ""
      ],
      "Low German": [
        ""
      ],
      "Nɛpal kasa": [
        ""
      ],
      "Newari": [
        ""
      ],
      "Ndonga": [
        ""
      ],
      "Nias": [
        ""
      ],
      "Niuean": [
        ""
      ],
      "Ao Naga": [
        ""
      ],
      "Flemish": [
        ""
      ],
      "Dɛɛkye": [
        ""
      ],
      "Kwasio": [
        ""
      ],
      "Ngiemboon": [
        ""
      ],
      "Norwegian Nynorsk": [
        ""
      ],
      "Nogai": [
        ""
      ],
      "Old Norse": [
        ""
      ],
      "Novial": [
        ""
      ],
      "NʼKo": [
        ""
      ],
      "South Ndebele": [
        ""
      ],
      "Northern Sotho": [
        ""
      ],
      "Nuer": [
        ""
      ],
      "Navajo": [
        ""
      ],
      "Classical Newari": [
        ""
      ],
      "Nyamwezi": [
        ""
      ],
      "Nyankole": [
        ""
      ],
      "Nyanja": [
        ""
      ],
      "Nyoro": [
        ""
      ],
      "Nzima": [
        ""
      ],
      "Occitan": [
        ""
      ],
      "Ojibwa": [
        ""
      ],
      "Oromo": [
        ""
      ],
      "Oriya": [
        ""
      ],
      "Osage": [
        ""
      ],
      "Ossetic": [
        ""
      ],
      "Ottoman Turkish": [
        ""
      ],
      "Pangasinan": [
        ""
      ],
      "Pahlavi": [
        ""
      ],
      "Pampanga": [
        ""
      ],
      "Papiamento": [
        "Paiement"
      ],
      "Pungyabi kasa": [
        ""
      ],
      "Picard": [
        ""
      ],
      "Pennsylvania German": [
        ""
      ],
      "Plautdietsch": [
        ""
      ],
      "Old Persian": [
        ""
      ],
      "Palatine German": [
        ""
      ],
      "Phoenician": [
        ""
      ],
      "Pali": [
        ""
      ],
      "Pɔland kasa": [
        ""
      ],
      "Piedmontese": [
        ""
      ],
      "Pontic": [
        ""
      ],
      "Pohnpeian": [
        ""
      ],
      "Prussian": [
        ""
      ],
      "Old Provençal": [
        ""
      ],
      "Pashto": [
        ""
      ],
      "Brazilian Portuguese": [
        ""
      ],
      "Pɔɔtugal kasa": [
        ""
      ],
      "European Portuguese": [
        ""
      ],
      "Kʼicheʼ": [
        ""
      ],
      "Chimborazo Highland Quichua": [
        ""
      ],
      "Quechua": [
        ""
      ],
      "Rajasthani": [
        ""
      ],
      "Rapanui": [
        ""
      ],
      "Rarotongan": [
        ""
      ],
      "Romagnol": [
        ""
      ],
      "Riffian": [
        ""
      ],
      "Romansh": [
        ""
      ],
      "Rundi": [
        ""
      ],
      "Rombo": [
        ""
      ],
      "Moldavian": [
        ""
      ],
      "Romany": [
        ""
      ],
      "Root": [
        ""
      ],
      "Romenia kasa": [
        ""
      ],
      "Rotuman": [
        ""
      ],
      "Rusyn": [
        ""
      ],
      "Roviana": [
        ""
      ],
      "Aromanian": [
        ""
      ],
      "Rahyia kasa": [
        ""
      ],
      "Rwa": [
        ""
      ],
      "Rewanda kasa": [
        ""
      ],
      "Sandawe": [
        ""
      ],
      "Sakha": [
        ""
      ],
      "Samaritan Aramaic": [
        ""
      ],
      "Samburu": [
        ""
      ],
      "Sanskrit": [
        ""
      ],
      "Sasak": [
        ""
      ],
      "Santali": [
        ""
      ],
      "Saurashtra": [
        ""
      ],
      "Ngambay": [
        ""
      ],
      "Sangu": [
        ""
      ],
      "Sicilian": [
        ""
      ],
      "Scots": [
        ""
      ],
      "Sardinian": [
        ""
      ],
      "Sassarese Sardinian": [
        ""
      ],
      "Sindhi": [
        ""
      ],
      "Seneca": [
        ""
      ],
      "Sena": [
        ""
      ],
      "Seri": [
        ""
      ],
      "Selkup": [
        ""
      ],
      "Northern Sami": [
        ""
      ],
      "Koyraboro Senni": [
        ""
      ],
      "Old Irish": [
        ""
      ],
      "Sango": [
        ""
      ],
      "Samogitian": [
        ""
      ],
      "Tachelhit": [
        ""
      ],
      "Shan": [
        ""
      ],
      "Serbo-Croatian": [
        ""
      ],
      "Chadian Arabic": [
        ""
      ],
      "Sidamo": [
        ""
      ],
      "Sinhala": [
        ""
      ],
      "Slovak": [
        ""
      ],
      "Lower Silesian": [
        ""
      ],
      "Selayar": [
        ""
      ],
      "Southern Sami": [
        ""
      ],
      "Lule Sami": [
        ""
      ],
      "Inari Sami": [
        ""
      ],
      "Skolt Sami": [
        ""
      ],
      "Soninke": [
        ""
      ],
      "Shona": [
        ""
      ],
      "Sogdien": [
        ""
      ],
      "Somalia kasa": [
        ""
      ],
      "Sranan Tongo": [
        ""
      ],
      "Serer": [
        ""
      ],
      "Swati": [
        ""
      ],
      "Saho": [
        ""
      ],
      "Saterland Frisian": [
        ""
      ],
      "Southern Sotho": [
        ""
      ],
      "Sukuma": [
        ""
      ],
      "Susu": [
        ""
      ],
      "Sundanese": [
        ""
      ],
      "Sumerian": [
        ""
      ],
      "Sweden kasa": [
        ""
      ],
      "Comorian": [
        ""
      ],
      "Congo Swahili": [
        ""
      ],
      "Swahili": [
        ""
      ],
      "Classical Syriac": [
        ""
      ],
      "Syriac": [
        ""
      ],
      "Silesian": [
        ""
      ],
      "Tamil kasa": [
        ""
      ],
      "Tulu": [
        ""
      ],
      "Timne": [
        ""
      ],
      "Teso": [
        ""
      ],
      "Tereno": [
        ""
      ],
      "Telugu": [
        ""
      ],
      "Tetum": [
        ""
      ],
      "Taeland kasa": [
        ""
      ],
      "Tigre": [
        ""
      ],
      "Tigrinya": [
        ""
      ],
      "Tiv": [
        "Conseil"
      ],
      "Tokelau": [
        ""
      ],
      "Tsakhur": [
        ""
      ],
      "Turkmen": [
        ""
      ],
      "Klingon": [
        ""
      ],
      "Tlingit": [
        ""
      ],
      "Tagalog": [
        ""
      ],
      "Talysh": [
        ""
      ],
      "Tamashek": [
        ""
      ],
      "Tswana": [
        ""
      ],
      "Nyasa Tonga": [
        ""
      ],
      "Tok Pisin": [
        ""
      ],
      "Tɛɛki kasa": [
        ""
      ],
      "Turoyo": [
        ""
      ],
      "Taroko": [
        ""
      ],
      "Tsakonian": [
        ""
      ],
      "Tsimshian": [
        ""
      ],
      "Tsonga": [
        ""
      ],
      "Tatar": [
        ""
      ],
      "Muslim Tat": [
        ""
      ],
      "Tumbuka": [
        ""
      ],
      "Tasawaq": [
        ""
      ],
      "Twi": [
        "Conseil"
      ],
      "Tahitian": [
        ""
      ],
      "Tuvinian": [
        ""
      ],
      "Central Atlas Tamazight": [
        ""
      ],
      "Udmurt": [
        ""
      ],
      "Ugaritic": [
        ""
      ],
      "Uyghur": [
        ""
      ],
      "Ukren kasa": [
        ""
      ],
      "Umbundu": [
        ""
      ],
      "Unknown Language": [
        "Langage actuel"
      ],
      "Urdu kasa": [
        ""
      ],
      "Uzbek": [
        ""
      ],
      "Vai": [
        ""
      ],
      "Venetian": [
        ""
      ],
      "Veps": [
        ""
      ],
      "Venda": [
        ""
      ],
      "Viɛtnam kasa": [
        ""
      ],
      "West Flemish": [
        ""
      ],
      "Main-Franconian": [
        ""
      ],
      "Votic": [
        ""
      ],
      "Volapük": [
        ""
      ],
      "Võro": [
        ""
      ],
      "Vunjo": [
        ""
      ],
      "Walser": [
        ""
      ],
      "Wolaytta": [
        ""
      ],
      "Waray": [
        ""
      ],
      "Washo": [
        ""
      ],
      "Walloon": [
        ""
      ],
      "Warlpiri": [
        ""
      ],
      "Wolof": [
        ""
      ],
      "Wu Chinese": [
        ""
      ],
      "Kalmyk": [
        ""
      ],
      "Xhosa": [
        ""
      ],
      "Mingrelian": [
        ""
      ],
      "Soga": [
        ""
      ],
      "Yao": [
        ""
      ],
      "Yapese": [
        ""
      ],
      "Yangben": [
        ""
      ],
      "Yemba": [
        ""
      ],
      "Yiddish": [
        ""
      ],
      "Yoruba": [
        ""
      ],
      "Nheengatu": [
        ""
      ],
      "Cantonese": [
        ""
      ],
      "Zapotec": [
        ""
      ],
      "Zhuang": [
        ""
      ],
      "Blissymbols": [
        ""
      ],
      "Zeelandic": [
        ""
      ],
      "Zenaga": [
        ""
      ],
      "Standard Moroccan Tamazight": [
        ""
      ],
      "Simplified Chinese": [
        ""
      ],
      "Traditional Chinese": [
        ""
      ],
      "Kyaena kasa": [
        ""
      ],
      "Zuni": [
        ""
      ],
      "Zulu": [
        ""
      ],
      "No linguistic content": [
        ""
      ],
      "Zaza": [
        ""
      ],
      "Could not load user attention request": [
        "Impossible de charger les frais de l'exchange"
      ],
      "No notification left": [
        ""
      ],
      "Ignore": [
        ""
      ],
      "Invalid dev experiment URI.": [
        ""
      ],
      "URI: %1$s": [
        "%1$s"
      ],
      "Experiment": [
        ""
      ],
      "Insert lost denomination": [
        "Dénomination"
      ],
      "Apply": [
        ""
      ],
      "Pending refresh": [
        "Opérations en cours"
      ],
      "Could not load the transfer payment status": [
        "Impossible de charger l’état du paiement"
      ],
      "Could not load the status of the order template": [
        "Impossible de charger la liste des transactions"
      ],
      "required": [
        ""
      ],
      "Review order": [
        ""
      ],
      "Missing URI": [
        ""
      ],
      "Check the link.": [
        "Cliquez sur l'icône du puzzle"
      ],
      "Could not parse the recovery URI": [
        "Impossible d'analyser l'URI payto"
      ],
      "Try another URI": [
        "Essayez-en un autre"
      ],
      "Import backup, show info": [
        "Importer la sauvegarde, afficher les informations"
      ],
      "Add new exchange": [
        "Ajouter un nouvel échange"
      ],
      "Add exchange for %1$s": [
        "Ajouter un exchange pour %1$s"
      ],
      "Enter the URL of an exchange you trust.": [
        "Entrez l'URL d'un exchange auquel vous faites confiance."
      ],
      "An exchange has been found! Review the information and click next": [
        "Un exchange a été trouvé ! Vérifiez les informations et cliquez sur Suivant"
      ],
      "This exchange is already in your list.": [
        "L'exchange est prêt pour le retrait"
      ],
      "Only exchange accessible through \"http\" and \"https\" are allowed.": [
        ""
      ],
      "This exchange protocol version is not supported: \" %1$s\".": [
        ""
      ],
      "This exchange currency \"%1$s\" doesn&apos;t match the expected currency %2$s.": [
        "Cet exchange ne correspond pas à la devise demandée %1$s"
      ],
      "No exchange found in that URL.": [
        "Aucun exchange trouvé pour %1$s"
      ],
      "loading": [
        "chargement"
      ],
      "Version": [
        "Version"
      ],
      "Can't use the URL: \"%1$s\"": [
        ""
      ],
      "Exchange URL": [
        "URL de l'exchange"
      ],
      "Add new contact": [
        "Ajoutez un autre compte"
      ],
      "Enter the details of a contact you trust.": [
        "Entrez l'URL d'un exchange auquel vous faites confiance."
      ],
      "Contact info valid.": [
        ""
      ],
      "This contact is already in your list.": [
        ""
      ],
      "Review contact": [
        ""
      ],
      "Contact alias": [
        ""
      ],
      "Alias type": [
        ""
      ],
      "Mailbox service": [
        ""
      ],
      "Mailbox address": [
        "Adresse Bitcoin"
      ],
      "Contact source": [
        ""
      ],
      "Could not load the list of contacts": [
        "Impossible de charger la liste des transactions"
      ],
      "Source": [
        ""
      ],
      "Send Cash": [
        "Envoyé %1$s"
      ],
      "Request Cash": [
        ""
      ],
      "No contacts found.": [
        ""
      ],
      "Could not bank choices for account": [
        "Impossible de charger les informations de récupération de sauvegarde"
      ],
      "Could not load the list of messages": [
        "Impossible de charger la liste des échanges connus"
      ],
      "Unexpected error when trying to initialize mailbox: %1$s": [
        ""
      ],
      "Unexpected error when trying to delete message: %1$s": [
        ""
      ],
      "Unexpected error when trying to fetch messages: %1$s": [
        ""
      ],
      "Date": [
        "Date"
      ],
      "You may copy your mailbox URI from below and register it at %1$s with your alias(es) to make it easier for others to find you.": [
        ""
      ],
      "copy uri": [
        "copier"
      ],
      "hide uri": [
        "cacher le code QR"
      ],
      "show uri": [
        "afficher le code QR"
      ],
      "Fetch messages": [
        "Message du commerçant"
      ],
      "Mailbox not initialized": [
        ""
      ],
      "Initialize mailbox": [
        ""
      ],
      "No messages yet.": [
        "Pas encore d'exchange"
      ],
      "Taler URI handler": [
        ""
      ],
      "Could not found a handler for the Taler URI": [
        "Impossible de trouver un exchange pour la devise %1$s"
      ],
      "The uri read in the path parameter is not valid: \"%1$s\"": [
        ""
      ],
      "Digital cash payment": [
        "Paiement en espèces numériques"
      ],
      "Digital cash refund": [
        "Remboursement en espèces numériques"
      ],
      "Digital cash withdrawal": [
        "Retrait d'argent numérique"
      ],
      "Digital cash deposit": [
        "Dépôt d'espèces numériques"
      ],
      "Digital cash invoice": [
        "Facture numérique"
      ],
      "Digital cash transfer": [
        "Transfert d'argent numérique"
      ],
      "Digital cash recovery": [
        "Récupération du portefeuille numérique"
      ],
      "Development experiment": [
        "Mode dévelopeur"
      ],
      "Return to wallet": [
        ""
      ],
      "Network is offline": [
        ""
      ],
      "Diagnostics timed out. Could not talk to the wallet backend.": [
        "Le diagnostic a expiré. Impossible de communiquer avec le backend du portefeuille."
      ],
      "Problems detected:": [
        "Problèmes détectés :"
      ],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).": [
        "Veuillez vérifier dans vos paramètres %1$s que vous avez activé IndexedDB (vérifiez le nom de la préférence %2$s)."
      ],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.": [
        "La base de données de votre portefeuille est obsolète. Actuellement, la migration automatique n'est pas prise en charge. Veuillez vous rendre à %1$s pour réinitialiser la base de données du portefeuille."
      ],
      "here": [
        ""
      ],
      "Running diagnostics": [
        "Exécution des diagnostics"
      ],
      "Edit": [
        "Editer"
      ],
      "List of products": [
        "Liste des produits"
      ],
      "free": [
        "gratuit"
      ],
      "Could not get provider information": [
        "Il est impossible d'obtenir les informations du prestataire"
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=(n!=1);",
  "lang": "fr",
  "completeness": 23
};

strings['fi'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "fi"
      },
      "Balance": [
        "Saldo"
      ],
      "Backup": [
        "Varmuuskopio"
      ],
      "Notifications": [
        ""
      ],
      "QR Reader and Taler URI": [
        "QR -lukija ja Taler URI"
      ],
      "Settings": [
        "Asetukset"
      ],
      "Dev tools": [
        ""
      ],
      "Contacts": [
        ""
      ],
      "Mailbox": [
        ""
      ],
      "Search transactions": [
        "Kauppiaan toimivalta"
      ],
      "%1$s": [
        "%1$s"
      ],
      "KYC requirement": [
        ""
      ],
      "PENDING OPERATIONS": [
        "ODOTTAVAT TOIMINNOT"
      ],
      "Error was thrown trying to: \"%1$s\"": [
        ""
      ],
      "Could not open the wallet database. Firefox is known to run into this problem under \"permanent private mode\".": [
        ""
      ],
      "Could not open the wallet database.": [
        ""
      ],
      "Unexpected request error, code: %1$s": [
        ""
      ],
      "more info": [
        ""
      ],
      "wrap text": [
        ""
      ],
      "copy content": [
        ""
      ],
      "less info": [
        ""
      ],
      "Retry operation": [
        "Toimituspaikka"
      ],
      "Loading": [
        "Lataa"
      ],
      "Could not load backup providers": [
        "Varmuuskopion tarjoajia ei voitu ladata"
      ],
      "No backup providers configured": [
        "Varmuuskopion tarjoajia ei ole määritetty"
      ],
      "Add provider": [
        "Lisää palveluntarjoaja"
      ],
      "Sync all backups": [
        "Synkronoi kaikki varmuuskopiot"
      ],
      "Sync now": [
        "Synkronoi nyt"
      ],
      "Last synced": [
        "Viimeksi synkronoitu"
      ],
      "Not synced": [
        "Ei synkronoitu"
      ],
      "Expires in": [
        "Vanhenee"
      ],
      "show less": [
        ""
      ],
      "show more": [
        ""
      ],
      "There was an error loading the provider detail for &quot;%1$s&quot;": [
        "Virhe ladattaessa palveluntarjoajan tietoja kohteelle &quot; %1$s&quot;"
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [
        "Ei tunneta palveluntarjoajaa, jonka URL-osoite on &quot;%1$s&quot;."
      ],
      "See providers": [
        "Katso palveluntarjoajat"
      ],
      "Last backup": [
        "Viimeisin varmuuskopio"
      ],
      "Back up": [
        "Varmuuskopioi"
      ],
      "Provider fee": [
        "Palveluntarjoajan maksu"
      ],
      "per year": [
        "vuodessa"
      ],
      "Extend": [
        "Laajenna"
      ],
      "terms has changed, extending the service will imply accepting the new terms of service": [
        "ehdot ovat muuttuneet, palvelun laajentaminen tarkoittaa uusien käyttöehtojen hyväksymistä"
      ],
      "old": [
        "vanha"
      ],
      "new": [
        "uusi"
      ],
      "fee": [
        "maksu"
      ],
      "storage": [
        "tila"
      ],
      "Remove provider": [
        "Poista palveluntarjoaja"
      ],
      "Pay": [
        ""
      ],
      "Withdraw": [
        ""
      ],
      "This provider has reported an error": [
        "Tämä palveluntarjoaja on ilmoittanut virheestä"
      ],
      "There is conflict with another backup from &quot;%1$s&quot;": [
        "Ristiriita toisen varmuuskopion kanssa kohteesta %1$s"
      ],
      "Backup is not readable": [
        "Varmuuskopiota ei voi lukea"
      ],
      "Unknown backup problem: %1$s": [
        "Tuntematon varmuuskopiointi ongelma: %1$s"
      ],
      "service paid": [
        "palvelu maksettu"
      ],
      "Backup valid until": [
        "Varmuuskopio voimassa"
      ],
      "Cancel": [
        "Peruuta"
      ],
      "Open pay page": [
        "Avaa maksusivu"
      ],
      "Open refund page": [
        "Avaa hyvityssivu"
      ],
      "Open withdraw page": [
        "Avaa nostosivu"
      ],
      "Your wallet is empty.": [
        ""
      ],
      "Get digital cash": [
        "Hanki digitaalista käteistä"
      ],
      "Try the demo bank and withdraw test money.": [
        ""
      ],
      "Could not load the balance": [
        "Ei voitu ladata saldosivua"
      ],
      "Receive": [
        "Vastaanottajan nimi"
      ],
      "Send %1$s": [
        "Lähetä %1$s"
      ],
      "This page has a withdrawal action.": [
        "Tällä sivulla on nosto toiminto."
      ],
      "This page has pay action.": [
        "Tällä sivulla on maksutoiminto."
      ],
      "This page has a refund action.": [
        "Tällä sivulla on hyvitys toiminto."
      ],
      "This page has a add exchange action.": [
        "Tällä sivulla on hyvitys toiminto."
      ],
      "Open add exchange page": [
        "Avaa hyvityssivu"
      ],
      "This page has an add contact action.": [
        "Tällä sivulla on hyvitys toiminto."
      ],
      "Open add contact page": [
        "Avaa maksusivu"
      ],
      "Taler Action": [
        "Taler toiminta"
      ],
      "This page has a malformed taler uri.": [
        "Tällä sivulla on väärin muotoiltu taler uri."
      ],
      "Dismiss": [
        "Hylkää"
      ],
      "this popup is being closed and you are being redirected to %1$s": [
        "tämä ponnahdusikkuna suljetaan ja sinut ohjataan osoitteeseen %1$s"
      ],
      "Could not load purchase proposal details": [
        "Ostoehdotuksen tietoja ei voitu ladata"
      ],
      "Order Id": [
        "Tilausnumero"
      ],
      "Summary": [
        "Yhteenveto"
      ],
      "Amount": [
        "Summa"
      ],
      "Choices": [
        ""
      ],
      "This purchase have %1$s choices": [
        ""
      ],
      "Merchant name": [
        "Kauppiaan nimi"
      ],
      "Merchant jurisdiction": [
        "Kauppiaan toimivalta"
      ],
      "Merchant address": [
        "Kauppiaan osoite"
      ],
      "Merchant logo": [
        "Kauppiaan logo"
      ],
      "Merchant website": [
        "Kauppiaan nettisivut"
      ],
      "Merchant email": [
        "Kauppiaan sähköposti"
      ],
      "Merchant public key": [
        "Kauppiaan julkinen avain"
      ],
      "Created at": [
        "Luotu"
      ],
      "Refund deadline": [
        "Palautuksen määräaika"
      ],
      "Auto refund": [
        "Automaattinen palautus"
      ],
      "Pay deadline": [
        "Maksun määräaika"
      ],
      "Fulfillment URL": [
        "Toteutus-URL"
      ],
      "Fulfillment message": [
        "Toteutusviesti"
      ],
      "Max deposit fee": [
        "Max talletusmaksu"
      ],
      "Exchanges": [
        "Vaihdot"
      ],
      "Could not load the status of the term of service": [
        "Tapahtumatietoja ei voitu ladata"
      ],
      "Select one option": [
        ""
      ],
      "Show terms of service": [
        ""
      ],
      "I accept the exchange terms of service": [
        ""
      ],
      "Review exchange terms of service": [
        ""
      ],
      "The exchange doesn't have a terms of service.": [
        ""
      ],
      "Skip it for now.": [
        ""
      ],
      "Format": [
        ""
      ],
      "Language": [
        ""
      ],
      "The exchange replied with a empty terms of service": [
        ""
      ],
      "No terms of service. The exchange replied with a empty document": [
        ""
      ],
      "No terms of service. The exchange replied with a empty text": [
        ""
      ],
      "Download Terms of Service": [
        ""
      ],
      "Hide terms of service": [
        ""
      ],
      "Amount is not valid": [
        ""
      ],
      "Bank account": [
        "Pankkitili"
      ],
      "Bitcoin address": [
        "Bitcoin osoite"
      ],
      "BIC/IBAN": [
        "IBAN"
      ],
      "IBAN": [
        "IBAN"
      ],
      "Bitcoin transfer details": [
        "Bitcoin -siirron tiedot"
      ],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.": [
        "Pörssi tarvitsee tapahtuman, jossa on 3 lähtöä, joista yksi on vaihtotili ja kaksi muuta ovat segwit fake -osoitteita metatiedoille vähimmäismäärällä."
      ],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts": [
        "Käytä bitcoincore-lompakossa &apos;Lisää vastaanottaja&apos; -painiketta lisätäksesi kaksi muuta vastaanottajaa ja kopioidaksesi osoitteet ja summat"
      ],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC": [
        "Varmista, että summa näyttää %1$s BTC, muuten sinun on vaihdettava perusyksikkö BTC:ksi"
      ],
      "Ethereum transfer details": [
        "Pankkisiirtotiedot"
      ],
      "You need to wire to the service provider account.": [
        ""
      ],
      "Make sure the amount show %1$s ETH": [
        ""
      ],
      "Bank transfer details": [
        "Pankkisiirtotiedot"
      ],
      "Account": [
        "Tili"
      ],
      "Bank host": [
        "Pankin isäntä"
      ],
      "BIC": [
        ""
      ],
      "Exchange": [
        ""
      ],
      "Reserve Pub": [
        ""
      ],
      "Step 1:": [
        ""
      ],
      "Copy this code and paste it into the subject/purpose field in your banking app or bank website": [
        ""
      ],
      "Subject": [
        "Aihe"
      ],
      "Step 2:": [
        ""
      ],
      "If you don't already have it in your banking favourites list, then copy and paste this IBAN and the name into the receiver fields in your banking app or website": [
        ""
      ],
      "Receiver name": [
        "Vastaanottajan nimi"
      ],
      "Receiver postal code": [
        "Vastaanottajan nimi"
      ],
      "Receiver town": [
        "Vastaanottajan nimi"
      ],
      "Step 3:": [
        ""
      ],
      "Finish the wire transfer setting the amount in your banking app or website, then this withdrawal will proceed automatically.": [
        ""
      ],
      "Make sure ALL data is correct, including the subject; otherwise, the money will not arrive in this wallet. You can use the copy buttons (%1$s) to prevent typing errors or the \"payto://\" URI below to copy just one value.": [
        ""
      ],
      "Alternative if your bank already supports PayTo URI, you can use this %1$s link instead": [
        ""
      ],
      "Continue with banking app or website": [
        ""
      ],
      "Show QR code": [
        ""
      ],
      "Could not load transaction information": [
        "Tapahtumatietoja ei voitu ladata"
      ],
      "Bank account confirmation required for the transaction to complete.": [
        ""
      ],
      "KYC check required for the transaction to complete.": [
        ""
      ],
      "No additional information has been provided.": [
        ""
      ],
      "Follow this link to the%1$s %2$s": [
        ""
      ],
      "This transaction is pending to complete.": [
        "Tätä tapahtumaa ei ole suoritettu loppuun"
      ],
      "Retry": [
        "Yritä uudelleen"
      ],
      "There was an error trying to complete the transaction.": [
        "Tapahtuman suorittamisessa tapahtui virhe"
      ],
      "This transaction was aborted.": [
        "Tätä tapahtumaa ei ole suoritettu loppuun"
      ],
      "This transaction failed.": [
        "Tätä tapahtumaa ei ole suoritettu loppuun"
      ],
      "Caution!": [
        "Varoitus!"
      ],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.": [
        "Jos olet jo siirtänyt rahaa vaihtoon, menetät mahdollisuuden saada kolikot siitä."
      ],
      "Confirm": [
        "Vahvista"
      ],
      "Doing a cancellation while the transaction still active might result in lost coins. Do you still want to cancel the transaction?": [
        ""
      ],
      "No": [
        ""
      ],
      "Yes": [
        ""
      ],
      "Abort": [
        ""
      ],
      "Resume": [
        ""
      ],
      "Suspend": [
        ""
      ],
      "Delete": [
        ""
      ],
      "Withdrawal": [
        "Nosto"
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Select one of the accounts and use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Details": [
        ""
      ],
      "Withdrawal incomplete.": [
        "Nosto"
      ],
      "If you have already sent money to the service provider account it will wire it back at %1$s": [
        "Jos olet jo siirtänyt rahaa vaihtoon, menetät mahdollisuuden saada kolikot siitä."
      ],
      "Payment": [
        ""
      ],
      "Refunds": [
        ""
      ],
      "%1$s %2$s on %3$s .": [
        ""
      ],
      "Refund is in progress.": [
        ""
      ],
      "Merchant created a refund for this order but was not automatically picked up.": [
        ""
      ],
      "Offer": [
        ""
      ],
      "Accept": [
        ""
      ],
      "Confirmation code": [
        ""
      ],
      "Merchant": [
        ""
      ],
      "Invoice ID": [
        ""
      ],
      "Deposit": [
        ""
      ],
      "Wire transfer deadline.": [
        "Bitcoin -siirron tiedot"
      ],
      "Wire transfer still pending.": [
        ""
      ],
      "The service provider deadline to make the wire transfer is: %1$s": [
        ""
      ],
      "Wire transfer completed.": [
        ""
      ],
      " ": [
        ""
      ],
      "Transfer details": [
        "Pankkisiirtotiedot"
      ],
      "Wire transfer in progress.": [
        ""
      ],
      "Refresh": [
        ""
      ],
      "Refund": [
        ""
      ],
      "Purchase summary": [
        ""
      ],
      "Credit": [
        ""
      ],
      "Invoice": [
        ""
      ],
      "URI": [
        ""
      ],
      "Debit": [
        ""
      ],
      "Transfer": [
        ""
      ],
      "Lost": [
        ""
      ],
      "Reason": [
        ""
      ],
      "Denomination expired.": [
        ""
      ],
      "Denomination vanished.": [
        ""
      ],
      "Denomination is unoffered.": [
        ""
      ],
      "Fees": [
        ""
      ],
      "Total": [
        ""
      ],
      "Sent": [
        ""
      ],
      "Converted": [
        ""
      ],
      "Bank fee": [
        "Max maksu"
      ],
      "Price": [
        ""
      ],
      "Subtotal": [
        ""
      ],
      "Refunded": [
        ""
      ],
      "Transfer identification": [
        ""
      ],
      "copy": [
        ""
      ],
      "hide qr": [
        ""
      ],
      "forward to contact": [
        ""
      ],
      "show qr": [
        ""
      ],
      "Wire transfer need a confirmation. Go to the %1$s and check wire transfer operation to exchange account is complete.": [
        ""
      ],
      "bank site": [
        "Pankin isäntä"
      ],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins.": [
        ""
      ],
      "Exchange is ready to send the coins, withdrawal in progress.": [
        ""
      ],
      "Send &nbsp; %1$s": [
        "Lähetä &nbsp; %1$s"
      ],
      "Pay &nbsp; %1$s": [
        ""
      ],
      "Merchant message": [
        ""
      ],
      "Balance is not enough.": [
        ""
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which exchange and auditors the merchant trust.": [
        ""
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which wire methods the merchant accepts.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to pay for this contract which is restricted.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to spend right know. There are some coins that need to be refreshed.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s available.": [
        ""
      ],
      "Provider is missing the global fee configuration, this likely means it is misconfigured.": [
        ""
      ],
      "Balance looks like it should be enough, but doesn't cover all fees requested by the merchant and payment processor. Please ensure there is at least %1$s %2$s more balance in your wallet or ask your merchant to cover more of the fees.": [
        ""
      ],
      "Pay with a mobile phone": [
        ""
      ],
      "Hide QR": [
        ""
      ],
      "Scan the QR code or &nbsp; %1$s": [
        ""
      ],
      "click here": [
        ""
      ],
      "Could not load the payment and balance status": [
        "Talletuksen tilaa ei voitu ladata"
      ],
      "Purchase": [
        ""
      ],
      "This purchase is age restricted.": [
        ""
      ],
      "Expires at": [
        "Vanhenee"
      ],
      "Already paid, you are going to be redirected to %1$s": [
        ""
      ],
      "Already paid": [
        ""
      ],
      "Already claimed": [
        ""
      ],
      "Could not load the refund status": [
        "Talletuksen tilaa ei voitu ladata"
      ],
      "You&apos;ve ignored the refund.": [
        ""
      ],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [
        ""
      ],
      "Order amount": [
        ""
      ],
      "Could not load the list of exchanges": [
        "Talletuksen tilaa ei voitu ladata"
      ],
      "Could not load info from URI": [
        "Ei voitu ladata saldosivua"
      ],
      "Could not load the withdrawal details": [
        "Ostoehdotuksen tietoja ei voitu ladata"
      ],
      "under %1$s": [
        "Lähetä %1$s"
      ],
      "should be greater than zero": [
        ""
      ],
      "can't be greater than %1$s": [
        ""
      ],
      "Could not load exchange details info": [
        "Ostoehdotuksen tietoja ei voitu ladata"
      ],
      "Close": [
        ""
      ],
      "There is no exchange available for currency %1$s": [
        ""
      ],
      "You can add more exchanges from the settings.": [
        ""
      ],
      "Exchange %1$s is not available": [
        ""
      ],
      "Exchange status can view accessed from the settings.": [
        ""
      ],
      "Service fee description": [
        ""
      ],
      "Select %1$s exchange": [
        ""
      ],
      "Reset": [
        ""
      ],
      "Use this exchange": [
        ""
      ],
      "Auditors": [
        "Tilintarkastajat"
      ],
      "Doesn&apos;t have auditors": [
        ""
      ],
      "currency": [
        ""
      ],
      "Coin operations": [
        ""
      ],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.": [
        ""
      ],
      "Deposits": [
        ""
      ],
      "Denomination": [
        ""
      ],
      "Current": [
        ""
      ],
      "Selected": [
        ""
      ],
      "Until": [
        ""
      ],
      "Withdrawals": [
        ""
      ],
      "Transfer operations": [
        ""
      ],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.": [
        ""
      ],
      "Operation": [
        ""
      ],
      "Fee": [
        "Maksu"
      ],
      "Wallet operations": [
        ""
      ],
      "Feature": [
        ""
      ],
      "Currency": [
        ""
      ],
      "This operation has already been completed.": [
        "Tätä tapahtumaa ei ole suoritettu loppuun"
      ],
      "This operation has already been aborted": [
        ""
      ],
      "This operation has started and should be completed in the bank.": [
        ""
      ],
      "You can confirm the operation in": [
        ""
      ],
      "this page": [
        ""
      ],
      "This operation has already been completed by another wallet.": [
        ""
      ],
      "This operation has already been used by another wallet.": [
        ""
      ],
      "It can be confirmed in": [
        ""
      ],
      "Age restriction": [
        ""
      ],
      "Withdraw &nbsp; %1$s": [
        ""
      ],
      "Withdraw to a mobile phone": [
        ""
      ],
      "Short description of the invoice": [
        ""
      ],
      "Create": [
        ""
      ],
      "Short description of the transfer": [
        ""
      ],
      "Could not load the invoice payment status": [
        "Talletuksen tilaa ei voitu ladata"
      ],
      "Receive &nbsp; %1$s": [
        ""
      ],
      "Could not load known bank accounts": [
        "Ei voitu ladata saldosivua"
      ],
      "Could not load supported wire methods": [
        "Varmuuskopion tarjoajia ei voitu ladata"
      ],
      "No wire methods supported for this currency": [
        ""
      ],
      "Try another wire method.": [
        ""
      ],
      "Known accounts for %1$s": [
        ""
      ],
      "To add a new account first select the account type.": [
        ""
      ],
      "Unable add this account": [
        ""
      ],
      "Add": [
        "Lisää"
      ],
      "IBAN accounts": [
        "Pankkitili"
      ],
      "Alias": [
        ""
      ],
      "Int. Account Number": [
        ""
      ],
      "Account name": [
        "Tili"
      ],
      "KYC": [
        ""
      ],
      "Taler accounts": [
        "Pankkitili"
      ],
      "Host": [
        ""
      ],
      "Bitcoin accounts": [
        "Pankkitili"
      ],
      "Address": [
        ""
      ],
      "Can't be empty": [
        ""
      ],
      "Bitcoin Account": [
        "Pankkitili"
      ],
      "Taler Bank": [
        "Taler toiminta"
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "Required": [
        ""
      ],
      "International Bank Account Number": [
        ""
      ],
      "Could not load balance information": [
        "Tapahtumatietoja ei voitu ladata"
      ],
      "Could not load fee for amount %1$s": [
        "Talletuksen tilaa ei voitu ladata"
      ],
      "A currency or an amount should be indicated": [
        ""
      ],
      "There is no enough balance to make a deposit for currency %1$s": [
        ""
      ],
      "Send %1$s to your account": [
        ""
      ],
      "There is no account to make a deposit for currency %1$s": [
        ""
      ],
      "Add account": [
        ""
      ],
      "Select account": [
        ""
      ],
      "Manage accounts": [
        "Pankkitili"
      ],
      "Brut amount": [
        "Pankkitili"
      ],
      "Deposit fee": [
        ""
      ],
      "Net amount": [
        "Summa"
      ],
      "Deposit&nbsp;%1$s %2$s": [
        ""
      ],
      "Need approval in the Bank": [
        ""
      ],
      "Waiting for wire transfer to complete": [
        ""
      ],
      "Exchange is waiting the wire transfer": [
        ""
      ],
      "Payment in progress": [
        ""
      ],
      "Executing refund...": [
        "Automaattinen palautus"
      ],
      "Refreshing coins...": [
        ""
      ],
      "Deposit in progress": [
        ""
      ],
      "Waiting to be paid": [
        ""
      ],
      "Receiving the transfer": [
        ""
      ],
      "Waiting to be received": [
        "Vastaanotettava"
      ],
      "Denomination expired": [
        ""
      ],
      "Denomination vanished": [
        ""
      ],
      "Denomination unoffered": [
        ""
      ],
      "ABORTED": [
        ""
      ],
      "FAILED": [
        ""
      ],
      "Could not load the list of transactions": [
        ""
      ],
      "unknown scope": [
        ""
      ],
      "Send": [
        "Lähetä"
      ],
      "Your transaction history is empty for this currency.": [
        ""
      ],
      "Review terms of service": [
        ""
      ],
      "Provider URL": [
        ""
      ],
      "Please review and accept this provider&apos;s terms of service": [
        ""
      ],
      "Pricing": [
        ""
      ],
      "free of charge": [
        ""
      ],
      "%1$s per year of service": [
        ""
      ],
      "Storage": [
        ""
      ],
      "%1$s megabytes of storage per year of service": [
        ""
      ],
      "Accept terms of service": [
        ""
      ],
      "Add backup provider": [
        ""
      ],
      "Backup providers may charge for their service": [
        ""
      ],
      "URL": [
        ""
      ],
      "Name": [
        ""
      ],
      "Next": [
        ""
      ],
      "Navigator": [
        ""
      ],
      "Automatically open wallet": [
        ""
      ],
      "Open the wallet when a payment action is found.": [
        ""
      ],
      "Version Info": [
        ""
      ],
      "Web Extension": [
        ""
      ],
      "The version of wallet core is not supported. (supported version: %1$s, wallet version: %2$s)": [
        ""
      ],
      "Exchange compatibility": [
        ""
      ],
      "Merchant compatibility": [
        ""
      ],
      "Bank compatibility": [
        ""
      ],
      "Wallet Core compatibility": [
        ""
      ],
      "Enable developer mode": [
        ""
      ],
      "Show more information and options in the UI": [
        ""
      ],
      "Display": [
        ""
      ],
      "Current Language": [
        ""
      ],
      "Show backup feature": [
        ""
      ],
      "Backup integration still in beta.": [
        ""
      ],
      "Show P2P alias feature": [
        ""
      ],
      "P2P aliases integration still in beta.": [
        ""
      ],
      "Show suspend/resume transaction": [
        ""
      ],
      "Prevent transaction from doing network request.": [
        ""
      ],
      "Show refresh transaction type in the transaction list": [
        ""
      ],
      "Refresh transaction will be hidden by default if the refresh operation doesn't have fee.": [
        ""
      ],
      "Show more account types on deposit": [
        ""
      ],
      "Extends the UI to more payment target types.": [
        ""
      ],
      "Show JSON on error": [
        ""
      ],
      "Print more information about the error. Useful for debugging.": [
        ""
      ],
      "Allow HTTP connections": [
        ""
      ],
      "Using HTTP connection may be faster but unsafe (wallet restart required)": [
        ""
      ],
      "Lang selector": [
        ""
      ],
      "Allows to manually change the language of the UI. Otherwise it will be automatically selected by your browser configuration.": [
        ""
      ],
      "Edit exchange management": [
        ""
      ],
      "Allows to see the list of exchange, remove, add and switch before withdrawal.": [
        ""
      ],
      "Select terms of service format": [
        ""
      ],
      "Allows to render the terms of service on different format selected by the user.": [
        ""
      ],
      "Show wallet activity": [
        ""
      ],
      "Show the wallet notification and observability event in the UI.": [
        ""
      ],
      "Enable v1 contract": [
        ""
      ],
      "v1 is under development and wallet-core converts v1 contract to v0. Enable this options for testing": [
        ""
      ],
      "GNU Taler Wallet installed!": [
        ""
      ],
      "You can open the wallet using the combination %1$s .": [
        ""
      ],
      "Also pinning the GNU Taler Wallet to your browser allows you to quick access without keyboard:": [
        ""
      ],
      "Click the puzzle icon": [
        ""
      ],
      "Search for GNU Taler Wallet": [
        ""
      ],
      "Click the pin icon": [
        ""
      ],
      "Next Steps": [
        ""
      ],
      "Try the demo": [
        ""
      ],
      "Click here to open the wallet activity tab.": [
        ""
      ],
      "Events": [
        ""
      ],
      "Active tasks": [
        ""
      ],
      "All events": [
        ""
      ],
      "Balance change": [
        "Saldo"
      ],
      "Backup failed": [
        "Varmuuskopio voimassa"
      ],
      "Transaction updated": [
        "Tätä tapahtumaa ei ole suoritettu loppuun"
      ],
      "Exchange updated": [
        "Vaihdot"
      ],
      "Idle": [
        ""
      ],
      "task.%1$s": [
        "%1$s"
      ],
      "wallet.%1$s(%2$s)": [
        ""
      ],
      "Bank account info changed": [
        "Pankkitili"
      ],
      "Contact added": [
        ""
      ],
      "Contact deleted": [
        ""
      ],
      "Mailbox message added": [
        ""
      ],
      "Mailbox message deleted": [
        ""
      ],
      "Type": [
        ""
      ],
      "Id": [
        ""
      ],
      "Since": [
        ""
      ],
      "Next try": [
        ""
      ],
      "Error": [
        ""
      ],
      "Transaction": [
        "Taler toiminta"
      ],
      "No exchange yet": [
        ""
      ],
      "Status": [
        ""
      ],
      "Terms of Service": [
        ""
      ],
      "Last Update": [
        ""
      ],
      "Actions": [
        "Taler toiminta"
      ],
      "ok": [
        ""
      ],
      "pending": [
        ""
      ],
      "proposed": [
        ""
      ],
      "unknown (exchange status should be updated)": [
        ""
      ],
      "Reload": [
        ""
      ],
      "Purge": [
        ""
      ],
      "Make regional": [
        ""
      ],
      "Make global": [
        ""
      ],
      "Forget ToS": [
        "Unohda"
      ],
      "Debug tools": [
        ""
      ],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?": [
        ""
      ],
      "reset": [
        ""
      ],
      "TESTING: This may delete all your coin, proceed with caution": [
        ""
      ],
      "run gc": [
        ""
      ],
      "import database": [
        ""
      ],
      "export database": [
        ""
      ],
      "Clear exchange key cache": [
        ""
      ],
      "Database exported at %1$s %2$s to download": [
        ""
      ],
      "Inject Taler support in all pages": [
        ""
      ],
      "Enabling this option will make `window.taler` be available in all sites": [
        ""
      ],
      "Exchange Entries": [
        "Vaihdot"
      ],
      "Add an exchange": [
        ""
      ],
      "Log levels": [
        ""
      ],
      "Coins": [
        ""
      ],
      "usable coins": [
        ""
      ],
      "id": [
        ""
      ],
      "denom": [
        ""
      ],
      "status": [
        ""
      ],
      "from refresh?": [
        ""
      ],
      "age key count": [
        ""
      ],
      "spent coins": [
        ""
      ],
      "click to show": [
        ""
      ],
      "URI is not valid. Taler URI should start with \"taler://\"": [
        ""
      ],
      "After the URI type it should follow a '/' with more information.": [
        ""
      ],
      "This URI type is not supported": [
        ""
      ],
      "This URI requires the bank host and operation id separated by /": [
        ""
      ],
      "This URI requires the merchant host, order id and session id separated by /": [
        ""
      ],
      "This URI requires the merchant host and order id separated by /": [
        ""
      ],
      "This URI requires the exchange host and key id separated by /": [
        ""
      ],
      "This URI requires the merchant host and template id separated by /": [
        ""
      ],
      "This URI requires the host key and provider list separated by /": [
        ""
      ],
      "This URI requires the experiment id": [
        ""
      ],
      "This URI requires the exchange host": [
        ""
      ],
      "This URI requires the alias type, the alias and the mailbox URI separated by /": [
        ""
      ],
      "This URI requires string after the first /": [
        ""
      ],
      "The bank host is invalid": [
        ""
      ],
      "The merchant host is invalid": [
        ""
      ],
      "The URI should end with /": [
        ""
      ],
      "The exchange host is invalid": [
        ""
      ],
      "The contact is invalid": [
        ""
      ],
      "The amount is invalid": [
        ""
      ],
      "A parameter is invalid": [
        ""
      ],
      "checking...": [
        ""
      ],
      "Unexpected error happen reading the camera: %1$s": [
        ""
      ],
      "Could not found a QR code in the file": [
        ""
      ],
      "Unexpected error happen reading the file: %1$s": [
        ""
      ],
      "Scan a QR code or enter taler:// URI below": [
        ""
      ],
      "Pay invoice": [
        "Maksun määräaika"
      ],
      "Withdrawal from bank": [
        "Nosto"
      ],
      "Claim refund": [
        "Automaattinen palautus"
      ],
      "Accept payment": [
        ""
      ],
      "Complete order": [
        ""
      ],
      "Restore wallet": [
        ""
      ],
      "Enable experiment": [
        ""
      ],
      "Withdraw from exchange": [
        ""
      ],
      "Add exchange": [
        ""
      ],
      "Notify transaction": [
        ""
      ],
      "Add contact": [
        ""
      ],
      "The exchange URL is invalid.": [
        ""
      ],
      "The exchange URL should end with '/'": [
        ""
      ],
      "Couldn't found an exchange in the URL specified.": [
        ""
      ],
      "HTTP request failed to %1$s: %2$s": [
        ""
      ],
      "HTTP request failed to %1$s": [
        ""
      ],
      "The merchant URL is invalid.": [
        ""
      ],
      "The merchant URL should end with '/'": [
        ""
      ],
      "Couldn't found an merchant in the URL specified.": [
        ""
      ],
      "The bank URL is invalid.": [
        ""
      ],
      "The bank URL should end with '/'": [
        ""
      ],
      "Couldn't found an bank in the URL specified.": [
        ""
      ],
      "Could not load exchanges": [
        "Ei voitu ladata saldosivua"
      ],
      "Choose a currency to proceed or add another exchange": [
        ""
      ],
      "Known currencies": [
        ""
      ],
      "Specify the origin": [
        ""
      ],
      "Use previous origins:": [
        ""
      ],
      "From another bank account": [
        ""
      ],
      "From another wallet": [
        ""
      ],
      "From a %1$s URI or QR code": [
        ""
      ],
      "Enter URI here": [
        ""
      ],
      "Specify the destination": [
        ""
      ],
      "Use previous destinations:": [
        ""
      ],
      "To another bank account": [
        "Pankkitili"
      ],
      "To another wallet": [
        ""
      ],
      "To a %1$s URI or QR code": [
        ""
      ],
      "Afghanistan": [
        ""
      ],
      "Albania": [
        ""
      ],
      "Algeria": [
        ""
      ],
      "United States": [
        ""
      ],
      "Andorra": [
        ""
      ],
      "Angola": [
        ""
      ],
      "Antigua and Barbuda": [
        ""
      ],
      "Argentina": [
        ""
      ],
      "Armenia": [
        ""
      ],
      "Australia": [
        ""
      ],
      "Austria": [
        ""
      ],
      "Azerbaijan": [
        ""
      ],
      "Bahamas": [
        ""
      ],
      "Bahrain": [
        ""
      ],
      "Bangladesh": [
        ""
      ],
      "Barbados": [
        ""
      ],
      "Botswana": [
        ""
      ],
      "Belarus": [
        ""
      ],
      "Belgium": [
        ""
      ],
      "Belize": [
        ""
      ],
      "Benin": [
        ""
      ],
      "Bhutan": [
        ""
      ],
      "Bolivia": [
        ""
      ],
      "Bosnia and Herzegovina": [
        ""
      ],
      "Brazil": [
        ""
      ],
      "United Kingdom": [
        ""
      ],
      "Brunei": [
        ""
      ],
      "Bulgaria": [
        ""
      ],
      "Burkina Faso": [
        ""
      ],
      "Myanmar": [
        ""
      ],
      "Burundi": [
        ""
      ],
      "Cambodia": [
        ""
      ],
      "Cameroon": [
        ""
      ],
      "Canada": [
        ""
      ],
      "Cape Verde": [
        ""
      ],
      "Central African Republic": [
        ""
      ],
      "Chad": [
        ""
      ],
      "Chile": [
        ""
      ],
      "China": [
        ""
      ],
      "Colombia": [
        ""
      ],
      "Comoros": [
        ""
      ],
      "Democratic Republic of the Congo": [
        ""
      ],
      "Republic of the Congo": [
        ""
      ],
      "Costa Rica": [
        ""
      ],
      "Croatia": [
        ""
      ],
      "Cuba": [
        ""
      ],
      "Cyprus": [
        ""
      ],
      "Czech Republic": [
        "Kauppiaan julkinen avain"
      ],
      "Denmark": [
        ""
      ],
      "Djibouti": [
        ""
      ],
      "Dominican Republic": [
        ""
      ],
      "Netherlands": [
        ""
      ],
      "Timor-Leste": [
        ""
      ],
      "Ecuador": [
        ""
      ],
      "Egypt": [
        ""
      ],
      "United Arab Emirates": [
        ""
      ],
      "Equatorial Guinea": [
        ""
      ],
      "Eritrea": [
        ""
      ],
      "Estonia": [
        ""
      ],
      "Ethiopia": [
        ""
      ],
      "Fiji": [
        ""
      ],
      "Philippines": [
        ""
      ],
      "Finland": [
        ""
      ],
      "France": [
        "Peruuta"
      ],
      "Gabon": [
        ""
      ],
      "Gambia": [
        ""
      ],
      "Georgia": [
        ""
      ],
      "Germany": [
        ""
      ],
      "Ghana": [
        ""
      ],
      "Greece": [
        ""
      ],
      "Grenada": [
        ""
      ],
      "Guatemala": [
        ""
      ],
      "Guinea-Bissau": [
        ""
      ],
      "Guinea": [
        ""
      ],
      "Guyana": [
        ""
      ],
      "Haiti": [
        ""
      ],
      "Honduras": [
        ""
      ],
      "Hungary": [
        ""
      ],
      "Kiribati": [
        ""
      ],
      "Iceland": [
        ""
      ],
      "India": [
        ""
      ],
      "Indonesia": [
        ""
      ],
      "Iran": [
        ""
      ],
      "Iraq": [
        ""
      ],
      "Ireland": [
        ""
      ],
      "Israel": [
        ""
      ],
      "Italy": [
        ""
      ],
      "Ivory Coast": [
        ""
      ],
      "Jamaica": [
        ""
      ],
      "Japan": [
        ""
      ],
      "Jordan": [
        ""
      ],
      "Kazakhstan": [
        ""
      ],
      "Kenya": [
        ""
      ],
      "Saint Kitts and Nevis": [
        ""
      ],
      "Kuwait": [
        ""
      ],
      "Kyrgyzstan": [
        ""
      ],
      "Laos": [
        ""
      ],
      "Latvia": [
        ""
      ],
      "Lebanon": [
        ""
      ],
      "Liberia": [
        ""
      ],
      "Libya": [
        ""
      ],
      "Liechtenstein": [
        ""
      ],
      "Lithuania": [
        ""
      ],
      "Luxembourg": [
        ""
      ],
      "North Macedonia": [
        ""
      ],
      "Madagascar": [
        ""
      ],
      "Malawi": [
        ""
      ],
      "Malaysia": [
        ""
      ],
      "Maldives": [
        ""
      ],
      "Mali": [
        ""
      ],
      "Malta": [
        ""
      ],
      "Marshall Islands": [
        ""
      ],
      "Mauritania": [
        ""
      ],
      "Mauritius": [
        ""
      ],
      "Mexico": [
        ""
      ],
      "Micronesia": [
        ""
      ],
      "Moldova": [
        ""
      ],
      "Monaco": [
        ""
      ],
      "Mongolia": [
        ""
      ],
      "Morocco": [
        ""
      ],
      "Lesotho": [
        ""
      ],
      "Mozambique": [
        ""
      ],
      "Namibia": [
        ""
      ],
      "Nauru": [
        ""
      ],
      "Nepal": [
        ""
      ],
      "New Zealand": [
        ""
      ],
      "Vanuatu": [
        ""
      ],
      "Nicaragua": [
        ""
      ],
      "Nigeria": [
        ""
      ],
      "Niger": [
        ""
      ],
      "North Korea": [
        ""
      ],
      "Norway": [
        ""
      ],
      "Oman": [
        ""
      ],
      "Pakistan": [
        ""
      ],
      "Palau": [
        ""
      ],
      "Panama": [
        ""
      ],
      "Papua New Guinea": [
        ""
      ],
      "Paraguay": [
        ""
      ],
      "Peru": [
        ""
      ],
      "Poland": [
        ""
      ],
      "Portugal": [
        ""
      ],
      "Qatar": [
        ""
      ],
      "Romania": [
        ""
      ],
      "Russia": [
        ""
      ],
      "Rwanda": [
        ""
      ],
      "Saint Lucia": [
        ""
      ],
      "El Salvador": [
        ""
      ],
      "Samoa": [
        ""
      ],
      "San Marino": [
        ""
      ],
      "São Tomé and Príncipe": [
        ""
      ],
      "Saudi Arabia": [
        ""
      ],
      "Senegal": [
        ""
      ],
      "Serbia": [
        ""
      ],
      "Seychelles": [
        ""
      ],
      "Sierra Leone": [
        ""
      ],
      "Singapore": [
        ""
      ],
      "Slovakia": [
        ""
      ],
      "Slovenia": [
        ""
      ],
      "Solomon Islands": [
        ""
      ],
      "Somalia": [
        ""
      ],
      "South Africa": [
        ""
      ],
      "South Korea": [
        ""
      ],
      "Spain": [
        ""
      ],
      "Sri Lanka": [
        ""
      ],
      "Sudan": [
        ""
      ],
      "Suriname": [
        ""
      ],
      "Eswatini": [
        ""
      ],
      "Sweden": [
        ""
      ],
      "Switzerland": [
        ""
      ],
      "Syria": [
        ""
      ],
      "Taiwan": [
        ""
      ],
      "Tajikistan": [
        ""
      ],
      "Tanzania": [
        ""
      ],
      "Thailand": [
        ""
      ],
      "Togo": [
        ""
      ],
      "Tonga": [
        ""
      ],
      "Trinidad and Tobago": [
        ""
      ],
      "Tunisia": [
        ""
      ],
      "Turkey": [
        ""
      ],
      "Tuvalu": [
        ""
      ],
      "Uganda": [
        ""
      ],
      "Ukraine": [
        ""
      ],
      "Uruguay": [
        ""
      ],
      "Uzbekistan": [
        ""
      ],
      "Venezuela": [
        ""
      ],
      "Vietnam": [
        ""
      ],
      "Yemen": [
        ""
      ],
      "Zambia": [
        ""
      ],
      "Zimbabwe": [
        ""
      ],
      "Afghan": [
        ""
      ],
      "Albanian": [
        ""
      ],
      "Algerian": [
        ""
      ],
      "American": [
        ""
      ],
      "Andorran": [
        ""
      ],
      "Angolan": [
        ""
      ],
      "Antiguan": [
        ""
      ],
      "Argentinean": [
        ""
      ],
      "Armenian": [
        ""
      ],
      "Australian": [
        ""
      ],
      "Austrian": [
        ""
      ],
      "Azerbaijani": [
        ""
      ],
      "Bahamian": [
        ""
      ],
      "Bahraini": [
        ""
      ],
      "Bangladeshi": [
        ""
      ],
      "Barbadian": [
        ""
      ],
      "Batswana": [
        ""
      ],
      "Belarusian": [
        ""
      ],
      "Belgian": [
        ""
      ],
      "Belizean": [
        ""
      ],
      "Beninese": [
        ""
      ],
      "Bhutanese": [
        ""
      ],
      "Bolivian": [
        ""
      ],
      "Bosnian": [
        ""
      ],
      "Brazilian": [
        ""
      ],
      "British": [
        ""
      ],
      "Bruneian": [
        ""
      ],
      "Bulgarian": [
        ""
      ],
      "Burkinabe": [
        ""
      ],
      "Burmese": [
        ""
      ],
      "Burundian": [
        ""
      ],
      "Cambodian": [
        ""
      ],
      "Cameroonian": [
        ""
      ],
      "Canadian": [
        ""
      ],
      "Cape Verdean": [
        ""
      ],
      "Central African": [
        ""
      ],
      "Chadian": [
        ""
      ],
      "Chilean": [
        ""
      ],
      "Chinese": [
        ""
      ],
      "Colombian": [
        ""
      ],
      "Comoran": [
        ""
      ],
      "Congolese": [
        ""
      ],
      "Costa Rican": [
        ""
      ],
      "Croatian": [
        ""
      ],
      "Cuban": [
        ""
      ],
      "Cypriot": [
        ""
      ],
      "Czech": [
        ""
      ],
      "Danish": [
        ""
      ],
      "Djiboutian": [
        ""
      ],
      "Dominican": [
        ""
      ],
      "Dutch": [
        ""
      ],
      "East Timorese": [
        ""
      ],
      "Ecuadorean": [
        ""
      ],
      "Egyptian": [
        ""
      ],
      "Emirati": [
        ""
      ],
      "Equatorial Guinean": [
        ""
      ],
      "Eritrean": [
        ""
      ],
      "Estonian": [
        ""
      ],
      "Ethiopian": [
        ""
      ],
      "Fijian": [
        ""
      ],
      "Filipino": [
        ""
      ],
      "Finnish": [
        ""
      ],
      "French": [
        ""
      ],
      "Gabonese": [
        ""
      ],
      "Gambian": [
        ""
      ],
      "Georgian": [
        ""
      ],
      "German": [
        ""
      ],
      "Ghanaian": [
        ""
      ],
      "Greek": [
        ""
      ],
      "Grenadian": [
        ""
      ],
      "Guatemalan": [
        ""
      ],
      "Guinea-Bissauan": [
        ""
      ],
      "Guinean": [
        ""
      ],
      "Guyanese": [
        ""
      ],
      "Haitian": [
        ""
      ],
      "Honduran": [
        ""
      ],
      "Hungarian": [
        ""
      ],
      "I-Kiribati": [
        ""
      ],
      "Icelander": [
        ""
      ],
      "Indian": [
        ""
      ],
      "Indonesian": [
        ""
      ],
      "Iranian": [
        ""
      ],
      "Iraqi": [
        ""
      ],
      "Irish": [
        ""
      ],
      "Israeli": [
        ""
      ],
      "Italian": [
        ""
      ],
      "Ivorian": [
        ""
      ],
      "Jamaican": [
        ""
      ],
      "Japanese": [
        ""
      ],
      "Jordanian": [
        ""
      ],
      "Kazakhstani": [
        ""
      ],
      "Kenyan": [
        ""
      ],
      "Kittian and Nevisian": [
        ""
      ],
      "Kuwaiti": [
        ""
      ],
      "Kyrgyz": [
        ""
      ],
      "Laotian": [
        ""
      ],
      "Latvian": [
        ""
      ],
      "Lebanese": [
        ""
      ],
      "Liberian": [
        ""
      ],
      "Libyan": [
        ""
      ],
      "Liechtensteiner": [
        ""
      ],
      "Lithuanian": [
        ""
      ],
      "Luxembourger": [
        ""
      ],
      "Macedonian": [
        ""
      ],
      "Malagasy": [
        ""
      ],
      "Malawian": [
        ""
      ],
      "Malaysian": [
        ""
      ],
      "Maldivian": [
        ""
      ],
      "Malian": [
        ""
      ],
      "Maltese": [
        ""
      ],
      "Marshallese": [
        ""
      ],
      "Mauritanian": [
        ""
      ],
      "Mauritian": [
        ""
      ],
      "Mexican": [
        ""
      ],
      "Micronesian": [
        ""
      ],
      "Moldovan": [
        ""
      ],
      "Monacan": [
        ""
      ],
      "Mongolian": [
        ""
      ],
      "Moroccan": [
        ""
      ],
      "Mosotho": [
        ""
      ],
      "Mozambican": [
        ""
      ],
      "Namibian": [
        ""
      ],
      "Nauruan": [
        ""
      ],
      "Nepalese": [
        ""
      ],
      "New Zealander": [
        ""
      ],
      "Ni-Vanuatu": [
        ""
      ],
      "Nicaraguan": [
        ""
      ],
      "Nigerian": [
        ""
      ],
      "Nigerien": [
        ""
      ],
      "North Korean": [
        ""
      ],
      "Norwegian": [
        ""
      ],
      "Omani": [
        ""
      ],
      "Pakistani": [
        ""
      ],
      "Palauan": [
        ""
      ],
      "Panamanian": [
        ""
      ],
      "Papua New Guinean": [
        ""
      ],
      "Paraguayan": [
        ""
      ],
      "Peruvian": [
        ""
      ],
      "Polish": [
        ""
      ],
      "Portuguese": [
        ""
      ],
      "Qatari": [
        ""
      ],
      "Romanian": [
        ""
      ],
      "Russian": [
        ""
      ],
      "Rwandan": [
        ""
      ],
      "Saint Lucian": [
        ""
      ],
      "Salvadoran": [
        ""
      ],
      "Samoan": [
        ""
      ],
      "San Marinese": [
        ""
      ],
      "Sao Tomean": [
        ""
      ],
      "Saudi": [
        ""
      ],
      "Senegalese": [
        ""
      ],
      "Serbian": [
        ""
      ],
      "Seychellois": [
        ""
      ],
      "Sierra Leonean": [
        ""
      ],
      "Singaporean": [
        ""
      ],
      "Slovakian": [
        ""
      ],
      "Slovenian": [
        ""
      ],
      "Solomon Islander": [
        ""
      ],
      "Somali": [
        ""
      ],
      "South African": [
        ""
      ],
      "South Korean": [
        ""
      ],
      "Spanish": [
        ""
      ],
      "Sri Lankan": [
        ""
      ],
      "Sudanese": [
        ""
      ],
      "Surinamer": [
        ""
      ],
      "Swazi": [
        ""
      ],
      "Swedish": [
        ""
      ],
      "Swiss": [
        ""
      ],
      "Syrian": [
        ""
      ],
      "Taiwanese": [
        ""
      ],
      "Tajik": [
        ""
      ],
      "Tanzanian": [
        ""
      ],
      "Thai": [
        ""
      ],
      "Togolese": [
        ""
      ],
      "Tongan": [
        ""
      ],
      "Trinidadian or Tobagonian": [
        ""
      ],
      "Tunisian": [
        ""
      ],
      "Turkish": [
        ""
      ],
      "Tuvaluan": [
        ""
      ],
      "Ugandan": [
        ""
      ],
      "Ukrainian": [
        ""
      ],
      "Uruguayan": [
        ""
      ],
      "Uzbekistani": [
        ""
      ],
      "Venezuelan": [
        ""
      ],
      "Vietnamese": [
        ""
      ],
      "Yemenite": [
        ""
      ],
      "Zambian": [
        ""
      ],
      "Zimbabwean": [
        ""
      ],
      "🇦🇫": [
        ""
      ],
      "🇦🇱": [
        ""
      ],
      "🇩🇿": [
        ""
      ],
      "🇺🇸": [
        ""
      ],
      "🇦🇩": [
        ""
      ],
      "🇦🇴": [
        ""
      ],
      "🇦🇬": [
        ""
      ],
      "🇦🇷": [
        ""
      ],
      "🇦🇲": [
        ""
      ],
      "🇦🇺": [
        ""
      ],
      "🇦🇹": [
        ""
      ],
      "🇦🇿": [
        ""
      ],
      "🇧🇸": [
        ""
      ],
      "🇧🇭": [
        ""
      ],
      "🇧🇩": [
        ""
      ],
      "🇧🇧": [
        ""
      ],
      "🇧🇼": [
        ""
      ],
      "🇧🇾": [
        ""
      ],
      "🇧🇪": [
        ""
      ],
      "🇧🇿": [
        ""
      ],
      "🇧🇯": [
        ""
      ],
      "🇧🇹": [
        ""
      ],
      "🇧🇴": [
        ""
      ],
      "🇧🇦": [
        ""
      ],
      "🇧🇷": [
        ""
      ],
      "🇬🇧": [
        ""
      ],
      "🇧🇳": [
        ""
      ],
      "🇧🇬": [
        ""
      ],
      "🇧🇫": [
        ""
      ],
      "🇲🇲": [
        ""
      ],
      "🇧🇮": [
        ""
      ],
      "🇰🇭": [
        ""
      ],
      "🇨🇲": [
        ""
      ],
      "🇨🇦": [
        ""
      ],
      "🇨🇻": [
        ""
      ],
      "🇨🇫": [
        ""
      ],
      "🇨🇱": [
        ""
      ],
      "🇨🇳": [
        ""
      ],
      "🇨🇴": [
        ""
      ],
      "🇨🇩": [
        ""
      ],
      "🇨🇬": [
        ""
      ],
      "🇨🇷": [
        ""
      ],
      "🇨🇺": [
        ""
      ],
      "🇨🇾": [
        ""
      ],
      "🇨🇿": [
        ""
      ],
      "🇩🇰": [
        ""
      ],
      "🇩🇯": [
        ""
      ],
      "🇩🇴": [
        ""
      ],
      "🇳🇱": [
        ""
      ],
      "🇹🇱": [
        ""
      ],
      "🇪🇨": [
        ""
      ],
      "🇪🇬": [
        ""
      ],
      "🇦🇪": [
        ""
      ],
      "🇬🇶": [
        ""
      ],
      "🇪🇷": [
        ""
      ],
      "🇪🇪": [
        ""
      ],
      "🇪🇹": [
        ""
      ],
      "🇫🇯": [
        ""
      ],
      "🇵🇭": [
        ""
      ],
      "🇫🇮": [
        ""
      ],
      "🇫🇷": [
        ""
      ],
      "🇬🇦": [
        ""
      ],
      "🇬🇲": [
        ""
      ],
      "🇬🇪": [
        ""
      ],
      "UAE Dirham": [
        ""
      ],
      "Afghani": [
        ""
      ],
      "Lek": [
        ""
      ],
      "Armenian Dram": [
        ""
      ],
      "Netherlands Antillean Guilder": [
        ""
      ],
      "Kwanza": [
        ""
      ],
      "Argentine Peso": [
        ""
      ],
      "Australian Dollar": [
        ""
      ],
      "Aruban Florin": [
        ""
      ],
      "Azerbaijan Manat": [
        ""
      ],
      "Convertible Mark": [
        ""
      ],
      "Barbados Dollar": [
        ""
      ],
      "Taka": [
        ""
      ],
      "Bulgarian Lev": [
        ""
      ],
      "Bahraini Dinar": [
        ""
      ],
      "Burundi Franc": [
        ""
      ],
      "Bermudian Dollar": [
        ""
      ],
      "Brunei Dollar": [
        ""
      ],
      "Boliviano": [
        ""
      ],
      "Mvdol": [
        ""
      ],
      "Brazilian Real": [
        ""
      ],
      "Bahamian Dollar": [
        ""
      ],
      "Ngultrum": [
        ""
      ],
      "Pula": [
        ""
      ],
      "Belarusian Ruble": [
        ""
      ],
      "Belize Dollar": [
        ""
      ],
      "Canadian Dollar": [
        ""
      ],
      "Congolese Franc": [
        ""
      ],
      "WIR Euro": [
        ""
      ],
      "Swiss Franc": [
        ""
      ],
      "WIR Franc": [
        ""
      ],
      "Unidad de Fomento": [
        ""
      ],
      "Chilean Peso": [
        ""
      ],
      "Yuan Renminbi": [
        ""
      ],
      "Colombian Peso": [
        ""
      ],
      "Unidad de Valor Real": [
        ""
      ],
      "Costa Rican Colon": [
        ""
      ],
      "Cuban Peso": [
        ""
      ],
      "Cabo Verde Escudo": [
        ""
      ],
      "Czech Koruna": [
        ""
      ],
      "Djibouti Franc": [
        ""
      ],
      "Danish Krone": [
        ""
      ],
      "Dominican Peso": [
        ""
      ],
      "Algerian Dinar": [
        ""
      ],
      "Egyptian Pound": [
        ""
      ],
      "Nakfa": [
        ""
      ],
      "Ethiopian Birr": [
        ""
      ],
      "Euro": [
        ""
      ],
      "Fiji Dollar": [
        ""
      ],
      "Falkland Islands Pound": [
        ""
      ],
      "Pound Sterling": [
        ""
      ],
      "Lari": [
        ""
      ],
      "Ghana Cedi": [
        ""
      ],
      "Gibraltar Pound": [
        ""
      ],
      "Dalasi": [
        ""
      ],
      "Guinean Franc": [
        ""
      ],
      "Quetzal": [
        ""
      ],
      "Guyana Dollar": [
        ""
      ],
      "Hong Kong Dollar": [
        ""
      ],
      "Lempira": [
        ""
      ],
      "Gourde": [
        ""
      ],
      "Forint": [
        ""
      ],
      "Rupiah": [
        ""
      ],
      "New Israeli Sheqel": [
        ""
      ],
      "Indian Rupee": [
        ""
      ],
      "Iraqi Dinar": [
        ""
      ],
      "Iranian Rial": [
        ""
      ],
      "Iceland Krona": [
        ""
      ],
      "Jamaican Dollar": [
        ""
      ],
      "Jordanian Dinar": [
        ""
      ],
      "Yen": [
        ""
      ],
      "Kenyan Shilling": [
        ""
      ],
      "Som": [
        ""
      ],
      "Riel": [
        ""
      ],
      "Comorian Franc ": [
        ""
      ],
      "North Korean Won": [
        ""
      ],
      "Won": [
        ""
      ],
      "Kuwaiti Dinar": [
        ""
      ],
      "Cayman Islands Dollar": [
        ""
      ],
      "Tenge": [
        ""
      ],
      "Lao Kip": [
        ""
      ],
      "Lebanese Pound": [
        ""
      ],
      "Sri Lanka Rupee": [
        ""
      ],
      "Liberian Dollar": [
        ""
      ],
      "Loti": [
        ""
      ],
      "Libyan Dinar": [
        ""
      ],
      "Moroccan Dirham": [
        ""
      ],
      "Moldovan Leu": [
        ""
      ],
      "Malagasy Ariary": [
        ""
      ],
      "Denar": [
        ""
      ],
      "Kyat": [
        ""
      ],
      "Tugrik": [
        ""
      ],
      "Pataca": [
        ""
      ],
      "Ouguiya": [
        ""
      ],
      "Mauritius Rupee": [
        ""
      ],
      "Rufiyaa": [
        ""
      ],
      "Malawi Kwacha": [
        ""
      ],
      "Mexican Peso": [
        ""
      ],
      "Mexican Unidad de Inversion (UDI)": [
        ""
      ],
      "Malaysian Ringgit": [
        ""
      ],
      "Mozambique Metical": [
        ""
      ],
      "Namibia Dollar": [
        ""
      ],
      "Naira": [
        ""
      ],
      "Cordoba Oro": [
        ""
      ],
      "Norwegian Krone": [
        ""
      ],
      "Nepalese Rupee": [
        ""
      ],
      "New Zealand Dollar": [
        ""
      ],
      "Rial Omani": [
        ""
      ],
      "Balboa": [
        ""
      ],
      "Sol": [
        "vanha"
      ],
      "Kina": [
        ""
      ],
      "Philippine Peso": [
        ""
      ],
      "Pakistan Rupee": [
        ""
      ],
      "Zloty": [
        ""
      ],
      "Guarani": [
        ""
      ],
      "Qatari Rial": [
        ""
      ],
      "Romanian Leu": [
        ""
      ],
      "Serbian Dinar": [
        ""
      ],
      "Russian Ruble": [
        ""
      ],
      "Rwanda Franc": [
        ""
      ],
      "Saudi Riyal": [
        ""
      ],
      "Solomon Islands Dollar": [
        ""
      ],
      "Seychelles Rupee": [
        ""
      ],
      "Sudanese Pound": [
        ""
      ],
      "Swedish Krona": [
        ""
      ],
      "Singapore Dollar": [
        ""
      ],
      "Saint Helena Pound": [
        ""
      ],
      "Leone": [
        ""
      ],
      "Somali Shilling": [
        ""
      ],
      "Surinam Dollar": [
        ""
      ],
      "South Sudanese Pound": [
        ""
      ],
      "Dobra": [
        ""
      ],
      "El Salvador Colon": [
        ""
      ],
      "Syrian Pound": [
        ""
      ],
      "Lilangeni": [
        ""
      ],
      "Baht": [
        ""
      ],
      "Somoni": [
        ""
      ],
      "Turkmenistan New Manat": [
        ""
      ],
      "Tunisian Dinar": [
        ""
      ],
      "Pa’anga": [
        ""
      ],
      "Turkish Lira": [
        ""
      ],
      "Trinidad and Tobago Dollar": [
        ""
      ],
      "New Taiwan Dollar": [
        ""
      ],
      "Tanzanian Shilling": [
        ""
      ],
      "Hryvnia": [
        ""
      ],
      "Uganda Shilling": [
        ""
      ],
      "US Dollar": [
        ""
      ],
      "US Dollar (Next day)": [
        ""
      ],
      "Uruguay Peso en Unidades Indexadas (UI)": [
        ""
      ],
      "Peso Uruguayo": [
        ""
      ],
      "Unidad Previsional": [
        ""
      ],
      "Uzbekistan Sum": [
        ""
      ],
      "Bolívar Soberano": [
        ""
      ],
      "Dong": [
        ""
      ],
      "Vatu": [
        ""
      ],
      "Tala": [
        ""
      ],
      "CFA Franc BEAC": [
        ""
      ],
      "East Caribbean Dollar": [
        ""
      ],
      "CFA Franc BCEAO": [
        ""
      ],
      "CFP Franc": [
        ""
      ],
      "Sucre": [
        ""
      ],
      "Yemeni Rial": [
        ""
      ],
      "Rand": [
        ""
      ],
      "Zambian Kwacha": [
        ""
      ],
      "Zimbabwe Gold": [
        ""
      ],
      "Afar": [
        ""
      ],
      "Abkhazian": [
        ""
      ],
      "Achinese": [
        ""
      ],
      "Acoli": [
        ""
      ],
      "Adangme": [
        ""
      ],
      "Adyghe": [
        ""
      ],
      "Avestan": [
        ""
      ],
      "Tunisian Arabic": [
        ""
      ],
      "Afrikaans": [
        ""
      ],
      "Afrihili": [
        ""
      ],
      "Aghem": [
        ""
      ],
      "Ainu": [
        ""
      ],
      "Akan": [
        ""
      ],
      "Akkadian": [
        ""
      ],
      "Alabama": [
        ""
      ],
      "Aleut": [
        ""
      ],
      "Gheg Albanian": [
        ""
      ],
      "Southern Altai": [
        ""
      ],
      "Amarik": [
        ""
      ],
      "Aragonese": [
        ""
      ],
      "Old English": [
        ""
      ],
      "Angika": [
        ""
      ],
      "Modern Standard Arabic": [
        ""
      ],
      "Arabik": [
        ""
      ],
      "Aramaic": [
        ""
      ],
      "Mapuche": [
        ""
      ],
      "Araona": [
        ""
      ],
      "Arapaho": [
        ""
      ],
      "Algerian Arabic": [
        ""
      ],
      "Arawak": [
        ""
      ],
      "Moroccan Arabic": [
        ""
      ],
      "Egyptian Arabic": [
        ""
      ],
      "Asu": [
        ""
      ],
      "Assamese": [
        ""
      ],
      "American Sign Language": [
        ""
      ],
      "Asturian": [
        ""
      ],
      "Avaric": [
        ""
      ],
      "Kotava": [
        ""
      ],
      "Awadhi": [
        ""
      ],
      "Aymara": [
        ""
      ],
      "South Azerbaijani": [
        ""
      ],
      "Bashkir": [
        ""
      ],
      "Baluchi": [
        ""
      ],
      "Balinese": [
        ""
      ],
      "Bavarian": [
        ""
      ],
      "Basaa": [
        ""
      ],
      "Bamun": [
        ""
      ],
      "Batak Toba": [
        ""
      ],
      "Ghomala": [
        ""
      ],
      "Belarus kasa": [
        ""
      ],
      "Beja": [
        ""
      ],
      "Bemba": [
        ""
      ],
      "Betawi": [
        ""
      ],
      "Bena": [
        ""
      ],
      "Bafut": [
        ""
      ],
      "Badaga": [
        ""
      ],
      "Bɔlgeria kasa": [
        ""
      ],
      "Bhojpuri": [
        ""
      ],
      "Bislama": [
        ""
      ],
      "Bikol": [
        ""
      ],
      "Bini": [
        ""
      ],
      "Banjar": [
        ""
      ],
      "Kom": [
        ""
      ],
      "Siksika": [
        ""
      ],
      "Bambara": [
        ""
      ],
      "Bengali kasa": [
        ""
      ],
      "Tibetan": [
        ""
      ],
      "Bishnupriya": [
        ""
      ],
      "Bakhtiari": [
        ""
      ],
      "Braj": [
        ""
      ],
      "Breton": [
        ""
      ],
      "Brahui": [
        ""
      ],
      "Bodo": [
        ""
      ],
      "Akoose": [
        ""
      ],
      "Buriat": [
        ""
      ],
      "Buginese": [
        ""
      ],
      "Bulu": [
        ""
      ],
      "Blin": [
        ""
      ],
      "Medumba": [
        ""
      ],
      "Catalan": [
        ""
      ],
      "Caddo": [
        ""
      ],
      "Carib": [
        ""
      ],
      "Cayuga": [
        ""
      ],
      "Atsam": [
        ""
      ],
      "Cebuano": [
        ""
      ],
      "Chechen": [
        ""
      ],
      "Chiga": [
        ""
      ],
      "Chibcha": [
        ""
      ],
      "Chamorro": [
        ""
      ],
      "Chagatai": [
        ""
      ],
      "Chuukese": [
        ""
      ],
      "Mari": [
        ""
      ],
      "Chinook Jargon": [
        ""
      ],
      "Choctaw": [
        ""
      ],
      "Chipewyan": [
        ""
      ],
      "Cherokee": [
        ""
      ],
      "Cheyenne": [
        ""
      ],
      "Central Kurdish": [
        ""
      ],
      "Corsican": [
        ""
      ],
      "Coptic": [
        ""
      ],
      "Capiznon": [
        ""
      ],
      "Cree": [
        ""
      ],
      "Crimean Turkish": [
        ""
      ],
      "Kashubian": [
        ""
      ],
      "Kyɛk kasa": [
        ""
      ],
      "Church Slavic": [
        ""
      ],
      "Chuvash": [
        ""
      ],
      "Welsh": [
        ""
      ],
      "Dakota": [
        ""
      ],
      "Dargwa": [
        ""
      ],
      "Taita": [
        ""
      ],
      "Austrian German": [
        ""
      ],
      "Swiss High German": [
        ""
      ],
      "Gyaaman": [
        ""
      ],
      "Delaware": [
        ""
      ],
      "Slave": [
        ""
      ],
      "Dogrib": [
        ""
      ],
      "Dinka": [
        ""
      ],
      "Zarma": [
        ""
      ],
      "Dogri": [
        ""
      ],
      "Lower Sorbian": [
        ""
      ],
      "Central Dusun": [
        ""
      ],
      "Duala": [
        ""
      ],
      "Middle Dutch": [
        ""
      ],
      "Divehi": [
        ""
      ],
      "Jola-Fonyi": [
        ""
      ],
      "Dyula": [
        ""
      ],
      "Dzongkha": [
        ""
      ],
      "Dazaga": [
        ""
      ],
      "Embu": [
        ""
      ],
      "Ewe": [
        ""
      ],
      "Efik": [
        ""
      ],
      "Emilian": [
        ""
      ],
      "Ancient Egyptian": [
        ""
      ],
      "Ekajuk": [
        ""
      ],
      "Greek kasa": [
        ""
      ],
      "Elamite": [
        ""
      ],
      "Australian English": [
        ""
      ],
      "Borɔfo": [
        ""
      ],
      "Canadian English": [
        ""
      ],
      "British English": [
        ""
      ],
      "Middle English": [
        ""
      ],
      "American English": [
        ""
      ],
      "Esperanto": [
        ""
      ],
      "Latin American Spanish": [
        ""
      ],
      "European Spanish": [
        ""
      ],
      "Mexican Spanish": [
        ""
      ],
      "Spain kasa": [
        ""
      ],
      "Central Yupik": [
        ""
      ],
      "Basque": [
        ""
      ],
      "Ewondo": [
        ""
      ],
      "Extremaduran": [
        ""
      ],
      "Fang": [
        ""
      ],
      "Pɛɛhyia kasa": [
        ""
      ],
      "Fanti": [
        ""
      ],
      "Fulah": [
        ""
      ],
      "Tornedalen Finnish": [
        ""
      ],
      "Faroese": [
        ""
      ],
      "Fon": [
        ""
      ],
      "Canadian French": [
        ""
      ],
      "Cajun French": [
        ""
      ],
      "Swiss French": [
        ""
      ],
      "Frɛnkye": [
        ""
      ],
      "Middle French": [
        ""
      ],
      "Old French": [
        ""
      ],
      "Arpitan": [
        ""
      ],
      "Northern Frisian": [
        ""
      ],
      "Eastern Frisian": [
        ""
      ],
      "Friulian": [
        ""
      ],
      "Western Frisian": [
        ""
      ],
      "Ga": [
        ""
      ],
      "Gagauz": [
        ""
      ],
      "Gan Chinese": [
        ""
      ],
      "Gayo": [
        ""
      ],
      "Gbaya": [
        ""
      ],
      "Zoroastrian Dari": [
        ""
      ],
      "Scottish Gaelic": [
        ""
      ],
      "Geez": [
        ""
      ],
      "Gilbertese": [
        ""
      ],
      "Galician": [
        ""
      ],
      "Gilaki": [
        ""
      ],
      "Middle High German": [
        ""
      ],
      "Old High German": [
        ""
      ],
      "Goan Konkani": [
        ""
      ],
      "Gondi": [
        ""
      ],
      "Gorontalo": [
        ""
      ],
      "Gothic": [
        ""
      ],
      "Grebo": [
        ""
      ],
      "Ancient Greek": [
        ""
      ],
      "Swiss German": [
        ""
      ],
      "Wayuu": [
        ""
      ],
      "Gujarati": [
        ""
      ],
      "Frafra": [
        ""
      ],
      "Gusii": [
        ""
      ],
      "Manx": [
        ""
      ],
      "Gwichʼin": [
        ""
      ],
      "Hausa": [
        ""
      ],
      "Haida": [
        ""
      ],
      "Hakka Chinese": [
        ""
      ],
      "Hawaiian": [
        ""
      ],
      "Hebrew": [
        ""
      ],
      "Fiji Hindi": [
        ""
      ],
      "Hindi": [
        ""
      ],
      "Hiligaynon": [
        ""
      ],
      "Hittite": [
        ""
      ],
      "Hmong": [
        ""
      ],
      "Hiri Motu": [
        ""
      ],
      "Upper Sorbian": [
        ""
      ],
      "Xiang Chinese": [
        ""
      ],
      "Hangri kasa": [
        ""
      ],
      "Hupa": [
        ""
      ],
      "Herero": [
        ""
      ],
      "Interlingua": [
        ""
      ],
      "Iban": [
        ""
      ],
      "Ibibio": [
        ""
      ],
      "Indonihyia kasa": [
        ""
      ],
      "Interlingue": [
        ""
      ],
      "Igbo": [
        ""
      ],
      "Sichuan Yi": [
        ""
      ],
      "Inupiaq": [
        ""
      ],
      "Iloko": [
        ""
      ],
      "Ingush": [
        ""
      ],
      "Ido": [
        ""
      ],
      "Icelandic": [
        ""
      ],
      "Italy kasa": [
        ""
      ],
      "Inuktitut": [
        ""
      ],
      "Ingrian": [
        ""
      ],
      "Gyapan kasa": [
        ""
      ],
      "Jamaican Creole English": [
        ""
      ],
      "Lojban": [
        ""
      ],
      "Ngomba": [
        ""
      ],
      "Machame": [
        ""
      ],
      "Judeo-Persian": [
        ""
      ],
      "Judeo-Arabic": [
        ""
      ],
      "Jutish": [
        ""
      ],
      "Gyabanis kasa": [
        ""
      ],
      "Kara-Kalpak": [
        ""
      ],
      "Kabyle": [
        ""
      ],
      "Kachin": [
        ""
      ],
      "Jju": [
        ""
      ],
      "Kamba": [
        ""
      ],
      "Kawi": [
        ""
      ],
      "Kabardian": [
        ""
      ],
      "Kanembu": [
        ""
      ],
      "Tyap": [
        ""
      ],
      "Makonde": [
        ""
      ],
      "Kabuverdianu": [
        ""
      ],
      "Kenyang": [
        ""
      ],
      "Koro": [
        ""
      ],
      "Kongo": [
        ""
      ],
      "Kaingang": [
        ""
      ],
      "Khasi": [
        ""
      ],
      "Khotanese": [
        ""
      ],
      "Koyra Chiini": [
        ""
      ],
      "Khowar": [
        ""
      ],
      "Kikuyu": [
        ""
      ],
      "Kirmanjki": [
        ""
      ],
      "Kuanyama": [
        ""
      ],
      "Kako": [
        ""
      ],
      "Kazakh": [
        ""
      ],
      "Kalaallisut": [
        ""
      ],
      "Kalenjin": [
        ""
      ],
      "Kimbundu": [
        ""
      ],
      "Kambodia kasa": [
        ""
      ],
      "Kannada": [
        ""
      ],
      "Komi-Permyak": [
        ""
      ],
      "Konkani": [
        ""
      ],
      "Korea kasa": [
        ""
      ],
      "Kosraean": [
        ""
      ],
      "Kpelle": [
        ""
      ],
      "Karachay-Balkar": [
        ""
      ],
      "Krio": [
        ""
      ],
      "Kinaray-a": [
        ""
      ],
      "Kanuri": [
        ""
      ],
      "Karelian": [
        ""
      ],
      "Kurukh": [
        ""
      ],
      "Shambala": [
        ""
      ],
      "Bafia": [
        ""
      ],
      "Colognian": [
        ""
      ],
      "Kashmiri": [
        ""
      ],
      "Kurdish": [
        ""
      ],
      "Kumyk": [
        ""
      ],
      "Kutenai": [
        ""
      ],
      "Komi": [
        ""
      ],
      "Cornish": [
        ""
      ],
      "Ladino": [
        "Lataa"
      ],
      "Langi": [
        ""
      ],
      "Lahnda": [
        ""
      ],
      "Latin": [
        ""
      ],
      "Lamba": [
        ""
      ],
      "Luxembourgish": [
        ""
      ],
      "Lezghian": [
        ""
      ],
      "Lingua Franca Nova": [
        ""
      ],
      "Ganda": [
        ""
      ],
      "Ligurian": [
        ""
      ],
      "Limburgish": [
        ""
      ],
      "Livonian": [
        ""
      ],
      "Lakota": [
        ""
      ],
      "Lombard": [
        ""
      ],
      "Lingala": [
        ""
      ],
      "Lao": [
        ""
      ],
      "Mongo": [
        ""
      ],
      "Lozi": [
        ""
      ],
      "Latgalian": [
        ""
      ],
      "Luba-Lulua": [
        ""
      ],
      "Luiseno": [
        ""
      ],
      "Luba-Katanga": [
        ""
      ],
      "Lunda": [
        ""
      ],
      "Luo": [
        ""
      ],
      "Mizo": [
        ""
      ],
      "Luyia": [
        ""
      ],
      "Literary Chinese": [
        ""
      ],
      "Laz": [
        ""
      ],
      "Madurese": [
        ""
      ],
      "Mafa": [
        ""
      ],
      "Magahi": [
        ""
      ],
      "Maithili": [
        ""
      ],
      "Makasar": [
        ""
      ],
      "Mandingo": [
        "Lataa"
      ],
      "Masai": [
        ""
      ],
      "Maba": [
        ""
      ],
      "Moksha": [
        ""
      ],
      "Mandar": [
        ""
      ],
      "Mende": [
        ""
      ],
      "Meru": [
        ""
      ],
      "Morisyen": [
        ""
      ],
      "Middle Irish": [
        ""
      ],
      "Makhuwa-Meetto": [
        ""
      ],
      "Metaʼ": [
        ""
      ],
      "Micmac": [
        ""
      ],
      "Maori": [
        ""
      ],
      "Minangkabau": [
        ""
      ],
      "Malayalam": [
        ""
      ],
      "Manchu": [
        ""
      ],
      "Manipuri": [
        ""
      ],
      "Mohawk": [
        ""
      ],
      "Mossi": [
        ""
      ],
      "Western Mari": [
        ""
      ],
      "Marathi": [
        ""
      ],
      "Malay kasa": [
        ""
      ],
      "Mundang": [
        ""
      ],
      "Multiple Languages": [
        ""
      ],
      "Creek": [
        ""
      ],
      "Mirandese": [
        ""
      ],
      "Marwari": [
        ""
      ],
      "Mentawai": [
        ""
      ],
      "Bɛɛmis kasa": [
        ""
      ],
      "Myene": [
        ""
      ],
      "Erzya": [
        ""
      ],
      "Mazanderani": [
        ""
      ],
      "Min Nan Chinese": [
        ""
      ],
      "Neapolitan": [
        ""
      ],
      "Nama": [
        ""
      ],
      "Norwegian Bokmål": [
        ""
      ],
      "North Ndebele": [
        ""
      ],
      "Low German": [
        ""
      ],
      "Nɛpal kasa": [
        ""
      ],
      "Newari": [
        ""
      ],
      "Ndonga": [
        ""
      ],
      "Nias": [
        ""
      ],
      "Niuean": [
        ""
      ],
      "Ao Naga": [
        ""
      ],
      "Flemish": [
        ""
      ],
      "Dɛɛkye": [
        ""
      ],
      "Kwasio": [
        ""
      ],
      "Ngiemboon": [
        ""
      ],
      "Norwegian Nynorsk": [
        ""
      ],
      "Nogai": [
        ""
      ],
      "Old Norse": [
        ""
      ],
      "Novial": [
        ""
      ],
      "NʼKo": [
        ""
      ],
      "South Ndebele": [
        ""
      ],
      "Northern Sotho": [
        ""
      ],
      "Nuer": [
        ""
      ],
      "Navajo": [
        ""
      ],
      "Classical Newari": [
        ""
      ],
      "Nyamwezi": [
        ""
      ],
      "Nyankole": [
        ""
      ],
      "Nyanja": [
        ""
      ],
      "Nyoro": [
        ""
      ],
      "Nzima": [
        ""
      ],
      "Occitan": [
        ""
      ],
      "Ojibwa": [
        ""
      ],
      "Oromo": [
        ""
      ],
      "Oriya": [
        ""
      ],
      "Osage": [
        ""
      ],
      "Ossetic": [
        ""
      ],
      "Ottoman Turkish": [
        ""
      ],
      "Pangasinan": [
        ""
      ],
      "Pahlavi": [
        ""
      ],
      "Pampanga": [
        ""
      ],
      "Papiamento": [
        ""
      ],
      "Pungyabi kasa": [
        ""
      ],
      "Picard": [
        ""
      ],
      "Pennsylvania German": [
        ""
      ],
      "Plautdietsch": [
        ""
      ],
      "Old Persian": [
        ""
      ],
      "Palatine German": [
        ""
      ],
      "Phoenician": [
        ""
      ],
      "Pali": [
        ""
      ],
      "Pɔland kasa": [
        ""
      ],
      "Piedmontese": [
        ""
      ],
      "Pontic": [
        ""
      ],
      "Pohnpeian": [
        ""
      ],
      "Prussian": [
        ""
      ],
      "Old Provençal": [
        ""
      ],
      "Pashto": [
        ""
      ],
      "Brazilian Portuguese": [
        ""
      ],
      "Pɔɔtugal kasa": [
        ""
      ],
      "European Portuguese": [
        ""
      ],
      "Kʼicheʼ": [
        ""
      ],
      "Chimborazo Highland Quichua": [
        ""
      ],
      "Quechua": [
        ""
      ],
      "Rajasthani": [
        ""
      ],
      "Rapanui": [
        ""
      ],
      "Rarotongan": [
        ""
      ],
      "Romagnol": [
        ""
      ],
      "Riffian": [
        ""
      ],
      "Romansh": [
        ""
      ],
      "Rundi": [
        ""
      ],
      "Rombo": [
        ""
      ],
      "Moldavian": [
        ""
      ],
      "Romany": [
        ""
      ],
      "Root": [
        ""
      ],
      "Romenia kasa": [
        ""
      ],
      "Rotuman": [
        ""
      ],
      "Rusyn": [
        ""
      ],
      "Roviana": [
        ""
      ],
      "Aromanian": [
        ""
      ],
      "Rahyia kasa": [
        ""
      ],
      "Rwa": [
        ""
      ],
      "Rewanda kasa": [
        ""
      ],
      "Sandawe": [
        ""
      ],
      "Sakha": [
        ""
      ],
      "Samaritan Aramaic": [
        ""
      ],
      "Samburu": [
        ""
      ],
      "Sanskrit": [
        ""
      ],
      "Sasak": [
        ""
      ],
      "Santali": [
        ""
      ],
      "Saurashtra": [
        ""
      ],
      "Ngambay": [
        ""
      ],
      "Sangu": [
        ""
      ],
      "Sicilian": [
        ""
      ],
      "Scots": [
        ""
      ],
      "Sardinian": [
        ""
      ],
      "Sassarese Sardinian": [
        ""
      ],
      "Sindhi": [
        ""
      ],
      "Seneca": [
        ""
      ],
      "Sena": [
        ""
      ],
      "Seri": [
        ""
      ],
      "Selkup": [
        ""
      ],
      "Northern Sami": [
        ""
      ],
      "Koyraboro Senni": [
        ""
      ],
      "Old Irish": [
        ""
      ],
      "Sango": [
        ""
      ],
      "Samogitian": [
        ""
      ],
      "Tachelhit": [
        ""
      ],
      "Shan": [
        ""
      ],
      "Serbo-Croatian": [
        ""
      ],
      "Chadian Arabic": [
        ""
      ],
      "Sidamo": [
        ""
      ],
      "Sinhala": [
        ""
      ],
      "Slovak": [
        ""
      ],
      "Lower Silesian": [
        ""
      ],
      "Selayar": [
        ""
      ],
      "Southern Sami": [
        ""
      ],
      "Lule Sami": [
        ""
      ],
      "Inari Sami": [
        ""
      ],
      "Skolt Sami": [
        ""
      ],
      "Soninke": [
        ""
      ],
      "Shona": [
        ""
      ],
      "Sogdien": [
        ""
      ],
      "Somalia kasa": [
        ""
      ],
      "Sranan Tongo": [
        ""
      ],
      "Serer": [
        ""
      ],
      "Swati": [
        ""
      ],
      "Saho": [
        ""
      ],
      "Saterland Frisian": [
        ""
      ],
      "Southern Sotho": [
        ""
      ],
      "Sukuma": [
        ""
      ],
      "Susu": [
        ""
      ],
      "Sundanese": [
        ""
      ],
      "Sumerian": [
        ""
      ],
      "Sweden kasa": [
        ""
      ],
      "Comorian": [
        ""
      ],
      "Congo Swahili": [
        ""
      ],
      "Swahili": [
        ""
      ],
      "Classical Syriac": [
        ""
      ],
      "Syriac": [
        ""
      ],
      "Silesian": [
        ""
      ],
      "Tamil kasa": [
        ""
      ],
      "Tulu": [
        ""
      ],
      "Timne": [
        ""
      ],
      "Teso": [
        ""
      ],
      "Tereno": [
        ""
      ],
      "Telugu": [
        ""
      ],
      "Tetum": [
        ""
      ],
      "Taeland kasa": [
        ""
      ],
      "Tigre": [
        ""
      ],
      "Tigrinya": [
        ""
      ],
      "Tiv": [
        ""
      ],
      "Tokelau": [
        ""
      ],
      "Tsakhur": [
        ""
      ],
      "Turkmen": [
        ""
      ],
      "Klingon": [
        ""
      ],
      "Tlingit": [
        ""
      ],
      "Tagalog": [
        ""
      ],
      "Talysh": [
        ""
      ],
      "Tamashek": [
        ""
      ],
      "Tswana": [
        ""
      ],
      "Nyasa Tonga": [
        ""
      ],
      "Tok Pisin": [
        ""
      ],
      "Tɛɛki kasa": [
        ""
      ],
      "Turoyo": [
        ""
      ],
      "Taroko": [
        ""
      ],
      "Tsakonian": [
        ""
      ],
      "Tsimshian": [
        ""
      ],
      "Tsonga": [
        ""
      ],
      "Tatar": [
        ""
      ],
      "Muslim Tat": [
        ""
      ],
      "Tumbuka": [
        ""
      ],
      "Tasawaq": [
        ""
      ],
      "Twi": [
        ""
      ],
      "Tahitian": [
        ""
      ],
      "Tuvinian": [
        ""
      ],
      "Central Atlas Tamazight": [
        ""
      ],
      "Udmurt": [
        ""
      ],
      "Ugaritic": [
        ""
      ],
      "Uyghur": [
        ""
      ],
      "Ukren kasa": [
        ""
      ],
      "Umbundu": [
        ""
      ],
      "Unknown Language": [
        ""
      ],
      "Urdu kasa": [
        ""
      ],
      "Uzbek": [
        ""
      ],
      "Vai": [
        ""
      ],
      "Venetian": [
        ""
      ],
      "Veps": [
        ""
      ],
      "Venda": [
        ""
      ],
      "Viɛtnam kasa": [
        ""
      ],
      "West Flemish": [
        ""
      ],
      "Main-Franconian": [
        ""
      ],
      "Votic": [
        ""
      ],
      "Volapük": [
        ""
      ],
      "Võro": [
        ""
      ],
      "Vunjo": [
        ""
      ],
      "Walser": [
        ""
      ],
      "Wolaytta": [
        ""
      ],
      "Waray": [
        ""
      ],
      "Washo": [
        ""
      ],
      "Walloon": [
        ""
      ],
      "Warlpiri": [
        ""
      ],
      "Wolof": [
        ""
      ],
      "Wu Chinese": [
        ""
      ],
      "Kalmyk": [
        ""
      ],
      "Xhosa": [
        ""
      ],
      "Mingrelian": [
        ""
      ],
      "Soga": [
        ""
      ],
      "Yao": [
        ""
      ],
      "Yapese": [
        ""
      ],
      "Yangben": [
        ""
      ],
      "Yemba": [
        ""
      ],
      "Yiddish": [
        ""
      ],
      "Yoruba": [
        ""
      ],
      "Nheengatu": [
        ""
      ],
      "Cantonese": [
        ""
      ],
      "Zapotec": [
        ""
      ],
      "Zhuang": [
        ""
      ],
      "Blissymbols": [
        ""
      ],
      "Zeelandic": [
        ""
      ],
      "Zenaga": [
        ""
      ],
      "Standard Moroccan Tamazight": [
        ""
      ],
      "Simplified Chinese": [
        ""
      ],
      "Traditional Chinese": [
        ""
      ],
      "Kyaena kasa": [
        ""
      ],
      "Zuni": [
        ""
      ],
      "Zulu": [
        ""
      ],
      "No linguistic content": [
        ""
      ],
      "Zaza": [
        ""
      ],
      "Could not load user attention request": [
        "Tapahtumatietoja ei voitu ladata"
      ],
      "No notification left": [
        ""
      ],
      "Ignore": [
        ""
      ],
      "Invalid dev experiment URI.": [
        ""
      ],
      "URI: %1$s": [
        "%1$s"
      ],
      "Experiment": [
        ""
      ],
      "Insert lost denomination": [
        ""
      ],
      "Apply": [
        ""
      ],
      "Pending refresh": [
        ""
      ],
      "Could not load the transfer payment status": [
        "Talletuksen tilaa ei voitu ladata"
      ],
      "Could not load the status of the order template": [
        "Tapahtumatietoja ei voitu ladata"
      ],
      "required": [
        ""
      ],
      "Review order": [
        ""
      ],
      "Missing URI": [
        ""
      ],
      "Check the link.": [
        ""
      ],
      "Could not parse the recovery URI": [
        ""
      ],
      "Try another URI": [
        ""
      ],
      "Import backup, show info": [
        ""
      ],
      "Add new exchange": [
        ""
      ],
      "Add exchange for %1$s": [
        ""
      ],
      "Enter the URL of an exchange you trust.": [
        ""
      ],
      "An exchange has been found! Review the information and click next": [
        ""
      ],
      "This exchange is already in your list.": [
        ""
      ],
      "Only exchange accessible through \"http\" and \"https\" are allowed.": [
        ""
      ],
      "This exchange protocol version is not supported: \" %1$s\".": [
        ""
      ],
      "This exchange currency \"%1$s\" doesn&apos;t match the expected currency %2$s.": [
        ""
      ],
      "No exchange found in that URL.": [
        ""
      ],
      "loading": [
        ""
      ],
      "Version": [
        ""
      ],
      "Can't use the URL: \"%1$s\"": [
        ""
      ],
      "Exchange URL": [
        ""
      ],
      "Add new contact": [
        ""
      ],
      "Enter the details of a contact you trust.": [
        ""
      ],
      "Contact info valid.": [
        ""
      ],
      "This contact is already in your list.": [
        ""
      ],
      "Review contact": [
        ""
      ],
      "Contact alias": [
        ""
      ],
      "Alias type": [
        ""
      ],
      "Mailbox service": [
        ""
      ],
      "Mailbox address": [
        "Bitcoin osoite"
      ],
      "Contact source": [
        ""
      ],
      "Could not load the list of contacts": [
        "Talletuksen tilaa ei voitu ladata"
      ],
      "Source": [
        ""
      ],
      "Send Cash": [
        "Lähetä %1$s"
      ],
      "Request Cash": [
        ""
      ],
      "No contacts found.": [
        ""
      ],
      "Could not bank choices for account": [
        ""
      ],
      "Could not load the list of messages": [
        "Talletuksen tilaa ei voitu ladata"
      ],
      "Unexpected error when trying to initialize mailbox: %1$s": [
        ""
      ],
      "Unexpected error when trying to delete message: %1$s": [
        ""
      ],
      "Unexpected error when trying to fetch messages: %1$s": [
        ""
      ],
      "Date": [
        ""
      ],
      "You may copy your mailbox URI from below and register it at %1$s with your alias(es) to make it easier for others to find you.": [
        ""
      ],
      "copy uri": [
        ""
      ],
      "hide uri": [
        ""
      ],
      "show uri": [
        ""
      ],
      "Fetch messages": [
        "Toteutusviesti"
      ],
      "Mailbox not initialized": [
        ""
      ],
      "Initialize mailbox": [
        ""
      ],
      "No messages yet.": [
        ""
      ],
      "Taler URI handler": [
        ""
      ],
      "Could not found a handler for the Taler URI": [
        ""
      ],
      "The uri read in the path parameter is not valid: \"%1$s\"": [
        ""
      ],
      "Digital cash payment": [
        ""
      ],
      "Digital cash refund": [
        ""
      ],
      "Digital cash withdrawal": [
        ""
      ],
      "Digital cash deposit": [
        "Digitaalinen käteistalletus"
      ],
      "Digital cash invoice": [
        "Digitaalinen käteistalletus"
      ],
      "Digital cash transfer": [
        ""
      ],
      "Digital cash recovery": [
        "Digitaalinen käteistalletus"
      ],
      "Development experiment": [
        ""
      ],
      "Return to wallet": [
        ""
      ],
      "Network is offline": [
        ""
      ],
      "Diagnostics timed out. Could not talk to the wallet backend.": [
        ""
      ],
      "Problems detected:": [
        ""
      ],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).": [
        ""
      ],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.": [
        ""
      ],
      "here": [
        ""
      ],
      "Running diagnostics": [
        ""
      ],
      "Edit": [
        ""
      ],
      "List of products": [
        ""
      ],
      "free": [
        ""
      ],
      "Could not get provider information": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "fi",
  "completeness": 9
};

strings['es'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "es"
      },
      "Balance": [
        "Saldo"
      ],
      "Backup": [
        "Copia de seguridad"
      ],
      "Notifications": [
        ""
      ],
      "QR Reader and Taler URI": [
        "Lector QR y Taler URI"
      ],
      "Settings": [
        "Configuración"
      ],
      "Dev tools": [
        "Herramientas de desarrollo"
      ],
      "Contacts": [
        ""
      ],
      "Mailbox": [
        ""
      ],
      "Search transactions": [
        "Comisiones de transacción"
      ],
      "%1$s": [
        "%1$s"
      ],
      "KYC requirement": [
        ""
      ],
      "PENDING OPERATIONS": [
        "OPERACIONES PENDIENTES"
      ],
      "Error was thrown trying to: \"%1$s\"": [
        ""
      ],
      "Could not open the wallet database. Firefox is known to run into this problem under \"permanent private mode\".": [
        ""
      ],
      "Could not open the wallet database.": [
        "No se pudo obtener la información de la URI payto"
      ],
      "Unexpected request error, code: %1$s": [
        ""
      ],
      "more info": [
        ""
      ],
      "wrap text": [
        ""
      ],
      "copy content": [
        ""
      ],
      "less info": [
        ""
      ],
      "Retry operation": [
        "Operaciones de billetera"
      ],
      "Loading": [
        "Cargando"
      ],
      "Could not load backup providers": [
        "No se pudo cargar los proveedores de copias de seguridad"
      ],
      "No backup providers configured": [
        "No hay proveedores de copias de seguridad configurados"
      ],
      "Add provider": [
        "Agregar proveedor"
      ],
      "Sync all backups": [
        "Sincronizar todas las copias de seguridad"
      ],
      "Sync now": [
        "Sincronizar ahora"
      ],
      "Last synced": [
        "Ultima vez sincronizado"
      ],
      "Not synced": [
        "No sincronizado"
      ],
      "Expires in": [
        "Expira en"
      ],
      "show less": [
        "Mostrar QR"
      ],
      "show more": [
        "Mostrar QR"
      ],
      "There was an error loading the provider detail for &quot;%1$s&quot;": [
        "Hubo un error cargando los detalles del proveedor para \"%1$s\""
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [
        "No hay proveedor conocido con la URL \"%1$s\"."
      ],
      "See providers": [
        "Ver proveedores"
      ],
      "Last backup": [
        "Última copia de seguridad"
      ],
      "Back up": [
        "Copia de seguridad"
      ],
      "Provider fee": [
        "Tarifa del proveedor"
      ],
      "per year": [
        "por año"
      ],
      "Extend": [
        "Extender"
      ],
      "terms has changed, extending the service will imply accepting the new terms of service": [
        "los términos han cambiado, extender el servicio implicará aceptar los nuevos términos de servicio"
      ],
      "old": [
        "viejo"
      ],
      "new": [
        "nuevo"
      ],
      "fee": [
        "tarifa"
      ],
      "storage": [
        "almacenamiento"
      ],
      "Remove provider": [
        "Eliminar proveedor"
      ],
      "Pay": [
        "Pago"
      ],
      "Withdraw": [
        "Retirar"
      ],
      "This provider has reported an error": [
        "Este proveedor ha reportado un error"
      ],
      "There is conflict with another backup from &quot;%1$s&quot;": [
        "Hay un conflicto con otra copia de seguridad de %1$s"
      ],
      "Backup is not readable": [
        "La copia de seguridad no es legible"
      ],
      "Unknown backup problem: %1$s": [
        "Problema de copia de seguridad desconocido: %1$s"
      ],
      "service paid": [
        "servicio pagado"
      ],
      "Backup valid until": [
        "Copia de seguridad válida hasta"
      ],
      "Cancel": [
        "Cancelar"
      ],
      "Open pay page": [
        "Abrir página de pago"
      ],
      "Open refund page": [
        "Abrir página de devolución"
      ],
      "Open withdraw page": [
        "Abrir página de retirada"
      ],
      "Your wallet is empty.": [
        ""
      ],
      "Get digital cash": [
        "Retirar dinero digital"
      ],
      "Try the demo bank and withdraw test money.": [
        ""
      ],
      "Could not load the balance": [
        "No se pudo cargar el balance de depósito"
      ],
      "Receive": [
        "Recibo"
      ],
      "Send %1$s": [
        "Envíar %1$s"
      ],
      "This page has a withdrawal action.": [
        "Esta página tiene una acción de retirada."
      ],
      "This page has pay action.": [
        "Esta página tiene una acción de pago."
      ],
      "This page has a refund action.": [
        "Esta página tiene una acción de devolución."
      ],
      "This page has a add exchange action.": [
        "Esta página tiene una acción de devolución."
      ],
      "Open add exchange page": [
        "Agregar exchange"
      ],
      "This page has an add contact action.": [
        "Esta página tiene una acción de devolución."
      ],
      "Open add contact page": [
        "Abrir página de pago"
      ],
      "Taler Action": [
        "Acción Taler"
      ],
      "This page has a malformed taler uri.": [
        "Esta página tiene una URI de Taler malformada."
      ],
      "Dismiss": [
        "Descartar"
      ],
      "this popup is being closed and you are being redirected to %1$s": [
        "Este popup está siendo cerrado y estás siendo redirigido a %1$s"
      ],
      "Could not load purchase proposal details": [
        "No se pudo cargar el detalle de la propuesta"
      ],
      "Order Id": [
        "Id de orden"
      ],
      "Summary": [
        "Resumen"
      ],
      "Amount": [
        "Monto"
      ],
      "Choices": [
        ""
      ],
      "This purchase have %1$s choices": [
        ""
      ],
      "Merchant name": [
        "Comerciante"
      ],
      "Merchant jurisdiction": [
        "Jurisdicción"
      ],
      "Merchant address": [
        "Dirección del comerciante"
      ],
      "Merchant logo": [
        "Logo"
      ],
      "Merchant website": [
        "Siti web"
      ],
      "Merchant email": [
        "Correo electrónico"
      ],
      "Merchant public key": [
        "Clave pública"
      ],
      "Created at": [
        "Creado en"
      ],
      "Refund deadline": [
        "Plazo de reembolso"
      ],
      "Auto refund": [
        "Devolución automática"
      ],
      "Pay deadline": [
        "Plazo de pago"
      ],
      "Fulfillment URL": [
        "URL de cumplimiento"
      ],
      "Fulfillment message": [
        "Mensaje de éxito"
      ],
      "Max deposit fee": [
        "Máxima comisión de depósito"
      ],
      "Exchanges": [
        "Proveedor de servicios de pago"
      ],
      "Could not load the status of the term of service": [
        "No se pudo cargar la lista de transacciones"
      ],
      "Select one option": [
        "Seleccione una opción"
      ],
      "Show terms of service": [
        "Mostrar términos de servicio"
      ],
      "I accept the exchange terms of service": [
        "Yo acepto los términos de servicio del exchange"
      ],
      "Review exchange terms of service": [
        "Revisar los términos de servicio"
      ],
      "The exchange doesn't have a terms of service.": [
        "El exchange no tiene los términos de servicio"
      ],
      "Skip it for now.": [
        ""
      ],
      "Format": [
        ""
      ],
      "Language": [
        "Lenguaje actual"
      ],
      "The exchange replied with a empty terms of service": [
        "El exchange respondió con unos términos de servicio vacíos"
      ],
      "No terms of service. The exchange replied with a empty document": [
        "El exchange respondió con unos términos de servicio vacíos"
      ],
      "No terms of service. The exchange replied with a empty text": [
        "El exchange respondió con unos términos de servicio vacíos"
      ],
      "Download Terms of Service": [
        "Descargar los términos de servicio"
      ],
      "Hide terms of service": [
        "Esconder los términos de servicio"
      ],
      "Amount is not valid": [
        ""
      ],
      "Bank account": [
        "Cuenta bancaria"
      ],
      "Bitcoin address": [
        "Dirección de Bitcoin"
      ],
      "BIC/IBAN": [
        "IBAN"
      ],
      "IBAN": [
        "IBAN"
      ],
      "Bitcoin transfer details": [
        "Detalle de transferencia Bitcoin"
      ],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.": [
        "El exchange necesita una transacción con 3 salidas, una salida es hacia la cuenta del exchange y las otras dos son direcciones segwit falsas para metadata con el monto mínimo."
      ],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts": [
        "En la billetera bitcoincore usar el botón \"Agregar destinatario\" para agregar dos destinatarios y copiar las direcciones y montos"
      ],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC": [
        "Asegurarse de que el monto muestre %1$s BTC, sino tendrá que cambiar la unidad a BTC"
      ],
      "Ethereum transfer details": [
        "Detalle de transferencia bancaria"
      ],
      "You need to wire to the service provider account.": [
        ""
      ],
      "Make sure the amount show %1$s ETH": [
        ""
      ],
      "Bank transfer details": [
        "Detalle de transferencia bancaria"
      ],
      "Account": [
        "Cuenta"
      ],
      "Bank host": [
        "Host del banco"
      ],
      "BIC": [
        ""
      ],
      "Exchange": [
        "Exchange"
      ],
      "Reserve Pub": [
        ""
      ],
      "Step 1:": [
        ""
      ],
      "Copy this code and paste it into the subject/purpose field in your banking app or bank website": [
        ""
      ],
      "Subject": [
        "Asunto"
      ],
      "Step 2:": [
        ""
      ],
      "If you don't already have it in your banking favourites list, then copy and paste this IBAN and the name into the receiver fields in your banking app or website": [
        ""
      ],
      "Receiver name": [
        "Nombre del receptor"
      ],
      "Receiver postal code": [
        "Nombre del receptor"
      ],
      "Receiver town": [
        "Nombre del receptor"
      ],
      "Step 3:": [
        ""
      ],
      "Finish the wire transfer setting the amount in your banking app or website, then this withdrawal will proceed automatically.": [
        ""
      ],
      "Make sure ALL data is correct, including the subject; otherwise, the money will not arrive in this wallet. You can use the copy buttons (%1$s) to prevent typing errors or the \"payto://\" URI below to copy just one value.": [
        ""
      ],
      "Alternative if your bank already supports PayTo URI, you can use this %1$s link instead": [
        ""
      ],
      "Continue with banking app or website": [
        ""
      ],
      "Show QR code": [
        ""
      ],
      "Could not load transaction information": [
        "No se pudo cargar información de la transacción"
      ],
      "Bank account confirmation required for the transaction to complete.": [
        ""
      ],
      "KYC check required for the transaction to complete.": [
        ""
      ],
      "No additional information has been provided.": [
        ""
      ],
      "Follow this link to the%1$s %2$s": [
        ""
      ],
      "This transaction is pending to complete.": [
        "Esta transacción no está completada"
      ],
      "Retry": [
        "Reintentar"
      ],
      "There was an error trying to complete the transaction.": [
        "Hubo un error intentando completar la transacción"
      ],
      "This transaction was aborted.": [
        "Esta transacción no está completada"
      ],
      "This transaction failed.": [
        "Esta transacción no está completada"
      ],
      "Caution!": [
        "Cuidado!"
      ],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.": [
        "Si tú ya has transferido dinero al exchange, perderás la oportunidad de recibir las monedas desde este."
      ],
      "Confirm": [
        "Confirmar"
      ],
      "Doing a cancellation while the transaction still active might result in lost coins. Do you still want to cancel the transaction?": [
        ""
      ],
      "No": [
        ""
      ],
      "Yes": [
        ""
      ],
      "Abort": [
        ""
      ],
      "Resume": [
        ""
      ],
      "Suspend": [
        ""
      ],
      "Delete": [
        ""
      ],
      "Withdrawal": [
        "Extracción"
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Select one of the accounts and use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Details": [
        "Detalles"
      ],
      "Withdrawal incomplete.": [
        "Extracción"
      ],
      "If you have already sent money to the service provider account it will wire it back at %1$s": [
        "Si tú ya has transferido dinero al exchange, perderás la oportunidad de recibir las monedas desde este."
      ],
      "Payment": [
        "Pago"
      ],
      "Refunds": [
        "Devoluciones"
      ],
      "%1$s %2$s on %3$s .": [
        "%1$s %2$s en %3$s"
      ],
      "Refund is in progress.": [
        "El proceso de devolución está en progreso."
      ],
      "Merchant created a refund for this order but was not automatically picked up.": [
        "El comerciante creó una devolución para esta orden pero no fue recogida automáticamente."
      ],
      "Offer": [
        "Oferta"
      ],
      "Accept": [
        "Aceptar"
      ],
      "Confirmation code": [
        ""
      ],
      "Merchant": [
        "Merchant"
      ],
      "Invoice ID": [
        "Id de factura"
      ],
      "Deposit": [
        "Depósito"
      ],
      "Wire transfer deadline.": [
        "Detalle de transferencia Bitcoin"
      ],
      "Wire transfer still pending.": [
        ""
      ],
      "The service provider deadline to make the wire transfer is: %1$s": [
        ""
      ],
      "Wire transfer completed.": [
        ""
      ],
      " ": [
        ""
      ],
      "Transfer details": [
        "Detalle de transferencia bancaria"
      ],
      "Wire transfer in progress.": [
        "El proceso de devolución está en progreso."
      ],
      "Refresh": [
        "Actualizar"
      ],
      "Refund": [
        "Devolución"
      ],
      "Purchase summary": [
        "Resumen de compra"
      ],
      "Credit": [
        "Crédito"
      ],
      "Invoice": [
        "Factura"
      ],
      "URI": [
        "URI"
      ],
      "Debit": [
        "Débito"
      ],
      "Transfer": [
        "Transferencia"
      ],
      "Lost": [
        ""
      ],
      "Reason": [
        ""
      ],
      "Denomination expired.": [
        "Operaciones"
      ],
      "Denomination vanished.": [
        "Operaciones"
      ],
      "Denomination is unoffered.": [
        "Operaciones"
      ],
      "Fees": [
        ""
      ],
      "Total": [
        "Total"
      ],
      "Sent": [
        ""
      ],
      "Converted": [
        ""
      ],
      "Bank fee": [
        "Máxima comisión"
      ],
      "Price": [
        "Precio"
      ],
      "Subtotal": [
        "Total"
      ],
      "Refunded": [
        "Reembolsado"
      ],
      "Transfer identification": [
        "Operaciones de transferencia"
      ],
      "copy": [
        "Copiar"
      ],
      "hide qr": [
        "Esconder QR"
      ],
      "forward to contact": [
        ""
      ],
      "show qr": [
        "Mostrar QR"
      ],
      "Wire transfer need a confirmation. Go to the %1$s and check wire transfer operation to exchange account is complete.": [
        ""
      ],
      "bank site": [
        "Host del banco"
      ],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins.": [
        "El banco confirmó la transferencia. Esperando que el exchange envíe las monedas"
      ],
      "Exchange is ready to send the coins, withdrawal in progress.": [
        "El exchange está listo para la retirada"
      ],
      "Send &nbsp; %1$s": [
        "Envíar %1$s"
      ],
      "Pay &nbsp; %1$s": [
        "Pagar %1$s"
      ],
      "Merchant message": [
        "Mensaje del comerciante"
      ],
      "Balance is not enough.": [
        "Tu balance no es suficiente."
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which exchange and auditors the merchant trust.": [
        ""
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which wire methods the merchant accepts.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to pay for this contract which is restricted.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to spend right know. There are some coins that need to be refreshed.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s available.": [
        ""
      ],
      "Provider is missing the global fee configuration, this likely means it is misconfigured.": [
        ""
      ],
      "Balance looks like it should be enough, but doesn't cover all fees requested by the merchant and payment processor. Please ensure there is at least %1$s %2$s more balance in your wallet or ask your merchant to cover more of the fees.": [
        ""
      ],
      "Pay with a mobile phone": [
        "Pagar con un teléfono móvil"
      ],
      "Hide QR": [
        "Esconder QR"
      ],
      "Scan the QR code or &nbsp; %1$s": [
        "Escanear el código QR o %1$s"
      ],
      "click here": [
        "hacer clic para mostrar"
      ],
      "Could not load the payment and balance status": [
        "No se pudo cargar el estado del pago"
      ],
      "Purchase": [
        "Compra"
      ],
      "This purchase is age restricted.": [
        ""
      ],
      "Expires at": [
        "Expira en"
      ],
      "Already paid, you are going to be redirected to %1$s": [
        "Ya pagado, estás siendo dirigido a %1$s"
      ],
      "Already paid": [
        "Ya pagado"
      ],
      "Already claimed": [
        "Ya reclamado"
      ],
      "Could not load the refund status": [
        "No se pudo cargar el estado de la devolución"
      ],
      "You&apos;ve ignored the refund.": [
        ""
      ],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [
        "El comerciante \"%1$s\" te está ofreciendo una devolución."
      ],
      "Order amount": [
        "Monto de la orden"
      ],
      "Could not load the list of exchanges": [
        "No se pudo cargar la lista de exchange"
      ],
      "Could not load info from URI": [
        "No se pudo obtener la información desde la URI"
      ],
      "Could not load the withdrawal details": [
        "No se pudo cargar el detalle de la propuesta"
      ],
      "under %1$s": [
        "Envíar %1$s"
      ],
      "should be greater than zero": [
        ""
      ],
      "can't be greater than %1$s": [
        ""
      ],
      "Could not load exchange details info": [
        "No se pudo cargar la comisión del exchange"
      ],
      "Close": [
        "Cerrar"
      ],
      "There is no exchange available for currency %1$s": [
        "No hay suficiente balance para hacer un depósito para la divisa %1$s"
      ],
      "You can add more exchanges from the settings.": [
        ""
      ],
      "Exchange %1$s is not available": [
        ""
      ],
      "Exchange status can view accessed from the settings.": [
        ""
      ],
      "Service fee description": [
        "Descripción de comisión de servicio"
      ],
      "Select %1$s exchange": [
        "Seleccionar exchange %1$s"
      ],
      "Reset": [
        "Reiniciar"
      ],
      "Use this exchange": [
        "Usar este exchange"
      ],
      "Auditors": [
        "Auditores"
      ],
      "Doesn&apos;t have auditors": [
        "No tiene auditores"
      ],
      "currency": [
        "divisa"
      ],
      "Coin operations": [
        "Operaciones de moneda"
      ],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.": [
        "Toda operación en esta sección puede ser diferente por valor de denominación y es válida por un período. El exchange cobrará el monto indicado cada vez que una es usada en dicha operación."
      ],
      "Deposits": [
        "Depósitos"
      ],
      "Denomination": [
        "Operaciones"
      ],
      "Current": [
        "Divisa"
      ],
      "Selected": [
        ""
      ],
      "Until": [
        "Hasta"
      ],
      "Withdrawals": [
        "Retiradas"
      ],
      "Transfer operations": [
        "Operaciones de transferencia"
      ],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.": [
        "Toda operación en esta sección puede ser diferente por tipo de transacción y es válida por un período. El exchange cobrará el monto indicado cada vez que se haga una transferencia."
      ],
      "Operation": [
        "Operación"
      ],
      "Fee": [
        "Comisión"
      ],
      "Wallet operations": [
        "Operaciones de billetera"
      ],
      "Feature": [
        "Característica"
      ],
      "Currency": [
        "Divisa"
      ],
      "This operation has already been completed.": [
        "Esta transacción no está completada"
      ],
      "This operation has already been aborted": [
        ""
      ],
      "This operation has started and should be completed in the bank.": [
        ""
      ],
      "You can confirm the operation in": [
        "No se pudo completar la operación de recolección"
      ],
      "this page": [
        "Usar este exchange"
      ],
      "This operation has already been completed by another wallet.": [
        ""
      ],
      "This operation has already been used by another wallet.": [
        ""
      ],
      "It can be confirmed in": [
        "Sin exchange configurado"
      ],
      "Age restriction": [
        "Restricción de edad"
      ],
      "Withdraw &nbsp; %1$s": [
        "Retirar %1$s"
      ],
      "Withdraw to a mobile phone": [
        "Retirar con un teléfono móvil"
      ],
      "Short description of the invoice": [
        "Especificar el destino del dinero"
      ],
      "Create": [
        "Crear"
      ],
      "Short description of the transfer": [
        "Especificar el destino del dinero"
      ],
      "Could not load the invoice payment status": [
        "No se pudo cargar el estado del pago"
      ],
      "Receive &nbsp; %1$s": [
        "Recibir %1$s"
      ],
      "Could not load known bank accounts": [
        "No se pudo cargar el estado del pago"
      ],
      "Could not load supported wire methods": [
        "No se pudo cargar los proveedores de copias de seguridad"
      ],
      "No wire methods supported for this currency": [
        ""
      ],
      "Try another wire method.": [
        "Hacia otra billetera"
      ],
      "Known accounts for %1$s": [
        "Agregar cuenta de banco para %1$s"
      ],
      "To add a new account first select the account type.": [
        ""
      ],
      "Unable add this account": [
        "No fue posible agregar esta cuenta"
      ],
      "Add": [
        "Agregar"
      ],
      "IBAN accounts": [
        "Agregar cuenta"
      ],
      "Alias": [
        ""
      ],
      "Int. Account Number": [
        "Número IBAN de cuenta bancaria"
      ],
      "Account name": [
        "Cuenta"
      ],
      "KYC": [
        ""
      ],
      "Taler accounts": [
        "Seleccionar cuenta"
      ],
      "Host": [
        ""
      ],
      "Bitcoin accounts": [
        "Cuenta bancaria"
      ],
      "Address": [
        "Detalle de dirección"
      ],
      "Can't be empty": [
        ""
      ],
      "Bitcoin Account": [
        "Cuenta bancaria"
      ],
      "Taler Bank": [
        "Acción Taler"
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "Required": [
        ""
      ],
      "International Bank Account Number": [
        "Número IBAN de cuenta bancaria"
      ],
      "Could not load balance information": [
        "No se pudo cargar la información de recuperación de copia de seguridad"
      ],
      "Could not load fee for amount %1$s": [
        "No se pudo cargar el estado de la devolución"
      ],
      "A currency or an amount should be indicated": [
        "Se debería especificar una divisa o un monto"
      ],
      "There is no enough balance to make a deposit for currency %1$s": [
        "No hay suficiente balance para hacer un depósito para la divisa %1$s"
      ],
      "Send %1$s to your account": [
        "Enviar %1$s a tu cuenta"
      ],
      "There is no account to make a deposit for currency %1$s": [
        "No hay una cuenta para hacer un depósito para la divisa %1$s"
      ],
      "Add account": [
        "Agregar cuenta"
      ],
      "Select account": [
        "Seleccionar cuenta"
      ],
      "Manage accounts": [
        "Cuenta bancaria"
      ],
      "Brut amount": [
        "Monto de la orden"
      ],
      "Deposit fee": [
        "Comisión de depósito"
      ],
      "Net amount": [
        "Cantidad a depositar"
      ],
      "Deposit&nbsp;%1$s %2$s": [
        "Depositar %1$s %2$s"
      ],
      "Need approval in the Bank": [
        ""
      ],
      "Waiting for wire transfer to complete": [
        "Esperando a que las monedas lleguen"
      ],
      "Exchange is waiting the wire transfer": [
        ""
      ],
      "Payment in progress": [
        "El proceso de devolución está en progreso."
      ],
      "Executing refund...": [
        "Devolución automática"
      ],
      "Refreshing coins...": [
        ""
      ],
      "Deposit in progress": [
        "El proceso de devolución está en progreso."
      ],
      "Waiting to be paid": [
        ""
      ],
      "Receiving the transfer": [
        "Transferencia de dinero digital"
      ],
      "Waiting to be received": [
        "A recibir"
      ],
      "Denomination expired": [
        "Operaciones"
      ],
      "Denomination vanished": [
        "Operaciones"
      ],
      "Denomination unoffered": [
        "Operaciones"
      ],
      "ABORTED": [
        ""
      ],
      "FAILED": [
        ""
      ],
      "Could not load the list of transactions": [
        "No se pudo cargar la lista de transacciones"
      ],
      "unknown scope": [
        ""
      ],
      "Send": [
        "Envíar"
      ],
      "Your transaction history is empty for this currency.": [
        "No hay historial para esta divisa."
      ],
      "Review terms of service": [
        "Revisar los términos de servicio"
      ],
      "Provider URL": [
        "URL del proveedor"
      ],
      "Please review and accept this provider&apos;s terms of service": [
        "Por favor revisa y acepta los términos de servicio del proveedor"
      ],
      "Pricing": [
        "Precios"
      ],
      "free of charge": [
        "Gratis"
      ],
      "%1$s per year of service": [
        "%1$s por año de servicio"
      ],
      "Storage": [
        "Alamcenamiento"
      ],
      "%1$s megabytes of storage per year of service": [
        "%1$s megabytes de almacenamiento por año de servicio"
      ],
      "Accept terms of service": [
        "Aceptar los términos de servicio"
      ],
      "Add backup provider": [
        "Agregar proveedor de copias de seguridad"
      ],
      "Backup providers may charge for their service": [
        "Los proveedores de copias de seguridad pueden cobrarte por su servicio"
      ],
      "URL": [
        "URL"
      ],
      "Name": [
        "Nombre"
      ],
      "Next": [
        "Siguiente"
      ],
      "Navigator": [
        "Navegador"
      ],
      "Automatically open wallet": [
        "Abrir automáticamente la billetera basada en el contenido de la página"
      ],
      "Open the wallet when a payment action is found.": [
        ""
      ],
      "Version Info": [
        "Versión"
      ],
      "Web Extension": [
        "Web Extension"
      ],
      "The version of wallet core is not supported. (supported version: %1$s, wallet version: %2$s)": [
        ""
      ],
      "Exchange compatibility": [
        "Compatibilidad con Exchange"
      ],
      "Merchant compatibility": [
        "Compatibilidad con Merchant"
      ],
      "Bank compatibility": [
        "Compatibilidad con Bank"
      ],
      "Wallet Core compatibility": [
        "Compatibilidad con Bank"
      ],
      "Enable developer mode": [
        "Modo desarrollador"
      ],
      "Show more information and options in the UI": [
        ""
      ],
      "Display": [
        "Pantalla"
      ],
      "Current Language": [
        "Lenguaje actual"
      ],
      "Show backup feature": [
        ""
      ],
      "Backup integration still in beta.": [
        ""
      ],
      "Show P2P alias feature": [
        ""
      ],
      "P2P aliases integration still in beta.": [
        ""
      ],
      "Show suspend/resume transaction": [
        ""
      ],
      "Prevent transaction from doing network request.": [
        ""
      ],
      "Show refresh transaction type in the transaction list": [
        ""
      ],
      "Refresh transaction will be hidden by default if the refresh operation doesn't have fee.": [
        ""
      ],
      "Show more account types on deposit": [
        ""
      ],
      "Extends the UI to more payment target types.": [
        ""
      ],
      "Show JSON on error": [
        ""
      ],
      "Print more information about the error. Useful for debugging.": [
        "Más información y opciones útiles para depuración"
      ],
      "Allow HTTP connections": [
        ""
      ],
      "Using HTTP connection may be faster but unsafe (wallet restart required)": [
        ""
      ],
      "Lang selector": [
        ""
      ],
      "Allows to manually change the language of the UI. Otherwise it will be automatically selected by your browser configuration.": [
        ""
      ],
      "Edit exchange management": [
        "Agregar exchange de todas maneras"
      ],
      "Allows to see the list of exchange, remove, add and switch before withdrawal.": [
        ""
      ],
      "Select terms of service format": [
        "Aceptar los términos de servicio"
      ],
      "Allows to render the terms of service on different format selected by the user.": [
        ""
      ],
      "Show wallet activity": [
        ""
      ],
      "Show the wallet notification and observability event in the UI.": [
        ""
      ],
      "Enable v1 contract": [
        ""
      ],
      "v1 is under development and wallet-core converts v1 contract to v0. Enable this options for testing": [
        ""
      ],
      "GNU Taler Wallet installed!": [
        ""
      ],
      "You can open the wallet using the combination %1$s .": [
        "Puedes abrir GNU Taler Wallet usando la combinación %1$s."
      ],
      "Also pinning the GNU Taler Wallet to your browser allows you to quick access without keyboard:": [
        "También fijando GNU Taler Wallet a to navegador Chrome permite un acceso rápido sin el teclado:"
      ],
      "Click the puzzle icon": [
        "Haz click en el ícono de rompecabezas"
      ],
      "Search for GNU Taler Wallet": [
        "Busca \"GNU Taler Wallet\""
      ],
      "Click the pin icon": [
        "Haz click en el ícono de fijar"
      ],
      "Next Steps": [
        "Próximos pasos"
      ],
      "Try the demo": [
        "Probar la demostración"
      ],
      "Click here to open the wallet activity tab.": [
        ""
      ],
      "Events": [
        ""
      ],
      "Active tasks": [
        ""
      ],
      "All events": [
        ""
      ],
      "Balance change": [
        "Saldo"
      ],
      "Backup failed": [
        "Copia de seguridad válida hasta"
      ],
      "Transaction updated": [
        "Comisiones de transacción"
      ],
      "Exchange updated": [
        "Comisión del exchange"
      ],
      "Idle": [
        ""
      ],
      "task.%1$s": [
        "%1$s"
      ],
      "wallet.%1$s(%2$s)": [
        ""
      ],
      "Bank account info changed": [
        "Número IBAN de cuenta bancaria"
      ],
      "Contact added": [
        ""
      ],
      "Contact deleted": [
        ""
      ],
      "Mailbox message added": [
        ""
      ],
      "Mailbox message deleted": [
        ""
      ],
      "Type": [
        ""
      ],
      "Id": [
        ""
      ],
      "Since": [
        ""
      ],
      "Next try": [
        "Próximos pasos"
      ],
      "Error": [
        "Error: %1$s"
      ],
      "Transaction": [
        "Comisiones de transacción"
      ],
      "No exchange yet": [
        "No hay exchanges todavía"
      ],
      "Status": [
        "estado"
      ],
      "Terms of Service": [
        "Términos de servicio"
      ],
      "Last Update": [
        ""
      ],
      "Actions": [
        "Acción Taler"
      ],
      "ok": [
        "ok"
      ],
      "pending": [
        ""
      ],
      "proposed": [
        ""
      ],
      "unknown (exchange status should be updated)": [
        "desconocido (el estado del exchange debería actualizarse)"
      ],
      "Reload": [
        "cargando"
      ],
      "Purge": [
        ""
      ],
      "Make regional": [
        ""
      ],
      "Make global": [
        ""
      ],
      "Forget ToS": [
        "Olvidar"
      ],
      "Debug tools": [
        "Herramientas de desarrollo"
      ],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?": [
        "Quieres DESTRUIR IRREVOCABLEMENTE todo dentro de tu billetera y PERDER TODAS TUS MONEDAS?"
      ],
      "reset": [
        "Reiniciar"
      ],
      "TESTING: This may delete all your coin, proceed with caution": [
        "TESTING: Esto puede borrar todas tus monedas, proceder con precaución"
      ],
      "run gc": [
        "Ejecutar GC"
      ],
      "import database": [
        "importar base de datos"
      ],
      "export database": [
        "exportar base de datos"
      ],
      "Clear exchange key cache": [
        "No hay exchanges todavía"
      ],
      "Database exported at %1$s %2$s to download": [
        "Base de datos exportada a %1$s %2$s para descargar"
      ],
      "Inject Taler support in all pages": [
        ""
      ],
      "Enabling this option will make `window.taler` be available in all sites": [
        ""
      ],
      "Exchange Entries": [
        "Proveedor de servicios de pago"
      ],
      "Add an exchange": [
        "Agregar un exchange"
      ],
      "Log levels": [
        ""
      ],
      "Coins": [
        "Monedas"
      ],
      "usable coins": [
        "monedas usables"
      ],
      "id": [
        "id"
      ],
      "denom": [
        "denominación"
      ],
      "status": [
        "estado"
      ],
      "from refresh?": [
        "desde refresco?"
      ],
      "age key count": [
        "cantidad de age key"
      ],
      "spent coins": [
        "monedas gastadas"
      ],
      "click to show": [
        "hacer clic para mostrar"
      ],
      "URI is not valid. Taler URI should start with \"taler://\"": [
        "El URI no es válido. Taler URI debería comenzar con `taler://`"
      ],
      "After the URI type it should follow a '/' with more information.": [
        ""
      ],
      "This URI type is not supported": [
        ""
      ],
      "This URI requires the bank host and operation id separated by /": [
        ""
      ],
      "This URI requires the merchant host, order id and session id separated by /": [
        ""
      ],
      "This URI requires the merchant host and order id separated by /": [
        ""
      ],
      "This URI requires the exchange host and key id separated by /": [
        ""
      ],
      "This URI requires the merchant host and template id separated by /": [
        ""
      ],
      "This URI requires the host key and provider list separated by /": [
        ""
      ],
      "This URI requires the experiment id": [
        ""
      ],
      "This URI requires the exchange host": [
        ""
      ],
      "This URI requires the alias type, the alias and the mailbox URI separated by /": [
        ""
      ],
      "This URI requires string after the first /": [
        ""
      ],
      "The bank host is invalid": [
        ""
      ],
      "The merchant host is invalid": [
        ""
      ],
      "The URI should end with /": [
        ""
      ],
      "The exchange host is invalid": [
        ""
      ],
      "The contact is invalid": [
        ""
      ],
      "The amount is invalid": [
        ""
      ],
      "A parameter is invalid": [
        ""
      ],
      "checking...": [
        ""
      ],
      "Unexpected error happen reading the camera: %1$s": [
        ""
      ],
      "Could not found a QR code in the file": [
        ""
      ],
      "Unexpected error happen reading the file: %1$s": [
        ""
      ],
      "Scan a QR code or enter taler:// URI below": [
        "Escanear un código QR o ingresar taler:// URI debajo"
      ],
      "Pay invoice": [
        "Factura digital"
      ],
      "Withdrawal from bank": [
        "Extracción"
      ],
      "Claim refund": [
        "Ya devuelto"
      ],
      "Accept payment": [
        "Aceptar"
      ],
      "Complete order": [
        ""
      ],
      "Restore wallet": [
        ""
      ],
      "Enable experiment": [
        ""
      ],
      "Withdraw from exchange": [
        "Retirar con un teléfono móvil"
      ],
      "Add exchange": [
        "Agregar exchange"
      ],
      "Notify transaction": [
        ""
      ],
      "Add contact": [
        "Agregar cuenta"
      ],
      "The exchange URL is invalid.": [
        ""
      ],
      "The exchange URL should end with '/'": [
        ""
      ],
      "Couldn't found an exchange in the URL specified.": [
        "No se pudo encontrar ningún exchange para la divisa %1$s"
      ],
      "HTTP request failed to %1$s: %2$s": [
        ""
      ],
      "HTTP request failed to %1$s": [
        ""
      ],
      "The merchant URL is invalid.": [
        ""
      ],
      "The merchant URL should end with '/'": [
        ""
      ],
      "Couldn't found an merchant in the URL specified.": [
        ""
      ],
      "The bank URL is invalid.": [
        ""
      ],
      "The bank URL should end with '/'": [
        ""
      ],
      "Couldn't found an bank in the URL specified.": [
        ""
      ],
      "Could not load exchanges": [
        "No se pudo cargar la comisión del exchange"
      ],
      "Choose a currency to proceed or add another exchange": [
        "Elija una divisa para proceder o agregue otro exchange"
      ],
      "Known currencies": [
        "Divisas conocidas"
      ],
      "Specify the origin": [
        "Especificar el origen del dinero"
      ],
      "Use previous origins:": [
        "Usar un origen previo:"
      ],
      "From another bank account": [
        "Desde mi cuenta de banco"
      ],
      "From another wallet": [
        "Desde otra billetera"
      ],
      "From a %1$s URI or QR code": [
        ""
      ],
      "Enter URI here": [
        "Ingresar URI"
      ],
      "Specify the destination": [
        "Especificar el destino del dinero"
      ],
      "Use previous destinations:": [
        "Usar destinos previos:"
      ],
      "To another bank account": [
        "Hacia mi cuenta de banco"
      ],
      "To another wallet": [
        "Hacia otra billetera"
      ],
      "To a %1$s URI or QR code": [
        ""
      ],
      "Afghanistan": [
        ""
      ],
      "Albania": [
        ""
      ],
      "Algeria": [
        ""
      ],
      "United States": [
        "Estado del reembolso"
      ],
      "Andorra": [
        ""
      ],
      "Angola": [
        ""
      ],
      "Antigua and Barbuda": [
        ""
      ],
      "Argentina": [
        ""
      ],
      "Armenia": [
        ""
      ],
      "Australia": [
        ""
      ],
      "Austria": [
        ""
      ],
      "Azerbaijan": [
        ""
      ],
      "Bahamas": [
        ""
      ],
      "Bahrain": [
        ""
      ],
      "Bangladesh": [
        ""
      ],
      "Barbados": [
        ""
      ],
      "Botswana": [
        ""
      ],
      "Belarus": [
        ""
      ],
      "Belgium": [
        ""
      ],
      "Belize": [
        ""
      ],
      "Benin": [
        ""
      ],
      "Bhutan": [
        ""
      ],
      "Bolivia": [
        ""
      ],
      "Bosnia and Herzegovina": [
        ""
      ],
      "Brazil": [
        ""
      ],
      "United Kingdom": [
        ""
      ],
      "Brunei": [
        ""
      ],
      "Bulgaria": [
        ""
      ],
      "Burkina Faso": [
        ""
      ],
      "Myanmar": [
        ""
      ],
      "Burundi": [
        ""
      ],
      "Cambodia": [
        ""
      ],
      "Cameroon": [
        ""
      ],
      "Canada": [
        ""
      ],
      "Cape Verde": [
        ""
      ],
      "Central African Republic": [
        ""
      ],
      "Chad": [
        ""
      ],
      "Chile": [
        ""
      ],
      "China": [
        ""
      ],
      "Colombia": [
        ""
      ],
      "Comoros": [
        ""
      ],
      "Democratic Republic of the Congo": [
        ""
      ],
      "Republic of the Congo": [
        ""
      ],
      "Costa Rica": [
        ""
      ],
      "Croatia": [
        ""
      ],
      "Cuba": [
        ""
      ],
      "Cyprus": [
        ""
      ],
      "Czech Republic": [
        "Clave pública"
      ],
      "Denmark": [
        ""
      ],
      "Djibouti": [
        ""
      ],
      "Dominican Republic": [
        ""
      ],
      "Netherlands": [
        ""
      ],
      "Timor-Leste": [
        ""
      ],
      "Ecuador": [
        ""
      ],
      "Egypt": [
        ""
      ],
      "United Arab Emirates": [
        ""
      ],
      "Equatorial Guinea": [
        ""
      ],
      "Eritrea": [
        ""
      ],
      "Estonia": [
        ""
      ],
      "Ethiopia": [
        ""
      ],
      "Fiji": [
        ""
      ],
      "Philippines": [
        ""
      ],
      "Finland": [
        ""
      ],
      "France": [
        "Cancelar"
      ],
      "Gabon": [
        ""
      ],
      "Gambia": [
        ""
      ],
      "Georgia": [
        ""
      ],
      "Germany": [
        ""
      ],
      "Ghana": [
        ""
      ],
      "Greece": [
        ""
      ],
      "Grenada": [
        ""
      ],
      "Guatemala": [
        ""
      ],
      "Guinea-Bissau": [
        ""
      ],
      "Guinea": [
        ""
      ],
      "Guyana": [
        ""
      ],
      "Haiti": [
        ""
      ],
      "Honduras": [
        ""
      ],
      "Hungary": [
        ""
      ],
      "Kiribati": [
        ""
      ],
      "Iceland": [
        ""
      ],
      "India": [
        ""
      ],
      "Indonesia": [
        ""
      ],
      "Iran": [
        ""
      ],
      "Iraq": [
        ""
      ],
      "Ireland": [
        ""
      ],
      "Israel": [
        ""
      ],
      "Italy": [
        ""
      ],
      "Ivory Coast": [
        ""
      ],
      "Jamaica": [
        ""
      ],
      "Japan": [
        ""
      ],
      "Jordan": [
        ""
      ],
      "Kazakhstan": [
        ""
      ],
      "Kenya": [
        ""
      ],
      "Saint Kitts and Nevis": [
        ""
      ],
      "Kuwait": [
        ""
      ],
      "Kyrgyzstan": [
        ""
      ],
      "Laos": [
        ""
      ],
      "Latvia": [
        ""
      ],
      "Lebanon": [
        ""
      ],
      "Liberia": [
        ""
      ],
      "Libya": [
        ""
      ],
      "Liechtenstein": [
        ""
      ],
      "Lithuania": [
        ""
      ],
      "Luxembourg": [
        ""
      ],
      "North Macedonia": [
        ""
      ],
      "Madagascar": [
        ""
      ],
      "Malawi": [
        ""
      ],
      "Malaysia": [
        ""
      ],
      "Maldives": [
        ""
      ],
      "Mali": [
        ""
      ],
      "Malta": [
        ""
      ],
      "Marshall Islands": [
        ""
      ],
      "Mauritania": [
        ""
      ],
      "Mauritius": [
        ""
      ],
      "Mexico": [
        ""
      ],
      "Micronesia": [
        ""
      ],
      "Moldova": [
        ""
      ],
      "Monaco": [
        ""
      ],
      "Mongolia": [
        ""
      ],
      "Morocco": [
        ""
      ],
      "Lesotho": [
        ""
      ],
      "Mozambique": [
        ""
      ],
      "Namibia": [
        ""
      ],
      "Nauru": [
        ""
      ],
      "Nepal": [
        ""
      ],
      "New Zealand": [
        ""
      ],
      "Vanuatu": [
        ""
      ],
      "Nicaragua": [
        ""
      ],
      "Nigeria": [
        ""
      ],
      "Niger": [
        ""
      ],
      "North Korea": [
        ""
      ],
      "Norway": [
        ""
      ],
      "Oman": [
        ""
      ],
      "Pakistan": [
        ""
      ],
      "Palau": [
        ""
      ],
      "Panama": [
        ""
      ],
      "Papua New Guinea": [
        ""
      ],
      "Paraguay": [
        ""
      ],
      "Peru": [
        ""
      ],
      "Poland": [
        ""
      ],
      "Portugal": [
        ""
      ],
      "Qatar": [
        ""
      ],
      "Romania": [
        ""
      ],
      "Russia": [
        ""
      ],
      "Rwanda": [
        ""
      ],
      "Saint Lucia": [
        ""
      ],
      "El Salvador": [
        ""
      ],
      "Samoa": [
        ""
      ],
      "San Marino": [
        ""
      ],
      "São Tomé and Príncipe": [
        ""
      ],
      "Saudi Arabia": [
        ""
      ],
      "Senegal": [
        ""
      ],
      "Serbia": [
        ""
      ],
      "Seychelles": [
        ""
      ],
      "Sierra Leone": [
        ""
      ],
      "Singapore": [
        ""
      ],
      "Slovakia": [
        ""
      ],
      "Slovenia": [
        ""
      ],
      "Solomon Islands": [
        ""
      ],
      "Somalia": [
        ""
      ],
      "South Africa": [
        ""
      ],
      "South Korea": [
        ""
      ],
      "Spain": [
        ""
      ],
      "Sri Lanka": [
        ""
      ],
      "Sudan": [
        ""
      ],
      "Suriname": [
        ""
      ],
      "Eswatini": [
        ""
      ],
      "Sweden": [
        ""
      ],
      "Switzerland": [
        ""
      ],
      "Syria": [
        ""
      ],
      "Taiwan": [
        ""
      ],
      "Tajikistan": [
        ""
      ],
      "Tanzania": [
        ""
      ],
      "Thailand": [
        ""
      ],
      "Togo": [
        ""
      ],
      "Tonga": [
        ""
      ],
      "Trinidad and Tobago": [
        ""
      ],
      "Tunisia": [
        ""
      ],
      "Turkey": [
        ""
      ],
      "Tuvalu": [
        "valor"
      ],
      "Uganda": [
        ""
      ],
      "Ukraine": [
        ""
      ],
      "Uruguay": [
        ""
      ],
      "Uzbekistan": [
        ""
      ],
      "Venezuela": [
        ""
      ],
      "Vietnam": [
        ""
      ],
      "Yemen": [
        ""
      ],
      "Zambia": [
        ""
      ],
      "Zimbabwe": [
        ""
      ],
      "Afghan": [
        ""
      ],
      "Albanian": [
        ""
      ],
      "Algerian": [
        ""
      ],
      "American": [
        ""
      ],
      "Andorran": [
        ""
      ],
      "Angolan": [
        ""
      ],
      "Antiguan": [
        ""
      ],
      "Argentinean": [
        ""
      ],
      "Armenian": [
        ""
      ],
      "Australian": [
        ""
      ],
      "Austrian": [
        ""
      ],
      "Azerbaijani": [
        ""
      ],
      "Bahamian": [
        ""
      ],
      "Bahraini": [
        ""
      ],
      "Bangladeshi": [
        ""
      ],
      "Barbadian": [
        ""
      ],
      "Batswana": [
        ""
      ],
      "Belarusian": [
        ""
      ],
      "Belgian": [
        ""
      ],
      "Belizean": [
        ""
      ],
      "Beninese": [
        ""
      ],
      "Bhutanese": [
        ""
      ],
      "Bolivian": [
        ""
      ],
      "Bosnian": [
        ""
      ],
      "Brazilian": [
        ""
      ],
      "British": [
        ""
      ],
      "Bruneian": [
        ""
      ],
      "Bulgarian": [
        ""
      ],
      "Burkinabe": [
        ""
      ],
      "Burmese": [
        ""
      ],
      "Burundian": [
        ""
      ],
      "Cambodian": [
        ""
      ],
      "Cameroonian": [
        ""
      ],
      "Canadian": [
        ""
      ],
      "Cape Verdean": [
        ""
      ],
      "Central African": [
        ""
      ],
      "Chadian": [
        ""
      ],
      "Chilean": [
        ""
      ],
      "Chinese": [
        ""
      ],
      "Colombian": [
        ""
      ],
      "Comoran": [
        ""
      ],
      "Congolese": [
        ""
      ],
      "Costa Rican": [
        ""
      ],
      "Croatian": [
        ""
      ],
      "Cuban": [
        ""
      ],
      "Cypriot": [
        ""
      ],
      "Czech": [
        ""
      ],
      "Danish": [
        ""
      ],
      "Djiboutian": [
        ""
      ],
      "Dominican": [
        "Operaciones"
      ],
      "Dutch": [
        ""
      ],
      "East Timorese": [
        ""
      ],
      "Ecuadorean": [
        ""
      ],
      "Egyptian": [
        ""
      ],
      "Emirati": [
        ""
      ],
      "Equatorial Guinean": [
        ""
      ],
      "Eritrean": [
        ""
      ],
      "Estonian": [
        ""
      ],
      "Ethiopian": [
        ""
      ],
      "Fijian": [
        ""
      ],
      "Filipino": [
        ""
      ],
      "Finnish": [
        ""
      ],
      "French": [
        ""
      ],
      "Gabonese": [
        ""
      ],
      "Gambian": [
        ""
      ],
      "Georgian": [
        ""
      ],
      "German": [
        ""
      ],
      "Ghanaian": [
        ""
      ],
      "Greek": [
        ""
      ],
      "Grenadian": [
        ""
      ],
      "Guatemalan": [
        ""
      ],
      "Guinea-Bissauan": [
        ""
      ],
      "Guinean": [
        ""
      ],
      "Guyanese": [
        ""
      ],
      "Haitian": [
        ""
      ],
      "Honduran": [
        ""
      ],
      "Hungarian": [
        ""
      ],
      "I-Kiribati": [
        ""
      ],
      "Icelander": [
        ""
      ],
      "Indian": [
        ""
      ],
      "Indonesian": [
        ""
      ],
      "Iranian": [
        ""
      ],
      "Iraqi": [
        ""
      ],
      "Irish": [
        ""
      ],
      "Israeli": [
        ""
      ],
      "Italian": [
        ""
      ],
      "Ivorian": [
        ""
      ],
      "Jamaican": [
        ""
      ],
      "Japanese": [
        ""
      ],
      "Jordanian": [
        ""
      ],
      "Kazakhstani": [
        ""
      ],
      "Kenyan": [
        ""
      ],
      "Kittian and Nevisian": [
        ""
      ],
      "Kuwaiti": [
        ""
      ],
      "Kyrgyz": [
        ""
      ],
      "Laotian": [
        ""
      ],
      "Latvian": [
        ""
      ],
      "Lebanese": [
        ""
      ],
      "Liberian": [
        ""
      ],
      "Libyan": [
        ""
      ],
      "Liechtensteiner": [
        ""
      ],
      "Lithuanian": [
        ""
      ],
      "Luxembourger": [
        ""
      ],
      "Macedonian": [
        ""
      ],
      "Malagasy": [
        ""
      ],
      "Malawian": [
        ""
      ],
      "Malaysian": [
        ""
      ],
      "Maldivian": [
        ""
      ],
      "Malian": [
        ""
      ],
      "Maltese": [
        ""
      ],
      "Marshallese": [
        ""
      ],
      "Mauritanian": [
        ""
      ],
      "Mauritian": [
        ""
      ],
      "Mexican": [
        ""
      ],
      "Micronesian": [
        ""
      ],
      "Moldovan": [
        ""
      ],
      "Monacan": [
        ""
      ],
      "Mongolian": [
        ""
      ],
      "Moroccan": [
        ""
      ],
      "Mosotho": [
        ""
      ],
      "Mozambican": [
        ""
      ],
      "Namibian": [
        ""
      ],
      "Nauruan": [
        ""
      ],
      "Nepalese": [
        ""
      ],
      "New Zealander": [
        ""
      ],
      "Ni-Vanuatu": [
        ""
      ],
      "Nicaraguan": [
        ""
      ],
      "Nigerian": [
        ""
      ],
      "Nigerien": [
        ""
      ],
      "North Korean": [
        ""
      ],
      "Norwegian": [
        ""
      ],
      "Omani": [
        ""
      ],
      "Pakistani": [
        ""
      ],
      "Palauan": [
        ""
      ],
      "Panamanian": [
        ""
      ],
      "Papua New Guinean": [
        ""
      ],
      "Paraguayan": [
        ""
      ],
      "Peruvian": [
        ""
      ],
      "Polish": [
        ""
      ],
      "Portuguese": [
        ""
      ],
      "Qatari": [
        ""
      ],
      "Romanian": [
        ""
      ],
      "Russian": [
        ""
      ],
      "Rwandan": [
        ""
      ],
      "Saint Lucian": [
        ""
      ],
      "Salvadoran": [
        ""
      ],
      "Samoan": [
        ""
      ],
      "San Marinese": [
        ""
      ],
      "Sao Tomean": [
        ""
      ],
      "Saudi": [
        ""
      ],
      "Senegalese": [
        ""
      ],
      "Serbian": [
        ""
      ],
      "Seychellois": [
        ""
      ],
      "Sierra Leonean": [
        ""
      ],
      "Singaporean": [
        ""
      ],
      "Slovakian": [
        ""
      ],
      "Slovenian": [
        ""
      ],
      "Solomon Islander": [
        ""
      ],
      "Somali": [
        ""
      ],
      "South African": [
        ""
      ],
      "South Korean": [
        ""
      ],
      "Spanish": [
        ""
      ],
      "Sri Lankan": [
        ""
      ],
      "Sudanese": [
        ""
      ],
      "Surinamer": [
        ""
      ],
      "Swazi": [
        ""
      ],
      "Swedish": [
        ""
      ],
      "Swiss": [
        ""
      ],
      "Syrian": [
        ""
      ],
      "Taiwanese": [
        ""
      ],
      "Tajik": [
        ""
      ],
      "Tanzanian": [
        ""
      ],
      "Thai": [
        ""
      ],
      "Togolese": [
        ""
      ],
      "Tongan": [
        ""
      ],
      "Trinidadian or Tobagonian": [
        ""
      ],
      "Tunisian": [
        ""
      ],
      "Turkish": [
        ""
      ],
      "Tuvaluan": [
        "valor"
      ],
      "Ugandan": [
        ""
      ],
      "Ukrainian": [
        ""
      ],
      "Uruguayan": [
        ""
      ],
      "Uzbekistani": [
        ""
      ],
      "Venezuelan": [
        ""
      ],
      "Vietnamese": [
        ""
      ],
      "Yemenite": [
        ""
      ],
      "Zambian": [
        ""
      ],
      "Zimbabwean": [
        ""
      ],
      "🇦🇫": [
        ""
      ],
      "🇦🇱": [
        ""
      ],
      "🇩🇿": [
        ""
      ],
      "🇺🇸": [
        ""
      ],
      "🇦🇩": [
        ""
      ],
      "🇦🇴": [
        ""
      ],
      "🇦🇬": [
        ""
      ],
      "🇦🇷": [
        ""
      ],
      "🇦🇲": [
        ""
      ],
      "🇦🇺": [
        ""
      ],
      "🇦🇹": [
        ""
      ],
      "🇦🇿": [
        ""
      ],
      "🇧🇸": [
        ""
      ],
      "🇧🇭": [
        ""
      ],
      "🇧🇩": [
        ""
      ],
      "🇧🇧": [
        ""
      ],
      "🇧🇼": [
        ""
      ],
      "🇧🇾": [
        ""
      ],
      "🇧🇪": [
        ""
      ],
      "🇧🇿": [
        ""
      ],
      "🇧🇯": [
        ""
      ],
      "🇧🇹": [
        ""
      ],
      "🇧🇴": [
        ""
      ],
      "🇧🇦": [
        ""
      ],
      "🇧🇷": [
        ""
      ],
      "🇬🇧": [
        ""
      ],
      "🇧🇳": [
        ""
      ],
      "🇧🇬": [
        ""
      ],
      "🇧🇫": [
        ""
      ],
      "🇲🇲": [
        ""
      ],
      "🇧🇮": [
        ""
      ],
      "🇰🇭": [
        ""
      ],
      "🇨🇲": [
        ""
      ],
      "🇨🇦": [
        ""
      ],
      "🇨🇻": [
        ""
      ],
      "🇨🇫": [
        ""
      ],
      "🇨🇱": [
        ""
      ],
      "🇨🇳": [
        ""
      ],
      "🇨🇴": [
        ""
      ],
      "🇨🇩": [
        ""
      ],
      "🇨🇬": [
        ""
      ],
      "🇨🇷": [
        ""
      ],
      "🇨🇺": [
        ""
      ],
      "🇨🇾": [
        ""
      ],
      "🇨🇿": [
        ""
      ],
      "🇩🇰": [
        ""
      ],
      "🇩🇯": [
        ""
      ],
      "🇩🇴": [
        ""
      ],
      "🇳🇱": [
        ""
      ],
      "🇹🇱": [
        ""
      ],
      "🇪🇨": [
        ""
      ],
      "🇪🇬": [
        ""
      ],
      "🇦🇪": [
        ""
      ],
      "🇬🇶": [
        ""
      ],
      "🇪🇷": [
        ""
      ],
      "🇪🇪": [
        ""
      ],
      "🇪🇹": [
        ""
      ],
      "🇫🇯": [
        ""
      ],
      "🇵🇭": [
        ""
      ],
      "🇫🇮": [
        ""
      ],
      "🇫🇷": [
        ""
      ],
      "🇬🇦": [
        ""
      ],
      "🇬🇲": [
        ""
      ],
      "🇬🇪": [
        ""
      ],
      "UAE Dirham": [
        ""
      ],
      "Afghani": [
        ""
      ],
      "Lek": [
        ""
      ],
      "Armenian Dram": [
        ""
      ],
      "Netherlands Antillean Guilder": [
        ""
      ],
      "Kwanza": [
        ""
      ],
      "Argentine Peso": [
        ""
      ],
      "Australian Dollar": [
        ""
      ],
      "Aruban Florin": [
        ""
      ],
      "Azerbaijan Manat": [
        ""
      ],
      "Convertible Mark": [
        ""
      ],
      "Barbados Dollar": [
        ""
      ],
      "Taka": [
        ""
      ],
      "Bulgarian Lev": [
        ""
      ],
      "Bahraini Dinar": [
        ""
      ],
      "Burundi Franc": [
        ""
      ],
      "Bermudian Dollar": [
        ""
      ],
      "Brunei Dollar": [
        ""
      ],
      "Boliviano": [
        ""
      ],
      "Mvdol": [
        ""
      ],
      "Brazilian Real": [
        ""
      ],
      "Bahamian Dollar": [
        ""
      ],
      "Ngultrum": [
        ""
      ],
      "Pula": [
        ""
      ],
      "Belarusian Ruble": [
        ""
      ],
      "Belize Dollar": [
        ""
      ],
      "Canadian Dollar": [
        ""
      ],
      "Congolese Franc": [
        ""
      ],
      "WIR Euro": [
        ""
      ],
      "Swiss Franc": [
        ""
      ],
      "WIR Franc": [
        ""
      ],
      "Unidad de Fomento": [
        ""
      ],
      "Chilean Peso": [
        ""
      ],
      "Yuan Renminbi": [
        ""
      ],
      "Colombian Peso": [
        ""
      ],
      "Unidad de Valor Real": [
        ""
      ],
      "Costa Rican Colon": [
        ""
      ],
      "Cuban Peso": [
        ""
      ],
      "Cabo Verde Escudo": [
        ""
      ],
      "Czech Koruna": [
        ""
      ],
      "Djibouti Franc": [
        ""
      ],
      "Danish Krone": [
        ""
      ],
      "Dominican Peso": [
        ""
      ],
      "Algerian Dinar": [
        ""
      ],
      "Egyptian Pound": [
        ""
      ],
      "Nakfa": [
        ""
      ],
      "Ethiopian Birr": [
        ""
      ],
      "Euro": [
        ""
      ],
      "Fiji Dollar": [
        ""
      ],
      "Falkland Islands Pound": [
        ""
      ],
      "Pound Sterling": [
        ""
      ],
      "Lari": [
        ""
      ],
      "Ghana Cedi": [
        ""
      ],
      "Gibraltar Pound": [
        ""
      ],
      "Dalasi": [
        ""
      ],
      "Guinean Franc": [
        ""
      ],
      "Quetzal": [
        ""
      ],
      "Guyana Dollar": [
        ""
      ],
      "Hong Kong Dollar": [
        ""
      ],
      "Lempira": [
        ""
      ],
      "Gourde": [
        ""
      ],
      "Forint": [
        ""
      ],
      "Rupiah": [
        ""
      ],
      "New Israeli Sheqel": [
        ""
      ],
      "Indian Rupee": [
        ""
      ],
      "Iraqi Dinar": [
        ""
      ],
      "Iranian Rial": [
        ""
      ],
      "Iceland Krona": [
        ""
      ],
      "Jamaican Dollar": [
        ""
      ],
      "Jordanian Dinar": [
        ""
      ],
      "Yen": [
        ""
      ],
      "Kenyan Shilling": [
        ""
      ],
      "Som": [
        ""
      ],
      "Riel": [
        ""
      ],
      "Comorian Franc ": [
        ""
      ],
      "North Korean Won": [
        ""
      ],
      "Won": [
        ""
      ],
      "Kuwaiti Dinar": [
        ""
      ],
      "Cayman Islands Dollar": [
        ""
      ],
      "Tenge": [
        ""
      ],
      "Lao Kip": [
        ""
      ],
      "Lebanese Pound": [
        ""
      ],
      "Sri Lanka Rupee": [
        ""
      ],
      "Liberian Dollar": [
        ""
      ],
      "Loti": [
        ""
      ],
      "Libyan Dinar": [
        ""
      ],
      "Moroccan Dirham": [
        ""
      ],
      "Moldovan Leu": [
        ""
      ],
      "Malagasy Ariary": [
        ""
      ],
      "Denar": [
        ""
      ],
      "Kyat": [
        ""
      ],
      "Tugrik": [
        ""
      ],
      "Pataca": [
        ""
      ],
      "Ouguiya": [
        ""
      ],
      "Mauritius Rupee": [
        ""
      ],
      "Rufiyaa": [
        ""
      ],
      "Malawi Kwacha": [
        ""
      ],
      "Mexican Peso": [
        ""
      ],
      "Mexican Unidad de Inversion (UDI)": [
        ""
      ],
      "Malaysian Ringgit": [
        ""
      ],
      "Mozambique Metical": [
        ""
      ],
      "Namibia Dollar": [
        ""
      ],
      "Naira": [
        ""
      ],
      "Cordoba Oro": [
        ""
      ],
      "Norwegian Krone": [
        ""
      ],
      "Nepalese Rupee": [
        ""
      ],
      "New Zealand Dollar": [
        ""
      ],
      "Rial Omani": [
        ""
      ],
      "Balboa": [
        ""
      ],
      "Sol": [
        "viejo"
      ],
      "Kina": [
        ""
      ],
      "Philippine Peso": [
        ""
      ],
      "Pakistan Rupee": [
        ""
      ],
      "Zloty": [
        ""
      ],
      "Guarani": [
        ""
      ],
      "Qatari Rial": [
        ""
      ],
      "Romanian Leu": [
        ""
      ],
      "Serbian Dinar": [
        ""
      ],
      "Russian Ruble": [
        ""
      ],
      "Rwanda Franc": [
        ""
      ],
      "Saudi Riyal": [
        ""
      ],
      "Solomon Islands Dollar": [
        ""
      ],
      "Seychelles Rupee": [
        ""
      ],
      "Sudanese Pound": [
        ""
      ],
      "Swedish Krona": [
        ""
      ],
      "Singapore Dollar": [
        ""
      ],
      "Saint Helena Pound": [
        ""
      ],
      "Leone": [
        ""
      ],
      "Somali Shilling": [
        ""
      ],
      "Surinam Dollar": [
        ""
      ],
      "South Sudanese Pound": [
        ""
      ],
      "Dobra": [
        ""
      ],
      "El Salvador Colon": [
        ""
      ],
      "Syrian Pound": [
        ""
      ],
      "Lilangeni": [
        ""
      ],
      "Baht": [
        ""
      ],
      "Somoni": [
        ""
      ],
      "Turkmenistan New Manat": [
        ""
      ],
      "Tunisian Dinar": [
        ""
      ],
      "Pa’anga": [
        ""
      ],
      "Turkish Lira": [
        ""
      ],
      "Trinidad and Tobago Dollar": [
        ""
      ],
      "New Taiwan Dollar": [
        ""
      ],
      "Tanzanian Shilling": [
        ""
      ],
      "Hryvnia": [
        ""
      ],
      "Uganda Shilling": [
        ""
      ],
      "US Dollar": [
        ""
      ],
      "US Dollar (Next day)": [
        ""
      ],
      "Uruguay Peso en Unidades Indexadas (UI)": [
        ""
      ],
      "Peso Uruguayo": [
        ""
      ],
      "Unidad Previsional": [
        ""
      ],
      "Uzbekistan Sum": [
        ""
      ],
      "Bolívar Soberano": [
        ""
      ],
      "Dong": [
        ""
      ],
      "Vatu": [
        ""
      ],
      "Tala": [
        ""
      ],
      "CFA Franc BEAC": [
        ""
      ],
      "East Caribbean Dollar": [
        ""
      ],
      "CFA Franc BCEAO": [
        ""
      ],
      "CFP Franc": [
        ""
      ],
      "Sucre": [
        ""
      ],
      "Yemeni Rial": [
        ""
      ],
      "Rand": [
        ""
      ],
      "Zambian Kwacha": [
        ""
      ],
      "Zimbabwe Gold": [
        ""
      ],
      "Afar": [
        ""
      ],
      "Abkhazian": [
        ""
      ],
      "Achinese": [
        ""
      ],
      "Acoli": [
        ""
      ],
      "Adangme": [
        ""
      ],
      "Adyghe": [
        ""
      ],
      "Avestan": [
        ""
      ],
      "Tunisian Arabic": [
        ""
      ],
      "Afrikaans": [
        ""
      ],
      "Afrihili": [
        ""
      ],
      "Aghem": [
        ""
      ],
      "Ainu": [
        ""
      ],
      "Akan": [
        ""
      ],
      "Akkadian": [
        ""
      ],
      "Alabama": [
        ""
      ],
      "Aleut": [
        ""
      ],
      "Gheg Albanian": [
        ""
      ],
      "Southern Altai": [
        ""
      ],
      "Amarik": [
        ""
      ],
      "Aragonese": [
        ""
      ],
      "Old English": [
        ""
      ],
      "Angika": [
        ""
      ],
      "Modern Standard Arabic": [
        ""
      ],
      "Arabik": [
        ""
      ],
      "Aramaic": [
        ""
      ],
      "Mapuche": [
        ""
      ],
      "Araona": [
        ""
      ],
      "Arapaho": [
        ""
      ],
      "Algerian Arabic": [
        ""
      ],
      "Arawak": [
        ""
      ],
      "Moroccan Arabic": [
        ""
      ],
      "Egyptian Arabic": [
        ""
      ],
      "Asu": [
        ""
      ],
      "Assamese": [
        ""
      ],
      "American Sign Language": [
        ""
      ],
      "Asturian": [
        ""
      ],
      "Avaric": [
        ""
      ],
      "Kotava": [
        ""
      ],
      "Awadhi": [
        ""
      ],
      "Aymara": [
        ""
      ],
      "South Azerbaijani": [
        ""
      ],
      "Bashkir": [
        ""
      ],
      "Baluchi": [
        ""
      ],
      "Balinese": [
        ""
      ],
      "Bavarian": [
        ""
      ],
      "Basaa": [
        ""
      ],
      "Bamun": [
        ""
      ],
      "Batak Toba": [
        ""
      ],
      "Ghomala": [
        ""
      ],
      "Belarus kasa": [
        ""
      ],
      "Beja": [
        ""
      ],
      "Bemba": [
        ""
      ],
      "Betawi": [
        ""
      ],
      "Bena": [
        ""
      ],
      "Bafut": [
        ""
      ],
      "Badaga": [
        ""
      ],
      "Bɔlgeria kasa": [
        ""
      ],
      "Bhojpuri": [
        ""
      ],
      "Bislama": [
        ""
      ],
      "Bikol": [
        ""
      ],
      "Bini": [
        ""
      ],
      "Banjar": [
        ""
      ],
      "Kom": [
        ""
      ],
      "Siksika": [
        ""
      ],
      "Bambara": [
        ""
      ],
      "Bengali kasa": [
        ""
      ],
      "Tibetan": [
        ""
      ],
      "Bishnupriya": [
        ""
      ],
      "Bakhtiari": [
        ""
      ],
      "Braj": [
        ""
      ],
      "Breton": [
        ""
      ],
      "Brahui": [
        ""
      ],
      "Bodo": [
        ""
      ],
      "Akoose": [
        ""
      ],
      "Buriat": [
        ""
      ],
      "Buginese": [
        ""
      ],
      "Bulu": [
        ""
      ],
      "Blin": [
        ""
      ],
      "Medumba": [
        ""
      ],
      "Catalan": [
        ""
      ],
      "Caddo": [
        ""
      ],
      "Carib": [
        ""
      ],
      "Cayuga": [
        ""
      ],
      "Atsam": [
        ""
      ],
      "Cebuano": [
        ""
      ],
      "Chechen": [
        ""
      ],
      "Chiga": [
        ""
      ],
      "Chibcha": [
        ""
      ],
      "Chamorro": [
        ""
      ],
      "Chagatai": [
        ""
      ],
      "Chuukese": [
        ""
      ],
      "Mari": [
        ""
      ],
      "Chinook Jargon": [
        ""
      ],
      "Choctaw": [
        ""
      ],
      "Chipewyan": [
        ""
      ],
      "Cherokee": [
        ""
      ],
      "Cheyenne": [
        ""
      ],
      "Central Kurdish": [
        ""
      ],
      "Corsican": [
        ""
      ],
      "Coptic": [
        ""
      ],
      "Capiznon": [
        ""
      ],
      "Cree": [
        ""
      ],
      "Crimean Turkish": [
        ""
      ],
      "Kashubian": [
        ""
      ],
      "Kyɛk kasa": [
        ""
      ],
      "Church Slavic": [
        ""
      ],
      "Chuvash": [
        ""
      ],
      "Welsh": [
        ""
      ],
      "Dakota": [
        ""
      ],
      "Dargwa": [
        ""
      ],
      "Taita": [
        ""
      ],
      "Austrian German": [
        ""
      ],
      "Swiss High German": [
        ""
      ],
      "Gyaaman": [
        ""
      ],
      "Delaware": [
        ""
      ],
      "Slave": [
        ""
      ],
      "Dogrib": [
        ""
      ],
      "Dinka": [
        ""
      ],
      "Zarma": [
        ""
      ],
      "Dogri": [
        ""
      ],
      "Lower Sorbian": [
        ""
      ],
      "Central Dusun": [
        ""
      ],
      "Duala": [
        ""
      ],
      "Middle Dutch": [
        ""
      ],
      "Divehi": [
        ""
      ],
      "Jola-Fonyi": [
        ""
      ],
      "Dyula": [
        ""
      ],
      "Dzongkha": [
        ""
      ],
      "Dazaga": [
        ""
      ],
      "Embu": [
        ""
      ],
      "Ewe": [
        ""
      ],
      "Efik": [
        ""
      ],
      "Emilian": [
        ""
      ],
      "Ancient Egyptian": [
        ""
      ],
      "Ekajuk": [
        ""
      ],
      "Greek kasa": [
        ""
      ],
      "Elamite": [
        ""
      ],
      "Australian English": [
        ""
      ],
      "Borɔfo": [
        ""
      ],
      "Canadian English": [
        ""
      ],
      "British English": [
        ""
      ],
      "Middle English": [
        ""
      ],
      "American English": [
        ""
      ],
      "Esperanto": [
        "Operación"
      ],
      "Latin American Spanish": [
        ""
      ],
      "European Spanish": [
        ""
      ],
      "Mexican Spanish": [
        ""
      ],
      "Spain kasa": [
        ""
      ],
      "Central Yupik": [
        ""
      ],
      "Basque": [
        ""
      ],
      "Ewondo": [
        ""
      ],
      "Extremaduran": [
        ""
      ],
      "Fang": [
        ""
      ],
      "Pɛɛhyia kasa": [
        ""
      ],
      "Fanti": [
        ""
      ],
      "Fulah": [
        ""
      ],
      "Tornedalen Finnish": [
        ""
      ],
      "Faroese": [
        ""
      ],
      "Fon": [
        ""
      ],
      "Canadian French": [
        ""
      ],
      "Cajun French": [
        "Divisa"
      ],
      "Swiss French": [
        ""
      ],
      "Frɛnkye": [
        ""
      ],
      "Middle French": [
        ""
      ],
      "Old French": [
        ""
      ],
      "Arpitan": [
        ""
      ],
      "Northern Frisian": [
        ""
      ],
      "Eastern Frisian": [
        ""
      ],
      "Friulian": [
        ""
      ],
      "Western Frisian": [
        ""
      ],
      "Ga": [
        ""
      ],
      "Gagauz": [
        ""
      ],
      "Gan Chinese": [
        ""
      ],
      "Gayo": [
        ""
      ],
      "Gbaya": [
        ""
      ],
      "Zoroastrian Dari": [
        ""
      ],
      "Scottish Gaelic": [
        ""
      ],
      "Geez": [
        ""
      ],
      "Gilbertese": [
        ""
      ],
      "Galician": [
        ""
      ],
      "Gilaki": [
        ""
      ],
      "Middle High German": [
        ""
      ],
      "Old High German": [
        ""
      ],
      "Goan Konkani": [
        ""
      ],
      "Gondi": [
        ""
      ],
      "Gorontalo": [
        ""
      ],
      "Gothic": [
        ""
      ],
      "Grebo": [
        ""
      ],
      "Ancient Greek": [
        ""
      ],
      "Swiss German": [
        ""
      ],
      "Wayuu": [
        ""
      ],
      "Gujarati": [
        ""
      ],
      "Frafra": [
        ""
      ],
      "Gusii": [
        ""
      ],
      "Manx": [
        ""
      ],
      "Gwichʼin": [
        ""
      ],
      "Hausa": [
        ""
      ],
      "Haida": [
        ""
      ],
      "Hakka Chinese": [
        ""
      ],
      "Hawaiian": [
        ""
      ],
      "Hebrew": [
        ""
      ],
      "Fiji Hindi": [
        ""
      ],
      "Hindi": [
        ""
      ],
      "Hiligaynon": [
        ""
      ],
      "Hittite": [
        ""
      ],
      "Hmong": [
        ""
      ],
      "Hiri Motu": [
        ""
      ],
      "Upper Sorbian": [
        ""
      ],
      "Xiang Chinese": [
        ""
      ],
      "Hangri kasa": [
        ""
      ],
      "Hupa": [
        ""
      ],
      "Herero": [
        ""
      ],
      "Interlingua": [
        ""
      ],
      "Iban": [
        ""
      ],
      "Ibibio": [
        ""
      ],
      "Indonihyia kasa": [
        ""
      ],
      "Interlingue": [
        ""
      ],
      "Igbo": [
        ""
      ],
      "Sichuan Yi": [
        ""
      ],
      "Inupiaq": [
        ""
      ],
      "Iloko": [
        ""
      ],
      "Ingush": [
        ""
      ],
      "Ido": [
        ""
      ],
      "Icelandic": [
        ""
      ],
      "Italy kasa": [
        ""
      ],
      "Inuktitut": [
        ""
      ],
      "Ingrian": [
        ""
      ],
      "Gyapan kasa": [
        ""
      ],
      "Jamaican Creole English": [
        ""
      ],
      "Lojban": [
        ""
      ],
      "Ngomba": [
        ""
      ],
      "Machame": [
        ""
      ],
      "Judeo-Persian": [
        ""
      ],
      "Judeo-Arabic": [
        ""
      ],
      "Jutish": [
        ""
      ],
      "Gyabanis kasa": [
        ""
      ],
      "Kara-Kalpak": [
        ""
      ],
      "Kabyle": [
        ""
      ],
      "Kachin": [
        ""
      ],
      "Jju": [
        ""
      ],
      "Kamba": [
        ""
      ],
      "Kawi": [
        ""
      ],
      "Kabardian": [
        ""
      ],
      "Kanembu": [
        ""
      ],
      "Tyap": [
        ""
      ],
      "Makonde": [
        ""
      ],
      "Kabuverdianu": [
        ""
      ],
      "Kenyang": [
        ""
      ],
      "Koro": [
        ""
      ],
      "Kongo": [
        ""
      ],
      "Kaingang": [
        ""
      ],
      "Khasi": [
        ""
      ],
      "Khotanese": [
        ""
      ],
      "Koyra Chiini": [
        ""
      ],
      "Khowar": [
        ""
      ],
      "Kikuyu": [
        ""
      ],
      "Kirmanjki": [
        ""
      ],
      "Kuanyama": [
        ""
      ],
      "Kako": [
        ""
      ],
      "Kazakh": [
        ""
      ],
      "Kalaallisut": [
        ""
      ],
      "Kalenjin": [
        ""
      ],
      "Kimbundu": [
        ""
      ],
      "Kambodia kasa": [
        ""
      ],
      "Kannada": [
        ""
      ],
      "Komi-Permyak": [
        ""
      ],
      "Konkani": [
        ""
      ],
      "Korea kasa": [
        ""
      ],
      "Kosraean": [
        ""
      ],
      "Kpelle": [
        ""
      ],
      "Karachay-Balkar": [
        ""
      ],
      "Krio": [
        ""
      ],
      "Kinaray-a": [
        ""
      ],
      "Kanuri": [
        ""
      ],
      "Karelian": [
        ""
      ],
      "Kurukh": [
        ""
      ],
      "Shambala": [
        ""
      ],
      "Bafia": [
        ""
      ],
      "Colognian": [
        ""
      ],
      "Kashmiri": [
        ""
      ],
      "Kurdish": [
        ""
      ],
      "Kumyk": [
        ""
      ],
      "Kutenai": [
        ""
      ],
      "Komi": [
        ""
      ],
      "Cornish": [
        ""
      ],
      "Ladino": [
        "Cargando"
      ],
      "Langi": [
        ""
      ],
      "Lahnda": [
        ""
      ],
      "Latin": [
        ""
      ],
      "Lamba": [
        ""
      ],
      "Luxembourgish": [
        ""
      ],
      "Lezghian": [
        ""
      ],
      "Lingua Franca Nova": [
        ""
      ],
      "Ganda": [
        ""
      ],
      "Ligurian": [
        ""
      ],
      "Limburgish": [
        ""
      ],
      "Livonian": [
        ""
      ],
      "Lakota": [
        ""
      ],
      "Lombard": [
        ""
      ],
      "Lingala": [
        ""
      ],
      "Lao": [
        ""
      ],
      "Mongo": [
        ""
      ],
      "Lozi": [
        ""
      ],
      "Latgalian": [
        ""
      ],
      "Luba-Lulua": [
        ""
      ],
      "Luiseno": [
        ""
      ],
      "Luba-Katanga": [
        ""
      ],
      "Lunda": [
        ""
      ],
      "Luo": [
        ""
      ],
      "Mizo": [
        ""
      ],
      "Luyia": [
        ""
      ],
      "Literary Chinese": [
        ""
      ],
      "Laz": [
        ""
      ],
      "Madurese": [
        "Reiniciar"
      ],
      "Mafa": [
        ""
      ],
      "Magahi": [
        ""
      ],
      "Maithili": [
        ""
      ],
      "Makasar": [
        ""
      ],
      "Mandingo": [
        "Cargando"
      ],
      "Masai": [
        ""
      ],
      "Maba": [
        ""
      ],
      "Moksha": [
        ""
      ],
      "Mandar": [
        ""
      ],
      "Mende": [
        ""
      ],
      "Meru": [
        ""
      ],
      "Morisyen": [
        ""
      ],
      "Middle Irish": [
        ""
      ],
      "Makhuwa-Meetto": [
        ""
      ],
      "Metaʼ": [
        ""
      ],
      "Micmac": [
        ""
      ],
      "Maori": [
        ""
      ],
      "Minangkabau": [
        ""
      ],
      "Malayalam": [
        ""
      ],
      "Manchu": [
        ""
      ],
      "Manipuri": [
        ""
      ],
      "Mohawk": [
        ""
      ],
      "Mossi": [
        ""
      ],
      "Western Mari": [
        ""
      ],
      "Marathi": [
        ""
      ],
      "Malay kasa": [
        ""
      ],
      "Mundang": [
        ""
      ],
      "Multiple Languages": [
        "Lenguaje actual"
      ],
      "Creek": [
        ""
      ],
      "Mirandese": [
        ""
      ],
      "Marwari": [
        ""
      ],
      "Mentawai": [
        ""
      ],
      "Bɛɛmis kasa": [
        ""
      ],
      "Myene": [
        ""
      ],
      "Erzya": [
        ""
      ],
      "Mazanderani": [
        ""
      ],
      "Min Nan Chinese": [
        ""
      ],
      "Neapolitan": [
        ""
      ],
      "Nama": [
        ""
      ],
      "Norwegian Bokmål": [
        ""
      ],
      "North Ndebele": [
        ""
      ],
      "Low German": [
        ""
      ],
      "Nɛpal kasa": [
        ""
      ],
      "Newari": [
        ""
      ],
      "Ndonga": [
        ""
      ],
      "Nias": [
        ""
      ],
      "Niuean": [
        ""
      ],
      "Ao Naga": [
        ""
      ],
      "Flemish": [
        ""
      ],
      "Dɛɛkye": [
        ""
      ],
      "Kwasio": [
        ""
      ],
      "Ngiemboon": [
        ""
      ],
      "Norwegian Nynorsk": [
        ""
      ],
      "Nogai": [
        ""
      ],
      "Old Norse": [
        ""
      ],
      "Novial": [
        ""
      ],
      "NʼKo": [
        ""
      ],
      "South Ndebele": [
        ""
      ],
      "Northern Sotho": [
        ""
      ],
      "Nuer": [
        ""
      ],
      "Navajo": [
        ""
      ],
      "Classical Newari": [
        ""
      ],
      "Nyamwezi": [
        ""
      ],
      "Nyankole": [
        ""
      ],
      "Nyanja": [
        ""
      ],
      "Nyoro": [
        ""
      ],
      "Nzima": [
        ""
      ],
      "Occitan": [
        ""
      ],
      "Ojibwa": [
        ""
      ],
      "Oromo": [
        ""
      ],
      "Oriya": [
        ""
      ],
      "Osage": [
        ""
      ],
      "Ossetic": [
        ""
      ],
      "Ottoman Turkish": [
        ""
      ],
      "Pangasinan": [
        ""
      ],
      "Pahlavi": [
        ""
      ],
      "Pampanga": [
        ""
      ],
      "Papiamento": [
        "Pago"
      ],
      "Pungyabi kasa": [
        ""
      ],
      "Picard": [
        ""
      ],
      "Pennsylvania German": [
        ""
      ],
      "Plautdietsch": [
        ""
      ],
      "Old Persian": [
        ""
      ],
      "Palatine German": [
        ""
      ],
      "Phoenician": [
        ""
      ],
      "Pali": [
        ""
      ],
      "Pɔland kasa": [
        ""
      ],
      "Piedmontese": [
        ""
      ],
      "Pontic": [
        ""
      ],
      "Pohnpeian": [
        ""
      ],
      "Prussian": [
        ""
      ],
      "Old Provençal": [
        ""
      ],
      "Pashto": [
        ""
      ],
      "Brazilian Portuguese": [
        ""
      ],
      "Pɔɔtugal kasa": [
        ""
      ],
      "European Portuguese": [
        ""
      ],
      "Kʼicheʼ": [
        ""
      ],
      "Chimborazo Highland Quichua": [
        ""
      ],
      "Quechua": [
        ""
      ],
      "Rajasthani": [
        ""
      ],
      "Rapanui": [
        ""
      ],
      "Rarotongan": [
        ""
      ],
      "Romagnol": [
        ""
      ],
      "Riffian": [
        ""
      ],
      "Romansh": [
        ""
      ],
      "Rundi": [
        ""
      ],
      "Rombo": [
        ""
      ],
      "Moldavian": [
        ""
      ],
      "Romany": [
        ""
      ],
      "Root": [
        ""
      ],
      "Romenia kasa": [
        ""
      ],
      "Rotuman": [
        ""
      ],
      "Rusyn": [
        ""
      ],
      "Roviana": [
        ""
      ],
      "Aromanian": [
        ""
      ],
      "Rahyia kasa": [
        ""
      ],
      "Rwa": [
        ""
      ],
      "Rewanda kasa": [
        ""
      ],
      "Sandawe": [
        ""
      ],
      "Sakha": [
        ""
      ],
      "Samaritan Aramaic": [
        ""
      ],
      "Samburu": [
        ""
      ],
      "Sanskrit": [
        ""
      ],
      "Sasak": [
        ""
      ],
      "Santali": [
        ""
      ],
      "Saurashtra": [
        ""
      ],
      "Ngambay": [
        ""
      ],
      "Sangu": [
        ""
      ],
      "Sicilian": [
        ""
      ],
      "Scots": [
        ""
      ],
      "Sardinian": [
        ""
      ],
      "Sassarese Sardinian": [
        ""
      ],
      "Sindhi": [
        ""
      ],
      "Seneca": [
        ""
      ],
      "Sena": [
        ""
      ],
      "Seri": [
        ""
      ],
      "Selkup": [
        ""
      ],
      "Northern Sami": [
        ""
      ],
      "Koyraboro Senni": [
        ""
      ],
      "Old Irish": [
        ""
      ],
      "Sango": [
        ""
      ],
      "Samogitian": [
        ""
      ],
      "Tachelhit": [
        ""
      ],
      "Shan": [
        ""
      ],
      "Serbo-Croatian": [
        ""
      ],
      "Chadian Arabic": [
        ""
      ],
      "Sidamo": [
        ""
      ],
      "Sinhala": [
        ""
      ],
      "Slovak": [
        ""
      ],
      "Lower Silesian": [
        ""
      ],
      "Selayar": [
        ""
      ],
      "Southern Sami": [
        ""
      ],
      "Lule Sami": [
        ""
      ],
      "Inari Sami": [
        ""
      ],
      "Skolt Sami": [
        ""
      ],
      "Soninke": [
        ""
      ],
      "Shona": [
        ""
      ],
      "Sogdien": [
        ""
      ],
      "Somalia kasa": [
        ""
      ],
      "Sranan Tongo": [
        ""
      ],
      "Serer": [
        ""
      ],
      "Swati": [
        ""
      ],
      "Saho": [
        ""
      ],
      "Saterland Frisian": [
        ""
      ],
      "Southern Sotho": [
        ""
      ],
      "Sukuma": [
        ""
      ],
      "Susu": [
        ""
      ],
      "Sundanese": [
        ""
      ],
      "Sumerian": [
        ""
      ],
      "Sweden kasa": [
        ""
      ],
      "Comorian": [
        ""
      ],
      "Congo Swahili": [
        ""
      ],
      "Swahili": [
        ""
      ],
      "Classical Syriac": [
        ""
      ],
      "Syriac": [
        ""
      ],
      "Silesian": [
        ""
      ],
      "Tamil kasa": [
        ""
      ],
      "Tulu": [
        ""
      ],
      "Timne": [
        ""
      ],
      "Teso": [
        ""
      ],
      "Tereno": [
        ""
      ],
      "Telugu": [
        ""
      ],
      "Tetum": [
        ""
      ],
      "Taeland kasa": [
        ""
      ],
      "Tigre": [
        ""
      ],
      "Tigrinya": [
        ""
      ],
      "Tiv": [
        "Propina"
      ],
      "Tokelau": [
        ""
      ],
      "Tsakhur": [
        ""
      ],
      "Turkmen": [
        ""
      ],
      "Klingon": [
        ""
      ],
      "Tlingit": [
        ""
      ],
      "Tagalog": [
        ""
      ],
      "Talysh": [
        ""
      ],
      "Tamashek": [
        ""
      ],
      "Tswana": [
        ""
      ],
      "Nyasa Tonga": [
        ""
      ],
      "Tok Pisin": [
        ""
      ],
      "Tɛɛki kasa": [
        ""
      ],
      "Turoyo": [
        ""
      ],
      "Taroko": [
        ""
      ],
      "Tsakonian": [
        ""
      ],
      "Tsimshian": [
        ""
      ],
      "Tsonga": [
        ""
      ],
      "Tatar": [
        ""
      ],
      "Muslim Tat": [
        ""
      ],
      "Tumbuka": [
        ""
      ],
      "Tasawaq": [
        ""
      ],
      "Twi": [
        "Propina"
      ],
      "Tahitian": [
        ""
      ],
      "Tuvinian": [
        ""
      ],
      "Central Atlas Tamazight": [
        ""
      ],
      "Udmurt": [
        ""
      ],
      "Ugaritic": [
        ""
      ],
      "Uyghur": [
        ""
      ],
      "Ukren kasa": [
        ""
      ],
      "Umbundu": [
        ""
      ],
      "Unknown Language": [
        "Lenguaje actual"
      ],
      "Urdu kasa": [
        ""
      ],
      "Uzbek": [
        ""
      ],
      "Vai": [
        ""
      ],
      "Venetian": [
        ""
      ],
      "Veps": [
        ""
      ],
      "Venda": [
        ""
      ],
      "Viɛtnam kasa": [
        ""
      ],
      "West Flemish": [
        ""
      ],
      "Main-Franconian": [
        ""
      ],
      "Votic": [
        ""
      ],
      "Volapük": [
        ""
      ],
      "Võro": [
        ""
      ],
      "Vunjo": [
        ""
      ],
      "Walser": [
        ""
      ],
      "Wolaytta": [
        ""
      ],
      "Waray": [
        ""
      ],
      "Washo": [
        ""
      ],
      "Walloon": [
        ""
      ],
      "Warlpiri": [
        ""
      ],
      "Wolof": [
        ""
      ],
      "Wu Chinese": [
        ""
      ],
      "Kalmyk": [
        ""
      ],
      "Xhosa": [
        ""
      ],
      "Mingrelian": [
        ""
      ],
      "Soga": [
        ""
      ],
      "Yao": [
        ""
      ],
      "Yapese": [
        ""
      ],
      "Yangben": [
        ""
      ],
      "Yemba": [
        ""
      ],
      "Yiddish": [
        ""
      ],
      "Yoruba": [
        ""
      ],
      "Nheengatu": [
        ""
      ],
      "Cantonese": [
        ""
      ],
      "Zapotec": [
        ""
      ],
      "Zhuang": [
        ""
      ],
      "Blissymbols": [
        ""
      ],
      "Zeelandic": [
        ""
      ],
      "Zenaga": [
        ""
      ],
      "Standard Moroccan Tamazight": [
        ""
      ],
      "Simplified Chinese": [
        ""
      ],
      "Traditional Chinese": [
        ""
      ],
      "Kyaena kasa": [
        ""
      ],
      "Zuni": [
        ""
      ],
      "Zulu": [
        ""
      ],
      "No linguistic content": [
        ""
      ],
      "Zaza": [
        ""
      ],
      "Could not load user attention request": [
        "No se pudo cargar la comisión del exchange"
      ],
      "No notification left": [
        ""
      ],
      "Ignore": [
        "Ignorar"
      ],
      "Invalid dev experiment URI.": [
        ""
      ],
      "URI: %1$s": [
        "%1$s"
      ],
      "Experiment": [
        ""
      ],
      "Insert lost denomination": [
        "Operaciones"
      ],
      "Apply": [
        ""
      ],
      "Pending refresh": [
        "Operaciones pendientes"
      ],
      "Could not load the transfer payment status": [
        "No se pudo cargar el estado del pago"
      ],
      "Could not load the status of the order template": [
        "No se pudo cargar la lista de transacciones"
      ],
      "required": [
        ""
      ],
      "Review order": [
        ""
      ],
      "Missing URI": [
        ""
      ],
      "Check the link.": [
        "Haz click en el ícono de rompecabezas"
      ],
      "Could not parse the recovery URI": [
        "No se pudo obtener la información de la URI payto"
      ],
      "Try another URI": [
        "Intentar otro"
      ],
      "Import backup, show info": [
        "Importar copia de seguridad, mostrar información"
      ],
      "Add new exchange": [
        "Agregar nuevo exchange"
      ],
      "Add exchange for %1$s": [
        "Agregar exchange para %1$s"
      ],
      "Enter the URL of an exchange you trust.": [
        "Ingresar la URL de un exchange en el que confíes."
      ],
      "An exchange has been found! Review the information and click next": [
        "Un exchange ha sido encontrado! Revisa la información y haz clic en siguiente"
      ],
      "This exchange is already in your list.": [
        "El exchange está listo para la retirada"
      ],
      "Only exchange accessible through \"http\" and \"https\" are allowed.": [
        ""
      ],
      "This exchange protocol version is not supported: \" %1$s\".": [
        ""
      ],
      "This exchange currency \"%1$s\" doesn&apos;t match the expected currency %2$s.": [
        "Este exchange no coincide con la divisa %1$s esperada"
      ],
      "No exchange found in that URL.": [
        "No se encontró exchange para %1$s"
      ],
      "loading": [
        "cargando"
      ],
      "Version": [
        "Versión"
      ],
      "Can't use the URL: \"%1$s\"": [
        ""
      ],
      "Exchange URL": [
        "URL del proveedor"
      ],
      "Add new contact": [
        "Agregar otra cuenta"
      ],
      "Enter the details of a contact you trust.": [
        "Ingresar la URL de un exchange en el que confíes."
      ],
      "Contact info valid.": [
        ""
      ],
      "This contact is already in your list.": [
        ""
      ],
      "Review contact": [
        ""
      ],
      "Contact alias": [
        ""
      ],
      "Alias type": [
        ""
      ],
      "Mailbox service": [
        ""
      ],
      "Mailbox address": [
        "Dirección de Bitcoin"
      ],
      "Contact source": [
        ""
      ],
      "Could not load the list of contacts": [
        "No se pudo cargar la lista de transacciones"
      ],
      "Source": [
        ""
      ],
      "Send Cash": [
        "Envíar %1$s"
      ],
      "Request Cash": [
        ""
      ],
      "No contacts found.": [
        ""
      ],
      "Could not bank choices for account": [
        "No se pudo cargar la información de recuperación de copia de seguridad"
      ],
      "Could not load the list of messages": [
        "No se pudo cargar la lista de exchange conocidos"
      ],
      "Unexpected error when trying to initialize mailbox: %1$s": [
        ""
      ],
      "Unexpected error when trying to delete message: %1$s": [
        ""
      ],
      "Unexpected error when trying to fetch messages: %1$s": [
        ""
      ],
      "Date": [
        "Fecha"
      ],
      "You may copy your mailbox URI from below and register it at %1$s with your alias(es) to make it easier for others to find you.": [
        ""
      ],
      "copy uri": [
        "Copiar"
      ],
      "hide uri": [
        "Esconder QR"
      ],
      "show uri": [
        "Mostrar QR"
      ],
      "Fetch messages": [
        "Mensaje del comerciante"
      ],
      "Mailbox not initialized": [
        ""
      ],
      "Initialize mailbox": [
        ""
      ],
      "No messages yet.": [
        "No hay exchanges todavía"
      ],
      "Taler URI handler": [
        ""
      ],
      "Could not found a handler for the Taler URI": [
        "No se pudo encontrar ningún exchange para la divisa %1$s"
      ],
      "The uri read in the path parameter is not valid: \"%1$s\"": [
        ""
      ],
      "Digital cash payment": [
        "Pago con dinero digital"
      ],
      "Digital cash refund": [
        "Devolución de dinero digital"
      ],
      "Digital cash withdrawal": [
        "Retirada de dinero digital"
      ],
      "Digital cash deposit": [
        "Depósito de dinero digital"
      ],
      "Digital cash invoice": [
        "Factura digital"
      ],
      "Digital cash transfer": [
        "Transferencia de dinero digital"
      ],
      "Digital cash recovery": [
        "Recuperación de billetera digital"
      ],
      "Development experiment": [
        "Modo desarrollador"
      ],
      "Return to wallet": [
        ""
      ],
      "Network is offline": [
        ""
      ],
      "Diagnostics timed out. Could not talk to the wallet backend.": [
        "El diagnóstico caducó. No nos pudimos comunicar con la billetera."
      ],
      "Problems detected:": [
        "Problemas detectados:"
      ],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).": [
        "Por favor revisa en tu configuración %1$s que tienes IndexedDB habilitado (el nombre de la preferencia %2$s)."
      ],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.": [
        "La base de datos de la billetera expiró. Por ahora la migración automática no está soportada. Por favor dirijasé a %1$s para reiniciar la base de datos de la billetera."
      ],
      "here": [
        ""
      ],
      "Running diagnostics": [
        "Ejecutando diagnósticos"
      ],
      "Edit": [
        "Editar"
      ],
      "List of products": [
        "Lista de productos"
      ],
      "free": [
        "Gratis"
      ],
      "Could not get provider information": [
        "No se pudo conseguir la información del proveedor"
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "es",
  "completeness": 23
};

strings['de'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "de"
      },
      "Balance": [
        "Salden"
      ],
      "Backup": [
        "Backup"
      ],
      "Notifications": [
        ""
      ],
      "QR Reader and Taler URI": [
        "QR-Code-Leser und Taler URI"
      ],
      "Settings": [
        "Einstellungen"
      ],
      "Dev tools": [
        "Debugging-Tools"
      ],
      "Contacts": [
        ""
      ],
      "Mailbox": [
        ""
      ],
      "Search transactions": [
        ""
      ],
      "%1$s": [
        "%1$s"
      ],
      "KYC requirement": [
        ""
      ],
      "PENDING OPERATIONS": [
        "AUSSTEHENDE VORGÄNGE"
      ],
      "Error was thrown trying to: \"%1$s\"": [
        ""
      ],
      "Could not open the wallet database. Firefox is known to run into this problem under \"permanent private mode\".": [
        ""
      ],
      "Could not open the wallet database.": [
        ""
      ],
      "Unexpected request error, code: %1$s": [
        ""
      ],
      "more info": [
        ""
      ],
      "wrap text": [
        ""
      ],
      "copy content": [
        ""
      ],
      "less info": [
        ""
      ],
      "Retry operation": [
        "Ausstehende Vorgänge"
      ],
      "Loading": [
        "Lädt Daten"
      ],
      "Could not load backup providers": [
        "Es wurden keine Backup-Anbieter gefunden"
      ],
      "No backup providers configured": [
        "Es wurden keine Backup-Anbieter konfiguriert"
      ],
      "Add provider": [
        "Anbieter hinzufügen"
      ],
      "Sync all backups": [
        "Synchronisiere alle Backups"
      ],
      "Sync now": [
        "Jetzt synchronisieren"
      ],
      "Last synced": [
        "Zuletzt synchronisiert"
      ],
      "Not synced": [
        "Nicht synchronisiert"
      ],
      "Expires in": [
        "Läuft ab in"
      ],
      "show less": [
        ""
      ],
      "show more": [
        ""
      ],
      "There was an error loading the provider detail for &quot;%1$s&quot;": [
        "Es gab einen Fehler beim Laden der Details des Backup-Anbieters &quot; %1$s&quot;"
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [
        "Kein Anbieter mit der URL &quot;%1$s&quot; gefunden."
      ],
      "See providers": [
        "Zahlungsdienste ansehen"
      ],
      "Last backup": [
        "Letztes Backup"
      ],
      "Back up": [
        "Backup ausführen"
      ],
      "Provider fee": [
        "Gebühr des Backup-Anbieters"
      ],
      "per year": [
        "pro Jahr"
      ],
      "Extend": [
        "Erweitern"
      ],
      "terms has changed, extending the service will imply accepting the new terms of service": [
        "Die AGB sind geändert worden, eine Weiternutzung des Diensts erfordert die Einwilligung in die neuen Allgemeinen Geschäftsbedingungen (AGB)"
      ],
      "old": [
        "alt"
      ],
      "new": [
        "neu"
      ],
      "fee": [
        "Gebühr"
      ],
      "storage": [
        ""
      ],
      "Remove provider": [
        "Anbieter entfernen"
      ],
      "Pay": [
        "Zahlung"
      ],
      "Withdraw": [
        "Abheben"
      ],
      "This provider has reported an error": [
        "Dieser Anbieter hat einen Fehler gemeldet"
      ],
      "There is conflict with another backup from &quot;%1$s&quot;": [
        "Es gibt einen Konflikt mit einem anderen Backup von %1$s"
      ],
      "Backup is not readable": [
        "Das Backup kann nicht gelesen werden"
      ],
      "Unknown backup problem: %1$s": [
        "Unbekanntes Problem mit dem Backup: %1$s"
      ],
      "service paid": [
        ""
      ],
      "Backup valid until": [
        "Backup gültig bis"
      ],
      "Cancel": [
        "Abbrechen"
      ],
      "Open pay page": [
        "Seite für Zahlungen aufrufen"
      ],
      "Open refund page": [
        "Seite für Rückerstattungen aufrufen"
      ],
      "Open withdraw page": [
        "Abhebeseite öffnen"
      ],
      "Your wallet is empty.": [
        "Ihre Geldbörse verzeichnet keine Vorkommnisse."
      ],
      "Get digital cash": [
        "Digitales Bargeld abheben"
      ],
      "Try the demo bank and withdraw test money.": [
        ""
      ],
      "Could not load the balance": [
        "Der Einzahlungssaldo konnte nicht geladen werden"
      ],
      "Receive": [
        "Name des Begünstigten"
      ],
      "Send %1$s": [
        "%1$s übertragen"
      ],
      "This page has a withdrawal action.": [
        "Diese Seite bietet eine Abhebungsmöglichkeit."
      ],
      "This page has pay action.": [
        "Diese Seite verlangt eine Zahlung."
      ],
      "This page has a refund action.": [
        "Diese Seite verfügt über eine Rückerstattungsfunktion."
      ],
      "This page has a add exchange action.": [
        "Diese Seite verfügt über eine Rückerstattungsfunktion."
      ],
      "Open add exchange page": [
        "Seite für Rückerstattungen aufrufen"
      ],
      "This page has an add contact action.": [
        "Diese Seite verfügt über eine Rückerstattungsfunktion."
      ],
      "Open add contact page": [
        "Seite für Zahlungen aufrufen"
      ],
      "Taler Action": [
        ""
      ],
      "This page has a malformed taler uri.": [
        "Diese Seite weist eine fehlerhafte Taler-URI auf."
      ],
      "Dismiss": [
        "Verwerfen"
      ],
      "this popup is being closed and you are being redirected to %1$s": [
        "Dieses Fenster wird geschlossen, Sie werden zu %1$s weitergeleitet"
      ],
      "Could not load purchase proposal details": [
        "Es war nicht möglich, die Details des Angebots zu laden"
      ],
      "Order Id": [
        ""
      ],
      "Summary": [
        "Zusammenfassung"
      ],
      "Amount": [
        "Betrag"
      ],
      "Choices": [
        ""
      ],
      "This purchase have %1$s choices": [
        ""
      ],
      "Merchant name": [
        "Händlername"
      ],
      "Merchant jurisdiction": [
        ""
      ],
      "Merchant address": [
        "Händleradresse"
      ],
      "Merchant logo": [
        "Händlerlogo"
      ],
      "Merchant website": [
        "Händlerwebsite"
      ],
      "Merchant email": [
        "Emailadresse des Händlers"
      ],
      "Merchant public key": [
        "Public Key des Händlers"
      ],
      "Created at": [
        "Erstellungsdatum"
      ],
      "Refund deadline": [
        "Rückerstattungsfrist"
      ],
      "Auto refund": [
        "Automatische Rückerstattung"
      ],
      "Pay deadline": [
        "Zahlungsfrist"
      ],
      "Fulfillment URL": [
        "Adresse digitaler Dienstleistung (Fulfillment-URL)"
      ],
      "Fulfillment message": [
        "Fulfillment message (Nachricht an Zahlende über bereitgestelltes digitales Gut)"
      ],
      "Max deposit fee": [
        ""
      ],
      "Exchanges": [
        "Zahlungsdienstleister"
      ],
      "Could not load the status of the term of service": [
        "Konnte die Liste der Zahlungsdienstleister nicht laden"
      ],
      "Select one option": [
        ""
      ],
      "Show terms of service": [
        "Allgemeine Geschäftsbedingungen (AGB) anzeigen"
      ],
      "I accept the exchange terms of service": [
        "Ich akzeptiere die Allgemeinen Geschäftsbedingungen (AGB)"
      ],
      "Review exchange terms of service": [
        "Allgemeine Geschäftsbedingungen (AGB) ansehen"
      ],
      "The exchange doesn't have a terms of service.": [
        "Dieser Zahlungsdienstleister hat keine Allgemeinen Geschäftsbedingungen (AGB)"
      ],
      "Skip it for now.": [
        ""
      ],
      "Format": [
        ""
      ],
      "Language": [
        ""
      ],
      "The exchange replied with a empty terms of service": [
        "Der Exchange wird mit Allgemeinen Geschäftsbedingungen ohne Inhalt antworten"
      ],
      "No terms of service. The exchange replied with a empty document": [
        "Der Exchange wird mit Allgemeinen Geschäftsbedingungen ohne Inhalt antworten"
      ],
      "No terms of service. The exchange replied with a empty text": [
        "Der Exchange wird mit Allgemeinen Geschäftsbedingungen ohne Inhalt antworten"
      ],
      "Download Terms of Service": [
        "Allgemeine Geschäftsbedingungen (AGB) herunterladen"
      ],
      "Hide terms of service": [
        "Allgemeine Geschäftsbedingungen (AGB) verstecken"
      ],
      "Amount is not valid": [
        ""
      ],
      "Bank account": [
        "Bankkonto"
      ],
      "Bitcoin address": [
        "Bitcoin Adresse"
      ],
      "BIC/IBAN": [
        "IBAN"
      ],
      "IBAN": [
        "IBAN"
      ],
      "Bitcoin transfer details": [
        ""
      ],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.": [
        ""
      ],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts": [
        ""
      ],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC": [
        ""
      ],
      "Ethereum transfer details": [
        "Überweisungsdetails"
      ],
      "You need to wire to the service provider account.": [
        ""
      ],
      "Make sure the amount show %1$s ETH": [
        ""
      ],
      "Bank transfer details": [
        "Überweisungsdetails"
      ],
      "Account": [
        "Konto"
      ],
      "Bank host": [
        "Server der Bank (Bank host)"
      ],
      "BIC": [
        ""
      ],
      "Exchange": [
        "Zahlungsdienstleister"
      ],
      "Reserve Pub": [
        ""
      ],
      "Step 1:": [
        ""
      ],
      "Copy this code and paste it into the subject/purpose field in your banking app or bank website": [
        ""
      ],
      "Subject": [
        "Verwendungszweck"
      ],
      "Step 2:": [
        ""
      ],
      "If you don't already have it in your banking favourites list, then copy and paste this IBAN and the name into the receiver fields in your banking app or website": [
        ""
      ],
      "Receiver name": [
        "Name des Begünstigten"
      ],
      "Receiver postal code": [
        "Name des Begünstigten"
      ],
      "Receiver town": [
        "Name des Begünstigten"
      ],
      "Step 3:": [
        ""
      ],
      "Finish the wire transfer setting the amount in your banking app or website, then this withdrawal will proceed automatically.": [
        ""
      ],
      "Make sure ALL data is correct, including the subject; otherwise, the money will not arrive in this wallet. You can use the copy buttons (%1$s) to prevent typing errors or the \"payto://\" URI below to copy just one value.": [
        ""
      ],
      "Alternative if your bank already supports PayTo URI, you can use this %1$s link instead": [
        ""
      ],
      "Continue with banking app or website": [
        ""
      ],
      "Show QR code": [
        ""
      ],
      "Could not load transaction information": [
        "Information zu der Transaktion konnten nicht geladen werden"
      ],
      "Bank account confirmation required for the transaction to complete.": [
        ""
      ],
      "KYC check required for the transaction to complete.": [
        ""
      ],
      "No additional information has been provided.": [
        ""
      ],
      "Follow this link to the%1$s %2$s": [
        ""
      ],
      "This transaction is pending to complete.": [
        "Diese Transaktion ist nicht abgeschlossen"
      ],
      "Retry": [
        "Erneut versuchen"
      ],
      "There was an error trying to complete the transaction.": [
        "Es kam zu einem Fehler bei der Finalisierung der Transaktion"
      ],
      "This transaction was aborted.": [
        "Diese Transaktion ist nicht abgeschlossen"
      ],
      "This transaction failed.": [
        "Diese Transaktion ist nicht abgeschlossen"
      ],
      "Caution!": [
        "Achtung!"
      ],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.": [
        ""
      ],
      "Confirm": [
        "Eingeben"
      ],
      "Doing a cancellation while the transaction still active might result in lost coins. Do you still want to cancel the transaction?": [
        ""
      ],
      "No": [
        ""
      ],
      "Yes": [
        ""
      ],
      "Abort": [
        ""
      ],
      "Resume": [
        ""
      ],
      "Suspend": [
        ""
      ],
      "Delete": [
        ""
      ],
      "Withdrawal": [
        "Abhebung"
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Select one of the accounts and use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Details": [
        "Detail-Angaben"
      ],
      "Withdrawal incomplete.": [
        "Abhebung"
      ],
      "If you have already sent money to the service provider account it will wire it back at %1$s": [
        ""
      ],
      "Payment": [
        "Zahlung"
      ],
      "Refunds": [
        ""
      ],
      "%1$s %2$s on %3$s .": [
        "%1$s %2$s über %3$s"
      ],
      "Refund is in progress.": [
        ""
      ],
      "Merchant created a refund for this order but was not automatically picked up.": [
        ""
      ],
      "Offer": [
        ""
      ],
      "Accept": [
        ""
      ],
      "Confirmation code": [
        ""
      ],
      "Merchant": [
        ""
      ],
      "Invoice ID": [
        ""
      ],
      "Deposit": [
        ""
      ],
      "Wire transfer deadline.": [
        "Überweisungsdetails"
      ],
      "Wire transfer still pending.": [
        ""
      ],
      "The service provider deadline to make the wire transfer is: %1$s": [
        ""
      ],
      "Wire transfer completed.": [
        ""
      ],
      " ": [
        ""
      ],
      "Transfer details": [
        "Überweisungsdetails"
      ],
      "Wire transfer in progress.": [
        ""
      ],
      "Refresh": [
        ""
      ],
      "Refund": [
        "Rückerstattung"
      ],
      "Purchase summary": [
        ""
      ],
      "Credit": [
        "Guthaben"
      ],
      "Invoice": [
        ""
      ],
      "URI": [
        ""
      ],
      "Debit": [
        ""
      ],
      "Transfer": [
        "Überweisung"
      ],
      "Lost": [
        ""
      ],
      "Reason": [
        ""
      ],
      "Denomination expired.": [
        ""
      ],
      "Denomination vanished.": [
        ""
      ],
      "Denomination is unoffered.": [
        ""
      ],
      "Fees": [
        ""
      ],
      "Total": [
        ""
      ],
      "Sent": [
        ""
      ],
      "Converted": [
        ""
      ],
      "Bank fee": [
        "Höchster Gebührenbetrag"
      ],
      "Price": [
        "Preis"
      ],
      "Subtotal": [
        ""
      ],
      "Refunded": [
        "Rückerstattet"
      ],
      "Transfer identification": [
        ""
      ],
      "copy": [
        "Kopieren"
      ],
      "hide qr": [
        ""
      ],
      "forward to contact": [
        ""
      ],
      "show qr": [
        ""
      ],
      "Wire transfer need a confirmation. Go to the %1$s and check wire transfer operation to exchange account is complete.": [
        ""
      ],
      "bank site": [
        "Server der Bank (Bank host)"
      ],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins.": [
        ""
      ],
      "Exchange is ready to send the coins, withdrawal in progress.": [
        "Der Zahlungsdienstleister ist für Abhebungen verfügbar"
      ],
      "Send &nbsp; %1$s": [
        ""
      ],
      "Pay &nbsp; %1$s": [
        ""
      ],
      "Merchant message": [
        ""
      ],
      "Balance is not enough.": [
        "Das Guthaben reicht nicht aus."
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which exchange and auditors the merchant trust.": [
        ""
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which wire methods the merchant accepts.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to pay for this contract which is restricted.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to spend right know. There are some coins that need to be refreshed.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s available.": [
        ""
      ],
      "Provider is missing the global fee configuration, this likely means it is misconfigured.": [
        ""
      ],
      "Balance looks like it should be enough, but doesn't cover all fees requested by the merchant and payment processor. Please ensure there is at least %1$s %2$s more balance in your wallet or ask your merchant to cover more of the fees.": [
        ""
      ],
      "Pay with a mobile phone": [
        ""
      ],
      "Hide QR": [
        ""
      ],
      "Scan the QR code or &nbsp; %1$s": [
        ""
      ],
      "click here": [
        "Klicken zum Anzeigen"
      ],
      "Could not load the payment and balance status": [
        "Der Einzahlungssaldo konnte nicht geladen werden"
      ],
      "Purchase": [
        ""
      ],
      "This purchase is age restricted.": [
        ""
      ],
      "Expires at": [
        "Läuft ab in"
      ],
      "Already paid, you are going to be redirected to %1$s": [
        ""
      ],
      "Already paid": [
        ""
      ],
      "Already claimed": [
        ""
      ],
      "Could not load the refund status": [
        "Konnte den Status der Erstattung nicht laden"
      ],
      "You&apos;ve ignored the refund.": [
        ""
      ],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [
        "Der Verkäufer %1$s bietet Ihnen eine Rückerstattung an."
      ],
      "Order amount": [
        ""
      ],
      "Could not load the list of exchanges": [
        "Konnte die Liste der Zahlungsdienstleister nicht laden"
      ],
      "Could not load info from URI": [
        "Konnte nichts laden"
      ],
      "Could not load the withdrawal details": [
        "Es war nicht möglich, die Details des Angebots zu laden"
      ],
      "under %1$s": [
        "%1$s übertragen"
      ],
      "should be greater than zero": [
        ""
      ],
      "can't be greater than %1$s": [
        ""
      ],
      "Could not load exchange details info": [
        "Konnte die Gebühren des Zahlungsdiensts nicht laden"
      ],
      "Close": [
        "Schließen"
      ],
      "There is no exchange available for currency %1$s": [
        ""
      ],
      "You can add more exchanges from the settings.": [
        ""
      ],
      "Exchange %1$s is not available": [
        ""
      ],
      "Exchange status can view accessed from the settings.": [
        ""
      ],
      "Service fee description": [
        ""
      ],
      "Select %1$s exchange": [
        ""
      ],
      "Reset": [
        "Zurücksetzen"
      ],
      "Use this exchange": [
        ""
      ],
      "Auditors": [
        "Auditor"
      ],
      "Doesn&apos;t have auditors": [
        ""
      ],
      "currency": [
        "Währung"
      ],
      "Coin operations": [
        ""
      ],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.": [
        ""
      ],
      "Deposits": [
        "Auszahlungen"
      ],
      "Denomination": [
        ""
      ],
      "Current": [
        "Währung"
      ],
      "Selected": [
        ""
      ],
      "Until": [
        ""
      ],
      "Withdrawals": [
        "Abhebungen"
      ],
      "Transfer operations": [
        ""
      ],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.": [
        ""
      ],
      "Operation": [
        ""
      ],
      "Fee": [
        "Gebühr"
      ],
      "Wallet operations": [
        ""
      ],
      "Feature": [
        ""
      ],
      "Currency": [
        "Währung"
      ],
      "This operation has already been completed.": [
        "Diese Transaktion ist nicht abgeschlossen"
      ],
      "This operation has already been aborted": [
        ""
      ],
      "This operation has started and should be completed in the bank.": [
        ""
      ],
      "You can confirm the operation in": [
        ""
      ],
      "this page": [
        ""
      ],
      "This operation has already been completed by another wallet.": [
        ""
      ],
      "This operation has already been used by another wallet.": [
        ""
      ],
      "It can be confirmed in": [
        ""
      ],
      "Age restriction": [
        "Altersbeschränkung"
      ],
      "Withdraw &nbsp; %1$s": [
        "Abhebung von &nbsp; %1$s"
      ],
      "Withdraw to a mobile phone": [
        ""
      ],
      "Short description of the invoice": [
        ""
      ],
      "Create": [
        "Anlegen"
      ],
      "Short description of the transfer": [
        ""
      ],
      "Could not load the invoice payment status": [
        "Konnte den Status der Einzahlung nicht laden"
      ],
      "Receive &nbsp; %1$s": [
        ""
      ],
      "Could not load known bank accounts": [
        "Der Einzahlungssaldo konnte nicht geladen werden"
      ],
      "Could not load supported wire methods": [
        "Es wurden keine Backup-Anbieter gefunden"
      ],
      "No wire methods supported for this currency": [
        ""
      ],
      "Try another wire method.": [
        "Von einem anderen Wallet"
      ],
      "Known accounts for %1$s": [
        ""
      ],
      "To add a new account first select the account type.": [
        ""
      ],
      "Unable add this account": [
        ""
      ],
      "Add": [
        "Hinzufügen"
      ],
      "IBAN accounts": [
        "Bankkonto"
      ],
      "Alias": [
        ""
      ],
      "Int. Account Number": [
        ""
      ],
      "Account name": [
        "Konto"
      ],
      "KYC": [
        ""
      ],
      "Taler accounts": [
        "Bankkonto"
      ],
      "Host": [
        ""
      ],
      "Bitcoin accounts": [
        "Bankkonto"
      ],
      "Address": [
        ""
      ],
      "Can't be empty": [
        ""
      ],
      "Bitcoin Account": [
        "Bankkonto"
      ],
      "Taler Bank": [
        ""
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "Required": [
        ""
      ],
      "International Bank Account Number": [
        ""
      ],
      "Could not load balance information": [
        "Es konnten keine Backup-Wiederherstellungsinformationen geladen werden"
      ],
      "Could not load fee for amount %1$s": [
        "Konnte den Status der Erstattung nicht laden"
      ],
      "A currency or an amount should be indicated": [
        ""
      ],
      "There is no enough balance to make a deposit for currency %1$s": [
        ""
      ],
      "Send %1$s to your account": [
        ""
      ],
      "There is no account to make a deposit for currency %1$s": [
        ""
      ],
      "Add account": [
        ""
      ],
      "Select account": [
        ""
      ],
      "Manage accounts": [
        "Bankkonto"
      ],
      "Brut amount": [
        "Bankkonto"
      ],
      "Deposit fee": [
        ""
      ],
      "Net amount": [
        "Betrag"
      ],
      "Deposit&nbsp;%1$s %2$s": [
        "Einzahlung von &nbsp;%1$s %2$s"
      ],
      "Need approval in the Bank": [
        ""
      ],
      "Waiting for wire transfer to complete": [
        ""
      ],
      "Exchange is waiting the wire transfer": [
        ""
      ],
      "Payment in progress": [
        ""
      ],
      "Executing refund...": [
        "Automatische Rückerstattung"
      ],
      "Refreshing coins...": [
        ""
      ],
      "Deposit in progress": [
        ""
      ],
      "Waiting to be paid": [
        ""
      ],
      "Receiving the transfer": [
        "Übertragung von e-Geld"
      ],
      "Waiting to be received": [
        ""
      ],
      "Denomination expired": [
        ""
      ],
      "Denomination vanished": [
        ""
      ],
      "Denomination unoffered": [
        ""
      ],
      "ABORTED": [
        ""
      ],
      "FAILED": [
        ""
      ],
      "Could not load the list of transactions": [
        ""
      ],
      "unknown scope": [
        ""
      ],
      "Send": [
        "Überweisen"
      ],
      "Your transaction history is empty for this currency.": [
        ""
      ],
      "Review terms of service": [
        "Allgemeine Geschäftsbedingungen (AGB) ansehen"
      ],
      "Provider URL": [
        ""
      ],
      "Please review and accept this provider&apos;s terms of service": [
        ""
      ],
      "Pricing": [
        ""
      ],
      "free of charge": [
        ""
      ],
      "%1$s per year of service": [
        ""
      ],
      "Storage": [
        ""
      ],
      "%1$s megabytes of storage per year of service": [
        ""
      ],
      "Accept terms of service": [
        ""
      ],
      "Add backup provider": [
        ""
      ],
      "Backup providers may charge for their service": [
        ""
      ],
      "URL": [
        "URL"
      ],
      "Name": [
        "Name"
      ],
      "Next": [
        "Nächste Seite"
      ],
      "Navigator": [
        ""
      ],
      "Automatically open wallet": [
        ""
      ],
      "Open the wallet when a payment action is found.": [
        ""
      ],
      "Version Info": [
        "Version"
      ],
      "Web Extension": [
        ""
      ],
      "The version of wallet core is not supported. (supported version: %1$s, wallet version: %2$s)": [
        ""
      ],
      "Exchange compatibility": [
        ""
      ],
      "Merchant compatibility": [
        ""
      ],
      "Bank compatibility": [
        ""
      ],
      "Wallet Core compatibility": [
        ""
      ],
      "Enable developer mode": [
        "Entwicklermodus"
      ],
      "Show more information and options in the UI": [
        ""
      ],
      "Display": [
        ""
      ],
      "Current Language": [
        ""
      ],
      "Show backup feature": [
        ""
      ],
      "Backup integration still in beta.": [
        ""
      ],
      "Show P2P alias feature": [
        ""
      ],
      "P2P aliases integration still in beta.": [
        ""
      ],
      "Show suspend/resume transaction": [
        ""
      ],
      "Prevent transaction from doing network request.": [
        ""
      ],
      "Show refresh transaction type in the transaction list": [
        ""
      ],
      "Refresh transaction will be hidden by default if the refresh operation doesn't have fee.": [
        ""
      ],
      "Show more account types on deposit": [
        ""
      ],
      "Extends the UI to more payment target types.": [
        ""
      ],
      "Show JSON on error": [
        ""
      ],
      "Print more information about the error. Useful for debugging.": [
        ""
      ],
      "Allow HTTP connections": [
        ""
      ],
      "Using HTTP connection may be faster but unsafe (wallet restart required)": [
        ""
      ],
      "Lang selector": [
        ""
      ],
      "Allows to manually change the language of the UI. Otherwise it will be automatically selected by your browser configuration.": [
        ""
      ],
      "Edit exchange management": [
        ""
      ],
      "Allows to see the list of exchange, remove, add and switch before withdrawal.": [
        ""
      ],
      "Select terms of service format": [
        "Allgemeine Geschäftsbedingungen (AGB) ansehen"
      ],
      "Allows to render the terms of service on different format selected by the user.": [
        ""
      ],
      "Show wallet activity": [
        ""
      ],
      "Show the wallet notification and observability event in the UI.": [
        ""
      ],
      "Enable v1 contract": [
        ""
      ],
      "v1 is under development and wallet-core converts v1 contract to v0. Enable this options for testing": [
        ""
      ],
      "GNU Taler Wallet installed!": [
        ""
      ],
      "You can open the wallet using the combination %1$s .": [
        ""
      ],
      "Also pinning the GNU Taler Wallet to your browser allows you to quick access without keyboard:": [
        ""
      ],
      "Click the puzzle icon": [
        ""
      ],
      "Search for GNU Taler Wallet": [
        ""
      ],
      "Click the pin icon": [
        ""
      ],
      "Next Steps": [
        ""
      ],
      "Try the demo": [
        ""
      ],
      "Click here to open the wallet activity tab.": [
        ""
      ],
      "Events": [
        ""
      ],
      "Active tasks": [
        ""
      ],
      "All events": [
        ""
      ],
      "Balance change": [
        "Salden"
      ],
      "Backup failed": [
        "Backup gültig bis"
      ],
      "Transaction updated": [
        "Diese Transaktion ist nicht abgeschlossen"
      ],
      "Exchange updated": [
        "Zahlungsdienstleister"
      ],
      "Idle": [
        ""
      ],
      "task.%1$s": [
        "%1$s"
      ],
      "wallet.%1$s(%2$s)": [
        ""
      ],
      "Bank account info changed": [
        "Bankkonto"
      ],
      "Contact added": [
        ""
      ],
      "Contact deleted": [
        ""
      ],
      "Mailbox message added": [
        ""
      ],
      "Mailbox message deleted": [
        ""
      ],
      "Type": [
        ""
      ],
      "Id": [
        ""
      ],
      "Since": [
        ""
      ],
      "Next try": [
        "Nächste Seite"
      ],
      "Error": [
        ""
      ],
      "Transaction": [
        ""
      ],
      "No exchange yet": [
        ""
      ],
      "Status": [
        ""
      ],
      "Terms of Service": [
        "Allgemeine Geschäftsbedingungen (AGB) herunterladen"
      ],
      "Last Update": [
        ""
      ],
      "Actions": [
        ""
      ],
      "ok": [
        "OK"
      ],
      "pending": [
        ""
      ],
      "proposed": [
        ""
      ],
      "unknown (exchange status should be updated)": [
        ""
      ],
      "Reload": [
        "Lädt Daten"
      ],
      "Purge": [
        ""
      ],
      "Make regional": [
        ""
      ],
      "Make global": [
        ""
      ],
      "Forget ToS": [
        "Vergessen"
      ],
      "Debug tools": [
        "Debugging-Tools"
      ],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?": [
        "Möchten Sie alle Werte Ihres Wallets UNWIDERRUFLICH ZERSTÖREN und ALLE IHRE MÜNZEN VERLIEREN?"
      ],
      "reset": [
        "zurücksetzen"
      ],
      "TESTING: This may delete all your coin, proceed with caution": [
        ""
      ],
      "run gc": [
        ""
      ],
      "import database": [
        "Datenbank importieren"
      ],
      "export database": [
        "Datenbank exportieren"
      ],
      "Clear exchange key cache": [
        ""
      ],
      "Database exported at %1$s %2$s to download": [
        ""
      ],
      "Inject Taler support in all pages": [
        ""
      ],
      "Enabling this option will make `window.taler` be available in all sites": [
        ""
      ],
      "Exchange Entries": [
        "Zahlungsdienstleister"
      ],
      "Add an exchange": [
        ""
      ],
      "Log levels": [
        ""
      ],
      "Coins": [
        ""
      ],
      "usable coins": [
        "Münzen verfügbar zur Verwendung"
      ],
      "id": [
        ""
      ],
      "denom": [
        ""
      ],
      "status": [
        ""
      ],
      "from refresh?": [
        ""
      ],
      "age key count": [
        ""
      ],
      "spent coins": [
        ""
      ],
      "click to show": [
        "Klicken zum Anzeigen"
      ],
      "URI is not valid. Taler URI should start with \"taler://\"": [
        ""
      ],
      "After the URI type it should follow a '/' with more information.": [
        ""
      ],
      "This URI type is not supported": [
        ""
      ],
      "This URI requires the bank host and operation id separated by /": [
        ""
      ],
      "This URI requires the merchant host, order id and session id separated by /": [
        ""
      ],
      "This URI requires the merchant host and order id separated by /": [
        ""
      ],
      "This URI requires the exchange host and key id separated by /": [
        ""
      ],
      "This URI requires the merchant host and template id separated by /": [
        ""
      ],
      "This URI requires the host key and provider list separated by /": [
        ""
      ],
      "This URI requires the experiment id": [
        ""
      ],
      "This URI requires the exchange host": [
        ""
      ],
      "This URI requires the alias type, the alias and the mailbox URI separated by /": [
        ""
      ],
      "This URI requires string after the first /": [
        ""
      ],
      "The bank host is invalid": [
        ""
      ],
      "The merchant host is invalid": [
        ""
      ],
      "The URI should end with /": [
        ""
      ],
      "The exchange host is invalid": [
        ""
      ],
      "The contact is invalid": [
        ""
      ],
      "The amount is invalid": [
        ""
      ],
      "A parameter is invalid": [
        ""
      ],
      "checking...": [
        ""
      ],
      "Unexpected error happen reading the camera: %1$s": [
        ""
      ],
      "Could not found a QR code in the file": [
        ""
      ],
      "Unexpected error happen reading the file: %1$s": [
        ""
      ],
      "Scan a QR code or enter taler:// URI below": [
        ""
      ],
      "Pay invoice": [
        "Zahlungsfrist"
      ],
      "Withdrawal from bank": [
        "Abhebung"
      ],
      "Claim refund": [
        "Automatische Rückerstattung"
      ],
      "Accept payment": [
        "Bezahlung bestätigen"
      ],
      "Complete order": [
        ""
      ],
      "Restore wallet": [
        ""
      ],
      "Enable experiment": [
        ""
      ],
      "Withdraw from exchange": [
        ""
      ],
      "Add exchange": [
        ""
      ],
      "Notify transaction": [
        ""
      ],
      "Add contact": [
        ""
      ],
      "The exchange URL is invalid.": [
        ""
      ],
      "The exchange URL should end with '/'": [
        ""
      ],
      "Couldn't found an exchange in the URL specified.": [
        ""
      ],
      "HTTP request failed to %1$s: %2$s": [
        ""
      ],
      "HTTP request failed to %1$s": [
        ""
      ],
      "The merchant URL is invalid.": [
        ""
      ],
      "The merchant URL should end with '/'": [
        ""
      ],
      "Couldn't found an merchant in the URL specified.": [
        ""
      ],
      "The bank URL is invalid.": [
        ""
      ],
      "The bank URL should end with '/'": [
        ""
      ],
      "Couldn't found an bank in the URL specified.": [
        ""
      ],
      "Could not load exchanges": [
        "Konnte die Gebühren des Zahlungsdiensts nicht laden"
      ],
      "Choose a currency to proceed or add another exchange": [
        ""
      ],
      "Known currencies": [
        ""
      ],
      "Specify the origin": [
        ""
      ],
      "Use previous origins:": [
        ""
      ],
      "From another bank account": [
        "Von meinem Bankkonto"
      ],
      "From another wallet": [
        "Von einem anderen Wallet"
      ],
      "From a %1$s URI or QR code": [
        ""
      ],
      "Enter URI here": [
        "URI eingeben"
      ],
      "Specify the destination": [
        ""
      ],
      "Use previous destinations:": [
        ""
      ],
      "To another bank account": [
        "Von meinem Bankkonto"
      ],
      "To another wallet": [
        ""
      ],
      "To a %1$s URI or QR code": [
        ""
      ],
      "Afghanistan": [
        ""
      ],
      "Albania": [
        ""
      ],
      "Algeria": [
        ""
      ],
      "United States": [
        ""
      ],
      "Andorra": [
        ""
      ],
      "Angola": [
        ""
      ],
      "Antigua and Barbuda": [
        ""
      ],
      "Argentina": [
        ""
      ],
      "Armenia": [
        ""
      ],
      "Australia": [
        ""
      ],
      "Austria": [
        ""
      ],
      "Azerbaijan": [
        ""
      ],
      "Bahamas": [
        ""
      ],
      "Bahrain": [
        ""
      ],
      "Bangladesh": [
        ""
      ],
      "Barbados": [
        ""
      ],
      "Botswana": [
        ""
      ],
      "Belarus": [
        ""
      ],
      "Belgium": [
        ""
      ],
      "Belize": [
        ""
      ],
      "Benin": [
        ""
      ],
      "Bhutan": [
        ""
      ],
      "Bolivia": [
        ""
      ],
      "Bosnia and Herzegovina": [
        ""
      ],
      "Brazil": [
        ""
      ],
      "United Kingdom": [
        ""
      ],
      "Brunei": [
        ""
      ],
      "Bulgaria": [
        ""
      ],
      "Burkina Faso": [
        ""
      ],
      "Myanmar": [
        ""
      ],
      "Burundi": [
        ""
      ],
      "Cambodia": [
        ""
      ],
      "Cameroon": [
        ""
      ],
      "Canada": [
        ""
      ],
      "Cape Verde": [
        ""
      ],
      "Central African Republic": [
        ""
      ],
      "Chad": [
        ""
      ],
      "Chile": [
        ""
      ],
      "China": [
        ""
      ],
      "Colombia": [
        ""
      ],
      "Comoros": [
        ""
      ],
      "Democratic Republic of the Congo": [
        ""
      ],
      "Republic of the Congo": [
        ""
      ],
      "Costa Rica": [
        ""
      ],
      "Croatia": [
        ""
      ],
      "Cuba": [
        ""
      ],
      "Cyprus": [
        ""
      ],
      "Czech Republic": [
        "Public Key des Händlers"
      ],
      "Denmark": [
        ""
      ],
      "Djibouti": [
        ""
      ],
      "Dominican Republic": [
        ""
      ],
      "Netherlands": [
        ""
      ],
      "Timor-Leste": [
        ""
      ],
      "Ecuador": [
        ""
      ],
      "Egypt": [
        ""
      ],
      "United Arab Emirates": [
        ""
      ],
      "Equatorial Guinea": [
        ""
      ],
      "Eritrea": [
        ""
      ],
      "Estonia": [
        ""
      ],
      "Ethiopia": [
        ""
      ],
      "Fiji": [
        ""
      ],
      "Philippines": [
        ""
      ],
      "Finland": [
        ""
      ],
      "France": [
        "Abbrechen"
      ],
      "Gabon": [
        ""
      ],
      "Gambia": [
        ""
      ],
      "Georgia": [
        ""
      ],
      "Germany": [
        ""
      ],
      "Ghana": [
        ""
      ],
      "Greece": [
        ""
      ],
      "Grenada": [
        ""
      ],
      "Guatemala": [
        ""
      ],
      "Guinea-Bissau": [
        ""
      ],
      "Guinea": [
        ""
      ],
      "Guyana": [
        ""
      ],
      "Haiti": [
        ""
      ],
      "Honduras": [
        ""
      ],
      "Hungary": [
        ""
      ],
      "Kiribati": [
        ""
      ],
      "Iceland": [
        ""
      ],
      "India": [
        ""
      ],
      "Indonesia": [
        ""
      ],
      "Iran": [
        ""
      ],
      "Iraq": [
        ""
      ],
      "Ireland": [
        ""
      ],
      "Israel": [
        ""
      ],
      "Italy": [
        ""
      ],
      "Ivory Coast": [
        ""
      ],
      "Jamaica": [
        ""
      ],
      "Japan": [
        ""
      ],
      "Jordan": [
        ""
      ],
      "Kazakhstan": [
        ""
      ],
      "Kenya": [
        ""
      ],
      "Saint Kitts and Nevis": [
        ""
      ],
      "Kuwait": [
        ""
      ],
      "Kyrgyzstan": [
        ""
      ],
      "Laos": [
        ""
      ],
      "Latvia": [
        ""
      ],
      "Lebanon": [
        ""
      ],
      "Liberia": [
        ""
      ],
      "Libya": [
        ""
      ],
      "Liechtenstein": [
        ""
      ],
      "Lithuania": [
        ""
      ],
      "Luxembourg": [
        ""
      ],
      "North Macedonia": [
        ""
      ],
      "Madagascar": [
        ""
      ],
      "Malawi": [
        ""
      ],
      "Malaysia": [
        ""
      ],
      "Maldives": [
        ""
      ],
      "Mali": [
        ""
      ],
      "Malta": [
        ""
      ],
      "Marshall Islands": [
        ""
      ],
      "Mauritania": [
        ""
      ],
      "Mauritius": [
        ""
      ],
      "Mexico": [
        ""
      ],
      "Micronesia": [
        ""
      ],
      "Moldova": [
        ""
      ],
      "Monaco": [
        ""
      ],
      "Mongolia": [
        ""
      ],
      "Morocco": [
        ""
      ],
      "Lesotho": [
        ""
      ],
      "Mozambique": [
        ""
      ],
      "Namibia": [
        ""
      ],
      "Nauru": [
        ""
      ],
      "Nepal": [
        ""
      ],
      "New Zealand": [
        ""
      ],
      "Vanuatu": [
        ""
      ],
      "Nicaragua": [
        ""
      ],
      "Nigeria": [
        ""
      ],
      "Niger": [
        ""
      ],
      "North Korea": [
        ""
      ],
      "Norway": [
        ""
      ],
      "Oman": [
        ""
      ],
      "Pakistan": [
        ""
      ],
      "Palau": [
        ""
      ],
      "Panama": [
        ""
      ],
      "Papua New Guinea": [
        ""
      ],
      "Paraguay": [
        ""
      ],
      "Peru": [
        ""
      ],
      "Poland": [
        ""
      ],
      "Portugal": [
        ""
      ],
      "Qatar": [
        ""
      ],
      "Romania": [
        ""
      ],
      "Russia": [
        ""
      ],
      "Rwanda": [
        ""
      ],
      "Saint Lucia": [
        ""
      ],
      "El Salvador": [
        ""
      ],
      "Samoa": [
        ""
      ],
      "San Marino": [
        ""
      ],
      "São Tomé and Príncipe": [
        ""
      ],
      "Saudi Arabia": [
        ""
      ],
      "Senegal": [
        ""
      ],
      "Serbia": [
        ""
      ],
      "Seychelles": [
        ""
      ],
      "Sierra Leone": [
        ""
      ],
      "Singapore": [
        ""
      ],
      "Slovakia": [
        ""
      ],
      "Slovenia": [
        ""
      ],
      "Solomon Islands": [
        ""
      ],
      "Somalia": [
        ""
      ],
      "South Africa": [
        ""
      ],
      "South Korea": [
        ""
      ],
      "Spain": [
        ""
      ],
      "Sri Lanka": [
        ""
      ],
      "Sudan": [
        ""
      ],
      "Suriname": [
        ""
      ],
      "Eswatini": [
        ""
      ],
      "Sweden": [
        ""
      ],
      "Switzerland": [
        ""
      ],
      "Syria": [
        ""
      ],
      "Taiwan": [
        ""
      ],
      "Tajikistan": [
        ""
      ],
      "Tanzania": [
        ""
      ],
      "Thailand": [
        ""
      ],
      "Togo": [
        ""
      ],
      "Tonga": [
        ""
      ],
      "Trinidad and Tobago": [
        ""
      ],
      "Tunisia": [
        ""
      ],
      "Turkey": [
        ""
      ],
      "Tuvalu": [
        "Wert"
      ],
      "Uganda": [
        ""
      ],
      "Ukraine": [
        ""
      ],
      "Uruguay": [
        ""
      ],
      "Uzbekistan": [
        ""
      ],
      "Venezuela": [
        ""
      ],
      "Vietnam": [
        ""
      ],
      "Yemen": [
        ""
      ],
      "Zambia": [
        ""
      ],
      "Zimbabwe": [
        ""
      ],
      "Afghan": [
        ""
      ],
      "Albanian": [
        ""
      ],
      "Algerian": [
        ""
      ],
      "American": [
        ""
      ],
      "Andorran": [
        ""
      ],
      "Angolan": [
        ""
      ],
      "Antiguan": [
        ""
      ],
      "Argentinean": [
        ""
      ],
      "Armenian": [
        ""
      ],
      "Australian": [
        ""
      ],
      "Austrian": [
        ""
      ],
      "Azerbaijani": [
        ""
      ],
      "Bahamian": [
        ""
      ],
      "Bahraini": [
        ""
      ],
      "Bangladeshi": [
        ""
      ],
      "Barbadian": [
        ""
      ],
      "Batswana": [
        ""
      ],
      "Belarusian": [
        ""
      ],
      "Belgian": [
        ""
      ],
      "Belizean": [
        ""
      ],
      "Beninese": [
        ""
      ],
      "Bhutanese": [
        ""
      ],
      "Bolivian": [
        ""
      ],
      "Bosnian": [
        ""
      ],
      "Brazilian": [
        ""
      ],
      "British": [
        ""
      ],
      "Bruneian": [
        ""
      ],
      "Bulgarian": [
        ""
      ],
      "Burkinabe": [
        ""
      ],
      "Burmese": [
        ""
      ],
      "Burundian": [
        ""
      ],
      "Cambodian": [
        ""
      ],
      "Cameroonian": [
        ""
      ],
      "Canadian": [
        ""
      ],
      "Cape Verdean": [
        ""
      ],
      "Central African": [
        ""
      ],
      "Chadian": [
        ""
      ],
      "Chilean": [
        ""
      ],
      "Chinese": [
        ""
      ],
      "Colombian": [
        ""
      ],
      "Comoran": [
        ""
      ],
      "Congolese": [
        ""
      ],
      "Costa Rican": [
        ""
      ],
      "Croatian": [
        ""
      ],
      "Cuban": [
        ""
      ],
      "Cypriot": [
        ""
      ],
      "Czech": [
        ""
      ],
      "Danish": [
        ""
      ],
      "Djiboutian": [
        ""
      ],
      "Dominican": [
        ""
      ],
      "Dutch": [
        ""
      ],
      "East Timorese": [
        ""
      ],
      "Ecuadorean": [
        ""
      ],
      "Egyptian": [
        ""
      ],
      "Emirati": [
        ""
      ],
      "Equatorial Guinean": [
        ""
      ],
      "Eritrean": [
        ""
      ],
      "Estonian": [
        ""
      ],
      "Ethiopian": [
        ""
      ],
      "Fijian": [
        ""
      ],
      "Filipino": [
        ""
      ],
      "Finnish": [
        ""
      ],
      "French": [
        ""
      ],
      "Gabonese": [
        ""
      ],
      "Gambian": [
        ""
      ],
      "Georgian": [
        ""
      ],
      "German": [
        ""
      ],
      "Ghanaian": [
        ""
      ],
      "Greek": [
        ""
      ],
      "Grenadian": [
        ""
      ],
      "Guatemalan": [
        ""
      ],
      "Guinea-Bissauan": [
        ""
      ],
      "Guinean": [
        ""
      ],
      "Guyanese": [
        ""
      ],
      "Haitian": [
        ""
      ],
      "Honduran": [
        ""
      ],
      "Hungarian": [
        ""
      ],
      "I-Kiribati": [
        ""
      ],
      "Icelander": [
        ""
      ],
      "Indian": [
        ""
      ],
      "Indonesian": [
        ""
      ],
      "Iranian": [
        ""
      ],
      "Iraqi": [
        ""
      ],
      "Irish": [
        ""
      ],
      "Israeli": [
        ""
      ],
      "Italian": [
        ""
      ],
      "Ivorian": [
        ""
      ],
      "Jamaican": [
        ""
      ],
      "Japanese": [
        ""
      ],
      "Jordanian": [
        ""
      ],
      "Kazakhstani": [
        ""
      ],
      "Kenyan": [
        ""
      ],
      "Kittian and Nevisian": [
        ""
      ],
      "Kuwaiti": [
        ""
      ],
      "Kyrgyz": [
        ""
      ],
      "Laotian": [
        ""
      ],
      "Latvian": [
        ""
      ],
      "Lebanese": [
        ""
      ],
      "Liberian": [
        ""
      ],
      "Libyan": [
        ""
      ],
      "Liechtensteiner": [
        ""
      ],
      "Lithuanian": [
        ""
      ],
      "Luxembourger": [
        ""
      ],
      "Macedonian": [
        ""
      ],
      "Malagasy": [
        ""
      ],
      "Malawian": [
        ""
      ],
      "Malaysian": [
        ""
      ],
      "Maldivian": [
        ""
      ],
      "Malian": [
        ""
      ],
      "Maltese": [
        ""
      ],
      "Marshallese": [
        ""
      ],
      "Mauritanian": [
        ""
      ],
      "Mauritian": [
        ""
      ],
      "Mexican": [
        ""
      ],
      "Micronesian": [
        ""
      ],
      "Moldovan": [
        ""
      ],
      "Monacan": [
        ""
      ],
      "Mongolian": [
        ""
      ],
      "Moroccan": [
        ""
      ],
      "Mosotho": [
        ""
      ],
      "Mozambican": [
        ""
      ],
      "Namibian": [
        ""
      ],
      "Nauruan": [
        ""
      ],
      "Nepalese": [
        ""
      ],
      "New Zealander": [
        ""
      ],
      "Ni-Vanuatu": [
        ""
      ],
      "Nicaraguan": [
        ""
      ],
      "Nigerian": [
        ""
      ],
      "Nigerien": [
        ""
      ],
      "North Korean": [
        ""
      ],
      "Norwegian": [
        ""
      ],
      "Omani": [
        ""
      ],
      "Pakistani": [
        ""
      ],
      "Palauan": [
        ""
      ],
      "Panamanian": [
        ""
      ],
      "Papua New Guinean": [
        ""
      ],
      "Paraguayan": [
        ""
      ],
      "Peruvian": [
        ""
      ],
      "Polish": [
        ""
      ],
      "Portuguese": [
        ""
      ],
      "Qatari": [
        ""
      ],
      "Romanian": [
        ""
      ],
      "Russian": [
        ""
      ],
      "Rwandan": [
        ""
      ],
      "Saint Lucian": [
        ""
      ],
      "Salvadoran": [
        ""
      ],
      "Samoan": [
        ""
      ],
      "San Marinese": [
        ""
      ],
      "Sao Tomean": [
        ""
      ],
      "Saudi": [
        ""
      ],
      "Senegalese": [
        ""
      ],
      "Serbian": [
        ""
      ],
      "Seychellois": [
        ""
      ],
      "Sierra Leonean": [
        ""
      ],
      "Singaporean": [
        ""
      ],
      "Slovakian": [
        ""
      ],
      "Slovenian": [
        ""
      ],
      "Solomon Islander": [
        ""
      ],
      "Somali": [
        ""
      ],
      "South African": [
        ""
      ],
      "South Korean": [
        ""
      ],
      "Spanish": [
        ""
      ],
      "Sri Lankan": [
        ""
      ],
      "Sudanese": [
        ""
      ],
      "Surinamer": [
        ""
      ],
      "Swazi": [
        ""
      ],
      "Swedish": [
        ""
      ],
      "Swiss": [
        ""
      ],
      "Syrian": [
        ""
      ],
      "Taiwanese": [
        ""
      ],
      "Tajik": [
        ""
      ],
      "Tanzanian": [
        ""
      ],
      "Thai": [
        ""
      ],
      "Togolese": [
        ""
      ],
      "Tongan": [
        ""
      ],
      "Trinidadian or Tobagonian": [
        ""
      ],
      "Tunisian": [
        ""
      ],
      "Turkish": [
        ""
      ],
      "Tuvaluan": [
        "Wert"
      ],
      "Ugandan": [
        ""
      ],
      "Ukrainian": [
        ""
      ],
      "Uruguayan": [
        ""
      ],
      "Uzbekistani": [
        ""
      ],
      "Venezuelan": [
        ""
      ],
      "Vietnamese": [
        ""
      ],
      "Yemenite": [
        ""
      ],
      "Zambian": [
        ""
      ],
      "Zimbabwean": [
        ""
      ],
      "🇦🇫": [
        ""
      ],
      "🇦🇱": [
        ""
      ],
      "🇩🇿": [
        ""
      ],
      "🇺🇸": [
        ""
      ],
      "🇦🇩": [
        ""
      ],
      "🇦🇴": [
        ""
      ],
      "🇦🇬": [
        ""
      ],
      "🇦🇷": [
        ""
      ],
      "🇦🇲": [
        ""
      ],
      "🇦🇺": [
        ""
      ],
      "🇦🇹": [
        ""
      ],
      "🇦🇿": [
        ""
      ],
      "🇧🇸": [
        ""
      ],
      "🇧🇭": [
        ""
      ],
      "🇧🇩": [
        ""
      ],
      "🇧🇧": [
        ""
      ],
      "🇧🇼": [
        ""
      ],
      "🇧🇾": [
        ""
      ],
      "🇧🇪": [
        ""
      ],
      "🇧🇿": [
        ""
      ],
      "🇧🇯": [
        ""
      ],
      "🇧🇹": [
        ""
      ],
      "🇧🇴": [
        ""
      ],
      "🇧🇦": [
        ""
      ],
      "🇧🇷": [
        ""
      ],
      "🇬🇧": [
        ""
      ],
      "🇧🇳": [
        ""
      ],
      "🇧🇬": [
        ""
      ],
      "🇧🇫": [
        ""
      ],
      "🇲🇲": [
        ""
      ],
      "🇧🇮": [
        ""
      ],
      "🇰🇭": [
        ""
      ],
      "🇨🇲": [
        ""
      ],
      "🇨🇦": [
        ""
      ],
      "🇨🇻": [
        ""
      ],
      "🇨🇫": [
        ""
      ],
      "🇨🇱": [
        ""
      ],
      "🇨🇳": [
        ""
      ],
      "🇨🇴": [
        ""
      ],
      "🇨🇩": [
        ""
      ],
      "🇨🇬": [
        ""
      ],
      "🇨🇷": [
        ""
      ],
      "🇨🇺": [
        ""
      ],
      "🇨🇾": [
        ""
      ],
      "🇨🇿": [
        ""
      ],
      "🇩🇰": [
        ""
      ],
      "🇩🇯": [
        ""
      ],
      "🇩🇴": [
        ""
      ],
      "🇳🇱": [
        ""
      ],
      "🇹🇱": [
        ""
      ],
      "🇪🇨": [
        ""
      ],
      "🇪🇬": [
        ""
      ],
      "🇦🇪": [
        ""
      ],
      "🇬🇶": [
        ""
      ],
      "🇪🇷": [
        ""
      ],
      "🇪🇪": [
        ""
      ],
      "🇪🇹": [
        ""
      ],
      "🇫🇯": [
        ""
      ],
      "🇵🇭": [
        ""
      ],
      "🇫🇮": [
        ""
      ],
      "🇫🇷": [
        ""
      ],
      "🇬🇦": [
        ""
      ],
      "🇬🇲": [
        ""
      ],
      "🇬🇪": [
        ""
      ],
      "UAE Dirham": [
        ""
      ],
      "Afghani": [
        ""
      ],
      "Lek": [
        ""
      ],
      "Armenian Dram": [
        ""
      ],
      "Netherlands Antillean Guilder": [
        ""
      ],
      "Kwanza": [
        ""
      ],
      "Argentine Peso": [
        ""
      ],
      "Australian Dollar": [
        ""
      ],
      "Aruban Florin": [
        ""
      ],
      "Azerbaijan Manat": [
        ""
      ],
      "Convertible Mark": [
        ""
      ],
      "Barbados Dollar": [
        ""
      ],
      "Taka": [
        ""
      ],
      "Bulgarian Lev": [
        ""
      ],
      "Bahraini Dinar": [
        ""
      ],
      "Burundi Franc": [
        ""
      ],
      "Bermudian Dollar": [
        ""
      ],
      "Brunei Dollar": [
        ""
      ],
      "Boliviano": [
        ""
      ],
      "Mvdol": [
        ""
      ],
      "Brazilian Real": [
        ""
      ],
      "Bahamian Dollar": [
        ""
      ],
      "Ngultrum": [
        ""
      ],
      "Pula": [
        ""
      ],
      "Belarusian Ruble": [
        ""
      ],
      "Belize Dollar": [
        ""
      ],
      "Canadian Dollar": [
        ""
      ],
      "Congolese Franc": [
        ""
      ],
      "WIR Euro": [
        ""
      ],
      "Swiss Franc": [
        ""
      ],
      "WIR Franc": [
        ""
      ],
      "Unidad de Fomento": [
        ""
      ],
      "Chilean Peso": [
        ""
      ],
      "Yuan Renminbi": [
        ""
      ],
      "Colombian Peso": [
        ""
      ],
      "Unidad de Valor Real": [
        ""
      ],
      "Costa Rican Colon": [
        ""
      ],
      "Cuban Peso": [
        ""
      ],
      "Cabo Verde Escudo": [
        ""
      ],
      "Czech Koruna": [
        ""
      ],
      "Djibouti Franc": [
        ""
      ],
      "Danish Krone": [
        ""
      ],
      "Dominican Peso": [
        ""
      ],
      "Algerian Dinar": [
        ""
      ],
      "Egyptian Pound": [
        ""
      ],
      "Nakfa": [
        ""
      ],
      "Ethiopian Birr": [
        ""
      ],
      "Euro": [
        ""
      ],
      "Fiji Dollar": [
        ""
      ],
      "Falkland Islands Pound": [
        ""
      ],
      "Pound Sterling": [
        ""
      ],
      "Lari": [
        ""
      ],
      "Ghana Cedi": [
        ""
      ],
      "Gibraltar Pound": [
        ""
      ],
      "Dalasi": [
        ""
      ],
      "Guinean Franc": [
        ""
      ],
      "Quetzal": [
        ""
      ],
      "Guyana Dollar": [
        ""
      ],
      "Hong Kong Dollar": [
        ""
      ],
      "Lempira": [
        ""
      ],
      "Gourde": [
        ""
      ],
      "Forint": [
        ""
      ],
      "Rupiah": [
        ""
      ],
      "New Israeli Sheqel": [
        ""
      ],
      "Indian Rupee": [
        ""
      ],
      "Iraqi Dinar": [
        ""
      ],
      "Iranian Rial": [
        ""
      ],
      "Iceland Krona": [
        ""
      ],
      "Jamaican Dollar": [
        ""
      ],
      "Jordanian Dinar": [
        ""
      ],
      "Yen": [
        ""
      ],
      "Kenyan Shilling": [
        ""
      ],
      "Som": [
        ""
      ],
      "Riel": [
        ""
      ],
      "Comorian Franc ": [
        ""
      ],
      "North Korean Won": [
        ""
      ],
      "Won": [
        ""
      ],
      "Kuwaiti Dinar": [
        ""
      ],
      "Cayman Islands Dollar": [
        ""
      ],
      "Tenge": [
        ""
      ],
      "Lao Kip": [
        ""
      ],
      "Lebanese Pound": [
        ""
      ],
      "Sri Lanka Rupee": [
        ""
      ],
      "Liberian Dollar": [
        ""
      ],
      "Loti": [
        ""
      ],
      "Libyan Dinar": [
        ""
      ],
      "Moroccan Dirham": [
        ""
      ],
      "Moldovan Leu": [
        ""
      ],
      "Malagasy Ariary": [
        ""
      ],
      "Denar": [
        ""
      ],
      "Kyat": [
        ""
      ],
      "Tugrik": [
        ""
      ],
      "Pataca": [
        ""
      ],
      "Ouguiya": [
        ""
      ],
      "Mauritius Rupee": [
        ""
      ],
      "Rufiyaa": [
        ""
      ],
      "Malawi Kwacha": [
        ""
      ],
      "Mexican Peso": [
        ""
      ],
      "Mexican Unidad de Inversion (UDI)": [
        ""
      ],
      "Malaysian Ringgit": [
        ""
      ],
      "Mozambique Metical": [
        ""
      ],
      "Namibia Dollar": [
        ""
      ],
      "Naira": [
        ""
      ],
      "Cordoba Oro": [
        ""
      ],
      "Norwegian Krone": [
        ""
      ],
      "Nepalese Rupee": [
        ""
      ],
      "New Zealand Dollar": [
        ""
      ],
      "Rial Omani": [
        ""
      ],
      "Balboa": [
        ""
      ],
      "Sol": [
        "alt"
      ],
      "Kina": [
        ""
      ],
      "Philippine Peso": [
        ""
      ],
      "Pakistan Rupee": [
        ""
      ],
      "Zloty": [
        ""
      ],
      "Guarani": [
        ""
      ],
      "Qatari Rial": [
        ""
      ],
      "Romanian Leu": [
        ""
      ],
      "Serbian Dinar": [
        ""
      ],
      "Russian Ruble": [
        ""
      ],
      "Rwanda Franc": [
        ""
      ],
      "Saudi Riyal": [
        ""
      ],
      "Solomon Islands Dollar": [
        ""
      ],
      "Seychelles Rupee": [
        ""
      ],
      "Sudanese Pound": [
        ""
      ],
      "Swedish Krona": [
        ""
      ],
      "Singapore Dollar": [
        ""
      ],
      "Saint Helena Pound": [
        ""
      ],
      "Leone": [
        ""
      ],
      "Somali Shilling": [
        ""
      ],
      "Surinam Dollar": [
        ""
      ],
      "South Sudanese Pound": [
        ""
      ],
      "Dobra": [
        ""
      ],
      "El Salvador Colon": [
        ""
      ],
      "Syrian Pound": [
        ""
      ],
      "Lilangeni": [
        ""
      ],
      "Baht": [
        ""
      ],
      "Somoni": [
        ""
      ],
      "Turkmenistan New Manat": [
        ""
      ],
      "Tunisian Dinar": [
        ""
      ],
      "Pa’anga": [
        ""
      ],
      "Turkish Lira": [
        ""
      ],
      "Trinidad and Tobago Dollar": [
        ""
      ],
      "New Taiwan Dollar": [
        ""
      ],
      "Tanzanian Shilling": [
        ""
      ],
      "Hryvnia": [
        ""
      ],
      "Uganda Shilling": [
        ""
      ],
      "US Dollar": [
        ""
      ],
      "US Dollar (Next day)": [
        ""
      ],
      "Uruguay Peso en Unidades Indexadas (UI)": [
        ""
      ],
      "Peso Uruguayo": [
        ""
      ],
      "Unidad Previsional": [
        ""
      ],
      "Uzbekistan Sum": [
        ""
      ],
      "Bolívar Soberano": [
        ""
      ],
      "Dong": [
        ""
      ],
      "Vatu": [
        ""
      ],
      "Tala": [
        ""
      ],
      "CFA Franc BEAC": [
        ""
      ],
      "East Caribbean Dollar": [
        ""
      ],
      "CFA Franc BCEAO": [
        ""
      ],
      "CFP Franc": [
        ""
      ],
      "Sucre": [
        ""
      ],
      "Yemeni Rial": [
        ""
      ],
      "Rand": [
        ""
      ],
      "Zambian Kwacha": [
        ""
      ],
      "Zimbabwe Gold": [
        ""
      ],
      "Afar": [
        ""
      ],
      "Abkhazian": [
        ""
      ],
      "Achinese": [
        ""
      ],
      "Acoli": [
        ""
      ],
      "Adangme": [
        ""
      ],
      "Adyghe": [
        ""
      ],
      "Avestan": [
        ""
      ],
      "Tunisian Arabic": [
        ""
      ],
      "Afrikaans": [
        ""
      ],
      "Afrihili": [
        ""
      ],
      "Aghem": [
        ""
      ],
      "Ainu": [
        ""
      ],
      "Akan": [
        ""
      ],
      "Akkadian": [
        ""
      ],
      "Alabama": [
        ""
      ],
      "Aleut": [
        ""
      ],
      "Gheg Albanian": [
        ""
      ],
      "Southern Altai": [
        ""
      ],
      "Amarik": [
        ""
      ],
      "Aragonese": [
        ""
      ],
      "Old English": [
        ""
      ],
      "Angika": [
        ""
      ],
      "Modern Standard Arabic": [
        ""
      ],
      "Arabik": [
        ""
      ],
      "Aramaic": [
        ""
      ],
      "Mapuche": [
        ""
      ],
      "Araona": [
        ""
      ],
      "Arapaho": [
        ""
      ],
      "Algerian Arabic": [
        ""
      ],
      "Arawak": [
        ""
      ],
      "Moroccan Arabic": [
        ""
      ],
      "Egyptian Arabic": [
        ""
      ],
      "Asu": [
        ""
      ],
      "Assamese": [
        ""
      ],
      "American Sign Language": [
        ""
      ],
      "Asturian": [
        ""
      ],
      "Avaric": [
        ""
      ],
      "Kotava": [
        ""
      ],
      "Awadhi": [
        ""
      ],
      "Aymara": [
        ""
      ],
      "South Azerbaijani": [
        ""
      ],
      "Bashkir": [
        ""
      ],
      "Baluchi": [
        ""
      ],
      "Balinese": [
        ""
      ],
      "Bavarian": [
        ""
      ],
      "Basaa": [
        ""
      ],
      "Bamun": [
        ""
      ],
      "Batak Toba": [
        ""
      ],
      "Ghomala": [
        ""
      ],
      "Belarus kasa": [
        ""
      ],
      "Beja": [
        ""
      ],
      "Bemba": [
        ""
      ],
      "Betawi": [
        ""
      ],
      "Bena": [
        ""
      ],
      "Bafut": [
        ""
      ],
      "Badaga": [
        ""
      ],
      "Bɔlgeria kasa": [
        ""
      ],
      "Bhojpuri": [
        ""
      ],
      "Bislama": [
        ""
      ],
      "Bikol": [
        ""
      ],
      "Bini": [
        ""
      ],
      "Banjar": [
        ""
      ],
      "Kom": [
        ""
      ],
      "Siksika": [
        ""
      ],
      "Bambara": [
        ""
      ],
      "Bengali kasa": [
        ""
      ],
      "Tibetan": [
        ""
      ],
      "Bishnupriya": [
        ""
      ],
      "Bakhtiari": [
        ""
      ],
      "Braj": [
        ""
      ],
      "Breton": [
        ""
      ],
      "Brahui": [
        ""
      ],
      "Bodo": [
        ""
      ],
      "Akoose": [
        ""
      ],
      "Buriat": [
        ""
      ],
      "Buginese": [
        ""
      ],
      "Bulu": [
        ""
      ],
      "Blin": [
        ""
      ],
      "Medumba": [
        ""
      ],
      "Catalan": [
        ""
      ],
      "Caddo": [
        ""
      ],
      "Carib": [
        ""
      ],
      "Cayuga": [
        ""
      ],
      "Atsam": [
        ""
      ],
      "Cebuano": [
        ""
      ],
      "Chechen": [
        ""
      ],
      "Chiga": [
        ""
      ],
      "Chibcha": [
        ""
      ],
      "Chamorro": [
        ""
      ],
      "Chagatai": [
        ""
      ],
      "Chuukese": [
        ""
      ],
      "Mari": [
        ""
      ],
      "Chinook Jargon": [
        ""
      ],
      "Choctaw": [
        ""
      ],
      "Chipewyan": [
        ""
      ],
      "Cherokee": [
        ""
      ],
      "Cheyenne": [
        ""
      ],
      "Central Kurdish": [
        ""
      ],
      "Corsican": [
        ""
      ],
      "Coptic": [
        ""
      ],
      "Capiznon": [
        ""
      ],
      "Cree": [
        ""
      ],
      "Crimean Turkish": [
        ""
      ],
      "Kashubian": [
        ""
      ],
      "Kyɛk kasa": [
        ""
      ],
      "Church Slavic": [
        ""
      ],
      "Chuvash": [
        ""
      ],
      "Welsh": [
        ""
      ],
      "Dakota": [
        ""
      ],
      "Dargwa": [
        ""
      ],
      "Taita": [
        ""
      ],
      "Austrian German": [
        ""
      ],
      "Swiss High German": [
        ""
      ],
      "Gyaaman": [
        ""
      ],
      "Delaware": [
        ""
      ],
      "Slave": [
        ""
      ],
      "Dogrib": [
        ""
      ],
      "Dinka": [
        ""
      ],
      "Zarma": [
        ""
      ],
      "Dogri": [
        ""
      ],
      "Lower Sorbian": [
        ""
      ],
      "Central Dusun": [
        ""
      ],
      "Duala": [
        ""
      ],
      "Middle Dutch": [
        ""
      ],
      "Divehi": [
        ""
      ],
      "Jola-Fonyi": [
        ""
      ],
      "Dyula": [
        ""
      ],
      "Dzongkha": [
        ""
      ],
      "Dazaga": [
        ""
      ],
      "Embu": [
        ""
      ],
      "Ewe": [
        ""
      ],
      "Efik": [
        ""
      ],
      "Emilian": [
        ""
      ],
      "Ancient Egyptian": [
        ""
      ],
      "Ekajuk": [
        ""
      ],
      "Greek kasa": [
        ""
      ],
      "Elamite": [
        ""
      ],
      "Australian English": [
        ""
      ],
      "Borɔfo": [
        ""
      ],
      "Canadian English": [
        ""
      ],
      "British English": [
        ""
      ],
      "Middle English": [
        ""
      ],
      "American English": [
        ""
      ],
      "Esperanto": [
        ""
      ],
      "Latin American Spanish": [
        ""
      ],
      "European Spanish": [
        ""
      ],
      "Mexican Spanish": [
        ""
      ],
      "Spain kasa": [
        ""
      ],
      "Central Yupik": [
        ""
      ],
      "Basque": [
        ""
      ],
      "Ewondo": [
        ""
      ],
      "Extremaduran": [
        ""
      ],
      "Fang": [
        ""
      ],
      "Pɛɛhyia kasa": [
        ""
      ],
      "Fanti": [
        ""
      ],
      "Fulah": [
        ""
      ],
      "Tornedalen Finnish": [
        ""
      ],
      "Faroese": [
        ""
      ],
      "Fon": [
        ""
      ],
      "Canadian French": [
        ""
      ],
      "Cajun French": [
        "Währung"
      ],
      "Swiss French": [
        ""
      ],
      "Frɛnkye": [
        ""
      ],
      "Middle French": [
        ""
      ],
      "Old French": [
        ""
      ],
      "Arpitan": [
        ""
      ],
      "Northern Frisian": [
        ""
      ],
      "Eastern Frisian": [
        ""
      ],
      "Friulian": [
        ""
      ],
      "Western Frisian": [
        ""
      ],
      "Ga": [
        ""
      ],
      "Gagauz": [
        ""
      ],
      "Gan Chinese": [
        ""
      ],
      "Gayo": [
        ""
      ],
      "Gbaya": [
        ""
      ],
      "Zoroastrian Dari": [
        ""
      ],
      "Scottish Gaelic": [
        ""
      ],
      "Geez": [
        ""
      ],
      "Gilbertese": [
        ""
      ],
      "Galician": [
        ""
      ],
      "Gilaki": [
        ""
      ],
      "Middle High German": [
        ""
      ],
      "Old High German": [
        ""
      ],
      "Goan Konkani": [
        ""
      ],
      "Gondi": [
        ""
      ],
      "Gorontalo": [
        ""
      ],
      "Gothic": [
        ""
      ],
      "Grebo": [
        ""
      ],
      "Ancient Greek": [
        ""
      ],
      "Swiss German": [
        ""
      ],
      "Wayuu": [
        ""
      ],
      "Gujarati": [
        ""
      ],
      "Frafra": [
        ""
      ],
      "Gusii": [
        ""
      ],
      "Manx": [
        ""
      ],
      "Gwichʼin": [
        ""
      ],
      "Hausa": [
        ""
      ],
      "Haida": [
        ""
      ],
      "Hakka Chinese": [
        ""
      ],
      "Hawaiian": [
        ""
      ],
      "Hebrew": [
        ""
      ],
      "Fiji Hindi": [
        ""
      ],
      "Hindi": [
        ""
      ],
      "Hiligaynon": [
        ""
      ],
      "Hittite": [
        ""
      ],
      "Hmong": [
        ""
      ],
      "Hiri Motu": [
        ""
      ],
      "Upper Sorbian": [
        ""
      ],
      "Xiang Chinese": [
        ""
      ],
      "Hangri kasa": [
        ""
      ],
      "Hupa": [
        ""
      ],
      "Herero": [
        ""
      ],
      "Interlingua": [
        ""
      ],
      "Iban": [
        ""
      ],
      "Ibibio": [
        ""
      ],
      "Indonihyia kasa": [
        ""
      ],
      "Interlingue": [
        ""
      ],
      "Igbo": [
        ""
      ],
      "Sichuan Yi": [
        ""
      ],
      "Inupiaq": [
        ""
      ],
      "Iloko": [
        ""
      ],
      "Ingush": [
        ""
      ],
      "Ido": [
        ""
      ],
      "Icelandic": [
        ""
      ],
      "Italy kasa": [
        ""
      ],
      "Inuktitut": [
        ""
      ],
      "Ingrian": [
        ""
      ],
      "Gyapan kasa": [
        ""
      ],
      "Jamaican Creole English": [
        ""
      ],
      "Lojban": [
        ""
      ],
      "Ngomba": [
        ""
      ],
      "Machame": [
        ""
      ],
      "Judeo-Persian": [
        ""
      ],
      "Judeo-Arabic": [
        ""
      ],
      "Jutish": [
        ""
      ],
      "Gyabanis kasa": [
        ""
      ],
      "Kara-Kalpak": [
        ""
      ],
      "Kabyle": [
        ""
      ],
      "Kachin": [
        ""
      ],
      "Jju": [
        ""
      ],
      "Kamba": [
        ""
      ],
      "Kawi": [
        ""
      ],
      "Kabardian": [
        ""
      ],
      "Kanembu": [
        ""
      ],
      "Tyap": [
        ""
      ],
      "Makonde": [
        ""
      ],
      "Kabuverdianu": [
        ""
      ],
      "Kenyang": [
        ""
      ],
      "Koro": [
        ""
      ],
      "Kongo": [
        ""
      ],
      "Kaingang": [
        ""
      ],
      "Khasi": [
        ""
      ],
      "Khotanese": [
        ""
      ],
      "Koyra Chiini": [
        ""
      ],
      "Khowar": [
        ""
      ],
      "Kikuyu": [
        ""
      ],
      "Kirmanjki": [
        ""
      ],
      "Kuanyama": [
        ""
      ],
      "Kako": [
        ""
      ],
      "Kazakh": [
        ""
      ],
      "Kalaallisut": [
        ""
      ],
      "Kalenjin": [
        ""
      ],
      "Kimbundu": [
        ""
      ],
      "Kambodia kasa": [
        ""
      ],
      "Kannada": [
        ""
      ],
      "Komi-Permyak": [
        ""
      ],
      "Konkani": [
        ""
      ],
      "Korea kasa": [
        ""
      ],
      "Kosraean": [
        ""
      ],
      "Kpelle": [
        ""
      ],
      "Karachay-Balkar": [
        ""
      ],
      "Krio": [
        ""
      ],
      "Kinaray-a": [
        ""
      ],
      "Kanuri": [
        ""
      ],
      "Karelian": [
        ""
      ],
      "Kurukh": [
        ""
      ],
      "Shambala": [
        ""
      ],
      "Bafia": [
        ""
      ],
      "Colognian": [
        ""
      ],
      "Kashmiri": [
        ""
      ],
      "Kurdish": [
        ""
      ],
      "Kumyk": [
        ""
      ],
      "Kutenai": [
        ""
      ],
      "Komi": [
        ""
      ],
      "Cornish": [
        ""
      ],
      "Ladino": [
        "Lädt Daten"
      ],
      "Langi": [
        ""
      ],
      "Lahnda": [
        ""
      ],
      "Latin": [
        ""
      ],
      "Lamba": [
        ""
      ],
      "Luxembourgish": [
        ""
      ],
      "Lezghian": [
        ""
      ],
      "Lingua Franca Nova": [
        ""
      ],
      "Ganda": [
        ""
      ],
      "Ligurian": [
        ""
      ],
      "Limburgish": [
        ""
      ],
      "Livonian": [
        ""
      ],
      "Lakota": [
        ""
      ],
      "Lombard": [
        ""
      ],
      "Lingala": [
        ""
      ],
      "Lao": [
        ""
      ],
      "Mongo": [
        ""
      ],
      "Lozi": [
        ""
      ],
      "Latgalian": [
        ""
      ],
      "Luba-Lulua": [
        ""
      ],
      "Luiseno": [
        ""
      ],
      "Luba-Katanga": [
        ""
      ],
      "Lunda": [
        ""
      ],
      "Luo": [
        ""
      ],
      "Mizo": [
        ""
      ],
      "Luyia": [
        ""
      ],
      "Literary Chinese": [
        ""
      ],
      "Laz": [
        ""
      ],
      "Madurese": [
        "zurücksetzen"
      ],
      "Mafa": [
        ""
      ],
      "Magahi": [
        ""
      ],
      "Maithili": [
        ""
      ],
      "Makasar": [
        ""
      ],
      "Mandingo": [
        "Lädt Daten"
      ],
      "Masai": [
        ""
      ],
      "Maba": [
        ""
      ],
      "Moksha": [
        ""
      ],
      "Mandar": [
        ""
      ],
      "Mende": [
        ""
      ],
      "Meru": [
        ""
      ],
      "Morisyen": [
        ""
      ],
      "Middle Irish": [
        ""
      ],
      "Makhuwa-Meetto": [
        ""
      ],
      "Metaʼ": [
        ""
      ],
      "Micmac": [
        ""
      ],
      "Maori": [
        ""
      ],
      "Minangkabau": [
        ""
      ],
      "Malayalam": [
        ""
      ],
      "Manchu": [
        ""
      ],
      "Manipuri": [
        ""
      ],
      "Mohawk": [
        ""
      ],
      "Mossi": [
        ""
      ],
      "Western Mari": [
        ""
      ],
      "Marathi": [
        ""
      ],
      "Malay kasa": [
        ""
      ],
      "Mundang": [
        ""
      ],
      "Multiple Languages": [
        ""
      ],
      "Creek": [
        ""
      ],
      "Mirandese": [
        ""
      ],
      "Marwari": [
        ""
      ],
      "Mentawai": [
        ""
      ],
      "Bɛɛmis kasa": [
        ""
      ],
      "Myene": [
        ""
      ],
      "Erzya": [
        ""
      ],
      "Mazanderani": [
        ""
      ],
      "Min Nan Chinese": [
        ""
      ],
      "Neapolitan": [
        ""
      ],
      "Nama": [
        ""
      ],
      "Norwegian Bokmål": [
        ""
      ],
      "North Ndebele": [
        ""
      ],
      "Low German": [
        ""
      ],
      "Nɛpal kasa": [
        ""
      ],
      "Newari": [
        ""
      ],
      "Ndonga": [
        ""
      ],
      "Nias": [
        ""
      ],
      "Niuean": [
        ""
      ],
      "Ao Naga": [
        ""
      ],
      "Flemish": [
        ""
      ],
      "Dɛɛkye": [
        ""
      ],
      "Kwasio": [
        ""
      ],
      "Ngiemboon": [
        ""
      ],
      "Norwegian Nynorsk": [
        ""
      ],
      "Nogai": [
        ""
      ],
      "Old Norse": [
        ""
      ],
      "Novial": [
        ""
      ],
      "NʼKo": [
        ""
      ],
      "South Ndebele": [
        ""
      ],
      "Northern Sotho": [
        ""
      ],
      "Nuer": [
        ""
      ],
      "Navajo": [
        ""
      ],
      "Classical Newari": [
        ""
      ],
      "Nyamwezi": [
        ""
      ],
      "Nyankole": [
        ""
      ],
      "Nyanja": [
        ""
      ],
      "Nyoro": [
        ""
      ],
      "Nzima": [
        ""
      ],
      "Occitan": [
        ""
      ],
      "Ojibwa": [
        ""
      ],
      "Oromo": [
        ""
      ],
      "Oriya": [
        ""
      ],
      "Osage": [
        ""
      ],
      "Ossetic": [
        ""
      ],
      "Ottoman Turkish": [
        ""
      ],
      "Pangasinan": [
        ""
      ],
      "Pahlavi": [
        ""
      ],
      "Pampanga": [
        ""
      ],
      "Papiamento": [
        "Zahlung"
      ],
      "Pungyabi kasa": [
        ""
      ],
      "Picard": [
        ""
      ],
      "Pennsylvania German": [
        ""
      ],
      "Plautdietsch": [
        ""
      ],
      "Old Persian": [
        ""
      ],
      "Palatine German": [
        ""
      ],
      "Phoenician": [
        ""
      ],
      "Pali": [
        ""
      ],
      "Pɔland kasa": [
        ""
      ],
      "Piedmontese": [
        ""
      ],
      "Pontic": [
        ""
      ],
      "Pohnpeian": [
        ""
      ],
      "Prussian": [
        ""
      ],
      "Old Provençal": [
        ""
      ],
      "Pashto": [
        ""
      ],
      "Brazilian Portuguese": [
        ""
      ],
      "Pɔɔtugal kasa": [
        ""
      ],
      "European Portuguese": [
        ""
      ],
      "Kʼicheʼ": [
        ""
      ],
      "Chimborazo Highland Quichua": [
        ""
      ],
      "Quechua": [
        ""
      ],
      "Rajasthani": [
        ""
      ],
      "Rapanui": [
        ""
      ],
      "Rarotongan": [
        ""
      ],
      "Romagnol": [
        ""
      ],
      "Riffian": [
        ""
      ],
      "Romansh": [
        ""
      ],
      "Rundi": [
        ""
      ],
      "Rombo": [
        ""
      ],
      "Moldavian": [
        ""
      ],
      "Romany": [
        ""
      ],
      "Root": [
        ""
      ],
      "Romenia kasa": [
        ""
      ],
      "Rotuman": [
        ""
      ],
      "Rusyn": [
        ""
      ],
      "Roviana": [
        ""
      ],
      "Aromanian": [
        ""
      ],
      "Rahyia kasa": [
        ""
      ],
      "Rwa": [
        ""
      ],
      "Rewanda kasa": [
        ""
      ],
      "Sandawe": [
        ""
      ],
      "Sakha": [
        ""
      ],
      "Samaritan Aramaic": [
        ""
      ],
      "Samburu": [
        ""
      ],
      "Sanskrit": [
        ""
      ],
      "Sasak": [
        ""
      ],
      "Santali": [
        ""
      ],
      "Saurashtra": [
        ""
      ],
      "Ngambay": [
        ""
      ],
      "Sangu": [
        ""
      ],
      "Sicilian": [
        ""
      ],
      "Scots": [
        ""
      ],
      "Sardinian": [
        ""
      ],
      "Sassarese Sardinian": [
        ""
      ],
      "Sindhi": [
        ""
      ],
      "Seneca": [
        ""
      ],
      "Sena": [
        ""
      ],
      "Seri": [
        ""
      ],
      "Selkup": [
        ""
      ],
      "Northern Sami": [
        ""
      ],
      "Koyraboro Senni": [
        ""
      ],
      "Old Irish": [
        ""
      ],
      "Sango": [
        ""
      ],
      "Samogitian": [
        ""
      ],
      "Tachelhit": [
        ""
      ],
      "Shan": [
        ""
      ],
      "Serbo-Croatian": [
        ""
      ],
      "Chadian Arabic": [
        ""
      ],
      "Sidamo": [
        ""
      ],
      "Sinhala": [
        ""
      ],
      "Slovak": [
        ""
      ],
      "Lower Silesian": [
        ""
      ],
      "Selayar": [
        ""
      ],
      "Southern Sami": [
        ""
      ],
      "Lule Sami": [
        ""
      ],
      "Inari Sami": [
        ""
      ],
      "Skolt Sami": [
        ""
      ],
      "Soninke": [
        ""
      ],
      "Shona": [
        ""
      ],
      "Sogdien": [
        ""
      ],
      "Somalia kasa": [
        ""
      ],
      "Sranan Tongo": [
        ""
      ],
      "Serer": [
        ""
      ],
      "Swati": [
        ""
      ],
      "Saho": [
        ""
      ],
      "Saterland Frisian": [
        ""
      ],
      "Southern Sotho": [
        ""
      ],
      "Sukuma": [
        ""
      ],
      "Susu": [
        ""
      ],
      "Sundanese": [
        ""
      ],
      "Sumerian": [
        ""
      ],
      "Sweden kasa": [
        ""
      ],
      "Comorian": [
        ""
      ],
      "Congo Swahili": [
        ""
      ],
      "Swahili": [
        ""
      ],
      "Classical Syriac": [
        ""
      ],
      "Syriac": [
        ""
      ],
      "Silesian": [
        ""
      ],
      "Tamil kasa": [
        ""
      ],
      "Tulu": [
        ""
      ],
      "Timne": [
        ""
      ],
      "Teso": [
        ""
      ],
      "Tereno": [
        ""
      ],
      "Telugu": [
        ""
      ],
      "Tetum": [
        ""
      ],
      "Taeland kasa": [
        ""
      ],
      "Tigre": [
        ""
      ],
      "Tigrinya": [
        ""
      ],
      "Tiv": [
        ""
      ],
      "Tokelau": [
        ""
      ],
      "Tsakhur": [
        ""
      ],
      "Turkmen": [
        ""
      ],
      "Klingon": [
        ""
      ],
      "Tlingit": [
        ""
      ],
      "Tagalog": [
        ""
      ],
      "Talysh": [
        ""
      ],
      "Tamashek": [
        ""
      ],
      "Tswana": [
        ""
      ],
      "Nyasa Tonga": [
        ""
      ],
      "Tok Pisin": [
        ""
      ],
      "Tɛɛki kasa": [
        ""
      ],
      "Turoyo": [
        ""
      ],
      "Taroko": [
        ""
      ],
      "Tsakonian": [
        ""
      ],
      "Tsimshian": [
        ""
      ],
      "Tsonga": [
        ""
      ],
      "Tatar": [
        ""
      ],
      "Muslim Tat": [
        ""
      ],
      "Tumbuka": [
        ""
      ],
      "Tasawaq": [
        ""
      ],
      "Twi": [
        ""
      ],
      "Tahitian": [
        ""
      ],
      "Tuvinian": [
        ""
      ],
      "Central Atlas Tamazight": [
        ""
      ],
      "Udmurt": [
        ""
      ],
      "Ugaritic": [
        ""
      ],
      "Uyghur": [
        ""
      ],
      "Ukren kasa": [
        ""
      ],
      "Umbundu": [
        ""
      ],
      "Unknown Language": [
        ""
      ],
      "Urdu kasa": [
        ""
      ],
      "Uzbek": [
        ""
      ],
      "Vai": [
        ""
      ],
      "Venetian": [
        ""
      ],
      "Veps": [
        ""
      ],
      "Venda": [
        ""
      ],
      "Viɛtnam kasa": [
        ""
      ],
      "West Flemish": [
        ""
      ],
      "Main-Franconian": [
        ""
      ],
      "Votic": [
        ""
      ],
      "Volapük": [
        ""
      ],
      "Võro": [
        ""
      ],
      "Vunjo": [
        ""
      ],
      "Walser": [
        ""
      ],
      "Wolaytta": [
        ""
      ],
      "Waray": [
        ""
      ],
      "Washo": [
        ""
      ],
      "Walloon": [
        ""
      ],
      "Warlpiri": [
        ""
      ],
      "Wolof": [
        ""
      ],
      "Wu Chinese": [
        ""
      ],
      "Kalmyk": [
        ""
      ],
      "Xhosa": [
        ""
      ],
      "Mingrelian": [
        ""
      ],
      "Soga": [
        ""
      ],
      "Yao": [
        ""
      ],
      "Yapese": [
        ""
      ],
      "Yangben": [
        ""
      ],
      "Yemba": [
        ""
      ],
      "Yiddish": [
        ""
      ],
      "Yoruba": [
        ""
      ],
      "Nheengatu": [
        ""
      ],
      "Cantonese": [
        ""
      ],
      "Zapotec": [
        ""
      ],
      "Zhuang": [
        ""
      ],
      "Blissymbols": [
        ""
      ],
      "Zeelandic": [
        ""
      ],
      "Zenaga": [
        ""
      ],
      "Standard Moroccan Tamazight": [
        ""
      ],
      "Simplified Chinese": [
        ""
      ],
      "Traditional Chinese": [
        ""
      ],
      "Kyaena kasa": [
        ""
      ],
      "Zuni": [
        ""
      ],
      "Zulu": [
        ""
      ],
      "No linguistic content": [
        ""
      ],
      "Zaza": [
        ""
      ],
      "Could not load user attention request": [
        "Konnte die Gebühren des Zahlungsdiensts nicht laden"
      ],
      "No notification left": [
        ""
      ],
      "Ignore": [
        ""
      ],
      "Invalid dev experiment URI.": [
        ""
      ],
      "URI: %1$s": [
        "%1$s"
      ],
      "Experiment": [
        ""
      ],
      "Insert lost denomination": [
        ""
      ],
      "Apply": [
        ""
      ],
      "Pending refresh": [
        "Ausstehende Vorgänge"
      ],
      "Could not load the transfer payment status": [
        "Konnte den Status der Erstattung nicht laden"
      ],
      "Could not load the status of the order template": [
        "Information zu der Transaktion konnten nicht geladen werden"
      ],
      "required": [
        ""
      ],
      "Review order": [
        ""
      ],
      "Missing URI": [
        ""
      ],
      "Check the link.": [
        ""
      ],
      "Could not parse the recovery URI": [
        ""
      ],
      "Try another URI": [
        "URI eingeben"
      ],
      "Import backup, show info": [
        ""
      ],
      "Add new exchange": [
        ""
      ],
      "Add exchange for %1$s": [
        ""
      ],
      "Enter the URL of an exchange you trust.": [
        ""
      ],
      "An exchange has been found! Review the information and click next": [
        ""
      ],
      "This exchange is already in your list.": [
        "Der Zahlungsdienstleister ist für Abhebungen verfügbar"
      ],
      "Only exchange accessible through \"http\" and \"https\" are allowed.": [
        ""
      ],
      "This exchange protocol version is not supported: \" %1$s\".": [
        ""
      ],
      "This exchange currency \"%1$s\" doesn&apos;t match the expected currency %2$s.": [
        ""
      ],
      "No exchange found in that URL.": [
        ""
      ],
      "loading": [
        "Lädt Daten"
      ],
      "Version": [
        "Version"
      ],
      "Can't use the URL: \"%1$s\"": [
        ""
      ],
      "Exchange URL": [
        "URL des Zahlungsdienstleisters (Exchange-URL)"
      ],
      "Add new contact": [
        ""
      ],
      "Enter the details of a contact you trust.": [
        ""
      ],
      "Contact info valid.": [
        ""
      ],
      "This contact is already in your list.": [
        ""
      ],
      "Review contact": [
        ""
      ],
      "Contact alias": [
        ""
      ],
      "Alias type": [
        ""
      ],
      "Mailbox service": [
        ""
      ],
      "Mailbox address": [
        "Bitcoin Adresse"
      ],
      "Contact source": [
        ""
      ],
      "Could not load the list of contacts": [
        "Konnte die Liste der Zahlungsdienstleister nicht laden"
      ],
      "Source": [
        ""
      ],
      "Send Cash": [
        "%1$s übertragen"
      ],
      "Request Cash": [
        ""
      ],
      "No contacts found.": [
        ""
      ],
      "Could not bank choices for account": [
        "Es konnten keine Backup-Wiederherstellungsinformationen geladen werden"
      ],
      "Could not load the list of messages": [
        "Konnte die Liste der Zahlungsdienstleister nicht laden"
      ],
      "Unexpected error when trying to initialize mailbox: %1$s": [
        ""
      ],
      "Unexpected error when trying to delete message: %1$s": [
        ""
      ],
      "Unexpected error when trying to fetch messages: %1$s": [
        ""
      ],
      "Date": [
        "Datum"
      ],
      "You may copy your mailbox URI from below and register it at %1$s with your alias(es) to make it easier for others to find you.": [
        ""
      ],
      "copy uri": [
        "Kopieren"
      ],
      "hide uri": [
        ""
      ],
      "show uri": [
        ""
      ],
      "Fetch messages": [
        "Fulfillment message (Nachricht an Zahlende über bereitgestelltes digitales Gut)"
      ],
      "Mailbox not initialized": [
        ""
      ],
      "Initialize mailbox": [
        ""
      ],
      "No messages yet.": [
        ""
      ],
      "Taler URI handler": [
        ""
      ],
      "Could not found a handler for the Taler URI": [
        ""
      ],
      "The uri read in the path parameter is not valid: \"%1$s\"": [
        ""
      ],
      "Digital cash payment": [
        ""
      ],
      "Digital cash refund": [
        "Erstattung von e-Geld"
      ],
      "Digital cash withdrawal": [
        "Abhebung von digitalem Bargeld"
      ],
      "Digital cash deposit": [
        ""
      ],
      "Digital cash invoice": [
        "Übertragung von e-Geld"
      ],
      "Digital cash transfer": [
        "Übertragung von e-Geld"
      ],
      "Digital cash recovery": [
        "Übertragung von e-Geld"
      ],
      "Development experiment": [
        "Entwicklermodus"
      ],
      "Return to wallet": [
        ""
      ],
      "Network is offline": [
        ""
      ],
      "Diagnostics timed out. Could not talk to the wallet backend.": [
        "Die Diagnostik ist abgeschlossen. Es war keine Kommunikation mit dem Wallet-Backend möglich."
      ],
      "Problems detected:": [
        "Ein Problem wurde festgestellt:"
      ],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).": [
        "Bitte prüfen Sie in Ihren %1$s-Einstellungen, dass Sie IndexedDB aktiviert haben, und überprüfen Sie %2$s."
      ],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.": [
        "Die Datenbank des Wallets ist veraltet. Aktuell wird jedoch keine Migration auf eine neue Version unterstützt. Bitte wählen Sie %1$s zum Zurücksetzen der Wallet-Datenbank."
      ],
      "here": [
        ""
      ],
      "Running diagnostics": [
        "Diagnostik wird durchgeführt"
      ],
      "Edit": [
        "Bearbeiten"
      ],
      "List of products": [
        ""
      ],
      "free": [
        ""
      ],
      "Could not get provider information": [
        ""
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "de",
  "completeness": 13
};

strings['ca'] = {
  "locale_data": {
    "messages": {
      "": {
        "domain": "messages",
        "plural_forms": "nplurals=2; plural=n != 1;",
        "lang": "ca"
      },
      "Balance": [
        "Saldo"
      ],
      "Backup": [
        "Còpia de seguretat"
      ],
      "Notifications": [
        ""
      ],
      "QR Reader and Taler URI": [
        "Lector QR i URI del Taler"
      ],
      "Settings": [
        "Configuració"
      ],
      "Dev tools": [
        "Eines de depuració"
      ],
      "Contacts": [
        ""
      ],
      "Mailbox": [
        ""
      ],
      "Search transactions": [
        "Comissions de transacció"
      ],
      "%1$s": [
        "%1$s"
      ],
      "KYC requirement": [
        ""
      ],
      "PENDING OPERATIONS": [
        "OPERACIONS PENDENTS"
      ],
      "Error was thrown trying to: \"%1$s\"": [
        ""
      ],
      "Could not open the wallet database. Firefox is known to run into this problem under \"permanent private mode\".": [
        ""
      ],
      "Could not open the wallet database.": [
        "No s'ha pogut processar l'URI «payto»"
      ],
      "Unexpected request error, code: %1$s": [
        ""
      ],
      "more info": [
        ""
      ],
      "wrap text": [
        ""
      ],
      "copy content": [
        ""
      ],
      "less info": [
        ""
      ],
      "Retry operation": [
        "Operacions de la cartera"
      ],
      "Loading": [
        "Carregant"
      ],
      "Could not load backup providers": [
        "No s'han pogut carregar els proveïdors de la còpia de seguretat"
      ],
      "No backup providers configured": [
        "No s'han configurat proveïdors de la còpia de seguretat"
      ],
      "Add provider": [
        "Afegir proveïdor"
      ],
      "Sync all backups": [
        "Sincronitzar les còpies de seguretat"
      ],
      "Sync now": [
        "Sincronitzar ara"
      ],
      "Last synced": [
        "Darrera sincronització"
      ],
      "Not synced": [
        "No sincronitzat"
      ],
      "Expires in": [
        "Caduca en"
      ],
      "show less": [
        "mostrar QR"
      ],
      "show more": [
        "mostrar QR"
      ],
      "There was an error loading the provider detail for &quot;%1$s&quot;": [
        "Hi ha hagut un error carregant els detalls del proveïdor per a &quot; %1$s&quot;"
      ],
      "There is not known provider with url &quot;%1$s&quot;.": [
        "No hi ha cap proveïdor conegut amb l'URL &quot;%1$s&quot;."
      ],
      "See providers": [
        "Veure proveïdors"
      ],
      "Last backup": [
        "Última còpia de seguretat"
      ],
      "Back up": [
        "Fer una còpia de seguretat"
      ],
      "Provider fee": [
        "Comissió de proveïdor"
      ],
      "per year": [
        "per any"
      ],
      "Extend": [
        "Estendre"
      ],
      "terms has changed, extending the service will imply accepting the new terms of service": [
        "Els termes han canviat, estendre el servei implicarà acceptar els nous termes d'ús"
      ],
      "old": [
        "antic"
      ],
      "new": [
        "nou"
      ],
      "fee": [
        "comissió"
      ],
      "storage": [
        "emmagatzematge"
      ],
      "Remove provider": [
        "Eliminar proveïdor"
      ],
      "Pay": [
        "Pagament"
      ],
      "Withdraw": [
        "Retirar"
      ],
      "This provider has reported an error": [
        "Aquest proveïdor ha reportat un error"
      ],
      "There is conflict with another backup from &quot;%1$s&quot;": [
        "Hi ha hagut un conflicte amb una altra còpia de seguretat de %1$s"
      ],
      "Backup is not readable": [
        "La còpia de seguretat no és llegible"
      ],
      "Unknown backup problem: %1$s": [
        "Problema desconegut amb la còpia de seguretat: %1$s"
      ],
      "service paid": [
        "servei pagat"
      ],
      "Backup valid until": [
        "Còpia de seguretat vàlida fins"
      ],
      "Cancel": [
        "Cancel·lar"
      ],
      "Open pay page": [
        "Obrir pàgina de pagament"
      ],
      "Open refund page": [
        "Obrir pàgina de reemborsament"
      ],
      "Open withdraw page": [
        "Obrir pàgina de retirada"
      ],
      "Your wallet is empty.": [
        ""
      ],
      "Get digital cash": [
        "Obtenir efectiu digital"
      ],
      "Try the demo bank and withdraw test money.": [
        ""
      ],
      "Could not load the balance": [
        "No s'ha pogut carregar el saldo del dipòsit"
      ],
      "Receive": [
        "Rebut"
      ],
      "Send %1$s": [
        "Enviar %1$s"
      ],
      "This page has a withdrawal action.": [
        "Aquesta pàgina té una acció de retirada."
      ],
      "This page has pay action.": [
        "Aquesta pàgina té una acció de pagament."
      ],
      "This page has a refund action.": [
        "Aquesta pàgina té una acció de reemborsament."
      ],
      "This page has a add exchange action.": [
        "Aquesta pàgina té una acció de reemborsament."
      ],
      "Open add exchange page": [
        "Afegir proveïdor"
      ],
      "This page has an add contact action.": [
        "Aquesta pàgina té una acció de propina."
      ],
      "Open add contact page": [
        "Obrir pàgina de pagament"
      ],
      "Taler Action": [
        "Acció de Taler"
      ],
      "This page has a malformed taler uri.": [
        "Aquesta pàgina té una URI de Taler mal formada."
      ],
      "Dismiss": [
        "Descartar"
      ],
      "this popup is being closed and you are being redirected to %1$s": [
        "Aquest missatge emergent s'està tancant i t'estem redirigint a %1$s"
      ],
      "Could not load purchase proposal details": [
        "No s'han pogut carregar els detalls de petició de compra"
      ],
      "Order Id": [
        "Identificador de sol·licitud"
      ],
      "Summary": [
        "Resum"
      ],
      "Amount": [
        "Quantitat"
      ],
      "Choices": [
        ""
      ],
      "This purchase have %1$s choices": [
        ""
      ],
      "Merchant name": [
        "Nom del comerciant"
      ],
      "Merchant jurisdiction": [
        "Jurisdicció del comerciant"
      ],
      "Merchant address": [
        "Adreça del comerciant"
      ],
      "Merchant logo": [
        "Logotip del comerciant"
      ],
      "Merchant website": [
        "Pàgina web del comerciant"
      ],
      "Merchant email": [
        "Correu electrònic del comerciant"
      ],
      "Merchant public key": [
        "Clau pública del comerciant"
      ],
      "Created at": [
        "Creat a"
      ],
      "Refund deadline": [
        "Data límit de reemborsament"
      ],
      "Auto refund": [
        "Reemborsament automàtic"
      ],
      "Pay deadline": [
        "Data límit per pagar"
      ],
      "Fulfillment URL": [
        "URL de compliment"
      ],
      "Fulfillment message": [
        "Missatge de compliment"
      ],
      "Max deposit fee": [
        "Comissió màxima de dipòsit"
      ],
      "Exchanges": [
        "Proveïdors de servei de pagament"
      ],
      "Could not load the status of the term of service": [
        "No s'ha pogut carregar la llista de transaccions"
      ],
      "Select one option": [
        "Selecciona una opció"
      ],
      "Show terms of service": [
        "Mostrar termes del servei"
      ],
      "I accept the exchange terms of service": [
        "Accepto els termes del servei"
      ],
      "Review exchange terms of service": [
        "Revisar termes de servei del proveïdor"
      ],
      "The exchange doesn't have a terms of service.": [
        "El proveïdor no té termes de servei"
      ],
      "Skip it for now.": [
        ""
      ],
      "Format": [
        ""
      ],
      "Language": [
        "Llengua actual"
      ],
      "The exchange replied with a empty terms of service": [
        "El proveïdor ha contestat amb termes de servei buits"
      ],
      "No terms of service. The exchange replied with a empty document": [
        "El proveïdor ha contestat amb termes de servei buits"
      ],
      "No terms of service. The exchange replied with a empty text": [
        "El proveïdor ha contestat amb termes de servei buits"
      ],
      "Download Terms of Service": [
        "Descarregar termes de servei"
      ],
      "Hide terms of service": [
        "Amager termes de servei"
      ],
      "Amount is not valid": [
        ""
      ],
      "Bank account": [
        "Compte bancari"
      ],
      "Bitcoin address": [
        "Adreça de Bitcoin"
      ],
      "BIC/IBAN": [
        "IBAN"
      ],
      "IBAN": [
        "IBAN"
      ],
      "Bitcoin transfer details": [
        "Detalls de transferència Bitcoin"
      ],
      "The exchange need a transaction with 3 output, one output is the exchange account and the other two are segwit fake address for metadata with an minimum amount.": [
        "El proveïdor de servei de pagament necessita una transacció amb tres «outputs»/sortides: una sortida és el compte del proveïdor del servei i les altres dues són adreces «segwit» falses per a metadades amb un import mínim."
      ],
      "In bitcoincore wallet use &apos;Add Recipient&apos; button to add two additional recipient and copy addresses and amounts": [
        "A la cartera bitcoincore utilitza el botó &apos;Add Recipient&apos; (afegir recipient) per afegir dos destinataris i copiar les adreces i imports"
      ],
      "Make sure the amount show %1$s BTC, else you have to change the base unit to BTC": [
        "Assegura't que l'import mostri %1$s BTC; si no, has de canviar la unitat base a BTC"
      ],
      "Ethereum transfer details": [
        "Detalls de la transferència bancària"
      ],
      "You need to wire to the service provider account.": [
        ""
      ],
      "Make sure the amount show %1$s ETH": [
        ""
      ],
      "Bank transfer details": [
        "Detalls de la transferència bancària"
      ],
      "Account": [
        "Compte"
      ],
      "Bank host": [
        "Servidor del banc"
      ],
      "BIC": [
        ""
      ],
      "Exchange": [
        "Proveïdor de canvi"
      ],
      "Reserve Pub": [
        ""
      ],
      "Step 1:": [
        ""
      ],
      "Copy this code and paste it into the subject/purpose field in your banking app or bank website": [
        ""
      ],
      "Subject": [
        "Concepte"
      ],
      "Step 2:": [
        ""
      ],
      "If you don't already have it in your banking favourites list, then copy and paste this IBAN and the name into the receiver fields in your banking app or website": [
        ""
      ],
      "Receiver name": [
        "Nom del beneficiari"
      ],
      "Receiver postal code": [
        "Nom del beneficiari"
      ],
      "Receiver town": [
        "Nom del beneficiari"
      ],
      "Step 3:": [
        ""
      ],
      "Finish the wire transfer setting the amount in your banking app or website, then this withdrawal will proceed automatically.": [
        ""
      ],
      "Make sure ALL data is correct, including the subject; otherwise, the money will not arrive in this wallet. You can use the copy buttons (%1$s) to prevent typing errors or the \"payto://\" URI below to copy just one value.": [
        ""
      ],
      "Alternative if your bank already supports PayTo URI, you can use this %1$s link instead": [
        ""
      ],
      "Continue with banking app or website": [
        ""
      ],
      "Show QR code": [
        ""
      ],
      "Could not load transaction information": [
        "No s'ha pogut carregar la informació de la transacció"
      ],
      "Bank account confirmation required for the transaction to complete.": [
        ""
      ],
      "KYC check required for the transaction to complete.": [
        ""
      ],
      "No additional information has been provided.": [
        ""
      ],
      "Follow this link to the%1$s %2$s": [
        ""
      ],
      "This transaction is pending to complete.": [
        "Aquesta transacció no està completada"
      ],
      "Retry": [
        "Reintentar"
      ],
      "There was an error trying to complete the transaction.": [
        "Hi ha hagut un error intentant completar la transacció"
      ],
      "This transaction was aborted.": [
        "Aquesta transacció no està completada"
      ],
      "This transaction failed.": [
        "Aquesta transacció no està completada"
      ],
      "Caution!": [
        "Compte!"
      ],
      "If you have already wired money to the exchange you will loose the chance to get the coins form it.": [
        "Si ja has enviat diners per transferència (per cable) al proveïdor, perdràs la oportunitat d'aconseguir-ne les monedes."
      ],
      "Confirm": [
        "Confirmar"
      ],
      "Doing a cancellation while the transaction still active might result in lost coins. Do you still want to cancel the transaction?": [
        ""
      ],
      "No": [
        ""
      ],
      "Yes": [
        ""
      ],
      "Abort": [
        ""
      ],
      "Resume": [
        ""
      ],
      "Suspend": [
        ""
      ],
      "Delete": [
        ""
      ],
      "Withdrawal": [
        "Retirada"
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Select one of the accounts and use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Now the payment service provider is waiting for %1$s to be transferred. Use the information below to complete the operation by making a wire transfer from your bank account.": [
        ""
      ],
      "Details": [
        "Detalls"
      ],
      "Withdrawal incomplete.": [
        "Retirada"
      ],
      "If you have already sent money to the service provider account it will wire it back at %1$s": [
        "Si ja has enviat diners per transferència (per cable) al proveïdor, perdràs la oportunitat d'aconseguir-ne les monedes."
      ],
      "Payment": [
        "Pagament"
      ],
      "Refunds": [
        "Reemborsaments"
      ],
      "%1$s %2$s on %3$s .": [
        "%1$s %2$s a %3$s"
      ],
      "Refund is in progress.": [
        "S'està processant el reemborsament."
      ],
      "Merchant created a refund for this order but was not automatically picked up.": [
        "El comerciant ha creat un reemborsament per a aquesta sol·licitud però no s'ha recollit automàticament."
      ],
      "Offer": [
        "Oferta"
      ],
      "Accept": [
        "Acceptar"
      ],
      "Confirmation code": [
        ""
      ],
      "Merchant": [
        "Comerciant"
      ],
      "Invoice ID": [
        "Identificador (ID) de factura"
      ],
      "Deposit": [
        "Dipòsit"
      ],
      "Wire transfer deadline.": [
        "Detalls de transferència Bitcoin"
      ],
      "Wire transfer still pending.": [
        ""
      ],
      "The service provider deadline to make the wire transfer is: %1$s": [
        ""
      ],
      "Wire transfer completed.": [
        ""
      ],
      " ": [
        ""
      ],
      "Transfer details": [
        "Detalls de la transferència bancària"
      ],
      "Wire transfer in progress.": [
        "S'està processant el reemborsament."
      ],
      "Refresh": [
        "Actualitzar"
      ],
      "Refund": [
        "Reemborsament"
      ],
      "Purchase summary": [
        "Resum de la compra"
      ],
      "Credit": [
        "Crèdit"
      ],
      "Invoice": [
        "Factura"
      ],
      "URI": [
        "URI"
      ],
      "Debit": [
        "Dèbit"
      ],
      "Transfer": [
        "Tranferència"
      ],
      "Lost": [
        ""
      ],
      "Reason": [
        ""
      ],
      "Denomination expired.": [
        "Denominació"
      ],
      "Denomination vanished.": [
        "Denominació"
      ],
      "Denomination is unoffered.": [
        "Denominació"
      ],
      "Fees": [
        ""
      ],
      "Total": [
        "Total"
      ],
      "Sent": [
        ""
      ],
      "Converted": [
        ""
      ],
      "Bank fee": [
        "Comissió màxima"
      ],
      "Price": [
        "Preu"
      ],
      "Subtotal": [
        "Total"
      ],
      "Refunded": [
        "Reemborsat"
      ],
      "Transfer identification": [
        "Operacions de transferència"
      ],
      "copy": [
        "copiar"
      ],
      "hide qr": [
        "amagar QR"
      ],
      "forward to contact": [
        ""
      ],
      "show qr": [
        "mostrar QR"
      ],
      "Wire transfer need a confirmation. Go to the %1$s and check wire transfer operation to exchange account is complete.": [
        ""
      ],
      "bank site": [
        "Servidor del banc"
      ],
      "Bank has confirmed the wire transfer. Waiting for the exchange to send the coins.": [
        "El banc ha confirmat la transferència (per cable). Esperant que el proveïdor enviï les monedes"
      ],
      "Exchange is ready to send the coins, withdrawal in progress.": [
        "El proveïdor està llest per a la retirada"
      ],
      "Send &nbsp; %1$s": [
        "Envia &nbsp; %1$s"
      ],
      "Pay &nbsp; %1$s": [
        "Pagar &nbsp; %1$s"
      ],
      "Merchant message": [
        "Missatge del comerciant"
      ],
      "Balance is not enough.": [
        "Actualment no tens prou saldo."
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which exchange and auditors the merchant trust.": [
        ""
      ],
      "Balance is not enough because merchant will just accept %1$s %2$s. To know more you can check which wire methods the merchant accepts.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to pay for this contract which is restricted.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s to spend right know. There are some coins that need to be refreshed.": [
        ""
      ],
      "Balance is not enough because you have %1$s %2$s available.": [
        ""
      ],
      "Provider is missing the global fee configuration, this likely means it is misconfigured.": [
        ""
      ],
      "Balance looks like it should be enough, but doesn't cover all fees requested by the merchant and payment processor. Please ensure there is at least %1$s %2$s more balance in your wallet or ask your merchant to cover more of the fees.": [
        ""
      ],
      "Pay with a mobile phone": [
        "Pagar amb un telèfon mòbil"
      ],
      "Hide QR": [
        "Amagar QR"
      ],
      "Scan the QR code or &nbsp; %1$s": [
        "Escanejar el codi QR o &nbsp; %1$s"
      ],
      "click here": [
        "clicar per a mostrar"
      ],
      "Could not load the payment and balance status": [
        "No s'ha pogut carregar l'estatus del pagament"
      ],
      "Purchase": [
        "Compra"
      ],
      "This purchase is age restricted.": [
        "Aquesta pàgina té una acció de propina."
      ],
      "Expires at": [
        "Caduca en"
      ],
      "Already paid, you are going to be redirected to %1$s": [
        "Ja s'ha pagat, seràs redirigit a %1$s"
      ],
      "Already paid": [
        "Ja s'ha pagat"
      ],
      "Already claimed": [
        "Ja s'ha reclamat"
      ],
      "Could not load the refund status": [
        "No s'ha pogut carregar l'estatus del reemborsament"
      ],
      "You&apos;ve ignored the refund.": [
        "Has ignorat la propina."
      ],
      "The merchant &quot;%1$s&quot; is offering you a refund.": [
        "El comerciant &quot;%1$s&quot; està oferint-te un reemborsament."
      ],
      "Order amount": [
        "Import de la sol·licitud"
      ],
      "Could not load the list of exchanges": [
        "No s'ha pogut carregar una llista de proveïdor"
      ],
      "Could not load info from URI": [
        "No s'ha pogut obtenir informació de l'URI"
      ],
      "Could not load the withdrawal details": [
        "No s'han pogut carregar els detalls de petició de compra"
      ],
      "under %1$s": [
        "Enviar %1$s"
      ],
      "should be greater than zero": [
        ""
      ],
      "can't be greater than %1$s": [
        ""
      ],
      "Could not load exchange details info": [
        "No s'han pogut carregar les comissions de canvi"
      ],
      "Close": [
        "Tancar"
      ],
      "There is no exchange available for currency %1$s": [
        "No hi ha prou saldo per fer el dipòsit amb la divisa %1$s"
      ],
      "You can add more exchanges from the settings.": [
        ""
      ],
      "Exchange %1$s is not available": [
        ""
      ],
      "Exchange status can view accessed from the settings.": [
        ""
      ],
      "Service fee description": [
        "Descripció de la comissió de servei"
      ],
      "Select %1$s exchange": [
        "Seleccionar proveïdor %1$s"
      ],
      "Reset": [
        "Resetejar"
      ],
      "Use this exchange": [
        "Utilitzar aquest proveïdor"
      ],
      "Auditors": [
        "Auditors"
      ],
      "Doesn&apos;t have auditors": [
        "No té auditors"
      ],
      "currency": [
        "divisa"
      ],
      "Coin operations": [
        "Operacions de moneda"
      ],
      "Every operation in this section may be different by denomination value and is valid for a period of time. The exchange will charge the indicated amount every time a coin is used in such operation.": [
        "Cada operació d'aquesta secció pot variar pel valor de la denominació i és vàlida per un període de temps limitat. El proveïdor cobrarà l'import indicat cada cop que una moneda és utilitzada en una operació d'aquesta mena."
      ],
      "Deposits": [
        "Dipòsits"
      ],
      "Denomination": [
        "Denominació"
      ],
      "Current": [
        "Divisa"
      ],
      "Selected": [
        ""
      ],
      "Until": [
        "Fins"
      ],
      "Withdrawals": [
        "Retirades"
      ],
      "Transfer operations": [
        "Operacions de transferència"
      ],
      "Every operation in this section may be different by transfer type and is valid for a period of time. The exchange will charge the indicated amount every time a transfer is made.": [
        "Cada operació d'aquesta secció pot diferir per tipus de transferència i és vàlida per un període de temps limitat. L'intercanvi cobrarà l'import indicat cada cop que es faci una transferència."
      ],
      "Operation": [
        "Operació"
      ],
      "Fee": [
        "Comissió"
      ],
      "Wallet operations": [
        "Operacions de la cartera"
      ],
      "Feature": [
        "Característica"
      ],
      "Currency": [
        "Divisa"
      ],
      "This operation has already been completed.": [
        "Aquesta transacció no està completada"
      ],
      "This operation has already been aborted": [
        ""
      ],
      "This operation has started and should be completed in the bank.": [
        ""
      ],
      "You can confirm the operation in": [
        "No s'ha pogut finalitzar l'operació de recollida"
      ],
      "this page": [
        "Obrir pàgina de propina"
      ],
      "This operation has already been completed by another wallet.": [
        ""
      ],
      "This operation has already been used by another wallet.": [
        ""
      ],
      "It can be confirmed in": [
        "No s'ha configurat cap proveïdor"
      ],
      "Age restriction": [
        "Restricció d'edat"
      ],
      "Withdraw &nbsp; %1$s": [
        "Retirar &nbsp; %1$s"
      ],
      "Withdraw to a mobile phone": [
        "Retirar a un telèfon mòbil"
      ],
      "Short description of the invoice": [
        "Especificar la destinació dels diners"
      ],
      "Create": [
        "Crear"
      ],
      "Short description of the transfer": [
        "Especificar la destinació dels diners"
      ],
      "Could not load the invoice payment status": [
        "No s'ha pogut carregar l'estatus del pagament"
      ],
      "Receive &nbsp; %1$s": [
        "Rebre &nbsp; %1$s"
      ],
      "Could not load known bank accounts": [
        "No s'ha pogut carregar l'estatus del pagament"
      ],
      "Could not load supported wire methods": [
        "No s'han pogut carregar els proveïdors de la còpia de seguretat"
      ],
      "No wire methods supported for this currency": [
        ""
      ],
      "Try another wire method.": [
        "A una altra cartera"
      ],
      "Known accounts for %1$s": [
        "Afegir compte bancari per a %1$s"
      ],
      "To add a new account first select the account type.": [
        ""
      ],
      "Unable add this account": [
        "No s'ha pogut afegir el compte"
      ],
      "Add": [
        "Afegir"
      ],
      "IBAN accounts": [
        "Afegir compte"
      ],
      "Alias": [
        ""
      ],
      "Int. Account Number": [
        ""
      ],
      "Account name": [
        "Compte"
      ],
      "KYC": [
        ""
      ],
      "Taler accounts": [
        "Seleccionar compte"
      ],
      "Host": [
        ""
      ],
      "Bitcoin accounts": [
        "Compte bancari"
      ],
      "Address": [
        "Adreça"
      ],
      "Can't be empty": [
        ""
      ],
      "Bitcoin Account": [
        "Compte bancari"
      ],
      "Taler Bank": [
        "Acció de Taler"
      ],
      "Unsupported country.": [
        ""
      ],
      "IBANs have fewer than 34 characters.": [
        ""
      ],
      "IBANs have more than 4 characters.": [
        ""
      ],
      "It should only contain numbers and letters.": [
        ""
      ],
      "The checksum is wrong.": [
        ""
      ],
      "Required": [
        ""
      ],
      "International Bank Account Number": [
        ""
      ],
      "Could not load balance information": [
        "No s'ha pogut carregar la informació de recuperació de la còpia de seguretat"
      ],
      "Could not load fee for amount %1$s": [
        "No s'ha pogut carregar l'estatus del reemborsament"
      ],
      "A currency or an amount should be indicated": [
        "S'ha d'indicar una divisa o import"
      ],
      "There is no enough balance to make a deposit for currency %1$s": [
        "No hi ha prou saldo per fer el dipòsit amb la divisa %1$s"
      ],
      "Send %1$s to your account": [
        "Envia %1$s al teu compte"
      ],
      "There is no account to make a deposit for currency %1$s": [
        "No hi ha cap compte per fer el dipòsit amb la divisa %1$s"
      ],
      "Add account": [
        "Afegir compte"
      ],
      "Select account": [
        "Seleccionar compte"
      ],
      "Manage accounts": [
        "Compte bancari"
      ],
      "Brut amount": [
        "Import de la sol·licitud"
      ],
      "Deposit fee": [
        "Comissió de dipòsit"
      ],
      "Net amount": [
        "Import de la sol·licitud"
      ],
      "Deposit&nbsp;%1$s %2$s": [
        "Depòsit&nbsp;%1$s %2$s"
      ],
      "Need approval in the Bank": [
        ""
      ],
      "Waiting for wire transfer to complete": [
        ""
      ],
      "Exchange is waiting the wire transfer": [
        ""
      ],
      "Payment in progress": [
        "S'està processant el reemborsament."
      ],
      "Executing refund...": [
        "Reemborsament automàtic"
      ],
      "Refreshing coins...": [
        ""
      ],
      "Deposit in progress": [
        "S'està processant el reemborsament."
      ],
      "Waiting to be paid": [
        ""
      ],
      "Receiving the transfer": [
        "Transferència d'efectiu digital"
      ],
      "Waiting to be received": [
        "Pendent de rebre"
      ],
      "Denomination expired": [
        "Denominació"
      ],
      "Denomination vanished": [
        "Denominació"
      ],
      "Denomination unoffered": [
        "Denominació"
      ],
      "ABORTED": [
        ""
      ],
      "FAILED": [
        ""
      ],
      "Could not load the list of transactions": [
        "No s'ha pogut carregar la llista de transaccions"
      ],
      "unknown scope": [
        ""
      ],
      "Send": [
        "Enviar"
      ],
      "Your transaction history is empty for this currency.": [
        "El teu historial de transaccions està buit per a aquesta divisa."
      ],
      "Review terms of service": [
        "Revisa els temes d'ús"
      ],
      "Provider URL": [
        "URL del proveïdor"
      ],
      "Please review and accept this provider&apos;s terms of service": [
        "Si us plau, revisa i accepta els termes d'ús d'aquest proveïdor"
      ],
      "Pricing": [
        "Preus"
      ],
      "free of charge": [
        "de franc"
      ],
      "%1$s per year of service": [
        "%1$s per any de servei"
      ],
      "Storage": [
        "Emmagatzematge"
      ],
      "%1$s megabytes of storage per year of service": [
        "%1$s megabytes d'emmagatzematge per any de servei"
      ],
      "Accept terms of service": [
        "Acceptar els termes d'ús"
      ],
      "Add backup provider": [
        "Afegeix un proveïdor de recanvi"
      ],
      "Backup providers may charge for their service": [
        "Els proveïdors de recanvi podrien cobrar pel seu servei"
      ],
      "URL": [
        "URL"
      ],
      "Name": [
        "Nom"
      ],
      "Next": [
        "Següent"
      ],
      "Navigator": [
        "Navegador"
      ],
      "Automatically open wallet": [
        "Obrir cartera automàticament depenent del contingut de la pàgina"
      ],
      "Open the wallet when a payment action is found.": [
        ""
      ],
      "Version Info": [
        "Versió"
      ],
      "Web Extension": [
        "Extensió web"
      ],
      "The version of wallet core is not supported. (supported version: %1$s, wallet version: %2$s)": [
        ""
      ],
      "Exchange compatibility": [
        "Compatibilitat del proveïdor"
      ],
      "Merchant compatibility": [
        "Compatibilitat del comerciant"
      ],
      "Bank compatibility": [
        "Compatibilitat del banc"
      ],
      "Wallet Core compatibility": [
        "Compatibilitat del banc"
      ],
      "Enable developer mode": [
        "Mode de desenvolupador"
      ],
      "Show more information and options in the UI": [
        ""
      ],
      "Display": [
        "Mostrar"
      ],
      "Current Language": [
        "Llengua actual"
      ],
      "Show backup feature": [
        ""
      ],
      "Backup integration still in beta.": [
        ""
      ],
      "Show P2P alias feature": [
        ""
      ],
      "P2P aliases integration still in beta.": [
        ""
      ],
      "Show suspend/resume transaction": [
        ""
      ],
      "Prevent transaction from doing network request.": [
        ""
      ],
      "Show refresh transaction type in the transaction list": [
        ""
      ],
      "Refresh transaction will be hidden by default if the refresh operation doesn't have fee.": [
        ""
      ],
      "Show more account types on deposit": [
        ""
      ],
      "Extends the UI to more payment target types.": [
        ""
      ],
      "Show JSON on error": [
        ""
      ],
      "Print more information about the error. Useful for debugging.": [
        "Més opcions i informació útils per a la depuració de programari"
      ],
      "Allow HTTP connections": [
        ""
      ],
      "Using HTTP connection may be faster but unsafe (wallet restart required)": [
        ""
      ],
      "Lang selector": [
        ""
      ],
      "Allows to manually change the language of the UI. Otherwise it will be automatically selected by your browser configuration.": [
        ""
      ],
      "Edit exchange management": [
        "De moment no hi ha cap proveïdor"
      ],
      "Allows to see the list of exchange, remove, add and switch before withdrawal.": [
        ""
      ],
      "Select terms of service format": [
        "Acceptar els termes d'ús"
      ],
      "Allows to render the terms of service on different format selected by the user.": [
        ""
      ],
      "Show wallet activity": [
        ""
      ],
      "Show the wallet notification and observability event in the UI.": [
        ""
      ],
      "Enable v1 contract": [
        ""
      ],
      "v1 is under development and wallet-core converts v1 contract to v0. Enable this options for testing": [
        ""
      ],
      "GNU Taler Wallet installed!": [
        ""
      ],
      "You can open the wallet using the combination %1$s .": [
        "Pots obrir la cartera de GNU Taler utilitzant la combinació %1$s."
      ],
      "Also pinning the GNU Taler Wallet to your browser allows you to quick access without keyboard:": [
        "A més, fixar la cartera del GNU Taler al teu navegador Chrome et permet d'accedir-hi ràpidament sense el teclat:"
      ],
      "Click the puzzle icon": [
        "Prem la icona del trencaclosques"
      ],
      "Search for GNU Taler Wallet": [
        "Busca la cartera del GNU Taler"
      ],
      "Click the pin icon": [
        "Clica la icona del pin"
      ],
      "Next Steps": [
        "Proper pas"
      ],
      "Try the demo": [
        "Provar la demostració"
      ],
      "Click here to open the wallet activity tab.": [
        ""
      ],
      "Events": [
        ""
      ],
      "Active tasks": [
        ""
      ],
      "All events": [
        ""
      ],
      "Balance change": [
        "Saldo"
      ],
      "Backup failed": [
        "Còpia de seguretat vàlida fins"
      ],
      "Transaction updated": [
        "Comissions de transacció"
      ],
      "Exchange updated": [
        "Proveïdor de canvi"
      ],
      "Idle": [
        ""
      ],
      "task.%1$s": [
        "%1$s"
      ],
      "wallet.%1$s(%2$s)": [
        ""
      ],
      "Bank account info changed": [
        "Compte bancari"
      ],
      "Contact added": [
        ""
      ],
      "Contact deleted": [
        ""
      ],
      "Mailbox message added": [
        ""
      ],
      "Mailbox message deleted": [
        ""
      ],
      "Type": [
        ""
      ],
      "Id": [
        ""
      ],
      "Since": [
        ""
      ],
      "Next try": [
        "Proper pas"
      ],
      "Error": [
        ""
      ],
      "Transaction": [
        "Comissions de transacció"
      ],
      "No exchange yet": [
        "De moment no hi ha cap proveïdor"
      ],
      "Status": [
        "estatus"
      ],
      "Terms of Service": [
        "Termes d'ús"
      ],
      "Last Update": [
        ""
      ],
      "Actions": [
        "Acció de Taler"
      ],
      "ok": [
        "OK"
      ],
      "pending": [
        ""
      ],
      "proposed": [
        ""
      ],
      "unknown (exchange status should be updated)": [
        "desconegut (l'estatus del proveïdor s'hauria d'actualitzar)"
      ],
      "Reload": [
        "carregant"
      ],
      "Purge": [
        ""
      ],
      "Make regional": [
        ""
      ],
      "Make global": [
        ""
      ],
      "Forget ToS": [
        "Oblidar"
      ],
      "Debug tools": [
        "Eines de depuració"
      ],
      "Do you want to IRREVOCABLY DESTROY everything inside your wallet and LOSE ALL YOUR COINS?": [
        "Segur que vols DESTRUIR IRREVERSIBLEMENT tot el contingut de la teva bitlletera i PERDRE TOTES LES TEVES MONEDES?"
      ],
      "reset": [
        "resetejar"
      ],
      "TESTING: This may delete all your coin, proceed with caution": [
        "PROVANT: Això podria esborrar totes les teves monedes, procedeix amb precaució"
      ],
      "run gc": [
        "Executar GC"
      ],
      "import database": [
        "importar base de dades"
      ],
      "export database": [
        "Exportar base de dades"
      ],
      "Clear exchange key cache": [
        "De moment no hi ha cap proveïdor"
      ],
      "Database exported at %1$s %2$s to download": [
        "Base de dades exportada a %1$s %2$s per ser descarregada"
      ],
      "Inject Taler support in all pages": [
        ""
      ],
      "Enabling this option will make `window.taler` be available in all sites": [
        ""
      ],
      "Exchange Entries": [
        "Proveïdors de servei de pagament"
      ],
      "Add an exchange": [
        "Afegir un proveïdor"
      ],
      "Log levels": [
        ""
      ],
      "Coins": [
        "Monedes"
      ],
      "usable coins": [
        "monedes utilitzables"
      ],
      "id": [
        "ID (identificació)"
      ],
      "denom": [
        "denominació"
      ],
      "status": [
        "estatus"
      ],
      "from refresh?": [
        "des d'una actualització?"
      ],
      "age key count": [
        "Nombre d'«age key»"
      ],
      "spent coins": [
        "monedes gastades"
      ],
      "click to show": [
        "clicar per a mostrar"
      ],
      "URI is not valid. Taler URI should start with \"taler://\"": [
        "L'URI no és vàlida. L'URI del Taler hauria de començar per «taler://»"
      ],
      "After the URI type it should follow a '/' with more information.": [
        ""
      ],
      "This URI type is not supported": [
        ""
      ],
      "This URI requires the bank host and operation id separated by /": [
        ""
      ],
      "This URI requires the merchant host, order id and session id separated by /": [
        ""
      ],
      "This URI requires the merchant host and order id separated by /": [
        ""
      ],
      "This URI requires the exchange host and key id separated by /": [
        ""
      ],
      "This URI requires the merchant host and template id separated by /": [
        ""
      ],
      "This URI requires the host key and provider list separated by /": [
        ""
      ],
      "This URI requires the experiment id": [
        ""
      ],
      "This URI requires the exchange host": [
        ""
      ],
      "This URI requires the alias type, the alias and the mailbox URI separated by /": [
        ""
      ],
      "This URI requires string after the first /": [
        ""
      ],
      "The bank host is invalid": [
        ""
      ],
      "The merchant host is invalid": [
        "El comerciant t'ofereix una propina"
      ],
      "The URI should end with /": [
        ""
      ],
      "The exchange host is invalid": [
        ""
      ],
      "The contact is invalid": [
        ""
      ],
      "The amount is invalid": [
        ""
      ],
      "A parameter is invalid": [
        ""
      ],
      "checking...": [
        ""
      ],
      "Unexpected error happen reading the camera: %1$s": [
        ""
      ],
      "Could not found a QR code in the file": [
        ""
      ],
      "Unexpected error happen reading the file: %1$s": [
        ""
      ],
      "Scan a QR code or enter taler:// URI below": [
        "Escaneja un codi QR o introdueix una URI taler:// a sota"
      ],
      "Pay invoice": [
        "Factura digital"
      ],
      "Withdrawal from bank": [
        "Retirada"
      ],
      "Claim refund": [
        "Ja s'ha reemborsat"
      ],
      "Accept payment": [
        "Acceptar"
      ],
      "Complete order": [
        ""
      ],
      "Restore wallet": [
        ""
      ],
      "Enable experiment": [
        ""
      ],
      "Withdraw from exchange": [
        "Retirar a un telèfon mòbil"
      ],
      "Add exchange": [
        "Afegir proveïdor"
      ],
      "Notify transaction": [
        ""
      ],
      "Add contact": [
        "Afegir compte"
      ],
      "The exchange URL is invalid.": [
        ""
      ],
      "The exchange URL should end with '/'": [
        ""
      ],
      "Couldn't found an exchange in the URL specified.": [
        "no s'ha pogut trobar cap proveïdor per a la divisa %1$s"
      ],
      "HTTP request failed to %1$s: %2$s": [
        ""
      ],
      "HTTP request failed to %1$s": [
        ""
      ],
      "The merchant URL is invalid.": [
        "El comerciant t'ofereix una propina"
      ],
      "The merchant URL should end with '/'": [
        ""
      ],
      "Couldn't found an merchant in the URL specified.": [
        ""
      ],
      "The bank URL is invalid.": [
        ""
      ],
      "The bank URL should end with '/'": [
        ""
      ],
      "Couldn't found an bank in the URL specified.": [
        ""
      ],
      "Could not load exchanges": [
        "No s'han pogut carregar les comissions de canvi"
      ],
      "Choose a currency to proceed or add another exchange": [
        "Tria una divisa per seguir endavant o afegeix un altre proveïdor"
      ],
      "Known currencies": [
        "Divises conegudes"
      ],
      "Specify the origin": [
        "Especificar l'origen dels diners"
      ],
      "Use previous origins:": [
        "Utilitzar orígens previs:"
      ],
      "From another bank account": [
        "Des del meu compte bancari"
      ],
      "From another wallet": [
        "Des d'una altra cartera"
      ],
      "From a %1$s URI or QR code": [
        ""
      ],
      "Enter URI here": [
        ""
      ],
      "Specify the destination": [
        "Especificar la destinació dels diners"
      ],
      "Use previous destinations:": [
        "Utilitzar destinacions prèvies:"
      ],
      "To another bank account": [
        "Al meu compte bancari"
      ],
      "To another wallet": [
        "A una altra cartera"
      ],
      "To a %1$s URI or QR code": [
        ""
      ],
      "Afghanistan": [
        ""
      ],
      "Albania": [
        ""
      ],
      "Algeria": [
        ""
      ],
      "United States": [
        ""
      ],
      "Andorra": [
        ""
      ],
      "Angola": [
        ""
      ],
      "Antigua and Barbuda": [
        ""
      ],
      "Argentina": [
        ""
      ],
      "Armenia": [
        ""
      ],
      "Australia": [
        ""
      ],
      "Austria": [
        ""
      ],
      "Azerbaijan": [
        ""
      ],
      "Bahamas": [
        ""
      ],
      "Bahrain": [
        ""
      ],
      "Bangladesh": [
        ""
      ],
      "Barbados": [
        ""
      ],
      "Botswana": [
        ""
      ],
      "Belarus": [
        ""
      ],
      "Belgium": [
        ""
      ],
      "Belize": [
        ""
      ],
      "Benin": [
        ""
      ],
      "Bhutan": [
        ""
      ],
      "Bolivia": [
        ""
      ],
      "Bosnia and Herzegovina": [
        ""
      ],
      "Brazil": [
        ""
      ],
      "United Kingdom": [
        ""
      ],
      "Brunei": [
        ""
      ],
      "Bulgaria": [
        ""
      ],
      "Burkina Faso": [
        ""
      ],
      "Myanmar": [
        ""
      ],
      "Burundi": [
        ""
      ],
      "Cambodia": [
        ""
      ],
      "Cameroon": [
        ""
      ],
      "Canada": [
        ""
      ],
      "Cape Verde": [
        ""
      ],
      "Central African Republic": [
        ""
      ],
      "Chad": [
        ""
      ],
      "Chile": [
        ""
      ],
      "China": [
        ""
      ],
      "Colombia": [
        ""
      ],
      "Comoros": [
        ""
      ],
      "Democratic Republic of the Congo": [
        ""
      ],
      "Republic of the Congo": [
        ""
      ],
      "Costa Rica": [
        ""
      ],
      "Croatia": [
        ""
      ],
      "Cuba": [
        ""
      ],
      "Cyprus": [
        ""
      ],
      "Czech Republic": [
        "Clau pública del comerciant"
      ],
      "Denmark": [
        ""
      ],
      "Djibouti": [
        ""
      ],
      "Dominican Republic": [
        ""
      ],
      "Netherlands": [
        ""
      ],
      "Timor-Leste": [
        ""
      ],
      "Ecuador": [
        ""
      ],
      "Egypt": [
        ""
      ],
      "United Arab Emirates": [
        ""
      ],
      "Equatorial Guinea": [
        ""
      ],
      "Eritrea": [
        ""
      ],
      "Estonia": [
        ""
      ],
      "Ethiopia": [
        ""
      ],
      "Fiji": [
        ""
      ],
      "Philippines": [
        ""
      ],
      "Finland": [
        ""
      ],
      "France": [
        "Cancel·lar"
      ],
      "Gabon": [
        ""
      ],
      "Gambia": [
        ""
      ],
      "Georgia": [
        ""
      ],
      "Germany": [
        ""
      ],
      "Ghana": [
        ""
      ],
      "Greece": [
        ""
      ],
      "Grenada": [
        ""
      ],
      "Guatemala": [
        ""
      ],
      "Guinea-Bissau": [
        ""
      ],
      "Guinea": [
        ""
      ],
      "Guyana": [
        ""
      ],
      "Haiti": [
        ""
      ],
      "Honduras": [
        ""
      ],
      "Hungary": [
        ""
      ],
      "Kiribati": [
        ""
      ],
      "Iceland": [
        ""
      ],
      "India": [
        ""
      ],
      "Indonesia": [
        ""
      ],
      "Iran": [
        ""
      ],
      "Iraq": [
        ""
      ],
      "Ireland": [
        ""
      ],
      "Israel": [
        ""
      ],
      "Italy": [
        ""
      ],
      "Ivory Coast": [
        ""
      ],
      "Jamaica": [
        ""
      ],
      "Japan": [
        ""
      ],
      "Jordan": [
        ""
      ],
      "Kazakhstan": [
        ""
      ],
      "Kenya": [
        ""
      ],
      "Saint Kitts and Nevis": [
        ""
      ],
      "Kuwait": [
        ""
      ],
      "Kyrgyzstan": [
        ""
      ],
      "Laos": [
        ""
      ],
      "Latvia": [
        ""
      ],
      "Lebanon": [
        ""
      ],
      "Liberia": [
        ""
      ],
      "Libya": [
        ""
      ],
      "Liechtenstein": [
        ""
      ],
      "Lithuania": [
        ""
      ],
      "Luxembourg": [
        ""
      ],
      "North Macedonia": [
        ""
      ],
      "Madagascar": [
        ""
      ],
      "Malawi": [
        ""
      ],
      "Malaysia": [
        ""
      ],
      "Maldives": [
        ""
      ],
      "Mali": [
        ""
      ],
      "Malta": [
        ""
      ],
      "Marshall Islands": [
        ""
      ],
      "Mauritania": [
        ""
      ],
      "Mauritius": [
        ""
      ],
      "Mexico": [
        ""
      ],
      "Micronesia": [
        ""
      ],
      "Moldova": [
        ""
      ],
      "Monaco": [
        ""
      ],
      "Mongolia": [
        ""
      ],
      "Morocco": [
        ""
      ],
      "Lesotho": [
        ""
      ],
      "Mozambique": [
        ""
      ],
      "Namibia": [
        ""
      ],
      "Nauru": [
        ""
      ],
      "Nepal": [
        ""
      ],
      "New Zealand": [
        ""
      ],
      "Vanuatu": [
        ""
      ],
      "Nicaragua": [
        ""
      ],
      "Nigeria": [
        ""
      ],
      "Niger": [
        ""
      ],
      "North Korea": [
        ""
      ],
      "Norway": [
        ""
      ],
      "Oman": [
        ""
      ],
      "Pakistan": [
        ""
      ],
      "Palau": [
        ""
      ],
      "Panama": [
        ""
      ],
      "Papua New Guinea": [
        ""
      ],
      "Paraguay": [
        ""
      ],
      "Peru": [
        ""
      ],
      "Poland": [
        ""
      ],
      "Portugal": [
        ""
      ],
      "Qatar": [
        ""
      ],
      "Romania": [
        ""
      ],
      "Russia": [
        ""
      ],
      "Rwanda": [
        ""
      ],
      "Saint Lucia": [
        ""
      ],
      "El Salvador": [
        ""
      ],
      "Samoa": [
        ""
      ],
      "San Marino": [
        ""
      ],
      "São Tomé and Príncipe": [
        ""
      ],
      "Saudi Arabia": [
        ""
      ],
      "Senegal": [
        ""
      ],
      "Serbia": [
        ""
      ],
      "Seychelles": [
        ""
      ],
      "Sierra Leone": [
        ""
      ],
      "Singapore": [
        ""
      ],
      "Slovakia": [
        ""
      ],
      "Slovenia": [
        ""
      ],
      "Solomon Islands": [
        ""
      ],
      "Somalia": [
        ""
      ],
      "South Africa": [
        ""
      ],
      "South Korea": [
        ""
      ],
      "Spain": [
        ""
      ],
      "Sri Lanka": [
        ""
      ],
      "Sudan": [
        ""
      ],
      "Suriname": [
        ""
      ],
      "Eswatini": [
        ""
      ],
      "Sweden": [
        ""
      ],
      "Switzerland": [
        ""
      ],
      "Syria": [
        ""
      ],
      "Taiwan": [
        ""
      ],
      "Tajikistan": [
        ""
      ],
      "Tanzania": [
        ""
      ],
      "Thailand": [
        ""
      ],
      "Togo": [
        ""
      ],
      "Tonga": [
        ""
      ],
      "Trinidad and Tobago": [
        ""
      ],
      "Tunisia": [
        ""
      ],
      "Turkey": [
        ""
      ],
      "Tuvalu": [
        "valor"
      ],
      "Uganda": [
        ""
      ],
      "Ukraine": [
        ""
      ],
      "Uruguay": [
        ""
      ],
      "Uzbekistan": [
        ""
      ],
      "Venezuela": [
        ""
      ],
      "Vietnam": [
        ""
      ],
      "Yemen": [
        ""
      ],
      "Zambia": [
        ""
      ],
      "Zimbabwe": [
        ""
      ],
      "Afghan": [
        ""
      ],
      "Albanian": [
        ""
      ],
      "Algerian": [
        ""
      ],
      "American": [
        ""
      ],
      "Andorran": [
        ""
      ],
      "Angolan": [
        ""
      ],
      "Antiguan": [
        ""
      ],
      "Argentinean": [
        ""
      ],
      "Armenian": [
        ""
      ],
      "Australian": [
        ""
      ],
      "Austrian": [
        ""
      ],
      "Azerbaijani": [
        ""
      ],
      "Bahamian": [
        ""
      ],
      "Bahraini": [
        ""
      ],
      "Bangladeshi": [
        ""
      ],
      "Barbadian": [
        ""
      ],
      "Batswana": [
        ""
      ],
      "Belarusian": [
        ""
      ],
      "Belgian": [
        ""
      ],
      "Belizean": [
        ""
      ],
      "Beninese": [
        ""
      ],
      "Bhutanese": [
        ""
      ],
      "Bolivian": [
        ""
      ],
      "Bosnian": [
        ""
      ],
      "Brazilian": [
        ""
      ],
      "British": [
        ""
      ],
      "Bruneian": [
        ""
      ],
      "Bulgarian": [
        ""
      ],
      "Burkinabe": [
        ""
      ],
      "Burmese": [
        ""
      ],
      "Burundian": [
        ""
      ],
      "Cambodian": [
        ""
      ],
      "Cameroonian": [
        ""
      ],
      "Canadian": [
        ""
      ],
      "Cape Verdean": [
        ""
      ],
      "Central African": [
        ""
      ],
      "Chadian": [
        ""
      ],
      "Chilean": [
        ""
      ],
      "Chinese": [
        ""
      ],
      "Colombian": [
        ""
      ],
      "Comoran": [
        ""
      ],
      "Congolese": [
        ""
      ],
      "Costa Rican": [
        ""
      ],
      "Croatian": [
        ""
      ],
      "Cuban": [
        ""
      ],
      "Cypriot": [
        ""
      ],
      "Czech": [
        ""
      ],
      "Danish": [
        ""
      ],
      "Djiboutian": [
        ""
      ],
      "Dominican": [
        "Denominació"
      ],
      "Dutch": [
        ""
      ],
      "East Timorese": [
        ""
      ],
      "Ecuadorean": [
        ""
      ],
      "Egyptian": [
        ""
      ],
      "Emirati": [
        ""
      ],
      "Equatorial Guinean": [
        ""
      ],
      "Eritrean": [
        ""
      ],
      "Estonian": [
        ""
      ],
      "Ethiopian": [
        ""
      ],
      "Fijian": [
        ""
      ],
      "Filipino": [
        ""
      ],
      "Finnish": [
        ""
      ],
      "French": [
        ""
      ],
      "Gabonese": [
        ""
      ],
      "Gambian": [
        ""
      ],
      "Georgian": [
        ""
      ],
      "German": [
        ""
      ],
      "Ghanaian": [
        ""
      ],
      "Greek": [
        ""
      ],
      "Grenadian": [
        ""
      ],
      "Guatemalan": [
        ""
      ],
      "Guinea-Bissauan": [
        ""
      ],
      "Guinean": [
        ""
      ],
      "Guyanese": [
        ""
      ],
      "Haitian": [
        ""
      ],
      "Honduran": [
        ""
      ],
      "Hungarian": [
        ""
      ],
      "I-Kiribati": [
        ""
      ],
      "Icelander": [
        ""
      ],
      "Indian": [
        ""
      ],
      "Indonesian": [
        ""
      ],
      "Iranian": [
        ""
      ],
      "Iraqi": [
        ""
      ],
      "Irish": [
        ""
      ],
      "Israeli": [
        ""
      ],
      "Italian": [
        ""
      ],
      "Ivorian": [
        ""
      ],
      "Jamaican": [
        ""
      ],
      "Japanese": [
        ""
      ],
      "Jordanian": [
        ""
      ],
      "Kazakhstani": [
        ""
      ],
      "Kenyan": [
        ""
      ],
      "Kittian and Nevisian": [
        ""
      ],
      "Kuwaiti": [
        ""
      ],
      "Kyrgyz": [
        ""
      ],
      "Laotian": [
        ""
      ],
      "Latvian": [
        ""
      ],
      "Lebanese": [
        ""
      ],
      "Liberian": [
        ""
      ],
      "Libyan": [
        ""
      ],
      "Liechtensteiner": [
        ""
      ],
      "Lithuanian": [
        ""
      ],
      "Luxembourger": [
        ""
      ],
      "Macedonian": [
        ""
      ],
      "Malagasy": [
        ""
      ],
      "Malawian": [
        ""
      ],
      "Malaysian": [
        ""
      ],
      "Maldivian": [
        ""
      ],
      "Malian": [
        ""
      ],
      "Maltese": [
        ""
      ],
      "Marshallese": [
        ""
      ],
      "Mauritanian": [
        ""
      ],
      "Mauritian": [
        ""
      ],
      "Mexican": [
        ""
      ],
      "Micronesian": [
        ""
      ],
      "Moldovan": [
        ""
      ],
      "Monacan": [
        ""
      ],
      "Mongolian": [
        ""
      ],
      "Moroccan": [
        ""
      ],
      "Mosotho": [
        ""
      ],
      "Mozambican": [
        ""
      ],
      "Namibian": [
        ""
      ],
      "Nauruan": [
        ""
      ],
      "Nepalese": [
        ""
      ],
      "New Zealander": [
        ""
      ],
      "Ni-Vanuatu": [
        ""
      ],
      "Nicaraguan": [
        ""
      ],
      "Nigerian": [
        ""
      ],
      "Nigerien": [
        ""
      ],
      "North Korean": [
        ""
      ],
      "Norwegian": [
        ""
      ],
      "Omani": [
        ""
      ],
      "Pakistani": [
        ""
      ],
      "Palauan": [
        ""
      ],
      "Panamanian": [
        ""
      ],
      "Papua New Guinean": [
        ""
      ],
      "Paraguayan": [
        ""
      ],
      "Peruvian": [
        ""
      ],
      "Polish": [
        ""
      ],
      "Portuguese": [
        ""
      ],
      "Qatari": [
        ""
      ],
      "Romanian": [
        ""
      ],
      "Russian": [
        ""
      ],
      "Rwandan": [
        ""
      ],
      "Saint Lucian": [
        ""
      ],
      "Salvadoran": [
        ""
      ],
      "Samoan": [
        ""
      ],
      "San Marinese": [
        ""
      ],
      "Sao Tomean": [
        ""
      ],
      "Saudi": [
        ""
      ],
      "Senegalese": [
        ""
      ],
      "Serbian": [
        ""
      ],
      "Seychellois": [
        ""
      ],
      "Sierra Leonean": [
        ""
      ],
      "Singaporean": [
        ""
      ],
      "Slovakian": [
        ""
      ],
      "Slovenian": [
        ""
      ],
      "Solomon Islander": [
        ""
      ],
      "Somali": [
        ""
      ],
      "South African": [
        ""
      ],
      "South Korean": [
        ""
      ],
      "Spanish": [
        ""
      ],
      "Sri Lankan": [
        ""
      ],
      "Sudanese": [
        ""
      ],
      "Surinamer": [
        ""
      ],
      "Swazi": [
        ""
      ],
      "Swedish": [
        ""
      ],
      "Swiss": [
        ""
      ],
      "Syrian": [
        ""
      ],
      "Taiwanese": [
        ""
      ],
      "Tajik": [
        ""
      ],
      "Tanzanian": [
        ""
      ],
      "Thai": [
        ""
      ],
      "Togolese": [
        ""
      ],
      "Tongan": [
        ""
      ],
      "Trinidadian or Tobagonian": [
        ""
      ],
      "Tunisian": [
        ""
      ],
      "Turkish": [
        ""
      ],
      "Tuvaluan": [
        "valor"
      ],
      "Ugandan": [
        ""
      ],
      "Ukrainian": [
        ""
      ],
      "Uruguayan": [
        ""
      ],
      "Uzbekistani": [
        ""
      ],
      "Venezuelan": [
        ""
      ],
      "Vietnamese": [
        ""
      ],
      "Yemenite": [
        ""
      ],
      "Zambian": [
        ""
      ],
      "Zimbabwean": [
        ""
      ],
      "🇦🇫": [
        ""
      ],
      "🇦🇱": [
        ""
      ],
      "🇩🇿": [
        ""
      ],
      "🇺🇸": [
        ""
      ],
      "🇦🇩": [
        ""
      ],
      "🇦🇴": [
        ""
      ],
      "🇦🇬": [
        ""
      ],
      "🇦🇷": [
        ""
      ],
      "🇦🇲": [
        ""
      ],
      "🇦🇺": [
        ""
      ],
      "🇦🇹": [
        ""
      ],
      "🇦🇿": [
        ""
      ],
      "🇧🇸": [
        ""
      ],
      "🇧🇭": [
        ""
      ],
      "🇧🇩": [
        ""
      ],
      "🇧🇧": [
        ""
      ],
      "🇧🇼": [
        ""
      ],
      "🇧🇾": [
        ""
      ],
      "🇧🇪": [
        ""
      ],
      "🇧🇿": [
        ""
      ],
      "🇧🇯": [
        ""
      ],
      "🇧🇹": [
        ""
      ],
      "🇧🇴": [
        ""
      ],
      "🇧🇦": [
        ""
      ],
      "🇧🇷": [
        ""
      ],
      "🇬🇧": [
        ""
      ],
      "🇧🇳": [
        ""
      ],
      "🇧🇬": [
        ""
      ],
      "🇧🇫": [
        ""
      ],
      "🇲🇲": [
        ""
      ],
      "🇧🇮": [
        ""
      ],
      "🇰🇭": [
        ""
      ],
      "🇨🇲": [
        ""
      ],
      "🇨🇦": [
        ""
      ],
      "🇨🇻": [
        ""
      ],
      "🇨🇫": [
        ""
      ],
      "🇨🇱": [
        ""
      ],
      "🇨🇳": [
        ""
      ],
      "🇨🇴": [
        ""
      ],
      "🇨🇩": [
        ""
      ],
      "🇨🇬": [
        ""
      ],
      "🇨🇷": [
        ""
      ],
      "🇨🇺": [
        ""
      ],
      "🇨🇾": [
        ""
      ],
      "🇨🇿": [
        ""
      ],
      "🇩🇰": [
        ""
      ],
      "🇩🇯": [
        ""
      ],
      "🇩🇴": [
        ""
      ],
      "🇳🇱": [
        ""
      ],
      "🇹🇱": [
        ""
      ],
      "🇪🇨": [
        ""
      ],
      "🇪🇬": [
        ""
      ],
      "🇦🇪": [
        ""
      ],
      "🇬🇶": [
        ""
      ],
      "🇪🇷": [
        ""
      ],
      "🇪🇪": [
        ""
      ],
      "🇪🇹": [
        ""
      ],
      "🇫🇯": [
        ""
      ],
      "🇵🇭": [
        ""
      ],
      "🇫🇮": [
        ""
      ],
      "🇫🇷": [
        ""
      ],
      "🇬🇦": [
        ""
      ],
      "🇬🇲": [
        ""
      ],
      "🇬🇪": [
        ""
      ],
      "UAE Dirham": [
        ""
      ],
      "Afghani": [
        ""
      ],
      "Lek": [
        ""
      ],
      "Armenian Dram": [
        ""
      ],
      "Netherlands Antillean Guilder": [
        ""
      ],
      "Kwanza": [
        ""
      ],
      "Argentine Peso": [
        ""
      ],
      "Australian Dollar": [
        ""
      ],
      "Aruban Florin": [
        ""
      ],
      "Azerbaijan Manat": [
        ""
      ],
      "Convertible Mark": [
        ""
      ],
      "Barbados Dollar": [
        ""
      ],
      "Taka": [
        ""
      ],
      "Bulgarian Lev": [
        ""
      ],
      "Bahraini Dinar": [
        ""
      ],
      "Burundi Franc": [
        ""
      ],
      "Bermudian Dollar": [
        ""
      ],
      "Brunei Dollar": [
        ""
      ],
      "Boliviano": [
        ""
      ],
      "Mvdol": [
        ""
      ],
      "Brazilian Real": [
        ""
      ],
      "Bahamian Dollar": [
        ""
      ],
      "Ngultrum": [
        ""
      ],
      "Pula": [
        ""
      ],
      "Belarusian Ruble": [
        ""
      ],
      "Belize Dollar": [
        ""
      ],
      "Canadian Dollar": [
        ""
      ],
      "Congolese Franc": [
        ""
      ],
      "WIR Euro": [
        ""
      ],
      "Swiss Franc": [
        ""
      ],
      "WIR Franc": [
        ""
      ],
      "Unidad de Fomento": [
        ""
      ],
      "Chilean Peso": [
        ""
      ],
      "Yuan Renminbi": [
        ""
      ],
      "Colombian Peso": [
        ""
      ],
      "Unidad de Valor Real": [
        ""
      ],
      "Costa Rican Colon": [
        ""
      ],
      "Cuban Peso": [
        ""
      ],
      "Cabo Verde Escudo": [
        ""
      ],
      "Czech Koruna": [
        ""
      ],
      "Djibouti Franc": [
        ""
      ],
      "Danish Krone": [
        ""
      ],
      "Dominican Peso": [
        ""
      ],
      "Algerian Dinar": [
        ""
      ],
      "Egyptian Pound": [
        ""
      ],
      "Nakfa": [
        ""
      ],
      "Ethiopian Birr": [
        ""
      ],
      "Euro": [
        ""
      ],
      "Fiji Dollar": [
        ""
      ],
      "Falkland Islands Pound": [
        ""
      ],
      "Pound Sterling": [
        ""
      ],
      "Lari": [
        ""
      ],
      "Ghana Cedi": [
        ""
      ],
      "Gibraltar Pound": [
        ""
      ],
      "Dalasi": [
        ""
      ],
      "Guinean Franc": [
        ""
      ],
      "Quetzal": [
        ""
      ],
      "Guyana Dollar": [
        ""
      ],
      "Hong Kong Dollar": [
        ""
      ],
      "Lempira": [
        ""
      ],
      "Gourde": [
        ""
      ],
      "Forint": [
        ""
      ],
      "Rupiah": [
        ""
      ],
      "New Israeli Sheqel": [
        ""
      ],
      "Indian Rupee": [
        ""
      ],
      "Iraqi Dinar": [
        ""
      ],
      "Iranian Rial": [
        ""
      ],
      "Iceland Krona": [
        ""
      ],
      "Jamaican Dollar": [
        ""
      ],
      "Jordanian Dinar": [
        ""
      ],
      "Yen": [
        ""
      ],
      "Kenyan Shilling": [
        ""
      ],
      "Som": [
        ""
      ],
      "Riel": [
        ""
      ],
      "Comorian Franc ": [
        ""
      ],
      "North Korean Won": [
        ""
      ],
      "Won": [
        ""
      ],
      "Kuwaiti Dinar": [
        ""
      ],
      "Cayman Islands Dollar": [
        ""
      ],
      "Tenge": [
        ""
      ],
      "Lao Kip": [
        ""
      ],
      "Lebanese Pound": [
        ""
      ],
      "Sri Lanka Rupee": [
        ""
      ],
      "Liberian Dollar": [
        ""
      ],
      "Loti": [
        ""
      ],
      "Libyan Dinar": [
        ""
      ],
      "Moroccan Dirham": [
        ""
      ],
      "Moldovan Leu": [
        ""
      ],
      "Malagasy Ariary": [
        ""
      ],
      "Denar": [
        ""
      ],
      "Kyat": [
        ""
      ],
      "Tugrik": [
        ""
      ],
      "Pataca": [
        ""
      ],
      "Ouguiya": [
        ""
      ],
      "Mauritius Rupee": [
        ""
      ],
      "Rufiyaa": [
        ""
      ],
      "Malawi Kwacha": [
        ""
      ],
      "Mexican Peso": [
        ""
      ],
      "Mexican Unidad de Inversion (UDI)": [
        ""
      ],
      "Malaysian Ringgit": [
        ""
      ],
      "Mozambique Metical": [
        ""
      ],
      "Namibia Dollar": [
        ""
      ],
      "Naira": [
        ""
      ],
      "Cordoba Oro": [
        ""
      ],
      "Norwegian Krone": [
        ""
      ],
      "Nepalese Rupee": [
        ""
      ],
      "New Zealand Dollar": [
        ""
      ],
      "Rial Omani": [
        ""
      ],
      "Balboa": [
        ""
      ],
      "Sol": [
        "antic"
      ],
      "Kina": [
        ""
      ],
      "Philippine Peso": [
        ""
      ],
      "Pakistan Rupee": [
        ""
      ],
      "Zloty": [
        ""
      ],
      "Guarani": [
        ""
      ],
      "Qatari Rial": [
        ""
      ],
      "Romanian Leu": [
        ""
      ],
      "Serbian Dinar": [
        ""
      ],
      "Russian Ruble": [
        ""
      ],
      "Rwanda Franc": [
        ""
      ],
      "Saudi Riyal": [
        ""
      ],
      "Solomon Islands Dollar": [
        ""
      ],
      "Seychelles Rupee": [
        ""
      ],
      "Sudanese Pound": [
        ""
      ],
      "Swedish Krona": [
        ""
      ],
      "Singapore Dollar": [
        ""
      ],
      "Saint Helena Pound": [
        ""
      ],
      "Leone": [
        ""
      ],
      "Somali Shilling": [
        ""
      ],
      "Surinam Dollar": [
        ""
      ],
      "South Sudanese Pound": [
        ""
      ],
      "Dobra": [
        ""
      ],
      "El Salvador Colon": [
        ""
      ],
      "Syrian Pound": [
        ""
      ],
      "Lilangeni": [
        ""
      ],
      "Baht": [
        ""
      ],
      "Somoni": [
        ""
      ],
      "Turkmenistan New Manat": [
        ""
      ],
      "Tunisian Dinar": [
        ""
      ],
      "Pa’anga": [
        ""
      ],
      "Turkish Lira": [
        ""
      ],
      "Trinidad and Tobago Dollar": [
        ""
      ],
      "New Taiwan Dollar": [
        ""
      ],
      "Tanzanian Shilling": [
        ""
      ],
      "Hryvnia": [
        ""
      ],
      "Uganda Shilling": [
        ""
      ],
      "US Dollar": [
        ""
      ],
      "US Dollar (Next day)": [
        ""
      ],
      "Uruguay Peso en Unidades Indexadas (UI)": [
        ""
      ],
      "Peso Uruguayo": [
        ""
      ],
      "Unidad Previsional": [
        ""
      ],
      "Uzbekistan Sum": [
        ""
      ],
      "Bolívar Soberano": [
        ""
      ],
      "Dong": [
        ""
      ],
      "Vatu": [
        ""
      ],
      "Tala": [
        ""
      ],
      "CFA Franc BEAC": [
        ""
      ],
      "East Caribbean Dollar": [
        ""
      ],
      "CFA Franc BCEAO": [
        ""
      ],
      "CFP Franc": [
        ""
      ],
      "Sucre": [
        ""
      ],
      "Yemeni Rial": [
        ""
      ],
      "Rand": [
        ""
      ],
      "Zambian Kwacha": [
        ""
      ],
      "Zimbabwe Gold": [
        ""
      ],
      "Afar": [
        ""
      ],
      "Abkhazian": [
        ""
      ],
      "Achinese": [
        ""
      ],
      "Acoli": [
        ""
      ],
      "Adangme": [
        ""
      ],
      "Adyghe": [
        ""
      ],
      "Avestan": [
        ""
      ],
      "Tunisian Arabic": [
        ""
      ],
      "Afrikaans": [
        ""
      ],
      "Afrihili": [
        ""
      ],
      "Aghem": [
        ""
      ],
      "Ainu": [
        ""
      ],
      "Akan": [
        ""
      ],
      "Akkadian": [
        ""
      ],
      "Alabama": [
        ""
      ],
      "Aleut": [
        ""
      ],
      "Gheg Albanian": [
        ""
      ],
      "Southern Altai": [
        ""
      ],
      "Amarik": [
        ""
      ],
      "Aragonese": [
        ""
      ],
      "Old English": [
        ""
      ],
      "Angika": [
        ""
      ],
      "Modern Standard Arabic": [
        ""
      ],
      "Arabik": [
        ""
      ],
      "Aramaic": [
        ""
      ],
      "Mapuche": [
        ""
      ],
      "Araona": [
        ""
      ],
      "Arapaho": [
        ""
      ],
      "Algerian Arabic": [
        ""
      ],
      "Arawak": [
        ""
      ],
      "Moroccan Arabic": [
        ""
      ],
      "Egyptian Arabic": [
        ""
      ],
      "Asu": [
        ""
      ],
      "Assamese": [
        ""
      ],
      "American Sign Language": [
        ""
      ],
      "Asturian": [
        ""
      ],
      "Avaric": [
        ""
      ],
      "Kotava": [
        ""
      ],
      "Awadhi": [
        ""
      ],
      "Aymara": [
        ""
      ],
      "South Azerbaijani": [
        ""
      ],
      "Bashkir": [
        ""
      ],
      "Baluchi": [
        ""
      ],
      "Balinese": [
        ""
      ],
      "Bavarian": [
        ""
      ],
      "Basaa": [
        ""
      ],
      "Bamun": [
        ""
      ],
      "Batak Toba": [
        ""
      ],
      "Ghomala": [
        ""
      ],
      "Belarus kasa": [
        ""
      ],
      "Beja": [
        ""
      ],
      "Bemba": [
        ""
      ],
      "Betawi": [
        ""
      ],
      "Bena": [
        ""
      ],
      "Bafut": [
        ""
      ],
      "Badaga": [
        ""
      ],
      "Bɔlgeria kasa": [
        ""
      ],
      "Bhojpuri": [
        ""
      ],
      "Bislama": [
        ""
      ],
      "Bikol": [
        ""
      ],
      "Bini": [
        ""
      ],
      "Banjar": [
        ""
      ],
      "Kom": [
        ""
      ],
      "Siksika": [
        ""
      ],
      "Bambara": [
        ""
      ],
      "Bengali kasa": [
        ""
      ],
      "Tibetan": [
        ""
      ],
      "Bishnupriya": [
        ""
      ],
      "Bakhtiari": [
        ""
      ],
      "Braj": [
        ""
      ],
      "Breton": [
        ""
      ],
      "Brahui": [
        ""
      ],
      "Bodo": [
        ""
      ],
      "Akoose": [
        ""
      ],
      "Buriat": [
        ""
      ],
      "Buginese": [
        ""
      ],
      "Bulu": [
        ""
      ],
      "Blin": [
        ""
      ],
      "Medumba": [
        ""
      ],
      "Catalan": [
        ""
      ],
      "Caddo": [
        ""
      ],
      "Carib": [
        ""
      ],
      "Cayuga": [
        ""
      ],
      "Atsam": [
        ""
      ],
      "Cebuano": [
        ""
      ],
      "Chechen": [
        ""
      ],
      "Chiga": [
        ""
      ],
      "Chibcha": [
        ""
      ],
      "Chamorro": [
        ""
      ],
      "Chagatai": [
        ""
      ],
      "Chuukese": [
        ""
      ],
      "Mari": [
        ""
      ],
      "Chinook Jargon": [
        ""
      ],
      "Choctaw": [
        ""
      ],
      "Chipewyan": [
        ""
      ],
      "Cherokee": [
        ""
      ],
      "Cheyenne": [
        ""
      ],
      "Central Kurdish": [
        ""
      ],
      "Corsican": [
        ""
      ],
      "Coptic": [
        ""
      ],
      "Capiznon": [
        ""
      ],
      "Cree": [
        ""
      ],
      "Crimean Turkish": [
        ""
      ],
      "Kashubian": [
        ""
      ],
      "Kyɛk kasa": [
        ""
      ],
      "Church Slavic": [
        ""
      ],
      "Chuvash": [
        ""
      ],
      "Welsh": [
        ""
      ],
      "Dakota": [
        ""
      ],
      "Dargwa": [
        ""
      ],
      "Taita": [
        ""
      ],
      "Austrian German": [
        ""
      ],
      "Swiss High German": [
        ""
      ],
      "Gyaaman": [
        ""
      ],
      "Delaware": [
        ""
      ],
      "Slave": [
        ""
      ],
      "Dogrib": [
        ""
      ],
      "Dinka": [
        ""
      ],
      "Zarma": [
        ""
      ],
      "Dogri": [
        ""
      ],
      "Lower Sorbian": [
        ""
      ],
      "Central Dusun": [
        ""
      ],
      "Duala": [
        ""
      ],
      "Middle Dutch": [
        ""
      ],
      "Divehi": [
        ""
      ],
      "Jola-Fonyi": [
        ""
      ],
      "Dyula": [
        ""
      ],
      "Dzongkha": [
        ""
      ],
      "Dazaga": [
        ""
      ],
      "Embu": [
        ""
      ],
      "Ewe": [
        ""
      ],
      "Efik": [
        ""
      ],
      "Emilian": [
        ""
      ],
      "Ancient Egyptian": [
        ""
      ],
      "Ekajuk": [
        ""
      ],
      "Greek kasa": [
        ""
      ],
      "Elamite": [
        ""
      ],
      "Australian English": [
        ""
      ],
      "Borɔfo": [
        ""
      ],
      "Canadian English": [
        ""
      ],
      "British English": [
        ""
      ],
      "Middle English": [
        ""
      ],
      "American English": [
        ""
      ],
      "Esperanto": [
        "Operació"
      ],
      "Latin American Spanish": [
        ""
      ],
      "European Spanish": [
        ""
      ],
      "Mexican Spanish": [
        ""
      ],
      "Spain kasa": [
        ""
      ],
      "Central Yupik": [
        ""
      ],
      "Basque": [
        ""
      ],
      "Ewondo": [
        ""
      ],
      "Extremaduran": [
        ""
      ],
      "Fang": [
        ""
      ],
      "Pɛɛhyia kasa": [
        ""
      ],
      "Fanti": [
        ""
      ],
      "Fulah": [
        ""
      ],
      "Tornedalen Finnish": [
        ""
      ],
      "Faroese": [
        ""
      ],
      "Fon": [
        ""
      ],
      "Canadian French": [
        ""
      ],
      "Cajun French": [
        "Divisa"
      ],
      "Swiss French": [
        ""
      ],
      "Frɛnkye": [
        ""
      ],
      "Middle French": [
        ""
      ],
      "Old French": [
        ""
      ],
      "Arpitan": [
        ""
      ],
      "Northern Frisian": [
        ""
      ],
      "Eastern Frisian": [
        ""
      ],
      "Friulian": [
        ""
      ],
      "Western Frisian": [
        ""
      ],
      "Ga": [
        ""
      ],
      "Gagauz": [
        ""
      ],
      "Gan Chinese": [
        ""
      ],
      "Gayo": [
        ""
      ],
      "Gbaya": [
        ""
      ],
      "Zoroastrian Dari": [
        ""
      ],
      "Scottish Gaelic": [
        ""
      ],
      "Geez": [
        ""
      ],
      "Gilbertese": [
        ""
      ],
      "Galician": [
        ""
      ],
      "Gilaki": [
        ""
      ],
      "Middle High German": [
        ""
      ],
      "Old High German": [
        ""
      ],
      "Goan Konkani": [
        ""
      ],
      "Gondi": [
        ""
      ],
      "Gorontalo": [
        ""
      ],
      "Gothic": [
        ""
      ],
      "Grebo": [
        ""
      ],
      "Ancient Greek": [
        ""
      ],
      "Swiss German": [
        ""
      ],
      "Wayuu": [
        ""
      ],
      "Gujarati": [
        ""
      ],
      "Frafra": [
        ""
      ],
      "Gusii": [
        ""
      ],
      "Manx": [
        ""
      ],
      "Gwichʼin": [
        ""
      ],
      "Hausa": [
        ""
      ],
      "Haida": [
        ""
      ],
      "Hakka Chinese": [
        ""
      ],
      "Hawaiian": [
        ""
      ],
      "Hebrew": [
        ""
      ],
      "Fiji Hindi": [
        ""
      ],
      "Hindi": [
        ""
      ],
      "Hiligaynon": [
        ""
      ],
      "Hittite": [
        ""
      ],
      "Hmong": [
        ""
      ],
      "Hiri Motu": [
        ""
      ],
      "Upper Sorbian": [
        ""
      ],
      "Xiang Chinese": [
        ""
      ],
      "Hangri kasa": [
        ""
      ],
      "Hupa": [
        ""
      ],
      "Herero": [
        ""
      ],
      "Interlingua": [
        ""
      ],
      "Iban": [
        ""
      ],
      "Ibibio": [
        ""
      ],
      "Indonihyia kasa": [
        ""
      ],
      "Interlingue": [
        ""
      ],
      "Igbo": [
        ""
      ],
      "Sichuan Yi": [
        ""
      ],
      "Inupiaq": [
        ""
      ],
      "Iloko": [
        ""
      ],
      "Ingush": [
        ""
      ],
      "Ido": [
        ""
      ],
      "Icelandic": [
        ""
      ],
      "Italy kasa": [
        ""
      ],
      "Inuktitut": [
        ""
      ],
      "Ingrian": [
        ""
      ],
      "Gyapan kasa": [
        ""
      ],
      "Jamaican Creole English": [
        ""
      ],
      "Lojban": [
        ""
      ],
      "Ngomba": [
        ""
      ],
      "Machame": [
        ""
      ],
      "Judeo-Persian": [
        ""
      ],
      "Judeo-Arabic": [
        ""
      ],
      "Jutish": [
        ""
      ],
      "Gyabanis kasa": [
        ""
      ],
      "Kara-Kalpak": [
        ""
      ],
      "Kabyle": [
        ""
      ],
      "Kachin": [
        ""
      ],
      "Jju": [
        ""
      ],
      "Kamba": [
        ""
      ],
      "Kawi": [
        ""
      ],
      "Kabardian": [
        ""
      ],
      "Kanembu": [
        ""
      ],
      "Tyap": [
        ""
      ],
      "Makonde": [
        ""
      ],
      "Kabuverdianu": [
        ""
      ],
      "Kenyang": [
        ""
      ],
      "Koro": [
        ""
      ],
      "Kongo": [
        ""
      ],
      "Kaingang": [
        ""
      ],
      "Khasi": [
        ""
      ],
      "Khotanese": [
        ""
      ],
      "Koyra Chiini": [
        ""
      ],
      "Khowar": [
        ""
      ],
      "Kikuyu": [
        ""
      ],
      "Kirmanjki": [
        ""
      ],
      "Kuanyama": [
        ""
      ],
      "Kako": [
        ""
      ],
      "Kazakh": [
        ""
      ],
      "Kalaallisut": [
        ""
      ],
      "Kalenjin": [
        ""
      ],
      "Kimbundu": [
        ""
      ],
      "Kambodia kasa": [
        ""
      ],
      "Kannada": [
        ""
      ],
      "Komi-Permyak": [
        ""
      ],
      "Konkani": [
        ""
      ],
      "Korea kasa": [
        ""
      ],
      "Kosraean": [
        ""
      ],
      "Kpelle": [
        ""
      ],
      "Karachay-Balkar": [
        ""
      ],
      "Krio": [
        ""
      ],
      "Kinaray-a": [
        ""
      ],
      "Kanuri": [
        ""
      ],
      "Karelian": [
        ""
      ],
      "Kurukh": [
        ""
      ],
      "Shambala": [
        ""
      ],
      "Bafia": [
        ""
      ],
      "Colognian": [
        ""
      ],
      "Kashmiri": [
        ""
      ],
      "Kurdish": [
        ""
      ],
      "Kumyk": [
        ""
      ],
      "Kutenai": [
        ""
      ],
      "Komi": [
        ""
      ],
      "Cornish": [
        ""
      ],
      "Ladino": [
        "Carregant"
      ],
      "Langi": [
        ""
      ],
      "Lahnda": [
        ""
      ],
      "Latin": [
        ""
      ],
      "Lamba": [
        ""
      ],
      "Luxembourgish": [
        ""
      ],
      "Lezghian": [
        ""
      ],
      "Lingua Franca Nova": [
        ""
      ],
      "Ganda": [
        ""
      ],
      "Ligurian": [
        ""
      ],
      "Limburgish": [
        ""
      ],
      "Livonian": [
        ""
      ],
      "Lakota": [
        ""
      ],
      "Lombard": [
        ""
      ],
      "Lingala": [
        ""
      ],
      "Lao": [
        ""
      ],
      "Mongo": [
        ""
      ],
      "Lozi": [
        ""
      ],
      "Latgalian": [
        ""
      ],
      "Luba-Lulua": [
        ""
      ],
      "Luiseno": [
        ""
      ],
      "Luba-Katanga": [
        ""
      ],
      "Lunda": [
        ""
      ],
      "Luo": [
        ""
      ],
      "Mizo": [
        ""
      ],
      "Luyia": [
        ""
      ],
      "Literary Chinese": [
        ""
      ],
      "Laz": [
        ""
      ],
      "Madurese": [
        "resetejar"
      ],
      "Mafa": [
        ""
      ],
      "Magahi": [
        ""
      ],
      "Maithili": [
        ""
      ],
      "Makasar": [
        ""
      ],
      "Mandingo": [
        "Carregant"
      ],
      "Masai": [
        ""
      ],
      "Maba": [
        ""
      ],
      "Moksha": [
        ""
      ],
      "Mandar": [
        ""
      ],
      "Mende": [
        ""
      ],
      "Meru": [
        ""
      ],
      "Morisyen": [
        ""
      ],
      "Middle Irish": [
        ""
      ],
      "Makhuwa-Meetto": [
        ""
      ],
      "Metaʼ": [
        ""
      ],
      "Micmac": [
        ""
      ],
      "Maori": [
        ""
      ],
      "Minangkabau": [
        ""
      ],
      "Malayalam": [
        ""
      ],
      "Manchu": [
        ""
      ],
      "Manipuri": [
        ""
      ],
      "Mohawk": [
        ""
      ],
      "Mossi": [
        ""
      ],
      "Western Mari": [
        ""
      ],
      "Marathi": [
        ""
      ],
      "Malay kasa": [
        ""
      ],
      "Mundang": [
        ""
      ],
      "Multiple Languages": [
        "Llengua actual"
      ],
      "Creek": [
        ""
      ],
      "Mirandese": [
        ""
      ],
      "Marwari": [
        ""
      ],
      "Mentawai": [
        ""
      ],
      "Bɛɛmis kasa": [
        ""
      ],
      "Myene": [
        ""
      ],
      "Erzya": [
        ""
      ],
      "Mazanderani": [
        ""
      ],
      "Min Nan Chinese": [
        ""
      ],
      "Neapolitan": [
        ""
      ],
      "Nama": [
        ""
      ],
      "Norwegian Bokmål": [
        ""
      ],
      "North Ndebele": [
        ""
      ],
      "Low German": [
        ""
      ],
      "Nɛpal kasa": [
        ""
      ],
      "Newari": [
        ""
      ],
      "Ndonga": [
        ""
      ],
      "Nias": [
        ""
      ],
      "Niuean": [
        ""
      ],
      "Ao Naga": [
        ""
      ],
      "Flemish": [
        ""
      ],
      "Dɛɛkye": [
        ""
      ],
      "Kwasio": [
        ""
      ],
      "Ngiemboon": [
        ""
      ],
      "Norwegian Nynorsk": [
        ""
      ],
      "Nogai": [
        ""
      ],
      "Old Norse": [
        ""
      ],
      "Novial": [
        ""
      ],
      "NʼKo": [
        ""
      ],
      "South Ndebele": [
        ""
      ],
      "Northern Sotho": [
        ""
      ],
      "Nuer": [
        ""
      ],
      "Navajo": [
        ""
      ],
      "Classical Newari": [
        ""
      ],
      "Nyamwezi": [
        ""
      ],
      "Nyankole": [
        ""
      ],
      "Nyanja": [
        ""
      ],
      "Nyoro": [
        ""
      ],
      "Nzima": [
        ""
      ],
      "Occitan": [
        ""
      ],
      "Ojibwa": [
        ""
      ],
      "Oromo": [
        ""
      ],
      "Oriya": [
        ""
      ],
      "Osage": [
        ""
      ],
      "Ossetic": [
        ""
      ],
      "Ottoman Turkish": [
        ""
      ],
      "Pangasinan": [
        ""
      ],
      "Pahlavi": [
        ""
      ],
      "Pampanga": [
        ""
      ],
      "Papiamento": [
        "Pagament"
      ],
      "Pungyabi kasa": [
        ""
      ],
      "Picard": [
        ""
      ],
      "Pennsylvania German": [
        ""
      ],
      "Plautdietsch": [
        ""
      ],
      "Old Persian": [
        ""
      ],
      "Palatine German": [
        ""
      ],
      "Phoenician": [
        ""
      ],
      "Pali": [
        ""
      ],
      "Pɔland kasa": [
        ""
      ],
      "Piedmontese": [
        ""
      ],
      "Pontic": [
        ""
      ],
      "Pohnpeian": [
        ""
      ],
      "Prussian": [
        ""
      ],
      "Old Provençal": [
        ""
      ],
      "Pashto": [
        ""
      ],
      "Brazilian Portuguese": [
        ""
      ],
      "Pɔɔtugal kasa": [
        ""
      ],
      "European Portuguese": [
        ""
      ],
      "Kʼicheʼ": [
        ""
      ],
      "Chimborazo Highland Quichua": [
        ""
      ],
      "Quechua": [
        ""
      ],
      "Rajasthani": [
        ""
      ],
      "Rapanui": [
        ""
      ],
      "Rarotongan": [
        ""
      ],
      "Romagnol": [
        ""
      ],
      "Riffian": [
        ""
      ],
      "Romansh": [
        ""
      ],
      "Rundi": [
        ""
      ],
      "Rombo": [
        ""
      ],
      "Moldavian": [
        ""
      ],
      "Romany": [
        ""
      ],
      "Root": [
        ""
      ],
      "Romenia kasa": [
        ""
      ],
      "Rotuman": [
        ""
      ],
      "Rusyn": [
        ""
      ],
      "Roviana": [
        ""
      ],
      "Aromanian": [
        ""
      ],
      "Rahyia kasa": [
        ""
      ],
      "Rwa": [
        ""
      ],
      "Rewanda kasa": [
        ""
      ],
      "Sandawe": [
        ""
      ],
      "Sakha": [
        ""
      ],
      "Samaritan Aramaic": [
        ""
      ],
      "Samburu": [
        ""
      ],
      "Sanskrit": [
        ""
      ],
      "Sasak": [
        ""
      ],
      "Santali": [
        ""
      ],
      "Saurashtra": [
        ""
      ],
      "Ngambay": [
        ""
      ],
      "Sangu": [
        ""
      ],
      "Sicilian": [
        ""
      ],
      "Scots": [
        ""
      ],
      "Sardinian": [
        ""
      ],
      "Sassarese Sardinian": [
        ""
      ],
      "Sindhi": [
        ""
      ],
      "Seneca": [
        ""
      ],
      "Sena": [
        ""
      ],
      "Seri": [
        ""
      ],
      "Selkup": [
        ""
      ],
      "Northern Sami": [
        ""
      ],
      "Koyraboro Senni": [
        ""
      ],
      "Old Irish": [
        ""
      ],
      "Sango": [
        ""
      ],
      "Samogitian": [
        ""
      ],
      "Tachelhit": [
        ""
      ],
      "Shan": [
        ""
      ],
      "Serbo-Croatian": [
        ""
      ],
      "Chadian Arabic": [
        ""
      ],
      "Sidamo": [
        ""
      ],
      "Sinhala": [
        ""
      ],
      "Slovak": [
        ""
      ],
      "Lower Silesian": [
        ""
      ],
      "Selayar": [
        ""
      ],
      "Southern Sami": [
        ""
      ],
      "Lule Sami": [
        ""
      ],
      "Inari Sami": [
        ""
      ],
      "Skolt Sami": [
        ""
      ],
      "Soninke": [
        ""
      ],
      "Shona": [
        ""
      ],
      "Sogdien": [
        ""
      ],
      "Somalia kasa": [
        ""
      ],
      "Sranan Tongo": [
        ""
      ],
      "Serer": [
        ""
      ],
      "Swati": [
        ""
      ],
      "Saho": [
        ""
      ],
      "Saterland Frisian": [
        ""
      ],
      "Southern Sotho": [
        ""
      ],
      "Sukuma": [
        ""
      ],
      "Susu": [
        ""
      ],
      "Sundanese": [
        ""
      ],
      "Sumerian": [
        ""
      ],
      "Sweden kasa": [
        ""
      ],
      "Comorian": [
        ""
      ],
      "Congo Swahili": [
        ""
      ],
      "Swahili": [
        ""
      ],
      "Classical Syriac": [
        ""
      ],
      "Syriac": [
        ""
      ],
      "Silesian": [
        ""
      ],
      "Tamil kasa": [
        ""
      ],
      "Tulu": [
        ""
      ],
      "Timne": [
        ""
      ],
      "Teso": [
        ""
      ],
      "Tereno": [
        ""
      ],
      "Telugu": [
        ""
      ],
      "Tetum": [
        ""
      ],
      "Taeland kasa": [
        ""
      ],
      "Tigre": [
        ""
      ],
      "Tigrinya": [
        ""
      ],
      "Tiv": [
        "Propina"
      ],
      "Tokelau": [
        ""
      ],
      "Tsakhur": [
        ""
      ],
      "Turkmen": [
        ""
      ],
      "Klingon": [
        ""
      ],
      "Tlingit": [
        ""
      ],
      "Tagalog": [
        ""
      ],
      "Talysh": [
        ""
      ],
      "Tamashek": [
        ""
      ],
      "Tswana": [
        ""
      ],
      "Nyasa Tonga": [
        ""
      ],
      "Tok Pisin": [
        ""
      ],
      "Tɛɛki kasa": [
        ""
      ],
      "Turoyo": [
        ""
      ],
      "Taroko": [
        ""
      ],
      "Tsakonian": [
        ""
      ],
      "Tsimshian": [
        ""
      ],
      "Tsonga": [
        ""
      ],
      "Tatar": [
        ""
      ],
      "Muslim Tat": [
        ""
      ],
      "Tumbuka": [
        ""
      ],
      "Tasawaq": [
        ""
      ],
      "Twi": [
        "Propina"
      ],
      "Tahitian": [
        ""
      ],
      "Tuvinian": [
        ""
      ],
      "Central Atlas Tamazight": [
        ""
      ],
      "Udmurt": [
        ""
      ],
      "Ugaritic": [
        ""
      ],
      "Uyghur": [
        ""
      ],
      "Ukren kasa": [
        ""
      ],
      "Umbundu": [
        ""
      ],
      "Unknown Language": [
        "Llengua actual"
      ],
      "Urdu kasa": [
        ""
      ],
      "Uzbek": [
        ""
      ],
      "Vai": [
        ""
      ],
      "Venetian": [
        ""
      ],
      "Veps": [
        ""
      ],
      "Venda": [
        ""
      ],
      "Viɛtnam kasa": [
        ""
      ],
      "West Flemish": [
        ""
      ],
      "Main-Franconian": [
        ""
      ],
      "Votic": [
        ""
      ],
      "Volapük": [
        ""
      ],
      "Võro": [
        ""
      ],
      "Vunjo": [
        ""
      ],
      "Walser": [
        ""
      ],
      "Wolaytta": [
        ""
      ],
      "Waray": [
        ""
      ],
      "Washo": [
        ""
      ],
      "Walloon": [
        ""
      ],
      "Warlpiri": [
        ""
      ],
      "Wolof": [
        ""
      ],
      "Wu Chinese": [
        ""
      ],
      "Kalmyk": [
        ""
      ],
      "Xhosa": [
        ""
      ],
      "Mingrelian": [
        ""
      ],
      "Soga": [
        ""
      ],
      "Yao": [
        ""
      ],
      "Yapese": [
        ""
      ],
      "Yangben": [
        ""
      ],
      "Yemba": [
        ""
      ],
      "Yiddish": [
        ""
      ],
      "Yoruba": [
        ""
      ],
      "Nheengatu": [
        ""
      ],
      "Cantonese": [
        ""
      ],
      "Zapotec": [
        ""
      ],
      "Zhuang": [
        ""
      ],
      "Blissymbols": [
        ""
      ],
      "Zeelandic": [
        ""
      ],
      "Zenaga": [
        ""
      ],
      "Standard Moroccan Tamazight": [
        ""
      ],
      "Simplified Chinese": [
        ""
      ],
      "Traditional Chinese": [
        ""
      ],
      "Kyaena kasa": [
        ""
      ],
      "Zuni": [
        ""
      ],
      "Zulu": [
        ""
      ],
      "No linguistic content": [
        ""
      ],
      "Zaza": [
        ""
      ],
      "Could not load user attention request": [
        "No s'han pogut carregar les comissions de canvi"
      ],
      "No notification left": [
        ""
      ],
      "Ignore": [
        ""
      ],
      "Invalid dev experiment URI.": [
        ""
      ],
      "URI: %1$s": [
        "%1$s"
      ],
      "Experiment": [
        ""
      ],
      "Insert lost denomination": [
        "Denominació"
      ],
      "Apply": [
        ""
      ],
      "Pending refresh": [
        "Operacions pendents"
      ],
      "Could not load the transfer payment status": [
        "No s'ha pogut carregar l'estatus del pagament"
      ],
      "Could not load the status of the order template": [
        "No s'ha pogut carregar la llista de transaccions"
      ],
      "required": [
        ""
      ],
      "Review order": [
        ""
      ],
      "Missing URI": [
        ""
      ],
      "Check the link.": [
        "Prem la icona del trencaclosques"
      ],
      "Could not parse the recovery URI": [
        "No s'ha pogut processar l'URI «payto»"
      ],
      "Try another URI": [
        "Torna a intentar"
      ],
      "Import backup, show info": [
        "Importar còpia de seguretat, mostrar informació"
      ],
      "Add new exchange": [
        "Afegir nou proveïdor"
      ],
      "Add exchange for %1$s": [
        "Afegir proveïdor per a %1$s"
      ],
      "Enter the URL of an exchange you trust.": [
        "Introdueix l'URL d'un proveïdor en qui confiïs."
      ],
      "An exchange has been found! Review the information and click next": [
        "S'ha trobat un proveïdor! Revisa la informació i clica «següent»"
      ],
      "This exchange is already in your list.": [
        "El proveïdor està llest per a la retirada"
      ],
      "Only exchange accessible through \"http\" and \"https\" are allowed.": [
        ""
      ],
      "This exchange protocol version is not supported: \" %1$s\".": [
        ""
      ],
      "This exchange currency \"%1$s\" doesn&apos;t match the expected currency %2$s.": [
        "Aquest proveïdor no accepta la divisa esperada: %1$s"
      ],
      "No exchange found in that URL.": [
        "No s'ha trobat cap proveïdor per a %1$s"
      ],
      "loading": [
        "carregant"
      ],
      "Version": [
        "Versió"
      ],
      "Can't use the URL: \"%1$s\"": [
        ""
      ],
      "Exchange URL": [
        "URL del proveïdor"
      ],
      "Add new contact": [
        "Afegir un altre compte"
      ],
      "Enter the details of a contact you trust.": [
        "Introdueix l'URL d'un proveïdor en qui confiïs."
      ],
      "Contact info valid.": [
        ""
      ],
      "This contact is already in your list.": [
        ""
      ],
      "Review contact": [
        ""
      ],
      "Contact alias": [
        ""
      ],
      "Alias type": [
        ""
      ],
      "Mailbox service": [
        ""
      ],
      "Mailbox address": [
        "Adreça de Bitcoin"
      ],
      "Contact source": [
        ""
      ],
      "Could not load the list of contacts": [
        "No s'ha pogut carregar la llista de transaccions"
      ],
      "Source": [
        ""
      ],
      "Send Cash": [
        "Enviar %1$s"
      ],
      "Request Cash": [
        ""
      ],
      "No contacts found.": [
        ""
      ],
      "Could not bank choices for account": [
        "No s'ha pogut carregar la informació de recuperació de la còpia de seguretat"
      ],
      "Could not load the list of messages": [
        "No s'ha pogut carregar la llista de proveïdors coneguts"
      ],
      "Unexpected error when trying to initialize mailbox: %1$s": [
        ""
      ],
      "Unexpected error when trying to delete message: %1$s": [
        ""
      ],
      "Unexpected error when trying to fetch messages: %1$s": [
        ""
      ],
      "Date": [
        "Data"
      ],
      "You may copy your mailbox URI from below and register it at %1$s with your alias(es) to make it easier for others to find you.": [
        ""
      ],
      "copy uri": [
        "copiar"
      ],
      "hide uri": [
        "amagar QR"
      ],
      "show uri": [
        "mostrar QR"
      ],
      "Fetch messages": [
        "Missatge del comerciant"
      ],
      "Mailbox not initialized": [
        ""
      ],
      "Initialize mailbox": [
        ""
      ],
      "No messages yet.": [
        "De moment no hi ha cap proveïdor"
      ],
      "Taler URI handler": [
        ""
      ],
      "Could not found a handler for the Taler URI": [
        "no s'ha pogut trobar cap proveïdor per a la divisa %1$s"
      ],
      "The uri read in the path parameter is not valid: \"%1$s\"": [
        ""
      ],
      "Digital cash payment": [
        "Pagament en efectiu digital"
      ],
      "Digital cash refund": [
        "Reemborsament d'efectiu digital"
      ],
      "Digital cash withdrawal": [
        "Retirada d'efectiu digital"
      ],
      "Digital cash deposit": [
        "Dipòsit d'efectiu digital"
      ],
      "Digital cash invoice": [
        "Factura digital"
      ],
      "Digital cash transfer": [
        "Transferència d'efectiu digital"
      ],
      "Digital cash recovery": [
        "Recuperació de cartera digital"
      ],
      "Development experiment": [
        "Mode de desenvolupador"
      ],
      "Return to wallet": [
        ""
      ],
      "Network is offline": [
        ""
      ],
      "Diagnostics timed out. Could not talk to the wallet backend.": [
        "El diagnòstic ha caducat. No s'ha pogut comunicar amb la cartera."
      ],
      "Problems detected:": [
        "Problemes detectats:"
      ],
      "Please check in your %1$s settings that you have IndexedDB enabled (check the preference name %2$s).": [
        "Si us plau, comprova a la configuració de %1$s si tens IndexedDB activat (busca el nom de preferència %2$s)."
      ],
      "Your wallet database is outdated. Currently automatic migration is not supported. Please go %1$s to reset the wallet database.": [
        "La base de dades de la teva cartera ha caducat. Actualment no hi ha suport per a la migració automàtica. Si us plau, ves a %1$s per resetejar la base de dades de la cartera."
      ],
      "here": [
        ""
      ],
      "Running diagnostics": [
        "Executant diagnòstics"
      ],
      "Edit": [
        "Editar"
      ],
      "List of products": [
        "Llista de productes"
      ],
      "free": [
        "de franc"
      ],
      "Could not get provider information": [
        "No s'ha pogut aconseguir informació del proveïdor"
      ]
    }
  },
  "domain": "messages",
  "plural_forms": "nplurals=2; plural=n != 1;",
  "lang": "ca",
  "completeness": 23
};

